// swad_text.c: text messages on screen, in several languages

/*
    SWAD (Shared Workspace At a Distance in Spanish),
    is a web platform developed at the University of Granada (Spain),
    and used to support university teaching.

    This file is part of SWAD core.
    Copyright (C) 1999-2017 Antonio Caas Vargas

    Catal translation:
	Antonio Caas Vargas
	Joan Llus Daz Rodrguez
    German translation:
	Antonio Caas Vargas
	Rafael Barranco-Droege
    English translation (finished):
	Antonio Caas Vargas
    Spanish translation (finished):
	Antonio Caas Vargas
    French translation:
	Antonio Caas Vargas
    Guarani translation:
	Antonio Caas Vargas
    Italian translation (finished):
	Antonio Caas Vargas
	Nicola Comunale Rizzo
	Francisco Manuel Herrero Prez
	Giuseppe Antonio Pagin
	Antonella Grande
    Polish translation:
	Antonio Caas Vargas
	Wojtek Kieca
	Tomasz Olechowski
	Mateusz Stanko
    Portuguese translation:
	Antonio Caas Vargas

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

/*
 Sabies que pots utilitzar SWAD en catal? https://swad.ugr.es/ca
 Wussten Sie, dass Sie verwenden knnen, SWAD auf Deutsch? https://swad.ugr.es/de
 Did you know that you can use SWAD in English? https://swad.ugr.es/en
 Saviez-vous que vous pouvez utiliser SWAD en franais? https://swad.ugr.es/fr
 Lo sapevate che  possibile utilizzare SWAD in italiano? https://swad.ugr.es/it
 Czy wiesz, ze mozesz uzywac SWAD po polsku? https://swad.ugr.es/pl
 Sabia que voc pode usar SWAD em portugus? https://swad.ugr.es/pt
 */

/*****************************************************************************/
/********************************* Headers ***********************************/
/*****************************************************************************/

#include <linux/stddef.h>	// For NULL

#include "swad_assignment.h"
#include "swad_attendance.h"
#include "swad_config.h"
#include "swad_country.h"
#include "swad_course.h"
#include "swad_degree.h"
#include "swad_file_browser.h"
#include "swad_forum.h"
#include "swad_holiday.h"
#include "swad_info.h"
#include "swad_menu.h"
#include "swad_notification.h"
#include "swad_photo.h"
#include "swad_QR.h"
#include "swad_record.h"
#include "swad_social.h"
#include "swad_statistic.h"
#include "swad_survey.h"
#include "swad_syllabus.h"
#include "swad_tab.h"
#include "swad_test.h"
#include "swad_text.h"
#include "swad_timetable.h"

/*****************************************************************************/
/******************************** Constants **********************************/
/*****************************************************************************/

const char *Txt_NEW_LINE = "\r\n";	// End of line in a file. If we put only \n the file does not look good in some Windows text editors

// The HTML entity for "" is &szlig; It stands for "S-Z ligature", because this symbol comes from the ligature of a Gothic S and a Z.
// For polish HTML entities, see http://pl.wikipedia.org/wiki/Alfabet_polski
// Guarani:
// E with tilde: &#7868;
// e with tilde: &#7869;
// I with tilde: &#296;
// i with tilde: &#297;

/***** Languages *****/
const char *Txt_STR_LANG_ID[1 + Txt_NUM_LANGUAGES] = // ISO 639-1 language codes
	{
	"",	// Txt_LANGUAGE_UNKNOWN
	"ca",	// Txt_LANGUAGE_CA
	"de",	// Txt_LANGUAGE_DE
	"en",	// Txt_LANGUAGE_EN
	"es",	// Txt_LANGUAGE_ES
	"fr",	// Txt_LANGUAGE_FR
	"gn",	// Txt_LANGUAGE_GN
	"it",	// Txt_LANGUAGE_IT
	"pl",	// Txt_LANGUAGE_PL
	"pt",	// Txt_LANGUAGE_PT
	};

#if   L==1
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_CA;
#elif L==2
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_DE;
#elif L==3
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_EN;
#elif L==4
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_ES;
#elif L==5
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_FR;
#elif L==6
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_GN;
#elif L==7
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_IT;
#elif L==8
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_PL;
#elif L==9
const unsigned Txt_Current_CGI_SWAD_Language = Txt_LANGUAGE_PT;
#endif

const char *Txt_STR_LANG_NAME[1 + Txt_NUM_LANGUAGES] =
	{
	"",
	"Catal&agrave;",
	"Deutsch",
	"English",
	"Espa&ntilde;ol",
	"Fran&ccedil;ais",
	"Ava&ntilde;e'&#7869;",
	"Italiano",
	"Polski",
	"Portugu&ecirc;s",
	};
const char *Txt_Do_you_want_to_change_the_language_to_LANGUAGE[1 + Txt_NUM_LANGUAGES] =
	{
	"",
	"Voleu canviar l'idioma a catal&agrave;?",
	"Wollen Sie die Sprache auf Deutsch umstellen?",
	"Do you want to change the language to English?",
	"&iquest;Desea cambiar el idioma a espa&ntilde;ol?",
	"Voulez-vous changer la langue au fran&ccedil;ais?",
	"&iquest;Oipota moambue pe &ntilde;e'&#7869; pe ava&ntilde;e'&#7869;?",
	"Vuoi cambiare la lingua in italiano?",
	"Czy chcesz zmieni&cacute; j&eogon;zyk na polski?",
	"Voc&ecirc; quer mudar o idioma para portugu&ecirc;s?",
	};
const char *Txt_Do_you_want_to_change_your_language_to_LANGUAGE[1 + Txt_NUM_LANGUAGES] =
	{
	"",
	"Voleu canviar el seu idioma a catal&agrave;?",
	"Wollen Sie Ihre Sprache auf Deutsch umstellen?",
	"Do you want to change your language to English?",
	"&iquest;Desea cambiar su idioma a espa&ntilde;ol?",
	"Voulez-vous changer votre langue au fran&ccedil;ais?",
	"&iquest;Oipota moambue pe &ntilde;e'&#7869; pe ava&ntilde;e'&#7869;?",
	"Vuoi cambiare la tua lingua in italiano?",
	"Czy chcesz zmieni&cacute; j&eogon;zyk na polski?",
	"Voc&ecirc; quer mudar suo idioma para portugu&ecirc;s?",
	};
const char *Txt_Switch_to_LANGUAGE[1 + Txt_NUM_LANGUAGES] =
	{
	"",
	"Canviar a catal&agrave;",
	"Umschalten auf Deutsch",
	"Switch to English",
	"Cambiar a espa&ntilde;ol",
	"Passer au fran&ccedil;ais",
	"Moambue pe &ntilde;e'&#7869; ava&ntilde;e'&#7869;",
	"Passare a italiano",
	"Prze&lstrok;&aogon;cz na polski",
	"Mudar para portugu&ecirc;s",
	};
const char *Txt_Switching_to_LANGUAGE[1 + Txt_NUM_LANGUAGES] =
	{
	"",
	"Canviant a catal&agrave;&hellip;",
	"Umschaltung auf Deutsch&hellip;",
	"Switching to English&hellip;",
	"Cambiando a espa&ntilde;ol&hellip;",
	"Passant au fran&ccedil;ais&hellip;",
	"Moambue pe &ntilde;e'&#7869; ava&ntilde;e'&#7869;&hellip;",
	"Passando a italiano&hellip;",
	"Prze&lstrok;&aogon;cz na polski&hellip;",
	"Mudando para portugu&ecirc;s&hellip;",
	};

/*****************************************************************************
		  #   #  ###       #   # ##### #   # #
		  ##  # #   #      #   #   #   ## ## #
		  # # # #   #      #####   #   # # # #
		  #  ## #   #      #   #   #   #   # #
		  #   #  ###       #   #   #   #   # #####
******************************************************************************/
/**** Don't use HTML entities like &oacute; when variable ends in NO_HTML ****/
/****              Charset must be windows-1252 in this file              ****/
/*****************************************************************************/

const char *Txt_Country_NO_HTML =
#if   L==1
	"Pas";
#elif L==2
	"Land";
#elif L==3
	"Country";
#elif L==4
	"Pas";
#elif L==5
	"Pays;";
#elif L==6
	"Teta";
#elif L==7
	"Paese";
#elif L==8
	"Kraj";
#elif L==9
	"Pa&iacute;s";
#endif

const char *Txt_Confirmation_of_your_email_NO_HTML =	// Don't use HTML entities like &egrave; here
#if   L==1
	"Confirmacin de su direccin de correo";	// Necessita traduccio
#elif L==2
	"Confirmation of your email";			// Need bersetzung
#elif L==3
	"Confirmation of your email";
#elif L==4
	"Confirmacin de su direccin de correo";
#elif L==5
	"Confirmation of your email";			// Besoin de traduction
#elif L==6
	"Confirmacin de su direccin de correo";	// Okoteve traduccin
#elif L==7
	"Confirmation of your email";			// Bisogno di traduzione
#elif L==8
	"Confirmation of your email";			// Potrzebujesz tlumaczenie
#elif L==9
	"Confirmation of your email";			// Necessita de traduo
#endif

// The following variables are compilated together in all languages because they are used in emails...
// ...and each email is sent in the language of the recipient

const char *Txt_Course_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"",
	"Assignatura",
	"Kurs",
	"Course",
	"Asignatura",
	"Matire",
	"Mbo'esyry",
	"Corso",
	"Kurs",
	"Disciplina",
	};

const char *Txt_Dear_NO_HTML[Usr_NUM_SEXS][1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &oacute; here
{
	{
	// Usr_SEX_UNKNOWN
	"",
	"Estimat/da",
	"Sehr geehrte/r",
	"Dear",
	"Estimado/a",
	"Cher/Chre",
	"Estimado/a",	// Okoteve traduccin
	"Egregio/a",
	"Drogi",
	"Prezado/a",
	},
	{
	// Usr_SEX_FEMALE
	"",
	"Estimada",
	"Sehr geehrte",
	"Dear",
	"Estimada",
	"Chre",
	"Estimada",	// Okoteve traduccin
	"Egregia",
	"Drogi",
	"Prezada",
	},
	{
	// Usr_SEX_MALE
	"",
	"Estimat",
	"Sehr geehrter",
	"Dear",
	"Estimado",
	"Cher",
	"Estimado",	// Okoteve traduccin
	"Egregio",
	"Drogi",
	"Prezado",
	},
	{
	// Usr_SEX_ALL
	"",
	"Estimat/da",
	"Sehr geehrte/r",
	"Dear",
	"Estimado/a",
	"Cher/Chre",
	"Estimado/a",	// Okoteve traduccin
	"Egregio/a",
	"Drogi",
	"Prezado/a",
	}
};

const char *Txt_Forum_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &oacute; here
	{
	"",
	"Frum",
	"Forum",
	"Forum",
	"Foro",
	"Forum",
	"Foro",		// Okoteve traduccin
	"Forum",
	"Forum",
	"Frum",
	};

const char *Txt_General_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &eacute; here
	{
	"",
	"General",
	"Allgemein",
	"General",
	"General",
	"Gnral",
	"General",	// Okoteve traduccin
	"Generale",
	"Oglne",
	"Geral",
	};

const char *Txt_Go_to_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &agrave; here
	{
	"",
	"Anar a",
	"Gehe zum",
	"Go to",
	"Ir a",
	"Aller ",
	"Ir a",		// Okoteve traduccin
	"Vai a",
	"Idz do",
	"Ir para",
	};

const char *Txt_If_you_just_request_from_X_the_confirmation_of_your_email_Y_NO_HTML =	// Don't use HTML entities like &egrave; here
#if   L==1
	"Si usted acaba de solicitar desde %s"
	" la confirmacin de su correo %s,"
	" pulse en el siguiente enlace"
	" para confirmar dicha direccin:"
	" %s/?act=%ld&key=%s\n\n"
	"Si no ha sido usted, no pulse en el enlace anterior!"
	" En este caso, le recomendamos que compruebe en %s"
	" si ha confirmado su direccin de correo.\n\n";	// Necessita traduccio
#elif L==2
	"If you just request from %s"
	" the confirmation of your email %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?act=%ld&key=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your email address.\n\n";	// Need bersetzung
#elif L==3
	"If you just request from %s"
	" the confirmation of your email %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?act=%ld&key=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your email address.\n\n";
#elif L==4
	"Si usted acaba de solicitar desde %s"
	" la confirmacin de su correo %s,"
	" pulse en el siguiente enlace"
	" para confirmar dicha direccin:"
	" %s/?act=%ld&key=%s\n\n"
	"Si no ha sido usted, no pulse en el enlace anterior!"
	" En este caso, le recomendamos que compruebe en %s"
	" si ha confirmado su direccin de correo.\n\n";
#elif L==5
	"If you just request from %s"
	" the confirmation of your email %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?act=%ld&key=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your email address.\n\n";	// Besoin de traduction
#elif L==6
	"Si usted acaba de solicitar desde %s"
	" la confirmacin de su correo %s,"
	" pulse en el siguiente enlace"
	" para confirmar dicha direccin:"
	" %s/?act=%ld&key=%s\n\n"
	"Si no ha sido usted, no pulse en el enlace anterior!"
	" En este caso, le recomendamos que compruebe en %s"
	" si ha confirmado su direccin de correo.\n\n";	// Okoteve traduccin
#elif L==7
	"If you just request from %s"
	" the confirmation of your email %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?act=%ld&key=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your email address.\n\n";	// Bisogno di traduzione
#elif L==8
	"If you just request from %s"
	" the confirmation of your email %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?act=%ld&key=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your email address.\n\n";	// Potrzebujesz tlumaczenie
#elif L==9
	"If you just request from %s"
	" the confirmation of your email %s,"
	" click on the following link"
	" to confirm that direction:"
	" %s/?act=%ld&key=%s\n\n"
	"If it was not you, do not click on the link above!"
	" In this case, we recommend that you check in %s"
	" if you have confirmed your email address.\n\n";	// Necessita de traduo
#endif

const char *Txt_If_you_no_longer_wish_to_receive_email_notifications_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"",
	"Si no desitgeu rebre ms avisos per correu,"
	" desactiveu la opci corresponent en les preferncies del seu perfil.",
	"Wenn Sie nicht mehr erhalten mchten, E-Mail-Benachrichtigungen,"
	" Deaktivieren Sie die Option in den Einstellungen Ihres Profils.",
	"If you no longer wish to receive email notifications,"
	" uncheck the option in the preferences of your profile.",
	"Si no desea recibir ms avisos por correo,"
	" desactive la opcin correspondiente en las preferencias de su perfil.",
	"Si vous ne souhaitez plus recevoir de notifications par email,"
	" dcochez l'option dans les prfrences de votre profil.",
	"Si no desea recibir ms avisos por correo,"
	" desactive la opcin correspondiente en las preferencias de su perfil.",	// Okoteve traduccin
	"Se non desideri pi ricevere notifiche via email,"
	" deselezionare l'opzione nelle preferenze del tuo profilo.",
	"Jesli nie chcesz otrzymywac powiadomien email,"
	" usu&nacute; zaznaczenie opcje w ustawieniach swojego profilu.",
	"Se j no desejar receber notificaes por email,"
	" desmarque a opo nas preferncias do seu perfil.",
	};

const char *Txt_Institutions_NO_HTML =
#if   L==1
	"Institucions";
#elif L==2
	"Hochschulen";
#elif L==3
	"Institutions";
#elif L==4
	"Instituciones";
#elif L==5
	"tablissements";
#elif L==6
	"Mbo'ehao";
#elif L==7
	"Istituzioni";
#elif L==8
	"Instytucje";
#elif L==9
	"Institues";
#endif

const char *Txt_MSG_From_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"",
	"De",
	"Von",
	"From",
	"De",
	"De",
	"De",	// Okoteve traduccin
	"Da",
	"Od",
	"De",
	};

const char *Txt_Notifications_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"",
	"Notificacions",
	"Anmeldungen",
	"Notifications",
	"Notificaciones",
	"Notifications",
	"Momarandu",
	"Notifiche",
	"Powiadomienia",
	"Notificaes",
	};

const char *Txt_New_password_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &ntilde; here.
	{
	"",
	"Nova contrasenya",
	"Neues Passwort",
	"New password",
	"Nueva contrasea",
	"Nouveau mot de passe",
	"Nueva contrasea",	// Okoteve traduccin
	"Nuova password",
	"Nowe has&lstrok;o",
	"Nova senha",
	};

const char *Txt_NOTIFY_EVENTS_SINGULAR_NO_HTML[Ntf_NUM_NOTIFY_EVENTS][1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &uuml; here.
{
	{
	// Ntf_EVENT_UNKNOWN
	"",
	"Esdeveniment desconegut",
	"Unbekannt Ereignis",
	"Unknown event",
	"Evento desconocido",
	"vnement inconnu",
	"Evento desconocido",			// Okoteve traduccin
	"Evento sconosciuto",
	"Nieznane zdarzenia",
	"Evento desconhecido",
	},
	{
	// Ntf_EVENT_DOCUMENT_FILE
	"",
	"Nou document",
	"Neue Dokumentdatei",
	"New document file",
	"Nuevo documento",
	"Nouveau fichier de document",
	"Nuevo documento",			// Okoteve traduccin
	"Nuovo documento",
	"Nowy plik dokumentu",
	"Novo arquivo de documento",
	},
	{
	// Ntf_EVENT_TEACHERS_FILE
	"",
	"Nou arxiu de professors",
	"Neue Lehrer Datei",
	"New teachers' file",
	"Nuevo archivo de profesores",
	"Nouveau fichier d'enseignants",
	"Nuevo archivo de profesores",		// Okoteve traduccin
	"Nuovo file di professori",
	"Nowy Nauczyciela plik",
	"Novo arquivo dos professores",
	},
	{
	// Ntf_EVENT_SHARED_FILE
	"",
	"Nou arxiu compartit",
	"Neue freigegebene Datei",
	"New shared file",
	"Nuevo archivo compartido",
	"Nouveau fichier partag",
	"Nuevo archivo compartido",		// Okoteve traduccin
	"Nuovo file condiviso",
	"Nowy Udostepniony plik",
	"Novo arquivo compartilhado",
	},
	{
	// Ntf_EVENT_ASSIGNMENT
	"",
	"Nova activitat",
	"Neue Aufgabe",
	"New assignment",
	"Nueva actividad",
	"Nouvelle activit",
	"Nueva actividad",			// Okoteve traduccin
	"Nuova attivit",
	"Nowe zadania",
	"Nova atividade",
	},
	{
	// Ntf_EVENT_EXAM_ANNOUNCEMENT
	"",
	"Nova convocatria d'examen",
	"Neue Aufrufe fr Prfung",
	"New announcement of exam",
	"Nueva convocatoria de examen",
	"Nouvelle convocation  un examen",
	"Nueva convocatoria de examen",		// Okoteve traduccin
	"Nuovo appello d'esame",
	"Nowe ogloszenie egzaminu",
	"Nova chamada para exame",
	},
	{
	// Ntf_EVENT_MARKS_FILE
	"",
	"Nou fitxer de calificacions",
	"Neue Datei mit Bewertung",
	"New file with marks",
	"Nuevo archivo de calificaciones",
	"Nouveau fichier avec des notes",
	"Nuevo archivo de calificaciones",	// Okoteve traduccin
	"Nuovo file con i risultati",
	"Nowy plik ze znakami",
	"Novo arquivo de notas",
	},
	{
	// Ntf_EVENT_ENROLLMENT_STUDENT
	"",
	"Nova inscripci com a estudiant",
	"New Einschreibung als Student",
	"New enrollment as a student",
	"Nueva inscripcin como estudiante",
	"Nouvelle inscriptionen tant qu'tudiant",
	"Nueva inscripcin como estudiante",			// Okoteve traduccin
	"Nuova iscrizione come studente",
	"Nowe zapisy jako student",
	"Nova inscrio como estudante",
	},
	{
	// Ntf_EVENT_ENROLLMENT_TEACHER
	"",
	"Nova inscripci com a professor",
	"New Einschreibung als Lehrkraft",
	"New enrollment as a teacher",
	"Nueva inscripcin como profesor",
	"Nouvelle inscription en tant qu'enseignant",
	"Nueva inscripcin como profesor",			// Okoteve traduccin
	"Nuova iscrizione come professore",
	"Nowe zapisy jako nauczyciel",
	"Nova inscrio como professor",
	},
	{
	// Ntf_EVENT_ENROLLMENT_REQUEST
	"",
	"Nova petici d'inscripci",
	"New Registrierungsanforderung",
	"New enrollment request",
	"Nueva peticin de inscripcin",
	"Nouvelle demande d'inscription",
	"Nueva peticin de inscripcin",	// Okoteve traduccin
	"Nuova richiesta di iscrizione",
	"Nowe prosby o rejestracji",
	"Novo pedido de inscrio",
	},
	{
	// Ntf_EVENT_TIMELINE_COMMENT
	"",
	"Nou comentari a un missatge social",
	"Neue Kommentar zu soziale Beitrag",
	"New comment to social post",
	"Nuevo comentario a un mensaje social",
	"Nouveau commentaire  message social",
	"Nuevo comentario a un mensaje social",			// Okoteve traduccin
	"Nuovo commento a un post sociale",
	"Nowy komentarz do post spoleczna",
	"Novo comentrio a um post sociais",
	},
	{
	// Ntf_EVENT_TIMELINE_FAV
	"",
	"Nou favorit a un missatge social",
	"Neue Favorit zu soziale Beitrag",
	"New favourite to social post",
	"Nuevo favorito a un mensaje social",
	"Nouveau favori  message social",
	"Nuevo favorito a un mensaje social",			// Okoteve traduccin
	"Nuovo preferito a un post sociale",
	"Nowy ulubiony do post spoleczna",
	"Novo favorito a um post sociais",
	},
	{
	// Ntf_EVENT_TIMELINE_SHARE
	"",
	"Nova compartici de missatge social",
	"Neue gemeinsame Nutzung von soziale Beitrag",
	"New sharing of social post",
	"Nueva comparticin de mensaje social",
	"Nouveau partage de message social",
	"Nueva comparticin de mensaje social",			// Okoteve traduccin
	"Nuova condivisione di post sociale",
	"Nowy podzial post spoleczna",
	"Nova partilha de post sociais",
	},
	{
	// Ntf_EVENT_TIMELINE_MENTION
	"",
	"Nova menci",
	"Neue Erwhnung",
	"New mention",
	"Nueva mencin",
	"Nouveau mention",
	"Nueva mencin",			// Okoteve traduccin
	"Nuovo menzione",
	"Nowa wzmianka",
	"Nova meno",
	},
	{
	// Ntf_EVENT_FOLLOWER
	"",
	"Nou seguidor",
	"Neue Anh&auml;nger",
	"New follower",
	"Nuevo seguidor",
	"Nouveau suiveur",
	"Nuevo seguidor",	// Okoteve traduccin
	"Nuovo follower",
	"Nowy obserwuj&aogon;",
	"Novo seguidor",
	},
	{
	// Ntf_EVENT_FORUM_POST_COURSE
	"",
	"Novo missatge en un frum",
	"Neue Nachricht in einem Forum",
	"New post in a forum",
	"Nuevo mensaje en un foro",
	"Nouveau message dans un forum",
	"Nuevo mensaje en un foro",		// Okoteve traduccin
	"Nuovo post in un forum",
	"Nowy post na forum",
	"Novo post em um frum",
	},
	{
	// Ntf_EVENT_FORUM_REPLY
	"",
	"Nova resposta en un frum",
	"Neue Antwort in einem Forum",
	"New reply in a forum",
	"Nueva respuesta en un foro",
	"Nouvelle rponse dans un forum",
	"Nueva respuesta en un foro",		// Okoteve traduccin
	"Nuova risposta in un forum",
	"Odpowiedz nowe na forum",
	"Nova resposta em um frum",
	},
	{
	// Ntf_EVENT_NOTICE
	"",
	"Nou avs",
	"Neuen Ankndigungen",
	"New notice",
	"Nuevo aviso",
	"Nouvel avis",
	"Nuevo aviso",				// Okoteve traduccin
	"Nuovo avviso",
	"Nowe obwieszczenie",
	"Novo anncio",
	},
	{
	// Ntf_EVENT_MESSAGE
	"",
	"Nou missatge",
	"Neue Nachricht",
	"New message",
	"Nuevo mensaje",
	"Nouveau message",
	"Nuevo mensaje",			// Okoteve traduccin
	"Nuovo messaggio",
	"Nowa wiadomosc",
	"Nova mensagem",
	},
	{
	// Ntf_EVENT_SURVEY
	"",
	"Nova enquesta",
	"Neuer Umfrage",
	"New survey",
	"Nueva encuesta",
	"Nouveau sondage",
	"Nueva encuesta",			// Okoteve traduccin
	"Nuovo sondaggio",
	"Nowe badania",
	"Novo inqu&eacute;rito",
	},
};

const char *Txt_NOTIFY_EVENTS_There_is_a_new_event_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %s in the following sentences
	{			// Don't use HTML entities like &uuml; here.
	"",
	"Hi ha un nou esdeveniment en %s",
	"Es ist eine neue Ereignis in %s",
	"There is a new event in %s",
	"Hay un nuevo evento en %s",
	"Il y un nouvel vnement dans %s",
	"Hay un nuevo evento en %s",	// Okoteve traduccin
	"C' un nuovo evento in %s",
	"Jest nowe wydarzenie w %s",
	"Existe um novo evento em %s",
	};

const char *Txt_NOTIFY_EVENTS_There_are_X_new_events_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Warning: it is very important to include %u and %s in the following sentences
	{			// Don't use HTML entities like &uuml; here.
	"",
	"Hi ha %u nous esdeveniments en %s",
	"Es gibt %u neue Ereignisse in %s",
	"There are %u new events in %s",
	"Hay %u nuevos eventos en %s",
	"Il y a %u nouveaux vnements dans %s",
	"Hay %u nuevos eventos en %s",	// Okoteve traduccin
	"Ci sono %u nuovi eventi in %s",
	"Istnieje %u nowych wydarzen w %s",
	"H %u novos eventos em %s",
	};

const char *Txt_only_teachers_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &oacute; here
	{
	"",
	", professors",
	", Lehrkrfte",
	", teachers",
	", profesores",
	", enseignants",
	", profesores",	// Okoteve traduccin
	", professori",
	", nauczyciel",
	", professores",
	};

const char *Txt_Please_do_not_reply_to_this_automatically_generated_email_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"",
	"Si us plau, no respongui a aquest correu generat automticament.",
	"Bitte, antworten Sie nicht auf diese automatisch generierte E-Mail.",
	"Please do not reply to this automatically generated email.",
	"Por favor, no responda a este correo generado automticamente.",
	"S'il vous plat, ne pas rpondre  ce courrier lectronique gnr automatiquement.",
	"Por favor, no responda a este correo generado automticamente.",	// Okoteve traduccin
	"Per favore, non rispondere a questa email generata automaticamente.",
	"Prosimy nie odpowiadac na to automatycznie wygenerowany email.",
	"Por favor, no responda a este email gerado automaticamente.",
	 };

const char *Txt_TAB_Messages_NO_HTML[1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
	{
	"",
	"Missatges",
	"Nachrichten",
	"Messages",
	"Mensajes",
	"Messages",
	"Marandu",
	"Messaggi",
	"Wiadomosci",
	"Mensagens",
	};

const char *Txt_The_following_password_has_been_assigned_to_you_to_log_in_X_NO_HTML =	// Warning: it is very important to include %s and %u in the following sentences. Don't use HTML entities like &egrave; here
#if   L==1
	"Se li ha assignat la segent contrasenya per entrar a %s: %s\n"
	"Si desitja activar aquesta nova contrasenya,"
	" ha de entrar a %s amb el seu ID (DNI/cdula), sobrenom o email"
	" i aquesta nova contrasenya abans de %u dies."
	" Un cop hagi entrat, pot canviar la contrasenya per una altra.\n"
	"Si no accedeix amb aquesta nova contrasenya,"
	" seguir tenint la contrasenya que tingus fins ara.\n"
	"Si ha rebut aquest missatge sense sol&middot;licitar-lo,"
	" es deu a que alg que coneix el vostre ID (DNI/cdula), el vostre sobrenom o el vostre email"
	" ha sol&middot;licitat l'enviament de una nova contrasenya per a vost.\n"
	"Aquesta nova contrasenya se ha enviat noms a la adrea de correu %s,"
	" que s la que apareix a la seva fitxa.\n\n";
#elif L==2
	"The following password has been assigned to you to log in %s: %s\n"
	"If you want to activate the new password,"
	" you must enter %s with your ID/nickname/email"
	" and this new password before %u days."
	" Once you have logged in, you can change your password.\n"
	"If you do not log in with this new password,"
	" the old password will remain valid.\n"
	"If you have received this message without having requested it,"
	" is that someone who knows your ID, nickname or email has requested a new password sent to you.\n"
	"This new password has been sent only to the email address %s, which is on your record card.\n\n";	// Need bersetzung
#elif L==3
	"The following password has been assigned to you to log in %s: %s\n"
	"If you want to activate the new password,"
	" you must enter %s with your ID/nickname/email"
	" and this new password before %u days."
	" Once you have logged in, you can change your password.\n"
	"If you do not log in with this new password,"
	" the old password will remain valid.\n"
	"If you have received this message without having requested it,"
	" is that someone who knows your ID, nickname or email has requested a new password sent to you.\n"
	"This new password has been sent only to the email address %s, which is on your record card.\n\n";
#elif L==4
	"Se le ha asignado la siguiente contrasea para entrar en %s: %s\n"
	"Si desea activar esta nueva contrasea,"
	" debe entrar en %s con su ID (DNI/cdula), apodo o direccin de correo"
	" y esta nueva contrasea antes de %u das."
	" Una vez que haya entrado, puede cambiar la contrasea por otra.\n"
	"Si no accede con esta nueva contrasea,"
	" seguir teniendo la contrasea que tuviera hasta ahora.\n"
	"Si ha recibido este mensaje sin solicitarlo,"
	" se debe a que alguien que conoce su ID (DNI/cdula), su apodo o su direccin de correo"
	" ha solicitado el envo de una nueva contrasea para usted.\n"
	"Esta nueva contrasea se ha enviado slo a la direccin de correo %s,"
	" que es la que aparece en su ficha.\n\n";
#elif L==5
	"The following password has been assigned to you to log in %s: %s\n"
	"If you want to activate the new password,"
	" you must enter %s with your ID/nickname/email"
	" and this new password before %u days."
	" Once you have logged in, you can change your password.\n"
	"If you do not log in with this new password,"
	" the old password will remain valid.\n"
	"If you have received this message without having requested it,"
	" is that someone who knows your ID, nickname or email has requested a new password sent to you.\n"
	"This new password has been sent only to the email address %s, which is on your record card.\n\n";	// Besoin de traduction
#elif L==6
	"Se le ha asignado la siguiente contrasea para entrar en %s: %s\n"
	"Si desea activar esta nueva contrasea,"
	" debe entrar en %s con su ID (DNI/c&eacute;dula), apodo o direccin de correo"
	" y esta nueva contrasea antes de %u das."
	" Una vez que haya entrado, puede cambiar la contrasea por otra.\n"
	"Si no accede con esta nueva contrasea,"
	" seguir teniendo la contrasea que tuviera hasta ahora.\n"
	"Si ha recibido este mensaje sin solicitarlo,"
	" se debe a que alguien que conoce su ID (DNI/cdula), su apodo o su direccin de correo"
	" ha solicitado el envo de una nueva contrasea para usted.\n"
	"Esta nueva contrasea se ha enviado slo a la direccin de correo %s,"
	" que es la que aparece en su ficha.\n\n";	// Okoteve traduccin
#elif L==7
	"La seguente password ti  stata assegnata per entrare in %s: %s\n"
	"Se vuoi attivare la nuova password,"
	" devi entrare in %s con il tuo ID / nome utente / email"
	" e li c' la nuova password prima di %u giorni."
	" Una volta che ti sei loggato, puoi scegliere la tua password.\n"
	"Se tu non entri con questa nuova password,"
	" rimarr valida la vecchia password.\n"
	"Se hai ricevuto questo messaggio senza averlo richiesto,"
	"  perch qualcuno che sa il tuo ID, nome utente o email ha richiesto una nuova password da inviarti.\n"
	"Questa nuova password  stata inviata solo all'indirizzo email %s, che  presente sulla tua scheda.\n\n";
#elif L==8
	"The following password has been assigned to you to log in %s: %s\n"
	"If you want to activate the new password,"
	" you must enter %s with your ID/nickname/email"
	" and this new password before %u days."
	" Once you have logged in, you can change your password.\n"
	"If you do not log in with this new password,"
	" the old password will remain valid.\n"
	"If you have received this message without having requested it,"
	" is that someone who knows your ID, nickname or email has requested a new password sent to you.\n"
	"This new password has been sent only to the email address %s, which is on your record card.\n\n";	// Potrzebujesz tlumaczenie
#elif L==9
	"The following password has been assigned to you to log in %s: %s\n"
	"If you want to activate the new password,"
	" you must enter %s with your ID/nickname/email"
	" and this new password before %u days."
	" Once you have logged in, you can change your password.\n"
	"If you do not log in with this new password,"
	" the old password will remain valid.\n"
	"If you have received this message without having requested it,"
	" is that someone who knows your ID, nickname or email has requested a new password sent to you.\n"
	"This new password has been sent only to the email address %s, which is on your record card.\n\n";	// Necessita de traduo
#endif

const char *Txt_Unknown_or_without_photo =
#if   L==1
	"Desconocido/a o sin fotograf&iacute;a";	// Necessita traduccio
#elif L==2
	"Unknown or without photo";			// Need bersetzung
#elif L==3
	"Unknown or without photo";
#elif L==4
	"Desconocido/a o sin fotograf&iacute;a";
#elif L==5
	"Unknown or without photo";			// Besoin de traduction
#elif L==6
	"Desconocido/a o sin fotograf&iacute;a";	// Okoteve traduccin
#elif L==7
	"Sconosciuto/a o senza foto";
#elif L==8
	"Unknown or without photo";			// Potrzebujesz tlumaczenie
#elif L==9
	"Unknown or without photo";			// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_could_not_create_file_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"%s: no s'ha pogut crear el fitxer.";
#elif L==2
	"%s: Datei konnte nicht erstellt werden.";
#elif L==3
	"%s: could not create file.";
#elif L==4
	"%s: no se ha podido crear el archivo.";
#elif L==5
	"%s: impossible de crer le fichier.";
#elif L==6
	"%s: no se ha podido crear el archivo.";	// Okoteve traduccin
#elif L==7
	"%s: impossibile creare il file.";
#elif L==8
	"%s: nie mozna utworzyc plik.";
#elif L==9
	"%s: no foi possvel criar arquivo.";
#endif

const char *Txt_UPLOAD_FILE_Forbidden_NO_HTML =
#if   L==1
	"No puede crear archivos aqu.";	// Necessita traduccio
#elif L==2
	"You can not create files here.";	// Need bersetzung
#elif L==3
	"You can not create files here.";
#elif L==4
	"No puede crear archivos aqu.";
#elif L==5
	"You can not create files here.";	// Besoin de traduction
#elif L==6
	"No puede crear archivos aqu.";	// Okoteve traduccin
#elif L==7
	"Non puoi creare file qui.";
#elif L==8
	"You can not create files here.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You can not create files here.";	// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_You_must_specify_the_file_NO_HTML =
#if   L==1
	"Debe especificar el archivo.";	// Necessita traduccio
#elif L==2
	"Die Datei muss angegeben werden.";
#elif L==3
	"You must specify the file.";
#elif L==4
	"Debe especificar el archivo.";
#elif L==5
	"Vous devez spcifier le fichier.";
#elif L==6
	"Debe especificar el archivo.";	// Okoteve traduccin
#elif L==7
	" necessario specificare il file.";
#elif L==8
	"You must specify the file.";	// Potrzebujesz tlumaczenie
#elif L==9
	"Voc deve especificar o arquivo.";
#endif

const char *Txt_UPLOAD_FILE_X_extension_not_allowed_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"%s: extensin no permitida.";		// Necessita traduccio
#elif L==2
	"%s: Endung nicht untersttzt.";
#elif L==3
	"%s: extension not allowed.";
#elif L==4
	"%s: extensin no permitida.";
#elif L==5
	"%s: extension not allowed.";		// Besoin de traduction
#elif L==6
	"%s: extensin no permitida.";		// Okoteve traduccin
#elif L==7
	"%s: estensione non permessa.";
#elif L==8
	"%s: extension not allowed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"%s: extension not allowed.";		// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_file_already_exists_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"%s: el archivo ya existe.";	// Necessita traduccio
#elif L==2
	"%s: file already exists.";	// Need bersetzung
#elif L==3
	"%s: file already exists.";
#elif L==4
	"%s: el archivo ya existe.";
#elif L==5
	"%s: file already exists.";	// Besoin de traduction
#elif L==6
	"%s: el archivo ya existe.";	// Okoteve traduccin
#elif L==7
	"%s: file already exists.";	// Bisogno di traduzione
#elif L==8
	"%s: file already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"%s: file already exists.";	// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_File_too_large_maximum_X_MiB_NO_HTML =	// Warning: it is very important to include %lu in the following sentences
#if   L==1
	"Archivo demasiado grande (mximo %lu MiB) /"
	" un altre problema d'enviament.";	// Necessita traduccio
#elif L==2
	"Datei zu gro (Maximum %lu MiB) /"
	" ein weiteres Upload-Problem.";
#elif L==3
	"File too large (maximum %lu MiB) /"
	" another upload problem.";
#elif L==4
	"Archivo demasiado grande (mximo %lu MiB) /"
	" otro problema de envo.";
#elif L==5
	"Fichier est trop gros (maximum %lu MiB) /"
	" un autre problme de tlchargement.";
#elif L==6
	"Archivo demasiado grande (mximo %lu MiB) /"
	" otro problema de envo.";		// Okoteve traduccin
#elif L==7
	"File troppo grande (massimo %lu MiB) /"
	" un altro problema di caricamento.";
#elif L==8
	"Plik jest za duzy (maksimum %lu MiB) /"
	" kolejny problem wgrac.";
#elif L==9
	"Arquivo muito grande (mximo %lu MiB) /"
	" outro problema de upload.";
#endif

const char *Txt_UPLOAD_FILE_Invalid_name_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Nombre no vlido.";	// Necessita traduccio
#elif L==2
	"Invalid name.";	// Need bersetzung
#elif L==3
	"Invalid name.";
#elif L==4
	"Nombre no vlido.";
#elif L==5
	"Invalid name.";	// Besoin de traduction
#elif L==6
	"Nombre no vlido.";	// Okoteve traduccin
#elif L==7
	"Nome non valido.";
#elif L==8
	"Invalid name.";	// Potrzebujesz tlumaczenie
#elif L==9
	"Invalid name.";	// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_invalid_name_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"%s: nombre no vlido.";	// Necessita traduccio
#elif L==2
	"%s: invalid name.";		// Need bersetzung
#elif L==3
	"%s: invalid name.";
#elif L==4
	"%s: nombre no vlido.";
#elif L==5
	"%s: invalid name.";		// Besoin de traduction
#elif L==6
	"%s: nombre no vlido.";	// Okoteve traduccin
#elif L==7
	"%s: nome non valido.";
#elif L==8
	"%s: invalid name.";		// Potrzebujesz tlumaczenie
#elif L==9
	"%s: invalid name.";		// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_MIME_type_Y_not_allowed_NO_HTML =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"%s: tipo MIME %s no permitido.";	// Necessita traduccio
#elif L==2
	"%s: MIME-Typ %s nicht untersttzt.";
#elif L==3
	"%s: MIME type %s not allowed.";
#elif L==4
	"%s: tipo MIME %s no permitido.";
#elif L==5
	"%s: MIME type %s not allowed.";	// Besoin de traduction
#elif L==6
	"%s: tipo MIME %s no permitido.";	// Okoteve traduccin
#elif L==7
	"%s: tipo MIME %s non consentito.";
#elif L==8
	"%s: MIME type %s not allowed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"%s: MIME type %s not allowed.";	// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_not_HTML_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"%s: no HTML";		// Necessita traduccio
#elif L==2
	"%s: not HTML";		// Need bersetzung
#elif L==3
	"%s: not HTML";
#elif L==4
	"%s: no HTML";		// Bisogno di traduzione
#elif L==5
	"%s: not HTML";		// Besoin de traduction
#elif L==6
	"%s: no HTML";		// Okoteve traduccin
#elif L==7
	"%s: not HTML";
#elif L==8
	"%s: not HTML";		// Potrzebujesz tlumaczenie
#elif L==9
	"%s: not HTML";		// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_quota_exceeded_NO_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"%s: cuota excedida.";		// Necessita traduccio
#elif L==2
	"%s: Kontingent berschritten.";
#elif L==3
	"%s: quota exceeded.";
#elif L==4
	"%s: cuota excedida.";
#elif L==5
	"%s: quota dpass.";
#elif L==6
	"%s: cuota excedida.";		// Okoteve traduccin
#elif L==7
	"%s: quota superata.";
#elif L==8
	"%s: przekroczono przydzial.";
#elif L==9
	"5s: quota excedida.";
#endif

const char *Txt_UPLOAD_FILE_Upload_time_too_long_maximum_X_minutes_NO_HTML =	// Warning: it is very important to include %lu in the following sentences
#if   L==1
	"Tiempo de subida demasiado largo (mximo %lu minutos).";	// Necessita traduccio
#elif L==2
	"Upload ist zu lang (Maximum %lu Minute).";
#elif L==3
	"Upload time too long (maximum %lu minutes).";
#elif L==4
	"Tiempo de subida demasiado largo (mximo %lu minutos).";
#elif L==5
	"Temps de tlchargement trop longs (maximum %lu minutes).";
#elif L==6
	"Tiempo de subida demasiado largo (mximo %lu minutos).";	// Okoteve traduccin
#elif L==7
	"Durata del caricamento troppo lungo (massimo %lu minuti).";
#elif L==8
	"Upload time too long (maximum %lu minuty).";			// Potrzebujesz tlumaczenie
#elif L==9
	"Tempo de upload muito longo (mximo %lu minutos).";
#endif

const char *Txt_user_NO_HTML[Usr_NUM_SEXS][1 + Txt_NUM_LANGUAGES] =	// Don't use HTML entities like &egrave; here
{
	{
	// Usr_SEX_UNKNOWN
	"",
	"usuari/a",
	"Benutzer",
	"user",
	"usuario/a",
	"utilisateur/se",
	"puruhra",
	"utente",
	"uzytkownika",
	"utilizador/a",
	},
	{
	// Usr_SEX_FEMALE
	"",
	"usuaria",
	"Benutzer",
	"user",
	"usuaria",
	"utilisateuse",
	"usuaria",	// Okoteve traduccin
	"utente",
	"uzytkownika",
	"utilizadora",
	},
	{
	// Usr_SEX_MALE
	"",
	"usuari",
	"Benutzer",
	"user",
	"usuario",
	"utilisateur",
	"puruhra",
	"utente",
	"uzytkownika",
	"utilizador",
	},
	{
	// Usr_SEX_ALL
	"",
	"usuari/a",
	"Benutzer",
	"user",
	"usuario/a",
	"utilisateur/se",
	"puruhra",
	"utente",
	"uzytkownika",
	"utilizador/a",
	}
};

const char *Txt_Users_NO_HTML =
#if   L==1
	"Usuaris";
#elif L==2
	"Benutzer";
#elif L==3
	"Users";
#elif L==4
	"Usuarios";
#elif L==5
	"Utilisateurs";
#elif L==6
	"Puruh&aacute;ra";
#elif L==7
	"Utenti";
#elif L==8
	"Uzytkownikw";
#elif L==9
	"Utilizadores";
#endif

/***** Messages translated (use HTML entities) *****/

const char *Txt_A_face_marked_in_green_has_been_detected_ =
#if   L==1
	"S'ha detectat un rostre (senyalat en verd) en posici&oacute; frontal"
	" i amb el fons prou clar.<br />"
	"Premeu sobre ell per confirmar la actualizaci&oacute; de la fotografia.";
#elif L==2
	"Ein Gesicht (gr&uuml;n markiert) wurde vor Position erfasst"
	" und der Hintergrund ist genug hell.<br />"
	"Klicken Sie darauf, um das Update des Bildes zu best&auml;tigen.";
#elif L==3
	"A face (marked in green) has been detected in front position"
	" and the background is light enough.<br />"
	"Click on it to confirm the update of the picture.";
#elif L==4
	"Se ha detectado un rostro (se&ntilde;alado en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Pulse sobre &eacute;l para confirmar la actualizaci&oacute;n de la fotograf&iacute;a.";
#elif L==5
	"Un visage (marqu&eacute; en vert) a &eacute;t&eacute; d&eacute;tect&eacute; en position avant"
	" et l'arri&egrave;re-plan est assez clair.<br />"
	 "Cliquez sur pour confirmer la mise &agrave; jour de l'image.";
#elif L==6
	"Se ha detectado un rostro (se&ntilde;alado en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Pulse sobre &eacute;l para confirmar la actualizaci&oacute;n de la fotograf&iacute;a.";	// Okoteve traduccin
#elif L==7
	"Un viso (segnato in verde) &egrave; stato individuato nella posizione centrale"
	" e lo sfondo &egrave; abbastanza chiaro.<br />"
	"Clicca su di esso per confermare l'aggiornamento dell'immagine.";
#elif L==8
	"Twarz (zaznaczone na zielono) zosta&lstrok; wykryty w przednim po&lstrok;o&zdot;eniu,"
	" a t&lstrok;o jest wystarczaj&aogon;co du&zdot;o &sacute;wiat&lstrok;a.<br />"
	"Kliknij go, aby potwierdzi&cacute; aktualizacj&eogon; obrazu.";
#elif L==9
	"Um rosto (marcados em verde) foi detectado em posi&ccedil;&atilde;o frontal"
	" eo fundo &eacute; claro o suficiente.<br />"
	"Clique nele para confirmar a atualiza&ccedil;&atilde;o da imagem.";
#endif

const char *Txt_A_face_marked_in_red_has_been_detected_ =
#if   L==1
	"S'ha detectat un rostre (senyalat en vermell) en posici&oacute; frontal,"
	" per&ograve; el fons &eacute;s massa fosc.";
#elif L==2
	"Ein Gesicht (rot markiert) wurde vor Position erkannt,"
	" aber der Hintergrund ist zu dunkel.";
#elif L==3
	"A face (marked in red) has been detected in front position,"
	" but the background is too dark.";
#elif L==4
	"Se ha detectado un rostro (se&ntilde;alado en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";
#elif L==5
	"Un visage (marqu&eacute; en rouge) a &eacute;t&eacute; d&eacute;tect&eacute; en position avant,"
	" mais le fond est trop sombre.";
#elif L==6
	"Se ha detectado un rostro (se&ntilde;alado en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";	// Okoteve traduccin
#elif L==7
	"Un viso (segnato in rosso) &egrave; stato individuato nella posizione centrale,"
	" ma lo sfondo &egrave; troppo scuro.";
#elif L==8
	"Twarz (zaznaczone na czerwono) zosta&lstrok; wykryty w przednim po&lstrok;o&zdot;eniu,"
	" ale t&lstrok;o jest zbyt ciemne.";
#elif L==9
	"Uma face (marcada a vermelho) foi detectada na posi&ccedil;&atilde;o da frente,"
	" mas o plano de fundo &eacute; demasiado escuro.";
#endif

const char *Txt_A_message_has_been_sent_to_email_address_X_to_confirm_that_address =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Se ha enviado un mensaje"
	" a la direcci&oacute;n de correo <strong>%s</strong>"
	" para confirmar dicha direcci&oacute;n.";	// Necessita traduccio
#elif L==2
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";			// Need bersetzung
#elif L==3
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";
#elif L==4
	"Se ha enviado un mensaje"
	" a la direcci&oacute;n de correo <strong>%s</strong>"
	" para confirmar dicha direcci&oacute;n.";
#elif L==5
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";			// Besoin de traduction
#elif L==6
	"Se ha enviado un mensaje"
	" a la direcci&oacute;n de correo <strong>%s</strong>"
	" para confirmar dicha direcci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";			// Bisogno di traduzione
#elif L==8
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";			// Potrzebujesz tlumaczenie
#elif L==9
	"A message has been sent"
	" to email address <strong>%s</strong>"
	" to confirm that address.";			// Necessita de traduo
#endif

const char *Txt_A_student_can_belong_to_several_groups =
#if   L==1
	"Un estudiant pot pert&agrave;nyer a diversos grups";
#elif L==2
	"Ein Student kann mehreren Gruppen angehren";
#elif L==3
	"A student can belong to several groups";
#elif L==4
	"Un estudiante puede pertenecer a varios grupos";
#elif L==5
	"Un &eacute;tudiant peut appartenir &agrave; plusieurs groupes";
#elif L==6
	"Un estudiante puede pertenecer a varios grupos";	// Okoteve traduccin
#elif L==7
	"Uno studente pu&ograve; appartenere a pi&ugrave; gruppi";
#elif L==8
	"Kazdy student moze nalezec do wielu grup";
#elif L==9
	"Um aluno pode pertencer a v&aacute;rios grupos";
#endif

const char *Txt_A_student_can_only_belong_to_one_group =
#if   L==1
	"Un estudiant nom&eacute;s pot pert&agrave;nyer a un grup";
#elif L==2
	"Ein Student kann nur einer Gruppe angeh&ouml;ren";
#elif L==3
	"A student can only belong to one group";
#elif L==4
	"Un estudiante solo puede pertenecer a un grupo";
#elif L==5
	"Un &eacute;tudiant ne peut appartenir &agrave; un groupe";
#elif L==6
	"Un estudiante solo puede pertenecer a un grupo";	// Okoteve traduccin
#elif L==7
	"Uno studente pu&ograve; appartenere solo a un gruppo";
#elif L==8
	"student moze nalezec tylko do jednej grupy";
#elif L==9
	"Um aluno pode pertencer a um grupo";
#endif

const char *Txt_A_teacher_or_administrator_has_enrolled_you_as_X_into_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"Un professor/a o un administrador/a l'ha inscrit"
	" com a <strong>%s</strong>"
	" en l'assignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;"
	" o prefereix ser eliminat de l'assignatura?";
#elif L==2
	"Ein Lehrer oder ein Administrator hat Sie"
	" als <strong>%s</strong>"
	" in den Kurs <strong>%s</strong> eingeschrieben."
	" M&ouml;chten Sie die Anmeldung best&auml;tigen"
	" oder ziehen Sie es vor,"
	" aus dem Kurs entfernt zu werden?";
#elif L==3
	"A teacher or an administrator has enrolled you"
	" as a <strong>%s</strong>"
	"into the course <strong>%s</strong>."
	" Do you want to confirm the enrollment"
	" or do you prefer to be removed from the course?";
#elif L==4
	"Un profesor/a o un administrador/a le ha inscrito"
	" como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;n"
	" o prefiere ser eliminado/a de la asignatura?";
#elif L==5
	"Un professeur ou un administrateur vous a enregistr&eacute;"
	" en tant que <strong>%s</strong>"
	" dans la mati&egrave;re <strong>%s</strong>."
	" Voulez-vous confirmer l'inscription"
	" ou pr&eacute;f&eacute;rez-vous &ecirc;tre retir&eacute; de la mati&egrave;re?";
#elif L==6
	"Un profesor/a o un administrador/a le ha inscrito"
	" como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>."
	" &iquest;Confirma la inscripci&oacute;n"
	" o prefiere ser eliminado/a de la asignatura?";		// Okoteve traduccin
#elif L==7
	"Un insegnante o amministratore ti a registrato"
	" come <strong>%s</strong>"
	" nel corso <strong>%s</strong>."
	" Vuoi confermare la registrazione"
	" o preferisci essere rimosso da questo corso?";
#elif L==8
	"Nauczyciel lub administrator zapisa&lstrok; ci&eogon;"
	" jako <strong>%s</strong>"
	" do kursu <strong>%s</strong>."
	" Czy chcesz, aby potwierdzi&cacute; rejestracj&eogon;"
	" czy wolisz zosta&cacute; usuni&eogon;te z kursu?";
#elif L==9
	"Um professor/a ou administrador/a tem registrado voc&ecirc;"
	" como <strong>%s</strong>"
	" na disciplina <strong>%s</strong>."
	" Voc&ecirc; quer para confirmar a inscri&ccedil;&atilde;o"
	" ou voc&ecirc; prefere para ser removido/a da disciplina?";
#endif

const char *Txt_A_user_can_not_have_more_than_X_IDs =
#if   L==1
	"Un usuari no pot tenir m&eacute;s de %u IDs (DNI/c&egrave;dulas)";
#elif L==2
	"Ein Benutzer kann nicht mehr als %u Ausweis-Nr.";
#elif L==3
	"A user can not have more than %u IDs.";
#elif L==4
	"Un usuario no puede tener m&aacute;s de %u ID (DNI/c&eacute;dulas).";
#elif L==5
	"Un utilisateur ne peut pas avoir plus de %u IDs.";
#elif L==6
	"Un usuario no puede tener m&aacute;s de %u ID (DNI/c&eacute;dulas).";	// Okoteve traduccin
#elif L==7
	"Un utente non pu&ograve; avere pi&ugrave; di %u ID.";
#elif L==8
	"U&zdot;ytkownik nie mo&zdot;e mie&cacute; wi&eogon;cej ni&zdot; %u identyfikatory.";
#elif L==9
	"Um usu&aacute;rio n&atilde;o pode ter mais de %u n&ordm;s. identif.";
#endif

const char *Txt_About_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Quant a %s";
#elif L==2
	"&Uuml;ber uns %s";
#elif L==3
	"About %s";
#elif L==4
	"Acerca de %s";
#elif L==5
	"&Agrave; propos de %s";
#elif L==6
	"%s rehegua";
#elif L==7
	"Riguardo %s";
#elif L==8
	"O %s";
#elif L==9
	"Sobre %s";
#endif

const char *Txt_Absent =
#if   L==1
	"Absent";
#elif L==2
	"Nicht vorhanden";
#elif L==3
	"Absent";
#elif L==4
	"Ausente";
#elif L==5
	"Absent";
#elif L==6
	"Ausente";	// Okoteve traduccin
#elif L==7
	"Assente";
#elif L==8
	"Nieobecny";
#elif L==9
	"Assente";
#endif

const char *Txt_Absents =
#if   L==1
	"Absents";
#elif L==2
	"Nicht vorhanden";
#elif L==3
	"Absents";
#elif L==4
	"Ausentes";
#elif L==5
	"Absents";
#elif L==6
	"Ausentes";	// Okoteve traduccin
#elif L==7
	"Assenti";
#elif L==8
	"Nieobecny";
#elif L==9
	"Assentes";
#endif

const char *Txt_accessible_for_reading_and_writing_by_administrators_of_the_centre =
#if   L==1
	"accessible per a lectura i escriptura per administradors del centre";
#elif L==2
	"zum Schreiben zug&auml;nglich f&uuml;r Administratoren der Lehrinstitut";
#elif L==3
	"accessible for reading and writing by administrators of the centre";
#elif L==4
	"accesible para lectura y escritura por administradores del centro";
#elif L==5
	"accessible en lecture et en &eacute;criture par les administrateurs du centre";
#elif L==6
	"accesible para lectura y escritura por administradores del centro";	// Okoteve traduccin
#elif L==7
	"accessibile per la lettura e la scrittura da parte dei amministratori del centro";
#elif L==8
	"dostep do odczytu i zapisu przez administratorzy centrum";
#elif L==9
	"acess&iacute;vel para leitura e escrita pelos administradores do centro";
#endif

const char *Txt_accessible_for_reading_and_writing_by_administrators_of_the_degree =
#if   L==1
	"accessible per a lectura i escriptura per administradors de la titulaci&oacute;";
#elif L==2
	"zum Schreiben zug&auml;nglich f&uuml;r Administratoren der Studiengang";
#elif L==3
	"accessible for reading and writing by administrators of the degree";
#elif L==4
	"accesible para lectura y escritura por administradores de la titulaci&oacute;n";
#elif L==5
	"accessible en lecture et en &eacute;criture par les administrateurs de l'&eacute;tude";
#elif L==6
	"accesible para lectura y escritura por administradores de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"accessibile per la lettura e la scrittura da parte dei amministratori della laurea";
#elif L==8
	"dostep do odczytu i zapisu przez administratorzy stopnia";
#elif L==9
	"acess&iacute;vel para leitura e escrita pelos administradores da titula&ccedil;&atilde;o";
#endif

const char *Txt_accessible_for_reading_and_writing_by_administrators_of_the_institution =
#if   L==1
	"accessible per a lectura i escriptura per administradors de l'instituci&oacute;";
#elif L==2
	"zum Schreiben zug&auml;nglich f&uuml;r Administratoren der Hochschule";
#elif L==3
	"accessible for reading and writing by administrators of the institution";
#elif L==4
	"accesible para lectura y escritura por administradores de la instituci&oacute;n";
#elif L==5
	"accessible en lecture et en &eacute;criture par les administrateurs du &eacute;tablissement";
#elif L==6
	"accesible para lectura y escritura por administradores de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7
	"accessibile per la lettura e la scrittura da parte dei amministratori della istituzione";
#elif L==8
	"dostep do odczytu i zapisu przez administratorzy instytucji";
#elif L==9
	"acess&iacute;vel para leitura e escrita pelos administradores da institui&ccedil;&atilde;o";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_centre =
#if   L==1
	"accessible per a lectura i escriptura per estudiants i professors del centre";
#elif L==2
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Lehrinstitut";
#elif L==3
	"accessible for reading and writing by students and teachers of the centre";
#elif L==4
	"accesible para lectura y escritura por estudiantes y profesores del centro";
#elif L==5
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants du centre";
#elif L==6
	"accesible para lectura y escritura por estudiantes y profesores del centro";	// Okoteve traduccin
#elif L==7
	"accessibile per la lettura e la scrittura da parte degli studenti e dei professori del centro";
#elif L==8
	"dostep do odczytu i zapisu przez uczni&oacute;w i nauczycieli centrum";
#elif L==9
	"acess&iacute;vel para leitura e escrita pelos alunos e professores do centro";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_course =
#if   L==1
	"accessible per a lectura i escriptura per estudiants i professors de l'assignatura";
#elif L==2
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Kurs";
#elif L==3
	"accessible for reading and writing by students and teachers of the course";
#elif L==4
	"accesible para lectura y escritura por estudiantes y profesores de la asignatura";
#elif L==5
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==6
	"accesible para lectura y escritura por estudiantes y profesores de la asignatura";	// Okoteve traduccin
#elif L==7
	"accessibile per la lettura e la scrittura da parte degli studenti e dei professori del corso";
#elif L==8
	"dostep do odczytu i zapisu przez uczni&oacute;w i nauczycieli zajec";
#elif L==9
	"acess&iacute;vel para leitura e escrita pelos alunos e professores da disciplina";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_degree =
#if   L==1
	"accessible per a lectura i escriptura per estudiants i professors de la titulaci&oacute;";
#elif L==2
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Studiengang";
#elif L==3
	"accessible for reading and writing by students and teachers of the degree";
#elif L==4
	"accesible para lectura y escritura por estudiantes y profesores de la titulaci&oacute;n";
#elif L==5
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants de l'&eacute;tude";
#elif L==6
	"accesible para lectura y escritura por estudiantes y profesores de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"accessibile per la lettura e la scrittura da parte degli studenti e dei professori della laurea";
#elif L==8
	"dostep do odczytu i zapisu przez uczni&oacute;w i nauczycieli stopnia";
#elif L==9
	"acess&iacute;vel para leitura e escrita pelos alunos e professores da titula&ccedil;&atilde;o";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_and_teachers_of_the_institution =
#if   L==1
	"accessible per a lectura i escriptura per estudiants i professors de la instituci&oacute;";
#elif L==2
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Hochschule";
#elif L==3
	"accessible for reading and writing by students and teachers of the institution";
#elif L==4
	"accesible para lectura y escritura por estudiantes y profesores de la instituci&oacute;n";
#elif L==5
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants et les enseignants du &eacute;tablissement";
#elif L==6
	"accesible para lectura y escritura por estudiantes y profesores de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7
	"accessibile per la lettura e la scrittura da parte degli studenti e dei professori della istituzione";
#elif L==8
	"dostep do odczytu i zapisu przez uczni&oacute;w i nauczycieli instytucji";
#elif L==9
	"acess&iacute;vel para leitura e escrita pelos alunos e professores da institui&ccedil;&atilde;o";
#endif

const char *Txt_accessible_for_reading_and_writing_by_students_of_the_group_and_teachers_of_the_course =
#if   L==1
	"accesible per a lectura i escriptura per estudiants del grup i professors de l'assignatura";
#elif L==2
	"zum Schreiben zug&auml;nglich f&uuml;r Studenten der Gruppe und Lehrkr&auml;fte der Kurs";
#elif L==3
	"accessible for reading and writing by students of the group and teachers of the course";
#elif L==4
	"accesible para lectura y escritura por estudiantes del grupo y profesores de la asignatura";
#elif L==5
	"accessible en lecture et en &eacute;criture par les &eacute;tudiants du groupe et les enseignants de la mati&egrave;re";
#elif L==6
	"accesible para lectura y escritura por estudiantes del grupo y profesores de la asignatura";	// Okoteve traduccin
#elif L==7
	"accessibile per la lettura e la scrittura da parte degli studenti del gruppo e dei professori del corso";
#elif L==8
	"dostep do odczytu i zapisu przez student&oacute;w i nauczycieli, grupy kursu";
#elif L==9
	"acess&iacute;vel para leitura e escrita pelos alunos do grupo e professores da disciplina";
#endif

const char *Txt_accessible_for_reading_and_writing_by_teachers_of_the_course =
#if   L==1
	"accessible per a lectura i escriptura per professors de l'assignatura";
#elif L==2
	"zum Schreiben zug&auml;nglich f&uuml;r Lehrkr&auml;fte der Kurs";
#elif L==3
	"accessible for reading and writing by teachers of the course";
#elif L==4
	"accesible para lectura y escritura por profesores de la asignatura";
#elif L==5
	"accessible en lecture et en &eacute;criture par les enseignants de la mati&egrave;re";
#elif L==6
	"accesible para lectura y escritura por profesores de la asignatura";	// Okoteve traduccin
#elif L==7
	"accessibile per la lettura e la scrittura da parte dei professori del corso";
#elif L==8
	"dostep do odczytu i zapisu przez nauczycieli zajec";
#elif L==9
	"acess&iacute;vel para leitura e escrita pelos professores da disciplina";
#endif

const char *Txt_accessible_for_reading_and_writing_by_you_and_the_teachers_of_the_course =
#if   L==1
	"accessible per a lectura per vost&egrave; i els professors de l'assignatura";
#elif L==2
	"zum Schreiben zug&auml;nglich f&uuml;r Sie und die Lehrkr&auml;fte der Kurs";
#elif L==3
	"accessible for reading and writing by you and the teachers of the course";
#elif L==4
	"accesible para lectura por usted y los profesores de la asignatura";
#elif L==5
	"accessible en lecture et en &eacute;criture par vous et par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==6
	"accesible para lectura por usted y los profesores de la asignatura";	// Okoteve traduccin
#elif L==7
	"accessibile per la lettura e la scrittura per te e i professori del corso";
#elif L==8
	"dostep do odczytu i zapisu przez Ciebie i nauczycieli zajec";
#elif L==9
	"acess&iacute;vel para leitura e escrita por voc&ecirc; e os professores da disciplina";
#endif

const char *Txt_accessible_only_for_reading_by_you_and_the_teachers_of_the_course =
#if   L==1
	"accesible nom&egrave; per a lectura per vost&eacute; i els professors de l'assignatura";
#elif L==2
	"zum Lesen zug&auml;nglich f&uuml;r Sie und die Lehrkr&auml;fte der Kurs";
#elif L==3
	"accessible only for reading by you and the teachers of the course";
#elif L==4
	"accesible solo para lectura por usted y los profesores de la asignatura";
#elif L==5
	"accessible uniquement pour la lecture par vous et par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==6
	"accesible solo para lectura por usted y los profesores de la asignatura";	// Okoteve traduccin
#elif L==7
	"accessibile solo per la lettura per te e i professori del corso";
#elif L==8
	"dost&eogon;pne tylko do czytania przez Ciebie i nauczycieli prowadzacych ten kurs";
#elif L==9
	"acess&iacute;vel apenas para leitura por voc&ecirc; e os professores da disciplina";
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_centre =
#if   L==1
	"accessible nom&eacute;s per a lectura per estudiants i professors del centre";
#elif L==2
	"zum Lesen zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Lehrinstitut";
#elif L==3
	"accessible only for reading by students and teachers of the centre";
#elif L==4
	"accesible solo para lectura por estudiantes y profesores del centro";
#elif L==5
	"accessible uniquement pour la lecture par les &eacute;tudiants et les enseignants du centre";
#elif L==6
	"accesible solo para lectura por estudiantes y profesores del centro";	// Okoteve traduccin
#elif L==7
	"accessibile solo per la lettura da parte degli studenti e dei professori del centro";
#elif L==8
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli centrum";
#elif L==9
	"acess&iacute;vel apenas para leitura pelos alunos e professores do centro";
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_course =
#if   L==1
	"accessible nom&eacute;s per a lectura per estudiants i professors de l'assignatura";
#elif L==2
	"zum Lesen zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Kurs";
#elif L==3
	"accessible only for reading by students and teachers of the course";
#elif L==4
	"accesible solo para lectura por estudiantes y profesores de la asignatura";
#elif L==5
	"accessible uniquement pour la lecture par les &eacute;tudiants et les enseignants de la mati&egrave;re";
#elif L==6
	"accesible solo para lectura por estudiantes y profesores de la asignatura";	// Okoteve traduccin
#elif L==7
	"accessibile solo per la lettura da parte degli studenti e dei professori del corso";
#elif L==8
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli zajec";
#elif L==9
	"acess&iacute;vel apenas para leitura pelos alunos e professores da disciplina";
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_degree =
#if   L==1
	"accessible nom&eacute;s per a lectura per estudiants i professors de la titulaci&oacute;";
#elif L==2
	"zum Lesen zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Studiengang";
#elif L==3
	"accessible only for reading by students and teachers of the degree";
#elif L==4
	"accesible solo para lectura por estudiantes y profesores de la titulaci&oacute;n";
#elif L==5
	"accessible uniquement pour la lecture par les &eacute;tudiants et les enseignants de l'&eacute;tude";
#elif L==6
	"accesible solo para lectura por estudiantes y profesores de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"accessibile solo per la lettura da parte degli studenti e dei professori della laurea";
#elif L==8
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli stopnia";
#elif L==9
	"acess&iacute;vel apenas para leitura pelos alunos e professores da titula&ccedil;&atilde;o";
#endif

const char *Txt_accessible_only_for_reading_by_students_and_teachers_of_the_institution =
#if   L==1
	"accessible nom&eacute;s per a lectura per estudiants i professors de l'instituci&oacute;";
#elif L==2
	"zum Lesen zug&auml;nglich f&uuml;r Studenten und Lehrkr&auml;fte der Hochschule";
#elif L==3
	"accessible only for reading by students and teachers of the institution";
#elif L==4
	"accesible solo para lectura por estudiantes y profesores de la instituci&oacute;n";
#elif L==5
	"accessible uniquement pour la lecture par les &eacute;tudiants et les enseignants du &eacute;tablissement";
#elif L==6
	"accesible solo para lectura por estudiantes y profesores de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7
	"accessibile solo per la lettura da parte degli studenti e dei professori della istituzione";
#elif L==8
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli instytucji";
#elif L==9
	"acess&iacute;vel apenas para leitura pelos alunos e professores da institui&ccedil;&atilde;o";
#endif

const char *Txt_accessible_only_for_reading_by_students_of_the_group_and_teachers_of_the_course =
#if   L==1
	"accesible nom&eacute;s per a lectura per estudiants del grup i professors de l'assignatura";
#elif L==2
	"zum Lesen zug&auml;nglich f&uuml;r Studenten der Gruppe und Lehrkr&auml;fte der Kurs";
#elif L==3
	"accessible only for reading by students of the group and teachers of the course";
#elif L==4
	"accesible solo para lectura por estudiantes del grupo y profesores de la asignatura";
#elif L==5
	"accessible uniquement pour la lecture par les &eacute;tudiants du groupe et les enseignants de la mati&egrave;re";
#elif L==6
	"accesible solo para lectura por estudiantes del grupo y profesores de la asignatura";	// Okoteve traduccin
#elif L==7
	"accessibile solo per la lettura da parte degli studenti del gruppo e dei professori del corso";
#elif L==8
	"dost&eogon;pne tylko do czytania przez uczni&oacute;w i nauczycieli grupy kursu";
#elif L==9
	"acess&iacute;vel apenas para leitura pelos alunos do grupo e professores da disciplina";
#endif

const char *Txt_Action =
#if   L==1
	"Acci&oacute;";
#elif L==2
	"Handlung";
#elif L==3
	"Action";
#elif L==4
	"Acci&oacute;n";
#elif L==5
	"Action";
#elif L==6
	"Tembiapo";
#elif L==7
	"Azione";
#elif L==8
	"Dzia&lstrok;anie";
#elif L==9
	"A&ccedil;&atilde;o";
#endif

const char *Txt_Administer_me =
#if   L==1
	"Administrarme";
#elif L==2
	"Verwalten Sie mich";
#elif L==3
	"Administer me";
#elif L==4
	"Administrarme";
#elif L==5
	"G&eacute;rer moi";
#elif L==6
	"Administrarme";	// Okoteve traduccin
#elif L==7
	"Gestire mi";
#elif L==8
	"Zarz&aogon;dzaj mnie";
#elif L==9
	"Gerenciar me";
#endif

const char *Txt_Administer_multiple_students =
#if   L==1
	"Administrar diversos estudiants";
#elif L==2
	"Verwalten Sie mehrere Studenten";
#elif L==3
	"Administer multiple students";
#elif L==4
	"Administrar varios estudiantes";
#elif L==5
	"G&eacute;rer plusieurs &eacute;tudiants";
#elif L==6
	"Administrar varios estudiantes";	// Okoteve traduccin
#elif L==7
	"Gestire pi&ugrave; studenti";
#elif L==8
	"Zarz&aogon;dzaj wielu studenci";
#elif L==9
	"Gerenciar v&aacute;rios estudantes";
#endif

const char *Txt_Administer_multiple_teachers =
#if   L==1
	"Administrar diversos professors";
#elif L==2
	"Verwalten Sie mehrere Lehrkr&auml;fte";
#elif L==3
	"Administer multiple teachers";
#elif L==4
	"Administrar varios profesores";
#elif L==5
	"G&eacute;rer plusieurs enseignants";
#elif L==6
	"Administrar varios profesores";	// Okoteve traduccin
#elif L==7
	"Gestire pi&ugrave; professori";
#elif L==8
	"Zarz&aogon;dzaj wielu nauczycieli";
#elif L==9
	"Gerenciar v&aacute;rios professores";
#endif

const char *Txt_Administer_one_user =
#if   L==1
	"Administrar un usuari";
#elif L==2
	"Verwalten eines Benutzer";
#elif L==3
	"Administer one user";
#elif L==4
	"Administrar un usuario";
#elif L==5
	"G&eacute;rer un utilisateur";
#elif L==6
	"Administrar un usuario";	// Okoteve traduccin
#elif L==7
	"Gestire un utente";
#elif L==8
	"Zarz&aogon;dzaj u&zdot;ytkownika";
#elif L==9
	"Gerenciar um utilizador";
#endif

const char *Txt_Administer_user =
#if   L==1
	"Administrar usuari";
#elif L==2
	"Verwalten Benutzer";
#elif L==3
	"Administer user";
#elif L==4
	"Administrar usuario";
#elif L==5
	"G&eacute;rer utilisateur";
#elif L==6
	"Administrar usuario";	// Okoteve traduccin
#elif L==7
	"Gestire utente";
#elif L==8
	"Zarz&aogon;dzaj u&zdot;ytkownika";
#elif L==9
	"Gerenciar utilizador";
#endif

const char *Txt_all =
#if   L==1
	"tot";
#elif L==2
	"alle";
#elif L==3
	"all";
#elif L==4
	"todo";
#elif L==5
	"tout";
#elif L==6
	"opakatu";
#elif L==7
	"tutto";
#elif L==8
	"wszystko";
#elif L==9
	"tudo";
#endif

const char *Txt_All_announcements_of_exams =
#if   L==1
	 "Totes les convocat&ograve;ries d'ex&agrave;mens";
#elif L==2
	 "Alle Aufrufe f&uuml;r Pr&uuml;fung";
#elif L==3
	 "All announcements of exams";
#elif L==4
	 "Todas las convocatorias de ex&aacute;menes";
#elif L==5
	 "Tous les convocations &agrave; examens";
#elif L==6
	 "Todas las convocatorias de ex&aacute;menes";	// Okoteve traduccin
#elif L==7
	 "Tutti i appelli d'esame";
#elif L==8
	 "Wszystkie ogloszenia egzamin&oacute;w";
#elif L==9
	 "Todas as chamadas para exames";
#endif

const char *Txt_all_degrees =
#if   L==1
	"Totes les titulacions";
#elif L==2
	"all degrees";			// Need bersetzung
#elif L==3
	"all degrees";
#elif L==4
	"todas las titulaciones";
#elif L==5
	"tous les degr&eacute;s";
#elif L==6
	"todas las titulaciones";	// Okoteve traduccin
#elif L==7
	"tutte le lauree";
#elif L==8
	"wszystkich stopni";
#elif L==9
	"all degrees";			// Necessita de traduo
#endif

const char *Txt_all_files_inside_the_root_folder =
#if   L==1
	"tots els arxius dins de la carpeta arrel";
#elif L==2
	"alle Dateien im Stammordner";
#elif L==3
	"all files inside the root folder";
#elif L==4
	"todos los archivos dentro de la carpeta ra&iacute;z";
#elif L==5
	"tous les fichiers dans le r&eacute;pertoire racine";
#elif L==6
	"todos los archivos dentro de la carpeta ra&iacute;z";	// Okoteve traduccin
#elif L==7
	"tutti i file all'interno della cartella principale";
#elif L==8
	"wszystkie pliki znajduj&aogon;ce si&eogon; w folderze g&lstrok;&oacute;wnym";
#elif L==9
	"todos os arquivos dentro do diret&oacute;rio raiz";
#endif

const char *Txt_All_groups =
#if   L==1
	"Tots els grups";
#elif L==2
	"Alle Gruppen";
#elif L==3
	"All groups";
#elif L==4
	"Todos los grupos";
#elif L==5
	"Tous les groupes";
#elif L==6
	"Todos los grupos";	// Okoteve traduccin
#elif L==7
	"Tutti i gruppi";
#elif L==8
	"Wszystkie grupy";
#elif L==9
	"Todos os grupos";
#endif

const char *Txt_All_notices =
#if   L==1
	"Tots els avisos";
#elif L==2
	"Alle Ank&uuml;ndigungen";
#elif L==3
	"All notices";
#elif L==4
	"Todos los avisos";
#elif L==5
	"Tous les avis";
#elif L==6
	"Todos los avisos";	// Okoteve traduccin
#elif L==7
	"Tutti i avvisi";
#elif L==8
	"Wszystkie ogloszenia";
#elif L==9
	"Todos os avisos";
#endif

const char *Txt_All_places =
#if   L==1
	"Tots els llocs";
#elif L==2
	"Alle Standorte";
#elif L==3
	"All places";
#elif L==4
	"Todos los lugares";
#elif L==5
	"Tous les emplacements";
#elif L==6
	"Todos los lugares";	// Okoteve traduccin
#elif L==7
	"Tutte le localit&agrave;";
#elif L==8
	"Wszystkie miejsca";
#elif L==9
	"Todas as localiza&ccedil;oes";
#endif

const char *Txt_All_tags =		// Tags used in test
#if   L==1
	"Tots els descriptors";
#elif L==2
	"Alle Tags";
#elif L==3
	"All tags";
#elif L==4
	"Todos los descriptores";
#elif L==5
	"Tous les descripteurs";
#elif L==6
	"Todos los descriptores";	// Okoteve traduccin
#elif L==7
	"Tutte le etichette";
#elif L==8
	"Wszystkie tagi";
#elif L==9
	"Todos os descritores";
#endif

const char *Txt_All_types_of_answers =		// Answers of a test
#if   L==1
	"Tots els tipus de respostes";
#elif L==2
	"Alle Antworttypen";
#elif L==3
	"All types of answers";
#elif L==4
	"Todos los tipos de respuestas";
#elif L==5
	"Tous les types de r&eacute;ponses";
#elif L==6
	"Todos los tipos de respuestas";	// Okoteve traduccin
#elif L==7
	"Tutti i tipi di risposte";
#elif L==8
	"Wszystkie typy odpowiedzi";
#elif L==9
	"Todos os tipos de respostas";
#endif

const char *Txt_Allow_teachers_to_consult_this_test =
#if   L==1
	"Permetre als professors consultar aquest test";
#elif L==2
	"Erm&ouml;glichen es den Lehrern, diese Test zu konsultieren";
#elif L==3
	"Allow teachers to consult this test";
#elif L==4
	"Permitir a los profesores consultar este test";
#elif L==5
	"Permettre aux enseignants de consulter ce test";
#elif L==6
	"Permitir a los profesores consultar este test";	// Okoteve traduccin
#elif L==7
	"Lasciare gli insegnanti di consultare questo test";
#elif L==8
	"Pozwalaj&aogon; nauczycielom skonsultowa&cacute; ten test";
#elif L==9
	"Permitir que os professores de consultar este teste";
#endif

const char *Txt_Alphabetic_BR_code_BR_ISO_3166_1 =
#if   L==1
	"Codi<br />alfab&egrave;tic<br />ISO 3166-1";
#elif L==2
	"Alphabetischer<br />Code<br />ISO 3166-1";
#elif L==3
	"Alphabetic<br />code<br />ISO 3166-1";
#elif L==4
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";
#elif L==5
	"Code<br />alphab&eacute;tique<br />ISO 3166-1";
#elif L==6
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";	// Okoteve traduccin
#elif L==7
	"Codice<br />alfabetico<br />ISO 3166-1";
#elif L==8
	"Alfabetycznego<br />kodu<br />ISO 3166-1";
#elif L==9
	"C&oacute;d.<br />alfab&eacute;tico<br />ISO 3166-1";
#endif

const char *Txt_Already_existed_an_assignment_with_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Ja existia una activitat amb la carpeta <strong>%s</strong>.";
#elif L==2
	"Es gibt bereits einen Aufgabe im Verzeichnis <strong>%s</strong>.";
#elif L==3
	"There is already an assignment with folder <strong>%s</strong>.";
#elif L==4
	"Ya exist&iacute;a una actividad con la carpeta <strong>%s</strong>.";
#elif L==5
	"Il existe d&eacute;j&agrave; une activit&eacute; avec le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6
	"Ya exist&iacute;a una actividad con la carpeta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Esiste gi&agrave; un'attivit&agrave; con la cartella <strong>%s</strong>.";
#elif L==8
	"Jest juz zadanie z folderu <strong>%s</strong>.";
#elif L==9
	"J&aacute; existe uma atividade com o diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_Already_existed_an_assignment_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Ja existia una activitat amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2
	"Es gibt bereits einen Aufgabe mit dem Namen <strong>%s</strong>.";
#elif L==3
	"Already existed an assignment with the title <strong>%s</strong>.";
#elif L==4
	"Ya exist&iacute;a una actividad con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5
	"Il existe d&eacute;j&agrave; une activit&eacute; du titre <strong>%s</strong>.";
#elif L==6
	"Ya exist&iacute;a una actividad con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Esiste gi&agrave; un'attivit&agrave; con il titolo <strong>%s</strong>.";
#elif L==8
	"Istniala juz przydzialu z tytulu <strong>%s</strong>.";
#elif L==9
	"J&aacute; existe uma atividade com o t&iacute;tulo <strong>%s</strong>.";
#endif

const char *Txt_Already_existed_an_event_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Ja existia un esdeveniment amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2
	"Es gibt bereits einen Ereignis mit dem Namen <strong>%s</strong>.";
#elif L==3
	"Already existed an event with the title <strong>%s</strong>.";
#elif L==4
	"Ya exist&iacute;a un evento con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5
	"Il existe d&eacute;j&agrave; un &eacute;v&eacute;nement du titre <strong>%s</strong>.";
#elif L==6
	"Ya exist&iacute;a un evento con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Esiste gi&agrave; un evento con il titolo <strong>%s</strong>.";
#elif L==8
	"Istniala juz wydarzenie z tytulu <strong>%s</strong>.";
#elif L==9
	"J&aacute; existe um evento com o t&iacute;tulo <strong>%s</strong>.";
#endif

const char *Txt_Already_existed_a_survey_with_the_title_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Ja existia una enquesta amb el t&iacute;tol <strong>%s</strong>.";
#elif L==2
	"Es gibt bereits einen Umfrage mit dem Namen <strong>%s</strong>.";
#elif L==3
	"Already existed a survey with the title <strong>%s</strong>.";
#elif L==4
	"Ya exist&iacute;a una encuesta con el t&iacute;tulo <strong>%s</strong>.";
#elif L==5
	"Il existe d&eacute;j&agrave; un sondage du titre <strong>%s</strong>.";
#elif L==6
	"Ya exist&iacute;a una encuesta con el t&iacute;tulo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Esiste gi&agrave; un sondaggio con il titolo <strong>%s</strong>.";
#elif L==8
	"Already existed a survey with the title <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"J&aacute; existe um inqu&eacute;rito com o t&iacute;tulo <strong>%s</strong>.";
#endif

const char *Txt_and =
#if   L==1
	"i";
#elif L==2
	"und";
#elif L==3
	"and";
#elif L==4
	"y";
#elif L==5
	"et";
#elif L==6
	"y";	// Okoteve traduccin
#elif L==7
	"e";
#elif L==8
	"i";
#elif L==9
	"e";
#endif

const char *Txt_and_X_other_recipients =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"&hellip;i altres %u destinataris";
#elif L==2
	"&hellip;und %u andere Empf&auml;nger";
#elif L==3
	"&hellip;and %u other recipients";
#elif L==4
	"&hellip;y otros %u destinatarios";
#elif L==5
	"&hellip;et %u autres destinataires";
#elif L==6
	"&hellip;y otros %u destinatarios";	// Okoteve traduccin
#elif L==7
	"&hellip;e altri %u destinatari";
#elif L==8
	"&hellip;i %u innych odbiorc&oacute;w";
#elif L==9
	"&hellip;e %u outros destinat&aacute;rios";
#endif

const char *Txt_Announcements =
#if   L==1
	"Anuncis";
#elif L==2
	"Bekanntmachungen";
#elif L==3
	"Announcements";
#elif L==4
	"Anuncios";
#elif L==5
	"Annonces";
#elif L==6
	"Anuncios";	// Okoteve traduccin
#elif L==7
	"Annunci";
#elif L==8
	"Og&lstrok;oszenia";
#elif L==9
	"An&uacute;ncios";
#endif

const char *Txt_Announcement_of_exam_removed =
#if   L==1
	"Convocat&ograve;ria d'examen eliminada.";
#elif L==2
	"Aufruf zur Pr&uuml;fung entfernt.";
#elif L==3
	"Announcement of exam removed.";
#elif L==4
	"Convocatoria de examen eliminada.";
#elif L==5
	"Convocation pour l'examen supprim&eacute;.";
#elif L==6
	"Convocatoria de examen eliminada.";	// Okoteve traduccin
#elif L==7
	"Appello d'esame rimosso.";
#elif L==8
	"Ogloszenie egzamin usuniete.";
#elif L==9
	"Chamada para exame removida.";
#endif

const char *Txt_Announcements_of_exams =
#if   L==1
	 "Convocat&ograve;ries d'ex&agrave;mens";
#elif L==2
	 "Aufrufe f&uuml;r Pr&uuml;fung";
#elif L==3
	 "Announcements of exams";
#elif L==4
	 "Convocatorias de ex&aacute;menes";
#elif L==5
	 "Convocations &agrave; examens";
#elif L==6
	 "Convocatorias de ex&aacute;menes";	// Okoteve traduccin
#elif L==7
	 "Appelli d'esame";
#elif L==8
	 "Ogloszenia egzamin&oacute;w";
#elif L==9
	 "Chamadas para exames";
#endif

const char *Txt_Another_centre =
#if   L==1
	"Un altre centre";
#elif L==2
	"Ein weiteres Lehrinstitut";
#elif L==3
	"Another centre";
#elif L==4
	"Otro centro";
#elif L==5
	"Un autre centre";
#elif L==6
	"Otro centro";	// Okoteve traduccin
#elif L==7
	"Un altro centro";
#elif L==8
	"Innym centrum";
#elif L==9
	"Outro centro";
#endif

const char *Txt_Another_country =
#if   L==1
	"Un altre pa&iacute;s";
#elif L==2
	"Ein weiterer Land";
#elif L==3
	"Another country";
#elif L==4
	"Otro pa&iacute;s";
#elif L==5
	"Un autre pays";
#elif L==6
	"Otro pa&iacute;s";	// Okoteve traduccin
#elif L==7
	"Un altro paese";
#elif L==8
	"Kolejny kraj";
#elif L==9
	"Outro pa&iacute;s";
#endif

const char *Txt_Another_department =
#if   L==1
	"Un altre departament";
#elif L==2
	"Eine weitere Abteilung";
#elif L==3
	"Another department";
#elif L==4
	"Otro departamento";
#elif L==5
	"Une autre d&eacute;partement";
#elif L==6
	"Otro departamento";	// Okoteve traduccin
#elif L==7
	"Un altro dipartimento";
#elif L==8
	"Kolejny dzia&lstrok;";
#elif L==9
	"Outro departamento";
#endif

const char *Txt_Another_ID =
#if   L==1
	"Un altre ID (DNI/c&egrave;dula)";
#elif L==2
	"Ein anderer Ausweis-Nr.";
#elif L==3
	"Another ID";
#elif L==4
	"Otro ID (DNI/c&eacute;dula)";
#elif L==5
	"Un autre num&eacute;ro d'identit&eacute;";
#elif L==6
	"Otro ID (DNI/c&eacute;dula)";		// Okoteve traduccin
#elif L==7
	"Un'altra carta d'identit&agrave;";
#elif L==8
	"Another ID";				// Potrzebujesz tlumaczenie
#elif L==9
	"Outro n&ordm; de identif.";
#endif

const char *Txt_Another_user_s_profile =
#if   L==1
	"Perfil d'un altre usuari";
#elif L==2
	"Profil eines anderen Benutzers";
#elif L==3
	"Another user's profile";
#elif L==4
	"Perfil de otro usuario";
#elif L==5
	"Profil d'un autre utilisateur";
#elif L==6
	"Perfil de otro usuario";	// Okoteve traduccin
#elif L==7
	"Altro profilo di utente";
#elif L==8
	"Profil innego u&zdot;ytkownika";
#elif L==9
	"Perfil de outro usu&aacute;rio";
#endif

const char *Txt_Add_this_ID =
#if   L==1
	"Afegir aquest ID";
#elif L==2
	"F&uuml;gen Sie diese Ausweis-Nr.";
#elif L==3
	"Add this ID";
#elif L==4
	"A&ntilde;adir este ID";
#elif L==5
	"Ajouter ce num&eacute;ro d'identit&eacute;";
#elif L==6
	"A&ntilde;adir este ID";	// Okoteve traduccin
#elif L==7
	"Aggiungi questa carta d'identit&agrave;";
#elif L==8
	"Add this ID";			// Potrzebujesz tlumaczenie
#elif L==9
	"Adicionar este n&ordm; identif.";
#endif

const char *Txt_Announcement_created =
#if   L==1
	"Anunci creat.";
#elif L==2
	"Bekanntmachung erstellt.";
#elif L==3
	"Announcement created.";
#elif L==4
	"Anuncio creado.";
#elif L==5
	"Annonce cr&eacute;&eacute;.";
#elif L==6
	"Anuncio creado.";		// Okoteve traduccin
#elif L==7
	"Annuncio creato.";
#elif L==8
	"Og&lstrok;oszenia stworzony.";
#elif L==9
	"An&uacute;ncio criado.";
#endif

const char *Txt_Announcement_removed =
#if   L==1
	"Anunci eliminat.";
#elif L==2
	"Bekanntmachung entfernt.";
#elif L==3
	"Announcement removed.";
#elif L==4
	"Anuncio eliminado.";
#elif L==5
	"Annonce supprim&eacute;.";
#elif L==6
	"Anuncio eliminado.";	// Okoteve traduccin
#elif L==7
	"Annuncio rimosso.";
#elif L==8
	"Og&lstrok;oszenia usuniete.";
#elif L==9
	"An&uacute;ncio removido.";
#endif

const char *Txt_Another_institution =
#if   L==1
	"Una altra instituci&oacute;";
#elif L==2
	"Eine weitere Hochschule";
#elif L==3
	"Another institution";
#elif L==4
	"Otra instituci&oacute;n";
#elif L==5
	"Un autre &eacute;tablissement";
#elif L==6
	"Otra instituci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Un'altra istituzione";
#elif L==8
	"Kolejna instytucja";
#elif L==9
	"Outra institui&ccedil;&atilde;o";
#endif

const char *Txt_Another_place =
#if   L==1
	"Un altre lloc";
#elif L==2
	"Ein weiterer Standort";
#elif L==3
	"Another place";
#elif L==4
	"Otro lugar";
#elif L==5
	"Un autre emplacement";
#elif L==6
	"Otro lugar";	// Okoteve traduccin
#elif L==7
	"Un'altra localit&grave;";
#elif L==8
	"Innym miejscem";
#elif L==9
	"Outra localiza&ccedil;&atilde;o";
#endif

const char *Txt_Answer_survey =
#if   L==1
	"Respondre enquesta";
#elif L==2
	"Beantwortung Umfrage";
#elif L==3
	"Answer survey";
#elif L==4
	"Responder encuesta";
#elif L==5
	"R&eacute;pondre sondage";
#elif L==6
	"Responder encuesta";	// Okoteve traduccin
#elif L==7
	"Rispondere sondaggio";
#elif L==8
	"Badanie odpowiedzi";
#elif L==9
	"Responder inqu&eacute;rito";
#endif

const char *Txt_Answers =		// Answers of test
#if   L==1
	"Respostes";
#elif L==2
	"Antworten";
#elif L==3
	"Answers";
#elif L==4
	"Respuestas";
#elif L==5
	"R&eacute;ponses";
#elif L==6
	"&Ntilde;embohov&aacute;i";
#elif L==7
	"Risposte";
#elif L==8
	"Odpowiedzi";
#elif L==9
	"Respostas";
#endif

const char *Txt_any_course =
#if   L==1
	"qualsevol assignatura";
#elif L==2
	"alle Kursen";
#elif L==3
	"any course";
#elif L==4
	"cualquier asignatura";
#elif L==5
	"toute mati&egrave;re";
#elif L==6
	"cualquier asignatura";	// Okoteve traduccin
#elif L==7
	"qualsiasi corso";
#elif L==8
	"kurs";
#elif L==9
	"qualquer disciplina";
#endif

const char *Txt_Any_type_of_degree =
#if   L==1
	"Qualsevol tipus de titulaci&oacute;";
#elif L==2
	"Alle Abschlussart";
#elif L==3
	"Any type of degree";
#elif L==4
	"Cualquier tipo de titulaci&oacute;n";
#elif L==5
	"Tout type d'&eacute;tude";
#elif L==6
	"Cualquier tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Qualsiasi tipo di laurea";
#elif L==8
	"Any type of degree";			// Potrzebujesz tlumaczenie
#elif L==9
	"Qualquer tipo de titula&ccedil;&atilde;o";
#endif

const char *Txt_Any_department =
#if   L==1
	"Qualsevol departament";
#elif L==2
	"Alle Abteilung";
#elif L==3
	"Any department";
#elif L==4
	"Cualquier departamento";
#elif L==5
	"Tout d&eacute;partement";
#elif L==6
	"Cualquier departamento";	// Okoteve traduccin
#elif L==7
	"Qualsiasi dipartimento";
#elif L==8
	"Wszystkie dzia&lstrok;y";
#elif L==9
	"Qualquer departamento";
#endif

const char *Txt_any_user =
#if   L==1
	"qualsevol usuari";
#elif L==2
	"alle Benutzer";
#elif L==3
	"any user";
#elif L==4
	"cualquier usuario";
#elif L==5
	"n'importe qu'utilisateur";
#elif L==6
	"cualquier usuario";	// Okoteve traduccin
#elif L==7
	"qualsiasi utente";
#elif L==8
	"wszystkich uzytkownik&oacute;w";
#elif L==9
	"qualquer utilizador";
#endif

const char *Txt_Application_key =
#if   L==1
	"Clau de aplicaci&oacute;";
#elif L==2
	"Application key";		// Need bersetzung
#elif L==3
	"Application key";
#elif L==4
	"Clave de aplicaci&oacute;n";
#elif L==5
	"Cl&eacute; d'application";
#elif L==6
	"Clave de aplicaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Codice di applicazione";
#elif L==8
	"Klawisz aplikacji";
#elif L==9
	"Application key";		// Necessita de traduo
#endif

const char *Txt_Assessment_criteria =
#if   L==1
	"Criteris d'avaluaci&oacute;";
#elif L==2
	"Bewertungskriterien";
#elif L==3
	"Assessment criteria";
#elif L==4
	"Criterios de evaluaci&oacute;n";
#elif L==5
	"Crit&egrave;res d'&eacute;valuation";
#elif L==6
	"Criterios de evaluaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Criteri di valutazione";
#elif L==8
	"Kryteria oceny";
#elif L==9
	"Crit&eacute;rios de avalia&ccedil;&atilde;o";
#endif

const char *Txt_Assignment =
#if   L==1
	"Activitat";
#elif L==2
	"Aufgabe";
#elif L==3
	"Assignment";
#elif L==4
	"Actividad";
#elif L==5
	"Activit&eacute;";
#elif L==6
	"Tar&eacute;a";
#elif L==7
	"Attivit&agrave;";
#elif L==8
	"Przyporzadkowanie";
#elif L==9
	"Atividade";
#endif

const char *Txt_ASSIGNMENT_TYPES[Asg_NUM_TYPES_SEND_WORK] =
	{
#if   L==1
	"No cal enviar treballs (fitxers)"
#elif L==2
	"Keine Hausarbeiten vorgesehen (Dateien)"
#elif L==3
	"No homework to be submitted (files)"
#elif L==4
	"No hay que enviar trabajos (archivos)"
#elif L==5
	"Ne soumettez pas de travail (fichiers)"
#elif L==6
	"No hay que enviar trabajos (archivos)"	// Okoteve traduccin
#elif L==7
	"Non ci sono lavori da presentare (files)"
#elif L==8
	"Nie ma pracy domowej, kt&oacute;re nalezy przedlozyc (pliki)"
#elif L==9
	"N&atilde;o submeter trabalhos (arquivos)"
#endif
	,
#if   L==1
	"Es obligatori enviar treballs (fitxers)"
#elif L==2
	"Hausarbeiten vorgesehen (files)"
#elif L==3
	"It is mandatory to submit homework (files)"
#elif L==4
	"Es obligatorio enviar trabajos (archivos)"
#elif L==5
	"Il est obligatoire de soumetter le travail (fichiers)"
#elif L==6
	"Es obligatorio enviar trabajos (archivos)"// Okoteve traduccin
#elif L==7
	"E' obbligatorio presentare il lavoro (files)"
#elif L==8
	"Jest to obowiazkowe do przedstawienia prac domowych (pliki)"
#elif L==9
	"&Eacute; obrigat&oacute;ria a apresenta&ccedil;&atilde;o de trabalhos (arquivos)"
#endif
	};

const char *Txt_Assignment_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La activitat <strong>%s</strong> ara est&agrave; oculta.";
#elif L==2
	"Die Aufgabe <strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==3
	"Assignment <strong>%s</strong> is now hidden.";
#elif L==4
	"La actividad <strong>%s</strong> ahora est&aacute; oculta.";
#elif L==5
	"L'activit&eacute; <strong>%s</strong> est maintenant cach&eacute;e.";
#elif L==6
	"La actividad <strong>%s</strong> ahora est&aacute; oculta.";	// Okoteve traduccin
#elif L==7
	"L'attivit&agrave; <strong>%s</strong> &egrave; ora nascosta.";
#elif L==8
	"Zadanie <strong>%s</strong> jest ukryte.";
#elif L==9
	"A atividade <strong>%s</strong> &eacute; agora oculta.";
#endif

const char *Txt_Assignment_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La activitat <strong>%s</strong> ara est&agrave; visible.";
#elif L==2
	"Die Aufgabe <strong>%s</strong> ist jetzt sichtbar.";
#elif L==3
	"Assignment <strong>%s</strong> is now visible.";
#elif L==4
	"La actividad <strong>%s</strong> ahora est&aacute; visible.";
#elif L==5
	"L'activit&eacute; <strong>%s</strong> est maintenant visible.";
#elif L==6
	"La actividad <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7
	"L'attivit&agrave; <strong>%s</strong> &egrave; ora visibile.";
#elif L==8
	"Zadanie <strong>%s</strong> jest teraz widoczny.";
#elif L==9
	"A atividade <strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_Assignment_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Activitat <strong>%s</strong> eliminada.";
#elif L==2
	"Aufgabe <strong>%s</strong> entfernt.";
#elif L==3
	"Assignment <strong>%s</strong> removed.";
#elif L==4
	"Actividad <strong>%s</strong> eliminada.";
#elif L==5
	"Activit&eacute; <strong>%s</strong> supprim&eacute;e.";
#elif L==6
	"Actividad <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7
	"Attivit&agrave; <strong>%s</strong> rimossa.";
#elif L==8
	"Zadanie <strong>%s</strong> usuniete.";
#elif L==9
	"Atividade <strong>%s</strong> removida.";
#endif

const char *Txt_Assignments =
#if   L==1
	"Activitats";
#elif L==2
	"Aufgaben";
#elif L==3
	"Assignments";
#elif L==4
	"Actividades";
#elif L==5
	"Activit&eacute;s";
#elif L==6
	"Tar&eacute;a";
#elif L==7
	"Attivit&agrave;";
#elif L==8
	"Zadania";
#elif L==9
	"Atividades";
#endif

const char *Txt_Assignments_and_other_works =
#if   L==1
	"Activitats i altres treballs";
#elif L==2
	"Aufgaben und andere Hausarbeiten";
#elif L==3
	"Assignments and other works";
#elif L==4
	"Actividades y otros trabajos";		// Okoteve traduccin
#elif L==5
	"Activit&eacute;s et autres travaux";
#elif L==6
	"Actividades y otros trabajos";
#elif L==7
	"Attivit&agrave; e altri lavori";
#elif L==8
	"Assignments and other works";		// Potrzebujesz tlumaczenie
#elif L==9
	"Atividades e outros trabalhos";
#endif

const char *Txt_START_END_TIME_HELP[Dat_NUM_START_END_TIME] =
   {
#if   L==1
	 "Ordenar per data de comen&ccedil;ament"
#elif L==2
	 "Sortieren nach Startdatum"
#elif L==3
	 "Sort by start date"
#elif L==4
	 "Ordenar por fecha de comienzo"
#elif L==5
	 "Trier par date de d&eacute;but"
#elif L==6
	 "Ordenar por fecha de comienzo"	// Okoteve traduccin
#elif L==7
	 "Ordina per data di inizio"
#elif L==8
	 "Sortuj wedlug daty rozpoczecia"
#elif L==9
	 "Classificar por data de in&iacute;cio"
#endif
	 ,
#if   L==1
	 "Ordenar per data de finalitzaci&oacute;"
#elif L==2
	 "Sortieren nach Enddatum"
#elif L==3
	 "Sort by end date"
#elif L==4
	 "Ordenar por fecha de finalizaci&oacute;n"
#elif L==5
	 "Trier par date de fin"
#elif L==6
	 "Ordenar por fecha de finalizaci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Ordina per data di fine"
#elif L==8
	 "Sortuj wedlug daty zakonczenia"
#elif L==9
	 "Classificar por data de t&eacute;rmino"
#endif
	};

const char *Txt_START_END_TIME[Dat_NUM_START_END_TIME] =
   {
#if   L==1
	 "Inici"
#elif L==2
	 "Start"
#elif L==3
	 "Start"
#elif L==4
	 "Inicio"
#elif L==5
	 "D&eacute;but"
#elif L==6
	 "&Ntilde;epyr&utilde;"
#elif L==7
	 "Inizio"
#elif L==8
	 "Pocz&aogon;tek"
#elif L==9
	 "In&iacute;cio"
#endif
	 ,
#if   L==1
	 "Final"
#elif L==2
	 "Ende"
#elif L==3
	 "End"
#elif L==4
	 "Final"
#elif L==5
	 "Fin"
#elif L==6
	 "Paha"
#elif L==7
	 "Fine"
#elif L==8
	 "Koniec"
#elif L==9
	 "Fim"
#endif
	};

const char *Txt_Assignments_area =
#if   L==1
	"Zona d'activitats";
#elif L==2
	"Aufgaben-Bereich";
#elif L==3
	"Assignments area";
#elif L==4
	"Zona de actividades";
#elif L==5
	"Zone d'activit&eacute;s";
#elif L==6
	"Zona de actividades";	// Okoteve traduccin
#elif L==7
	"Area di attivit&agrave;";
#elif L==8
	"Obszarze Zadania";
#elif L==9
	"Zona de atividades";
#endif

const char *Txt_assignments_area =
#if   L==1
	"zona d'activitats";	// Necessita traduccio
#elif L==2
	"Aufgaben-Bereich";
#elif L==3
	"assignments area";
#elif L==4
	"zona de actividades";
#elif L==5
	"zone d'activit&eacute;s";
#elif L==6
	"zona de actividades";	// Okoteve traduccin
#elif L==7
	"area di attivit&agrave;";
#elif L==8
	"obszarze zadan";
#elif L==9
	"zona de atividades";
#endif

const char *Txt_Attendance =	// Number of times a students has attended to face-to-face class
#if   L==1
	"Assist&egrave;ncia";
#elif L==2
	"Anwesenheit";
#elif L==3
	"Attendance";
#elif L==4
	"Asistencia";
#elif L==5
	"Pr&eacute;sence";
#elif L==6
	"Asistencia";	// Okoteve traduccin
#elif L==7
	"Presenza";
#elif L==8
	"Bytno&sacute;&cacute;";
#elif L==9
	"Presen&ccedil;a";
#endif

const char *Txt_Attendance_list =
#if   L==1
	"Llista assistencia";
#elif L==2
	"Anwesenheitsliste";
#elif L==3
	"Attendance list";
#elif L==4
	"Lista asistencia";
#elif L==5
	"Liste fr&eacute;quentation";
#elif L==6
	"Lista asistencia";	// Okoteve traduccin
#elif L==7
	"Elenco presenze";
#elif L==8
	"Lista obecno&sacute;ci";
#elif L==9
	"Lista assiduidade";
#endif

const char *Txt_Availability =
#if   L==1
	"Disponibilitat";
#elif L==2
	"Verf&uuml;gbarkeit";
#elif L==3
	"Availability";
#elif L==4
	"Disponibilidad";
#elif L==5
	"Disponibilit&eacute;";
#elif L==6
	"Disponibilidad";	// Okoteve traduccin
#elif L==7
	"Disponibilit&agrave;";
#elif L==8
	"Dost&eogon;pno&sacute;&cacute;";
#elif L==9
	"Disponibilidade";
#endif

const char *Txt_Average_number_BR_of_ASSIG_BR_per_course =
#if   L==1
	"Nombre mitj&acute;<br />d'activitats<br />per assignatura";
#elif L==2
	"Durchschnittliche Anzahl<br />von Aufgaben<br />pro Kurs";
#elif L==3
	"Average number<br />of assignments<br />per course";
#elif L==4
	"N&ordm; medio<br />de actividades<br />por asignatura";
#elif L==5
	"Nombre moyen<br />d'activit&eacute;s<br />par mati&egrave;re";
#elif L==6
	"N&ordm; medio<br />de actividades<br />por asignatura";	// Okoteve traduccin
#elif L==7
	"Numero medio<br />di attivit&agrave;<br />per corso";
#elif L==8
	"&Sacute;rednia liczba<br />zadania<br />na kurs";
#elif L==9
	"N&ordm; medio<br />de atividades<br />por disciplina";
#endif

const char *Txt_Average_number_BR_of_questions_BR_per_survey =
#if   L==1
	"Nombre mitj&acute;<br />de preguntes<br />per enquesta";
#elif L==2
	"Durchschnittliche Anzahl<br />von Fragen<br />pro Umfrage";
#elif L==3
	"Average number<br />of questions<br />per survey";
#elif L==4
	"N&ordm; medio<br />de preguntas<br />por encuesta";
#elif L==5
	"Nombre moyen<br />de questions<br />par sondage";
#elif L==6
	"N&ordm; medio<br />de preguntas<br />por encuesta";	// Okoteve traduccin
#elif L==7
	"Numero medio<br />di domande<br />per sondaggio";
#elif L==8
	"&Sacute;rednia liczba<br />pyta&nacute;<br />na ankiety";
#elif L==9
	"N&ordm; medio<br />de quest&otilde;es<br />por inqu&eacute;rito";
#endif

const char *Txt_Average_BR_number_BR_of_test_BR_questions_BR_per_course =
#if   L==1
	"Nombre mitj&acute;<br />de preguntes<br />de test per<br />assignatura";
#elif L==2
	"Durchschnittliche Anzahl<br />von Testfragen<br />pro Kurs";
#elif L==3
	"Average<br />number<br />of test<br />questions<br />per course";
#elif L==4
	"N&ordm; medio<br />de preguntas<br />de test por<br />asignatura";
#elif L==5
	"Nombre moyen<br />de questions<br />de test par<br />mati&egrave;re";
#elif L==6
	"N&ordm; medio<br />de preguntas<br />de test por<br />asignatura";	// Okoteve traduccin
#elif L==7
	"Numero medio<br />di domande<br />di test<br />per corso";
#elif L==8
	"&Sacute;rednia liczba<br />pyta&nacute;<br />testowych<br />na kurs";
#elif L==9
	"N&ordm; medio<br />de quest&otilde;es<br />de test por<br />disciplina";
#endif

const char *Txt_Average_number_BR_of_surveys_BR_per_course =
#if   L==1
	"Nombre mitj&acute;<br />d'enquestes<br />per assignatura";
#elif L==2
	"Durchschnittliche Anzahl<br />von Umfragen<br />pro Kurs";
#elif L==3
	"Average number<br />of surveys<br />per course";
#elif L==4
	"N&ordm; medio<br />de encuestas<br />por asignatura";
#elif L==5
	"Nombre moyen<br />de sondages<br />par mati&egrave;re";
#elif L==6
	"N&ordm; medio<br />de encuestas<br />por asignatura";	// Okoteve traduccin
#elif L==7
	"Numero medio<br />di sondaggi<br />per corso";
#elif L==8
	"&Sacute;rednia liczba<br />badania<br />na kurs";
#elif L==9
	"N&ordm; medio<br />de inqu&eacute;ritos<br />por disciplina";
#endif

const char *Txt_Average_number_of_courses_to_which_a_user_belongs =
#if   L==1
	"Nombre mitj&acute; d'assignatures a las quals pertany cada usuari";
#elif L==2
	"Durchschnittliche Anzahl von Kursen, an denen der Benutzer teilnimmt";
#elif L==3
	"Average number of courses to which a user belongs";
#elif L==4
	"N&ordm; medio de asignaturas a las que pertenece cada usuario";
#elif L==5
	"Nombre moyen de cours &agrave; qui appartient un utilisateur";
#elif L==6
	"N&ordm; medio de asignaturas a las que pertenece cada usuario";	// Okoteve traduccin
#elif L==7
	"Numero medio di corsi a cui appartiene un utente";
#elif L==8
	"&Sacute;rednia liczba kurs&oacute;w do kt&oacute;rej nalezy uzytkownik";
#elif L==9
	"N&ordm; medio de disciplinas em que cada utilizador pertence";
#endif

const char *Txt_Average_BR_number_of_BR_times_that_BR_a_question_BR_has_been_BR_responded =
#if   L==1
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />cada pregunta";	// Necessita traduccio
#elif L==2
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Need bersetzung
#elif L==3
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";
#elif L==4
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />cada pregunta";
#elif L==5
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Besoin de traduction
#elif L==6
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />cada pregunta";	// Okoteve traduccin
#elif L==7
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Bisogno di traduzione
#elif L==8
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Potrzebujesz tlumaczenie
#elif L==9
	"Average<br />number of<br />times that<br />a question<br />has been<br />responded";	// Necessita de traduo
#endif

const char *Txt_Average_BR_number_of_BR_times_that_BR_questions_BR_have_been_BR_responded_BR_per_course =
#if   L==1
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />por asignatura";	// Necessita traduccio
#elif L==2
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Need bersetzung
#elif L==3
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";
#elif L==4
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />por asignatura";
#elif L==5
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Besoin de traduction
#elif L==6
	"N&ordm; medio<br />de veces<br />que se ha<br />respondido<br />por asignatura";			// Okoteve traduccin
#elif L==7
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Bisogno di traduzione
#elif L==8
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Potrzebujesz tlumaczenie
#elif L==9
	"Average<br />number of<br />times that<br />questions<br />have been<br />responded<br />per course";	// Necessita de traduo
#endif

const char *Txt_Average_number_of_users_belonging_to_a_course =
#if   L==1
	"Nombre mitj&agrave; de usuaris que pertanyen a cada assignatura";
#elif L==2
	"Durchschnittliche Anzahl der Benutzer, die an einer Kurs teilnehmen";
#elif L==3
	"Average number of users belonging to a course";
#elif L==4
	"N&ordm; medio de usuarios que pertenecen a cada asignatura";
#elif L==5
	"Nombre moyen d'utilisateurs appartenant &agrave; une mati&egrave;re";
#elif L==6
	"N&ordm; medio de usuarios que pertenecen a cada asignatura";	// Okoteve traduccin
#elif L==7
	"Numero medio di utenti appartenenti a un corso";
#elif L==8
	"&Sacute;rednia liczba uzytkownik&oacute;w nalezacych do kursu";
#elif L==9
	"N&ordm; medio de utilizadores que pertencem a uma disciplina";
#endif

const char *Txt_AVERAGE_PHOTO_TYPES[Pho_NUM_AVERAGE_PHOTO_TYPES] =
   {
#if   L==1
	 "mitjana de totes les fotos"
#elif L==2
	 "median of all the photos"			// Need bersetzung
#elif L==3
	 "median of all the photos"
#elif L==4
	 "mediana de todas las fotos"
#elif L==5
	 "m&eacute;dian de toutes les photos"
#elif L==6
	 "mediana de todas las fotos"			// Okoteve traduccin
#elif L==7
	 "mediana di tutte le foto"
#elif L==8
	 "mediana wszystkich zdjec"
#elif L==9
	 "median of all the photos"			// Necessita de traduo
#endif
	 ,
#if   L==1
	 "mitjana aritm&egrave;tica de totes les fotos"
#elif L==2
	 "arithmetic mean of all the photos"		// Need bersetzung
#elif L==3
	 "arithmetic mean of all the photos"
#elif L==4
	 "media aritm&eacute;tica de todas las fotos"
#elif L==5
	 "moyenne arithm&eacute;tique de toutes les photos"
#elif L==6
	 "media aritm&eacute;tica de todas las fotos"	// Okoteve traduccin
#elif L==7
	 "media aritmetica di tutte le foto"
#elif L==8
	 "&sacute;rednia arytmetyczna wszystkich zdjec"
#elif L==9
	 "arithmetic mean of all the photos"		// Necessita de traduo
#endif
	};

const char *Txt_Average_BR_score_BR_per_question_BR_from_0_to_1 =
#if   L==1
	"Puntuaci&oacute;n<br />media<br />por pregunta<br />(de 0 a 1)";	// Necessita traduccio
#elif L==2
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Need bersetzung
#elif L==3
	"Average<br />score<br />per question<br />(from 0 to 1)";
#elif L==4
	"Puntuaci&oacute;n<br />media<br />por pregunta<br />(de 0 a 1)";
#elif L==5
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Besoin de traduction
#elif L==6
	"Puntuaci&oacute;n<br />media<br />por pregunta<br />(de 0 a 1)";	// Okoteve traduccin
#elif L==7
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Bisogno di traduzione
#elif L==8
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Potrzebujesz tlumaczenie
#elif L==9
	"Average<br />score<br />per question<br />(from 0 to 1)";		// Necessita de traduo
#endif

const char *Txt_Average_type =
#if   L==1
	"Tipus de mitjana";
#elif L==2
	"Art des Mittelwerts";
#elif L==3
	"Average type";
#elif L==4
	"Tipo de promedio";
#elif L==5
	"Type de moyenne";
#elif L==6
	"Tipo de promedio";	// Okoteve traduccin
#elif L==7
	"Tipo di media";
#elif L==8
	"&Sacute;rednia typu";
#elif L==9
	"Tipo de m&eacute;dia";
#endif

const char *Txt_Banned_users =
#if   L==1
	"Usuaris bloquejats";
#elif L==2
	"Gebannt Benutzer";
#elif L==3
	"Banned users";
#elif L==4
	"Usuarios bloqueados";
#elif L==5
	"Utilisateurs bannis";
#elif L==6
	"Usuarios bloqueados";	// Okoteve traduccin
#elif L==7
	"Utenti vietati";
#elif L==8
	"Zablokowani u&zdot;ytkownicy";
#elif L==9
	"Usu&aacute;rios banidos";
#endif

const char *Txt_Banner =
#if   L==1
	"Banner";
#elif L==2
	"Werbebanner";
#elif L==3
	"Banner";
#elif L==4
	"Banner";
#elif L==5
	"Banni&eacute;re";
#elif L==6
	"Banner";
#elif L==7
	"Banner";
#elif L==8
	"Banner";
#elif L==9
	"Banner";
#endif

const char *Txt_Banner_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Banner <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2
	"Werbebanner <strong>%s</strong> entfernt.";
#elif L==3
	"Banner <strong>%s</strong> removed.";
#elif L==4
	"Banner <strong>%s</strong> eliminado.";
#elif L==5
	"Banni&eacute;re <strong>%s</strong> supprim&eacute;e.";
#elif L==6
	"Banner <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Banner <strong>%s</strong> rimosso.";
#elif L==8
	"Banner <strong>%s</strong> usuniety.";
#elif L==9
	"Banner <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Banners =
#if   L==1
	"Banners";
#elif L==2
	"Werbebanners";
#elif L==3
	"Banners";
#elif L==4
	"Banners";
#elif L==5
	"Banni&eacute;res";
#elif L==6
	"Banners";
#elif L==7
	"Banners";
#elif L==8
	"Banners";
#elif L==9
	"Banners";
#endif

const char *Txt_Before_creating_a_new_account_check_if_you_have_been_already_registered_with_your_ID =
#if   L==1
	"Abans de crear un compte nou,"
	" comprovi si ja li han inscrit amb el seu ID";
#elif L==2
	"Bevor ein neues Benutzerkonto erstellen,"
	" wenn Sie bereits mit Ihrer ID registriert worden sind";
#elif L==3
	"Before creating a new account,"
	" check if you have been already registered with your ID";
#elif L==4
	"Antes de crear una nueva cuenta,"
	" compruebe si ya le han inscrito con su ID";
#elif L==5
	"Avant de cr&eacute;er un nouveau compte,"
	" v&eacute;rifiez si vous avez d&eacute;j&agrave; enregistr&eacute; avec votre ID";
#elif L==6
	"Antes de crear una nueva cuenta,"
	" compruebe si ya le han inscrito con su ID";	// Okoteve traduccin
#elif L==7
	"Prima di creare un nuovo account,"
	" verificare se sono stati gi&agrave; registrato con il tuo ID";
#elif L==8
	"Przed utworzeniem nowego konta,"
	" nale&zdot;y sprawdzi&cacute;, czy zosta&lstrok; on ju&zdot; zarejestrowany z identyfikatorem";
#elif L==9
	"Antes de criar uma nova conta,"
	" verifique se voc&ecirc; j&aacute; foi registrado/a com o seu ID";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_fill_your_nickname =
#if   L==1
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de crear el seu sobrenom.";
#elif L==2
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihr Benutzernamen erstellen.";
#elif L==3
	"Before going to any other option"
	" you must fill your nickname.";
#elif L==4
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su apodo.";
#elif L==5
	"Avant d'aller &agrave; une autre option"
	" vous devez cr&eacute;er votre surnom.";
#elif L==6
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su apodo.";			// Okoteve traduccin
#elif L==7
	"Prima di scegliere qualsiasi altra opzione"
	" devi creare il tuo soprannome.";
#elif L==8
	"Przed udaniem si&eogon; do innej opcji"
	" nale&zdot;y wype&lstrok;ni&cacute; sw&oacute;j nick";
#elif L==9
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	" voc&ecirc; deve criar sua alcunha.";
#endif

const char *Txt_Before_going_to_any_other_option_you_must_create_your_password =
#if   L==1
	"Abans d'accedir a qualsevol altra opci&oacute;"
	" ha de crear la seva contrasenya.";
#elif L==2
	"Bevor ich auf jede andere Option"
	" m&uuml;ssen Sie Ihr Passwort erstellen.";
#elif L==3
	"Before going to any other option"
	" you must create your password.";
#elif L==4
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su contrase&ntilde;a.";
#elif L==5
	"Avant d'aller &agrave; une autre option"
	" vous devez cr&eacute;er votre mot de passe.";
#elif L==6
	"Antes de acceder a cualquier otra opci&oacute;n"
	" debe crear su contrase&ntilde;a.";		// Okoteve traduccin
#elif L==7
	"Prima di scegliere qualsiasi altra opzione"
	" devi creare una password.";
#elif L==8
	"Przed p&oacute;jsciem do jakiejkolwiek"
	" innej opcji nalezy utworzy&cacute; has&lstrok;o.";
#elif L==9
	"Antes de ir para qualquer outra op&ccedil;&atilde;o"
	"voc&ecirc; deve criar sua senha.";
#endif

const char *Txt_Briefcases =
#if   L==1
	"Maletins";
#elif L==2
	"Aktentasche";
#elif L==3
	"Briefcases";
#elif L==4
	"Maletines";
#elif L==5
	"Porte-documents";
#elif L==6
	"Maletines";	// Okoteve traduccin
#elif L==7
	"Penna USB";
#elif L==8
	"Briefcases";	// Potrzebujesz tlumaczenie
#elif L==9
	"Pasta";
#endif

const char *Txt_Calculate =
#if   L==1
	"Calcular";
#elif L==2
	"Berechnen";
#elif L==3
	"Calculate";
#elif L==4
	"Calcular";
#elif L==5
	"Calculer";
#elif L==6
	"Calcular";	// Okoteve traduccin
#elif L==7
	"Calcolare";
#elif L==8
	"Oblicza&cacute;";
#elif L==9
	"Calcular";
#endif

const char *Txt_Calculate_average_photo_of_a_degree =
#if   L==1
	"Calcular fotografia mitjana d'una titulaci&oacute;";
#elif L==2
	"Calculate average photo of a degree";				// Need bersetzung
#elif L==3
	"Calculate average photo of a degree";
#elif L==4
	"Calcular fotograf&iacute;a promedio de una titulaci&oacute;n";
#elif L==5
	"Calculate average photo of a degree";				// Besoin de traduction
#elif L==6
	"Calcular fotograf&iacute;a promedio de una titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Calcolare la media delle foto di laurea";
#elif L==8
	"Obliczyc &sacute;rednia zdjec stopnia";
#elif L==9
	"Calculate average photo of a degree";				// Necessita de traduo
#endif

const char *Txt_Calculate_average_photo_of_THE_DEGREE_X =
#if   L==1
	"Calcular fotografia mitjana de";
#elif L==2
	"Calculate average photo of";			// Need bersetzung
#elif L==3
	"Calculate average photo of";
#elif L==4
	"Calcular fotograf&iacute;a promedio de";
#elif L==5
	"Calculate average photo of";			// Besoin de traduction
#elif L==6
	"Calcular fotograf&iacute;a promedio de";	// Okoteve traduccin
#elif L==7
	"Calcolare la media della foto di";
#elif L==8
	"Obliczyc &sacute;rednia zdjecie";
#elif L==9
	"Calculate average photo of";			// Necessita de traduo
#endif

const char *Txt_Calendar =
#if   L==1
	"Calendari";
#elif L==2
	"Kalender";
#elif L==3
	"Calendar";
#elif L==4
	"Calendario";
#elif L==5
	"Calendrier";
#elif L==6
	"Arapapaha";
#elif L==7
	"Calendario";
#elif L==8
	"Kalendarz";
#elif L==9
	"Calend&aacute;rio";
#endif

const char *Txt_Call_the_roll =
#if   L==1
	"Passar llista";
#elif L==2
	"Anwesenheit feststellen";
#elif L==3
	"Call the roll";
#elif L==4
	"Pasar lista";
#elif L==5
	"Faire l'appel";
#elif L==6
	"Pasar lista";	// Okoteve traduccin
#elif L==7
	"Appello";
#elif L==8
	"Imienne";
#elif L==9
	"Lista de presen&ccedil;a";
#endif

const char *Txt_Can_not_create_the_folder_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No s'ha creat la carpeta <strong>%s</strong>"
	" perque s'excediria la quota de disc.";
#elif L==2
	"Verzeichnis <strong>%s</strong> kann aus Platzgrnden"
	" nicht erstellt werden.";
#elif L==3
	"Cannot create the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5
	"Impossible de cr&eacute;er le r&eacute;pertoire <strong>%s</strong>"
	" parce qu'il d&eacute;passerait le quota de disque.";
#elif L==6
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7
	"Non si pu&ograve; creare la cartella <strong>%s</strong>"
	" perch&egrave; si &egrave; superata la quota del disco.";
#elif L==8
	"Nie mozna utworzy&cacute; folder <strong>%s</strong>,"
	" poniewaz to przekracza kontyngentu na dysku.";
#elif L==9
	"N&atilde;o &eacute; poss&iacute;vel criar o diret&oacute;rio <strong>%s</strong>"
	" porque iria exceder a cota de disco.";
#endif

const char *Txt_Can_not_create_the_folder_X_because_there_is_already_a_folder_or_a_file_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No s'ha creat la carpeta <strong>%s</strong>"
	" perque ja existeix una carpeta o un fitxer amb aquest nom.";
#elif L==2
	"Cannot create the folder <strong>%s</strong>"
	" because there is already a folder or a file with that name.";	// Need bersetzung
#elif L==3
	"Cannot create the folder <strong>%s</strong>"
	" because there is already a folder or a file with that name.";
#elif L==4
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque ya existe una carpeta o un archivo con ese nombre.";
#elif L==5
	"Impossible de cr&eacute;er le r&eacute;pertoire <strong>%s</strong>"
	" car il existe d&eacute;j&agrave; un r&eacute;pertoire ou un fichier avec ce nom.";
#elif L==6
	"No se ha creado la carpeta <strong>%s</strong>"
	" porque ya existe una carpeta o un archivo con ese nombre.";	// Okoteve traduccin
#elif L==7
	"Non puoi creare la cartella <strong>%s</strong>"
	" perch&egrave; gi&agrave; esiste una cartella o un file con quel nome.";
#elif L==8
	"Nie mozna utworzy&cacute; folder <strong>%s</strong>"
	" poniewaz istnieje juz folder lub plik o tej nazwie.";
#elif L==9
	"N&atilde;o &eacute; poss&iacute;vel criar o diret&oacute;rio <strong>%s</strong>"
	" porque j&aacute; existe um diret&oacute;rio ou um arquivo com esse nome.";
#endif

const char *Txt_Can_not_create_the_link_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No s'ha creat l'enlla&ccedil; <strong>%s</strong>"
	" perque s'excediria la quota de disc.";
#elif L==2
	"Link <strong>%s</strong> kann aus Platzgr&uuml;nden"
	" nicht erstellt werden.";
#elif L==3
	"Cannot create the link <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4
	"No se ha creado el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5
	"Impossible de cr&eacute;er le lien <strong>%s</strong>"
	" parce qu'il d&eacute;passerait le quota de disque.";
#elif L==6
	"No se ha creado el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7
	"Non si pu&ograve; creare il link <strong>%s</strong>"
	" perch&egrave; si &egrave; superata la quota del disco.";
#elif L==8
	"Nie mozna utworzy&cacute; link <strong>%s</strong>,"
	" poniewaz to przekracza kontyngentu na dysku.";
#elif L==9
	"N&atilde;o &eacute; poss&iacute;vel criar a liga&ccedil;&atilde;o <strong>%s</strong>"
	" porque iria exceder a cota de disco.";
#endif

const char *Txt_Can_not_create_the_link_X_because_there_is_already_a_folder_or_a_link_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No s'ha creat l'enlla&ccedil; <strong>%s</strong>"
	" perque ja existeix una carpeta o un enlla&ccedil; amb aquest nom.";
#elif L==2
	"Cannot create the link <strong>%s</strong>"
	" because there is already a folder or a link with that name.";	// Need bersetzung
#elif L==3
	"Cannot create the link <strong>%s</strong>"
	" because there is already a folder or a link with that name.";
#elif L==4
	"No se ha creado el enlace <strong>%s</strong>"
	" porque ya existe una carpeta o un enlace con ese nombre.";
#elif L==5
	"Impossible de cr&eacute;er le lien <strong>%s</strong>"
	" car il existe d&eacute;j&agrave; un r&eacute;pertoire ou un lien avec ce nom.";
#elif L==6
	"No se ha creado el enlace <strong>%s</strong>"
	" porque ya existe una carpeta o un enlace con ese nombre.";	// Okoteve traduccin
#elif L==7
	"Non puoi creare il link <strong>%s</strong>"
	" perch&egrave; gi&agrave; esiste una cartella o un link con quel nome.";
#elif L==8
	"Nie mozna utworzy&cacute; link <strong>%s</strong>"
	" poniewaz istnieje juz folder lub link o tej nazwie.";
#elif L==9
	"N&atilde;o &eacute; poss&iacute;vel criar a liga&ccedil;&atilde;o <strong>%s</strong>"
	" porque j&aacute; existe um diret&oacute;rio ou uma liga&ccedil;&atilde;o com esse nome.";
#endif

const char *Txt_Can_not_rename_a_folder_of_assignment =
#if   L==1
	"No es pot canviar el nom de una carpeta de activitat.";
#elif L==2
	"Das Verzeichnis der Aufgabe kann nicht umbenannt werden.";
#elif L==3
	"Cannot rename a folder of assignment.";
#elif L==4
	"No se puede cambiar el nombre de una carpeta de actividad.";
#elif L==5
	"Impossible de renommer un r&eacute;pertoire d'activit&eacute;.";
#elif L==6
	"No se puede cambiar el nombre de una carpeta de actividad.";	// Okoteve traduccin
#elif L==7
	"Non &egrave; possibile rinominare una cartella di attivit&agrave;.";
#elif L==8
	"Nie mozna zmienic nazwy folderu zadania.";
#elif L==9
	"N&atilde;o &eacute; poss&iacute;vel renomear um diret&oacute;rio de actividade.";
#endif

const char *Txt_Centre =
#if   L==1
	"Centre";
#elif L==2
	"Lehrinstitut";
#elif L==3
	"Centre";
#elif L==4
	"Centro";
#elif L==5
	"Centre";
#elif L==6
	"Mbo'ehao";
#elif L==7
	"Centro";
#elif L==8
	"Centrum";
#elif L==9
	"Centro";
#endif

const char *Txt_centre =
#if   L==1
	"centre";
#elif L==2
	"Lehrinstitut";
#elif L==3
	"centre";
#elif L==4
	"centro";
#elif L==5
	"centre";
#elif L==6
	"mbo'ehao";
#elif L==7
	"centro";
#elif L==8
	"centrum";
#elif L==9
	"centro";
#endif

const char *Txt_Centre_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Centre <strong>%s</strong> eliminat.";
#elif L==2
	"Lehrinstitut <strong>%s</strong> entfernt.";
#elif L==3
	"Centre <strong>%s</strong> removed.";
#elif L==4
	"Centro <strong>%s</strong> eliminado.";
#elif L==5
	"Centre <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6
	"Centro <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Centro <strong>%s</strong> rimosso.";
#elif L==8
	"Centrum <strong>%s</strong> usuniete.";
#elif L==9
	"Centre <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_CENTRE_STATUS[Crs_NUM_STATUS_TXT] =
   {
#if   L==1	// Crs_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==2
	 "Unbekannt"
#elif L==3
	 "Unknown"
#elif L==4
	 "Desconocido"
#elif L==5
	 "Inconnu"
#elif L==6
	 "Desconocido"			// Okoteve traduccin
#elif L==7
	 "Sconosciuto"
#elif L==8
	 "Nieznany"
#elif L==9
	 "Desconhecido"
#endif
	 ,
#if   L==1	// Crs_STATUS_ACTIVE
	 "Activo"
#elif L==2
	 "Aktive"
#elif L==3
	 "Active"
#elif L==4
	 "Activo"
#elif L==5
	 "Actif"
#elif L==6
	 "Activo"	// Okoteve traduccin
#elif L==7
	 "Attivo"
#elif L==8
	 "Aktywne"
#elif L==9
	 "Ativo"
#endif
	 ,
#if   L==1	// Crs_STATUS_PENDING
	 "Pendent"
#elif L==2
	 "Offen"
#elif L==3
	 "Pending"
#elif L==4
	 "Pendiente"
#elif L==5
	 "En attente"
#elif L==6
	 "Pendiente"	// Okoteve traduccin
#elif L==7
	 "In attesa"
#elif L==8
	 "Do czasu"
#elif L==9
	 "Pendente"
#endif
	 ,
#if   L==1	// Crs_STATUS_REMOVED
	 "Eliminado"
#elif L==2
	 "Entfernt"
#elif L==3
	 "Removed"
#elif L==4
	 "Eliminado"
#elif L==5
	 "Supprim&eacute;"
#elif L==6
	 "Eliminado"	// Okoteve traduccin
#elif L==7
	 "Rimosso"
#elif L==8
	 "Usuniete"
#elif L==9
	 "Suprimido"
#endif
	};

const char *Txt_Centres =
#if   L==1
	"Centres";
#elif L==2
	"Lehrinstitute";
#elif L==3
	"Centres";
#elif L==4
	"Centros";
#elif L==5
	"Centres";
#elif L==6
	"Mbo'ehao";
#elif L==7
	"Centri";
#elif L==8
	"Centra";
#elif L==9
	"Centros";
#endif

const char *Txt_Centres_ABBREVIATION =
#if   L==1
	"Cent.";
#elif L==2
	"Lehrin.";
#elif L==3
	"Cent.";
#elif L==4
	"Cent.";
#elif L==5
	"Cent.";
#elif L==6
	"Mbo'ehao";
#elif L==7
	"Cent.";
#elif L==8
	"Cent.";
#elif L==9
	"Cent.";
#endif

const char *Txt_centres =
#if   L==1
	"centres";
#elif L==2
	"Lehrinstitute";
#elif L==3
	"centres";
#elif L==4
	"centros";
#elif L==5
	"centres";
#elif L==6
	"mbo'ehao";
#elif L==7
	"centri";
#elif L==8
	"centra";
#elif L==9
	"centros";
#endif

const char *Txt_CENTRES_HELP_ORDER[2] =
   {
#if   L==1
	 "Ordenar per centre"
#elif L==2
	 "Nach Lehrinstitut Sortieren"
#elif L==3
	 "Sort by centre"
#elif L==4
	 "Ordenar por centro"
#elif L==5
	 "Trier par centre"
#elif L==6
	 "Ordenar por centro"	// Okoteve traduccin
#elif L==7
	 "Ordina per centro"
#elif L==8
	 "Sortuj centrum"
#elif L==9
	 "Classificar por centro"
#endif
	 ,
#if   L==1
	 "Ordenar per n&ordm; d'usuaris"
#elif L==2
	 "Nach Anzahl der Benutzer sortieren"
#elif L==3
	 "Sort by no. of users"
#elif L==4
	 "Ordenar por n&ordm; de usuarios"
#elif L==5
	 "Trier par nombre d'utilisateurs"
#elif L==6
	 "Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7
	 "Ordina per numero di utenti"
#elif L==8
	 "Sortuj wedlug liczby uzytkownikow"
#elif L==9
	 "Classificar por n&ordm; de utilizadores"
#endif
	};

const char *Txt_Centres_of_INSTITUTION_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Centres de %s";
#elif L==2
	"Lehrinstitute von %s";
#elif L==3
	"Centres of %s";
#elif L==4
	"Centros de %s";
#elif L==5
	"Centres de %s";
#elif L==6
	"Centros de %s";	// Okoteve traduccin
#elif L==7
	"Centri di %s";
#elif L==8
	"Centra %s";
#elif L==9
	"Centros de %s";
#endif

const char *Txt_CENTRES_ORDER[2] =
   {
#if   L==1
	 "Centre"
#elif L==2
	 "Lehrinstitut"
#elif L==3
	 "Centre"
#elif L==4
	 "Centro"
#elif L==5
	 "Centre"
#elif L==6
	 "Mbo'ehao"
#elif L==7
	 "Centro"
#elif L==8
	 "Centrum"
#elif L==9
	 "Centro"
#endif
	 ,
#if   L==1
	 "Usuaris del centre"
#elif L==2
	 "Benutzer der Lehrinstitut"
#elif L==3
	 "Users of the centre"
#elif L==4
	 "Usuarios del centro"
#elif L==5
	 "Utilisateurs du centre"
#elif L==6
	 "Usuarios del centro"	// Okoteve traduccin
#elif L==7
	 "Utenti del centro"
#elif L==8
	 "U&zdot;ytkownicy centrum"
#elif L==9
	 "Utilizadores do centro"
#endif
	};

const char *Txt_Centres_with_pending_degrees =
#if   L==1
	"Centres amb titulacions pendents d'activar";
#elif L==2
	"Lehrinstitute mit anh&auml;ngig Studiengang";
#elif L==3
	"Centres with pending degrees";
#elif L==4
	"Centros con titulaciones pendientes de activar";
#elif L==5
	"Centres avec des &eacute;tudes en attente";
#elif L==6
	"Centros con titulaciones pendientes de activar";	// Okoteve traduccin
#elif L==7
	"Centri con lauree in sospeso";
#elif L==8
	"Centra z stopnie oczekuj&aogon;cy";
#elif L==9
	"Centros com titula&ccedil;&otilde;es pendentes";
#endif

const char *Txt_Change_email =
#if   L==1
	"Canviar correu";
#elif L==2
	"E-Mail &auml;ndern";
#elif L==3
	"Change email";
#elif L==4
	"Cambiar correo";
#elif L==5
	"Changer courrier";
#elif L==6
	"Cambiar correo";	// Okoteve traduccin
#elif L==7
	"Cambiare email";
#elif L==8
	"Zmie&nacute; email";
#elif L==9
	"Alterar email";
#endif

const char *Txt_Change_IDs =
#if   L==1
	"Canviar IDs";
#elif L==2
	"Ausweis-Nr. &auml;ndern";
#elif L==3
	"Change IDs";
#elif L==4
	"Cambiar ID";
#elif L==5
	"Changer IDs";
#elif L==6
	"Cambiar ID";		// Okoteve traduccin
#elif L==7
	"Cambiare ID";
#elif L==8
	"Zmie&nacute; identyfikatory";
#elif L==9
	"Alterar n&ordm;s. identif.";
#endif

const char *Txt_Change_image =
#if   L==1
	"Canviar imatge";
#elif L==2
	"Abbild &auml;ndern";
#elif L==3
	"Change image";
#elif L==4
	"Cambiar imagen";
#elif L==5
	"Changer image";
#elif L==6
	"Moambue ta'&atilde;nga";
#elif L==7
	"Cambiare immagine";
#elif L==8
	"Zmie&nacute; obraz";
#elif L==9
	"Alterar imagem";
#endif

const char *Txt_Change_logo =
#if   L==1
	"Canviar logo";
#elif L==2
	"Logo &auml;ndern";
#elif L==3
	"Change logo";
#elif L==4
	"Cambiar logo";
#elif L==5
	"Changer logo";
#elif L==6
	"Moambue logo";
#elif L==7
	"Cambiare logo";
#elif L==8
	"Zmie&nacute; logo";
#elif L==9
	"Alterar logotipo";
#endif

const char *Txt_Change_my_groups =
#if   L==1
	"Canviar de grups";
#elif L==2
	"Gruppen &auml;ndern";
#elif L==3
	"Change my groups";
#elif L==4
	"Cambiar de grupos";
#elif L==5
	"Modifier mes groupes";
#elif L==6
	"Cambiar de grupos";	// Okoteve traduccin
#elif L==7
	"Cambia i miei gruppi";
#elif L==8
	"Zmie&nacute;moje grupy";
#elif L==9
	"Alterar meus grupos";
#endif

const char *Txt_Change_nickname =
#if   L==1
	"Canviar sobrenom";
#elif L==2
	"Benutzername &auml;ndern";
#elif L==3
	"Change nickname";
#elif L==4
	"Cambiar apodo";
#elif L==5
	"Changer le surnom";
#elif L==6
	"Cambiar apodo";	// Okoteve traduccin
#elif L==7
	"Cambiare nome utente";
#elif L==8
	"Zmie&nacute; nick";
#elif L==9
	"Alterar alcunha";
#endif

const char *Txt_Change_password =
#if   L==1
	"Canviar contrasenya";
#elif L==2
	"Passwort &auml;ndern";
#elif L==3
	"Change password";
#elif L==4
	"Cambiar contrase&ntilde;a";
#elif L==5
	"Changer mot de passe";
#elif L==6
	"Cambiar contrase&ntilde;a";	// Okoteve traduccin
#elif L==7
	"Cambiare password";
#elif L==8
	"Zmie&nacute; has&lstrok;o";
#elif L==9
	"Alterar senha";
#endif

const char *Txt_Change_photo =
#if   L==1
	"Canviar foto";
#elif L==2
	"Foto &auml;ndern";
#elif L==3
	"Change photo";
#elif L==4
	"Cambiar foto";
#elif L==5
	"Changer photo";
#elif L==6
	"Cambiar foto";		// Okoteve traduccin
#elif L==7
	"Cambiare foto";
#elif L==8
	"Zmie&nacute; zdjecie";
#elif L==9
	"Alterar foto";
#endif

const char *Txt_Change_preferences =
#if   L==1
	"Canviar prefer&egrave;ncies";
#elif L==2
	"Einstellungen &auml;ndern";
#elif L==3
	"Change preferences";
#elif L==4
	"Cambiar preferencias";
#elif L==5
	"Changer mes pr&eacute;f&eacute;rences";
#elif L==6
	"Cambiar preferencias";	// Okoteve traduccin
#elif L==7
	"Cambiare preferenze";
#elif L==8
	"Zmiana preferencji";
#elif L==9
	"Alterar prefer&ecirc;ncias";
#endif

const char *Txt_CHAT_Room =
#if   L==1
	"Sala";
#elif L==2
	"Raum";
#elif L==3
	"Room";
#elif L==4
	"Sala";
#elif L==5
	"Salle";
#elif L==6
	"Sala";	// Okoteve traduccin
#elif L==7
	"Room";
#elif L==8
	"Pok&oacute;j";
#elif L==9
	"Room";
#endif

const char *Txt_CHAT_Room_code =
#if   L==1
	"Codi de sala";
#elif L==2
	"Raum-code";
#elif L==3
	"Room code";
#elif L==4
	"C&oacute;digo de sala";
#elif L==5
	"Code de salle";
#elif L==6
	"C&oacute;digo de sala";	// Okoteve traduccin
#elif L==7
	"Chiave di room";
#elif L==8
	"Kod pokoju";
#elif L==9
	"C&oacute;digo de room";
#endif

const char *Txt_Chat_rooms =
#if   L==1
	"Sales de xat";
#elif L==2
	"Chat-R&auml;ume";
#elif L==3
	"Chat rooms";
#elif L==4
	"Salas de chat";
#elif L==5
	"Salles de chat";
#elif L==6
	"Salas de chat";	// Okoteve traduccin
#elif L==7
	"Chat rooms";
#elif L==8
	"Czat";
#elif L==9
	"Chat rooms";
#endif

const char *Txt_Check =
#if   L==1
	"Comprovar";
#elif L==2
	"&Uuml;berpr&uuml;fen";
#elif L==3
	"Check";
#elif L==4
	"Comprobar";
#elif L==5
	"V&eacute;rifier";
#elif L==6
	"Comprobar";	// Okoteve traduccin
#elif L==7
	"Verificare";
#elif L==8
	"Sprawd&zacute;";
#elif L==9
	"Verificar";
#endif

const char *Txt_Check_another_ID =
#if   L==1
	"Comprovar altre ID";
#elif L==2
	"&Uuml;berpr&uuml;fen Sie eine andere ID";
#elif L==3
	"Check another ID";
#elif L==4
	"Comprobar otro ID";
#elif L==5
	"V&eacute;rifier un autre ID";
#elif L==6
	"Comprobar otro ID";	// Okoteve traduccin
#elif L==7
	"Verificare un altro ID";
#elif L==8
	"Sprawdzi&cacute; inny identyfikator";
#elif L==9
	"Verificar outra ID";
#endif

const char *Txt_Check_marks_in_file_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Consultar qualificacions en el fitxer %s";
#elif L==2
	"Bewertung in Datei %s abrufen";
#elif L==3
	"Check marks in file %s";
#elif L==4
	"Consultar calificaciones en el archivo %s";
#elif L==5
	"Check marks in file %s";			// Besoin de traduction
#elif L==6
	"Consultar calificaciones en el archivo %s";	// Okoteve traduccin
#elif L==7
	"Consultare i risultati nel file %s";
#elif L==8
	"Zaznaczenia w pliku %s";
#elif L==9
	"Check marks in file %s";			// Necessita de traduo
#endif

const char *Txt_Classroom =
#if   L==1
	"Aula";
#elif L==2
	"Klassenzimmer";
#elif L==3
	"Classroom";
#elif L==4
	"Aula";
#elif L==5
	"Classe";
#elif L==6
	"Aula";	// Okoteve traduccin
#elif L==7
	"Aula";
#elif L==8
	"Klasa";
#elif L==9
	"Sala de aula";
#endif

const char *Txt_Click =
#if   L==1
	"Clic";
#elif L==2
	"Klick";
#elif L==3
	"Click";
#elif L==4
	"Clic";
#elif L==5
	"Clic";
#elif L==6
	"Clic";	// Okoteve traduccin
#elif L==7
	"Clic";
#elif L==8
	"Kliknij";
#elif L==9
	"Clique";
#endif

const char *Txt_Clicks =
#if   L==1
	"Clics";
#elif L==2
	"Klicks";
#elif L==3
	"Clicks";
#elif L==4
	"Clics";
#elif L==5
	"Clics";
#elif L==6
	"Clics";	// Okoteve traduccin
#elif L==7
	"Clic";
#elif L==8
	"Klikni&eogon;cia";
#elif L==9
	"Cliques";
#endif

const char *Txt_clicks =
#if   L==1
	"clics";
#elif L==2
	"Klicks";
#elif L==3
	"clicks";
#elif L==4
	"clics";
#elif L==5
	"clics";
#elif L==6
	"clics";	// Okoteve traduccin
#elif L==7
	"clic";
#elif L==8
	"klikni&eogon;cia";
#elif L==9
	"cliques";
#endif

const char *Txt_Clicks_per_day =
#if   L==1
	"Clics per dia";
#elif L==2
	"Klicks pro Tag";
#elif L==3
	"Clicks per day";
#elif L==4
	"Clics por d&iacute;a";
#elif L==5
	"Clics par jour";
#elif L==6
	"Clics por d&iacute;a";	// Okoteve traduccin
#elif L==7
	"Clic al giorno";
#elif L==8
	"Klikni&eogon;&cacute; dziennie";
#elif L==9
	"Cliques por dia";
#endif

const char *Txt_Closed_survey =
#if   L==1
	"Enquesta tancada";
#elif L==2
	"Closed survey";		// Need bersetzung
#elif L==3
	"Closed survey";
#elif L==4
	"Encuesta cerrada";
#elif L==5
	"Sondage ferm&eacute;";
#elif L==6
	"Encuesta cerrada";		// Okoteve traduccin
#elif L==7
	"Sondaggio chiuso";
#elif L==8
	"Zamkniete badania";
#elif L==9
	"Closed survey";		// Necessita de traduo
#endif

const char *Txt_Code =
#if   L==1
	"Codi";
#elif L==2
	"Code";
#elif L==3
	"Code";
#elif L==4
	"C&oacute;d.";
#elif L==5
	"Code";
#elif L==6
	"Papanga";
#elif L==7
	"Cod.";
#elif L==8
	"Kod";
#elif L==9
	"C&oacute;d.";
#endif

const char *Txt_Color_of_the_graphic =
#if   L==1
	"Color de la gr&agrave;fica";
#elif L==2
	"Farbe der Grafik";
#elif L==3
	"Color of the graphic";
#elif L==4
	"Color de la gr&aacute;fica";
#elif L==5
	"Couleur de l'image";
#elif L==6
	"Color de la gr&aacute;fica";	// Okoteve traduccin
#elif L==7
	"Colore del grafico";
#elif L==8
	"Kolor grafiki";
#elif L==9
	"Cor do gr&aacute;fico";
#endif

const char *Txt_Columns =
#if   L==1
	"Columnes";
#elif L==2
	"Spalten";
#elif L==3
	"Columns";
#elif L==4
	"Columnas";
#elif L==5
	"Colonnes";
#elif L==6
	"Columnas";	// Okoteve traduccin
#elif L==7
	"Colonne";
#elif L==8
	"Kolumny";
#elif L==9
	"Colunas";
#endif

const char *Txt_columns =
#if   L==1
	"columnes";
#elif L==2
	"Spalten";
#elif L==3
	"columns";
#elif L==4
	"columnas";
#elif L==5
	"colonnes";
#elif L==6
	"columnas";	// Okoteve traduccin
#elif L==7
	"colonne";
#elif L==8
	"kolumny";
#elif L==9
	"colunas";
#endif

const char *Txt_Comment =	// As a verb
#if   L==1
	"Comentar";
#elif L==2
	"Kommentieren";
#elif L==3
	"Comment";
#elif L==4
	"Comentar";
#elif L==5
	"Commenter";
#elif L==6
	"Comentar";	// Okoteve traduccin
#elif L==7
	"Commentare";
#elif L==8
	"Komentowa&cacute;";
#elif L==9
	"Comentar";
#endif

const char *Txt_Comment_removed =
#if   L==1
	"Comentari eliminat.";
#elif L==2
	"Kommentar entfernt.";
#elif L==3
	"Comment removed.";
#elif L==4
	"Comentario eliminado.";
#elif L==5
	"Commentaire supprim&eacute;.";
#elif L==6
	"Comentario eliminado.";		// Okoteve traduccin
#elif L==7
	"Commento rimosso.";
#elif L==8
	"Komentarz usuniety.";
#elif L==9
	"Coment&aacute;rio removido.";
#endif

const char *Txt_Comments =
#if   L==1
	"Comentaris";
#elif L==2
	"Kommentare";
#elif L==3
	"Comments";
#elif L==4
	"Comentarios";
#elif L==5
	"Commentaires";
#elif L==6
	"Comentarios";	// Okoteve traduccin
#elif L==7
	"Commenti";
#elif L==8
	"Komentarze";
#elif L==9
	"Coment&aacute;rios";
#endif

const char *Txt_Teachers_comment =
#if   L==1
	"Comentari de professors";
#elif L==2
	"Lehrern-Kommentar";
#elif L==3
	"Teachers' comment";
#elif L==4
	"Comentario de profesores";
#elif L==5
	"Commentaire des enseignants";
#elif L==6
	"Comentario de profesores";	// Okoteve traduccin
#elif L==7
	"Commento degli professori";
#elif L==8
	"Komentarz nauczycieli";
#elif L==9
	"Coment&aacute;rio dos professores";
#endif

const char *Txt_Configure =
#if   L==1
	"Configurar";
#elif L==2
	"Konfigurieren";
#elif L==3
	"Configure";
#elif L==4
	"Configurar";
#elif L==5
	"Configurer";
#elif L==6
	"Configurar";	// Okoteve traduccin
#elif L==7
	"Configurare";
#elif L==8
	"Konfiguracja";
#elif L==9
	"Configurar";
#endif

const char *Txt_Configure_tests =
#if   L==1
	"Configurar tests";
#elif L==2
	"Konfigurieren Tests";
#elif L==3
	"Configure tests";
#elif L==4
	"Configurar tests";
#elif L==5
	"Configurer tests";
#elif L==6
	"Configurar tests";	// Okoteve traduccin
#elif L==7
	"Configurare test";
#elif L==8
	"Test Konfiguracja";
#elif L==9
	"Configurar testes";
#endif

const char *Txt_Confirm =
#if   L==1
	"Confirmar";
#elif L==2
	"Best&auml;tigen";
#elif L==3
	"Confirm";
#elif L==4
	"Confirmar";
#elif L==5
	"Confirmer";
#elif L==6
	"Confirmar";	// Okoteve traduccin
#elif L==7
	"Confermare";
#elif L==8
	"Potwierdzenie";
#elif L==9
	"Confirmar";
#endif

const char *Txt_Confirm_email =
#if   L==1
	"Confirmar correu";
#elif L==2
	"E-Mail best&auml;tigen";
#elif L==3
	"Confirm email";
#elif L==4
	"Confirmar correo";
#elif L==5
	"Confirmer courrier";
#elif L==6
	"Confirmar correo";	// Okoteve traduccin
#elif L==7
	"Confermare email";
#elif L==8
	"Potwierdzenie email";
#elif L==9
	"Confirmar email";
#endif

const char *Txt_Confirm_ID =
#if   L==1
	"Confirmar ID";
#elif L==2
	"ID best&auml;tigen";
#elif L==3
	"Confirm ID";
#elif L==4
	"Confirmar ID";
#elif L==5
	"Confirmer ID";
#elif L==6
	"Confirmar ID";	// Okoteve traduccin
#elif L==7
	"Confermare ID";
#elif L==8
	"Potwierdzenie ID";
#elif L==9
	"Confirmar ID";
#endif

const char *Txt_Confirm_my_enrollment =
#if   L==1
	"Confirmar la meva inscripci&oacute;";
#elif L==2
	"Best&auml;tigen Sie meine Registrierung";
#elif L==3
	"Confirm my enrollment";
#elif L==4
	"Confirmar mi inscripci&oacute;n";
#elif L==5
	"Confirmer mon inscription";
#elif L==6
	"Confirmar mi inscripci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Confermare la mia iscrizione";
#elif L==8
	"Potwierdzenie mojej rejestracji";
#elif L==9
	"Confirmar o meu registo";
#endif

const char *Txt_Congratulations_You_have_created_your_account_X_Now_Y_will_request_you_ =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"Enhorabona! Ha creat el seu compte @%s."
	" Ara %s li demanar&agrave; algunes dades per completar el seu perfil."
	" Despr&eacute;s podr&agrave; sol&middot;licitar la seva inscripci&oacute; en assignatures."
	" Si no hi ha una assignatura,"
	" vost&egrave; pot crear abans de sol&middot;licitar la seva inscripci&oacute; en ella"
	" (el mateix &eacute;s aplicable a institucions, centres i titulacions).";
#elif L==2
	"Herzlichen Gl&uuml;ckwunsch! Du hast Dein Benutzerkonto <strong>@%s</strong> erstellt."
	" Jetzt %s finden Sie einige Daten in Ihrem Profil."
	" Vervollst&auml;ndigen anfordern und dann werden Sie in der Lage,"
	" f&uuml;r die Einschreibung in Kurse an."
	" Wenn ein Kurs nicht vorhanden ist,"
	" k&ouml;nnen Sie es vor der Anwendung f&uuml;r die Immatrikulation in es zu erstellen"
	" (das gleiche gilt f&uuml;r Einrichtungen, Zentren und Grad).";
#elif L==3
	"Congratulations! You have created your account <strong>@%s</strong>."
	" Now %s will request you some data to complete your profile."
	" Then you will be able to apply for enrollment in courses."
	" If a course does not exist,"
	" you can create it before applying for enrollment in it"
	" (the same applies to institutions, centres and degrees).";
#elif L==4
	"&iexcl;Enhorabuena! Ha creado su cuenta <strong>@%s</strong>."
	" Ahora %s le solicitar&aacute; algunos datos para completar su perfil."
	" Usted podr&aacute; despu&eacute;s solicitar su inscripci&oacute;n en asignaturas."
	" Si una asignatura no existe,"
	" puede crearla antes de solicitar su inscripci&oacute;n en ella"
	" (lo mismo es aplicable a instituciones, centros y titulaciones).";
#elif L==5
	"F&eacute;licitations! Vous avez cr&eacute;&eacute; votre compte <strong>@%s</strong>."
	" Maintenant %s vous demander&aacute; certaines donn&eacute;es pour compl&eacute;ter votre profil."
	" Vous serez alors en mesure de demander l'inscription &agrave; des mati&egrave;res."
	" Si une mati&egrave;re ne existe pas,"
	" vous pouvez le cr&eacute;er avant de demander l'inscription en elle"
	" (la m&ecirc;me chose se applique aux institutions, centres et degr&eacute;s).";
#elif L==6
	"&iexcl;Enhorabuena! Ha creado su cuenta <strong>@%s</strong>."
	" Ahora %s le solicitar&aacute; algunos datos para completar su perfil."
	" Usted podr&aacute; despu&eacute;s solicitar su inscripci&oacute;n en asignaturas."
	" Si una asignatura no existe,"
	" puede crearla antes de solicitar su inscripci&oacute;n en ella"
	" (lo mismo es aplicable a instituciones, centros y titulaciones).";	// Okoteve traduccin
#elif L==7
	"Congratulazioni! Avete creato il vostro account <strong>@%s</strong>."
	" Ora %s chieder&agrave; alcuni dati per completare il tuo profilo."
	" Allora si sar&agrave; in grado di applicare per l'iscrizione ai corsi."
	" Se un corso non esiste, &egrave; possibile crearlo prima di applicare per l'iscrizione in esso"
	" (lo stesso vale per le istituzioni, centri e gradi).";
#elif L==8
	"Gratulujemy! Utworzeniu konta <strong>@%s</strong>."
	" Teraz %s zwr&oacute;ci Ci kilka danych, aby zako&nacute;czy&cacute; sw&oacute;j profil."
	" Nast&eogon;pnie b&eogon;dzie mo&zdot;na ubiega&cacute; si&eogon; o przyj&eogon;cie na kursy."
	" Je&sacute;li kurs nie istnieje,"
	" mo&zdot;na go utworzy&cacute; przed ubieganiem si&egrave; o przyj&eogon;cie w nim"
	" (to samo odnosi si&eogon; do instytucji, plac&oacute;wek i stopni).";
#elif L==9
	"Parab&eacute;ns! Voc&ecirc; criou seu conta <strong>@%s</strong>."
	" Agora %s vai pedir-lhe alguns dados para completar o seu perfil."
	" Em seguida, voc&ecirc; ser&aacute; capaz de aplicar para a matrcula em cursos."
	" Se uma disciplina n&atilde;o existir,"
	" voc&ecirc; pode cri&aacute;-lo antes de solicitar a inscri&ccedil;&atilde;o do mesmo"
	" (o mesmo se aplica &agrave;s institui&ccedil;&otilde;es, centros e graus).";
#endif

const char *Txt_connected_PLURAL =
#if   L==1
	"connectats";
#elif L==2
	"angemeldet";
#elif L==3
	"connected";
#elif L==4
	"conectados";
#elif L==5
	"connect&eacute;s";
#elif L==6
	"conectados";	// Okoteve traduccin
#elif L==7
	"connessi";
#elif L==8
	"online";
#elif L==9
	"conectados";
#endif

const char *Txt_connected_SINGULAR =
#if   L==1
	"connectat";
#elif L==2
	"angemeldet";
#elif L==3
	"connected";
#elif L==4
	"conectado";
#elif L==5
	"connect&eacute;";
#elif L==6
	"conectado";	// Okoteve traduccin
#elif L==7
	"connesso";
#elif L==8
	"online";
#elif L==9
	"conectado";
#endif

const char *Txt_Connected_users =
#if   L==1
	"Usuaris connectats";
#elif L==2
	"Angemeldet Benutzer";
#elif L==3
	"Connected users";
#elif L==4
	"Usuarios conectados";
#elif L==5
	"Utilisateurs connect&eacute;s";
#elif L==6
	"Usuarios conectados";	// Okoteve traduccin
#elif L==7
	"Utenti connessi";
#elif L==8
	"U&zdot;ytkownicy online";
#elif L==9
	"Utilizadores conectados";
#endif

const char *Txt_Continue =
#if   L==1
	"Continuar";
#elif L==2
	"Weiter";
#elif L==3
	"Continue";
#elif L==4
	"Continuar";
#elif L==5
	"Continuer";
#elif L==6
	"Hemo&ntilde;a";
#elif L==7
	"Continua";
#elif L==8
	"Kontynuuj";
#elif L==9
	"Continuar";
#endif

const char *Txt_Contract =	// As a verb
#if   L==1
	"Contreure";
#elif L==2
	"Zusammenziehen";
#elif L==3
	"Contract";
#elif L==4
	"Contraer";
#elif L==5
	"Contracter";
#elif L==6
	"Contraer";	// Okoteve traduccin
#elif L==7
	"Contratto";
#elif L==8
	"Umowy";
#elif L==9
	"Contrair";
#endif

const char *Txt_Control_of_class_attendance =
#if   L==1
	"Control d'assistencia a classe";
#elif L==2
	"Anwesenheitskontrolle";
#elif L==3
	"Control of class attendance";
#elif L==4
	"Control de asistencia a clase";
#elif L==5
	"Contr&ouml;le de la fr&eacute;quentation scolaire";
#elif L==6
	"Control de asistencia a clase";	// Okoteve traduccin
#elif L==7
	"Controllo delle presenze in classe";
#elif L==8
	"Kontrola obecnosc na zajeciach";
#elif L==9
	"Controlo de assiduidade de classe";
#endif

const char *Txt_Copy_FOLDER_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Copiar %s";
#elif L==2
	"Kopieren %s";
#elif L==3
	"Copy %s";
#elif L==4
	"Copiar %s";
#elif L==5
	"Copier %s";
#elif L==6
	"Ekopia %s";
#elif L==7
	"Copiare %s";
#elif L==8
	"Kopia %s";
#elif L==9
	"Copiar %s";
#endif

const char *Txt_Copy_not_allowed =
#if   L==1
	"C&ograve;pia no perm&egrave;s";
#elif L==2
	"Kopier nicht erlaubt";
#elif L==3
	"Copy not allowed";
#elif L==4
	"Copia no permitida";
#elif L==5
	"Copie pas autoris&eacute;e";
#elif L==6
	"Copia no permitida";	// Okoteve traduccin
#elif L==7
	"Copia non ammessi";
#elif L==8
	"Kopiowanie nie wolno";
#elif L==9
	"Copia n&atilde;o permitida";
#endif

const char *Txt_Copy_source =
#if   L==1
	"Origen de c&ograve;pia";
#elif L==2
	"Kopier-Quelle";
#elif L==3
	"Copy source";
#elif L==4
	"Origen de copia";
#elif L==5
	"Source de copie";
#elif L==6
	"Origen de copia";	// Okoteve traduccin
#elif L==7
	"Copia sorgente";
#elif L==8
	"Kopiowanie zr&oacute;dla";
#elif L==9
	"Origem da copia";
#endif

const char *Txt_Could_not_detect_any_face_in_front_position_ =
#if   L==1
	"No s'ha detectat cap rostre en posici&oacute; frontal.";
#elif L==2
	"Could not detect any face in front position.";				// Need bersetzung
#elif L==3
	"Could not detect any face in front position.";
#elif L==4
	"No se ha detectado ning&uacute;n rostro en posici&oacute;n frontal.";
#elif L==5
	"Could not detect any face in front position.";				// Besoin de traduction
#elif L==6
	"No se ha detectado ning&uacute;n rostro en posici&oacute;n frontal.";	// Okoteve traduccin
#elif L==7
	"Non puoi rilevare qualsiasi viso in posizione frontale.";
#elif L==8
	"Nie wykryto zadnej twarzy w przednim.";
#elif L==9
	"Could not detect any face in front position.";				// Necessita de traduo
#endif

const char *Txt_Could_not_get_the_official_list_of_your_groups_and_students_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==1
	"No s'ha pogut obtenir la llista oficial dels seus grups i estudiants.<br />"
	"Es posible que haya expirado el plazo para obtener dicha lista.<br />"
	"Entre de nuevo a %s desde <a href=\"%s\" target=\"_blank\">%s</a> y reint&eacute;ntelo.";	// Necessita traduccio
#elif L==2
	"Could not get the official list of your groups and students.<br />"
	"It is possible that the deadline expired for getting the list.<br />"
	"Log in again to %s from <a href=\"%s\" target=\"_blank\">%s</a> and try again.";		// Need bersetzung
#elif L==3
	"Could not get the official list of your groups and students.<br />"
	"It is possible that the deadline expired for getting the list.<br />"
	"Log in again to %s from <a href=\"%s\" target=\"_blank\">%s</a> and try again.";
#elif L==4
	"No se ha podido obtener la lista oficial de sus grupos y estudiantes.<br />"
	"Es posible que haya expirado el plazo para obtener dicha lista.<br />"
	"Entre de nuevo a %s desde <a href=\"%s\" target=\"_blank\">%s</a> y reint&eacute;ntelo.";
#elif L==5
	"Could not get the official list of your groups and students.<br />"
	"It is possible that the deadline expired for getting the list.<br />"
	"Log in again to %s from <a href=\"%s\" target=\"_blank\">%s</a> and try again.";		// Besoin de traduction
#elif L==6
	"No se ha podido obtener la lista oficial de sus grupos y estudiantes.<br />"
	"Es posible que haya expirado el plazo para obtener dicha lista.<br />"
	"Entre de nuevo a %s desde <a href=\"%s\" target=\"_blank\">%s</a> y reint&eacute;ntelo.";	// Okoteve traduccin
#elif L==7
	"Non puoi prendere la lista ufficiale dei tuoi gruppi e degli studenti.<br />"
	"E' possibile che il tempo per acquisire la lista sia scaduto.<br />"
	"Loggati nuovamente a %s da %s e prova ancora.";
#elif L==8
	"Nie udalo sie uzyskac oficjalne liste grup i student&oacute;w.<br />"
	"Jest mozliwe, ze uplynal termin uzyskania listy<br />"
	"Zaloguj sie ponownie do %s <a href=\"%s\" target=\"_blank\">%s</a> i spr&oacute;buj ponownie";	// Please, check this!
#elif L==9
	"Could not get the official list of your groups and students.<br />"
	"It is possible that the deadline expired for getting the list.<br />"
	"Log in again to %s from <a href=\"%s\" target=\"_blank\">%s</a> and try again.";		// Necessita de traduo
#endif

const char *Txt_Countries =
#if   L==1
	"Pa&iuml;sos";
#elif L==2
	"L&auml;nder";
#elif L==3
	"Countries";
#elif L==4
	"Pa&iacute;ses";
#elif L==5
	"Pays";
#elif L==6
	"Tet&atilde;";
#elif L==7
	"Paesi";
#elif L==8
	"Kraje";
#elif L==9
	"Pa&iacute;ses";
#endif

const char *Txt_COUNTRIES_HELP_ORDER[2] =
   {
#if   L==1
	 "Ordenar por pa&iacute;s"	// Necessita traduccio
#elif L==2
	 "Nach Land sortieren"
#elif L==3
	 "Sort by country"
#elif L==4
	 "Ordenar por pa&iacute;s"
#elif L==5
	 "Trier par pays"
#elif L==6
	 "Ordenar por pa&iacute;s"	// Okoteve traduccin
#elif L==7
	 "Ordina per paese"
#elif L==8
	 "Sortuj wedlug kraju"
#elif L==9
	 "Classificar por pa&iacute;s"
#endif
	 ,
#if   L==1
	 "Ordenar per n&ordm; d'usuaris"
#elif L==2
	 "Nach Anzahl der Benutzer sortieren"
#elif L==3
	 "Sort by no. of users"
#elif L==4
	 "Ordenar por n&ordm; de usuarios"
#elif L==5
	 "Trier par nombre d'utilisateurs"
#elif L==6
	 "Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7
	 "Ordina per numero di utenti"
#elif L==8
	 "Sortuj wedlug liczby uzytkownikow"
#elif L==9
	 "Classificar por n&ordm; de utilizadores"
#endif
	};

const char *Txt_Countries_with_pending_institutions =
#if   L==1
	"Pa&iuml;sos amb institucions pendents d'activar";
#elif L==2
	"L&auml;nder mit anh&auml;ngig Hochschulen";
#elif L==3
	"Countries with pending institutions";
#elif L==4
	"Pa&iacute;ses con instituciones pendientes de activar";
#elif L==5
	"Pays avec des &eacute;tablissements en attente";
#elif L==6
	"Pa&iacute;ses con instituciones pendientes de activar";	// Okoteve traduccin
#elif L==7
	"Paesi con istituzioni in sospeso";
#elif L==8
	"Kraje z instytucje oczekuj&aogon;cy";
#elif L==9
	"Pa&iacute;ses com institu&ccedil;&otilde;es pendentes";
#endif

const char *Txt_COUNTRIES_ORDER[2] =
   {
#if   L==1
	 "Pa&iacute;s"
#elif L==2
	 "Land"
#elif L==3
	 "Country"
#elif L==4
	 "Pa&iacute;s"
#elif L==5
	 "Pays"
#elif L==6
	 "Tet&atilde;"
#elif L==7
	 "Paese"
#elif L==8
	 "Kraj"
#elif L==9
	 "Pa&iacute;s"
#endif
	 ,
#if   L==1
	 "Usuaris del pa&iacute;s"
#elif L==2
	 "Benutzer des Landes"
#elif L==3
	 "Users of the country"
#elif L==4
	 "Usuarios del pa&iacute;s"
#elif L==5
	 "Utilisateurs du pays"
#elif L==6
	 "Usuarios del pa&iacute;s"	// Okoteve traduccin
#elif L==7
	 "Utenti del paese"
#elif L==8
	 "U&zdot;ytkownicy kraju"
#elif L==9
	 "Utilizadores do pa&iacute;s"
#endif
	};

const char *Txt_Country =
#if   L==1
	"Pa&iacute;s";
#elif L==2
	"Land";
#elif L==3
	"Country";
#elif L==4
	"Pa&iacute;s";
#elif L==5
	"Pays;";
#elif L==6
	"Tet&atilde;";
#elif L==7
	"Paese";
#elif L==8
	"Kraj";
#elif L==9
	"Pa&iacute;s";
#endif

const char *Txt_Country_of_your_institution =
#if   L==1
	"Pa&iacute;s de la seva instituci&oacute;";
#elif L==2
	"Land Ihrer Hochschule";
#elif L==3
	"Country of your institution";
#elif L==4
	"Pa&iacute;s de su instituci&oacute;n";
#elif L==5
	"Pays de votre &eacute;tablissement";
#elif L==6
	"Tet&atilde; mbo'ehao";
#elif L==7
	"Paese della vostra istituzione";
#elif L==8
	"Kraj swojej instytucji";
#elif L==9
	"Pa&iacute;s da sua institu&ccedil;&atilde;o";
#endif

const char *Txt_Country_unspecified =
#if   L==1
	"Pa&iacute;s sense especificar";
#elif L==2
	"Land nicht angegeben";
#elif L==3
	"Country unspecified";
#elif L==4
	"Pa&iacute;s sin especificar";
#elif L==5
	"Pays non sp&eacute;cifi&eacute;";
#elif L==6
	"Pa&iacute;s sin especificar";	// Okoteve traduccin
#elif L==7
	"Paese non specificato";
#elif L==8
	"Kraj nieokreslony";
#elif L==9
	"Pa&iacute;s indeterminado";
#endif

const char *Txt_Country_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Pa&iacute;s <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2
	"Land <strong>%s</strong> entfernt.";
#elif L==3
	"Country <strong>%s</strong> removed.";
#elif L==4
	"Pa&iacute;s <strong>%s</strong> eliminado.";
#elif L==5
	"Pays <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6
	"Pa&iacute;s <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Paese <strong>%s</strong> rimosso.";
#elif L==8
	"Kraj <strong>%s</strong> usuniete.";
#elif L==9
	"Pa&iacute;s <strong>%s</strong> removed.";	// Necessita de traduo
#endif

const char *Txt_Course =
#if   L==1
	"Assignatura";
#elif L==2
	"Kurs";
#elif L==3
	"Course";
#elif L==4
	"Asignatura";
#elif L==5
	"Mati&egrave;re";
#elif L==6
	"Mbo'esyry";
#elif L==7
	"Corso";
#elif L==8
	"Kurs";
#elif L==9
	"Disciplina";
#endif

const char *Txt_course =
#if   L==1
	"assignatura";
#elif L==2
	"Kurs";
#elif L==3
	"course";
#elif L==4
	"asignatura";
#elif L==5
	"mati&egrave;re";
#elif L==6
	"mbo'esyry";
#elif L==7
	"corso";
#elif L==8
	"kurs";
#elif L==9
	"disciplina";
#endif

const char *Txt_COURSE_STATUS[Crs_NUM_STATUS_TXT] =
   {
#if   L==1	// Crs_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==2
	 "Unbekannt"
#elif L==3
	 "Unknown"
#elif L==4
	 "Desconocido"
#elif L==5
	 "Inconnu"
#elif L==6
	 "Desconocido"			// Okoteve traduccin
#elif L==7
	 "Sconosciuto"
#elif L==8
	 "Nieznany"
#elif L==9
	 "Desconhecido"
#endif
	 ,
#if   L==1	// Crs_STATUS_ACTIVE
	 "Activa"
#elif L==2
	 "Aktive"
#elif L==3
	 "Active"
#elif L==4
	 "Activa"
#elif L==5
	 "Actif"
#elif L==6
	 "Activa"	// Okoteve traduccin
#elif L==7
	 "Attivo"
#elif L==8
	 "Aktywne"
#elif L==9
	 "Ativa"
#endif
	 ,
#if   L==1	// Crs_STATUS_PENDING
	 "Pendent"
#elif L==2
	 "Offen"
#elif L==3
	 "Pending"
#elif L==4
	 "Pendiente"
#elif L==5
	 "En attente"
#elif L==6
	 "Pendiente"	// Okoteve traduccin
#elif L==7
	 "In attesa"
#elif L==8
	 "Do czasu"
#elif L==9
	 "Pendente"
#endif
	 ,
#if   L==1	// Crs_STATUS_REMOVED
	 "Eliminada"
#elif L==2
	 "Entfernt"
#elif L==3
	 "Removed"
#elif L==4
	 "Eliminada"
#elif L==5
	 "Supprim&eacute;"
#elif L==6
	 "Eliminada"	// Okoteve traduccin
#elif L==7
	 "Rimosso"
#elif L==8
	 "Usuniete"
#elif L==9
	 "Suprimida"
#endif
	};

const char *Txt_COURSE_With_users =
#if   L==1
	"Amb usuaris";
#elif L==2
	"Mit Benutzer";
#elif L==3
	"With users";
#elif L==4
	"Con usuarios";
#elif L==5
	"Avec utilisateurs";
#elif L==6
	"Con usuarios";	// Okoteve traduccin
#elif L==7
	"Con utenti";
#elif L==8
	"Z uzytkownikow";
#elif L==9
	"Com utilizadores";
#endif

const char *Txt_COURSE_Without_users =
#if   L==1
	"Sense usuaris";
#elif L==2
	"Ohne Benutzer";
#elif L==3
	"Without users";
#elif L==4
	"Sin usuarios";
#elif L==5
	"Sans utilisateurs";
#elif L==6
	"Sin usuarios";	// Okoteve traduccin
#elif L==7
	"Senza utenti";
#elif L==8
	"Bez uzytkownikow";
#elif L==9
	"Sem utilizadores";
#endif

const char *Txt_Course_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Asignatura <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2
	"Kurs <strong>%s</strong> entfernt.";
#elif L==3
	"Course <strong>%s</strong> removed.";
#elif L==4
	"Asignatura <strong>%s</strong> eliminada.";
#elif L==5
	"Mati&egrave;re <strong>%s</strong> supprim&eacute;.";
#elif L==6
	"Asignatura <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7
	"Corso <strong>%s</strong> rimosso.";
#elif L==8
	"Kurs <strong>%s</strong> usuniete.";
#elif L==9
	"Course <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Courses =
#if   L==1
	"Assignatures";
#elif L==2
	"Kursen";
#elif L==3
	"Courses";
#elif L==4
	"Asignaturas";
#elif L==5
	"Mati&egrave;res";
#elif L==6
	"Mbo'esyry";
#elif L==7
	"Corsi";
#elif L==8
	"Kurs&oacute;w";
#elif L==9
	"Disciplinas";
#endif

const char *Txt_Courses_ABBREVIATION =
#if   L==1
	"Assig.";
#elif L==2
	"Veran.";
#elif L==3
	"Cours.";
#elif L==4
	"Asig.";
#elif L==5
	"Mat.";
#elif L==6
	"Asig.";	// Okoteve traduccin
#elif L==7
	"Corsi";
#elif L==8
	"Kurs.";
#elif L==9
	"Disc.";
#endif

const char *Txt_courses =
#if   L==1
	"assignatures";
#elif L==2
	"Kursen";
#elif L==3
	"courses";
#elif L==4
	"asignaturas";
#elif L==5
	"mati&egrave;res";
#elif L==6
	"mbo'esyry";
#elif L==7
	"corsi";
#elif L==8
	"kurs&oacute;w";
#elif L==9
	"disciplinas";
#endif

const char *Txt_courses_ABBREVIATION =
#if   L==1
	"assig.";
#elif L==2
	"Veran.";
#elif L==3
	"cours.";
#elif L==4
	"asig.";
#elif L==5
	"mat.";
#elif L==6
	"asig.";	// Okoteve traduccin
#elif L==7
	"corsi";
#elif L==8
	"kurs.";
#elif L==9
	"disc.";
#endif

const char *Txt_X_courses_have_been_eliminated =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Se ha(n) eliminado %u asignatura(s).";		// Necessita traduccio
#elif L==2
	"%u course(s) have been eliminated.";		// Need bersetzung
#elif L==3
	"%u course(s) have been eliminated.";
#elif L==4
	"Se ha(n) eliminado %u asignatura(s).";
#elif L==5
	"%u course(s) have been eliminated.";		// Besoin de traduction
#elif L==6
	"Se ha(n) eliminado %u asignatura(s).";		// Okoteve traduccin
#elif L==7
	"%u corsi sono stati rimossi.";
#elif L==8
	"%u course(s) have been eliminated.";		// Potrzebujesz tlumaczenie
#elif L==9
	"%u course(s) have been eliminated.";		// Necessita de traduo
#endif

const char *Txt_Courses_of_DEGREE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Assignatures de %s";
#elif L==2
	"Kursen der %s";
#elif L==3
	"Courses of %s";
#elif L==4
	"Asignaturas de %s";
#elif L==5
	"Mati&egrave;res de %s";
#elif L==6
	"Asignaturas de %s";	// Okoteve traduccin
#elif L==7
	"Corsi di %s";
#elif L==8
	"Kurs&oacute;w %s";
#elif L==9
	"Disciplinas de %s";
#endif

const char *Txt_Create_account =
#if   L==1
	"Crea compte";
#elif L==2
	"Benutzerkonto anlegen";
#elif L==3
	"Create account";
#elif L==4
	"Crear cuenta";
#elif L==5
	"Cr&eacute;er compte";
#elif L==6
	"Crear cuenta";		// Okoteve traduccin
#elif L==7
	"Registrati";
#elif L==8
	"Za&lstrok;&oacute;&zdot; konto";
#elif L==9
	"Criar conta";
#endif

const char *Txt_Create_announcement =
#if   L==1
	"Crear anunci";
#elif L==2
	"Erstelle Bekanntmachung";
#elif L==3
	"Create announcement";
#elif L==4
	"Crear anuncio";
#elif L==5
	"Cr&eacute;er annonce";
#elif L==6
	"Crear anuncio";		// Okoteve traduccin
#elif L==7
	"Crea annuncio";
#elif L==8
	"Stw&oacute;rz og&lstrok;oszenia";
#elif L==9
	"Criar an&uacute;ncio";
#endif

const char *Txt_Create_another_centre =
#if   L==1
	"Crear un altre centre";
#elif L==2
	"Erstellen Sie ein anderes Lehrinstitut";
#elif L==3
	"Create another centre";
#elif L==4
	"Crear otro centro";
#elif L==5
	"Cr&eacute;er un autre centre";
#elif L==6
	"Crear otro centro";	// Okoteve traduccin
#elif L==7
	"Crea un altro centro";
#elif L==8
	"Tworzenie inny centrum";
#elif L==9
	"Criar um outro centro";
#endif

const char *Txt_Create_another_course =
#if   L==1
	"Crear una altra assignatura";
#elif L==2
	"Erstellen Sie ein anderes Kurs";
#elif L==3
	"Create another course";
#elif L==4
	"Crear otra asignatura";
#elif L==5
	"Cr&eacute;er une autre mati&egrave;re";
#elif L==6
	"Crear otra asignatura";	// Okoteve traduccin
#elif L==7
	"Crea un altro corso";
#elif L==8
	"Tworzenie inny kursu";
#elif L==9
	"Criar uma outra disciplina";
#endif

const char *Txt_Create_another_degree =
#if   L==1
	"Crear una altra titulaci&oacute;";
#elif L==2
	"Erstellen Sie ein anderes Studiengang";
#elif L==3
	"Create another degree";
#elif L==4
	"Crear otra titulaci&oacute;n";
#elif L==5
	"Cr&eacute;er un autre &eacute;tude";
#elif L==6
	"Crear otra titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Crea una altra laurea";
#elif L==8
	"Tworzenie inny stopnia";
#elif L==9
	"Criar uma outra titula&ccedil;&atilde;o";
#endif

const char *Txt_Create_another_institution =
#if   L==1
	"Crear una altra instituci&oacute;";
#elif L==2
	"Erstellen Sie ein anderes Hochschule";
#elif L==3
	"Create another institution";
#elif L==4
	"Crear otra instituci&oacute;n";
#elif L==5
	"Cr&eacute;er un autre &eacute;tablissement";
#elif L==6
	"Crear otra instituci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Crea una altra istituzione";
#elif L==8
	"Tworzenie inny instytucji";
#elif L==9
	"Criar uma outra institu&ccedil;&atilde;o";
#endif

const char *Txt_Create_assignment =
#if   L==1
	"Crear actividad";	// Necessita traduccio
#elif L==2
	"Aufgabe erstellen";
#elif L==3
	"Create assignment";
#elif L==4
	"Crear actividad";
#elif L==5
	"Cr&eacute;er activit&eacute;";
#elif L==6
	"Crear actividad";	// Okoteve traduccin
#elif L==7
	"Crea attivit&agrave;";
#elif L==8
	"Utw&oacute;rz zadanie";
#elif L==9
	"Criar atividade";
#endif

const char *Txt_Create_banner =
#if   L==1
	"Crear banner";	// Necessita traduccio
#elif L==2
	"Werbebanner erstellen";
#elif L==3
	"Create banner";
#elif L==4
	"Crear banner";
#elif L==5
	"Cr&eacute;er banni&eacute;re";
#elif L==6
	"Crear banner";	// Okoteve traduccin
#elif L==7
	"Crea banner";
#elif L==8
	"Utw&oacute;rz banner";
#elif L==9
	"Criar banner";
#endif

const char *Txt_Create_centre =
#if   L==1
	"Crear centre";
#elif L==2
	"Lehrinstitut eingeben";
#elif L==3
	"Create centre";
#elif L==4
	"Crear centro";
#elif L==5
	"Cr&eacute;er centre";
#elif L==6
	"Crear centro";		// Okoteve traduccin
#elif L==7
	"Crea centro";
#elif L==8
	"Tworzenie centrum";
#elif L==9
	"Criar centro";
#endif

const char *Txt_Create_country =
#if   L==1
	"Crear pa&iacute;s";	// Necessita traduccio
#elif L==2
	"Land eingeben";
#elif L==3
	"Create country";
#elif L==4
	"Crear pa&iacute;s";
#elif L==5
	"Cr&eacute;er pays";
#elif L==6
	"Crear pa&iacute;s";	// Okoteve traduccin
#elif L==7
	"Crea paese";
#elif L==8
	"Tworzenie kraju";
#elif L==9
	"Criar pa&iacute;s";
#endif

const char *Txt_Create_course =
#if   L==1
	"Crear assignatura";
#elif L==2
	"Kurs eingeben";
#elif L==3
	"Create course";
#elif L==4
	"Crear asignatura";
#elif L==5
	"Cr&eacute;er mati&egrave;re";
#elif L==6
	"Crear asignatura";	// Okoteve traduccin
#elif L==7
	"Crea corso";
#elif L==8
	"Tworzenie kursu";
#elif L==9
	"Criar disciplina";
#endif

const char *Txt_Create_degree =
#if   L==1
	"Crear titulaci&oacute;";
#elif L==2
	"Studiengang eingeben";
#elif L==3
	"Create degree";
#elif L==4
	"Crear titulaci&oacute;n";
#elif L==5
	"Cr&eacute;er &eacute;tude";
#elif L==6
	"Crear titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Crea laurea";
#elif L==8
	"Tworzenie stopnia";
#elif L==9
	"Criar titula&ccedil;&atilde;o";
#endif

const char *Txt_Create_department =
#if   L==1
	"Crear departamento";	// Necessita traduccio
#elif L==2
	"Abteilung eingeben";
#elif L==3
	"Create department";
#elif L==4
	"Crear departamento";
#elif L==5
	"Cr&eacute;er d&eacute;partement";
#elif L==6
	"Crear departamento";	// Okoteve traduccin
#elif L==7
	"Crea dipartimento";
#elif L==8
	"Tworzenie dzia&lstrok;u";
#elif L==9
	"Criar departamento";
#endif

const char *Txt_Create_email_domain =
#if   L==1
	"Crear dominio de correo";	// Necessita traduccio
#elif L==2
	"E-Mail-Dom&auml;ne anlegen";
#elif L==3
	"Create email domain";
#elif L==4
	"Crear dominio de correo";
#elif L==5
	"Cr&eacute;er domaine de messagerie";
#elif L==6
	"Crear dominio de correo";	// Okoteve traduccin
#elif L==7
	"Crea campo mail";
#elif L==8
	"Tworzenie domeny mail";
#elif L==9
	"Criar dom&iacute;nio de email";
#endif

const char *Txt_Create_email_message =
#if   L==1
	"Crear mensaje de correo";	// Necessita traduccio
#elif L==2
	"Neue E-Mail-Nachricht";
#elif L==3
	"Create email message";
#elif L==4
	"Crear mensaje de correo";
#elif L==5
	"Cr&eacute;er un email message";
#elif L==6
	"Crear mensaje de correo";	// Okoteve traduccin
#elif L==7
	"Crea messaggio email";
#elif L==8
	"Tworzenie wiadomosci email";
#elif L==9
	"Criar uma mensagem de email";
#endif

const char *Txt_Create_event =
#if   L==1
	"Crear evento";	// Necessita traduccio
#elif L==2
	"Ereignis erstellen";
#elif L==3
	"Create event";
#elif L==4
	"Crear evento";
#elif L==5
	"Cr&eacute;er &eacute;v&eacute;nement";
#elif L==6
	"Crear evento";	// Okoteve traduccin
#elif L==7
	"Crea evento";
#elif L==8
	"Utw&oacute;rz wydarzenie";
#elif L==9
	"Criar evento";
#endif

const char *Txt_Create_folder =
#if   L==1
	"Crear carpeta";	// Necessita traduccio
#elif L==2
	"Verzeichnis anlegen";
#elif L==3
	"Create folder";
#elif L==4
	"Crear carpeta";
#elif L==5
	"Cr&eacute;er r&eacute;pertoire";
#elif L==6
	"Crear carpeta";	// Okoteve traduccin
#elif L==7
	"Crea cartella";
#elif L==8
	"Utw&oacute;rz folder";
#elif L==9
	"Criar diret&oacute;rio";
#endif

const char *Txt_Create_group =
#if   L==1
	"Crear grupo";		// Necessita traduccio
#elif L==2
	"Gruppe erstellen";
#elif L==3
	"Create group";
#elif L==4
	"Crear grupo";
#elif L==5
	"Cr&eacute;er groupe";
#elif L==6
	"Crear grupo";		// Okoteve traduccin
#elif L==7
	"Crea gruppo";
#elif L==8
	"Create group";		// Potrzebujesz tlumaczenie
#elif L==9
	"Criar grupo";
#endif

const char *Txt_Create_holiday =
#if   L==1
	"Crear festividad";	// Necessita traduccio
#elif L==2
	"Feiertag eingeben";
#elif L==3
	"Create holiday";
#elif L==4
	"Crear festividad";
#elif L==5
	"Cr&eacute;er f&ecirc;te";
#elif L==6
	"Crear festividad";	// Okoteve traduccin
#elif L==7
	"Crea festivit&agrave;";
#elif L==8
	"Tworzenie wakacje";
#elif L==9
	"Criar f&eacute;ria";
#endif

const char *Txt_Create_institution =
#if   L==1
	"Crear instituci&oacute;";
#elif L==2
	"Hochschule eingeben";
#elif L==3
	"Create institution";
#elif L==4
	"Crear instituci&oacute;n";
#elif L==5
	"Cr&eacute;er &eacute;tablissement";
#elif L==6
	"Crear instituci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Crea istituzione";
#elif L==8
	"Tworzenie instytucji";
#elif L==9
	"Criar institu&ccedil;&atilde;o";
#endif

const char *Txt_Create_link =
#if   L==1
	"Crear enlla&ccedil;";
#elif L==2
	"Link erstellen";
#elif L==3
	"Create link";
#elif L==4
	"Crear enlace";
#elif L==5
	"Cr&eacute;er lien";
#elif L==6
	"Crear enlace";	// Okoteve traduccin
#elif L==7
	"Crea link";
#elif L==8
	"Utw&oacute;rz link";
#elif L==9
	"Criar liga&ccedil;&atilde;o";
#endif

const char *Txt_Create_notice =
#if   L==1
	"Crear av&iacute;s";
#elif L==2
	"Erstelle Ank&uuml;ndigung";
#elif L==3
	"Create notice";
#elif L==4
	"Crear aviso";
#elif L==5
	"Cr&eacute;er avis";
#elif L==6
	"Crear aviso";		// Okoteve traduccin
#elif L==7
	"Crea avviso";
#elif L==8
	"Stw&oacute;rz powiadomienie";
#elif L==9
	"Criar aviso";
#endif

const char *Txt_Create_BR_notification =
#if   L==1
	"Crear<br />notificaci&oacute;";
#elif L==2
	"Anmeldung<br />eingeben";
#elif L==3
	"Create<br />notification";
#elif L==4
	"Crear<br />notificaci&oacute;n";
#elif L==5
	"Cr&eacute;er<br />notification";
#elif L==6
	"Crear<br />notificaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Crea<br />notifica";
#elif L==8
	"Tworzenie<br />powiadomienie";
#elif L==9
	"Criar<br />notifica&ccedil;&atilde;o";
#endif

const char *Txt_Create_password =
#if   L==1
	"Crear contrasenya";
#elif L==2
	"Passwort erstellen";
#elif L==3
	"Crear password";
#elif L==4
	"Cambiar contrase&ntilde;a";
#elif L==5
	"Cr&eacute;er le mot de passe";
#elif L==6
	"Crear contrase&ntilde;a";	// Okoteve traduccin
#elif L==7
	"Cambiare password";
#elif L==8
	"Utw&oacute;rz has&lstrok;o";
#elif L==9
	"Criar senha";
#endif

const char *Txt_Create_place =
#if   L==1
	"Crear lugar";	// Necessita traduccio
#elif L==2
	"Standort eingeben";
#elif L==3
	"Create place";
#elif L==4
	"Crear lugar";
#elif L==5
	"Cr&eacute;er emplacement";
#elif L==6
	"Crear lugar";	// Okoteve traduccin
#elif L==7
	"Crea localit&grave;";
#elif L==8
	"Tworzenie miejsce";
#elif L==9
	"Criar localiza&ccedil;&atilde;o";
#endif

const char *Txt_Create_plugin =
#if   L==1
	"Crear complemento";	// Necessita traduccio
#elif L==2
	"Plugin erstellen";
#elif L==3
	"Create plugin";
#elif L==4
	"Crear complemento";
#elif L==5
	"Cr&eacute;er plugin";
#elif L==6
	"Crear complemento";	// Okoteve traduccin
#elif L==7
	"Crea plugin";
#elif L==8
	"Tworzenie plugin";
#elif L==9
	"Criar plugin";
#endif

const char *Txt_Create_question =
#if   L==1
	"Crear pregunta";
#elif L==2
	"Frage erstellen";
#elif L==3
	"Create question";
#elif L==4
	"Crear pregunta";
#elif L==5
	"Cr&eacute;er question";
#elif L==6
	"Crear pregunta";	// Okoteve traduccin
#elif L==7
	"Crea domanda";
#elif L==8
	"Tworzenie pytanie";
#elif L==9
	"Criar quest&atilde;o";
#endif

const char *Txt_Create_record_field =
#if   L==1
	"Crear campo de ficha";	// Necessita traduccio
#elif L==2
	"Datenfeld erstellen";
#elif L==3
	"Create record field";
#elif L==4
	"Crear campo de ficha";
#elif L==5
	"Cr&eacute;er champ de carte";
#elif L==6
	"Crear campo de ficha";	// Okoteve traduccin
#elif L==7
	"Crea campo di scheda";
#elif L==8
	"Utw&oacute;rz rekord pola";
#elif L==9
	"Criar campo de cart&atilde;o";
#endif

const char *Txt_Create_survey =
#if   L==1
	"Crear enquesta";
#elif L==2
	"Umfrage erstellen";
#elif L==3
	"Create survey";
#elif L==4
	"Crear encuesta";
#elif L==5
	"Cr&eacute;er sondage";
#elif L==6
	"Crear encuesta";	// Okoteve traduccin
#elif L==7
	"Crea sondaggio";
#elif L==8
	"Tworzenie ankiety";
#elif L==9
	"Criar inqu&eacute;rito";
#endif

const char *Txt_Create_type_of_degree =
#if   L==1
	"Crear tipo de titulaci&oacute;n";	// Necessita traduccio
#elif L==2
	"Abschlussart definieren";
#elif L==3
	"Create type of degree";
#elif L==4
	"Crear tipo de titulaci&oacute;n";
#elif L==5
	"Cr&eacute;er type d'&eacute;tude";
#elif L==6
	"Crear tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Crea tipo di laurea";
#elif L==8
	"Tworzenie typu stopnia";
#elif L==9
	"Criar tipo de titula&ccedil;&atilde;o";
#endif

const char *Txt_Create_type_of_group =
#if   L==1
	"Crear tipo de grupo";	// Necessita traduccio
#elif L==2
	"Gruppen-Typ definieren";
#elif L==3
	"Create type of group";
#elif L==4
	"Crear tipo de grupo";
#elif L==5
	"Cr&eacute;er type de groupe";
#elif L==6
	"Crear tipo de grupo";	// Okoteve traduccin
#elif L==7
	"Crea tipo di gruppo";
#elif L==8
	"Tworzenie typu grupy";
#elif L==9
	"Criar tipo de grupo";
#endif

const char *Txt_Create_ZIP_file =
#if   L==1
	"Crear archivo ZIP";	// Necessita traduccio
#elif L==2
	"ZIP-Datei erstellen";
#elif L==3
	"Create ZIP file";
#elif L==4
	"Crear archivo ZIP";
#elif L==5
	"Cr&eacute;er fichier ZIP";
#elif L==6
	"Crear archivo ZIP";	// Okoteve traduccin
#elif L==7
	"Crea file ZIP";
#elif L==8
	"Tworzenie pliku ZIP";
#elif L==9
	"Criar arquivo ZIP";
#endif

const char *Txt_Created_new_announcement_of_exam =
#if   L==1
	"Creada nueva convocatoria de examen.";	// Necessita traduccio
#elif L==2
	"Neuer Aufruf zur Pr&uuml;fung erstellt.";
#elif L==3
	"Created new announcement of exam.";
#elif L==4
	"Creada nueva convocatoria de examen.";
#elif L==5
	"Cr&eacute;&eacute; nouvelle convocation &agrave; un examen.";
#elif L==6
	"Creada nueva convocatoria de examen.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo appello d'esame.";
#elif L==8
	"Utworzono nowe ogloszenie egzamin.";
#elif L==9
	"Criado nova chamada para exame.";
#endif

const char *Txt_Created_new_assignment_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creada nueva actividad <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neue Aufgabe <strong>%s</strong> erstellt.";
#elif L==3
	"Created new assignment <strong>%s</strong>.";
#elif L==4
	"Creada nueva actividad <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute;e nouvelle activit&eacute; <strong>%s</strong>.";
#elif L==6
	"Creada nueva actividad <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creata nuova attivit&agrave; <strong>%s</strong>.";
#elif L==8
	"Utworzono nowa zadania <strong>%s</strong>.";
#elif L==9
	"Criada nova atividade <strong>%s</strong>.";
#endif

const char *Txt_Created_new_banner_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo banner <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neuer Werbebanner <strong>%s</strong> erstellt.";
#elif L==3
	"Created new banner <strong>%s</strong>.";
#elif L==4
	"Creado nuevo banner <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouvelle banni&eacute;re <strong>%s</strong>.";
#elif L==6
	"Creado nuevo banner <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo banner <strong>%s</strong>.";
#elif L==8
	"Utworzono nowy banner <strong>%s</strong>.";
#elif L==9
	"Criado novo banner <strong>%s</strong>.";
#endif

const char *Txt_Created_new_centre_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo centro <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neues Lehrinstitut <strong>%s</strong> eingegeben.";
#elif L==3
	"Created new centre <strong>%s</strong>.";
#elif L==4
	"Creado nuevo centro <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouveau centre <strong>%s</strong>.";
#elif L==6
	"Creado nuevo centro <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo centro <strong>%s</strong>.";
#elif L==8
	"Utworzono nowe centrum <strong>%s</strong>.";
#elif L==9
	"Criado novo centro <strong>%s</strong>.";
#endif

const char *Txt_Created_new_country_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo pa&iacute;s <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neues Land <strong>%s</strong> eingegeben.";
#elif L==3
	"Created new country <strong>%s</strong>.";
#elif L==4
	"Creado nuevo pa&iacute;s <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouveau pays <strong>%s</strong>.";
#elif L==6
	"Creado nuevo pa&iacute;s <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo paese <strong>%s</strong>.";
#elif L==8
	"Utworzono nowa kraju <strong>%s</strong>.";
#elif L==9
	"Criado novo pa&iacute;s <strong>%s</strong>.";
#endif

const char *Txt_Created_new_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creada nueva asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neue Kurs <strong>%s</strong> eingegeben.";
#elif L==3
	"Created new course <strong>%s</strong>.";
#elif L==4
	"Creada nueva asignatura <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute;e nouvelle mati&egrave;re <strong>%s</strong>.";
#elif L==6
	"Creada nueva asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo corso <strong>%s</strong>.";
#elif L==8
	"Utworzono nowa oczywiscie <strong>%s</strong>.";
#elif L==9
	"Criada nova disciplina <strong>%s</strong>.";
#endif

const char *Txt_Created_new_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neuer Studiengang <strong>%s</strong> eingegeben.";
#elif L==3
	"Created new degree <strong>%s</strong>.";
#elif L==4
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute;e nouvel &eacute;tude <strong>%s</strong>.";
#elif L==6
	"Creada nueva titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creata nuova laurea <strong>%s</strong>.";
#elif L==8
	"Utworzono nowy poziom <strong>%s</strong>.";
#elif L==9
	"Criada nova titula&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_department_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo departamento <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neue Abteilung <strong>%s</strong> eingegeben.";
#elif L==3
	"Created new department <strong>%s</strong>.";
#elif L==4
	"Creado nuevo departamento <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouveau d&eacute;partement <strong>%s</strong>.";
#elif L==6
	"Creado nuevo departamento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo dipartimento <strong>%s</strong>.";
#elif L==8
	"Utworzono nowy dzia&lstrok; <strong>%s</strong>.";
#elif L==9
	"Criado novo departamento <strong>%s</strong>.";
#endif

const char *Txt_Created_new_email_domain_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo dominio de correo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neue E-Mail-Dom&auml;ne <strong>%s</strong> angelegt.";
#elif L==3
	"Created new email domain <strong>%s</strong>.";
#elif L==4
	"Creado nuevo dominio de correo <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouvel domaine de messagerie <strong>%s</strong>.";
#elif L==6
	"Creado nuevo dominio de correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo campo mail <strong>%s</strong>.";
#elif L==8
	"Utworzono nowa domena mail <strong>%s</strong>.";
#elif L==9
	"Criado novo dom&iacute;nio de email <strong>%s</strong>.";
#endif

const char *Txt_Created_new_event_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo evento <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neues Ereignis <strong>%s</strong> eingegeben.";
#elif L==3
	"Created new event <strong>%s</strong>.";
#elif L==4
	"Creado nuevo evento <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouveau &eacute;v&eacute;nement <strong>%s</strong>.";
#elif L==6
	"Creado nuevo evento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo evento <strong>%s</strong>.";
#elif L==8
	"Utworzono nowe wydarzenie <strong>%s</strong>.";
#elif L==9
	"Criado novo evento <strong>%s</strong>.";
#endif

const char *Txt_Created_new_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo grupo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neue Gruppe <strong>%s</strong> erstellt.";
#elif L==3
	"Created new group <strong>%s</strong>.";
#elif L==4
	"Creado nuevo grupo <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouveau groupe <strong>%s</strong>.";
#elif L==6
	"Creado nuevo grupo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo gruppo <strong>%s</strong>.";
#elif L==8
	"Stworzyl nowa grupe <strong>%s</strong>.";
#elif L==9
	"Criado novo grupo <strong>%s</strong>.";
#endif

const char *Txt_Created_new_holiday_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creada nueva festividad <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neuer Feiertag <strong>%s</strong> eingegeben.";
#elif L==3
	"Created new holiday <strong>%s</strong>.";
#elif L==4
	"Creada nueva festividad <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouvelle f&ecirc;te <strong>%s</strong>.";
#elif L==6
	"Creada nueva festividad <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creata nuova festivit&agrave; <strong>%s</strong>.";
#elif L==8
	"Utworzono nowe swieto <strong>%s</strong>.";
#elif L==9
	"Criado nova f&eacute;ria <strong>%s</strong>.";
#endif

const char *Txt_Created_new_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creada nueva instituci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neue Hochschule <strong>%s</strong> eingegeben.";
#elif L==3
	"Created new institution <strong>%s</strong>.";
#elif L==4
	"Creada nueva instituci&oacute;n <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute;e nouvel &eacute;tablissement <strong>%s</strong>.";
#elif L==6
	"Creada nueva instituci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creata nuova istituzione <strong>%s</strong>.";
#elif L==8
	"Utworzono nowa instytucje <strong>%s</strong>.";
#elif L==9
	"Criada nova institu&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_link_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo enlace <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neuer Link <strong>%s</strong> erstellt.";
#elif L==3
	"Created new link <strong>%s</strong>.";
#elif L==4
	"Creado nuevo enlace <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouveau lien <strong>%s</strong>.";
#elif L==6
	"Creado nuevo enlace <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo link <strong>%s</strong>.";
#elif L==8
	"Utworzono nowy link <strong>%s</strong>.";
#elif L==9
	"Criado nova liga&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_place_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo lugar <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neuer Standort <strong>%s</strong> eingegeben.";
#elif L==3
	"Created new place <strong>%s</strong>.";
#elif L==4
	"Creado nuevo lugar <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouvel emplacement <strong>%s</strong>.";
#elif L==6
	"Creado nuevo lugar <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creata nuova localit&grave; <strong>%s</strong>.";
#elif L==8
	"Utworzono nowe miejsce <strong>%s</strong>.";
#elif L==9
	"Criado nova localiza&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_plugin_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo complemento <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neuer Plugin <strong>%s</strong> erstellt.";
#elif L==3
	"Created new plugin <strong>%s</strong>.";
#elif L==4
	"Creado nuevo complemento <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouveau plugin <strong>%s</strong>.";
#elif L==6
	"Creado nuevo complemento <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo plugin <strong>%s</strong>.";
#elif L==8
	"Utworzono nowy plugin <strong>%s</strong>.";
#elif L==9
	"Criado novo plugin <strong>%s</strong>.";
#endif

const char *Txt_Created_new_record_field_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo campo de ficha <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neuer Datenfeld <strong>%s</strong> erstellt.";
#elif L==3
	"Created new record field <strong>%s</strong>.";
#elif L==4
	"Creado nuevo campo de ficha <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouveau champ de carte <strong>%s</strong>.";
#elif L==6
	"Creado nuevo campo de ficha <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo campo di scheda <strong>%s</strong>.";
#elif L==8
	"Utworzono nowe pola rekordu <strong>%s</strong>.";
#elif L==9
	"Created new record field <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_Created_new_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creada nueva encuesta <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neuer Umfrage <strong>%s</strong> erstellt.";
#elif L==3
	"Created new survey <strong>%s</strong>.";
#elif L==4
	"Creada nueva encuesta <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouveau sondage <strong>%s</strong>.";
#elif L==6
	"Creada nueva encuesta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo sondaggio <strong>%s</strong>.";
#elif L==8
	"Utworzono nowe badania <strong>%s</strong>.";
#elif L==9
	"Criado novo inqu&eacute;rito <strong>%s</strong>.";
#endif

const char *Txt_Created_new_type_of_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neue Abschlussart <strong>%s</strong> definiert.";
#elif L==3
	"Created new type of degree <strong>%s</strong>.";
#elif L==4
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouveau type d'&eacute;tude <strong>%s</strong>.";
#elif L==6
	"Creado nuevo tipo de titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo tipo di laurea <strong>%s</strong>.";
#elif L==8
	"Stworzyl nowy typ stopnia <strong>%s</strong>.";
#elif L==9
	"Criado novo tipo de titula&ccedil;&atilde;o <strong>%s</strong>.";
#endif

const char *Txt_Created_new_type_of_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Creado nuevo tipo de grupo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Neuer Gruppen-Typ <strong>%s</strong> definiert.";
#elif L==3
	"Created new type of group <strong>%s</strong>.";
#elif L==4
	"Creado nuevo tipo de grupo <strong>%s</strong>.";
#elif L==5
	"Cr&eacute;&eacute; nouveau type de groupe <strong>%s</strong>.";
#elif L==6
	"Creado nuevo tipo de grupo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Creato nuovo tipo di gruppo <strong>%s</strong>.";
#elif L==8
	"Stworzyl nowy typ grupy <strong>%s</strong>.";
#elif L==9
	"Criado novo tipo de grupo <strong>%s</strong>.";
#endif

const char *Txt_Created_tables_in_the_database_that_did_not_exist =
#if   L==1
	"Creadas las tablas de la base de datos que no exist&iacute;an.";	// Necessita traduccio
#elif L==2
	"Neue Tabellen in der Datenbank erstellt.";
#elif L==3
	"Created tables in the database that did not exist.";
#elif L==4
	"Creadas las tablas de la base de datos que no exist&iacute;an.";
#elif L==5
	"Tables cr&eacute;&eacute;es dans la base de donn&eacute;es qui n'existaient pas.";
#elif L==6
	"Creadas las tablas de la base de datos que no exist&iacute;an.";	// Okoteve traduccin
#elif L==7
	"Tabelle create in un database inesistente.";
#elif L==8
	"Wykonawca tabel w bazie danych, kt&oacute;re nie istnialy.";
#elif L==9
	"Criadas tabelas no banco de dados que n&atilde;o existiam.";
#endif

const char *Txt_Creating_database_tables_if_they_do_not_exist =
#if   L==1
	"Creando tablas de la base de datos si no existen&hellip;";	// Necessita traduccio
#elif L==2
	"Creating database tables if they do not exist&hellip;";	// Need bersetzung
#elif L==3
	"Creating database tables if they do not exist&hellip;";
#elif L==4
	"Creando tablas de la base de datos si no existen&hellip;";
#elif L==5
	"Creating database tables if they do not exist&hellip;";	// Besoin de traduction
#elif L==6
	"Creando tablas de la base de datos si no existen&hellip;";	// Okoteve traduccin
#elif L==7
	"Creazione tabelle della base di dati se non esistono&hellip;";
#elif L==8
	"Tworzenie tabel bazy danych, jesli ich nie ma &hellip;";
#elif L==9
	"Creating database tables if they do not exist&hellip;";	// Necessita de traduo
#endif

const char *Txt_crs =	// Abbreviation of "course" (three characters + dot)
#if   L==1
	"asg.";		// Necessita traduccio
#elif L==2
	"Ver.";
#elif L==3
	"crs.";
#elif L==4
	"asg.";
#elif L==5
	"mat.";
#elif L==6
	"mbo'esyry";
#elif L==7
	"crs.";
#elif L==8
	"Oczywiscie.";
#elif L==9
	"crs.";
#endif

const char *Txt_Current_email =
#if   L==1
	"Correu actual";
#elif L==2
	"Aktueller E-Mail";
#elif L==3
	"Current email";
#elif L==4
	"Correo actual";
#elif L==5
	"Courrier actuel";
#elif L==6
	"Correo actual";	// Okoteve traduccin
#elif L==7
	"Email corrente";
#elif L==8
	"Obecny email";
#elif L==9
	"Email atual";
#endif

const char *Txt_Current_image =
#if   L==1
	"Imatge actual";
#elif L==2
	"Aktuelles Abbild";
#elif L==3
	"Current image";
#elif L==4
	"Imagen actual";
#elif L==5
	"Image actuelle";
#elif L==6
	"Imagen actual";	// Okoteve traduccin
#elif L==7
	"Immagine corrente";
#elif L==8
	"Bie&zdot;&aogon;cy obraz";
#elif L==9
	"Imagem atual";
#endif

const char *Txt_Current_nickname =
#if   L==1
	"Sobrenom actual";
#elif L==2
	"Aktueller Benutzername";
#elif L==3
	"Current nickname";
#elif L==4
	"Apodo actual";
#elif L==5
	"Surnom actuel";
#elif L==6
	"Apodo actual";	// Okoteve traduccin
#elif L==7
	"Nome utente corrente";
#elif L==8
	"Obecny pseudonim";
#elif L==9
	"Alcunha atual";
#endif

const char *Txt_Current_password =
#if   L==1
	"Contrasenya actual";
#elif L==2
	"Aktuelles Passwort";
#elif L==3
	"Current password";
#elif L==4
	"Contrase&ntilde;a actual";
#elif L==5
	"Mot de passe actuel";
#elif L==6
	"Contrase&ntilde;a actual";	// Okoteve traduccin
#elif L==7
	"Password corrente";
#elif L==8
	"Obecne has&lstrok;o";
#elif L==9
	"Senha atual";
#endif

const char *Txt_DATA_PROTECTION_CLAUSE =
#if   L==1
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Necessita traduccio
#elif L==2
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Need bersetzung
#elif L==3
	"Data protection clause";
#elif L==4
	"Pol&iacute;tica de protecci&oacute;n de datos";
#elif L==5
	"Clause de protection des donn&eacute;es";
#elif L==6
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Okoteve traduccin
#elif L==7
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Bisogno di traduzione
#elif L==8
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Potrzebujesz tlumaczenie
#elif L==9
	"Pol&iacute;tica de protecci&oacute;n de datos";	// Necessita de traduo
#endif

const char *Txt_Date =
#if   L==1
	"Data";
#elif L==2
	"Datum";
#elif L==3
	"Date";
#elif L==4
	"Fecha";
#elif L==5
	"Date";
#elif L==6
	"Arange";
#elif L==7
	"Data";
#elif L==8
	"Data";
#elif L==9
	"Data";
#endif

const char *Txt_Date_of_birth =
#if   L==1
	"Data naixement";
#elif L==2
	"Geburtsdatum";
#elif L==3
	"Date of birth";
#elif L==4
	"Fecha nacimiento";
#elif L==5
	"Date naissance";
#elif L==6
	"Fecha nacimiento";	// Okoteve traduccin
#elif L==7
	"Data di nascita";
#elif L==8
	"Data urodzenia";
#elif L==9
	"Data nascimento";
#endif

const char *Txt_Date_of_creation =
#if   L==1
	"Data de creaci&oacute;";
#elif L==2
	"Datum der Erstellung";
#elif L==3
	"Date of creation";
#elif L==4
	"Fecha de creaci&oacute;n";
#elif L==5
	"Date de cr&eacute;ation";
#elif L==6
	"Fecha de creaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Data di creazione";
#elif L==8
	"Data utworzenia";
#elif L==9
	"Data da cria&ccedil;&atilde;o";
#endif

const char *Txt_Day =
#if   L==1
	"D&iacute;a";	// Necessita traduccio
#elif L==2
	"Tag";
#elif L==3
	"Day";
#elif L==4
	"D&iacute;a";
#elif L==5
	"Jour";
#elif L==6
	"D&iacute;a";	// Okoteve traduccin
#elif L==7
	"Giorno";
#elif L==8
	"Dzien";
#elif L==9
	"Dia";
#endif

const char *Txt_day =
#if   L==1
	"dia";
#elif L==2
	"Tag";
#elif L==3
	"day";
#elif L==4
	"d&iacute;a";
#elif L==5
	"jour";
#elif L==6
	"&aacute;ra";
#elif L==7
	"giorno";
#elif L==8
	"dzie&nacute;";
#elif L==9
	"dia";
#endif

const char *Txt_days =
#if   L==1
	"dies";
#elif L==2
	"Tage";
#elif L==3
	"days";
#elif L==4
	"d&iacute;as";
#elif L==5
	"jours";
#elif L==6
	"&aacute;ra";
#elif L==7
	"giorni";
#elif L==8
	"dni;";
#elif L==9
	"dias";
#endif

const char *Txt_DAYS_CAPS[7] =
   {
#if   L==1
	 "DILLUNS"
#elif L==2
	 "MONTAG"
#elif L==3
	 "MONDAY"
#elif L==4
	 "LUNES"
#elif L==5
	 "LUNDI"
#elif L==6
	 "ARAKOI"
#elif L==7
	 "LUNED&Igrave;"
#elif L==8
	 "Poniedzialek"
#elif L==9
	 "SEGUNDA-FEIRA"
#endif
	 ,
#if   L==1
	 "DIMARTS"
#elif L==2
	 "DIENSTAG"
#elif L==3
	 "TUESDAY"
#elif L==4
	 "MARTES"
#elif L==5
	 "MARDI"
#elif L==6
	 "ARAAPY"
#elif L==7
	 "MARTED&Igrave;"
#elif L==8
	 "Wtorek"
#elif L==9
	 "TER&Ccedil;A-FEIRA"
#endif
	 ,
#if   L==1
	 "DIMECRES"
#elif L==2
	 "MITTWOCH"
#elif L==3
	 "WEDNESDAY"
#elif L==4
	 "MI&Eacute;RCOLES"
#elif L==5
	 "MERCREDI"
#elif L==6
	 "ARARUNDY"
#elif L==7
	 "MERCOLED&Igrave;"
#elif L==8
	 "Sroda"
#elif L==9
	 "QUARTA-FEIRA"
#endif
	 ,
#if   L==1
	 "DIJOUS"
#elif L==2
	 "DONNERSTAG"
#elif L==3
	 "THURSDAY"
#elif L==4
	 "JUEVES"
#elif L==5
	 "JEUDI"
#elif L==6
	 "ARAPO"
#elif L==7
	 "GIOVED&Igrave;"
#elif L==8
	 "Czwartek"
#elif L==9
	 "QUINTA-FEIRA"
#endif
	 ,
#if   L==1
	 "DIVENDRES"
#elif L==2
	 "FREITAG"
#elif L==3
	 "FRIDAY"
#elif L==4
	 "VIERNES"
#elif L==5
	 "VENDREDI"
#elif L==6
	 "ARAPOTE&#296;"
#elif L==7
	 "VENERD&Igrave;"
#elif L==8
	 "Piatek"
#elif L==9
	 "SEXTA-FEIRA"
#endif
	 ,
#if   L==1
	 "DISSABTE"
#elif L==2
	 "SAMSTAG"
#elif L==3
	 "SATURDAY"
#elif L==4
	 "S&Aacute;BADO"
#elif L==5
	 "SAMEDI"
#elif L==6
	 "ARAPOK&Otilde;I"
#elif L==7
	 "SABATO"
#elif L==8
	 "Sobota"
#elif L==9
	 "SABADO"
#endif
	 ,
#if   L==1
	 "DIUMENGE"
#elif L==2
	 "SONNTAG"
#elif L==3
	 "SUNDAY"
#elif L==4
	 "DOMINGO"
#elif L==5
	 "DIMANCHE"
#elif L==6
	 "ARATE&#296;"
#elif L==7
	 "DOMENICA"
#elif L==8
	 "Niedziela"
#elif L==9
	 "DOMINGO"
#endif
	};

const char *Txt_DAYS_SMALL[7] =
   {
#if   L==1
	 "dilluns"
#elif L==2
	 "Montag"
#elif L==3
	 "Monday"
#elif L==4
	 "lunes"
#elif L==5
	 "Lundi"
#elif L==6
	 "arak&otilde;i"
#elif L==7
	 "Luned&igrave;"
#elif L==8
	 "Poniedzialek"
#elif L==9
	 "Segunda-feira"
#endif
	 ,
#if   L==1
	 "dimarts"
#elif L==2
	 "Dienstag"
#elif L==3
	 "Tuesday"
#elif L==4
	 "martes"
#elif L==5
	 "Mardi"
#elif L==6
	 "araapy"
#elif L==7
	 "Marted&igrave;"
#elif L==8
	 "Wtorek"
#elif L==9
	 "Ter&ccedil;a-feira"
#endif
	 ,
#if   L==1
	 "dimecres"
#elif L==2
	 "Mittwoch"
#elif L==3
	 "Wednesday"
#elif L==4
	 "mi&eacute;rcoles"
#elif L==5
	 "Mercredi"
#elif L==6
	 "ararundy"
#elif L==7
	 "Mercoled&igrave;"
#elif L==8
	 "Sroda"
#elif L==9
	 "Quarta-feira"
#endif
	 ,
#if   L==1
	 "dijous"
#elif L==2
	 "Donnerstag"
#elif L==3
	 "Thursday"
#elif L==4
	 "jueves"
#elif L==5
	 "Jeudi"
#elif L==6
	 "arapo"
#elif L==7
	 "Gioved&igrave;"
#elif L==8
	 "Czwartek"
#elif L==9
	 "Quinta-feira"
#endif
	 ,
#if   L==1
	 "divendres"
#elif L==2
	 "Freitag"
#elif L==3
	 "Friday"
#elif L==4
	 "viernes"
#elif L==5
	 "Vendredi"
#elif L==6
	 "arapote&#297;"
#elif L==7
	 "Venerd&igrave;"
#elif L==8
	 "Piatek"
#elif L==9
	 "Sexta-feira"
#endif
	 ,
#if   L==1
	 "dissabte"
#elif L==2
	 "Samstag"
#elif L==3
	 "Saturday"
#elif L==4
	 "s&aacute;bado"
#elif L==5
	 "Samedi"
#elif L==6
	 "arapok&otilde;i"
#elif L==7
	 "Sabato"
#elif L==8
	 "Sobota"
#elif L==9
	 "Sabado"
#endif
	 ,
#if   L==1
	 "diumenge"
#elif L==2
	 "Sonntag"
#elif L==3
	 "Sunday"
#elif L==4
	 "domingo"
#elif L==5
	 "Dimanche"
#elif L==6
	 "arate&#297;"
#elif L==7
	 "Domenica"
#elif L==8
	 "Niedziela"
#elif L==9
	 "Domingo"
#endif
	};

const char *Txt_Decrease_level_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Disminuir nivel de %s";	// Necessita traduccio
#elif L==2
	"Decrease level of %s";		// Need bersetzung
#elif L==3
	"Decrease level of %s";
#elif L==4
	"Disminuir nivel de %s";
#elif L==5
	"Decrease level of %s";		// Besoin de traduction
#elif L==6
	"Disminuir nivel de %s";	// Okoteve traduccin
#elif L==7
	"Decrementare livello di %s";
#elif L==8
	"Spadek poziomu %s";
#elif L==9
	"Decrease level of %s";		// Necessita de traduo
#endif

const char *Txt_default =
#if   L==1
	"predeterminado";	// Necessita traduccio
#elif L==2
	"Vorgabe";
#elif L==3
	"default";
#elif L==4
	"predeterminado";
#elif L==5
	"d&eacute;faut";
#elif L==6
	"predeterminado";	// Okoteve traduccin
#elif L==7
	"predefinito";
#elif L==8
	"Domyslnie";
#elif L==9
	"padr&atilde;o";
#endif

const char *Txt_Degree =
#if   L==1
	"Titulaci&oacute;";
#elif L==2
	"Studiengang";
#elif L==3
	"Degree";
#elif L==4
	"Titulaci&oacute;n";
#elif L==5
	"&Eacute;tude";
#elif L==6
	"Arandur&atilde;";
#elif L==7
	"Laurea";
#elif L==8
	"Stopien";
#elif L==9
	"Titula&ccedil;&atilde;o";
#endif

const char *Txt_degree =
#if   L==1
	"titulaci&oacute;";
#elif L==2
	"Studiengang";
#elif L==3
	"degree";
#elif L==4
	"titulaci&oacute;n";
#elif L==5
	"&eacute;tude";
#elif L==6
	"arandur&atilde;";
#elif L==7
	"laurea";
#elif L==8
	"stopien";
#elif L==9
	"titula&ccedil;&atilde;o";
#endif

const char *Txt_DEGREE_STATUS[Deg_NUM_STATUS_TXT] =
   {
#if   L==1	// Deg_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==2
	 "Unbekannt"
#elif L==3
	 "Unknown"
#elif L==4
	 "Desconocido"
#elif L==5
	 "Inconnu"
#elif L==6
	 "Desconocido"			// Okoteve traduccin
#elif L==7
	 "Sconosciuto"
#elif L==8
	 "Nieznany"
#elif L==9
	 "Desconhecido"
#endif
	 ,
#if   L==1	// Deg_STATUS_ACTIVE
	 "Activa"
#elif L==2
	 "Aktive"
#elif L==3
	 "Active"
#elif L==4
	 "Activa"
#elif L==5
	 "Actif"
#elif L==6
	 "Activa"	// Okoteve traduccin
#elif L==7
	 "Attiva"
#elif L==8
	 "Aktywne"
#elif L==9
	 "Ativa"
#endif
	 ,
#if   L==1	// Deg_STATUS_PENDING
	 "Pendent"
#elif L==2
	 "Offen"
#elif L==3
	 "Pending"
#elif L==4
	 "Pendiente"
#elif L==5
	 "En attente"
#elif L==6
	 "Pendiente"	// Okoteve traduccin
#elif L==7
	 "In attesa"
#elif L==8
	 "Do czasu"
#elif L==9
	 "Pendente"
#endif
	 ,
#if   L==1	// Deg_STATUS_REMOVED
	 "Eliminada"
#elif L==2
	 "Entfernt"
#elif L==3
	 "Removed"
#elif L==4
	 "Eliminada"
#elif L==5
	 "Supprim&eacute;"
#elif L==6
	 "Eliminada"	// Okoteve traduccin
#elif L==7
	 "Rimossa"
#elif L==8
	 "Usuniete"
#elif L==9
	 "Suprimida"
#endif
	};

const char *Txt_DEGREE_With_courses =
#if   L==1
	"Amb assignatures";
#elif L==2
	"Mit Kursen";
#elif L==3
	"With courses";
#elif L==4
	"Con asignaturas";
#elif L==5
	"Avec des mati&egrave;res";
#elif L==6
	"Con asignaturas";	// Okoteve traduccin
#elif L==7
	"Con corsi";
#elif L==8
	"Z kurs&oacute;w";
#elif L==9
	"Com disciplinas";
#endif

const char *Txt_DEGREE_Without_courses =
#if   L==1
	"Sense assignatures";
#elif L==2
	"Ohne Kursen";
#elif L==3
	"Without courses";
#elif L==4
	"Sin asignaturas";
#elif L==5
	"Sans mati&egrave;res";
#elif L==6
	"Sin asignaturas";	// Okoteve traduccin
#elif L==7
	"Senza corsi";
#elif L==8
	"Bez kurs&oacute;w";
#elif L==9
	"Sem disciplinas";
#endif

const char *Txt_Degree_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Titulaci&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2
	"Studiengang <strong>%s</strong> entfernt.";
#elif L==3
	"Degree <strong>%s</strong> removed.";
#elif L==4
	"Titulaci&oacute;n <strong>%s</strong> eliminada.";
#elif L==5
	"Degree <strong>%s</strong> removed.";			// Besoin de traduction
#elif L==6
	"Titulaci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7
	"Laurea <strong>%s</strong> rimossa.";
#elif L==8
	"Studia <strong>%s</strong> usuniete.";
#elif L==9
	"Degree <strong>%s</strong> removed.";			// Necessita de traduo
#endif

const char *Txt_Degrees =
#if   L==1
	"Titulacions";
#elif L==2
	"Studiengang";
#elif L==3
	"Degrees";
#elif L==4
	"Titulaciones";
#elif L==5
	"&Eacute;tudes";
#elif L==6
	"Arandur&atilde;";
#elif L==7
	"Lauree";
#elif L==8
	"Stopnie";
#elif L==9
	"Titula&ccedil;oes";
#endif

const char *Txt_Degrees_ABBREVIATION =
#if   L==1
	"Titul.";
#elif L==2
	"Studieng.";
#elif L==3
	"Degrees";
#elif L==4
	"Titul.";
#elif L==5
	"&Eacute;tudes";
#elif L==6
	"Titul.";	// Okoteve traduccin
#elif L==7
	"Lauree";
#elif L==8
	"Stopni";
#elif L==9
	"Titul.";
#endif

const char *Txt_degrees =
#if   L==1
	"titulacions";
#elif L==2
	"Studiengang";
#elif L==3
	"degrees";
#elif L==4
	"titulaciones";
#elif L==5
	"&eacute;tudes";
#elif L==6
	"arandur&atilde;";
#elif L==7
	"lauree";
#elif L==8
	"stopnie";
#elif L==9
	"titula&ccedil;oes";
#endif

const char *Txt_Degrees_of_CENTRE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Titulacions de %s";
#elif L==2
	"Studiengang von %s";
#elif L==3
	"Degrees of %s";
#elif L==4
	"Titulaciones de %s";
#elif L==5
	"&Eacute;tudes de %s";
#elif L==6
	"Titulaciones de %s";	// Okoteve traduccin
#elif L==7
	"Lauree di %s";
#elif L==8
	"Stopnie %s";
#elif L==9
	"Titula&ccedil;oes de %s";
#endif

const char *Txt_Degrees_with_pending_courses =
#if   L==1
	"Titulacions amb assignatures pendents d'activar";
#elif L==2
	"Studiengang mit anh&auml;ngig Kursen";
#elif L==3
	"Degrees with pending courses";
#elif L==4
	"Titulaciones con asignaturas pendientes de activar";
#elif L==5
	"&Eacute;tudes avec des mati&egrave;res en attente";
#elif L==6
	"Titulaciones con asignaturas pendientes de activar";	// Okoteve traduccin
#elif L==7
	"Lauree con corsi in sospeso";
#elif L==8
	"Stopnie z kurs&otilde;w oczekuj&aogon;cy";
#elif L==9
	"Titula&ccedil;&atilde;os com disciplinas pendentes";
#endif

const char *Txt_Delete_messages_received =
#if   L==1
	"Eliminar missatges rebuts";
#elif L==2
	"Entfernen empfangene Nachrichten";
#elif L==3
	"Delete messages received";
#elif L==4
	"Eliminar mensajes recibidos";
#elif L==5
	"&Eacute;liminer messages re&ccedil;us";
#elif L==6
	"Eliminar mensajes recibidos";	// Okoteve traduccin
#elif L==7
	"Rimuovere messaggi ricevuti";
#elif L==8
	"Usu&nacute; wiadomo&sacute;ci otrzymane";
#elif L==9
	"Eliminar mensagens recebidas";
#endif

const char *Txt_Delete_messages_sent =
#if   L==1
	"Eliminar mensajes enviados";	// Necessita traduccio
#elif L==2
	"Entfernen gesendete Nachrichten";
#elif L==3
	"Delete messages sent";
#elif L==4
	"Eliminar mensajes enviados";
#elif L==5
	"&Eacute;liminer messages envoy&eacute;s";
#elif L==6
	"Eliminar mensajes enviados";	// Okoteve traduccin
#elif L==7
	"Rimuovere messaggi inviati";
#elif L==8
	"Usu&nacute; wiadomo&sacute;ci wyslanych";
#elif L==9
	"Eliminar mensagens enviadas";
#endif

const char *Txt_Department =
#if   L==1
	"Departament";
#elif L==2
	"Abteilung";
#elif L==3
	"Department";
#elif L==4
	"Departamento";
#elif L==5
	"D&eacute;partement";
#elif L==6
	"Peh&#7869;";
#elif L==7
	"Dipartimento";
#elif L==8
	"Wydzia&lstrok;";
#elif L==9
	"Departamento";
#endif

const char *Txt_Department_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Departamento <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2
	"Abteilung <strong>%s</strong> entfernt.";
#elif L==3
	"Department <strong>%s</strong> removed.";
#elif L==4
	"Departamento <strong>%s</strong> eliminado.";
#elif L==5
	"D&eacute;partement <strong>%s</strong> supprim&eacute;.";
#elif L==6
	"Departamento <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Dipartamento <strong>%s</strong> rimosso.";
#elif L==8
	"Zaklad <strong>%s</strong> usuniete.";
#elif L==9
	"Department <strong>%s</strong> removed.";	// Necessita de traduo
#endif

const char *Txt_Department_unspecified =
#if   L==1
	"Departament sense especificar";
#elif L==2
	"Abteilung nicht angegeben";
#elif L==3
	"Department unspecified";
#elif L==4
	"Departamento sin especificar";
#elif L==5
	"D&eacute;partement non sp&eacute;cifi&eacute;";
#elif L==6
	"Departamento sin especificar";	// Okoteve traduccin
#elif L==7
	"Dipartimento non specificato";
#elif L==8
	"Departament nieokreslone";
#elif L==9
	"Departamento indeterminado";
#endif

const char *Txt_Departments =
#if   L==1
	"Departaments";
#elif L==2
	"Abteilungen";
#elif L==3
	"Departments";
#elif L==4
	"Departamentos";
#elif L==5
	"D&eacute;partements";
#elif L==6
	"Peh&#7869;";
#elif L==7
	"Dipartimenti";
#elif L==8
	"Wydzia&lstrok;y";
#elif L==9
	"Departamentos";
#endif

const char *Txt_Departments_ABBREVIATION =
#if   L==1
	"Depar.";
#elif L==2
	"Abteilung";
#elif L==3
	"Depar";
#elif L==4
	"Depar.";
#elif L==5
	"Depar.";
#elif L==6
	"Depar.";	// Okoteve traduccin
#elif L==7
	"Dipar.";
#elif L==8
	"Wydzialy";
#elif L==9
	"Depar.";
#endif

const char *Txt_DEPARTMENTS_HELP_ORDER[2] =
   {
#if   L==1
	 "Ordenar por departamento"	// Necessita traduccio
#elif L==2
	 "Sortieren nach Abteilung"
#elif L==3
	 "Sort by department"
#elif L==4
	 "Ordenar por departamento"
#elif L==5
	 "Trier par d&eacute;partement"
#elif L==6
	 "Ordenar por departamento"	// Okoteve traduccin
#elif L==7
	 "Ordina per dipartimento"
#elif L==8
	 "Sortuj wedlug dzial&oacute;w"
#elif L==9
	 "Classificar por departamento"
#endif
	 ,
#if   L==1
	 "Ordenar por n&ordm; de profesores"	// Necessita traduccio
#elif L==2
	 "Sortieren nach Anzahl der Lehrkr&auml;fte"
#elif L==3
	 "Sort by no. of teachers"
#elif L==4
	 "Ordenar por n&ordm; de profesores"
#elif L==5
	 "Trier par nombre d'enseignants"
#elif L==6
	 "Ordenar por n&ordm; de profesores"	// Okoteve traduccin
#elif L==7
	 "Ordina per numero di professori"
#elif L==8
	 "Sortuj wedlug liczby nauczycieli"
#elif L==9
	 "Classificar por n&ordm; de professores"
#endif
	};

const char *Txt_DEPARTMENTS_ORDER[2] =
   {
#if   L==1
	 "Departament"
#elif L==2
	 "Abteilung"
#elif L==3
	 "Department"
#elif L==4
	 "Departamento"
#elif L==5
	 "D&eacute;partement"
#elif L==6
	 "Peh&#7869;"
#elif L==7
	 "Dipartimento"
#elif L==8
	 "Wydzia&lstrok;"
#elif L==9
	 "Departamento"
#endif
	 ,
#if   L==1
	 "Professors"
#elif L==2
	 "Lehrkr&auml;fte"
#elif L==3
	 "Teachers"
#elif L==4
	 "Profesores"
#elif L==5
	 "Enseignants"
#elif L==6
	 "Profesores"	// Okoteve traduccin
#elif L==7
	 "Professori"
#elif L==8
	 "Nauczycieli"
#elif L==9
	 "Professores"
#endif
	};

const char *Txt_Description =
#if   L==1
	"Descripci&oacute;n";	// Necessita traduccio
#elif L==2
	"Beschreibung";
#elif L==3
	"Description";
#elif L==4
	"Descripci&oacute;n";
#elif L==5
	"Description";
#elif L==6
	"Descripci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Descrizione";
#elif L==8
	"Opis";
#elif L==9
	"Descri&ccedil;&atilde;o";
#endif

const char *Txt_Details =
#if   L==1
	"Detalls";
#elif L==2
	"Einzelheiten";
#elif L==3
	"Details";
#elif L==4
	"Detalles";
#elif L==5
	"D&eacute;tails";
#elif L==6
	"Detalles";	// Okoteve traduccin
#elif L==7
	"Dettagli";
#elif L==8
	"Szczeg&oacute;&lstrok;y";
#elif L==9
	"Detalhes";
#endif

const char *Txt_Disclaimer_the_files_hosted_here_ =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"Aviso legal: los archivos alojados aqu&iacute;, salvo que se indique lo contrario,"
	" est&aacute;n sujetos a derechos de propiedad intelectual"
	" y su titularidad corresponde a los usuarios que los han subido."
	" %s no se responsabiliza de la informaci&oacute;n contenida en dichos archivos."
	" Si usted cree conveniente retirar cualquier archivo cuyo contenido le pertenezca"
	" o que infrinja la ley, puede comunicarlo a %s.";	// Necessita traduccio
#elif L==2
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Need bersetzung
#elif L==3
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";
#elif L==4
	"Aviso legal: los archivos alojados aqu&iacute;, salvo que se indique lo contrario,"
	" est&aacute;n sujetos a derechos de propiedad intelectual"
	" y su titularidad corresponde a los usuarios que los han subido."
	" %s no se responsabiliza de la informaci&oacute;n contenida en dichos archivos."
	" Si usted cree conveniente retirar cualquier archivo cuyo contenido le pertenezca"
	" o que infrinja la ley, puede comunicarlo a %s.";
#elif L==5
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Besoin de traduction
#elif L==6
	"Aviso legal: los archivos alojados aqu&iacute;, salvo que se indique lo contrario,"
	" est&aacute;n sujetos a derechos de propiedad intelectual"
	" y su titularidad corresponde a los usuarios que los han subido."
	" %s no se responsabiliza de la informaci&oacute;n contenida en dichos archivos."
	" Si usted cree conveniente retirar cualquier archivo cuyo contenido le pertenezca"
	" o que infrinja la ley, puede comunicarlo a %s.";	// Okoteve traduccin
#elif L==7
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Bisogno di traduzione
#elif L==8
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Potrzebujesz tlumaczenie
#elif L==9
	"Disclaimer: the files hosted here, unless otherwise noted,"
	" are subject to intellectual property rights"
	" and their ownership belongs to the users who have uploaded them."
	" %s is not responsible for the information contained in those files."
	" If you think convenient to remove any file whose contents belongs to you"
	" or that violates the law, you can contact %s.";	// Necessita de traduo
#endif

const char *Txt_Display_photos =
#if   L==1
	"Incloure fotos";
#elif L==2
	"Bilder anzeigen";
#elif L==3
	"Display photos";
#elif L==4
	"Incluir fotos";
#elif L==5
	"Inclure des photos";
#elif L==6
	"Incluir fotos";	// Okoteve traduccin
#elif L==7
	"Mostra foto";
#elif L==8
	"Wyswietlanie zdjec";
#elif L==9
	"Incluir fotos";
#endif

const char *Txt_distributed_by =
#if   L==1
	"distribu&iuml;t per";
#elif L==2
	"vertrieben von";
#elif L==3
	"distributed by";
#elif L==4
	"distribuido por";
#elif L==5
	"distribu&eacute; par";
#elif L==6
	"distribuido por";	// Okoteve traduccin
#elif L==7
	"distribuito da";
#elif L==8
	"wyplacanych przez";
#elif L==9
	"distribu&iacute;do por";
#endif

const char *Txt_Do_not_reload_this_page_because_the_message_will_be_sent_again_ =
#if   L==1
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";	// Necessita traduccio
#elif L==2
	"Do not reload this page because the message would be sent again and would arrive repeated.";			// Need bersetzung
#elif L==3
	"Do not reload this page because the message would be sent again and would arrive repeated.";
#elif L==4
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";
#elif L==5
	"Do not reload this page because the message would be sent again and would arrive repeated.";			// Besoin de traduction
#elif L==6
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";	// Okoteve traduccin
#elif L==7
	"Non ricaricare questa pagina perch&egrave; il messaggio sar&agrave; nuovamente inviato.";
#elif L==8
	"Nie przeladuj strone, poniewaz wiadomosc zostanie wyslana ponownie przyjedzie powtarzane.";
#elif L==9
	"Do not reload this page because the message would be sent again and would arrive repeated.";			// Necessita de traduo
#endif

const char *Txt_Do_not_reload_this_page_because_the_post_will_be_sent_again_ =
#if   L==1
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";	// Necessita traduccio
#elif L==2
	"Do not reload this page because the post would be sent again and would arrive repeated.";			// Need bersetzung
#elif L==3
	"Do not reload this page because the post would be sent again and would arrive repeated.";
#elif L==4
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";
#elif L==5
	"Do not reload this page because the post would be sent again and would arrive repeated.";			// Besoin de traduction
#elif L==6
	"No recargue esta p&aacute;gina, pues el mensaje se enviar&iacute;a de nuevo y llegar&iacute;a repetido.";	// Okoteve traduccin
#elif L==7
	"Non ricaricare questa pagina perch&egrave; il post sar&agrave; nuovamente inviato.";
#elif L==8
	"Nie przeladuj strone, poniewaz post zostanie wyslany ponownie przyjedzie powtarzane.";
#elif L==9
	"Do not reload this page because the post would be sent again and would arrive repeated.";			// Necessita de traduo
#endif

const char *Txt_Do_not_show_again =
#if   L==1
	"No mostrar de nou";
#elif L==2
	"Nicht mehr anzeigen";
#elif L==3
	"Do not show again";
#elif L==4
	"No mostrar de nuevo";
#elif L==5
	"Ne plus afficher";
#elif L==6
	"No mostrar de nuevo";	// Okoteve traduccin
#elif L==7
	"Non mostrare pi&ugrave;";
#elif L==8
	"Nie pokazuj ponownie";
#elif L==9
	"N&atilde;o mostrar novamente";
#endif

const char *Txt_Do_you_really_want_to_be_removed_as_an_administrator_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea dejar de ser administrador/a"
	" de <strong>%s</strong>?";		// Necessita traduccio
#elif L==2
	"Do you really want to be removed as an administrator"
	" of <strong>%s</strong>?";		// Need bersetzung
#elif L==3
	"Do you really want to be removed as an administrator"
	" of <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea dejar de ser administrador/a"
	" de <strong>%s</strong>?";
#elif L==5
	"Do you really want to be removed as an administrator"
	" of <strong>%s</strong>?";		// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea dejar de ser administrador/a"
	" de <strong>%s</strong>?";		// Okoteve traduccin
#elif L==7
	"Vuole realmente essere rimosso come amministratore"
	" de <strong>%s</strong>?";
#elif L==8
	"Czy naprawde chcesz byc usuniety jako administrator"
	" <strong>%s</strong>?";
#elif L==9
	"Do you really want to be removed as an administrator"
	" <strong>%s</strong>?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_be_removed_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea ser eliminado de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Necessita traduccio
#elif L==2
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrollment in groups, etc.)?";			// Need bersetzung
#elif L==3
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrollment in groups, etc.)?";
#elif L==4
	"&iquest;Realmente desea ser eliminado de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";
#elif L==5
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrollment in groups, etc.)?";			// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea ser eliminado de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente essere rimosso dal corso <strong>%s</strong>"
	" (rimuovendo i tuoi lavori, registrazione in gruppi, etc.)?";
#elif L==8
	"Czy na pewno chcesz zostac usuniete z kursu <strong>%s</strong>"
	" (usuwanie swoja prace, rejestracja w grupach, itp.)?";
#elif L==9
	"Do you really want to be removed from the course <strong>%s</strong>"
	" (removing your works, enrollment in groups, etc.)?";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_completely_eliminate_the_following_user =
#if   L==1
	"&iquest;Realmente desea eliminar completamente la cuenta del siguiente usuario?";	// Necessita traduccio
#elif L==2
	"Do you really want to completely eliminate the following user?";	// Need bersetzung
#elif L==3
	"Do you really want to completely eliminate the following user?";
#elif L==4
	"&iquest;Realmente desea eliminar completamente la cuenta del siguiente usuario?";
#elif L==5
	"Do you really want to completely eliminate the following user?";	// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar completamente la cuenta del siguiente usuario?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere totalmente il seguente utente?";
#elif L==8
	"Czy na pewno chcesz, aby calkowicie wyeliminowac nastepujacych uzytkownik&oacute;w?";
#elif L==9
	"Do you really want to completely eliminate the following user?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_completely_eliminate_your_user_account =
#if   L==1
	"&iquest;Realmente desea eliminar completamente su cuenta de usuario/a?";	// Necessita traduccio
#elif L==2
	"Do you really want to completely eliminate your user account?";	// Need bersetzung
#elif L==3
	"Do you really want to completely eliminate your user account?";
#elif L==4
	"&iquest;Realmente desea eliminar completamente su cuenta de usuario/a?";
#elif L==5
	"Do you really want to completely eliminate your user account?";	// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar completamente su cuenta de usuario/a?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere totalmente il tuo account?";
#elif L==8
	"Czy na pewno chcesz, aby calkowicie wyeliminowac konta uzytkownika?";
#elif L==9
	"Do you really want to completely eliminate your user account?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_create_a_new_user_account_with_the_following_data =
#if   L==1
	"Realment voleu crear un nou compte d'usuari"
	" amb les seg&uuml;ents dades?";
#elif L==2
	"Wollen Sie wirklich, um ein neues Benutzerkonto"
	" mit den folgenden Daten zu erstellen?";
#elif L==3
	"Do you really want to create a new user account"
	" with the following data?";
#elif L==4
	"&iquest;Realmente desea crear una nueva cuenta de usuario/a"
	" con los siguientes datos?";
#elif L==5
	"Voulez-vous vraiment cr&eacute;er un nouveau compte d'utilisateur"
	" avec les donn&eacute;es suivantes?";
#elif L==6
	"&iquest;Realmente desea crear una nueva cuenta de usuario/a"
	" con los siguientes datos?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente creare un nuovo account utente"
	" con i seguenti dati?";
#elif L==8
	"Czy na pewno chcesz, aby utworzy&cacute; nowe konto u&zdot;ytkownika"
	" z nast&eogon;puj&aogon;cymi danymi?";
#elif L==9
	"Voc&ecirc; realmente deseja criar uma nova conta de usu&aacute;rio"
	" com os seguintes dados?";
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_received_from_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s?";	// Necessita traduccio
#elif L==2
	"Do you really want to delete all messages received from %s from %s?";			// Need bersetzung
#elif L==3
	"Do you really want to delete all messages received from %s from %s?";
#elif L==4
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s?";
#elif L==5
	"Do you really want to delete all messages received from %s from %s?";			// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere tutti i messaggi ricevuti da %s da %s?";
#elif L==8
	"Czy na pewno chcesz usunac wszystkie wiadomosci otrzymane od %s z %s?";
#elif L==9
	"Do you really want to delete all messages received from %s from %s?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_received_from_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s relacionados con <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Do you really want to delete all messages received from %s from %s related to <strong>%s</strong>?";			// Need bersetzung
#elif L==3
	"Do you really want to delete all messages received from %s from %s related to <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s relacionados con <strong>%s</strong>?";
#elif L==5
	"Do you really want to delete all messages received from %s from %s related to <strong>%s</strong>?";			// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar todos los mensajes recibidos de %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere tutti i messaggi ricevuti da %s da %s relativi a <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac wszystkie wiadomosci otrzymane od %s z %s podobne do <strong>%s</strong>?";
#elif L==9
	"Do you really want to delete all messages received from %s from %s related to <strong>%s</strong>?";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_sent_to_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s?";	// Necessita traduccio
#elif L==2
	"Do you really want to delete all messages sent to %s from %s?";		// Need bersetzung
#elif L==3
	"Do you really want to delete all messages sent to %s from %s?";
#elif L==4
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s?";
#elif L==5
	"Do you really want to delete all messages sent to %s from %s?";		// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere tutti i messaggi inviati a %s da %s?";
#elif L==8
	"Czy na pewno chcesz usunac wszystkie wiadomosci wyslane do %s z %s?";
#elif L==9
	"Do you really want to delete all messages sent to %s from %s?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_delete_all_messages_sent_to_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s relacionados con <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";				// Need bersetzung
#elif L==3
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s relacionados con <strong>%s</strong>?";
#elif L==5
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";				// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar todos los mensajes enviados a %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere tutti i messaggi inviati a %s da %s relativi a<strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac wszystkie wiadomosci wyslane do %s z %s podobne do <strong>%s</strong>?";
#elif L==9
	"Do you really want to delete all messages sent to %s from %s related to <strong>%s</strong>?";				// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_delete_the_unread_messages_received_from_USER_X_from_COURSE_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos de %s desde %s?";	// Necessita traduccio
#elif L==2
	"Do you really want to delete the unread messages received from %s from %s?";			// Need bersetzung
#elif L==3
	"Do you really want to delete the unread messages received from %s from %s?";
#elif L==4
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos de %s desde %s?";
#elif L==5
	"Do you really want to delete the unread messages received from %s from %s?";			// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos de %s desde %s?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere tutti i messaggi non letti ricevuti da %s da %s?";
#elif L==8
	"Czy na pewno chcesz usunac nieprzeczytanych wiadomosci otrzymane od %s z %s?";
#elif L==9
	"Do you really want to delete the unread messages received from %s from %s?";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_delete_the_unread_messages_received_from_USER_X_from_COURSE_Y_related_to_CONTENT_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos"
	" de %s desde %s relacionados con <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Do you really want to delete the unread messages received"
	" from %s from %s related to <strong>%s</strong>?";		// Need bersetzung
#elif L==3
	"Do you really want to delete the unread messages received"
	" from %s from %s related to <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos"
	" de %s desde %s relacionados con <strong>%s</strong>?";
#elif L==5
	"Do you really want to delete the unread messages received"
	" from %s from %s related to <strong>%s</strong>?";		// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar los mensajes no le&iacute;dos recibidos"
	" de %s desde %s relacionados con <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere tutti i messaggi non letti ricevuti"
	" da %s da %s relativi a <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac nieprzeczytanych wiadomosci otrzymane"
	" od %s z %s podobne do <strong>%s</strong>?";
#elif L==9
	"Do you really want to delete the unread messages received"
	" from %s from %s related to <strong>%s</strong>?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_register_the_following_user_as_an_administrator_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea inscribir el siguiente usuario"
	" como administrador de <strong>%s</strong>?";		// Necessita traduccio
#elif L==2
	"Do you really want to register the following user"
	" as an administrator of <strong>%s</strong>?";		// Need bersetzung
#elif L==3
	"Do you really want to register the following user"
	" as an administrator of <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea inscribir el siguiente usuario"
	" como administrador de <strong>%s</strong>?";
#elif L==5
	"Do you really want to register the following user"
	" as an administrator of <strong>%s</strong>?";		// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea inscribir el siguiente usuario"
	" como administrador de <strong>%s</strong>?";		// Okoteve traduccin
#elif L==7
	"Vuoi realmente registrare il seguente utente"
	" come amministratore de <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz sie zarejestrowac nastepujace u&zdot;ytkownika"
	" jako administratora <strong>%s</strong>?";
#elif L==9
	"Do you really want to register the following user"
	" as an administrator of <strong>%s</strong>?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_reject_the_enrollment_request_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==1
	"&iexcl;Realmente desea rechazar la solicitud de inscripci&oacute;n"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>?";		// Necessita traduccio
#elif L==2
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Need bersetzung
#elif L==3
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";
#elif L==4
	"&iexcl;Realmente desea rechazar la solicitud de inscripci&oacute;n"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>?";
#elif L==5
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Besoin de traduction
#elif L==6
	"&iexcl;Realmente desea rechazar la solicitud de inscripci&oacute;n"
	" de <strong>%s</strong> como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>?";		// Okoteve traduccin
#elif L==7
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Bisogno di traduzione
#elif L==8
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Potrzebujesz tlumaczenie
#elif L==9
	"Do you really want to reject the enrollment request"
	" from <strong>%s</strong> as <strong>%s</strong>"
	" in the course <strong>%s</strong>?";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_X_students_from_the_course_Y_ =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar los %u estudiantes de la asignatura <strong>%s</strong>?<br />"
	"Aunque no se eliminar&aacute;n los datos de los estudiantes en otras asignaturas,"
	" ya no aparecer&aacute;n inscritos en esta asignatura,"
	" y adem&aacute;s <strong>se eliminar&aacute;n todos los trabajos enviados por ellos en esta asignatura,"
	" sus fichas personalizadas para esta asignatura,"
	" y su inscripci&oacute;n en grupos de esta asignatura.</strong>.";		// Necessita traduccio
#elif L==2
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enrolled in this course,"
	" and also <strong>all the works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrollment in groups of this course will be removed.</strong>.";	// Need bersetzung
#elif L==3
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enrolled in this course,"
	" and also <strong>all the works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrollment in groups of this course will be removed.</strong>.";
#elif L==4
	"&iquest;Realmente desea eliminar los %u estudiantes de la asignatura <strong>%s</strong>?<br />"
	"Aunque no se eliminar&aacute;n los datos de los estudiantes en otras asignaturas,"
	" ya no aparecer&aacute;n inscritos en esta asignatura,"
	" y adem&aacute;s <strong>se eliminar&aacute;n todos los trabajos enviados por ellos en esta asignatura,"
	" sus fichas personalizadas para esta asignatura,"
	" y su inscripci&oacute;n en grupos de esta asignatura.</strong>.";
#elif L==5
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enrolled in this course,"
	" and also <strong>all the works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrollment in groups of this course will be removed.</strong>.";	// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar los %u estudiantes de la asignatura <strong>%s</strong>?<br />"
	"Aunque no se eliminar&aacute;n los datos de los estudiantes en otras asignaturas,"
	" ya no aparecer&aacute;n inscritos en esta asignatura,"
	" y adem&aacute;s <strong>se eliminar&aacute;n todos los trabajos enviados por ellos en esta asignatura,"
	" sus fichas personalizadas para esta asignatura,"
	" y su inscripci&oacute;n en grupos de esta asignatura.</strong>.";		// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere gli %u studenti dal corso <strong>%s</strong>?<br />"
	"Sebbene i dati degli studenti non saranno rimossi in altri corsi,"
	" essi non saranno pi&ugrave; registrati in questo corso,"
	" e cos <strong>tutti i lavori presentati da loro in questo corso,"
	" le loro schede per questo corso,"
	" e la loro registrazione nei gruppi di questo corso saranno rimossi.</strong>.";
#elif L==8
	"Czy na pewno chcesz usunac %u student&oacute;w z kursu <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" nie beda juz zarejestrowane w tym kurs"
	" a takze <strong>wszystkie prace zlozone przez nich w tym kursie,"
	" swoich kart zapisu na ten kurs,"
	" i ich rejestracja w grupach zajec zostana usuniete.</strong>.";
#elif L==9
	"Do you really want to remove the %u students from the course <strong>%s</strong>?<br />"
	"Although students' data will not be removed in other courses,"
	" they will no longer enrolled in this course,"
	" and also <strong>all the works submitted by them in this course,"
	" their record cards for this course,"
	" and their enrollment in groups of this course will be removed.</strong>.";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_assignment_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar la actividad <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Wollen Sie die Aufgabe <strong>%s</strong> wirklich entfernen?";
#elif L==3
	"Do you really want to remove the assignment <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar la actividad <strong>%s</strong>?";
#elif L==5
	"Voulez-vous vraiment supprimer l'activit&eacute; <strong>%s</strong>?";
#elif L==6
	"&iquest;Realmente desea eliminar la actividad <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere l'attivit&agrave; <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac przypisanie <strong>%s</strong>?";
#elif L==9
	"Voc&ecirc; realmente deseja remover a atividade <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_entire_thread =
#if   L==1
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n?";	// Necessita traduccio
#elif L==2
	"Wollen Sie die gesamten Thread wirklich entfernen?";
#elif L==3
	"Do you really want to remove the entire thread?";
#elif L==4
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n?";
#elif L==5
	"Voulez-vous vraiment supprimer tout le fil?";
#elif L==6
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere l'intera discussione?";
#elif L==8
	"Czy na pewno chcesz usunac caly watek?";
#elif L==9
	"Voc&ecirc; realmente deseja remover todo o thread?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_entire_thread_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Wollen Sie die gesamten Thread <strong>%s</strong> wirklich entfernen?";
#elif L==3
	"Do you really want to remove the entire thread <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n <strong>%s</strong>?";
#elif L==5
	"Voulez-vous vraiment supprimer tout le fil <strong>%s</strong>?";
#elif L==6
	"&iquest;Realmente desea eliminar toda la discusi&oacute;n <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere l'intera discussione <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac caly watek <strong>%s</strong>?";
#elif L==9
	"Voc&ecirc; realmente deseja remover todo o thread <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_event_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar el evento <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Wollen Sie der Ereignis <strong>%s</strong> wirklich entfernen?";
#elif L==3
	"Do you really want to remove the event <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar el evento <strong>%s</strong>?";
#elif L==5
	"Voulez-vous vraiment supprimer l'&eacute;v&eacute;nement <strong>%s</strong>?";
#elif L==6
	"&iquest;Realmente desea eliminar el evento <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere l'evento <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac wydarzenie <strong>%s</strong>?";
#elif L==9
	"Voc&ecirc; realmente deseja remover o evento <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_field_X_from_the_records_of_X =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar el campo <strong>%s</strong> de las fichas de <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Do you really want to remove the field <strong>%s</strong> from the records of <strong>%s</strong>?";	// Need bersetzung
#elif L==3
	"Do you really want to remove the field <strong>%s</strong> from the records of <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar el campo <strong>%s</strong> de las fichas de <strong>%s</strong>?";
#elif L==5
	"Do you really want to remove the field <strong>%s</strong> from the records of <strong>%s</strong>?";	// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar el campo <strong>%s</strong> de las fichas de <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il campo <strong>%s</strong> dalle schede di <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac pole <strong>%s</strong> z ewidencji <strong>%s</strong>?";
#elif L==9
	"Do you really want to remove the field <strong>%s</strong> from the records of <strong>%s</strong>?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Wollen Sie <strong>%s</strong> wirklich entfernen?";
#elif L==3
	"Do you really want to remove <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar <strong>%s</strong>?";
#elif L==5
	"Voulez-vous vraiment supprimer <strong>%s</strong>?";
#elif L==6
	"&iquest;Realmente desea eliminar <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac <strong>%s</strong>?";
#elif L==9
	"Voc&ecirc; realmente deseja remover <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar"
	" la carpeta <strong>%s</strong> y todo su contenido?";		// Necessita traduccio
#elif L==2
	"Wollen Sie wirklich"
	" das Verzeichnis <strong>%s</strong> und alle Inhalte entfernen?";
#elif L==3
	"Do you really want to remove"
	" the folder <strong>%s</strong> and all its contents?";
#elif L==4
	"&iquest;Realmente desea eliminar"
	" la carpeta <strong>%s</strong> y todo su contenido?";
#elif L==5
	"Voulez-vous vraiment supprimer"
	" le r&eacute;pertoire <strong>%s</strong> et l'ensemble de son contenu?";
#elif L==6
	"&iquest;Realmente desea eliminar"
	" la carpeta <strong>%s</strong> y todo su contenido?";		// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere"
	" la cartella <strong>%s</strong> e tutto il suo contenuto?";
#elif L==8
	"Czy na pewno chcesz usunac"
	" folder <strong>%s</strong> i wszystkie jego skladniki?";
#elif L==9
	"Voc&ecirc; realmente deseja remover"
	" o diret&oacute;rio <strong>%s</strong> e todo o seu conte&uacute;do?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_announcement_of_exam =
#if   L==1
	"De veres voleu eliminar la seg&uuml;ent convocat&ograve;ria d'examen?";
#elif L==2
	"Wollen Sie wirklich, um die folgende Aufruf f&uuml;r Pr&uuml;fung?";
#elif L==3
	"Do you really want to remove the following announcement of exam?";
#elif L==4
	"&iquest;Realmente desea eliminar la siguiente convocatoria de examen?";
#elif L==5
	"Voulez-vous vraiment supprimer la convocation &agrave; un examen suivante?";
#elif L==6
	"&iquest;Realmente desea eliminar la siguiente convocatoria de examen?";		// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il seguente appello d'esame?";
#elif L==8
	"Czy na pewno chcesz usun&aogon;&cacute; nast&eogon;puj&aogon;c&aogon; zapowied&zacute; egzaminu?";
#elif L==9
	"Voc&ecirc; realmente deseja remover a seguinte chamada para exame?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_notice =
#if   L==1
	"De veres voleu eliminar el seg&uuml;ent av&iacute;s?";
#elif L==2
	"Wollen Sie wirklich, um die folgende Ank&uuml;ndigung zu entfernen?";
#elif L==3
	"Do you really want to remove the following notice?";
#elif L==4
	"&iquest;Realmente desea eliminar el siguiente aviso?";
#elif L==5
	"Voulez-vous vraiment supprimer l'avis suivant?";
#elif L==6
	"&iquest;Realmente desea eliminar el siguiente aviso?";		// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il seguente avviso?";
#elif L==8
	"Czy na pewno chcesz usunac nastepujace obwieszczenie?";
#elif L==9
	"Voc&ecirc; realmente deseja remover o seguinte aviso?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_comment =
#if   L==1
	"De veres voleu eliminar el seg&uuml;ent comentari?";
#elif L==2
	"Wollen Sie wirklich, um die folgende Kommentar zu entfernen?";
#elif L==3
	"Do you really want to remove the following comment?";
#elif L==4
	"&iquest;Realmente desea eliminar el siguiente comentario?";
#elif L==5
	"Voulez-vous vraiment supprimer le commentaire suivant?";
#elif L==6
	"&iquest;Realmente desea eliminar el siguiente comentario?";		// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il seguente commento?";
#elif L==8
	"Czy na pewno chcesz usunac nastepujace komentarz?";
#elif L==9
	"Voc&ecirc; realmente deseja remover o seguinte coment&aacute;rio?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_post =
#if   L==1
	"De veres voleu eliminar el seg&uuml;ent missatge?";
#elif L==2
	"Wollen Sie wirklich, um die folgende Beitrag zu entfernen?";
#elif L==3
	"Do you really want to remove the following post?";
#elif L==4
	"&iquest;Realmente desea eliminar el siguiente mensaje?";
#elif L==5
	"Voulez-vous vraiment supprimer le message suivant?";
#elif L==6
	"&iquest;Realmente desea eliminar el siguiente mensaje?";		// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il seguente post?";
#elif L==8
	"Czy na pewno chcesz usunac nastepujace post?";
#elif L==9
	"Voc&ecirc; realmente deseja remover o seguinte post?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_as_an_administrator_of_the_centre_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador del centro <strong>%s</strong>?";		// Necessita traduccio
#elif L==2
	"Do you really want to remove the following user"
	" as an administrator of the centre <strong>%s</strong>?";	// Need bersetzung
#elif L==3
	"Do you really want to remove the following user"
	" as an administrator of the centre <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador del centro <strong>%s</strong>?";
#elif L==5
	"Do you really want to remove the following user"
	" as an administrator of the centre <strong>%s</strong>?";	// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador del centro <strong>%s</strong>?";		// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il seguente utente"
	" come amministratore del centro <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac nastepujace u&zdot;ytkownika"
	" jako administratora centrum <strong>%s</strong>?";
#elif L==9
	"Do you really want to remove the following user"
	" as an administrator of the centre <strong>%s</strong>?";	// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador de la titulaci&oacute;n <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Do you really want to remove the following user"
	" as an administrator of the degree <strong>%s</strong>?";		// Need bersetzung
#elif L==3
	"Do you really want to remove the following user"
	" as an administrator of the degree <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador de la titulaci&oacute;n <strong>%s</strong>?";
#elif L==5
	"Do you really want to remove the following user"
	" as an administrator of the degree <strong>%s</strong>?";		// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador de la titulaci&oacute;n <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il seguente utente"
	" come amministratore della laurea <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac nastepujace u&zdot;ytkownika"
	" jako administratora stopnia <strong>%s</strong>?";
#elif L==9
	"Do you really want to remove the following user"
	" as an administrator of the degree <strong>%s</strong>?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_as_an_administrator_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador de <strong>%s</strong>?";		// Necessita traduccio
#elif L==2
	"Do you really want to remove the following user"
	" as an administrator of <strong>%s</strong>?";		// Need bersetzung
#elif L==3
	"Do you really want to remove the following user"
	" as an administrator of <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador de <strong>%s</strong>?";
#elif L==5
	"Do you really want to remove the following user"
	" as an administrator of <strong>%s</strong>?";		// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar el siguiente usuario"
	" como administrador de <strong>%s</strong>?";		// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il seguente utente"
	" come amministratore de <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac nastepujace u&zdot;ytkownika"
	" jako administratora <strong>%s</strong>?";
#elif L==9
	"Do you really want to remove the following user"
	" as an administrator of <strong>%s</strong>?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_following_user_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar el siguiente usuario de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Necessita traduccio
#elif L==2
	"Do you really want to remove the following user from the course <strong>%s</strong>"
	" (removing her/his works, enrollment in groups, etc.)?";		// Need bersetzung
#elif L==3
	"Do you really want to remove the following user from the course <strong>%s</strong>"
	" (removing her/his works, enrollment in groups, etc.)?";
#elif L==4
	"&iquest;Realmente desea eliminar el siguiente usuario de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";
#elif L==5
	"Do you really want to remove the following user from the course <strong>%s</strong>"
	" (removing her/his works, enrollment in groups, etc.)?";		// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar el siguiente usuario de la asignatura <strong>%s</strong>"
	" (eliminando sus trabajos, adscripci&oacute;n a grupos, etc.)?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il seguente utente dal corso <strong>%s</strong>"
	" (rimuovendo i suoi lavori, registrazione in gruppi, etc.)?";
#elif L==8
	"Do you really want to remove the following user z kursu <strong>%s</strong>"
	" (removing her/his works, enrollment in groups, etc.)?";
#elif L==9
	"Do you really want to remove the following user from the course <strong>%s</strong>"
	" (removing her/his works, enrollment in groups, etc.)?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_group_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Do you really want to remove the group <strong>%s</strong>?";		// Need bersetzung
#elif L==3
	"Do you really want to remove the group <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?";
#elif L==5
	"Do you really want to remove the group <strong>%s</strong>?";		// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il gruppo <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac grupe <strong>%s</strong>?";
#elif L==9
	"Do you really want to remove the group <strong>%s</strong>?";		// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_group_X_1_student_ =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, un/a estudiante dejar&aacute; de pertenecer a ese grupo.";	// Necessita traduccio
#elif L==2
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove one student from that group.";			// Need bersetzung
#elif L==3
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove one student from that group.";
#elif L==4
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, un/a estudiante dejar&aacute; de pertenecer a ese grupo.";
#elif L==5
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove one student from that group.";			// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, un/a estudiante dejar&aacute; de pertenecer a ese grupo.";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai uno studente da questo gruppo.";
#elif L==8
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"W ten spos&oacute;b usunie jeden uczen z tej grupy.";
#elif L==9
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove one student from that group.";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_group_X_Y_students_ =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, %u estudiantes dejar&aacute;n de pertenecer a ese grupo.";	// Necessita traduccio
#elif L==2
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove %u students from that group.";			// Need bersetzung
#elif L==3
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove %u students from that group.";
#elif L==4
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, %u estudiantes dejar&aacute;n de pertenecer a ese grupo.";
#elif L==5
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove %u students from that group.";			// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar el grupo <strong>%s</strong>?<br />"
	"Si lo hace, %u estudiantes dejar&aacute;n de pertenecer a ese grupo.";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai %u studenti da questo gruppo.";
#elif L==8
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"W ten spos&oacute;b usunie %u uczni&oacute;w z tej grupy.";
#elif L==9
	"Do you really want to remove the group <strong>%s</strong>?<br />"
	"Doing so will remove %u students from that group.";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_photo_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"De veres voleu eliminar la foto de <strong>%s</strong>?";
#elif L==2
	"Wollen Sie die wirklich das Foto von <strong>%s</strong> zu entfernen?";
#elif L==3
	"Do you really want to remove the photo of <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar la foto de <strong>%s</strong>?";
#elif L==5
	"Voulez-vous vraiment supprimer la photo de <strong>%s</strong>?";
#elif L==6
	"&iquest;Realmente desea eliminar la foto de <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere la foto di <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usun&aogon;&cacute; zdj&eogon;cie <strong>%s</strong>?";
#elif L==9
	"Voc&ecirc; realmente deseja remover la foto de <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_question_X =	// Warning: it is very important to include %lu in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar la pregunta <strong>%lu</strong>?";	// Necessita traduccio
#elif L==2
	"Wollen Sie die Frage <strong>%lu</strong> wirklich entfernen?";
#elif L==3
	"Do you really want to remove the question <strong>%lu</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar la pregunta <strong>%lu</strong>?";
#elif L==5
	"Voulez-vous vraiment supprimer la question <strong>%lu</strong>?";
#elif L==6
	"&iquest;Realmente desea eliminar la pregunta <strong>%lu</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere la domanda <strong>%lu</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac pytanie <strong>%lu</strong>?";
#elif L==9
	"Voc&ecirc; realmente deseja remover a quest&atilde;o <strong>%lu</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar la encuesta <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Wollen Sie die Umfrage <strong>%s</strong> wirklich entfernen?";
#elif L==3
	"Do you really want to remove the survey <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea eliminar la encuesta <strong>%s</strong>?";
#elif L==5
	"Voulez-vous vraiment supprimer le sondage <strong>%s</strong>?";
#elif L==6
	"&iquest;Realmente desea eliminar la encuesta <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il sondaggio <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz usunac badania <strong>%s</strong>?";
#elif L==9
	"Voc&ecirc; realmente deseja remover o inqu&eacute;rito <strong>%s</strong>?";
#endif

const char *Txt_Do_you_really_want_to_remove_the_type_of_group_X_1_group_ =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n un grupo de ese tipo.";	// Necessita traduccio
#elif L==2
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove one group of that type.";			// Need bersetzung
#elif L==3
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove one group of that type.";
#elif L==4
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n un grupo de ese tipo.";
#elif L==5
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove one group of that type.";			// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n un grupo de ese tipo.";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il tipo di gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai anche un gruppo di questo tipo.";
#elif L==8
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Spowoduje to takze usuniecie jednej grupie tego typu.";
#elif L==9
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove one group of that type.";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_the_type_of_group_X_Y_groups_ =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n %u grupos de ese tipo.";	// Necessita traduccio
#elif L==2
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove %u groups of that type.";			// Need bersetzung
#elif L==3
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove %u groups of that type.";
#elif L==4
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n %u grupos de ese tipo.";
#elif L==5
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove %u groups of that type.";			// Besoin de traduction
#elif L==6
	"&iquest;Realmente desea eliminar el tipo de grupo <strong>%s</strong>?<br />"
	"Si lo hace, eliminar&aacute; tambi&eacute;n %u grupos de ese tipo.";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere il tipo di gruppo <strong>%s</strong>?<br />"
	"Facendo cos rimuoverai anche %u gruppi di questo tipo.";
#elif L==8
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Spowoduje to r&oacute;wniez usuniecie grup %u tego typu.";
#elif L==9
	"Do you really want to remove the type of group <strong>%s</strong>?<br />"
	"Doing so will also remove %u groups of that type.";			// Necessita de traduo
#endif

const char *Txt_Do_you_really_want_to_remove_your_photo =
#if   L==1
	"De veres voleu eliminar la seva foto?";
#elif L==2
	"Wollen Sie die Ihr Foto zu entfernen?";
#elif L==3
	"Do you really want to remove your photo?";
#elif L==4
	"&iquest;Realmente desea eliminar su foto?";
#elif L==5
	"Voulez-vous vraiment supprimer votre photo?";
#elif L==6
	"&iquest;Realmente desea eliminar su foto?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente rimuovere la tua foto?";
#elif L==8
	"Czy na pewno chcesz usun&aogon;&cacute; swoje zdj&eogon;cie?";
#elif L==9
	"Voc&ecirc; realmente deseja remover a sua foto?";
#endif

const char *Txt_Do_you_really_want_to_reset_the_survey_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&iquest;Realmente desea poner a cero la encuesta <strong>%s</strong>?";	// Necessita traduccio
#elif L==2
	"Wollen Sie die Umfrage <strong>%s</strong> wirklich reset?";
#elif L==3
	"Do you really want to reset the survey <strong>%s</strong>?";
#elif L==4
	"&iquest;Realmente desea poner a cero la encuesta <strong>%s</strong>?";
#elif L==5
	"Voulez-vous vraiment reset le sondage <strong>%s</strong>?";
#elif L==6
	"&iquest;Realmente desea poner a cero la encuesta <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi realmente resettare il sondaggio <strong>%s</strong>?";
#elif L==8
	"Czy na pewno chcesz zresetowac badania <strong>%s</strong>?";
#elif L==9
	"Voc&ecirc; realmente deseja reiniciar o inqu&eacute;rito <strong>%s</strong>?";
#endif

const char *Txt_Do_you_think_you_are_one_of_these_users =
#if   L==1
	"Creu que vost&egrave; &eacute;s un d'aquests usuaris?";
#elif L==2
	"Denken Sie, Sie einer dieser Benutzer sind?";
#elif L==3
	"Do you think you are one of these users?";
#elif L==4
	"&iquest;Cree que usted es uno de estos usuarios?";
#elif L==5
	"Pensez-vous que vous &ecirc;tes un de ces utilisateurs?";
#elif L==6
	"&iquest;Cree que usted es uno de estos usuarios?";	// Okoteve traduccin
#elif L==7
	"Pensi che sei uno di questi utenti?";
#elif L==8
	"My&sacute;lisz, &zdot;e jeste&sacute; jednym z tych u&zdot;ytkownik&oacute;w?";
#elif L==9
	"Voc&ecirc; acha que voc&ecirc; &eacute; um desses usu&aacute;rios?";
#endif

const char *Txt_Do_you_think_you_are_this_user =
#if   L==1
	"Creu que vost&egrave; &eacute;s aquest usuari?";
#elif L==2
	"Denken Sie, Sie dieser Benutzer?";
#elif L==3
	"Do you think you are this user?";
#elif L==4
	"&iquest;Cree que usted es este usuario/a?";
#elif L==5
	"Pensez-vous que vous &ecirc;tes cet utilisateur?";
#elif L==6
	"&iquest;Cree que usted es este usuario/a?";	// Okoteve traduccin
#elif L==7
	"Pensi che sei questo utente?";
#elif L==8
	"Czy uwa&zdot;asz, &zdot;e ta osoba?";
#elif L==9
	"Voc&ecirc; acha que voc&ecirc; &eacute; deste usu&aacute;rio?";
#endif

const char *Txt_Do_you_want_to_confirm_the_ID_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Voleu confirmar l'ID (DNI/c&egrave;dula) <strong>%s</strong>?";
#elif L==2
	"Wollen Sie die ID <strong>%s</strong> zu best&auml;tigen?";
#elif L==3
	"Do you want to confirm the ID (identity number/passport) <strong>%s</strong>?";
#elif L==4
	"&iquest;Desea confirmar el ID (DNI/c&eacute;dula) <strong>%s</strong>?";
#elif L==5
	"Voulez-vous confirmer l'ID <strong>%s</strong>?";
#elif L==6
	"&iquest;Desea confirmar el ID (DNI/c&eacute;dula) <strong>%s</strong>?";	// Okoteve traduccin
#elif L==7
	"Vuoi confermare l'ID <strong>%s</strong>?";
#elif L==8
	"Czy chcesz potwierdzi&cacute; I <strong>%s</strong>?";
#elif L==9
	"Voc&ecirc; quer confirmar o ID <strong>%s</strong>?";
#endif

const char *Txt_Document =
#if   L==1
	"Document";
#elif L==2
	"Dokument";
#elif L==3
	"Document";
#elif L==4
	"Documento";
#elif L==5
	"Document";
#elif L==6
	"Documento";	// Okoteve traduccin
#elif L==7
	"Documento";
#elif L==8
	"Document";	// Potrzebujesz tlumaczenie
#elif L==9
	"Documento";
#endif

const char *Txt_document_in_my_courses =
#if   L==1
	"document en les meves assignatures";
#elif L==2
	"Dokument in meinen Kursen";
#elif L==3
	"document in my courses";
#elif L==4
	"documento en mis asignaturas";
#elif L==5
	"document &agrave; mes mati&egrave;res";
#elif L==6
	"documento en mis asignaturas";	// Okoteve traduccin
#elif L==7
	"documento in miei corsi";
#elif L==8
	"dokument w moich kursach";
#elif L==9
	"documento em minha disciplinas";
#endif

const char *Txt_Documents =
#if   L==1
	"Documents";
#elif L==2
	"Dokumente";
#elif L==3
	"Documents";
#elif L==4
	"Documentos";
#elif L==5
	"Documents";
#elif L==6
	"Documentos";	// Okoteve traduccin
#elif L==7
	"Documenti";
#elif L==8
	"Documents";	// Potrzebujesz tlumaczenie
#elif L==9
	"Documentos";
#endif

const char *Txt_document_from_me =
#if   L==1
	"document meu";
#elif L==2
	"Dokument von mir";
#elif L==3
	"document from me";
#elif L==4
	"documento m&iacute;o";
#elif L==5
	"document de moi";
#elif L==6
	"documento m&iacute;o";	// Okoteve traduccin
#elif L==7
	"documento da me";
#elif L==8
	"dokument ode mnie";
#elif L==9
	"documento de mim";
#endif

const char *Txt_documents_from_me =
#if   L==1
	"documents meus";
#elif L==2
	"Dokumente von mir";
#elif L==3
	"documents from me";
#elif L==4
	"documentos m&iacute;os";
#elif L==5
	"documents de moi";
#elif L==6
	"documentos m&iacute;os";	// Okoteve traduccin
#elif L==7
	"documenti da me";
#elif L==8
	"dokumenty ode mnie";
#elif L==9
	"documentos de mim";
#endif

const char *Txt_documents_in_my_courses =
#if   L==1
	"documents en les meves assignatures";
#elif L==2
	"Dokumente in meinen Kursen";
#elif L==3
	"documents in my courses";
#elif L==4
	"documentos en mis asignaturas";
#elif L==5
	"documents &agrave; mes mati&egrave;res";
#elif L==6
	"documentos en mis asignaturas";	// Okoteve traduccin
#elif L==7
	"documenti in miei corsi";
#elif L==8
	"dokumenty w moich kursach";
#elif L==9
	"documentos em minha disciplinas";
#endif

const char *Txt_Documents_area =
#if   L==1
	"Zona de documents";
#elif L==2
	"Dokumente-Bereich";
#elif L==3
	"Documents area";
#elif L==4
	"Zona de documentos";
#elif L==5
	"Zone de documents";
#elif L==6
	"Zona de documentos";	// Okoteve traduccin
#elif L==7
	"Area documenti";
#elif L==8
	"Obszar Dokument&oacute;w";
#elif L==9
	"Zona de documentos";
#endif

const char *Txt_Documents_management_area =
#if   L==1
	"Zona d'administraci&oacute; de documents";
#elif L==2
	"Dokumente-Verwaltung";
#elif L==3
	"Documents management area";
#elif L==4
	"Zona de administraci&oacute;n de documentos";
#elif L==5
	"Zone de gestion de documents";
#elif L==6
	"Zona de administraci&oacute;n de documentos";	// Okoteve traduccin
#elif L==7
	"Area di gestione dei documenti";
#elif L==8
	"Obszar zarzadzania Dokumenty";
#elif L==9
	"Zona de gest&atilde;o de documentos";
#endif

const char *Txt_documents_management_area =
#if   L==1
	"zona de administraci&oacute;n de documentos";	// Necessita traduccio
#elif L==2
	"Dokumente-Verwaltung";
#elif L==3
	"documents management area";
#elif L==4
	"zona de administraci&oacute;n de documentos";
#elif L==5
	"zone de gestion de documents";
#elif L==6
	"zona de administraci&oacute;n de documentos";	// Okoteve traduccin
#elif L==7
	"area di gestione dei documenti";
#elif L==8
	"obszaru zarzadzania dokumentami";
#elif L==9
	"zona de gest&atilde;o de documentos";
#endif

const char *Txt_Done =
#if   L==1
	"Fet";
#elif L==2
	"Gemacht";
#elif L==3
	"Done";
#elif L==4
	"Hecho";
#elif L==5
	"Termin&eacute;";
#elif L==6
	"Hecho";	// Okoteve traduccin
#elif L==7
	"Fatto";
#elif L==8
	"Gotowe";
#elif L==9
	"Feito";
#endif

const char *Txt_Done_assess_test =
#if   L==1
	"He acabat; corregir test";
#elif L==2
	"Geschehen; beurteilen Test";
#elif L==3
	"Done; assess test";
#elif L==4
	"He terminado; corregir test";
#elif L==5
	"Fait; corriger test";
#elif L==6
	"He terminado; corregir test";	// Okoteve traduccin
#elif L==7
	"Finito; correggere test";
#elif L==8
	"Sporz&aogon;dzono; oceny test";
#elif L==9
	"Feito; corrigir teste";
#endif

const char *Txt_Download =	// As a verb
#if   L==1
	"Descarregar";
#elif L==2
	"Download";
#elif L==3
	"Download";
#elif L==4
	"Descargar";
#elif L==5
	"T&eacute;l&eacute;charger";
#elif L==6
	"Descargar";	// Okoteve traduccin
#elif L==7
	"Scarica";
#elif L==8
	"Pobierz";
#elif L==9
	"Baixar";
#endif

const char *Txt_Download_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Descarregar %s";
#elif L==2
	"Laden Sie %s";
#elif L==3
	"Download %s";
#elif L==4
	"Descargar %s";
#elif L==5
	"T&eacute;l&eacute;charger %s";
#elif L==6
	"Descargar %s";		// Okoteve traduccin
#elif L==7
	"Scarica %s";
#elif L==8
	"Pobierz %s";
#elif L==9
	"Download %s";		// Necessita de traduo
#endif

const char *Txt_download =	// As a noun
#if   L==1
	"desc&agrave;rrega";
#elif L==2
	"Download";
#elif L==3
	"download";
#elif L==4
	"descarga";
#elif L==5
	"t&eacute;l&eacute;chargement";
#elif L==6
	"descarga";	// Okoteve traduccin
#elif L==7
	"download";
#elif L==8
	"pobrania";
#elif L==9
	"download";
#endif

const char *Txt_Downloads =	// As a noun
#if   L==1
	"Desc&agrave;rregues";
#elif L==2
	"Downloads";
#elif L==3
	"Downloads";
#elif L==4
	"Descargas";
#elif L==5
	"T&eacute;l&eacute;chargements";
#elif L==6
	"Descargas";	// Okoteve traduccin
#elif L==7
	"Downloads";
#elif L==8
	"Pobrania";
#elif L==9
	"Downloads";
#endif

const char *Txt_downloads =	// As a noun
#if   L==1
	"desc&agrave;rregues";
#elif L==2
	"Downloads";
#elif L==3
	"downloads";
#elif L==4
	"descargas";
#elif L==5
	"t&eacute;l&eacute;chargements";
#elif L==6
	"descargas";	// Okoteve traduccin
#elif L==7
	"downloads";
#elif L==8
	"pobrania";
#elif L==9
	"downloads";
#endif

const char *Txt_Duplicate_USERS =
#if   L==1
	"Duplicats";
#elif L==2
	"Doppelte";
#elif L==3
	"Duplicates";
#elif L==4
	"Duplicados";
#elif L==5
	"En double";
#elif L==6
	"Duplicados";	// Okoteve traduccin
#elif L==7
	"Duplicati";
#elif L==8
	"Duplikaty";
#elif L==9
	"Duplicados";
#endif

const char *Txt_Edit =
#if   L==1
	"Editar";
#elif L==2
	"Bearbeiten";
#elif L==3
	"Edit";
#elif L==4
	"Editar";
#elif L==5
	"&Eacute;diter";
#elif L==6
	"Jehaijey";
#elif L==7
	"Editare";
#elif L==8
	"Edytuj";
#elif L==9
	"Editar";
#endif

const char *Txt_Edit_assignment =
#if   L==1
	"Editar activitat";
#elif L==2
	"Aufgabe bearbeiten";
#elif L==3
	"Edit assignment";
#elif L==4
	"Editar actividad";
#elif L==5
	"&Eacute;diter activit&eacute;";
#elif L==6
	"Editar actividad";	// Okoteve traduccin
#elif L==7
	"Editare attivit&agrave;";
#elif L==8
	"Edytuj zadanie";
#elif L==9
	"Editar atividade";
#endif

const char *Txt_Edit_event =
#if   L==1
	"Editar evento";	// Necessita traduccio
#elif L==2
	"Ereignis bearbeiten";
#elif L==3
	"Edit event";
#elif L==4
	"Editar evento";
#elif L==5
	"&Eacute;diter &eacute;v&eacute;nement";
#elif L==6
	"Editar evento";	// Okoteve traduccin
#elif L==7
	"Editare evento";
#elif L==8
	"Edytuj wydarzenie";
#elif L==9
	"Editar evento";
#endif

const char *Txt_Edit_my_institution =
#if   L==1
	"Edita la meva instituci&oacute;";
#elif L==2
	"Bearbeiten meine Hochschule";
#elif L==3
	"Edit my institution";
#elif L==4
	"Editar mi instituci&oacute;n";
#elif L==5
	"&Eacute;diter mon &eacute;tablissement";
#elif L==6
	"Editar mi instituci&oacute;n";		// Okoteve traduccin
#elif L==7
	"Editare la mia istituzione";
#elif L==8
	"Edycja moich instytucji";
#elif L==9
	"Editar minha institu&ccedil;&atilde;o";
#endif

const char *Txt_Edit_my_personal_data =
#if   L==1
	"Edita meves dades personals";
#elif L==2
	"Bearbeiten meiner pers&ouml;nlichen Daten";
#elif L==3
	"Edit my personal data";
#elif L==4
	"Editar mis datos personales";
#elif L==5
	"&Eacute;diter mes donn&eacute;es personnelles";
#elif L==6
	"Editar mis datos personales";		// Okoteve traduccin
#elif L==7
	"Editare i miei dati personali";
#elif L==8
	"Edycja moich danych osobowych";
#elif L==9
	"Editar os meus dados pessoais";
#endif

const char *Txt_Edit_my_webs_networks =
#if   L==1
	"Edita meves webs / xarxes";
#elif L==2
	"Bearbeiten meiner Webs / Netzwerke";
#elif L==3
	"Edit my webs / networks";
#elif L==4
	"Editar mis webs / redes";
#elif L==5
	"&Eacute;diter mes webs / r&eacute;seaux";
#elif L==6
	"Editar mis webs / redes";	// Okoteve traduccin
#elif L==7
	"Editare i miei webs / reti";
#elif L==8
	"Edycja moje webs / sieci";
#elif L==9
	"Editar as minhas webs / redes";
#endif

const char *Txt_Edit_office_hours =
#if   L==1
	"Editar tutories";
#elif L==2
	"Gesch&auml;ftszeiten bearbeiten";
#elif L==3
	"Edit office hours";
#elif L==4
	"Editar tutor&iacute;as";
#elif L==5
	"&Eacute;diter tuteur";
#elif L==6
	"Editar tutor&iacute;as";	// Okoteve traduccin
#elif L==7
	"Editare tutoraggio";
#elif L==8
	"Edytuj godzin pracy biura";
#elif L==9
	"Editar tutor";
#endif

const char *Txt_Edit_question =
#if   L==1
	"Editar pregunta";
#elif L==2
	"Frage bearbeiten";
#elif L==3
	"Edit question";
#elif L==4
	"Editar pregunta";
#elif L==5
	"&Eacute;diter question";
#elif L==6
	"Editar pregunta";	// Okoteve traduccin
#elif L==7
	"Editare domanda";
#elif L==8
	"Edytuj pytanie";
#elif L==9
	"Editar quest&atilde;o";
#endif

const char *Txt_Edit_record_fields =
#if   L==1
	"Editar camps de fitxes";
#elif L==2
	"Datensatzfelder bearbeiten";
#elif L==3
	"Edit record fields";
#elif L==4
	"Editar campos de fichas";
#elif L==5
	"Modifier champs de cartes";
#elif L==6
	"Editar campos de fichas";	// Okoteve traduccin
#elif L==7
	"Editare campo schede";
#elif L==8
	"Edycja rekord w dziedzinie";
#elif L==9
	"Editar campos cart&otilde;es";
#endif

const char *Txt_Edit_survey =
#if   L==1
	"Editar enquesta";
#elif L==2
	"Umfrage bearbeiten";
#elif L==3
	"Edit survey";
#elif L==4
	"Editar encuesta";
#elif L==5
	"&Eacute;diter sondage";
#elif L==6
	"Editar encuesta";	// Okoteve traduccin
#elif L==7
	"Editare sondaggio";
#elif L==8
	"Edycja ankiety";
#elif L==9
	"Editar inqu&eacute;rito";
#endif

const char *Txt_Edit_plain_text =
#if   L==1
	"Editar text sense format";
#elif L==2
	"Plain Text bearbeiten";
#elif L==3
	"Edit plain text";
#elif L==4
	"Editar texto sin formato";
#elif L==5
	"&Eacute;diter texte simple";
#elif L==6
	"Editar texto sin formato";	// Okoteve traduccin
#elif L==7
	"Editare testo normale";
#elif L==8
	"Edycja tekstu zwyk&lstrok;y";
#elif L==9
	"Editar texto simples";
#endif

const char *Txt_Edit_rich_text =
#if   L==1
	"Editar text enriquit";
#elif L==2
	"Rich Text bearbeiten";
#elif L==3
	"Edit rich text";
#elif L==4
	"Editar texto enriquecido";
#elif L==5
	"&Eacute;diter texte enrichi";
#elif L==6
	"Editar texto enriquecido";	// Okoteve traduccin
#elif L==7
	"Editare testo formattato";
#elif L==8
	"Edycja tekstu sformatowany";
#elif L==9
	"Editar texto rico";
#endif

const char *Txt_eg_A_B =
#if   L==1
	"per ex.: A, B,&hellip;";
#elif L==2
	"z. B.: A, B,&hellip;";
#elif L==3
	"e.g., A, B,&hellip;";
#elif L==4
	"por ej.: A, B,&hellip;";
#elif L==5
	"par exemple: A, B,&hellip;";
#elif L==6
	"por ej.: A, B,&hellip;";	// Okoteve traduccin
#elif L==7
	"ad esempio: A, B,&hellip;";
#elif L==8
	"np. A, B, &hellip;";
#elif L==9
	"ex.: A, B,&hellip;";
#endif

const char *Txt_eg_Lectures_Practicals =
#if   L==1
	"per ex.: Teoria, Pr&agrave;ctiques,&hellip;";
#elif L==2
	"z. B.: Vorlesung, &Uuml;bung,&hellip;";
#elif L==3
	"e.g., Lectures, Practicals,&hellip;";
#elif L==4
	"por ej.: Teor&iacute;a, Pr&aacute;cticas,&hellip;";
#elif L==5
	"par exemple: Th&eacute;orie, Pratique,&hellip;";
#elif L==6
	"por ej.: Teor&iacute;a, Pr&aacute;cticas,&hellip;";	// Okoteve traduccin
#elif L==7
	"ad esempio: Teoria, Pratica,&hellip;";
#elif L==8
	"np. Wyklady, Praktyki &hellip;";
#elif L==9
	"ex.: Teoria, Pr&aacute;tica,&hellip;";
#endif

const char *Txt_ELAPSED_TIME =
#if   L==1
	"Fa";
#elif L==2
	"Vor";
#elif L==3
	"Ago";
#elif L==4
	"Hace";
#elif L==5
	"Y a";
#elif L==6
	"Hace";	// Okoteve traduccin
#elif L==7
	"Fa";
#elif L==8
	"Temu";
#elif L==9
	"Atr&aacute;s";
#endif

const char *Txt_Eliminate =
#if   L==1
	"Eliminar";
#elif L==2
	"Entfernen";
#elif L==3
	"Eliminate";
#elif L==4
	"Eliminar";
#elif L==5
	"&Eacute;liminer";
#elif L==6
	"Mbogue";
#elif L==7
	"Rimuovere";
#elif L==8
	"Wyeliminowac";
#elif L==9
	"Eliminar";
#endif

const char *Txt_Eliminate_all_courses_whithout_users_PART_1_OF_2 =
#if   L==1
	"Eliminar todas las asignaturas sin usuarios y sin accesos en los &uacute;ltimos";	// Necessita traduccio
#elif L==2
	"Eliminate all courses without users and without accesses in the last";			// Need bersetzung
#elif L==3
	"Eliminate all courses without users and without accesses in the last";
#elif L==4
	"Eliminar todas las asignaturas sin usuarios y sin accesos en los &uacute;ltimos";
#elif L==5
	"Eliminate all courses without users and without accesses in the last";			// Besoin de traduction
#elif L==6
	"Eliminar todas las asignaturas sin usuarios y sin accesos en los &uacute;ltimos";	// Okoteve traduccin
#elif L==7
	"Eliminate all courses without users and without accesses in the last";			// Bisogno di traduzione
#elif L==8
	"Eliminate all courses without users and without accesses in the last";			// Potrzebujesz tlumaczenie
#elif L==9
	"Eliminate all courses without users and without accesses in the last";			// Necessita de traduo
#endif

const char *Txt_Eliminate_all_courses_whithout_users_PART_2_OF_2 =
#if   L==1
	"meses.";		// Necessita traduccio
#elif L==2
	"months.";		// Need bersetzung
#elif L==3
	"months.";
#elif L==4
	"meses.";
#elif L==5
	"months.";		// Besoin de traduction
#elif L==6
	"meses.";		// Okoteve traduccin
#elif L==7
	"mesi.";
#elif L==8
	"months.";		// Potrzebujesz tlumaczenie
#elif L==9
	"months.";		// Necessita de traduo
#endif

const char *Txt_Eliminate_all_users_who_are_not_enrolled_on_any_courses_PART_1_OF_2 =
#if   L==1
	"Eliminar todos los usuarios"
	" que no est&eacute;n inscritos en ninguna asignatura"
	" y que lleven m&aacute;s de";				// Necessita traduccio
#elif L==2
	"Eliminate all users"
	" who are not enrolled on any courses"
	" and with more than";					// Need bersetzung
#elif L==3
	"Eliminate all users"
	" who are not enrolled on any courses"
	" and with more than";
#elif L==4
	"Eliminar todos los usuarios"
	" que no est&eacute;n inscritos en ninguna asignatura"
	" y que lleven m&aacute;s de";
#elif L==5
	"Eliminate all users"
	" who are not enrolled on any courses"
	" and with more than";					// Besoin de traduction
#elif L==6
	"Eliminar todos los usuarios"
	" que no est&eacute;n inscritos en ninguna asignatura"
	" y que lleven m&aacute;s de";				// Okoteve traduccin
#elif L==7
	"Rimuovere tutti gli utenti"
	" che non sono registrati ad alcun corso"
	" e con pi&ugrave; di";
#elif L==8
	"Wyeliminuj wszystkich uzytkownik&oacute;w,"
	" kt&oacute;rzy nie sa zarejestrowani"
	" na kazdym z kurs&oacute;w";
#elif L==9
	"Eliminate all users"
	" who are not enrolled on any courses"
	" and with more than";					// Necessita de traduo
#endif

const char *Txt_Eliminate_all_users_who_are_not_enrolled_on_any_courses_PART_2_OF_2 =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"meses sin acceder a %s.";		// Necessita traduccio
#elif L==2
	"months without access to %s.";		// Need bersetzung
#elif L==3
	"months without access to %s.";
#elif L==4
	"meses sin acceder a %s.";
#elif L==5
	"months without access to %s.";		// Besoin de traduction
#elif L==6
	"meses sin acceder a %s.";		// Okoteve traduccin
#elif L==7
	"mesi senza accedere a %s.";
#elif L==8
	"months without access to %s.";		// Potrzebujesz tlumaczenie
#elif L==9
	"months without access to %s.";		// Necessita de traduo
#endif

const char *Txt_Eliminate_from_the_platform_the_users_indicated_in_step_1 =
#if   L==1
	"Eliminar completamente de la plataforma"
	" los usuarios indicados en el paso 1";		// Necessita traduccio
#elif L==2
	"Die Benutzer auf der Liste aus dem System entfernen";
#elif L==3
	"Eliminate from the platform"
	" the users indicated on step 1";
#elif L==4
	"Eliminar completamente de la plataforma"
	" los usuarios indicados en el paso 1";
#elif L==5
	"&Eacute;liminer de la plate-forme"
	" les utilisateurs qui sont sur la liste";
#elif L==6
	"Eliminar completamente de la plataforma"
	" los usuarios indicados en el paso 1";		// Okoteve traduccin
#elif L==7
	"Rimuovere dalla piattaforma"
	" gli utenti che sono in lista";
#elif L==8
	"Wyeliminuj z platformy"
	" uzytkownikow kt&oacute;rzy sa na liscie";
#elif L==9
	"Eliminar da plataforma"
	" os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Eliminate_old_courses =
#if   L==1
	"Eliminar assignatures antics";
#elif L==2
	"Fr&uuml;here Kursen entfernen";
#elif L==3
	"Eliminate old courses";
#elif L==4
	"Eliminar asignaturas antiguas";
#elif L==5
	"&Eacute;liminer anciens mati&egrave;res";
#elif L==6
	"Eliminar asignaturas antiguas";	// Okoteve traduccin
#elif L==7
	"Rimuovere vecchi corsi";
#elif L==8
	"Wyeleminuj starych kurs&oacute;w";
#elif L==9
	"Eliminar antigas disciplinas";
#endif

const char *Txt_Eliminate_old_users =
#if   L==1
	"Eliminar usuaris antics";
#elif L==2
	"Fr&uuml;here Benutzer entfernen";
#elif L==3
	"Eliminate old users";
#elif L==4
	"Eliminar usuarios antiguos";
#elif L==5
	"&Eacute;liminer anciens utilisateurs";
#elif L==6
	"Eliminar usuarios antiguos";	// Okoteve traduccin
#elif L==7
	"Rimuovere vecchi utenti";
#elif L==8
	"Usu&nacute; starych uzytkownik&oacute;w";
#elif L==9
	"Eliminar antigos utilizadores";
#endif

const char *Txt_Eliminating_X_courses_whithout_users_and_with_more_than_Y_months_without_access =	// Warning: it is very important to include %lu and %u in the following sentences
#if   L==1
	"Eliminando %lu asignatura(s) sin usuarios"
	" y que llevaba(n) m&aacute;s de %u meses sin accesos.";	// Necessita traduccio
#elif L==2
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Need bersetzung
#elif L==3
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";
#elif L==4
	"Eliminando %lu asignatura(s) sin usuarios"
	" y que llevaba(n) m&aacute;s de %u meses sin accesos.";
#elif L==5
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Besoin de traduction
#elif L==6
	"Eliminando %lu asignatura(s) sin usuarios"
	" y que llevaba(n) m&aacute;s de %u meses sin accesos.";	// Okoteve traduccin
#elif L==7
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Bisogno di traduzione
#elif L==8
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Eliminating %lu course(s) without users"
	" and with more than %u months without access.";		// Necessita de traduo
#endif

const char *Txt_Eliminate_my_user_account =
#if   L==1
	"Eliminar la meva compte d'usuari";
#elif L==2
	"Entfernen mein Benutzerkonto";
#elif L==3
	"Eliminate my user account";
#elif L==4
	"Eliminar mi cuenta de usuario/a";
#elif L==5
	"Supprimer mon compte d'utilisateur";
#elif L==6
	"Eliminar mi cuenta de usuario/a";		// Okoteve traduccin
#elif L==7
	"Rimuovere il mio account utente";
#elif L==8
	"Usu&nacute; moje konto u&zdot;ytkownika";
#elif L==9
	"Remover minha conta de usu&aacute;rio";
#endif

const char *Txt_Eliminate_user_account =
#if   L==1
	"Eliminar compte d'usuari";
#elif L==2
	"Entfernen Benutzerkonto";
#elif L==3
	"Eliminate user account";
#elif L==4
	"Eliminar cuenta de usuario/a";
#elif L==5
	"Supprimer compte d'utilisateur";
#elif L==6
	"Eliminar cuenta de usuario/a";		// Okoteve traduccin
#elif L==7
	"Rimuovere account utente";
#elif L==8
	"Usu&nacute; konto u&zdot;ytkownika";
#elif L==9
	"Remover conta de usu&aacute;rio";
#endif

const char *Txt_Eliminating_X_users_who_were_not_enrolled_in_any_course_and_with_more_than_Y_months_without_access_to_Z =	// Warning: it is very important to include %lu, %u and %s in the following sentences
#if   L==1
	"Eliminando %lu usuario(s) que no estaba(n) inscrito(s) en ninguna asignatura"
	" y que llevaba(n) m&aacute;s de %u meses sin acceder a %s.";	// Necessita traduccio
#elif L==2
	"Eliminating %lu user(s) who were not enrolled in any course"
	" and with more than %u months without access to %s.";		// Need bersetzung
#elif L==3
	"Eliminating %lu user(s) who were not enrolled in any course"
	" and with more than %u months without access to %s.";
#elif L==4
	"Eliminando %lu usuario(s) que no estaba(n) inscrito(s) en ninguna asignatura"
	" y que llevaba(n) m&aacute;s de %u meses sin acceder a %s.";
#elif L==5
	"Eliminating %lu user(s) who were not enrolled in any course"
	" and with more than %u months without access to %s.";		// Besoin de traduction
#elif L==6
	"Eliminando %lu usuario(s) que no estaba(n) inscrito(s) en ninguna asignatura"
	" y que llevaba(n) m&aacute;s de %u meses sin acceder a %s.";	// Okoteve traduccin
#elif L==7
	"Rimuovendo %lu utenti che non siano registrati in alcun corso"
	" e con pi&ugrave; di %u mesi senza accedere a %s.";
#elif L==8
	"Eliminowanie uzytkownik&oacute;w, kt&oacute;rzy nie lu% odnotowano w kurs"
	" i ponad miesiac u% bez dostepu do %s.";
#elif L==9
	"Eliminating %lu user(s) who were not enrolled in any course"
	" and with more than %u months without access to %s.";		// Necessita de traduo
#endif

const char *Txt_Email =
#if   L==1
	"Correu";
#elif L==2
	"E-Mail";
#elif L==3
	"Email";
#elif L==4
	"Correo";
#elif L==5
	"Courrier";
#elif L==6
	"&Ntilde;e'&#7869;veve";
#elif L==7
	"Email";
#elif L==8
	"Email";
#elif L==9
	"Email";
#endif

const char *Txt_Email_domain_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Dominio de correo <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2
	"E-Mail-Dom&auml;ne <strong>%s</strong> entfernt.";
#elif L==3
	"Email domain <strong>%s</strong> removed.";
#elif L==4
	"Dominio de correo <strong>%s</strong> eliminado.";
#elif L==5
	"Mail domain <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6
	"Dominio de correo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Campo mail <strong>%s</strong> rimosso.";
#elif L==8
	"Domeny poczty <strong>%s</strong> usuniete.";
#elif L==9
	"Mail domain <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Email_domains_allowed_for_notifications =
#if   L==1
	"Dominios de correo permitidos para notificaciones";	// Necessita traduccio
#elif L==2
	"E-Mail-Dom&auml;nen, an die Benachrichtigungen gesendet werden k&ouml;nnen";
#elif L==3
	"Email domains allowed for notifications";
#elif L==4
	"Dominios de correo permitidos para notificaciones";
#elif L==5
	"Email domains allowed for notifications";		// Besoin de traduction
#elif L==6
	"Dominios de correo permitidos para notificaciones";	// Okoteve traduccin
#elif L==7
	"Campo mail permesso per notifiche";
#elif L==8
	"Mail domen dozwolone powiadomien";
#elif L==9
	"Email domains allowed for notifications";		// Necessita de traduo
#endif

const char *Txt_EMAIL_DOMAIN_HELP_ORDER[3] =
   {
#if   L==1
	 "Ordenar por dominio de correo"	// Necessita traduccio
#elif L==2
	 "Nach E-Mail-Dom&auml;ne sortieren"
#elif L==3
	 "Sort by email domain"
#elif L==4
	 "Ordenar por dominio de correo"
#elif L==5
	 "Trier par domaine de messagerie"
#elif L==6
	 "Ordenar por dominio de correo"	// Okoteve traduccin
#elif L==7
	 "Ordina per campo mail"
#elif L==8
	 "Sortuj wedlug domeny poczty"
#elif L==9
	 "Classificar por dom&iacute;nio de email"
#endif
	 ,
#if   L==1
	 "Ordenar por informaci&oacute;n"	// Necessita traduccio
#elif L==2
	 "Nach Information sortieren"
#elif L==3
	 "Sort by information"
#elif L==4
	 "Ordenar por informaci&oacute;n"
#elif L==5
	 "Trier par information"
#elif L==6
	 "Ordenar por informaci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Ordina per informazione"
#elif L==8
	 "Sortuj wedlug informacji"
#elif L==9
	 "Classificar por informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "Ordenar por n&ordm; de usuarios"	// Necessita traduccio
#elif L==2
	 "Nach Anzahl der Benutzer sortieren"
#elif L==3
	 "Sort by no. of users"
#elif L==4
	 "Ordenar por n&ordm; de usuarios"
#elif L==5
	 "Trier par nombre d'utilisateurs"
#elif L==6
	 "Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7
	 "Ordina per numero di utenti"
#elif L==8
	 "Sortuj wedlug numerow uzytkownikow"
#elif L==9
	 "Classificar por n&ordm; de utilizadores"
#endif
	};

const char *Txt_EMAIL_DOMAIN_ORDER[3] =
   {
#if   L==1
	 "Domini de correu"
#elif L==2
	 "E-Mail-Dom&auml;ne"
#elif L==3
	 "Email domain"
#elif L==4
	 "Dominio de correo"
#elif L==5
	 "Domaine de messagerie"
#elif L==6
	 "Dominio de correo"	// Okoteve traduccin
#elif L==7
	 "Campo mail"
#elif L==8
	 "Domeny poczty"
#elif L==9
	 "Dom&iacute;nio de email"
#endif
	 ,
#if   L==1
	 "Informaci&oacute;"
#elif L==2
	 "Information"
#elif L==3
	 "Information"
#elif L==4
	 "Informaci&oacute;n"
#elif L==5
	 "Information"
#elif L==6
	 "Informaci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Informazione"
#elif L==8
	 "Informacja"
#elif L==9
	 "Informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "Usuaris"
#elif L==2
	 "Benutzer"
#elif L==3
	 "Users"
#elif L==4
	 "Usuarios"
#elif L==5
	 "Utilisateurs"
#elif L==6
	 "Usuarios"	// Okoteve traduccin
#elif L==7
	 "Utenti"
#elif L==8
	 "Uzytkownikow"
#elif L==9
	 "Utilizadores"
#endif
	};

const char *Txt_Email_X_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Correo %s confirmado";	// Necessita traduccio
#elif L==2
	"Email %s confirmed";	// Need bersetzung
#elif L==3
	"Email %s confirmed";
#elif L==4
	"Correo %s confirmado";
#elif L==5
	"Email %s confirmed";	// Besoin de traduction
#elif L==6
	"Correo %s confirmado";	// Okoteve traduccin
#elif L==7
	"Email %s confirmed";	// Bisogno di traduzione
#elif L==8
	"Email %s confirmed";	// Potrzebujesz tlumaczenie
#elif L==9
	"Email %s confirmed";	// Necessita de traduo
#endif

const char *Txt_Email_X_has_already_been_confirmed_before =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El correo <strong>%s</strong>"
	" ya hab&iacute;a sido confirmado anteriormente.";	// Necessita traduccio
#elif L==2
	"Email <strong>%s</strong>"
	" has already been confirmed before.";			// Need bersetzung
#elif L==3
	"Email <strong>%s</strong>"
	" has already been confirmed before.";
#elif L==4
	"El correo <strong>%s</strong>"
	" ya hab&iacute;a sido confirmado anteriormente.";
#elif L==5
	"Email <strong>%s</strong>"
	" has already been confirmed before.";			// Besoin de traduction
#elif L==6
	"El correo <strong>%s</strong>"
	" ya hab&iacute;a sido confirmado anteriormente.";	// Okoteve traduccin
#elif L==7
	"Email <strong>%s</strong>"
	" has already been confirmed before.";			// Bisogno di traduzione
#elif L==8
	"Email <strong>%s</strong>"
	" has already been confirmed before.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Email <strong>%s</strong>"
	" has already been confirmed before.";			// Necessita de traduo
#endif

const char *Txt_Email_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Correu <strong>%s</strong> eliminat.";
#elif L==2
	"E-Mail <strong>%s</strong> entfernt.";
#elif L==3
	"Email <strong>%s</strong> removed.";
#elif L==4
	"Correo <strong>%s</strong> eliminado.";
#elif L==5
	"Courrier <strong>%s</strong> supprim&eacute;.";
#elif L==6
	"Correo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Email <strong>%s</strong> rimosso.";
#elif L==8
	"Email <strong>%s</strong> usuniete.";
#elif L==9
	"Email <strong>%s</strong> removido.";
#endif

const char *Txt_Email_unconfirmed =
#if   L==1
	"Correo no confirmado";	// Necessita traduccio
#elif L==2
	"Email unconfirmed";	// Need bersetzung
#elif L==3
	"Email unconfirmed";
#elif L==4
	"Correo no confirmado";
#elif L==5
	"Email unconfirmed";	// Besoin de traduction
#elif L==6
	"Correo no confirmado";	// Okoteve traduccin
#elif L==7
	"Email unconfirmed";	// Bisogno di traduzione
#elif L==8
	"Email unconfirmed";	// Potrzebujesz tlumaczenie
#elif L==9
	"Email unconfirmed";	// Necessita de traduo
#endif

const char *Txt_End_date =
#if   L==1
	"Data final";
#elif L==2
	"Enddatum";
#elif L==3
	"End date";
#elif L==4
	"Fecha final";
#elif L==5
	"Date finale";
#elif L==6
	"Fecha final";	// Okoteve traduccin
#elif L==7
	"Data finale";
#elif L==8
	"Data koncowa";
#elif L==9
	"Data final";
#endif

const char *Txt_Enroll_in_groups =
#if   L==1
	"Inscriure en grups";
#elif L==2
	"Anmeldung mich in Gruppen";
#elif L==3
	"Enroll in groups";
#elif L==4
	"Inscribirme en grupos";
#elif L==5
	"Inscriver moi dans groupes";
#elif L==6
	"Inscribirme en grupos";	// Okoteve traduccin
#elif L==7
	"Iscriver mi in gruppi";
#elif L==8
	"Zarejestruj mnie w grupach";
#elif L==9
	"Inscreva-me em grupos ";
#endif

const char *Txt_Enrollment =
#if   L==1
	"Inscripci&oacute;";
#elif L==2
	"Einschreibung";
#elif L==3
	"Enrollment";
#elif L==4
	"Inscripci&oacute;n";
#elif L==5
	"Inscription";
#elif L==6
	"Inscripci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Registrazione";
#elif L==8
	"Rejestracja";
#elif L==9
	"Inscri&ccedil;&atilde;o";
#endif

const char *Txt_Enrollment_confirmed =
#if   L==1
	"Inscripci&oacute;n confirmada";	// Necessita traduccio
#elif L==2
	"Enrollment confirmed";			// Need bersetzung
#elif L==3
	"Enrollment confirmed";
#elif L==4
	"Inscripci&oacute;n confirmada";
#elif L==5
	"Enrollment confirmed";			// Besoin de traduction
#elif L==6
	"Inscripci&oacute;n confirmada";	// Okoteve traduccin
#elif L==7
	"Registrazione confermata";
#elif L==8
	"Rejestracja potwierdzona";
#elif L==9
	"Enrollment confirmed";			// Necessita de traduo
#endif

const char *Txt_Enrollment_of_X_rejected =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Inscripci&oacute;n de <strong>%s</strong> rechazada.";	// Necessita traduccio
#elif L==2
	"Enrollment of <strong>%s</strong> rejected.";		// Need bersetzung
#elif L==3
	"Enrollment of <strong>%s</strong> rejected.";
#elif L==4
	"Inscripci&oacute;n de <strong>%s</strong> rechazada.";
#elif L==5
	"Enrollment of <strong>%s</strong> rejected.";		// Besoin de traduction
#elif L==6
	"Inscripci&oacute;n de <strong>%s</strong> rechazada.";	// Okoteve traduccin
#elif L==7
	"Enrollment of <strong>%s</strong> rejected.";		// Bisogno di traduzione
#elif L==8
	"Enrollment of <strong>%s</strong> rejected.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Enrollment of <strong>%s</strong> rejected.";		// Necessita de traduo
#endif

const char *Txt_Enrollment_not_confirmed =
#if   L==1
	"Inscripci&oacute;n pendiente de confirmaci&oacute;n";	// Necessita traduccio
#elif L==2
	"Enrollment not confirmed";				// Need bersetzung
#elif L==3
	"Enrollment not confirmed";
#elif L==4
	"Inscripci&oacute;n pendiente de confirmaci&oacute;n";
#elif L==5
	"Enrollment not confirmed";				// Besoin de traduction
#elif L==6
	"Inscripci&oacute;n pendiente de confirmaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Registrazione non confermata";
#elif L==8
	"Rejestracja nie potwierdzona";
#elif L==9
	"Enrollment not confirmed";				// Necessita de traduo
#endif

const char *Txt_Enrollment_requests =
#if   L==1
	"Peticions d'inscripci&oacute;";
#elif L==2
	"Registrierungsanforderungen";
#elif L==3
	"Enrollment requests";
#elif L==4
	"Solicitudes de inscripci&oacute;n";
#elif L==5
	"Demandes d'inscription";
#elif L==6
	"Solicitudes de inscripci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Richieste di registrazione";
#elif L==8
	"Pro&sacute;by o rejestracji";
#elif L==9
	"Pedidos de inscri&ccedil;&atilde;o";
#endif

const char *Txt_Enter_a_new_item_here =
#if   L==1
	"Escriba aqu&iacute; un nuevo apartado";	// Necessita traduccio
#elif L==2
	"Neuen Eintrag hier eingeben";
#elif L==3
	"Enter a new item here";
#elif L==4
	"Escriba aqu&iacute; un nuevo apartado";
#elif L==5
	"Ecrivez ici un nouveau point";
#elif L==6
	"Escriba aqu&iacute; un nuevo apartado";	// Okoteve traduccin
#elif L==7
	"Inserisci qui un nuovo oggetto(tambi&egrave;n item est&agrave; bien)";
#elif L==8
	"Wpisz nowy element o";
#elif L==9
	"Escreva aqui uma nova rubrica";
#endif

const char *Txt_Enter_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Entrar des %s";
#elif L==2
	"Geben Sie %s";
#elif L==3
	"Enter from %s";
#elif L==4
	"Entrar desde %s";
#elif L==5
	"Entrez du %s";
#elif L==6
	"Entrar desde %s";	// Okoteve traduccin
#elif L==7
	"Inserisci da %s";
#elif L==8
	"Wprowadzi&cacute; z %s";
#elif L==9
	"Digite partir %s";
#endif

const char *Txt_Equation_centered =
#if   L==1
	"Equaci&oacute; centrada";
#elif L==2
	"Gleichung zentriert";
#elif L==3
	"Equation centered";
#elif L==4
	"Ecuaci&oacute;n centrada";
#elif L==5
	"Equation centr&eacute;";
#elif L==6
	"Ecuaci&oacute;n centrada";	// Okoteve traduccin
#elif L==7
	"Equazione centrato";
#elif L==8
	"R&oacute;wnanie centered";
#elif L==9
	"equa&ccedil;&atilde;o centrado";
#endif

const char *Txt_Error_getting_data_from_a_recipient =
#if   L==1
	"Error obteniendo datos de un destinatario.";	// Necessita traduccio
#elif L==2
	"Error getting data from a recipient.";		// Need bersetzung
#elif L==3
	"Error getting data from a recipient.";
#elif L==4
	"Error obteniendo datos de un destinatario.";
#elif L==5
	"Error getting data from a recipient.";		// Besoin de traduction
#elif L==6
	"Error obteniendo datos de un destinatario.";	// Okoteve traduccin
#elif L==7
	"Errore acquisizione dati da un destinatario.";
#elif L==8
	"Blad podczas pobierania danych z odbiorca.";
#elif L==9
	"Error getting data from a recipient.";		// Necessita de traduo
#endif

const char *Txt_Error_receiving_or_processing_image =
#if   L==1
	"Error rebent o processant imatge.";
#elif L==2
	"Fehler beim Empfangen oder Verarbeiten von Abbild.";
#elif L==3
	"Error receiving or processing image.";
#elif L==4
	"Error recibiendo o procesando imagen.";
#elif L==5
	"Erreur de r&eacute;ception ou de traitement d'image.";
#elif L==6
	"Error recibiendo o procesando imagen.";	// Okoteve traduccin
#elif L==7
	"Errore durante la ricezione o la elaborazione delle immagini.";
#elif L==8
	"B&lstrok;&aogon;d odbioru lub przetwarzania obrazu.";
#elif L==9
	"Erro no recebimento ou processamento de imagem.";
#endif

const char *Txt_Event =
#if   L==1
	"Esdeveniment";
#elif L==2
	"Ereignis";
#elif L==3
	"Event";
#elif L==4
	"Evento";
#elif L==5
	"&Eacute;v&eacute;nement";
#elif L==6
	"Evento";	// Okoteve traduccin
#elif L==7
	"Evento";
#elif L==8
	"Wydarzenie";
#elif L==9
	"Evento";
#endif

const char *Txt_Event_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"L'esdeveniment <strong>%s</strong> ara est&agrave; ocult.";
#elif L==2
	"Der Ereignis <strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==3
	"Event <strong>%s</strong> is now hidden.";
#elif L==4
	"El evento <strong>%s</strong> ahora est&aacute; oculto.";
#elif L==5
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant cach&eacute;.";
#elif L==6
	"El evento <strong>%s</strong> ahora est&aacute; oculto.";	// Okoteve traduccin
#elif L==7
	"L'evento <strong>%s</strong> &egrave; ora nascosto.";
#elif L==8
	"Wydarzenie <strong>%s</strong> jest ukryte.";
#elif L==9
	"O evento <strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_Event_X_is_now_private =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"L'esdevenimen <strong>%s</strong> ara &eacute;s privat.";
#elif L==2
	"Der Ereignis <strong>%s</strong> ist jetzt privat.";
#elif L==3
	"Event <strong>%s</strong> is now private.";
#elif L==4
	"El evento <strong>%s</strong> ahora es privado.";
#elif L==5
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant priv&eacute;.";
#elif L==6
	"El evento <strong>%s</strong> ahora es privado.";	// Okoteve traduccin
#elif L==7
	"L'evento <strong>%s</strong> &egrave; ora privato.";
#elif L==8
	"Wydarzenie <strong>%s</strong> jest obecnie prywatne.";
#elif L==9
	"O evento <strong>%s</strong> &eacute; agora privado.";
#endif

const char *Txt_Event_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"L'esdevenimen <strong>%s</strong> ara est&agrave; visible.";
#elif L==2
	"Der Ereignis <strong>%s</strong> ist jetzt sichtbar.";
#elif L==3
	"Event <strong>%s</strong> is now visible.";
#elif L==4
	"El evento <strong>%s</strong> ahora est&aacute; visible.";
#elif L==5
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant visible.";
#elif L==6
	"El evento <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7
	"L'evento <strong>%s</strong> &egrave; ora visibile.";
#elif L==8
	"Wydarzenie <strong>%s</strong> jest teraz widoczny.";
#elif L==9
	"O evento <strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_Event_X_is_now_visible_to_users_of_your_courses =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"L'esdevenimen <strong>%s</strong> ara &eacute;s visible"
	" pels usuaris de les seves assignatures.";
#elif L==2
	"Der Ereignis <strong>%s</strong> ist jetzt sichtbar"
	" f&uuml;r Benutzer Ihrer Kurse.";
#elif L==3
	"Event <strong>%s</strong> is now visible"
	" to users of your courses.";
#elif L==4
	"El evento <strong>%s</strong> ahora es visible"
	" por los usuarios de sus asignaturas.";
#elif L==5
	"L'&eacute;v&eacute;nement <strong>%s</strong> est maintenant visible"
	" pour les utilisateurs de vos mati&egrave;res.";
#elif L==6
	"El evento <strong>%s</strong> ahora es visible"
	" por los usuarios de sus asignaturas.";	// Okoteve traduccin
#elif L==7
	"L'evento <strong>%s</strong> &egrave; ora visibile"
	" agli utenti dei corsi.";
#elif L==8
	"Wydarzenie <strong>%s</strong> jest teraz widoczne"
	" dla u&zdot;ytkownik&oacute;w swoich kursach.";
#elif L==9
	"O evento <strong>%s</strong> &eacute; agora vis&iacute;vel"
	" para os usu&aacute;rios de seus disciplinas.";
#endif

const char *Txt_Event_private_click_to_make_it_visible_to_the_users_of_your_courses =
#if   L==1
	"Esdeveniment privat; feu clic per fer-lo visible pels usuaris de les seves assignatures";
#elif L==2
	"Ereignis privat; klicken Sie, um es f&uuml;r die Nutzer Ihrer Kurse sichtbar zu machen";
#elif L==3
	"Event private, click to make it visible to the users of your courses";
#elif L==4
	"Evento privado; pulse para hacerlo visible por los usuarios de sus asignaturas";
#elif L==5
	"Event privat, cliquez pour le rendre visible aux utilisateurs de vos mati&egrave;res";
#elif L==6
	"Evento privado; pulse para hacerlo visible por los usuarios de sus asignaturas";	// Okoteve traduccin
#elif L==7
	"Evento privato, fare clic per renderlo visibile agli utenti dei corsi";
#elif L==8
	"Event privat, kliknij aby by&lstrok; on widoczny dla u&zdot;ytkownik&oacute;w swoich kursach";
#elif L==9
	"Evento privat, clique para torn&aacute;-lo vis&iacute;vel para os usu&aacute;rios de seus disciplinas";
#endif

const char *Txt_Event_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Esdeveniment <strong>%s</strong> eliminat.";
#elif L==2
	"Ereignis <strong>%s</strong> entfernt.";
#elif L==3
	"Event <strong>%s</strong> removed.";
#elif L==4
	"Evento <strong>%s</strong> eliminado.";
#elif L==5
	"&Eacute;v&eacute;nement <strong>%s</strong> supprim&eacute;.";
#elif L==6
	"Event <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Evento <strong>%s</strong> rimosso.";
#elif L==8
	"Wydarzenie <strong>%s</strong> usuniete.";
#elif L==9
	"Evento <strong>%s</strong> removido.";
#endif

const char *Txt_Event_visible_to_the_users_of_your_courses_click_to_make_it_private =
#if   L==1
	"Esdeveniment visible pels usuaris de les seves assignatures; feu clic per fer-lo privat";
#elif L==2
	"Ereignis Nutzer Ihrer Kurse sichtbar zu machen; klicken Sie, um es f&uuml;r die privat";
#elif L==3
	"Event visible to the users of your courses, click to make it private";
#elif L==4
	"Evento visible por los usuarios de sus asignaturas; pulse para hacerlo privado";
#elif L==5
	"Event visible aux utilisateurs de vos mati&egrave;res, cliquez pour le rendre privat";
#elif L==6
	"Evento visible por los usuarios de sus asignaturas; pulse para hacerlo privado";	// Okoteve traduccin
#elif L==7
	"Evento visibile agli utenti dei corsi, fare clic per renderlo privato";
#elif L==8
	"Event widoczny dla u&zdot;ytkownik&oacute;w swoich kurs&oacute;w, kliknij, aby prywatne";
#elif L==9
	"Evento vis&iacute;vel para os usu&aacute;rios de seus disciplinas, clique para torn&aacute;-lo privat";
#endif

const char *Txt_Events =
#if   L==1
	 "Esdeveniments";
#elif L==2
	 "Ereignisse";
#elif L==3
	 "Events";
#elif L==4
	 "Eventos";
#elif L==5
	 "&Eacute;v&eacute;nements";
#elif L==6
	 "Eventos";	// Okoteve traduccin
#elif L==7
	 "Eventi";
#elif L==8
	 "Wydarzen";
#elif L==9
	 "Eventos";
#endif

const char *Txt_Exam =
#if   L==1
	"ex&agrave;men";
#elif L==2
	"Pr&uuml;fung";
#elif L==3
	"Exam";
#elif L==4
	"Examen";
#elif L==5
	"Examen";
#elif L==6
	"Kuaara'&atilde;";
#elif L==7
	"Esame";
#elif L==8
	"Egzamin";
#elif L==9
	"Exame";
#endif

const char *Txt_EXAM_ANNOUNCEMENT =
#if   L==1
	"CONVOCAT&Ograve;RIA D'EXAMEN";
#elif L==2
	"PR&Uuml;FUNGSAUFRUF";
#elif L==3
	"ANNOUNCEMENT OF EXAM";
#elif L==4
	"CONVOCATORIA DE EXAMEN";
#elif L==5
	"CONVOCATION &Agrave; UN EXAMEN";
#elif L==6
	"CONVOCATORIA DE EXAMEN";	// Okoteve traduccin
#elif L==7
	"APPELLO D'ESAME";
#elif L==8
	"OGLOSZENIE egzaminu";
#elif L==9
	"CHAMADA DE EXAME";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Approximate_duration =
#if   L==1
	"Duraci&oacute;n aproximada";	// Necessita traduccio
#elif L==2
	"Voraussichtliche Dauer";
#elif L==3
	"Approximate duration";
#elif L==4
	"Duraci&oacute;n aproximada";
#elif L==5
	"Dur&eacute;e approximative";
#elif L==6
	"Duraci&oacute;n aproximada";	// Okoteve traduccin
#elif L==7
	"Durata approssimativa";
#elif L==8
	"Przyblizony czas trwania";
#elif L==9
	"Dura&ccedil;&atilde;o aproximada";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Course =
#if   L==1
	"Assignatura";
#elif L==2
	"Kurs";
#elif L==3
	"Course";
#elif L==4
	"Asignatura";
#elif L==5
	"Mati&egrave;re";
#elif L==6
	"Mbo'esyry";
#elif L==7
	"Corso";
#elif L==8
	"Kurs";
#elif L==9
	"Disciplina";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Documentation_required =
#if   L==1
	"Documentaci&oacute;n exigida";	// Necessita traduccio
#elif L==2
	"Ausweis erforderlich";
#elif L==3
	"ID required";
#elif L==4
	"Documentaci&oacute;n exigida";
#elif L==5
	"Pi&egrave;ce d'identit obligatoire";
#elif L==6
	"Documentaci&oacute;n exigida";	// Okoteve traduccin
#elif L==7
	"Documentazione richiesta";
#elif L==8
	"ID wymagane";
#elif L==9
	"Documenta&ccedil;&atilde;o necess&aacute;ria";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Exam_date =
#if   L==1
	"Fecha del examen";	// Necessita traduccio
#elif L==2
	"Pr&uuml;fungsdatum";
#elif L==3
	"Exam date";
#elif L==4
	"Fecha del examen";
#elif L==5
	"Date de l'examen";
#elif L==6
	"Fecha del examen";	// Okoteve traduccin
#elif L==7
	"Data d'esame";
#elif L==8
	"Termin egzaminu";
#elif L==9
	"Exame data";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Material_allowed =
#if   L==1
	"Material permitido";	// Necessita traduccio
#elif L==2
	"Erlaubte Unterlagen";
#elif L==3
	"Material allowed";
#elif L==4
	"Material permitido";
#elif L==5
	"Mat&eacute;riel autoris&eacute;";
#elif L==6
	"Material permitido";	// Okoteve traduccin
#elif L==7
	"Materiale permesso";
#elif L==8
	"Material dozwolone";
#elif L==9
	"Material autorizado";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Material_required =
#if   L==1
	"Material obligatorio";	// Necessita traduccio
#elif L==2
	"Erforderliche Unterlagen";
#elif L==3
	"Material required";
#elif L==4
	"Material obligatorio";
#elif L==5
	"Mat&eacute;riel n&eacute;cessaire";
#elif L==6
	"Material obligatorio";	// Okoteve traduccin
#elif L==7
	"Materiale richiesto";
#elif L==8
	"Materialy niezbedne do";
#elif L==9
	"Material obrigat&oacute;rio";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Mode =
#if   L==1
	"Modalidad<br />(escrito, oral,&hellip;)";	// Necessita traduccio
#elif L==2
	"Pr&uuml;fungsart<br />(schriftlich, m&uuml;ndlich,&hellip;)";
#elif L==3
	"Mode<br />(written, oral,&hellip;)";
#elif L==4
	"Modalidad<br />(escrito, oral,&hellip;)";
#elif L==5
	"Mode<br />(&eacute;crit, oral,&hellip;)";
#elif L==6
	"Modalidad<br />(escrito, oral,&hellip;)";	// Okoteve traduccin
#elif L==7
	"Modalit&agrave;<br />(scritto, orale,&hellip;)";
#elif L==8
	"Tryb<br />(pisemnej, ustnej,&hellip;)";
#elif L==9
	"Modalidade<br />(escrita, oral,&hellip;)";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Other_information =
#if   L==1
	"Otras indicaciones";	// Necessita traduccio
#elif L==2
	"Weitere Informationen";
#elif L==3
	"Other information";
#elif L==4
	"Otras indicaciones";
#elif L==5
	"Autres informations";
#elif L==6
	"Otras indicaciones";	// Okoteve traduccin
#elif L==7
	"Altre informazioni";
#elif L==8
	"Inne informacje";
#elif L==9
	"Outras informa&ccedil;&otilde;es";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Place_of_exam =
#if   L==1
	"Lugar de realizaci&oacute;n";	// Necessita traduccio
#elif L==2
	"Pr&uuml;fungsort";
#elif L==3
	"Place of exam";
#elif L==4
	"Lugar de realizaci&oacute;n";
#elif L==5
	"Lieu de l'examen";
#elif L==6
	"Lugar de realizaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Luogo d'esame";
#elif L==8
	"Miejsce egzaminu";
#elif L==9
	"Local do exame";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Session =
#if   L==1
	"Convocatoria<br />(junio, septiembre,&hellip;)";	// Necessita traduccio
#elif L==2
	"Einberufung<br />(Juni, September,&hellip;)";
#elif L==3
	"Session<br />(june, september,&hellip;)";
#elif L==4
	"Convocatoria<br />(junio, septiembre,&hellip;)";
#elif L==5
	"P&eacute;riode<br />(juin, septembre,&hellip;)";
#elif L==6
	"Convocatoria<br />(junio, septiembre,&hellip;)";	// Okoteve traduccin
#elif L==7
	"Sessione<br />(giugno, settembre,&hellip;)";
#elif L==8
	"Sesja<br />(czerwiec, wrzesien,&hellip;)";
#elif L==9
	"Per&iacute;odo<br />(junho, setembro,&hellip;)";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Start_time =
#if   L==1
	"Hora de inicio";	// Necessita traduccio
#elif L==2
	"Beginn um";
#elif L==3
	"Start time";
#elif L==4
	"Hora de inicio";
#elif L==5
	"Heure de d&eacute;but";
#elif L==6
	"Hora de inicio";	// Okoteve traduccin
#elif L==7
	"Ora d'inizio";
#elif L==8
	"Czas rozpoczecia";
#elif L==9
	"Hora de in&iacute;cio";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Structure_of_the_exam =
#if   L==1
	"Estructura del examen";	// Necessita traduccio
#elif L==2
	"Pr&uuml;fungsstruktur";
#elif L==3
	"Structure of the exam";
#elif L==4
	"Estructura del examen";
#elif L==5
	"Structure de l'examen";
#elif L==6
	"Estructura del examen";	// Okoteve traduccin
#elif L==7
	"Struttura d'esame";
#elif L==8
	"Struktura egzaminu";
#elif L==9
	"Estrutura do exame";
#endif

const char *Txt_EXAM_ANNOUNCEMENT_Year_or_semester =	// 1, 2, 3..., not 1984, 2038...
#if   L==1
	"Any o semestre";
#elif L==2
	"Jahr oder Semester";
#elif L==3
	"Year or semester";
#elif L==4
	"A&ntilde;o o semestre";
#elif L==5
	"Ann&eacute;e ou semestre";
#elif L==6
	"A&ntilde;o o semestre";	// Okoteve traduccin
#elif L==7
	"Anno o semestre";
#elif L==8
	"Rok lub semestr";
#elif L==9
	"Ano ou semestre";
#endif

const char *Txt_Exam_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Examen de %s";		// Necessita traduccio
#elif L==2
	"Exam of %s";		// Need bersetzung
#elif L==3
	"Exam of %s";
#elif L==4
	"Examen de %s";
#elif L==5
	"Exam of %s";		// Besoin de traduction
#elif L==6
	"Examen de %s";		// Okoteve traduccin
#elif L==7
	"Esame di %s";
#elif L==8
	"Egzamin z %s";
#elif L==9
	"Exam of %s";		// Necessita de traduo
#endif

const char *Txt_Existing_question =
#if   L==1
	"Pregunta ja existent";
#elif L==2
	"Bestehende Frage";
#elif L==3
	"Existing question";
#elif L==4
	"Pregunta ya existente";
#elif L==5
	"Question existante";
#elif L==6
	"Pregunta ya existente";	// Okoteve traduccin
#elif L==7
	"Domanda esistente";
#elif L==8
	"Istniej&aogon;ca pytanie";
#elif L==9
	"Quest&atilde;o existente";
#endif

const char *Txt_Expand =
#if   L==1
	"Expandir";
#elif L==2
	"Erweitern";
#elif L==3
	"Expand";
#elif L==4
	"Expandir";
#elif L==5
	"&Eacute;largir";
#elif L==6
	"Expandir";	// Okoteve traduccin
#elif L==7
	"Espandere";
#elif L==8
	"Rozwin";
#elif L==9
	"Expandir";
#endif

const char *Txt_Export_questions =
#if   L==1
	"Exportar preguntas";	// Necessita traduccio
#elif L==2
	"Export questions";	// Need bersetzung
#elif L==3
	"Export questions";
#elif L==4
	"Exportar preguntas";
#elif L==5
	"Exporter questions";
#elif L==6
	"Exportar preguntas";	// Okoteve traduccin
#elif L==7
	"Export questions";	// Bisogno di traduzione
#elif L==8
	"Export questions";	// Potrzebujesz tlumaczenie
#elif L==9
	"Export questions";	// Necessita de traduo
#endif

const char *Txt_Export_questions_to_XML_file =
#if   L==1
	"Exportar preguntas a un archivo XML";	// Necessita traduccio
#elif L==2
	"Export questions to XML file";		// Need bersetzung
#elif L==3
	"Export questions to XML file";
#elif L==4
	"Exportar preguntas a un archivo XML";
#elif L==5
	"Exporter des questions vers un fichier XML";
#elif L==6
	"Exportar preguntas a un archivo XML";	// Okoteve traduccin
#elif L==7
	"Export questions to XML file";		// Bisogno di traduzione
#elif L==8
	"Export questions to XML file";		// Potrzebujesz tlumaczenie
#elif L==9
	"Export questions to XML file";		// Necessita de traduo
#endif

const char *Txt_X_faces_have_been_detected_in_front_position_1_Z_ =	// Warning: it is very important to include two %u in the following sentences
#if   L==1
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"1 con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.";	// Necessita traduccio
#elif L==2
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";			// Need bersetzung
#elif L==3
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";
#elif L==4
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"1 con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.";
#elif L==5
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";			// Besoin de traduction
#elif L==6
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"1 con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir el rostro se&ntilde;alado en verde, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==7
	"%u visi sono stati individuati in posizione centrale:<br />"
	"1 con lo sfondo abbastanza chiaro (segnato in verde).<br />"
	"%u con lo sfondo troppo scuro (segnato in rosso).<br />"
	"Se vuoi scegliere il viso segnato in verde, clicca su di esso.";
#elif L==8
	"%u twarze zostaly wykryte w przednim:<br />"
	"1 z dosc jasnym tle (zaznaczone na zielono).<br />"
	"%u z tlem zbyt ciemny (zaznaczone na czerwono).<br />"
	"Jesli chcesz wybrac twarz zaznaczone na zielono, kliknij na niego.";
#elif L==9
	"%u faces have been detected in front position:<br />"
	"1 with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose the face marked in green, click on it.";	// Necessita de traduo
#endif

const char *Txt_X_faces_have_been_detected_in_front_position_Y_Z_ =	// Warning: it is very important to include three %u in the following sentences
#if   L==1
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"%u con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.";	// Necessita traduccio
#elif L==2
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";			// Need bersetzung
#elif L==3
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";
#elif L==4
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"%u con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.";
#elif L==5
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";			// Besoin de traduction
#elif L==6
	"Se han detectado %u rostros en posici&oacute;n frontal:<br />"
	"%u con el fondo suficientemente claro (en color verde),<br />"
	"%u con el fondo demasiado oscuro (en color rojo).<br />"
	"Si desea elegir uno de los rostros se&ntilde;alados en verde, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==7
	"%u visi sono stati individuati in posizione centrale:<br />"
	"%u con lo sfondo abbastanza chiaro (segnato il verde).<br />"
	"%u con lo sfondo troppo scuro (segnato in rosso).<br />"
	"Se vuoi scegliere uno dei visi segnati in verde, clicca su di esso.";
#elif L==8
	"%u twarze zostaly wykryte w przednim:<br />"
	"%u z dosc jasnym tlem (zaznaczone na zielono).<br />"
	"%u z tlem zbyt ciemnym (zaznaczone na czerwono).<br />"
	"Jesli chcesz wybrac jedna z twarzy zaznaczone na zielono, kliknij na niego.";
#elif L==9
	"%u faces have been detected in front position:<br />"
	"%u with the background light enough (marked in green).<br />"
	"%u with the background too dark (marked in red).<br />"
	"If you want to choose one of the faces marked in green, click on it.";			// Necessita de traduo
#endif

const char *Txt_X_faces_marked_in_green_have_been_detected_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"S'han detectat %u cares (assenyalats en verd) en posici&oacute; frontal"
	" i amb el fons prou clar.<br />"
	"Si voleu triar un d'aquests rostres, premi sobre ell.";
#elif L==2
	"%u Fl&auml;chen (gr&uuml;n markiert) wurden vor Position erfasst,"
	" und der Hintergrund ist leicht genug.<br />"
	"Wenn Sie eine dieser Fl&auml;chen zu w&auml;hlen, klicken Sie darauf.";
#elif L==3
	"%u faces (marked in green) have been detected in front position,"
	" and the background is light enough.<br />"
	"If you want to choose one of those faces, click on it.";
#elif L==4
	"Se han detectado %u rostros (se&ntilde;alados en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Si desea elegir uno de esos rostros, pulse sobre &eacute;l.";
#elif L==5
	"%u visages ont &eacute;t&eacute; d&eacute;tect&eacute;s (indiqu&eacute;s en vert) dans la position avant"
	" et avec un fond suffisamment claire.<br />"
	"Si vous voulez choisir un de ces visages, cliquez dessus.";
#elif L==6
	"Se han detectado %u rostros (se&ntilde;alados en verde) en posici&oacute;n frontal"
	" y con el fondo suficientemente claro.<br />"
	"Si desea elegir uno de esos rostros, pulse sobre &eacute;l.";	// Okoteve traduccin
#elif L==7
	"%u visi (segnati in verde) sono stati individuati in posizione frontale,"
	" e lo sfondo &egrave; abbastanza chiaro.<br />"
	"Se vuoi scegliere uno di questi visi, clicca su di esso.";
#elif L==8
	"%u powierzchnie zosta&lstrok;y wykryte (oznaczony na zielono) w po&lstrok;o&zdot;eniu przednim,"
	" o dostatecznie jasne t&lstrok;o.<br />"
	"Je&zdot;eli chcesz wybra&cacute; jedn&aogon; z tych twarzy, kliknij na ni&aogon;.";
#elif L==9
	"%u faces foram detectados (indicado em verde) na posi&ccedil;&atilde;o frontal"
	" e com fundo suficientemente claro.<br />"
	"Se voc&ecirc; quer escolher um daqueles rostos, clique sobre ele.";
#endif

const char *Txt_X_faces_marked_in_red_have_been_detected_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"S'han detectat %u cares (assenyalats en vermell) en posici&oacute; frontal,"
	" per&ograve; el fons &eacute;s massa fosc.";
#elif L==2
	"%u wurden Fl&auml;chen (in rot abgebildet) in der vorderen Position erkannt,"
	" aber der Hintergrund ist zu dunkel.";
#elif L==3
	"%u faces (marked in red) have been detected in front position,"
	" but the background is too dark.";
#elif L==4
	"Se han detectado %u rostros (se&ntilde;alados en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";
#elif L==5
	"%u faces (en rouge) ont &eacute;t&eacute; d&eacute;tect&eacute;s dans la position avant,"
	" mais le fond est trop sombre.";
#elif L==6
	"Se han detectado %u rostros (se&ntilde;alados en rojo) en posici&oacute;n frontal,"
	" pero el fondo es demasiado oscuro.";	// Okoteve traduccin
#elif L==7
	"%u visi (segnati in rosso) sono stati individuati in posizione centrale,"
	" ma lo sfondo &egrave; troppo scuro.";
#elif L==8
	"%u wykryto twarze (pokazane na czerwono) w przednim po&lstrok;o&ztot;eniu,"
	" ale t&lstrok;o jest zbyt ciemne.";
#elif L==9
	"%u rostos foram detectados (mostrados em vermelho) na posi&ccedil;&atilde;o frontal,"
	" mas o fundo &eacute; muito escuro.";
#endif

const char *Txt_Faces_detected =
#if   L==1
	"Rostros detectados";		// Necessita traduccio
#elif L==2
	"Faces detected";		// Need bersetzung
#elif L==3
	"Faces detected";
#elif L==4
	"Rostros detectados";
#elif L==5
	"Faces detected";		// Besoin de traduction
#elif L==6
	"Rostros detectados";		// Okoteve traduccin
#elif L==7
	"Faces detected";		// Bisogno di traduzione
#elif L==8
	"Faces detected";		// Potrzebujesz tlumaczenie
#elif L==9
	"Faces detected";		// Necessita de traduo
#endif

const char *Txt_Failed_email_confirmation_key =
#if   L==1
	"Error en la clave de confirmaci&oacute;n de correo.";	// Necessita traduccio
#elif L==2
	"Failed email confirmation key.";			// Need bersetzung
#elif L==3
	"Failed email confirmation key.";
#elif L==4
	"Error en la clave de confirmaci&oacute;n de correo.";
#elif L==5
	"Failed email confirmation key.";			// Besoin de traduction
#elif L==6
	"Error en la clave de confirmaci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7
	"Failed email confirmation key.";			// Bisogno di traduzione
#elif L==8
	"Failed email confirmation key.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Failed email confirmation key.";			// Necessita de traduo
#endif

const char *Txt_Family_address =
#if   L==1
	"Domicilio familiar";	// Necessita traduccio
#elif L==2
	"Heimatadresse";
#elif L==3
	"Family address";
#elif L==4
	"Domicilio familiar";
#elif L==5
	"Adresse famille";
#elif L==6
	"Domicilio familiar";	// Okoteve traduccin
#elif L==7
	"Domicilio familiare";
#elif L==8
	"Adres rodziny";
#elif L==9
	"Endere&ccedil;o Fam&iacute;lia";
#endif

const char *Txt_Feedback =
#if   L==1
	"Realimentaci&oacute;n";	// Necessita traduccio
#elif L==2
	"Feedback";
#elif L==3
	"Feedback";
#elif L==4
	"Realimentaci&oacute;n";
#elif L==5
	"Feedback";
#elif L==6
	"Realimentaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Feedback";
#elif L==8
	"Opinie";
#elif L==9
	"Feedback";
#endif

const char *Txt_Feedback_to_students =
#if   L==1
	"Realimentaci&oacute;n para estudiantes";	// Necessita traduccio
#elif L==2
	"Feedback an die Studenten";
#elif L==3
	"Feedback to students";
#elif L==4
	"Realimentaci&oacute;n para estudiantes";
#elif L==5
	"Feedback pour les &eacute;tudiants";
#elif L==6
	"Realimentaci&oacute;n para estudiantes";	// Okoteve traduccin
#elif L==7
	"Feedback per gli studenti";
#elif L==8
	"Opinie student&oacute;w";
#elif L==9
	"Feedback para estudantes";
#endif

const char *Txt_Field_BR_name =
#if   L==1
	"Nombre<br />del campo";	// Necessita traduccio
#elif L==2
	"Name des Feldes";
#elif L==3
	"Field<br />name";
#elif L==4
	"Nombre<br />del campo";
#elif L==5
	"Nom<br />du champ";
#elif L==6
	"Nombre<br />del campo";	// Okoteve traduccin
#elif L==7
	"Nome<br />del campo";
#elif L==8
	"Nazwa<br />pola";
#elif L==9
	"Nome<br />do campo";
#endif

const char *Txt_Figures =
#if   L==1
	"Xifres";
#elif L==2
	"Zahlen";
#elif L==3
	"Figures";
#elif L==4
	"Cifras";
#elif L==5
	"Chiffres";
#elif L==6
	"Cifras";	// Okoteve traduccin
#elif L==7
	"Cifre";
#elif L==8
	"Cyfry";
#elif L==9
	"N&uacute;meros";
#endif

const char *Txt_File =
#if   L==1
	"Fitxer";
#elif L==2
	"Datei";
#elif L==3
	"File";
#elif L==4
	"Archivo";
#elif L==5
	"Fichier";
#elif L==6
	"Jehai'aty";
#elif L==7
	"File";
#elif L==8
	"Plik";
#elif L==9
	"Arquivo";
#endif

const char *Txt_file =
#if   L==1
	"fitxer";
#elif L==2
	"Datei";
#elif L==3
	"file";
#elif L==4
	"archivo";
#elif L==5
	"fichier";
#elif L==6
	"jehai'aty";
#elif L==7
	"file";
#elif L==8
	"Plik";
#elif L==9
	"arquivo";
#endif

const char *Txt_X_file =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Fitxer %s";
#elif L==2
	"%s-Datei";
#elif L==3
	"%s file";
#elif L==4
	"Archivo %s";
#elif L==5
	"Fichier %s";
#elif L==6
	"Archivo %s";	// Okoteve traduccin
#elif L==7
	"File %s";
#elif L==8
	"Plik %s";
#elif L==9
	"Arquivo %s";
#endif

const char *Txt_FILE_uncompressed =
#if   L==1
	"sense comprimir";
#elif L==2
	"unkomprimierte";
#elif L==3
	"uncrompressed";
#elif L==4
	"sin comprimir";
#elif L==5
	"non compress&eacute;";
#elif L==6
	"sin comprimir";	// Okoteve traduccin
#elif L==7
	"non compresso";
#elif L==8
	"nieskompresowane";
#elif L==9
	"descompactado";
#endif

const char *Txt_Filename =
#if   L==1
	"Nom del fitxer";
#elif L==2
	"Dateiname";
#elif L==3
	"Filename";
#elif L==4
	"Nombre del archivo";
#elif L==5
	"Nom du fichier";
#elif L==6
	"Nombre del archivo";	// Okoteve traduccin
#elif L==7
	"Nome del file";
#elif L==8
	"Nazwa";
#elif L==9
	"Nome do arquivo";
#endif

const char *Txt_file_folder =
#if   L==1
	"fitxer/carpeta";
#elif L==2
	"Datei/Verzeichnis";
#elif L==3
	"file/folder";
#elif L==4
	"archivo/carpeta";
#elif L==5
	"fichier/r&eacute;pertoire";
#elif L==6
	"archivo/carpeta";	// Okoteve traduccin
#elif L==7
	"file/cartella";
#elif L==8
	"Plik/folder";
#elif L==9
	"arquivo/diret&oacute;rio";
#endif

const char *Txt_FILE_FOLDER_OR_LINK_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"<strong>%s</strong> ahora est&aacute; oculto.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==3
	"<strong>%s</strong> is now hidden.";
#elif L==4
	"<strong>%s</strong> ahora est&aacute; oculto.";
#elif L==5
	"<strong>%s</strong> est maintenant cach&eacute;.";
#elif L==6
	"<strong>%s</strong> ahora est&aacute; oculto.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; ora nascosto.";
#elif L==8
	"<strong>%s</strong> jest ukryty.";
#elif L==9
	"<strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_FILE_FOLDER_OR_LINK_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"<strong>%s</strong> ahora est&aacute; visible.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> ist jetzt sichtbar.";
#elif L==3
	"<strong>%s</strong> is now visible.";
#elif L==4
	"<strong>%s</strong> ahora est&aacute; visible.";
#elif L==5
	"<strong>%s</strong> est maintenant visible.";
#elif L==6
	"<strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; ora visibile.";
#elif L==8
	"<strong>%s</strong> jest teraz widoczny.";
#elif L==9
	"<strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_FILE_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"<strong>%s</strong> eliminat.";
#elif L==2
	"<strong>%s</strong> entfernt.";
#elif L==3
	"<strong>%s</strong> removed.";
#elif L==4
	"<strong>%s</strong> eliminado.";
#elif L==5
	"<strong>%s</strong> supprim&eacute;.";
#elif L==6
	"<strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> rimosso.";
#elif L==8
	"<strong>%s</strong> usuniety.";
#elif L==9
	"<strong>%s</strong> removido.";
#endif

const char *Txt_File_size =
#if   L==1
	"Mida del fitxer";
#elif L==2
	"Dateigr&ouml;&szlig;e";
#elif L==3
	"File size";
#elif L==4
	"Tama&ntilde;o del archivo";
#elif L==5
	"Taille du fichier";
#elif L==6
	"Tama&ntilde;o del archivo";	// Okoteve traduccin
#elif L==7
	"Dimensioni del file";
#elif L==8
	"Rozmiar pliku";
#elif L==9
	"Tamanho do arquivo";
#endif

const char *Txt_File_with_the_logo =
#if   L==1
	"Fitxer amb el logo";
#elif L==2
	"Datei mit dem Logo";
#elif L==3
	"File with the logo";
#elif L==4
	"Archivo con el logo";
#elif L==5
	"Fichier avec le logo";
#elif L==6
	"Archivo con el logo";	// Okoteve traduccin
#elif L==7
	"File con il logo";
#elif L==8
	"Plik ze logo";
#elif L==9
	"Arquivo com o logotipo";
#endif

const char *Txt_File_with_the_photo =
#if   L==1
	"Fitxer amb la foto";
#elif L==2
	"Datei mit dem Foto";
#elif L==3
	"File with the photo";
#elif L==4
	"Archivo con la foto";
#elif L==5
	"Fichier avec la photo";
#elif L==6
	"Archivo con la foto";	// Okoteve traduccin
#elif L==7
	"File con la foto";
#elif L==8
	"Plik ze zdjeciem";
#elif L==9
	"Arquivo com a foto";
#endif

const char *Txt_File_zone =
#if   L==1
	"Zona d'arxius";
#elif L==2
	"Dateienbereich";
#elif L==3
	"File zone";
#elif L==4
	"Zona de archivos";
#elif L==5
	"Zone de fichiers";
#elif L==6
	"Zona de archivos";	// Okoteve traduccin
#elif L==7
	"Area file";
#elif L==8
	"File zone";		// Potrzebujesz tlumaczenie
#elif L==9
	"Zona de arquivos";
#endif

const char *Txt_File_zones =
#if   L==1
	"Zones d'arxius";
#elif L==2
	"Dateienbereiche";
#elif L==3
	"File zones";
#elif L==4
	"Zonas de archivos";
#elif L==5
	"Zones de fichiers";
#elif L==6
	"Zonas de archivos";	// Okoteve traduccin
#elif L==7
	"Area file";
#elif L==8
	"Plik obszar&oacute;w";
#elif L==9
	"Zonas de arquivos";
#endif

const char *Txt_File_zones_disabled =
#if   L==1
	"Zonas de archivos inhabilitadas";	// Necessita traduccio
#elif L==2
	"File zones disabled";			// Need bersetzung
#elif L==3
	"File zones disabled";
#elif L==4
	"Zonas de archivos inhabilitadas";
#elif L==5
	"File zones disabled";			// Besoin de traduction
#elif L==6
	"Zonas de archivos inhabilitadas";	// Okoteve traduccin
#elif L==7
	"Area file disabilitata";
#elif L==8
	"Plik strefy niedostepnej";
#elif L==9
	"File zones disabled";			// Necessita de traduo
#endif

const char *Txt_File_zones_of_the_group_X_are_now_disabled =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Las zonas de archivos del grupo %s est&aacute;n ahora inhabilitadas.";	// Necessita traduccio
#elif L==2
	"File zones of the group %s are now disabled.";				// Need bersetzung
#elif L==3
	"File zones of the group %s are now disabled.";
#elif L==4
	"Las zonas de archivos del grupo %s est&aacute;n ahora inhabilitadas.";
#elif L==5
	"File zones of the group %s are now disabled.";				// Besoin de traduction
#elif L==6
	"Las zonas de archivos del grupo %s est&aacute;n ahora inhabilitadas.";	// Okoteve traduccin
#elif L==7
	"Le aree file del gruppo %s sono ora disabilitate.";
#elif L==8
	"Plik strefy grupy %s jest teraz niedostepny.";
#elif L==9
	"File zones of the group %s are now disabled.";				// Necessita de traduo
#endif

const char *Txt_File_zones_of_the_group_X_are_now_enabled =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Las zonas de archivos del grupo %s est&aacute;n ahora habilitadas.";	// Necessita traduccio
#elif L==2
	"File zones of the group %s are now enabled.";				// Need bersetzung
#elif L==3
	"File zones of the group %s are now enabled.";
#elif L==4
	"Las zonas de archivos del grupo %s est&aacute;n ahora habilitadas.";
#elif L==5
	"File zones of the group %s are now enabled.";				// Besoin de traduction
#elif L==6
	"Las zonas de archivos del grupo %s est&aacute;n ahora habilitadas.";	// Okoteve traduccin
#elif L==7
	"Le aree file del gruppo %s sono ora abilitate.";
#elif L==8
	"Plik strefy grupa %s sa obecnie dost&eogon;pne.";
#elif L==9
	"File zones of the group %s are now enabled.";				// Necessita de traduo
#endif

const char *Txt_File_zones_of_the_group_X_enabled_click_to_disable_them =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Zonas de archivos del grupo %s habilitadas; pulse para inhabilitarlas";	// Necessita traduccio
#elif L==2
	"File zones of the group %s enabled, click to disable them";			// Need bersetzung
#elif L==3
	"File zones of the group %s enabled, click to disable them";
#elif L==4
	"Zonas de archivos del grupo %s habilitadas; pulse para inhabilitarlas";
#elif L==5
	"File zones of the group %s enabled, click to disable them";			// Besoin de traduction
#elif L==6
	"Zonas de archivos del grupo %s habilitadas; pulse para inhabilitarlas";	// Okoteve traduccin
#elif L==7
	"Aree file del gruppo %s abilitate, clicca per disabilitarle";
#elif L==8
	"Plik strefy grupy %s aktywny, kliknij, aby je wylaczyc";
#elif L==9
	"File zones of the group %s enabled, click to disable them";			// Necessita de traduo
#endif

const char *Txt_File_zones_of_the_group_X_disabled_click_to_enable_them =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Zonas de archivos del grupo %s inhabilitadas; pulse para habilitarlas";	// Necessita traduccio
#elif L==2
	"File zones of the group %s disabled, click to enable them";			// Need bersetzung
#elif L==3
	"File zones of the group %s disabled, click to enable them";
#elif L==4
	"Zonas de archivos del grupo %s inhabilitadas; pulse para habilitarlas";
#elif L==5
	"File zones of the group %s disabled, click to enable them";			// Besoin de traduction
#elif L==6
	"Zonas de archivos del grupo %s inhabilitadas; pulse para habilitarlas";	// Okoteve traduccin
#elif L==7
	"Aree file del gruppo %s disabilitate, clicca per abilitarle";
#elif L==8
	"Plik strefy grupy %s jes niedostepny, kliknij aby wlaczyc";
#elif L==9
	"File zones of the group %s disabled, click to enable them";			// Necessita de traduo
#endif

const char *Txt_File_zones_of_the_group_X_Y_removed =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"Zonas de archivos del grupo <strong>%s %s</strong> eliminadas.";	// Necessita traduccio
#elif L==2
	"Dateienbereiche der Gruppe <strong>%s %s</strong> entfernt.";
#elif L==3
	"File areas of the group <strong>%s %s</strong> removed.";
#elif L==4
	"Zonas de archivos del grupo <strong>%s %s</strong> eliminadas.";
#elif L==5
	"File zones of the group <strong>%s %s</strong> removed.";		// Besoin de traduction
#elif L==6
	"Zonas de archivos del grupo <strong>%s %s</strong> eliminadas.";	// Okoteve traduccin
#elif L==7
	"Aree file del gruppo <strong>%s %s</strong> rimosse.";
#elif L==8
	"Plik obszarach grupy <strong>%s %s</strong> usuniety.";
#elif L==9
	"File zones of the group <strong>%s %s</strong> removed.";	// Necessita de traduo
#endif

const char *Txt_Files =
#if   L==1
	"Fitxers";
#elif L==2
	"Dateien";
#elif L==3
	"Files";
#elif L==4
	"Archivos";
#elif L==5
	"Fichiers";
#elif L==6
	"Archivos";	// Okoteve traduccin
#elif L==7
	"File";
#elif L==8
	"Pliki";
#elif L==9
	"Arquivos";
#endif

const char *Txt_files =
#if   L==1
	"fitxers";
#elif L==2
	"Dateien";
#elif L==3
	"files";
#elif L==4
	"archivos";
#elif L==5
	"fichiers";
#elif L==6
	"archivos";	// Okoteve traduccin
#elif L==7
	"file";
#elif L==8
	"pliki";
#elif L==9
	"arquivos";
#endif

const char *Txt_Files_assignments =
#if   L==1
	"Archivos actividades";	// Necessita traduccio
#elif L==2
	"Dateien der Aufgaben";
#elif L==3
	"Files assignments";
#elif L==4
	"Archivos actividades";
#elif L==5
	"Fichiers activites";
#elif L==6
	"Archivos actividades";	// Okoteve traduccin
#elif L==7
	"File attivit&agrave";
#elif L==8
	"Pliki zadan";
#elif L==9
	"Arquivos atividades";
#endif

const char *Txt_Files_copied =
#if   L==1
	"Arxius copiats";
#elif L==2
	"Dateien kopiert";
#elif L==3
	"Files copied";
#elif L==4
	"Archivos copiados";
#elif L==5
	"Fichiers copi&eacute;s";
#elif L==6
	"Archivos copiados";	// Okoteve traduccin
#elif L==7
	"File copiati";
#elif L==8
	"Pliki kopiowane";
#elif L==9
	"Arquivos copiados";
#endif

const char *Txt_Files_uploaded =
#if   L==1
	"Fitxers pujats";
#elif L==2
	"Dateien hochgeladen";
#elif L==3
	"Files uploaded";
#elif L==4
	"Archivos subidos";
#elif L==5
	"Fichiers t&eacute;l&eacute;charg&eacute;s";
#elif L==6
	"Archivos subidos";	// Okoteve traduccin
#elif L==7
	"File caricati";
#elif L==8
	"Pliki dodane";
#elif L==9
	"Arquivos enviados";
#endif

const char *Txt_Files_of_marks_must_contain_a_table_in_HTML_format_ =
#if   L==1
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";			// Necessita traduccio
#elif L==2
	"Bewertungsdateien mssen eine Tabelle im <strong>HTML-Format</strong> ohne Bilder enthalten."
	" Die HTML-Dateien k&ouml;nnen z.B. aus einer Kalkulationstabelle erstellt werden,"
	" indem der gewnschte Bereich aus der Tabelle ausgew&auml;hlt und als HTML gespeichert wird.<br />"
	" Eine Tabelle kann <strong>mehrere Kopf- und Fuzeilen enthalten</strong>."
	" Zwischen solchen Zeilen muss eine <strong>einzige Zeile f&uuml; jeden Studenten bestehen</strong>,"
	" mit einer beliebigen Anzahl von Spalten, <strong>wovon die erste die Ausweis-Nr. enthalten muss</strong>.<br />"
	" Ruft ein Student die Tabelle auf, sieht er nur die f&uuml;r ihn bestimmte Zeile.";
#elif L==3
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures."
	" Each HTML file may be generated, for example, from a spreadsheet,"
	" by selecting the desired portion of the table and saving it as web page.<br />"
	" A table can have <strong>multiple header and footer rows</strong>."
	" Between them it should be <strong>a single row for each student</strong>,"
	" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />"
	" A student will see a table with header rows, followed by the row with their marks, and footer rows.";
#elif L==4
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";
#elif L==5
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures."
	" Each HTML file may be generated, for example, from a spreadsheet,"
	" by selecting the desired portion of the table and saving it as web page.<br />"
	" A table can have <strong>multiple header and footer rows</strong>."
	" Between them it should be <strong>a single row for each student</strong>,"
	" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />"
	" A student will see a table with header rows,"
	" followed by the row with their marks,"
	" and footer rows.";				// Besoin de traduction
#elif L==6
	"Los archivos de calificaciones deben contener una tabla en <strong>formato HTML</strong> sin im&aacute;genes."
	" Cada archivo HTML se puede generar, por ejemplo, desde una hoja de c&aacute;lculo,"
	" seleccionando la parte deseada de la tabla y guard&aacute;ndola como p&aacute;gina web.<br />"
	" Una tabla puede tener <strong>varias filas de cabecera y varias filas de pie</strong>."
	" Entre ellas debe haber <strong>una &uacute;nica fila para cada estudiante</strong>,"
	" con cualquier n&uacute;mero de columnas, conteniendo <strong>la primera columna el ID (DNI/c&eacute;dula) del estudiante</strong>.<br />"
	" Un estudiante ver&aacute; una tabla con las filas de cabecera,"
	" seguidas por la fila con sus calificaciones,"
	" y por las filas de pie.";			// Okoteve traduccin
#elif L==7
	"I file dei risultati devono apparire in una tabella in <strong>formato HTML</strong> senza immagini."
	" Ogni file HTML pu&ograve; essere generato, per esempio, da un foglio di calcolo elettronico,"
	" selezionando la desiderata porzione di tabella e salvandolo come una pagina web.<br />"
	" Una tabella pu&ograve; avere <strong>intestazioni multiple e righe a pi&egrave; di pagina</strong>."
	" Tra di loro dovrebbe esserci <strong>una singola riga per ogni studente</strong>,"
	" con alcuni numeri di colonne, <strong>la prima colonna contiene l'ID dello studente</strong>.<br />"
	" Uno studente vedr&agrave; una tabella con righe intestate,"
	" seguite da una riga con i loro risultati,"
	" e righe a pi&egrave; di pagina.";
#elif L==8
	"Pliki znak&oacute;w musza zawierac tabele <strong>w formacie HTML</strong> bez zdjecia."
	" Kazdy plik HTML moze zostac wygenerowany, na przyklad z arkusza kalkulacyjnego,"
	" wybierajac odpowiednia czesc tabeli i zapisanie go jako strone sieci Web.<br />"
	" Tabela moze miec <strong>wielu wiersze nagl&oacute;wka i stopki</strong>."
	" Pomiedzy nimi nalezy <strong>jednym wierszu dla kazdego ucznia</strong>,"
	" z dowolna iloscia kolumn, <strong>pierwszej kolumnie zawierajacej studenta ID</strong>.<br />"
	" Student zobaczy tabele z wierszy nagl&oacute;wka, a nastepnie przez rzad z ich znak&oacute;w towarowych i stopki.";
#elif L==9
	"Files of marks must contain a table in <strong>HTML format</strong> without pictures."
	" Each HTML file may be generated, for example, from a spreadsheet,"
	" by selecting the desired portion of the table and saving it as web page.<br />"
	" A table can have <strong>multiple header and footer rows</strong>."
	" Between them it should be <strong>a single row for each student</strong>,"
	" with any number of columns, <strong>the first column containing the student's ID</strong>.<br />"
	" A student will see a table with header rows,"
	" followed by the row with their marks,"
	" and footer rows.";				// Necessita de traduo
#endif

const char *Txt_Files_removed =
#if   L==1
	"Arxius eliminats";
#elif L==2
	"Dateien entfernt";
#elif L==3
	"Files removed";
#elif L==4
	"Archivos eliminados";
#elif L==5
	"Fichiers supprim&eacute;s";
#elif L==6
	"Archivos eliminados";	// Okoteve traduccin
#elif L==7
	"File rimossi";
#elif L==8
	"Pliki usuni&eogon;te";
#elif L==9
	"Arquivos removidos";
#endif

const char *Txt_Files_works =
#if   L==1
	"Archivos trabajos";	// Necessita traduccio
#elif L==2
	"Dateien Hausarbeiten";
#elif L==3
	"Files homework";
#elif L==4
	"Archivos trabajos";
#elif L==5
	"Fichiers travaux";
#elif L==6
	"Archivos trabajos";	// Okoteve traduccin
#elif L==7
	"File lavori";
#elif L==8
	"Pliki domowe";
#elif L==9
	"Arquivos trabalhos";
#endif

const char *Txt_Filter =	// As a noun
#if   L==1
	"Filtre";
#elif L==2
	"Filter";
#elif L==3
	"Filter";
#elif L==4
	"Filtro";
#elif L==5
	"Filtre";
#elif L==6
	"Ayga";
#elif L==7
	"Filtro";
#elif L==8
	"Filtr";
#elif L==9
	"Filtro";
#endif

const char *Txt_First_day_of_the_week =
#if   L==1
	"Primer dia de la setmana";
#elif L==2
	"Erster Tag der Woche";
#elif L==3
	"First day of the week";
#elif L==4
	"Primer d&iacute;a de la semana";
#elif L==5
	"Premier jour de la semaine";
#elif L==6
	"Primer d&iacute;a de la semana";	// Okoteve traduccin
#elif L==7
	"Primo giorno della settimana";
#elif L==8
	"Pierwszy dzie&nacute; tygodnia";
#elif L==9
	"Primeiro dia da semana";
#endif

const char *Txt_first_message_not_allowed =
#if   L==1
	"primer mensaje no permitido";	// Necessita traduccio
#elif L==2
	"erste Nachricht nicht erlaubt";
#elif L==3
	"first message not allowed";
#elif L==4
	"primer mensaje no permitido";
#elif L==5
	"premier message non admis";
#elif L==6
	"primer mensaje no permitido";	// Okoteve traduccin
#elif L==7
	"primo messaggio non ammesso";
#elif L==8
	"Pierwszy komunikat nie jest dozwolony";
#elif L==9
	"primeira mensagem n&atilde;o permitida";
#endif

const char *Txt_First_name =
#if   L==1
	"Nom";
#elif L==2
	"Vorname";
#elif L==3
	"First name";
#elif L==4
	"Nombre";
#elif L==5
	"Pr&eacute;nom";
#elif L==6
	"T&eacute;ra";
#elif L==7
	"Primo nome";
#elif L==8
	"Imi&eogon;";
#elif L==9
	"Nome";
#endif

const char *Txt_Folder =
#if   L==1
	"Carpeta";
#elif L==2
	"Verzeichnis";
#elif L==3
	"Folder";
#elif L==4
	"Carpeta";
#elif L==5
	"R&eacute;pertoire";
#elif L==6
	"Carpeta";	// Okoteve traduccin
#elif L==7
	"Cartella";
#elif L==8
	"Folder";
#elif L==9
	"Diret&oacute;rio";
#endif

const char *Txt_folder =
#if   L==1
	"carpeta";
#elif L==2
	"Verzeichnis";
#elif L==3
	"folder";
#elif L==4
	"carpeta";
#elif L==5
	"r&eacute;pertoire";
#elif L==6
	"carpeta";	// Okoteve traduccin
#elif L==7
	"cartella";
#elif L==8
	"folder";
#elif L==9
	"diret&oacute;rio";
#endif

const char *Txt_Folder_X_and_all_its_contents_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.";	// Necessita traduccio
#elif L==2
	"Verzeichnis <strong>%s</strong> und alle Inhalte wurden entfernt.";
#elif L==3
	"Folder <strong>%s</strong> and all its contents removed.";
#elif L==4
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.";
#elif L==5
	"R&eacute;pertoire <strong>%s</strong> et l'ensemble de son contenu supprim&eacute;s.";
#elif L==6
	"Carpeta <strong>%s</strong> y todo su contenido eliminados.";	// Okoteve traduccin
#elif L==7
	"Cartella <strong>%s</strong> e tutto il suo contenuto rimossi.";
#elif L==8
	"Folder <strong>%s</strong> i cala jego zawartosc usunieta.";
#elif L==9
	"Diret&oacute;rio <strong>%s</strong> e todo o seu conte&uacute;do removidos.";
#endif

const char *Txt_Folder_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Carpeta <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2
	"Verzeichnis <strong>%s</strong> entfernt.";
#elif L==3
	"Folder <strong>%s</strong> removed.";
#elif L==4
	"Carpeta <strong>%s</strong> eliminada.";
#elif L==5
	"R&eacute;pertoire <strong>%s</strong> supprim&eacute;.";
#elif L==6
	"Carpeta <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7
	"Cartella <strong>%s</strong> rimossa.";
#elif L==8
	"Folder <strong>%s</strong> usuniety.";
#elif L==9
	"Diret&oacute;rio <strong>%s</strong> removido.";
#endif

const char *Txt_Folders =
#if   L==1
	"Carpetes";
#elif L==2
	"Verzeichnisse";
#elif L==3
	"Folders";
#elif L==4
	"Carpetas";
#elif L==5
	"R&eacute;pertoires";
#elif L==6
	"Carpetas";	// Okoteve traduccin
#elif L==7
	"Cartelle";
#elif L==8
	"Foldery";
#elif L==9
	"Diret&oacute;rios";
#endif

const char *Txt_folders =
#if   L==1
	"carpetes";
#elif L==2
	"Verzeichnisse";
#elif L==3
	"folders";
#elif L==4
	"carpetas";
#elif L==5
	"r&eacute;pertoires";
#elif L==6
	"carpetas";	// Okoteve traduccin
#elif L==7
	"cartelle";
#elif L==8
	"foldery";
#elif L==9
	"diret&oacute;rios";
#endif

const char *Txt_Folders_copied =
#if   L==1
	"Carpetes copiades";
#elif L==2
	"Verzeichnisse kopiert";
#elif L==3
	"Folders copied";
#elif L==4
	"Carpetas copiadas";
#elif L==5
	"R&eacute;pertoires copi&eacute;s";
#elif L==6
	"Carpetas copiadas";	// Okoteve traduccin
#elif L==7
	"Cartelle copiate";
#elif L==8
	"foldery skopiowane";
#elif L==9
	"Diret&oacute;rios copiados";
#endif

const char *Txt_Folders_renamed =
#if   L==1
	"Carpetes anomenades";
#elif L==2
	"Verzeichnisse umbenannt";
#elif L==3
	"Folders renamed";
#elif L==4
	"Carpetas renombradas";
#elif L==5
	"R&eacute;pertoires renomm&eacute;s";
#elif L==6
	"Carpetas renombradas";	// Okoteve traduccin
#elif L==7
	"Cartelle rinominate";
#elif L==8
	"Foldery zmienione";
#elif L==9
	"Diret&oacute;rios renomeados";
#endif

const char *Txt_Folders_removed =
#if   L==1
	"Carpetes eliminades";
#elif L==2
	"Verzeichnisse entfernt";
#elif L==3
	"Folders removed";
#elif L==4
	"Carpetas eliminadas";
#elif L==5
	"R&eacute;pertoires supprim&eacute;s";
#elif L==6
	"Carpetas eliminadas";	// Okoteve traduccin
#elif L==7
	"Cartelle rimossi";
#elif L==8
	"Foldery usuni&eogon;te";
#elif L==9
	"Diret&oacute;rios removidos";
#endif

const char *Txt_Folders_not_renamed =
#if   L==1
	"Carpetes no anomenades";
#elif L==2
	"Verzeichnisse nicht umbenannt";
#elif L==3
	"Folders not renamed";
#elif L==4
	"Carpetas no renombradas";
#elif L==5
	"R&eacute;pertoires pas renomm&eacute;s";
#elif L==6
	"Carpetas no renombradas";	// Okoteve traduccin
#elif L==7
	"Cartelle non rinominate";
#elif L==8
	"Foldery nie zmienione";
#elif L==9
	"Diret&oacute;rios n&atilde;o renomeados";
#endif

const char *Txt_Follow =
#if   L==1
	"Seguir";
#elif L==2
	"Folgen";
#elif L==3
	"Follow";
#elif L==4
	"Seguir";
#elif L==5
	"Suivre";
#elif L==6
	"Seguir";	// Okoteve traduccin
#elif L==7
	"Seguire";
#elif L==8
	"&Sacute;ledzi&cacute;";
#elif L==9
	"Seguir";
#endif

const char *Txt_FollowPerFollow[2] =
      {
#if   L==1
	"Mitjana de seguits per cada seguidor"
#elif L==2
	"Durchschnittliche gefolgt pro anh&auml;nger"
#elif L==3
	"Average followed per follower"
#elif L==4
	"Promedio de seguidos por cada seguidor"
#elif L==5
	"Moyenne de suivis par suiveur"
#elif L==6
	"Promedio de seguidos por cada seguidor"	// Okoteve traduccin
#elif L==7
	"Media di seguiti per seguace"
#elif L==8
        "&Sacute;rednia liczba obserwowani za obserwuj&aogon;"
#elif L==9
	 "M&eacute;dia por seguidos por seguidor"
#endif
      ,
#if   L==1
	"Mitjana de seguidors per cada seguit"
#elif L==2
	"Durchschnittliche anh&auml;nger pro gefolgt"
#elif L==3
	"Average followers per followed"
#elif L==4
	"Promedio de seguidores por cada seguido"
#elif L==5
	"Moyenne de suiveurs par suivie"
#elif L==6
	"Promedio de seguidores por cada seguido"	// Okoteve traduccin
#elif L==7
	"Media di seguace per seguito"
#elif L==8
        "&Sacute;rednia liczba obserwuj&aogon; za obserwowani"
#elif L==9
	"M&eacute;dia por seguidores por seguido"
#endif
      };

const char *Txt_Followed =
#if   L==1
	"Seguits";
#elif L==2
	"Gefolgt";
#elif L==3
	"Followed";
#elif L==4
	"Seguidos";
#elif L==5
	"Suivis";
#elif L==6
	"Seguidos";	// Okoteve traduccin
#elif L==7
	"Seguiti";
#elif L==8
	"Obserwowani";
#elif L==9
	"Seguidos";
#endif

const char *Txt_Followers =
#if   L==1
	"Seguidors";
#elif L==2
	"Anh&auml;nger";
#elif L==3
	"Followers";
#elif L==4
	"Seguidores";
#elif L==5
	"Suiveurs";
#elif L==6
	"Seguidores";	// Okoteve traduccin
#elif L==7
	"Seguaci";
#elif L==8
	"Obserwuj&aogon;";
#elif L==9
	"Seguidores";
#endif

const char *Txt_Following =
#if   L==1
	"Seg&uuml;ent";
#elif L==2
	"Folgende";
#elif L==3
	"Following";
#elif L==4
	"Siguiendo";
#elif L==5
	"Suivant";
#elif L==6
	"Siguiendo";	// Okoteve traduccin
#elif L==7
	"Seguendo";
#elif L==8
	"Nast&eogon;puj&aogon;cy";
#elif L==9
	"Seguinte";
#endif

const char *Txt_Following_unfollow =
#if   L==1
	"Seg&uuml;ent, deixar de seguir";
#elif L==2
	"Folgende, entfolgen";
#elif L==3
	"Following, unfollow";
#elif L==4
	"Siguiendo, dejar de seguir";
#elif L==5
	"Suivant, se d&eacute;sabonner";
#elif L==6
	"Siguiendo, dejar de seguir";	// Okoteve traduccin
#elif L==7
	"Seguendo, smetti";
#elif L==8
	"Nast&eogon;puj&aogon;cy, nie obserwuj";
#elif L==9
	"Seguinte, deixar de seguir";
#endif

const char *Txt_FOLLOWS_YOU =
#if   L==1
	"ET SEGUEIX";
#elif L==2
	"FOLGT DIR";
#elif L==3
	"FOLLOWS YOU";
#elif L==4
	"TE SIGUE";
#elif L==5
	"VOUS SUIT";
#elif L==6
	"TE SIGUE";	// Okoteve traduccin
#elif L==7
	"TI SEGUE";
#elif L==8
	"OBSERWUJE CI&Eogon;";
#elif L==9
	"SEGUE VOC&Ecirc;";
#endif

const char *Txt_For_security_enter_your_password =
#if   L==1
	"Per a m&eacute;s seguretat, introdueixi la seva contrasenya";
#elif L==2
	"Aus Sicherheitsgr&uuml;nden geben Sie Ihr Passwort";
#elif L==3
	"For security, enter your password";
#elif L==4
	"Para mayor seguridad, introduzca su contrase&ntilde;a";
#elif L==5
	"Pour plus de s&eacute;curit&eacute;, entrez votre mot de passe";
#elif L==6
	"Para mayor seguridad, introduzca su contrase&ntilde;a";	// Okoteve traduccin
#elif L==7
	"Per motivi di sicurezza, immettere la password";
#elif L==8
	"Dla bezpiecze&nacute;stwa, wprowad&zacute; has&lstrok;o";
#elif L==9
	"Por seguran&ccedil;a, digite a sua senha";
#endif

const char *Txt_Force_students_to_read_this_information =
#if   L==1
	"Obligar els estudiants a llegir aquesta informaci&oacute;";
#elif L==2
	"Erzwingen Studenten lesen diese Informationen";
#elif L==3
	"Force students to read this information";
#elif L==4
	"Obligar a los estudiantes a leer esta informaci&oacute;n";
#elif L==5
	"Forcer les &eacute;tudiants &agrave; lire cette information";
#elif L==6
	"Obligar a los estudiantes a leer esta informaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Obbligare gli studenti a leggere questa informazione";
#elif L==8
	"Studenci maja przeczytac ta informacje";
#elif L==9
	"For&ccedil;ar os estudantes a ler esta informa&ccedil;&atilde;o";
#endif

const char *Txt_Forgotten_password =
#if   L==1
	"Contrasenya oblidada?";
#elif L==2
	"Passwort vergessen?";
#elif L==3
	"Forgotten password?";
#elif L==4
	"&iquest;Contrasea olvidada?";
#elif L==5
	"Mot de passe oubli&eacute;?";
#elif L==6
	"&iquest;Contrasea olvidada?";		// Okoteve traduccin
#elif L==7
	"Password dimenticata?";
#elif L==8
	"Zapomniane has&lstrok;o?";
#elif L==9
	"Senha esquecida?";
#endif

const char *Txt_Forum =
#if   L==1
	"Foro";		// Necessita traduccio
#elif L==2
	"Forum";
#elif L==3
	"Forum";
#elif L==4
	"Foro";
#elif L==5
	"Forum";
#elif L==6
	"Foro";		// Okoteve traduccin
#elif L==7
	"Forum";
#elif L==8
	"Forum";
#elif L==9
	"F&oacute;rum";
#endif

const char *Txt_Forum_posts =
#if   L==1
	"Mensajes en foros";	// Necessita traduccio
#elif L==2
	"Forum-Beitr&auml;ge";
#elif L==3
	"Forum posts";
#elif L==4
	"Mensajes en foros";
#elif L==5
	"Messages de forum";
#elif L==6
	"Mensajes en foros";	// Okoteve traduccin
#elif L==7
	"Post del forum";
#elif L==8
	"Forum postow";
#elif L==9
	"Posts de f&oacute;rum";
#endif

const char *Txt_Forum_threads =
#if   L==1
	"Discusiones de foro";	// Necessita traduccio
#elif L==2
	"Forum-Threads";
#elif L==3
	"Forum threads";
#elif L==4
	"Discusiones de foro";
#elif L==5
	"Fils de forum";
#elif L==6
	"Discusiones de foro";	// Okoteve traduccin
#elif L==7
	"Discussione del forum";
#elif L==8
	"Watki forum";
#elif L==9
	"Threads de f&oacute;rum";
#endif

const char *Txt_FORUM_THREAD_HELP_ORDER[2] =
   {
#if   L==1
	 "Ordenar por fecha del primer mensaje"	// Necessita traduccio
#elif L==2
	 "Nach Datum der Nachrichten sortieren (aufsteigend)"
#elif L==3
	 "Sort by date of first message"
#elif L==4
	 "Ordenar por fecha del primer mensaje"
#elif L==5
	 "Trier par date du premier message"
#elif L==6
	 "Ordenar por fecha del primer mensaje"	// Okoteve traduccin
#elif L==7
	 "Ordina per data del primo messaggio"
#elif L==8
	 "Sortuj wedlug daty pierwszej wiadomosci"
#elif L==9
	 "Classificar por data da primeira mensagem"
#endif
	 ,
#if   L==1
	 "Ordenar por fecha del &uacute;ltimo mensaje"	// Necessita traduccio
#elif L==2
	 "Nach Datum der Nachrichten sortieren (absteigend)"
#elif L==3
	 "Sort by date of the last message"
#elif L==4
	 "Ordenar por fecha del &uacute;ltimo mensaje"
#elif L==5
	 "Trier par date du dernier message"
#elif L==6
	 "Ordenar por fecha del &uacute;ltimo mensaje"	// Okoteve traduccin
#elif L==7
	 "Ordina per data dell'ultimo messaggio"
#elif L==8
	 "Sortuj wedlug daty ostatniego komunikatu"
#elif L==9
	 "Classificar pela data da &uacute;ltima mensagem"
#endif
	};

const char *Txt_FORUM_THREAD_ORDER[2] =
   {
#if   L==1
	 "Primer mensaje"	// Necessita traduccio
#elif L==2
	 "Erste Nachricht"
#elif L==3
	 "First message"
#elif L==4
	 "Primer mensaje"
#elif L==5
	 "Premier message"
#elif L==6
	 "Primer mensaje"	// Okoteve traduccin
#elif L==7
	 "Primo messaggio"
#elif L==8
	 "Pierwsza wiadomosc"
#elif L==9
	 "Primeira mensagem"
#endif
	 ,
#if   L==1
	 "&Uacute;ltimo mensaje"	// Necessita traduccio
#elif L==2
	 "Letzte Nachricht"
#elif L==3
	 "Last message"
#elif L==4
	 "&Uacute;ltimo mensaje"
#elif L==5
	 "Dernier message"
#elif L==6
	 "&Uacute;ltimo mensaje"	// Okoteve traduccin
#elif L==7
	 "Ultimo messaggio"
#elif L==8
	 "Ostatnia wiadomosc"
#elif L==9
	 "&Uacute;ltima mensagem"
#endif
	};

const char *Txt_FORUM_WHICH_FORUM[For_NUM_WHICH_FORUMS] =
   {
#if   L==1
	 "F&ograve;rums d'aquest lloc"
#elif L==2
	 "Foren zu diesem Standort"
#elif L==3
	 "Forums of this location"
#elif L==4
	 "Foros de este sitio"
#elif L==5
	 "Forums de cet emplacement"
#elif L==6
	 "Foros de este sitio"	// Okoteve traduccin
#elif L==7
	 "Forum di questo sito"
#elif L==8
	 "Forum tej lokalizacji"
#elif L==9
	 "F&oacute;runs deste site"
#endif
	 ,
#if   L==1
	 "Tots els meus f&ograve;rums"
#elif L==2
	 "Alle meine Foren"
#elif L==3
	 "All my forums"
#elif L==4
	 "Todos mis foros"
#elif L==5
	 "Tous mes forums"
#elif L==6
	 "Todos mis foros"	// Okoteve traduccin
#elif L==7
	 "Tutti i miei forum"
#elif L==8
	 "wszystkie moje forum"
#elif L==9
	 "Todos os meus f&oacute;runs"
#endif
	};

const char *Txt_Forums =
#if   L==1
	"F&ograve;rums";
#elif L==2
	"Foren";
#elif L==3
	"Forums";
#elif L==4
	"Foros";
#elif L==5
	"Forums";
#elif L==6
	"Foros";	// Okoteve traduccin
#elif L==7
	"Forum";
#elif L==8
	"Forum";
#elif L==9
	"F&oacute;runs";
#endif

const char *Txt_Found_an_index_htm_file =
#if   L==1
	"Se ha encontrado un archivo <em>index.htm</em>.";	// Necessita traduccio
#elif L==2
	"Found an <em>index.htm</em> file.";			// Need bersetzung
#elif L==3
	"Found an <em>index.htm</em> file.";
#elif L==4
	"Se ha encontrado un archivo <em>index.htm</em>.";
#elif L==5
	"Found an <em>index.htm</em> file.";			// Besoin de traduction
#elif L==6
	"Se ha encontrado un archivo <em>index.htm</em>.";	// Okoteve traduccin
#elif L==7
	"Si &egrave; trovato un file <em>index.htm</em>.";
#elif L==8
	"Znaleziono <em>index.htm</em> plik.";
#elif L==9
	"Found an <em>index.htm</em> file.";		// Necessita de traduo
#endif

const char *Txt_Found_an_index_html_file =
#if   L==1
	"Se ha encontrado un archivo <em>index.html</em>.";	// Necessita traduccio
#elif L==2
	"Found an <em>index.html</em> file.";			// Need bersetzung
#elif L==3
	"Found an <em>index.html</em> file.";
#elif L==4
	"Se ha encontrado un archivo <em>index.html</em>.";
#elif L==5
	"Found an <em>index.html</em> file.";			// Besoin de traduction
#elif L==6
	"Se ha encontrado un archivo <em>index.html</em>.";	// Okoteve traduccin
#elif L==7
	"Si &egrave; trovato un file <em>index.html</em>.";
#elif L==8
	"Znaleziono <em>index.html</em> plik.";
#elif L==9
	"Found an <em>index.html</em> file.";			// Necessita de traduo
#endif

const char *Txt_Frequent_ACTIONS =	// Check that this sentence fits within the allocated space
#if   L==1
	"Freq&uuml;ents";
#elif L==2
	"H&auml;ufige";
#elif L==3
	"Frequent";
#elif L==4
	"Frecuentes";
#elif L==5
	"Frequent";
#elif L==6
	"Frecuentes";	// Okoteve traduccin
#elif L==7
	"Frequenti";
#elif L==8
	"Cz&eogon;ste";
#elif L==9
	"Freq&uuml;entes";
#endif

const char *Txt_from =
#if   L==1
	"de";	// Necessita traduccio
#elif L==2
	"von";
#elif L==3
	"from";
#elif L==4
	"de";
#elif L==5
	"de";
#elif L==6
	"de";	// Okoteve traduccin
#elif L==7
	"da";
#elif L==8
	"z";
#elif L==9
	"de";
#endif

const char *Txt_From_now_on_the_number_of_editing_lines_of_the_field_X_is_Y =	// Warning: it is very important to include %s and %u (in this order) in the following sentences
#if   L==1
	"A partir de ahora el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> es <strong>%u</strong>.";	// Necessita traduccio
#elif L==2
	"From now on the number of editing lines of the field <strong>%s</strong> is <strong>%u</strong>.";	// Need bersetzung
#elif L==3
	"From now on the number of editing lines of the field <strong>%s</strong> is <strong>%u</strong>.";
#elif L==4
	"A partir de ahora el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> es <strong>%u</strong>.";
#elif L==5
	"From now on the number of editing lines of the field <strong>%s</strong> is <strong>%u</strong>.";	// Besoin de traduction
#elif L==6
	"A partir de ahora el n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> es <strong>%u</strong>.";	// Okoteve traduccin
#elif L==7
	"A partire da ora il numero di linee editate <strong>%s</strong> &egrave; <strong>%u</strong>.";
#elif L==8
	"Od tej pory liczba linii w polu edycji <strong>%s</strong> jest <strong>%u</strong>.";
#elif L==9
	"From now on the number of editing lines of the field <strong>%s</strong> is <strong>%u</strong>.";	// Necessita de traduo
#endif

const char *Txt_from_this_course =
#if   L==1
	"desde esta asignatura";	// Necessita traduccio
#elif L==2
	"von dieser Kurs";
#elif L==3
	"from this course";
#elif L==4
	"desde esta asignatura";
#elif L==5
	"de cette mati&egrave;re";
#elif L==6
	"desde esta asignatura";	// Okoteve traduccin
#elif L==7
	"da questo corso";
#elif L==8
	"z tego kursu";
#elif L==9
	"desta disciplina";
#endif

const char *Txt_From_this_time_you_can_receive_messages_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"A partir de este momento, puede recibir mensajes de %s.";	// Necessita traduccio
#elif L==2
	"From this time, you can receive messages from %s.";		// Need bersetzung
#elif L==3
	"From this time, you can receive messages from %s.";
#elif L==4
	"A partir de este momento, puede recibir mensajes de %s.";
#elif L==5
	"From this time, you can receive messages from %s.";		// Besoin de traduction
#elif L==6
	"A partir de este momento, puede recibir mensajes de %s.";	// Okoteve traduccin
#elif L==7
	"From this time, you can receive messages from %s.";		// Bisogno di traduzione
#elif L==8
	"From this time, you can receive messages from %s.";		// Potrzebujesz tlumaczenie
#elif L==9
	"From this time, you can receive messages from %s.";		// Necessita de traduo
#endif

const char *Txt_From_this_time_you_will_not_receive_messages_from_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"A partir de este momento, no recibir&aacute; mensajes de %s.";	// Necessita traduccio
#elif L==2
	"From this time, you will not receive messages from %s.";	// Need bersetzung
#elif L==3
	"From this time, you will not receive messages from %s.";
#elif L==4
	"A partir de este momento, no recibir&aacute; mensajes de %s.";
#elif L==5
	"From this time, you will not receive messages from %s.";	// Besoin de traduction
#elif L==6
	"A partir de este momento, no recibir&aacute; mensajes de %s.";	// Okoteve traduccin
#elif L==7
	"From this time, you will not receive messages from %s.";	// Bisogno di traduzione
#elif L==8
	"From this time, you will not receive messages from %s.";	// Potrzebujesz tlumaczenie
#elif L==9
	"From this time, you will not receive messages from %s.";	// Necessita de traduo
#endif

const char *Txt_Full_name =
#if   L==1
	"Nom complet";
#elif L==2
	"Vollst&auml;ndiger Name";
#elif L==3
	"Full name";
#elif L==4
	"Nombre completo";
#elif L==5
	"Nom complet";
#elif L==6
	"Nombre completo";	// Okoteve traduccin
#elif L==7
	"Nome completo";
#elif L==8
	"Pelne imie i nazwisko";
#elif L==9
	"Nome completo";
#endif

const char *Txt_Full_name_of_the_degree =
#if   L==1
	"Nom complet de la titulaci&oacute;";
#elif L==2
	"Vollst&auml;ndiger Name der Studiengang";
#elif L==3
	"Full name of the degree";
#elif L==4
	"Nombre completo de la titulaci&oacute;n";
#elif L==5
	"Nom complet du &eacute;tude";
#elif L==6
	"Nombre completo de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Nome completo della laurea";
#elif L==8
	"Pe&lstrok;na nazwa stopie&nacute;";
#elif L==9
	"Nome completo da titula&ccedil;&atilde;o";
#endif

const char *Txt_Full_name_of_the_centre =
#if   L==1
	"Nom complet del centre";
#elif L==2
	"Vollst&auml;ndiger Name der Lehrinstitut";
#elif L==3
	"Full name of the centre";
#elif L==4
	"Nombre completo del centro";
#elif L==5
	"Nom complet du centre";
#elif L==6
	"Nombre completo del centro";	// Okoteve traduccin
#elif L==7
	"Nome completo del centro";
#elif L==8
	"Pe&lstrok;na nazwa centrum";
#elif L==9
	"Nome completo do centro";
#endif

const char *Txt_Full_name_of_the_course =
#if   L==1
	"Nom complet de l'assignatura";
#elif L==2
	"Vollst&auml;ndiger Name der Kurs";
#elif L==3
	"Full name of the course";
#elif L==4
	"Nombre completo de la asignatura";
#elif L==5
	"Nom complet de la mati&egrave;re";
#elif L==6
	"Nombre completo de la asignatura";	// Okoteve traduccin
#elif L==7
	"Nome completo del corso";
#elif L==8
	"Pe&lstrok;na nazwa kurs";
#elif L==9
	"Nome completo da disciplina";
#endif

const char *Txt_Full_name_of_the_institution =
#if   L==1
	"Nom complet de la instituci&oacute;";
#elif L==2
	"Vollst&auml;ndiger Name der Institution";
#elif L==3
	"Full name of the institution";
#elif L==4
	"Nombre completo de la instituci&oacute;n";
#elif L==5
	"Nom complet de l'institution";
#elif L==6
	"Nombre completo de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Nome completo dell'istituzione";
#elif L==8
	"Pe&lstrok;na nazwa instytucji";
#elif L==9
	"Nome completo da institui&ccedil;&atilde;o";
#endif

const char *Txt_Function =
#if   L==1
	"Funci&oacute;";
#elif L==2
	"Funktion";
#elif L==3
	"Function";
#elif L==4
	"Funci&oacute;n";
#elif L==5
	"Fonction";
#elif L==6
	"Funci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Funzione";
#elif L==8
	"Funkcja";
#elif L==9
	"Fun&ccedil;&atilde;o";
#endif

const char *Txt_General =
#if   L==1
	"General";
#elif L==2
	"Allgemein";
#elif L==3
	"General";
#elif L==4
	"General";
#elif L==5
	"G&eacute;n&eacute;ral";
#elif L==6
	"General";	// Okoteve traduccin
#elif L==7
	"Generale";
#elif L==8
	"Ogolne";
#elif L==9
	"Geral";
#endif

const char *Txt_general_topics =
#if   L==1
	"temas generales";	// Necessita traduccio
#elif L==2
	"allgemeine Themen";
#elif L==3
	"general topics";
#elif L==4
	"temas generales";
#elif L==5
	"affaires g&eacute;n&eacute;rales";
#elif L==6
	"temas generales";	// Okoteve traduccin
#elif L==7
	"temi generali";
#elif L==8
	"oglne tematy";
#elif L==9
	"temas gerais";
#endif

const char *Txt_Generate_report =
#if   L==1
	"Generar informe";
#elif L==2
	"Generieren Nutzungsbericht";
#elif L==3
	"Generate report";
#elif L==4
	"Generar informe";
#elif L==5
	"G&eacute;n&eacute;rer le rapport";
#elif L==6
	"Generar informe";	// Okoteve traduccin
#elif L==7
	"Crea rapporto";
#elif L==8
	"Wygeneruj raport";
#elif L==9
	"Gerar relat&oacute;rio";
#endif

const char *Txt_Generate_test =
#if   L==1
	"Generar test";
#elif L==2
	"Generieren Test";
#elif L==3
	"Generate test";
#elif L==4
	"Generar test";
#elif L==5
	"G&eacute;n&eacute;rer test";
#elif L==6
	"Generar test";	// Okoteve traduccin
#elif L==7
	"Crea test";
#elif L==8
	"Wygeneruj test";
#elif L==9
	"Gerar teste";
#endif

const char *Txt_Get_a_new_password =
#if   L==1
	"Rebre una nova contrasenya";
#elif L==2
	"Neues Passwort beantragen";
#elif L==3
	"Get a new password";
#elif L==4
	"Recibir una nueva contrase&ntilde;a";
#elif L==5
	"Recevoir un nouveau mot de passe";
#elif L==6
	"Recibir una nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==7
	"Ottenere una nuova password";
#elif L==8
	"Uzyskaj nowe has&lstrok;o";
#elif L==9
	"Obter uma nova senha";
#endif

const char *Txt_Go_back =
#if   L==1
	"Tornar enrere";
#elif L==2
	"Zur&uuml;ckgehen";
#elif L==3
	"Go back";
#elif L==4
	"Volver atr&aacute;s";
#elif L==5
	"Retour";
#elif L==6
	"Volver atr&aacute;s";	// Okoteve traduccin
#elif L==7
	"Tornare";
#elif L==8
	"Wraca&cacute;";
#elif L==9
	"Voltar";
#endif

const char *Txt_Go_to_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Ir a %s";	// Necessita traduccio
#elif L==2
	"Gehe zum %s";
#elif L==3
	"Go to %s";
#elif L==4
	"Ir a %s";
#elif L==5
	"Aller &agrave; %s";
#elif L==6
	"Ir a %s";	// Okoteve traduccin
#elif L==7
	"Vai a %s";
#elif L==8
	"Przejdz do %s";
#elif L==9
	"Ir para %s";
#endif

const char *Txt_Go_to_course_and_reply =
#if   L==1
	"Ir a asignatura y responder";		// Necessita traduccio
#elif L==2
	"Kurs einblenden und antworten";
#elif L==3
	"Go to course and reply";
#elif L==4
	"Ir a asignatura y responder";
#elif L==5
	"Aller &agrave; la mati&egrave;re et r&eacute;pondre";
#elif L==6
	"Ir a asignatura y responder";		// Okoteve traduccin
#elif L==7
	"Vai a corso e rispondere";
#elif L==8
	"Idz do kursu i odpowiedz";
#elif L==9
	"Ir para a disciplina e responder";
#endif

const char *Txt_Go_to_course_and_reply_again =
#if   L==1
	"Ir a asignatura y responder otra vez";		// Necessita traduccio
#elif L==2
	"Kurs einblenden und erneut antworten";
#elif L==3
	"Go to course and reply again";
#elif L==4
	"Ir a asignatura y responder otra vez";
#elif L==5
	"Aller &agrave; la mati&egrave;re et r&eacute;pondre &agrave; nouveau";
#elif L==6
	"Ir a asignatura y responder otra vez";		// Okoteve traduccin
#elif L==7
	"Vai a corso e rispondere nuovamente";
#elif L==8
	"Idz do kursu i odpowiedz ponownie";
#elif L==9
	"Ir para a disciplina e responder novamente";
#endif

const char *Txt_Go_to_one_of_my_courses =
#if   L==1
	"Anar a una de les meves assignatures";
#elif L==2
	"Gehen Sie zu einem meiner Kurse";
#elif L==3
	"Go to one of my courses";
#elif L==4
	"Ir a una de mis asignaturas";
#elif L==5
	"Allez &agrave; l'une de mes mati&egrave;res";
#elif L==6
	"Ir a una de mis asignaturas";	// Okoteve traduccin
#elif L==7
	"Vai a uno dei miei corsi";
#elif L==8
	"Id&zacute; do jednej z moich kurs&oacute;w";
#elif L==9
	"V&aacute; para uma das minha disciplinas";
#endif

const char *Txt_Group =
#if   L==1
	"Grup";
#elif L==2
	"Gruppe";
#elif L==3
	"Group";
#elif L==4
	"Grupo";
#elif L==5
	"Groupe";
#elif L==6
	"Aty";
#elif L==7
	"Gruppo";
#elif L==8
	"Grupa";
#elif L==9
	"Grupo";
#endif

const char *Txt_group =
#if   L==1
	"grup";
#elif L==2
	"Gruppe";
#elif L==3
	"group";
#elif L==4
	"grupo";
#elif L==5
	"groupe";
#elif L==6
	"aty";
#elif L==7
	"gruppo";
#elif L==8
	"grupa";
#elif L==9
	"grupo";
#endif

const char *Txt_Group_closed =
#if   L==1
	"Grupo cerrado";	// Necessita traduccio
#elif L==2
	"Group closed";		// Need bersetzung
#elif L==3
	"Group closed";
#elif L==4
	"Grupo cerrado";
#elif L==5
	"Group closed";		// Besoin de traduction
#elif L==6
	"Grupo cerrado";	// Okoteve traduccin
#elif L==7
	"Gruppo chiuso";
#elif L==8
	"Grupa zamknieta";
#elif L==9
	"Group closed";		// Necessita de traduo
#endif

const char *Txt_Group_X_closed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Grupo %s cerrado";		// Necessita traduccio
#elif L==2
	"Group %s closed";		// Need bersetzung
#elif L==3
	"Group %s closed";
#elif L==4
	"Grupo %s cerrado";
#elif L==5
	"Group %s closed";		// Besoin de traduction
#elif L==6
	"Grupo %s cerrado";		// Okoteve traduccin
#elif L==7
	"Gruppo %s chiuso";
#elif L==8
	"Grupa %s zamknieta";
#elif L==9
	"Group %s closed";		// Necessita de traduo
#endif

const char *Txt_Group_X_closed_click_to_open_it =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Grupo %s cerrado; pulse para abrirlo";		// Necessita traduccio
#elif L==2
	"Group %s closed, click to open it";		// Need bersetzung
#elif L==3
	"Group %s closed, click to open it";
#elif L==4
	"Grupo %s cerrado; pulse para abrirlo";
#elif L==5
	"Group %s closed, click to open it";		// Besoin de traduction
#elif L==6
	"Grupo %s cerrado; pulse para abrirlo";		// Okoteve traduccin
#elif L==7
	"Gruppo %s chiuso, clicca per aprirlo";
#elif L==8
	"Grupa %s zamknieta, kliknij zeby otworzyc";
#elif L==9
	"Group %s closed, click to open it";		// Necessita de traduo
#endif

const char *Txt_Group_name =
#if   L==1
	"Nombre del grupo";	// Necessita traduccio
#elif L==2
	"Name der Gruppe";
#elif L==3
	"Group name";
#elif L==4
	"Nombre del grupo";
#elif L==5
	"Nom du groupe";
#elif L==6
	"Nombre del grupo";	// Okoteve traduccin
#elif L==7
	"Nome del gruppo";
#elif L==8
	"Nazwa grupy";
#elif L==9
	"Nome do grupo";
#endif

const char *Txt_Group_X_open =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Grupo %s abierto";		// Necessita traduccio
#elif L==2
	"Group %s open";		// Need bersetzung
#elif L==3
	"Group %s open";
#elif L==4
	"Grupo %s abierto";
#elif L==5
	"Group %s open";		// Besoin de traduction
#elif L==6
	"Grupo %s abierto";		// Okoteve traduccin
#elif L==7
	"Gruppo %s aperto";
#elif L==8
	"Grupa %s otwarta";
#elif L==9
	"Group %s open";		// Necessita de traduo
#endif

const char *Txt_Group_X_open_click_to_close_it =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Grupo %s abierto; pulse para cerrarlo";	// Necessita traduccio
#elif L==2
	"Group %s open, click to close it";		// Need bersetzung
#elif L==3
	"Group %s open, click to close it";
#elif L==4
	"Grupo %s abierto; pulse para cerrarlo";
#elif L==5
	"Group %s open, click to close it";		// Besoin de traduction
#elif L==6
	"Grupo %s abierto; pulse para cerrarlo";	// Okoteve traduccin
#elif L==7
	"Gruppo %s aperto, clicca per chiuderlo";
#elif L==8
	"Grupa %s otwarta, kliknij zeby zamknac";
#elif L==9
	"Group %s open, click to close it";		// Necessita de traduo
#endif

const char *Txt_Group_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Grupo <strong>%s</strong> eliminado.";		// Necessita traduccio
#elif L==2
	"Gruppe <strong>%s</strong> entfernt.";
#elif L==3
	"Group <strong>%s</strong> removed.";
#elif L==4
	"Grupo <strong>%s</strong> eliminado.";
#elif L==5
	"Groupe <strong>%s</strong> supprim&eacute;.";
#elif L==6
	"Grupo <strong>%s</strong> eliminado.";		// Okoteve traduccin
#elif L==7
	"Gruppo <strong>%s</strong> rimosso.";
#elif L==8
	"Grupa <strong>%s</strong> usunieta.";
#elif L==9
	"Group <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Groups =
#if   L==1
	"Grups";
#elif L==2
	"Gruppen";
#elif L==3
	"Groups";
#elif L==4
	"Grupos";
#elif L==5
	"Groupes";
#elif L==6
	"Aty";
#elif L==7
	"Gruppi";
#elif L==8
	"Grupy";
#elif L==9
	"Grupos";
#endif

const char *Txt_Groups_OF_A_USER =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Grups de %s";
#elif L==2
	"%s groups";		// Need bersetzung
#elif L==3
	"%s groups";
#elif L==4
	"Grupos de %s";
#elif L==5
	"Groupes de %s";
#elif L==6
	"Grupos de %s";		// Okoteve traduccin
#elif L==7
	"Gruppi di %s";
#elif L==8
	"%s grupy";
#elif L==9
	"%s groups";		// Necessita de traduo
#endif

const char *Txt_Guided_academic_assignments =
#if   L==1
	"Actividades acad&eacute;micas dirigidas";	// Necessita traduccio
#elif L==2
	"Gef&uuml;hrte akademische Aufgaben";
#elif L==3
	"Guided academic assignments";
#elif L==4
	"Actividades acad&eacute;micas dirigidas";
#elif L==5
	"Activit&eacute;s acad&eacute;miques guid&eacute;es";
#elif L==6
	"Actividades acad&eacute;micas dirigidas";	// Okoteve traduccin
#elif L==7
	"Attivit&agrave; accademiche guidate";
#elif L==8
	"Zwiedzanie dyplomowe";
#elif L==9
	"Atividades acad&eacute;micas guided";
#endif

const char *Txt_Happy_birthday =
#if   L==1
	"Feli&ccedil; aniversari!";
#elif L==2
	"Gute zum Geburtstag!";
#elif L==3
	"Happy birthday!";
#elif L==4
	"&iexcl;Feliz cumplea&ntilde;os!";
#elif L==5
	"Joyeux anniversaire!";
#elif L==6
	"&iexcl;Feliz cumplea&ntilde;os!";	// Okoteve traduccin
#elif L==7
	"Buon compleanno!";
#elif L==8
	"Wszystkiego najlepszego!";
#elif L==9
	"Feliz anivers&aacute;rio!";
#endif

const char *Txt_Help =
#if   L==1
	"Ajuda";
#elif L==2
	"Hilfe";
#elif L==3
	"Help";
#elif L==4
	"Ayuda";
#elif L==5
	"Aide";
#elif L==6
	"Pytyv&otilde;";
#elif L==7
	"Aiuto";
#elif L==8
	"Pomoc";
#elif L==9
	"Ajuda";
#endif

const char *Txt_HELP_email =
#if   L==1
	"Exemple: garcia@exemplemail.com";
#elif L==2
	"Beispiel: muller@beispielmail.com";
#elif L==3
	"Example: smith@examplemail.com";
#elif L==4
	"Ejemplo: gonzalez@ejemplomail.com";
#elif L==5
	"Exemple: dubois@exemplemail.com";
#elif L==6
	"Ejemplo: gonzalez@ejemplomail.com";	// Okoteve traduccin
#elif L==7
	"Esempio: rossi@esempiomail.com";
#elif L==8
	"Przyk&lstrok;ad: nowak@przykladmail.com";
#elif L==9
	"Exemplo: silva@exemplomail.com";
#endif

const char *Txt_HELP_nickname =
#if   L==1
	"Exemple: @garcia";
#elif L==2
	"Beispiel: @muller";
#elif L==3
	"Example: @smith";
#elif L==4
	"Ejemplo: @gonzalez";
#elif L==5
	"Exemple: @dubois";
#elif L==6
	"Ejemplo: @gonzalez";	// Okoteve traduccin
#elif L==7
	"Esempio: @rossi";
#elif L==8
	"Przyk&lstrok;ad: @nowak";
#elif L==9
	"Exemplo: @silva";
#endif

const char *Txt_HELP_password =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"%u o m&eacute;s car&agrave;cters";
#elif L==2
	"%u oder mehr Zeichen";
#elif L==3
	"%u or more characters";
#elif L==4
	"%u o m&aacute;s caracteres";
#elif L==5
	"%u ou plusieurs caract&egrave;res";
#elif L==6
	"%u o m&aacute;s caracteres";	// Okoteve traduccin
#elif L==7
	"%u o pi&ugrave; caratteri";
#elif L==8
	"%u lub wi&eogon;cej znak&oacute;w";
#elif L==9
	"%u ou mais caracteres";
#endif

const char *Txt_Hidden_MALE_PLURAL =
#if   L==1
	"Ocultos";	// Necessita traduccio
#elif L==2
	"Versteckt";
#elif L==3
	"Hidden";
#elif L==4
	"Ocultos";
#elif L==5
	"Cach&eacute;s";
#elif L==6
	"Ocultos";	// Okoteve traduccin
#elif L==7
	"Nascosti";
#elif L==8
	"Ukryty";
#elif L==9
	"Ocultos";
#endif

const char *Txt_hidden_document =
#if   L==1
	"document ocult";
#elif L==2
	"Dokument verborgen";
#elif L==3
	"hidden document";
#elif L==4
	"documento oculto";
#elif L==5
	"document cach&eacute;";
#elif L==6
	"documento oculto";	// Okoteve traduccin
#elif L==7
	"documento nascosto";
#elif L==8
	"hidden document";	// Potrzebujesz tlumaczenie
#elif L==9
	"documento oculto";
#endif

const char *Txt_hidden_documents =
#if   L==1
	"documents ocults";
#elif L==2
	"Dokumente verborgen";
#elif L==3
	"hidden documents";
#elif L==4
	"documentos ocultos";
#elif L==5
	"documents cach&eacute;s";
#elif L==6
	"documentos ocultos";	// Okoteve traduccin
#elif L==7
	"documenti nascosti";
#elif L==8
	"hidden documents";	// Potrzebujesz tlumaczenie
#elif L==9
	"documentos ocultos";
#endif

const char *Txt_Hidden_survey =
#if   L==1
	"Encuesta oculta";	// Necessita traduccio
#elif L==2
	"Ausgeblendet Umfrage";
#elif L==3
	"Hidden survey";
#elif L==4
	"Encuesta oculta";
#elif L==5
	"Sondage cach&eacute;";
#elif L==6
	"Encuesta oculta";	// Okoteve traduccin
#elif L==7
	"Sondaggio nascosto";
#elif L==8
	"Ukryte badania";
#elif L==9
	"Inqu&eacute;rito oculto";
#endif

const char *Txt_Hide =
#if   L==1
	"Amagar";
#elif L==2
	"Ausblenden";
#elif L==3
	"Hide";
#elif L==4
	"Ocultar";
#elif L==5
	"Masquer";
#elif L==6
	"Ocultar";	// Okoteve traduccin
#elif L==7
	"Nascondi";
#elif L==8
	"Ukryj";
#elif L==9
	"Ocultar";
#endif

const char *Txt_Hide_FOLDER_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Amagar %s";
#elif L==2
	"Ausblenden %s";
#elif L==3
	"Hide %s";
#elif L==4
	"Ocultar %s";
#elif L==5
	"Masquer %s";
#elif L==6
	"Ocultar %s";	// Okoteve traduccin
#elif L==7
	"Nascondi %s";
#elif L==8
	"Ukryj %s";
#elif L==9
	"Ocultar %s";
#endif

const char *Txt_Hide_left_column =
#if   L==1
	"Ocultar columna izquierda";	// Necessita traduccio
#elif L==2
	"Links Spalte ausblenden";
#elif L==3
	"Hide left column";
#elif L==4
	"Ocultar columna izquierda";
#elif L==5
	"Masquer la colonne de gauche";
#elif L==6
	"Ocultar columna izquierda";	// Okoteve traduccin
#elif L==7
	"Nascondi colonna di sinistra";
#elif L==8
	"Ukryj lewa kolumne";
#elif L==9
	"Ocultar coluna da esquerda";
#endif

const char *Txt_Hide_message =
#if   L==1
	"Amaga missatge";
#elif L==2
	"Ausblende Nachricht";
#elif L==3
	"Hide message";
#elif L==4
	"Ocultar mensaje";
#elif L==5
	"Masquer message";
#elif L==6
	"Ocultar mensaje";	// Okoteve traduccin
#elif L==7
	"Nascondi messaggio";
#elif L==8
	"Hide message";		// Potrzebujesz tlumaczenie
#elif L==9
	"Ocultar mensagem";
#endif

const char *Txt_Hide_right_column =
#if   L==1
	"Ocultar columna derecha";	// Necessita traduccio
#elif L==2
	"Rechte Spalte ausblenden";
#elif L==3
	"Hide right column";
#elif L==4
	"Ocultar columna derecha";
#elif L==5
	"Masquer la colonne de droite";
#elif L==6
	"Ocultar columna derecha";	// Okoteve traduccin
#elif L==7
	"Nascondi colonna di destra";
#elif L==8
	"Ukryj prawa kolumne";
#elif L==9
	"Ocultar coluna da direita";
#endif

const char *Txt_historical_log =
#if   L==1
	"registre hist&ograve;ric";
#elif L==2
	"historische log";
#elif L==3
	"historical log";
#elif L==4
	"registro hist&oacute;rico";
#elif L==5
	"journal historique";
#elif L==6
	"registro hist&oacute;rico";		// Okoteve traduccin
#elif L==7
	"registro storico";
#elif L==8
	"dziennik historyczny";
#elif L==9
	"log hist&oacute;rico";
#endif

const char *Txt_Hits =	// hits = visits, clicks, page views...
#if   L==1
	"Accessos";
#elif L==2
	"Anmeldungen";
#elif L==3
	"Hits";
#elif L==4
	"Accesos (clics)";
#elif L==5
	"Visites";
#elif L==6
	"Accesos (clics)";		// Okoteve traduccin
#elif L==7
	"Visite";
#elif L==8
	"Ods&lstrok;on";
#elif L==9
	"Visitas";
#endif

const char *Txt_Hits_as_a_USER =	// hits = visits, clicks, page views... Warning: it is very important to include %s in the following sentences
#if   L==1
	"Accessos (clics) com a %s";
#elif L==2
	"Anmeldungen (Klicks) als %s";
#elif L==3
	"Hits (clicks) as a %s";
#elif L==4
	"Accesos (clics) como %s";
#elif L==5
	"Visites (clics) en tant que %s";
#elif L==6
	"Accesos (clics) como %s";		// Okoteve traduccin
#elif L==7
	"Visite (scatti) come %s";
#elif L==8
	"Ods&lstrok;on (klikni&eogon;cia) jako %s";
#elif L==9
	"Visitas (cliques) como %s";
#endif

const char *Txt_Hits_per_action =	// hits = visits, clicks, page views...
#if   L==1
	"Accessos (clics) per acci&oacute;";
#elif L==2
	"Anmeldungen (Klicks) pro Handlung";
#elif L==3
	"Hits (clicks) per action";
#elif L==4
	"Accesos (clics) por acci&oacute;n";
#elif L==5
	"Visites (clics) par action";
#elif L==6
	"Accesos (clics) por acci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Visite (clic) per azione";
#elif L==8
	"Ods&lstrok;on (klikni&eogon;cia) dzia&lstrok;anie";
#elif L==9
	"Visitas (cliques) por a&ccedil;&atilde;o";
#endif

const char *Txt_Hits_per_year =	// hits = visits, clicks, page views...
#if   L==1
	"Accessos (clics) per any";
#elif L==2
	"Anmeldungen (Klicks) pro Jahr";
#elif L==3
	"Hits (clicks) per year";
#elif L==4
	"Accesos (clics) por a&ntilde;o";
#elif L==5
	"Visites (clics) par an";
#elif L==6
	"Accesos (clics) por a&ntilde;o";	// Okoteve traduccin
#elif L==7
	"Visite (clic) all'anno";
#elif L==8
	"Ods&lstrok;on (klikni&eogon;cia) rocznie";
#elif L==9
	"Visitas (cliques) por ano";
#endif

const char *Txt_Hits_without_course_selected =	// hits = visits, clicks, page views...
#if   L==1
	"Accessos (clics) sense assignatura seleccionada";
#elif L==2
	"Anmeldungen (Klicks) ohne Kurs ausgew&auml;hlt";
#elif L==3
	"Hits (clicks) without course selected";
#elif L==4
	"Accesos (clics) sin asignatura seleccionada";
#elif L==5
	"Visites (clics) sans mati&egrave;re s&eacute;lectionn&eacute;e";
#elif L==6
	"Accesos (clics) sin asignatura seleccionada";		// Okoteve traduccin
#elif L==7
	"Visite (scatti) senza corso selezionato";
#elif L==8
	"Ods&lstrok;on (klikni&eogon;cia) bez oczywi&sacute;cie wybrana";
#elif L==9
	"Visitas (cliques) sem disciplina selecionada";
#endif

const char *Txt_Holiday =
#if   L==1
	"Festivitat";
#elif L==2
	"Feiertag";
#elif L==3
	"Holiday";
#elif L==4
	"Festividad";
#elif L==5
	"F&ecirc;te";
#elif L==6
	"Festividad";
#elif L==7
	"Festivit&agrave;";
#elif L==8
	"Wakacje";
#elif L==9
	"F&eacute;ria";
#endif

const char *Txt_Holiday_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Festividad <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2
	"Feiertag <strong>%s</strong> entfernt.";
#elif L==3
	"Holiday <strong>%s</strong> removed.";
#elif L==4
	"Festividad <strong>%s</strong> eliminada.";
#elif L==5
	"Holiday <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6
	"Festividad <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7
	"Festivit&agrave; <strong>%s</strong> rimossa.";
#elif L==8
	"Wakacje <strong>%s</strong> usuniete.";
#elif L==9
	"Holiday <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_HOLIDAY_TYPES[Hld_NUM_TYPES_HOLIDAY] =
   {
#if   L==1
	 "D&iacute;a festivo"	// Necessita traduccio
#elif L==2
	 "Urlaub"
#elif L==3
	 "Holiday"
#elif L==4
	 "D&iacute;a festivo"
#elif L==5
	 "Jour f&eacute;ri&eacute;"
#elif L==6
	 "Arete"
#elif L==7
	 "Festivit&agrave;"
#elif L==8
	 "wakacje"
#elif L==9
	 "Feriado"
#endif
	 ,
#if   L==1
	 "Per&iacute;odo no lectivo"	// Necessita traduccio
#elif L==2
	 "Non-Schule Zeitraum"
#elif L==3
	 "Non school period"
#elif L==4
	 "Per&iacute;odo no lectivo"
#elif L==5
	 "P&eacute;riode sans &eacute;cole"
#elif L==6
	 "Per&iacute;odo no lectivo"	// Okoteve traduccin
#elif L==7
	 "Periodo senza lezione"
#elif L==8
	 "Okres nauki szkolnej dla"
#elif L==9
	 "Per&iacute;odo sem escola"
#endif
	};

const char *Txt_Holidays =
#if   L==1
	"Dies festius";
#elif L==2
	"Feiertage";
#elif L==3
	"Holidays";
#elif L==4
	"D&iacute;as festivos";
#elif L==5
	"Jours f&eacute;ri&eacute;s";
#elif L==6
	"Arete";
#elif L==7
	"Festivit&agrave;";
#elif L==8
	"Noclegi";
#elif L==9
	"F&eacute;rias";
#endif

const char *Txt_HOLIDAYS_HELP_ORDER[2] =
   {
#if   L==1
	 "Ordenar por lugar"	// Necessita traduccio
#elif L==2
	 "Nach Standort sortieren"
#elif L==3
	 "Sort by place"
#elif L==4
	 "Ordenar por lugar"
#elif L==5
	 "Trier par emplacement"
#elif L==6
	 "Ordenar por lugar"	// Okoteve traduccin
#elif L==7
	 "Ordina per localit&grave;"
#elif L==8
	 "Sortuj wedlug miejsca"
#elif L==9
	 "Classificar por localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "Ordenar por fecha"	// Necessita traduccio
#elif L==2
	 "Sortieren nach Datum"
#elif L==3
	 "Sort by date"
#elif L==4
	 "Ordenar por fecha"
#elif L==5
	 "Trier par date"
#elif L==6
	 "Ordenar por fecha"	// Okoteve traduccin
#elif L==7
	 "Ordina per data"
#elif L==8
	 "Sortuj wedlug daty"
#elif L==9
	 "Classificar por data"
#endif
	};

const char *Txt_HOLIDAYS_ORDER[2] =
   {
#if   L==1
	 "Lloc"
#elif L==2
	 "Standort"
#elif L==3
	 "Place"
#elif L==4
	 "Lugar"
#elif L==5
	 "Emplacement"
#elif L==6
	 "Tenda"
#elif L==7
	 "Luogo"
#elif L==8
	 "Miejsce"
#elif L==9
	 "Localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "Data inicial"
#elif L==2
	 "Startdatum"
#elif L==3
	 "Start date"
#elif L==4
	 "Fecha inicial"
#elif L==5
	 "Date initiale"
#elif L==6
	 "Fecha inicial"	// Okoteve traduccin
#elif L==7
	 "Data iniziale"
#elif L==8
	 "Data rozpoczecia"
#elif L==9
	 "Data de in&iacute;cio"
#endif
	};

const char *Txt_Home_PAGE =
#if   L==1
	"Inicio";	// Necessita traduccio
#elif L==2
	"Home";
#elif L==3
	"Home";
#elif L==4
	"Inicio";
#elif L==5
	"Accueil";
#elif L==6
	"Inicio";	// Okoteve traduccin
#elif L==7
	"Home";
#elif L==8
	"Dom";
#elif L==9
	"Home";
#endif

const char *Txt_hour =
#if   L==1
	"hora";
#elif L==2
	"Stunde";
#elif L==3
	"hour";
#elif L==4
	"hora";
#elif L==5
	"heure";
#elif L==6
	"aravo";
#elif L==7
	"ora";
#elif L==8
	"godzina";
#elif L==9
	"hora";
#endif

const char *Txt_hours =
#if   L==1
	"horas";	// Necessita traduccio
#elif L==2
	"Stunden";
#elif L==3
	"hours";
#elif L==4
	"horas";
#elif L==5
	"heures";
#elif L==6
	"aravo";
#elif L==7
	"ore";
#elif L==8
	"godziny";
#elif L==9
	"horas";
#endif

const char *Txt_hours_ABBREVIATION =	// Abbreviation of "hours"
#if   L==1
	"h";	// Necessita traduccio
#elif L==2
	"S";
#elif L==3
	"h";
#elif L==4
	"h";
#elif L==5
	"h";
#elif L==6
	"a";
#elif L==7
	"o";
#elif L==8
	"g";
#elif L==9
	"h";
#endif

const char *Txt_I_have_read_this_information =
#if   L==1
	"He le&iacute;do esta informaci&oacute;n";	// Necessita traduccio
#elif L==2
	"Ich habe diese Information lesen";
#elif L==3
	"I have read this information";
#elif L==4
	"He le&iacute;do esta informaci&oacute;n";
#elif L==5
	"J'ai lu cette information";
#elif L==6
	"He le&iacute;do esta informaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Ho letto questa informazione";
#elif L==8
	"Czytalem te informacje";
#elif L==9
	"Eu li esta informa&ccedil;&atilde;o";
#endif

const char *Txt_I_understand_that_this_action_can_not_be_undone =
#if   L==1
	"Entenc que aquesta acci&oacute; no es pot desfer.";
#elif L==2
	"Ich verstehe, dass diese Aktion kann nicht r&uuml;ckg&auml;ngig gemacht werden.";
#elif L==3
	"I understand that this action can not be undone.";
#elif L==4
	"Entiendo que esta acci&oacute;n no se puede deshacer.";
#elif L==5
	"Je comprends que cette action ne peut &ecirc;tre annul&eacute;e.";
#elif L==6
	"Entiendo que esta acci&oacute;n no se puede deshacer.";		// Okoteve traduccin
#elif L==7
	"Mi rendo conto che questa azione non pu&ograve; essere annullata.";
#elif L==8
	"Rozumiem, &zdot;e dzia&lstrok;anie nie mo&zdot;na cofn&aogon;&cacute;.";
#elif L==9
	"Eu entendo que esta a&ccedil;&atilde;o n&atilde;o pode ser desfeita.";
#endif

const char *Txt_Icons =
#if   L==1
	"Icones";
#elif L==2
	"Symbole";
#elif L==3
	"Icons";
#elif L==4
	"Iconos";
#elif L==5
	"Ic&ocirc;nes";
#elif L==6
	"Iconos";	// Okoteve traduccin
#elif L==7
	"Icone";
#elif L==8
	"Ikony";
#elif L==9
	"&Iacute;cones";
#endif

const char *Txt_ID =
#if   L==1
	"ID (DNI/c&egrave;dula)";
#elif L==2
	"Ausweis-Nr.";
#elif L==3
	"ID (identity number/passport)";
#elif L==4
	"ID (DNI/c&eacute;dula)";
#elif L==5
	"Num. d'identit&eacute;";
#elif L==6
	"ID (DNI/c&eacute;dula)";	// Okoteve traduccin
#elif L==7
	"Numero carta d'identit&agrave;";
#elif L==8
	"Dow&oacute;d osobisty";
#elif L==9
	"N&ordm; identif.";
#endif

const char *Txt_ID_X_had_already_been_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El ID (DNI/c&eacute;dula) %s ja havia estat confirmat.";
#elif L==2
	"ID %s hatte bereits best&auml;tigt worden.";
#elif L==3
	"ID %s had already been confirmed.";
#elif L==4
	"El ID (DNI/c&eacute;dula) %s ya hab&iacute;a sido confirmado.";
#elif L==5
	"L'ID %s avait d&eacute;j&agrave; &eacute;t&eacute; confirm&eacute;e.";
#elif L==6
	"El ID (DNI/c&eacute;dula) %s ya hab&iacute;a sido confirmado.";		// Okoteve traduccin
#elif L==7
	"L'ID %s era gi&agrave; stato confermato.";
#elif L==8
	"ID %s zosta&lstrok;a ju&zdot; potwierdzona.";
#elif L==9
	"O ID %s j&aacute; havia sido confirmada.";
#endif

const char *Txt_ID_X_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"ID (DNI/c&eacute;dula) %s confirmat";
#elif L==2
	"ID %s best&auml;tigt";
#elif L==3
	"ID %s confirmed";
#elif L==4
	"ID (DNI/c&eacute;dula) %s confirmado";
#elif L==5
	"ID %s confirm&eacute;";
#elif L==6
	"ID (DNI/c&eacute;dula) %s confirmado";		// Okoteve traduccin
#elif L==7
	"ID %s confermato";
#elif L==8
	"ID %s potwierdzone";
#elif L==9
	"ID %s confirmado";
#endif

const char *Txt_ID_X_not_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"ID (DNI/c&eacute;dula) %s no confirmat";
#elif L==2
	"ID %s nicht best&auml;tigt";
#elif L==3
	"ID %s not confirmed";
#elif L==4
	"ID (DNI/c&eacute;dula) %s no confirmado";
#elif L==5
	"ID %s pas confirm&eacute;";
#elif L==6
	"ID (DNI/c&eacute;dula) %s no confirmado";		// Okoteve traduccin
#elif L==7
	"ID %s non confermato";
#elif L==8
	"ID %s nie potwierdzone";
#elif L==9
	"ID %s n&atilde;o confirmado";
#endif

const char *Txt_ID_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"ID (DNI/c&eacute;dula) <strong>%s</strong> eliminat.";
#elif L==2
	"ID <strong>%s</strong> entfernt.";
#elif L==3
	"ID <strong>%s</strong> removed.";
#elif L==4
	"ID (DNI/c&eacute;dula) <strong>%s</strong> eliminado.";
#elif L==5
	"ID <strong>%s</strong> supprim&eacute;.";
#elif L==6
	"ID (DNI/c&eacute;dula) <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"ID <strong>%s</strong> rimosso.";
#elif L==8
	"ID <strong>%s</strong> usuniete.";
#elif L==9
	"Email <strong>%s</strong> removido.";
#endif

const char *Txt_Identified_views =
#if   L==1
	"Accessos identificats";
#elif L==2
	"Identifizierte Ansichten";
#elif L==3
	"Identified views";
#elif L==4
	"Accesos identificados";
#elif L==5
	"Vues identifi&eacute;es";
#elif L==6
	"Accesos identificados";	// Okoteve traduccin
#elif L==7
	"Visualizzazioni identificati";
#elif L==8
	"Zidentyfikowane ods&lstrok;on";
#elif L==9
	"Acessos identificados";
#endif

const char *Txt_If_there_are_multiple_versions_of_the_ID_ =
#if   L==1
	"Si hi ha diverses versions de la ID"
	" (per ex. DNI amb o sense lletra al final),"
	" indiqueu totes les versions.";
#elif L==2
	"Wenn es mehrere Versionen der ID"
	" (zB. mit oder ohne Buchstabe am Ende),"
	" schreiben alle Versionen.";
#elif L==3
	"If there are multiple versions of the ID"
	" (eg. with or without letter at the end),"
	" write out all versions.";
#elif L==4
	"Si existen varias versiones del ID"
	" (por ej. DNI con o sin letra al final),"
	" indique todas las versiones.";
#elif L==5
	"S'il existe plusieurs versions du ID"
	" (par example avec ou sans lettre &agrave; la fin),"
	" &eacute;crire toutes les versions.";
#elif L==6
	"Si existen varias versiones del ID"
	" (por ej. DNI con o sin letra al final),"
	" indique todas las versiones.";	// Okoteve traduccin
#elif L==7
	"Se ci sono pi&ugrave; versioni del ID"
	" (ad es. con o senza lettera alla fine),"
	" scrivere tutte le versioni.";
#elif L==8
	"Je&sacute;li istnieje wiele wersji identyfikatora"
	" (np. Z lub bez litery na ko&nacute;cu),"
	" wypisa&cacute; wszystkie wersje.";
#elif L==9
	"Se houver v&aacute;rias vers&otilde;es do n&ordm; de identifica&ccedil;&atilde;o"
	" (eg., com ou sem letra no final),"
	" escreva todas as vers&otilde;es.";
#endif

const char *Txt_If_this_is_a_new_user_in_X_you_should_indicate_her_his_ID =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Si se trata de un usuario nuevo en %s,"
	" debe indicar el ID (DNI/c&eacute;dula).";	// Necessita traduccio
#elif L==2
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// bersetzung notwendig
#elif L==3
	"If this is a new user in %s,"
	" you should indicate her/his ID.";
#elif L==4
	"Si se trata de un usuario nuevo en %s,"
	" debe indicar el ID (DNI/c&eacute;dula).";
#elif L==5
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// Besoin de traduction
#elif L==6
	"Si se trata de un usuario nuevo en %s,"
	" debe indicar el ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==7
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// Bisogno di traduzione
#elif L==8
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// Potrzebujesz tlumaczenie
#elif L==9
	"If this is a new user in %s,"
	" you should indicate her/his ID.";		// Necessita de traduo
#endif

const char *Txt_If_you_can_not_find_your_institution_your_centre_your_degree_or_your_courses_you_can_create_them =
#if   L==1
	"Si no trobes la seva instituci&oacute;, el seu centre,"
	" la seva titulaci&oacute; o les seves assignatures, pot crear-los.";
#elif L==2
	"Wenn Sie Ihre Institution, Ihre Zentrum, Ihr Studium"
	" oder Ihre Kurse nicht finden k&oacute;nnen, k&oacute;nnen Sie sie erstellen.";
#elif L==3
	"If you can not find your institution, your centre,"
	" your degree or your courses, you can create them.";
#elif L==4
	"Si no encuentra su instituci&oacute;n, su centro,"
	" su titulaci&oacute;n o sus asignaturas, puede crearlos.";
#elif L==5
	"Si vous ne trouvez pas votre &eacute;tablissement, votre centre,"
	" votre &eacute;tude ou de vos mati&egrave;res, vous pouvez les cr&eacute;er.";
#elif L==6
	"Si no encuentra su instituci&oacute;n, su centro,"
	" su titulaci&oacute;n o sus asignaturas, puede crearlos.";	// Okoteve traduccin
#elif L==7
	"Se non riesci a trovare il vostro istituto, il centro,"
	" il vostro grado o vostri corsi, &egrave; possibile creare.";
#elif L==8
	"Je&sacute;li nie mo&zdot;esz znale&zacute;&cacute; instytucj&eogo, Centrum,"
	" studia lub kursy swoich, mo&zdot;na je tworzy&cacute;.";
#elif L==9
	"Se voc&ecirc; n&atilde;o consegue encontrar a sua institui&ccedil;&atilde;o, o seu centro,"
	" o seu grau ou seus disciplinas, voc&ecirc; pode cri&aacute;-los.";
#endif

const char *Txt_If_you_have_an_Android_device_try_SWADroid =
#if   L==1
	"Si teniu un dispositiu Android, provi SWADroid";
#elif L==2
	"Wenn Sie ein Android-Ger&auml;t haben, versuchen SWADroid";
#elif L==3
	"If you have an Android device, try SWADroid";
#elif L==4
	"Si tiene un dispositivo Android, pruebe SWADroid";
#elif L==5
	"Si vous avez un appareil Android, essayer SWADroid";
#elif L==6
	"Si tiene un dispositivo Android, pruebe SWADroid";	// Okoteve traduccin
#elif L==7
	"Se avete un dispositivo Android, provate SWADroid";
#elif L==8
	"Je&sacute;li masz telefon z systemem Android, spr&oacute;buj SWADroid";
#elif L==9
	"Se voc&ecirc; tiver um dispositivo Android, tente SWADroid";
#endif

const char *Txt_If_you_have_forgotten_your_password_ =
#if   L==1
	"Si usted ha olvidado su contrase&ntilde;a,"
	" se le enviar&aacute; una nueva"
	" a la direcci&oacute;n de correo"
	" que escribi&oacute; en su ficha personal.";	// Necessita traduccio
#elif L==2
	"Wenn Sie Ihr Passwort vergessen haben,"
	" eine neue wird"
	" die E-Mail von Ihnen angegebene Adresse"
	" in Ihren Profil gesendet werden vergessen.";
#elif L==3
	"If you have forgotten your password,"
	" a new one will be sent"
	" to the email address"
	" you entered in your record.";
#elif L==4
	"Si usted ha olvidado su contrase&ntilde;a,"
	" se le enviar&aacute; una nueva"
	" a la direcci&oacute;n de correo"
	" que escribi&oacute; en su ficha personal.";
#elif L==5
	"Si vous avez oubli&eacute; votre mot de passe,"
	" une nouvelle sera envoy&eacute;"
	" &agrave; l'adresse &eacute;lectronique"
	" que vous avez entr&eacute; dans votre carte.";
#elif L==6
	"Si usted ha olvidado su contrase&ntilde;a,"
	" se le enviar&aacute; una nueva"
	" a la direcci&oacute;n de correo"
	" que escribi&oacute; en su ficha personal.";	// Okoteve traduccin
#elif L==7
	"Se hai dimenticato la tua password,"
	" una nuova verr&agrave; inviata"
	" all'indirizzo email"
	" che hai inserito nella tua scheda.";
#elif L==8
	"Jesli nie pamietasz hasla,"
	" nowe zostanie wyslane na adres email,"
	" kt&oacute;ry wszedl w rekordzie.";
#elif L==9
	"Se voc&ecirc; tiver esquecido sua senha,"
	" uma nova ser&aacute; enviada"
	" para o email"
	" que voc&ecirc; inseriu em seu cart&atilde;o.";
#endif

const char *Txt_If_you_have_written_your_ID_nickname_or_email_correctly_ =
#if   L==1
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y el correo en su ficha es correcto,"
	" recibir&aacute; un mensaje con una nueva contrase&ntilde;a.";	// Necessita traduccio
#elif L==2
	"Wenn Sie Ihre @Benutzername, Email oder Ausweis-Nr.,"
	" korrekt geschrieben wurden, und die E-Mail-Adresse auf Ihren Profil korrekt ist,"
	" erhalten Sie eine E-Mail mit einem neuen Passwort.";
#elif L==3
	"If you have written your @nick, email or ID correctly,"
	" and the email address on your record is correct,"
	" you will receive an email with a new password.";
#elif L==4
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y el correo en su ficha es correcto,"
	" recibir&aacute; un mensaje con una nueva contrase&ntilde;a.";
#elif L==5
	"Si vous avez &eacute;crit votre @surnom, courrier ou num&eacute;ro d'identit&eacute; correctement,"
	" ainsi que l'adresse email sur votre carte est correct,"
	" vous recevrez un email avec un nouveau mot de passe.";
#elif L==6
	"Si usted ha escrito correctamente su @apodo, correo o ID (DNI/c&eacute;dula),"
	" y el correo en su ficha es correcto,"
	" recibir&aacute; un mensaje con una nueva contrase&ntilde;a.";	// Okoteve traduccin
#elif L==7
	"Se hai scritto il tuo nome utente, email o numero di carta d'identit&agrave; correttamente,"
	" e l'indirizzo email della tua scheda &egrave; corretto,"
	" riceverai una email con una nuova password.";
#elif L==8
	"Jesli napisales swoja nazwe lub nick poprawnie,"
	" a adres email na zapis jest poprawny,"
	" otrzymasz maila z nowym haslem.";
#elif L==9
	"Se voc&ecirc; ter escrito o suo alcunha, email ou n&ordm; identif. corretamente,"
	" bem como o endere&ccedil;o de email em seu cart&atilde;o est&aacute; correto,"
	" voc&ecirc; receber&aacute; um email com uma nova senha.";
#endif

const char *Txt_Image =
#if   L==1
	"Imatge";
#elif L==2
	"Abbild";
#elif L==3
	"Image";
#elif L==4
	"Imagen";
#elif L==5
	"Image";
#elif L==6
	"Ta'&atilde;nga";
#elif L==7
	"Immagine";
#elif L==8
	"Obraz";
#elif L==9
	"Imagem";
#endif

const char *Txt_Image_title_attribution =
#if   L==1
	"T&iacute;tol/atribuci&oacute; de la imatge";
#elif L==2
	"Bildtitel / Namensnennung";
#elif L==3
	"Image title/attribution";
#elif L==4
	"T&iacute;tulo/atribuci&oacute;n de la imagen";
#elif L==5
	"Titre/attribution de l'image";
#elif L==6
	"T&iacute;tulo/atribuci&oacute;n de la imagen";	// Okoteve traduccin
#elif L==7
	"Titolo/attribuzione delle immagine";
#elif L==8
	"Tytu&lstrok;/przypisanie obrazu";
#elif L==9
	"T&iacute;tulo/atribui&ccedil;&atilde;o da imagem";
#endif

const char *Txt_Image_not_found =
#if   L==1
	"Imatge no trobat";
#elif L==2
	"Abbild nicht gefunden";
#elif L==3
	"Image not found";
#elif L==4
	"Imagen no encontrada";
#elif L==5
	"Image non trouv&eacute;e";
#elif L==6
	"Ta'&atilde;nga nah&aacute;niri juhupapyre";
#elif L==7
	"Immagine non trovata";
#elif L==8
	"Obraz nie znaleziono";
#elif L==9
	"Imagem n&atilde;o encontrada";
#endif

const char *Txt_Import_questions =
#if   L==1
	"Importar preguntas";	// Necessita traduccio
#elif L==2
	"Import questions";	// Need bersetzung
#elif L==3
	"Import questions";
#elif L==4
	"Importar preguntas";
#elif L==5
	"Import questions";	// Besoin de traduction
#elif L==6
	"Importar preguntas";	// Okoteve traduccin
#elif L==7
	"Import questions";	// Bisogno di traduzione
#elif L==8
	"Import questions";	// Potrzebujesz tlumaczenie
#elif L==9
	"Import questions";	// Necessita de traduo
#endif

const char *Txt_Import_questions_from_XML_file =
#if   L==1
	"Importar preguntas de un archivo XML";	// Necessita traduccio
#elif L==2
	"Import questions from XML file";	// Need bersetzung
#elif L==3
	"Import questions from XML file";
#elif L==4
	"Importar preguntas de un archivo XML";
#elif L==5
	"Import questions from XML file";	// Besoin de traduction
#elif L==6
	"Importar preguntas de un archivo XML";	// Okoteve traduccin
#elif L==7
	"Import questions from XML file";	// Bisogno di traduzione
#elif L==8
	"Import questions from XML file";	// Potrzebujesz tlumaczenie
#elif L==9
	"Import questions from XML file";	// Necessita de traduo
#endif

const char *Txt_Imported_questions =
#if   L==1
	"Preguntas importadas";	// Necessita traduccio
#elif L==2
	"Imported questions";	// Need bersetzung
#elif L==3
	"Imported questions";
#elif L==4
	"Preguntas importadas";
#elif L==5
	"Imported questions";	// Besoin de traduction
#elif L==6
	"Preguntas importadas";	// Okoteve traduccin
#elif L==7
	"Imported questions";	// Bisogno di traduzione
#elif L==8
	"Imported questions";	// Potrzebujesz tlumaczenie
#elif L==9
	"Imported questions";	// Necessita de traduo
#endif

const char *Txt_SEARCH_X_in_Y =	// Search string "in" documentos, users, etc.
#if   L==1
	"en";
#elif L==2
	"im";
#elif L==3
	"in";
#elif L==4
	"en";
#elif L==5
	"dans";
#elif L==6
	"en";	// Okoteve traduccin
#elif L==7
	"in";
#elif L==8
	"w";
#elif L==9
	"em";
#endif

const char *Txt_In_a_type_of_group_with_single_enrollment_students_can_not_be_registered_in_more_than_one_group =
#if   L==1
	"En un tipo de grupo de adscripci&oacute;n &uacute;nica"
	" no es posible apuntar estudiantes a m&aacute;s de un grupo.";	// Necessita traduccio
#elif L==2
	"In a type of group with single enrollment,"
	" students can not be registered in more than one group.";	// Need bersetzung
#elif L==3
	"In a type of group with single enrollment,"
	" tudents can not be registered in more than one group.";
#elif L==4
	"En un tipo de grupo de adscripci&oacute;n &uacute;nica"
	" no es posible apuntar estudiantes a m&aacute;s de un grupo.";
#elif L==5
	"In a type of group with single enrollment,"
	" students can not be registered in more than one group.";	// Besoin de traduction
#elif L==6
	"En un tipo de grupo de adscripci&oacute;n &uacute;nica"
	" no es posible apuntar estudiantes a m&aacute;s de un grupo.";	// Okoteve traduccin
#elif L==7
	"In un tipo di gruppo a singola registrazione,"
	" non puoi registrare studenti in pi&ugrave; di un gruppo.";
#elif L==8
	"W rodzaju grupy z jednej rejestracji,"
	" nie mozesz rejestrowac uczni&oacute;w w wiecej niz jednej grupy.";
#elif L==9
	"In a type of group with single enrollment,"
	" students can not be registered in more than one group.";	// Necessita de traduo
#endif

const char *Txt_In_the_year_X_of_the_degree_Y_already_existed_a_course_with_the_name_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1
	"En el a&ntilde;o/semestre <strong>%s</strong> de la titulaci&oacute;n <strong>%s</strong>"
	" ya exist&iacute;a una asignatura con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"In the year/semester <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"In the year/semester <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";
#elif L==4
	"En el a&ntilde;o/semestre <strong>%s</strong> de la titulaci&oacute;n <strong>%s</strong>"
	" ya exist&iacute;a una asignatura con el nombre <strong>%s</strong>.";
#elif L==5
	"In the year/semester <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"En el a&ntilde;o/semestre <strong>%s</strong> de la titulaci&oacute;n <strong>%s</strong>"
	" ya exist&iacute;a una asignatura con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Nell'anno/semester <strong>%s</strong> di laurea <strong>%s</strong>"
	" gi&agrave; esiste un corso con il nome <strong>%s</strong>.";
#elif L==8
	"W roku/semestr <strong>%s</strong> stopnia <strong>%s</strong>"
	" istnialy juz kurs&oacute;w z nazwa <strong>%s</strong>.";
#elif L==9
	"In the year/semester <strong>%s</strong> of the degree <strong>%s</strong>"
	" already existed a course with the name <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_Incomplete_timetable_for_lack_of_space =
#if   L==1
	"Horari incomplet per falta d'espai";
#elif L==2
	"Unvollst&auml;ndige Studenplan f&uuml;r Platzmangel";
#elif L==3
	"Incomplete timetable for lack of space";
#elif L==4
	"Horario incompleto por falta de espacio";
#elif L==5
	"Horaire incompl&egrave;te par manque de place";
#elif L==6
	"Horario incompleto por falta de espacio";	// Okoteve traduccin
#elif L==7
	"Orario incompleto per mancanza di spazio";
#elif L==8
	"Niekompletne harmonogram brak miejsca";
#elif L==9
	"Hor&aacute;rio incompleto por falta de espa&ccedil;o";
#endif

const char *Txt_Increase_level_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Aumentar nivel de %s";		// Necessita traduccio
#elif L==2
	"Increase level of %s";		// Need bersetzung
#elif L==3
	"Increase level of %s";
#elif L==4
	"Aumentar nivel de %s";
#elif L==5
	"Increase level of %s";		// Besoin de traduction
#elif L==6
	"Aumentar nivel de %s";		// Okoteve traduccin
#elif L==7
	"Incrementare livello di %s";
#elif L==8
	"Wzrost poziomu %s";
#elif L==9
	"Increase level of %s";		// Necessita de traduo
#endif

const char *Txt_Indicators =
#if   L==1
	"Indicadors";
#elif L==2
	"Indikatoren";
#elif L==3
	"Indicators";
#elif L==4
	"Indicadores";
#elif L==5
	"Indicateurs";
#elif L==6
	"Indicadores";	// Okoteve traduccin
#elif L==7
	"Indicatori";
#elif L==8
	"Wska&zacute;niki";
#elif L==9
	"Indocadores";
#endif

const char *Txt_Indicators_of_courses =
#if   L==1
	"Indicadors d'assignatures";
#elif L==2
	"Anzeigen von Kursen";
#elif L==3
	"Indicators of courses";
#elif L==4
	"Indicadores de asignaturas";
#elif L==5
	"Indicateurs de mati&egrave;res";
#elif L==6
	"Indicadores de asignaturas";	// Okoteve traduccin
#elif L==7
	"Indicatori di corsi";
#elif L==8
	"Wska&zacute;niki kurs&oacute;w";
#elif L==9
	"Indicadores de disciplinas";
#endif

const char *Txt_INFO_TITLE[Inf_NUM_INFO_TYPES] =
   {
   // Inf_INTRODUCTION
#if   L==1
	 "Informaci&oacute;"
#elif L==2
	 "Information"
#elif L==3
	 "Information"
#elif L==4
	 "Informaci&oacute;n"
#elif L==5
	 "Information"
#elif L==6
	 "Informaci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Informazione"
#elif L==8
	 "Informacja"
#elif L==9
	 "Informa&ccedil;&atilde;o"
#endif
	 ,
	 // Inf_TEACHING_GUIDE
#if   L==1
	 "Guia docent"
#elif L==2
	 "Hinweise zur Kurs"
#elif L==3
	 "Teaching guide"
#elif L==4
	 "Gu&iacute;a docente"
#elif L==5
	 "Guide p&eacute;dagogique"
#elif L==6
	 "Gu&iacute;a docente"	// Okoteve traduccin
#elif L==7
	 "Guida didattica"
#elif L==8
	 "Nauczanie przypomnienie"
#elif L==9
	 "Guia de ensino"
#endif
	 ,
	 // Inf_LECTURES
#if   L==1
	 "Programa teoria"
#elif L==2
	 "Studienplan (Vorlesungen)"
#elif L==3
	 "Syllabus (lectures)"
#elif L==4
	 "Programa teor&iacute;a"
#elif L==5
	 "Plan th&eacute;orie"
#elif L==6
	 "Programa teor&iacute;a"	// Okoteve traduccin
#elif L==7
	 "Programma teoria"
#elif L==8
	 "Syllabus (wyklady)"
#elif L==9
	 "Programa teoria"
#endif
	 ,
	 // Inf_PRACTICALS
#if   L==1
	 "Programa pr&agrave;ctiques"
#elif L==2
	 "Studienplan (&Uuml;bungen)"
#elif L==3
	 "Syllabus (practicals)"
#elif L==4
	 "Programa pr&aacute;cticas"
#elif L==5
	 "Plan pratique"
#elif L==6
	 "Programa pr&aacute;cticas"	// Okoteve traduccin
#elif L==7
	 "Programma pratica"
#elif L==8
	 "Syllabus (practicals)"
#elif L==9
	 "Programa pr&aacute;tica"
#endif
	 ,
	 // Inf_BIBLIOGRAPHY
#if   L==1
	 "Bibliografia"
#elif L==2
	 "Literatur"
#elif L==3
	 "Bibliography"
#elif L==4
	 "Bibliograf&iacute;a"
#elif L==5
	 "Bibliographie"
#elif L==6
	 "Arandukapurupyre"
#elif L==7
	 "Bibliografia"
#elif L==8
	 "Bibliografia"
#elif L==9
	 "Bibliografia"
#endif
	 ,
	 // Inf_FAQ
#if   L==1
	 "FAQ"
#elif L==2
	 "FAQ"
#elif L==3
	 "FAQ"
#elif L==4
	 "FAQ"
#elif L==5
	 "FAQ"
#elif L==6
	 "FAQ"	// Okoteve traduccin
#elif L==7
	 "FAQ"
#elif L==8
	 "FAQ"
#elif L==9
	 "FAQ"
#endif
	 ,
	 // Inf_LINKS
#if   L==1
	 "Enlla&ccedil;os"
#elif L==2
	 "Links"
#elif L==3
	 "Links"
#elif L==4
	 "Enlaces"
#elif L==5
	 "Liens"
#elif L==6
	 "Ambu&eacute;re joaju"
#elif L==7
	 "Link"
#elif L==8
	 "Linki"
#elif L==9
	 "Liga&ccedil;&otilde;es"
#endif
	 ,
	 // Inf_ASSESSMENT
#if   L==1
	 "Sistema d'avaluaci&oacute;"
#elif L==2
	 "Bewertungssystem"
#elif L==3
	 "Assessment system"
#elif L==4
	 "Sistema de evaluaci&oacute;n"
#elif L==5
	 "Syst&egrave;me d'&eacute;valuation"
#elif L==6
	 "Sistema de evaluaci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Sistema di valutazione"
#elif L==8
	 "System oceniania"
#elif L==9
	 "Sistema de avalia&ccedil;&atilde;o"
#endif
	};

const char *Txt_INFO_SRC_FULL_TEXT[Inf_NUM_INFO_SOURCES] =
   {
#if   L==1
	 "No ofrecer ninguna informaci&oacute;n"	// Necessita traduccio
#elif L==2
	 "Geben keine Auskunft"
#elif L==3
	 "Do not give any information"
#elif L==4
	 "No ofrecer ninguna informaci&oacute;n"
#elif L==5
	 "Ne pas donner d'information"
#elif L==6
	 "No ofrecer ninguna informaci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Non dare alcuna informazione"
#elif L==8
	 "Nie podawaj zadnych informacji"
#elif L==9
	 "N&atilde;o d&ecirc; nenhuma informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "Editor interno"	// Necessita traduccio
#elif L==2
	 "Interner Editor"
#elif L==3
	 "Internal editor"
#elif L==4
	 "Editor interno"
#elif L==5
	 "&Eacute;diteur interne"
#elif L==6
	 "Editor interno"	// Okoteve traduccin
#elif L==7
	 "Editor interno"
#elif L==8
	 "Wewnetrzny edytor"
#elif L==9
	 "Interno editor"
#endif
	 ,
#if   L==1
	 "Text sense format"
#elif L==2
	 "Plain Text"
#elif L==3
	 "Plain text"
#elif L==4
	 "Texto sin formato"
#elif L==5
	 "Texte brut"
#elif L==6
	 "Texto sin formato"	// Okoteve traduccin
#elif L==7
	 "Testo normale"
#elif L==8
	 "Zwykly tekst"
#elif L==9
	 "Texto plano"
#endif
	 ,
#if   L==1
	 "Text enriquit"
#elif L==2
	 "Rich Text"
#elif L==3
	 "Rich text"
#elif L==4
	 "Texto enriquecido"
#elif L==5
	 "Texte enrichi"
#elif L==6
	 "Texto enriquecido"	// Okoteve traduccin
#elif L==7
	 "Testo formattato"
#elif L==8
	 "Rich text"		// Potrzebujesz tlumaczenie
#elif L==9
	 "Rich text"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "P&aacute;gina web alojada aqu&iacute;"	// Necessita traduccio
#elif L==2
	 "Webseite gehostet hier"
#elif L==3
	 "Web page hosted here"
#elif L==4
	 "P&aacute;gina web alojada aqu&iacute;"
#elif L==5
	 "Page web h&eacute;berg&eacute;e ici"
#elif L==6
	 "P&aacute;gina web alojada aqu&iacute;"	// Okoteve traduccin
#elif L==7
	 "Pagina web ospitata qui"
#elif L==8
	 "Strona internetowa hostowana tutaj"
#elif L==9
	 "P&aacute;gina da Web hospedada aqui"
#endif
	 ,
#if   L==1
	 "P&aacute;gina web alojada en otro servidor"	// Necessita traduccio
#elif L==2
	 "Webseite auf einem anderen Server gehostet"
#elif L==3
	 "Web page hosted on another server"
#elif L==4
	 "P&aacute;gina web alojada en otro servidor"
#elif L==5
	 "Page Web h&eacute;berg&eacute;e sur un autre serveur"
#elif L==6
	 "P&aacute;gina web alojada en otro servidor"	// Okoteve traduccin
#elif L==7
	 "Pagina Web ospitata su un altro server"
#elif L==8
	 "Strona internetowa umieszczona na innym serwerze"
#elif L==9
	 "P&aacute;gina da Web hospedada em outro servidor"
#endif
	};

const char *Txt_INFO_SRC_HELP[Inf_NUM_INFO_SOURCES] =
	{
	NULL,
#if   L==1
	"editor integrado, solo para los temarios de teor&iacute;a y pr&aacute;cticas"// Necessita traduccio
#elif L==2
	"Interner Editor, nur f&uuml;r Studienplan (Vorlesung und &Uuml;bungen)"
#elif L==3
	"internal editor, only for syllabus (lectures and practicals)"
#elif L==4
	"editor integrado, solo para los temarios de teor&iacute;a y pr&aacute;cticas"
#elif L==5
	"&eacute;diteur interne, que pour les plans (th&eacute;orie et pratique)"
#elif L==6
	"editor integrado, solo para los temarios de teor&iacute;a y pr&aacute;cticas"// Okoteve traduccin
#elif L==7
	"editor interno, solo per programmi di teoria e di pratica"
#elif L==8
	"Wewnetrzny edytor, tylko program (wyklady i practicals)"
#elif L==9
	"interno editor, apenas para programas de teoria y pr&aacute;tica"
#endif
	,
#if   L==1
	"editor de text sense format"
#elif L==2
	"Plain Text Editor"
#elif L==3
	"plain text editor"
#elif L==4
	"editor de texto sin formato"
#elif L==5
	"&eacute;diteur de texte simple"
#elif L==6
	"editor de texto sin formato"		// Okoteve traduccin
#elif L==7
	"editor di testo normale"
#elif L==8
	"edytor tekstu zwyk&lstrok;y"
#elif L==9
	"editor de texto simples"
#endif
	,
#if   L==1
	"editor de text enriquit"
#elif L==2
	"Rich Text Editor"
#elif L==3
	"rich text editor"
#elif L==4
	"editor de texto enriquecido"
#elif L==5
	"&eacute;diteur de texte enrichi"
#elif L==6
	"editor de texto enriquecido"	// Okoteve traduccin
#elif L==7
	"editor di testo formattato"
#elif L==8
	"edytor tekstu sformatowany"
#elif L==9
	"editor de texto rico"
#endif
	,
#if   L==1
	"env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"// Necessita traduccio
#elif L==2
	"senden Sie eine HTML-Datei, oder als ZIP-Datei mit einer Datei index.html / index.htm und Bild-Dateien"
#elif L==3
	"send an HTML file, or a ZIP file containing a file index.html / index.htm and image files"
#elif L==4
	"env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"
#elif L==5
	"envoyer un fichier HTML, ou un fichier ZIP contenant un fichier index.html / index.htm et les fichiers d'images"
#elif L==6
	"env&iacute;e un archivo HTML, o bien un archivo ZIP que contenga un archivo index.html/index.htm y archivos con im&aacute;genes"// Okoteve traduccin
#elif L==7
	"inviare un file HTML, o un file ZIP contenente un file index.html / index.htm e file di immagine"
#elif L==8
	"wyslac plik HTML lub plik ZIP zawierajacy plik index.html / index.htm i pliki graficzne"
#elif L==9
	"enviar um arquivo HTML, ou um arquivo ZIP que cont&eacute;m um arquivo index.html / index.htm e limas da imagem"
#endif
	,
#if   L==1
	"escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"	// Necessita traduccio
#elif L==2
	"Geben Sie die Adresse einer Web-Seite geleitet werden, der sich auf einem anderen Server"
#elif L==3
	"enter the address of a web page that is hosted on another server"
#elif L==4
	"escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"
#elif L==5
	"entrer l'adresse d'une page Web qui est h&eacute;berg&eacute;e sur un autre serveur"
#elif L==6
	"escriba la direcci&oacute;n web de una p&aacute;gina que se encuentre alojada en otro servidor"	// Okoteve traduccin
#elif L==7
	"immettere l'indirizzo di una pagina web che &egrave; ospitata su un altro server"
#elif L==8
	"wprowadzic adres strony internetowej, kt&oacute;ra znajduje sie na innym serwerze"
#elif L==9
	"introduza o endere&ccedil;o de uma p&aacute;gina Web que est&aacute; hospedada em outro servidor"
#endif
	};

const char *Txt_INFO_SRC_SHORT_TEXT[Inf_NUM_INFO_SOURCES] =
   {
#if   L==1
	 "Ninguna informaci&oacute;n"	// Necessita traduccio
#elif L==2
	 "Keine Angaben"
#elif L==3
	 "No information"
#elif L==4
	 "Ninguna informaci&oacute;n"
#elif L==5
	 "Aucune information"
#elif L==6
	 "Ninguna informaci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Nessuna informazione"
#elif L==8
	 "Brak informacji"
#elif L==9
	 "Sem informa&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "Editor interno"	// Necessita traduccio
#elif L==2
	 "Interner Editor"
#elif L==3
	 "Internal editor"
#elif L==4
	 "Editor interno"
#elif L==5
	 "&Eacute;diteur interne"
#elif L==6
	 "Editor interno"	// Okoteve traduccin
#elif L==7
	 "Editor interno"
#elif L==8
	 "Wewnetrzny edytor"
#elif L==9
	 "Interno editor"
#endif
	 ,
#if   L==1
	 "Text sense format"
#elif L==2
	 "Plain Text"
#elif L==3
	 "Plain text"
#elif L==4
	 "Texto sin formato"
#elif L==5
	 "Plain text"
#elif L==6
	 "Texto sin formato"	// Okoteve traduccin
#elif L==7
	 "Testo normale"
#elif L==8
	 "Zwykly tekst"
#elif L==9
	 "Texto plano"
#endif
	 ,
#if   L==1
	 "Text enriquit"
#elif L==2
	 "Rich Text"
#elif L==3
	 "Rich text"
#elif L==4
	 "Texto enriquecido"
#elif L==5
	 "Texte enrichi"
#elif L==6
	 "Texto enriquecido"	// Okoteve traduccin
#elif L==7
	 "Testo formattato"
#elif L==8
	 "Rich text"		// Potrzebujesz tlumaczenie
#elif L==9
	 "Rich text"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "P&aacute;gina interna"	// Necessita traduccio
#elif L==2
	 "Interne Webseite"
#elif L==3
	 "Internal page"
#elif L==4
	 "P&aacute;gina interna"
#elif L==5
	 "Page interne"
#elif L==6
	 "P&aacute;gina interna"	// Okoteve traduccin
#elif L==7
	 "Pagina interna"
#elif L==8
	 "Strona wewnetrzna"
#elif L==9
	 "Interna p&aacute;gina"
#endif
	 ,
#if   L==1
	 "P&aacute;gina externa"	// Necessita traduccio
#elif L==2
	 "Externe Webseite"
#elif L==3
	 "External page"
#elif L==4
	 "P&aacute;gina externa"
#elif L==5
	 "Page externe"
#elif L==6
	 "P&aacute;gina externa"	// Okoteve traduccin
#elif L==7
	 "Pagina esterna"
#elif L==8
	 "Strona zewnetrzna"
#elif L==9
	 "Externa p&aacute;gina"
#endif
	};

const char *Txt_Informants =
#if   L==1
	"Informadors";
#elif L==2
	"Informanten";
#elif L==3
	"Informants";
#elif L==4
	"Informantes";
#elif L==5
	"Informateurs";
#elif L==6
	"Informantes";	// Okoteve traduccin
#elif L==7
	"Informatori";
#elif L==8
	"informator&oacute;w";
#elif L==9
	"Informantes";
#endif

const char *Txt_Inline_math =
#if   L==1
	"F&oacute;rmula";
#elif L==2
	"Inline Mathematik";
#elif L==3
	"Inline math";
#elif L==4
	"F&oacute;rmula";
#elif L==5
	"Formule";
#elif L==6
	"F&oacute;rmula";	// Okoteve traduccin
#elif L==7
	"Matematica inline";
#elif L==8
	"Jazda matematyki";
#elif L==9
	"F&oacute;rmula";
#endif

const char *Txt_Institution =
#if   L==1
	"Instituci&oacute;";
#elif L==2
	"Hochschule";
#elif L==3
	"Institution";
#elif L==4
	"Instituci&oacute;n";
#elif L==5
	"&Eacute;tablissement";
#elif L==6
	"Mbo'ehao";
#elif L==7
	"Istituzione";
#elif L==8
	"Instytucji";
#elif L==9
	"Institu&ccedil;&atilde;o";
#endif

const char *Txt_institution =
#if   L==1
	"instituci&oacute;";
#elif L==2
	"Hochschule";
#elif L==3
	"institution";
#elif L==4
	"instituci&oacute;n";
#elif L==5
	"&eacute;tablissement";
#elif L==6
	"mbo'ehao";
#elif L==7
	"istituzione";
#elif L==8
	"instytucji";
#elif L==9
	"institu&ccedil;&atilde;o";
#endif

const char *Txt_Institution_centre_and_department =
#if   L==1
	"Instituci&oacute;, centre i departament";
#elif L==2
	"Hochschule, Lehrinstitut und Abteilung";
#elif L==3
	"Institution, centre and department";
#elif L==4
	"Instituci&oacute;n, centro y departamento";
#elif L==5
	"&Eacute;tablissement, centre et d&eacute;partement";
#elif L==6
	"Mbo'ehao, mbo'ehao, peh&#7869;";
#elif L==7
	"Istituzione, centro e dipartimento";
#elif L==8
	"Instytucji, centrum i wydzia&lstrok;";
#elif L==9
	"Institu&ccedil;&atilde;o, centro e departamento";
#endif

const char *Txt_Institution_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Instituci&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2
	"Hochschule <strong>%s</strong> entfernt.";
#elif L==3
	"Institution <strong>%s</strong> removed.";
#elif L==4
	"Instituci&oacute;n <strong>%s</strong> eliminada.";
#elif L==5
	"Institution <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6
	"Instituci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7
	"Istituzione <strong>%s</strong> rimossa.";
#elif L==8
	"Instytucja <strong>%s</strong> usunieta.";
#elif L==9
	"Institution <strong>%s</strong> removed.";		// Necessita de traduo
#endif


const char *Txt_INSTITUTION_STATUS[Ins_NUM_STATUS_TXT] =
   {
#if   L==1	// Ins_STATUS_UNKNOWN
	 "Desconegudo"
#elif L==2
	 "Unbekannt"
#elif L==3
	 "Unknown"
#elif L==4
	 "Desconocido"
#elif L==5
	 "Inconnu"
#elif L==6
	 "Desconocido"			// Okoteve traduccin
#elif L==7
	 "Sconosciuto"
#elif L==8
	 "Nieznany"
#elif L==9
	 "Desconhecido"
#endif
	 ,
#if   L==1	// Ins_STATUS_ACTIVE
	 "Activa"
#elif L==2
	 "Aktive"
#elif L==3
	 "Active"
#elif L==4
	 "Activa"
#elif L==5
	 "Actif"
#elif L==6
	 "Activa"	// Okoteve traduccin
#elif L==7
	 "Attiva"
#elif L==8
	 "Aktywne"
#elif L==9
	 "Ativa"
#endif
	 ,
#if   L==1	// Ins_STATUS_PENDING
	 "Pendent"
#elif L==2
	 "Offen"
#elif L==3
	 "Pending"
#elif L==4
	 "Pendiente"
#elif L==5
	 "En attente"
#elif L==6
	 "Pendiente"	// Okoteve traduccin
#elif L==7
	 "In attesa"
#elif L==8
	 "Do czasu"
#elif L==9
	 "Pendente"
#endif
	 ,
#if   L==1	// Ins_STATUS_REMOVED
	 "Eliminada"
#elif L==2
	 "Entfernt"
#elif L==3
	 "Removed"
#elif L==4
	 "Eliminada"
#elif L==5
	 "Supprim&eacute;"
#elif L==6
	 "Eliminada"	// Okoteve traduccin
#elif L==7
	 "Rimossa"
#elif L==8
	 "Usuniete"
#elif L==9
	 "Suprimida"
#endif
	};

const char *Txt_Institutional_code =
#if   L==1
	"Codi institucional";
#elif L==2
	"Institutionelle Code";
#elif L==3
	"Institutional code";
#elif L==4
	"C&oacute;digo institucional";
#elif L==5
	"Code institutionnel";
#elif L==6
	"C&oacute;digo institucional";	// Okoteve traduccin
#elif L==7
	"Codice istituzionale";
#elif L==8
	"Kod instytucjonalna";
#elif L==9
	"C&oacute;digo institutional";
#endif

const char *Txt_Institutional_BR_code =
#if   L==1
	"Codi<br />instituc.";
#elif L==2
	"Institutionelle<br />Code";
#elif L==3
	"Institut.<br />code";
#elif L==4
	"C&oacute;digo<br />instituc.";
#elif L==5
	"Code<br />institut.";
#elif L==6
	"C&oacute;digo<br />instituc.";	// Okoteve traduccin
#elif L==7
	"Codice<br />istituz.";
#elif L==8
	"Kod<br />instytuc.";
#elif L==9
	"C&oacute;digo<br />instituc.";
#endif

const char *Txt_Institutions =
#if   L==1
	"Institucions";
#elif L==2
	"Hochschulen";
#elif L==3
	"Institutions";
#elif L==4
	"Instituciones";
#elif L==5
	"&Eacute;tablissements";
#elif L==6
	"Mbo'ehao";
#elif L==7
	"Istituzioni";
#elif L==8
	"Instytucje";
#elif L==9
	"Institu&ccedil;&otilde;es";
#endif

const char *Txt_Institutions_ABBREVIATION =
#if   L==1
	"Instit.";
#elif L==2
	"Hochsc.";
#elif L==3
	"Instit.";
#elif L==4
	"Instit.";
#elif L==5
	"&Eacute;tabl.";
#elif L==6
	"Mbo'ehao";
#elif L==7
	"Istit.";
#elif L==8
	"Instyt.";
#elif L==9
	"Instit.";
#endif

const char *Txt_institutions =
#if   L==1
	"institucions";
#elif L==2
	"Hochschulen";
#elif L==3
	"institutions";
#elif L==4
	"instituciones";
#elif L==5
	"&eacute;tablissements";
#elif L==6
	"mbo'ehao";
#elif L==7
	"istituzioni";
#elif L==8
	"instytucje";
#elif L==9
	"institu&ccedil;&otilde;es";
#endif

const char *Txt_INSTITUTIONS_HELP_ORDER[2] =
   {
#if   L==1
	 "Ordenar por instituci&oacute;n"	// Necessita traduccio
#elif L==2
	 "Nach Hochschule sortieren"
#elif L==3
	 "Sort by institution"
#elif L==4
	 "Ordenar por instituci&oacute;n"
#elif L==5
	 "Trier par &eacute;tablissement"
#elif L==6
	 "Ordenar por instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Ordina per istituzione"
#elif L==8
	 "Sortuj wedlug instytucji"
#elif L==9
	 "Classificar por institui&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "Ordenar por n&ordm; de usuarios"	// Necessita traduccio
#elif L==2
	 "Nach Anzahl der Benutzer sortieren"
#elif L==3
	 "Sort by no. of users"
#elif L==4
	 "Ordenar por n&ordm; de usuarios"
#elif L==5
	 "Trier par nombre d'utilisateurs"
#elif L==6
	 "Ordenar por n&ordm; de usuarios"	// Okoteve traduccin
#elif L==7
	 "Ordina per numero di utenti"
#elif L==8
	 "Sortowanie wedlug liczby uzytkownikow"
#elif L==9
	 "Classificar por n&ordm; de utilizadores"
#endif
	};

const char *Txt_Institutions_by_number_of_centres =
#if   L==1
	"Institucions segons nombre de centres";
#elif L==2
	"Hochschulen nach der Anzahl der Lehrinstitute";
#elif L==3
	"Institutions by number of centres";
#elif L==4
	"Instituciones seg&uacute;n n&uacute;mero de centros";
#elif L==5
	"&Eacute;tablissements par nombre de centres";
#elif L==6
	"Instituciones seg&uacute;n n&uacute;mero de centros";	// Okoteve traduccin
#elif L==7
	"Istituzioni per numero di centri";
#elif L==8
	"Instytucje wed&lstrok;ug liczby o&sacute;rodk&oacute;w";
#elif L==9
	"Institu&ccedil;&otilde;es por n&uacute;mero de centros";
#endif

const char *Txt_Institutions_by_number_of_degrees =
#if   L==1
	"Institucions segons nombre de titulacions";
#elif L==2
	"Hochschulen nach der Anzahl der Studiengang";
#elif L==3
	"Institutions by number of degrees";
#elif L==4
	"Instituciones seg&uacute;n n&uacute;mero de titulaciones";
#elif L==5
	"&Eacute;tablissements par nombre d'&eacute;tudes";
#elif L==6
	"Instituciones seg&uacute;n n&uacute;mero de titulaciones";	// Okoteve traduccin
#elif L==7
	"Istituzioni per numero di lauree";
#elif L==8
	"Instytucje wed&lstrok;ug liczby stopni";
#elif L==9
	"Institu&ccedil;&otilde;es por n&uacute;mero de titula&ccedil;&otilde;es";
#endif

const char *Txt_Institutions_by_number_of_courses =
#if   L==1
	"Institucions segons nombre de assignatures";
#elif L==2
	"Hochschulen nach der Anzahl der Kursen";
#elif L==3
	"Institutions by number of courses";
#elif L==4
	"Instituciones seg&uacute;n n&uacute;mero de asignaturas";
#elif L==5
	"&Eacute;tablissements par nombre de mati&egrave;res";
#elif L==6
	"Instituciones seg&uacute;n n&uacute;mero de asignaturas";	// Okoteve traduccin
#elif L==7
	"Istituzioni per numero di corsi";
#elif L==8
	"Instytucje wed&lstrok;ug liczby kurs&otilde;w";
#elif L==9
	"Institu&ccedil;&otilde;es por n&uacute;mero de disciplinas";
#endif

const char *Txt_Institutions_by_number_of_users_in_courses =
#if   L==1
	"Institucions segons nombre d'usuaris en assignatures";
#elif L==2
	"Hochschulen nach der Anzahl der Benutzer in Kurse";
#elif L==3
	"Institutions by number of users in courses";
#elif L==4
	"Instituciones seg&uacute;n n&uacute;mero de usuarios en asignaturas";
#elif L==5
	"&Eacute;tablissements par nombre d'utilisateurs dans mati&egrave;res";
#elif L==6
	"Instituciones seg&uacute;n n&uacute;mero de usuarios en asignaturas";	// Okoteve traduccin
#elif L==7
	"Istituzioni per numero di utenti in corsi";
#elif L==8
	"Instytucje wed&lstrok;ug liczby u&zdot;ytkownik&oacute;w w kurs&otilde;w";
#elif L==9
	"Institu&ccedil;&otilde;es por n&uacute;mero de usu&aacute;rios em disciplinas";
#endif

const char *Txt_Institutions_by_number_of_users_who_claim_to_belong_to_them =
#if   L==1
	"Institucions segons nombre d'usuaris que diuen pert&agrave;nyer-hi";
#elif L==2
	"Hochschulen nach der Anzahl der Benutzer, die behaupten, zu ihnen geh&ouml;ren";
#elif L==3
	"Institutions by number of users who claim to belong to them";
#elif L==4
	"Instituciones seg&uacute;n n&uacute;mero de usuarios que dicen pertenecer a ellas";
#elif L==5
	"&Eacute;tablissements par nombre d'utilisateurs qui pr&eacute;tendent leur appartiennent";
#elif L==6
	"Instituciones seg&uacute;n n&uacute;mero de usuarios que dicen pertenecer a ellas";	// Okoteve traduccin
#elif L==7
	"Istituzioni per numero di utenti che affermano di appartenere a loro";
#elif L==8
	"Instytucje wed&lstrok;ug liczby u&zdot;ytkownik&oacute;w, kt&oacute;rzy twierdz&aogon;, &zdot;e nale&zdot;&aogon; do nich";
#elif L==9
	"Institu&ccedil;&otilde;es por n&uacute;mero de usu&aacute;rios que afirmam pertencer a elas";
#endif

const char *Txt_Institutions_of_COUNTRY_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Institucions de %s";
#elif L==2
	"Hochschulen von %s";
#elif L==3
	"Institutions of %s";
#elif L==4
	"Instituciones de %s";
#elif L==5
	"&Eacute;tablissements de %s";
#elif L==6
	"Instituciones de %s";	// Okoteve traduccin
#elif L==7
	"Istituzioni di %s";
#elif L==8
	"Instytucje %s";
#elif L==9
	"Institu&ccedil;&otilde;es de %s";
#endif

const char *Txt_INSTITUTIONS_ORDER[2] =
   {
#if   L==1
	 "Instituci&oacute;"
#elif L==2
	 "Hochschule"
#elif L==3
	 "Institution"
#elif L==4
	 "Instituci&oacute;n"
#elif L==5
	 "&Eacute;tablissement"
#elif L==6
	 "Mbo'ehao"
#elif L==7
	 "Istituzione"
#elif L==8
	 "Instytucji"
#elif L==9
	 "Institui&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "Usuaris de la instituci&oacute;"
#elif L==2
	 "Benutzer der Hochschule"
#elif L==3
	 "Users of the institution"
#elif L==4
	 "Usuarios de la instituci&oacute;n"
#elif L==5
	 "Utilisateurs du &eacute;tablissement"
#elif L==6
	 "Usuarios de la instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Utenti della istituzione"
#elif L==8
	 "U&zdot;ytkownicy instytucji"
#elif L==9
	 "Utilizadores da institu&ccedil;&atilde;o"
#endif
	};

const char *Txt_Institutions_with_pending_centres =
#if   L==1
	"Institucions amb centres pendents d'activar";
#elif L==2
	"Hochschulen mit anh&auml;ngig Lehrinstitute";
#elif L==3
	"Institutions with pending centres";
#elif L==4
	"Instituciones con centros pendientes de activar";
#elif L==5
	"&Eacute;tablissements avec des centres en attente";
#elif L==6
	"Instituciones con centros pendientes de activar";	// Okoteve traduccin
#elif L==7
	"Istituzioni con centri in sospeso";
#elif L==8
	"Instytucji z centra oczekuj&aogon;cy";
#elif L==9
	"Institu&ccedil;&otilde;es com centros pendentes";
#endif

const char *Txt_Integer_number =
#if   L==1
	"Nombre enter";
#elif L==2
	"Ganze Zahl";
#elif L==3
	"Integer number";
#elif L==4
	"N&ordm; entero";
#elif L==5
	"Nombre entier";
#elif L==6
	"N&ordm; entero";	// Okoteve traduccin
#elif L==7
	"Numero intero";
#elif L==8
	"Liczba calkowita";
#elif L==9
	"N&ordm; inteiro";
#endif

const char *Txt_Internal_code =
#if   L==1
	"Codi intern";
#elif L==2
	"Interner Code";
#elif L==3
	"Internal code";
#elif L==4
	"C&oacute;digo interno";
#elif L==5
	"Code interne";
#elif L==6
	"C&oacute;digo interno";	// Okoteve traduccin
#elif L==7
	"Codice interno";
#elif L==8
	"Kod wewn&eogon;trzny";
#elif L==9
	"C&oacute;digo interno";
#endif

const char *Txt_IP =		// Internet Protocol address
#if   L==1
	"IP";
#elif L==2
	"IP";
#elif L==3
	"IP";
#elif L==4
	"IP";
#elif L==5
	"IP";
#elif L==6
	"IP";	// Okoteve traduccin
#elif L==7
	"IP";
#elif L==8
	"IP";
#elif L==9
	"IP";
#endif

const char *Txt_It_is_mandatory_to_choose_a_group =
#if   L==1
	"Elegir grupo es obligatorio";		// Necessita traduccio
#elif L==2
	"Es muss eine Gruppe ausgew&auml;hlt werden";
#elif L==3
	"It is mandatory to choose a group";
#elif L==4
	"Elegir grupo es obligatorio";
#elif L==5
	"Il est obligatoire de choisir un groupe";
#elif L==6
	"Elegir grupo es obligatorio";		// Okoteve traduccin
#elif L==7
	"E 'obbligatorio scegliere un gruppo";
#elif L==8
	"Jest to obowiazkowe, aby wybrac grupe";
#elif L==9
	"Escolher um grupo &eacute; obrigat&oacute;rio";
#endif

const char *Txt_It_is_optional_to_choose_a_group =
#if   L==1
	"Elegir grupo es voluntario";		// Necessita traduccio
#elif L==2
	"Die Ausw&auml;hl einer Gruppe ist freiwillig";
#elif L==3
	"It is optional to choose a group";
#elif L==4
	"Elegir grupo es voluntario";
#elif L==5
	"Il est volontaire de choisir un groupe";
#elif L==6
	"Elegir grupo es voluntario";		// Okoteve traduccin
#elif L==7
	"E' facoltativo scegliere un gruppo";
#elif L==8
	"Opcjonalnie mozna wybrac grupe";
#elif L==9
	"Escolher um grupo &eacute; volunt&aacute;rio";
#endif

const char *Txt_Its_me =
#if   L==1
	"S&oacute;c jo!";
#elif L==2
	"Da ich bin!";
#elif L==3
	"It's me!";
#elif L==4
	"&iexcl;Soy yo!";
#elif L==5
	"C'est moi!";
#elif L==6
	"&iexcl;Soy yo!";	// Okoteve traduccin
#elif L==7
	"Sono io!";
#elif L==8
	"To ja!";
#elif L==9
	"Sou eu!";
#endif

const char *Txt_Language =
#if   L==1
	"Idioma (language)";
#elif L==2
	"Sprache (language)";
#elif L==3
	"Language";
#elif L==4
	"Idioma (language)";
#elif L==5
	"Langue (language)";
#elif L==6
	"&Ntilde;e'&#7869; (language)";
#elif L==7
	"Lingua (language)";
#elif L==8
	"J&eogon;zyk (language)";
#elif L==9
	"Lingua (language)";
#endif

const char *Txt_Last_clicks =
#if   L==1
	"&Uacute;ltims clics";
#elif L==2
	"Letzte Klicks";
#elif L==3
	"Last clicks";
#elif L==4
	"&Uacute;ltimos clics";
#elif L==5
	"Derni&egrave;re clics";
#elif L==6
	"&Uacute;ltimos clics";	// Okoteve traduccin
#elif L==7
	"Ultimi clic";
#elif L==8
	"Ostatnie dwa klikni&eogon;cia";
#elif L==9
	"&Uacute;ltimos cliques";
#endif

const char *Txt_Last_clicks_in_real_time =
#if   L==1
	"&Uacute;ltims clics en temps real";
#elif L==2
	"Letzte Klicks in Echtzeit";
#elif L==3
	"Last clicks in real time";
#elif L==4
	"&Uacute;ltimos clics en tiempo real";
#elif L==5
	"Derni&egrave;re clics en temps r&eacute;el";
#elif L==6
	"&Uacute;ltimos clics en tiempo real";	// Okoteve traduccin
#elif L==7
	"Ultimi clic in tempo reale";
#elif L==8
	"Ostatnie dwa klikni&eogon;cia w czasie rzeczywistym";
#elif L==9
	"&Uacute;ltimos cliques em tempo real";
#endif

const char *Txt_LAYOUT_SIDE_COLUMNS[4] =
   {
#if   L==1
	 "Solo zona central"	// Necessita traduccio
#elif L==2
	 "Nur mittleren Bereich anzeigen"
#elif L==3
	 "Display central area only"
#elif L==4
	 "Solo zona central"
#elif L==5
	 "Display central area only"	// Besoin de traduction
#elif L==6
	 "Solo zona central"	// Okoteve traduccin
#elif L==7
	 "Mostra solo la parte centrale"
#elif L==8
	 "Wyswietl centralnej czesci tylko"
#elif L==9
	 "Display central area only"	// Necessita de traduo
#endif
	 ,
#if   L==1
	 "Columna derecha visible"	// Necessita traduccio
#elif L==2
	 "Rechte Spalte anzeigen"
#elif L==3
	 "Display right column"
#elif L==4
	 "Columna derecha visible"
#elif L==5
	 "Display right column"		// Besoin de traduction
#elif L==6
	 "Columna derecha visible"	// Okoteve traduccin
#elif L==7
	 "Mostra la colonna destra"
#elif L==8
	 "Wyswietl prawa kolumne"
#elif L==9
	 "Display right column"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "Columna izquierda visible"	// Necessita traduccio
#elif L==2
	 "Linke Spalte anzeigen"
#elif L==3
	 "Display left column"
#elif L==4
	 "Columna izquierda visible"
#elif L==5
	 "Display left column"		// Besoin de traduction
#elif L==6
	 "Columna izquierda visible"	// Okoteve traduccin
#elif L==7
	 "Mostra la colonna sinistra"
#elif L==8
	 "Wyswietl lewa kolumne"
#elif L==9
	 "Display left column"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "Columnas izquierda y derecha visibles"	// Necessita traduccio
#elif L==2
	 "Linke und rechte Spalte anzeigen"
#elif L==3
	 "Display left and right columns"
#elif L==4
	 "Columnas izquierda y derecha visibles"
#elif L==5
	 "Display left and right columns"		// Besoin de traduction
#elif L==6
	 "Columnas izquierda y derecha visibles"	// Okoteve traduccin
#elif L==7
	 "Mostra la colonna sinistra e destra"
#elif L==8
	 "Wyswietl lewa i prawa kolumne"
#elif L==9
	 "Display left and right columns"		// Necessita de traduo
#endif
	};

const char *Txt_level =
#if   L==1
	"nivel";	// Necessita traduccio
#elif L==2
	"Ebene";
#elif L==3
	"level";
#elif L==4
	"nivel";
#elif L==5
	"niveau";
#elif L==6
	"nivel";	// Okoteve traduccin
#elif L==7
	"livello";
#elif L==8
	"poziom";
#elif L==9
	"n&iacute;vel";
#endif

const char *Txt_Levels =
#if   L==1
	"Niveles";	// Necessita traduccio
#elif L==2
	"Ebenen";
#elif L==3
	"Levels";
#elif L==4
	"Niveles";
#elif L==5
	"Niveaux";
#elif L==6
	"Niveles";	// Okoteve traduccin
#elif L==7
	"Livelli";
#elif L==8
	"Poziomy";
#elif L==9
	"N&iacute;veis";
#endif

const char *Txt_levels =
#if   L==1
	"niveles";	// Necessita traduccio
#elif L==2
	"Ebenen";
#elif L==3
	"levels";
#elif L==4
	"niveles";
#elif L==5
	"niveaux";
#elif L==6
	"niveles";	// Okoteve traduccin
#elif L==7
	"livelli";
#elif L==8
	"poziomy";
#elif L==9
	"n&iacute;veis";
#endif

const char *Txt_License =
#if   L==1
	"Llic&egrave;ncia";
#elif L==2
	"Lizenz";
#elif L==3
	"License";
#elif L==4
	"Licencia";
#elif L==5
	"Licence";
#elif L==6
	"Licencia";	// Okoteve traduccin
#elif L==7
	"Licenza";
#elif L==8
	"Pozwolenie";
#elif L==9
	"Licen&ccedil;a";
#endif

const char *Txt_LICENSES[Brw_NUM_LICENSES] =
	{
#if   L==1
	"desconeguda / altra"
#elif L==2
	"Unbekannte / sonstige"
#elif L==3
	"unknown / other"
#elif L==4
	"desconocida / otra"
#elif L==5
	"inconnue / autre"
#elif L==6
	"desconocida / otra"			// Okoteve traduccin
#elif L==7
	"sconosciuta / altri"
#elif L==8
	"nieznana / inny"
#elif L==9
	"desconhecida / outra"
#endif
	,
#if   L==1
	"Tots els drets reservats"		// All Rights Reserved
#elif L==2
	"Alle Rechte vorbehalten"		// All Rights Reserved
#elif L==3
	"All rights reserved"			// All Rights Reserved
#elif L==4
	"Todos los derechos reservados"		// All Rights Reserved
#elif L==5
	"Tous droits r&eacute;serv&eacute;s"	// All Rights Reserved
#elif L==6
	"Todos los derechos reservados"		// All Rights Reserved	// Okoteve traduccin
#elif L==7
	"Tutti i diritti riservati"		// All Rights Reserved
#elif L==8
	"Wszelkie prawa zastrze&zdot;one"	// All Rights Reserved
#elif L==9
	"Todos os direitos reservados"		// All Rights Reserved
#endif
	,
#if   L==1
	"CC Reconeixement"			// CC Attribution License
#elif L==2
	"CC Namensnennung"			// CC Attribution License
#elif L==3
	"CC Attribution"			// CC Attribution License
#elif L==4
	"CC Reconocimiento"			// CC Attribution License
#elif L==5
	"CC Attribution"			// CC Attribution License
#elif L==6
	"CC Reconocimiento"			// CC Attribution License	// Okoteve traduccin
#elif L==7
	"CC Attribuzione"			// CC Attribution License
#elif L==8
	"CC Uznanie autorstwa"			// CC Attribution License
#elif L==9
	"CC Atribui&ccedil;&atilde;o"		// CC Attribution License
#endif
	,
#if   L==1
	"CC Reconeixement - Compartir igual"				// CC Attribution-ShareAlike License
#elif L==2
	"CC Namensnennung - Weitergabe unter gleichen Bedingungen"	// CC Attribution-ShareAlike License
#elif L==3
	"CC Attribution - Share Alike"					// CC Attribution-ShareAlike License
#elif L==4
	"CC Reconocimiento - Compartir bajo la misma licencia"		// CC Attribution-ShareAlike License
#elif L==5
	"CC Attribution - Partage &agrave; l'Identique"			// CC Attribution-ShareAlike License
#elif L==6
	"CC Reconocimiento - Compartir bajo la misma licencia"		// CC Attribution-ShareAlike License	// Okoteve traduccin
#elif L==7
	"CC Attribuzione - Condividi allo stesso modo"			// CC Attribution-ShareAlike License
#elif L==8
	"CC Uznanie autorstwa - Na tych samych warunkach"		// CC Attribution-ShareAlike License
#elif L==9
	"CC Atribui&ccedil;&atilde;o - Compartilha Igual"		// CC Attribution-ShareAlike License
#endif
	,
#if   L==1
	"CC Reconeixement - Sense obra derivada"				// CC Attribution-NoDerivs License
#elif L==2
	"CC Namensnennung - Keine Bearbeitung"					// CC Attribution-NoDerivs License
#elif L==3
	"CC Attribution - No Derivative Works"					// CC Attribution-NoDerivs License
#elif L==4
	"CC Reconocimiento - Sin obras derivadas"				// CC Attribution-NoDerivs License
#elif L==5
	"CC Attribution - Pas de travaux d&eacute;riv&eacute;s"			// CC Attribution-NoDerivs License
#elif L==6
	"CC Reconocimiento - Sin obras derivadas"				// CC Attribution-NoDerivs License	// Okoteve traduccin
#elif L==7
	"CC Attribuzione - Non opere derivate"					// CC Attribution-NoDerivs License
#elif L==8
	"CC Uznanie autorstwa - Bez utwor&oacute;w zale&zdot;nych"		// CC Attribution-NoDerivs License
#elif L==9
	"CC Atribui&ccedil;&atilde;o - Trabalhos Derivados Pro&iacute;bidos"	// CC Attribution-NoDerivs License
#endif
	,
#if   L==1
	"CC Reconeixement - No comercial"			// CC Attribution-NonCommercial License
#elif L==2
	"CC Namensnennung - Keine kommerzielle Nutzung"		// CC Attribution-NonCommercial License
#elif L==3
	"CC Attribution - Noncommercial"			// CC Attribution-NonCommercial License
#elif L==4
	"CC Reconocimiento - No comercial"			// CC Attribution-NonCommercial License
#elif L==5
	"CC Attribution - Pas d'Utilisation Commerciale"	// CC Attribution-NonCommercial License
#elif L==6
	"CC Reconocimiento - No comercial"			// CC Attribution-NonCommercial License	// Okoteve traduccin
#elif L==7
	"CC Attribuzione - Non commerciale"			// CC Attribution-NonCommercial License
#elif L==8
	"CC Uznanie autorstwa - U&zdot;ycie niekomercyjne"	// CC Attribution-NonCommercial License
#elif L==9
	"CC Atribui&ccedil;&atilde;o - N&atilde;oComercial"	// CC Attribution-NonCommercial License
#endif
	,
#if   L==1
	"CC Reconeixement - No comercial - Compartir igual"					// CC Attribution-NonCommercial-ShareAlike License
#elif L==2
	"CC Namensnennung - Keine kommerzielle Nutzung - Weitergabe unter gleichen Bedingungen"	// CC Attribution-NonCommercial-ShareAlike License
#elif L==3
	"CC Attribution - Noncommercial - Share Alike"						// CC Attribution-NonCommercial-ShareAlike License
#elif L==4
	"CC Reconocimiento - No comercial - Compartir bajo la misma licencia"			// CC Attribution-NonCommercial-ShareAlike License
#elif L==5
	"CC Attribution - Pas d'Utilisation Commerciale - Partage &agrave; l'Identique"		// CC Attribution-NonCommercial-ShareAlike License
#elif L==6
	"CC Reconocimiento - No comercial - Compartir bajo la misma licencia"			// CC Attribution-NonCommercial-ShareAlike License	// Okoteve traduccin
#elif L==7
	"CC Attribuzione - Non commerciale - Condividi allo stesso modo"			// CC Attribution-NonCommercial-ShareAlike License
#elif L==8
	"CC Uznanie autorstwa - U&zdot;ycie niekomercyjne - Na tych samych warunkach"		// CC Attribution-NonCommercial-ShareAlike License
#elif L==9
	"CC Atribui&ccedil;&atilde;o - N&atilde;oComercial - Compartilha Igual"			// CC Attribution-NonCommercial-ShareAlike License
#endif
	,
#if   L==1
	"CC Reconeixement - No comercial - Sense obra derivada"						// CC Attribution-NonCommercial-NoDerivs License
#elif L==2
	"CC Namensnennung - Keine kommerzielle Nutzung - Keine Bearbeitung"				// CC Attribution-NonCommercial-NoDerivs License
#elif L==3
	"CC Attribution - Noncommercial - No Derivative Works"						// CC Attribution-NonCommercial-NoDerivs License
#elif L==4
	"CC Reconocimiento - No comercial - Sin obras derivadas"					// CC Attribution-NonCommercial-NoDerivs License
#elif L==5
	"CC Attribution - Pas dUtilisation Commerciale - Pas de travaux d&eacute;riv&eacute;s"		// CC Attribution-NonCommercial-NoDerivs License
#elif L==6
	"CC Reconocimiento - No comercial - Sin obras derivadas"					// CC Attribution-NonCommercial-NoDerivs License	// Okoteve traduccin
#elif L==7
	"CC Attribuzione - Non commerciale - Non opere derivate"					// CC Attribution-NonCommercial-NoDerivs License
#elif L==8
	"CC Uznanie autorstwa - U&zdot;ycie niekomercyjne - Bez utwor&oacute;w zale&zdot;nych"		// CC Attribution-NonCommercial-NoDerivs License
#elif L==9
	"CC Atribui&ccedil;&atilde;o - N&atilde;oComercial - Trabalhos Derivados Pro&iacute;bidos"	// CC Attribution-NonCommercial-NoDerivs License
#endif
	};

const char *Txt_Link =
#if   L==1
	"Enlla&ccedil;";
#elif L==2
	"Link";
#elif L==3
	"Link";
#elif L==4
	"Enlace";
#elif L==5
	"Lien";
#elif L==6
	"Enlace";	// Okoteve traduccin
#elif L==7
	"Link";
#elif L==8
	"Link";
#elif L==9
	"Liga&ccedil;&atilde;o.";
#endif

const char *Txt_link =
#if   L==1
	"enlla&ccedil;";
#elif L==2
	"Link";
#elif L==3
	"link";
#elif L==4
	"enlace";
#elif L==5
	"lien";
#elif L==6
	"enlace";	// Okoteve traduccin
#elif L==7
	"link";
#elif L==8
	"link";
#elif L==9
	"liga&ccedil;&atilde;o.";
#endif

const char *Txt_Link_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Enlla&ccedil; <strong>%s</strong> eliminat.";
#elif L==2
	"Link <strong>%s</strong> entfernt.";
#elif L==3
	"Link <strong>%s</strong> removed.";
#elif L==4
	"Enlace <strong>%s</strong> eliminado.";
#elif L==5
	"Lien <strong>%s</strong> supprim&eacute;.";
#elif L==6
	"Enlace <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Link <strong>%s</strong> rimosso.";
#elif L==8
	"Link <strong>%s</strong> usuniety.";
#elif L==9
	"Liga&ccedil;&atilde;o <strong>%s</strong> removida.";
#endif

const char *Txt_Link_to_announcement_of_exam =
#if   L==1
	"Enlla&ccedil; a convocat&ograve;ria d'examen";
#elif L==2
	"Link zu Aufruf f&uuml;r Pr&uuml;fung";
#elif L==3
	"Link to announcement of exam";
#elif L==4
	"Enlace a convocatoria de examen";
#elif L==5
	"Lien vers convocation &agrave; un examen";
#elif L==6
	"Enlace a convocatoria de examen";	// Okoteve traduccin
#elif L==7
	"Link al appello d'esame";
#elif L==8
	"Link do ogloszenie egzaminu";
#elif L==9
	"Link para chamada para exame";
#endif

const char *Txt_List_edit_questions =
#if   L==1
	"Llistar / editar preguntes";
#elif L==2
	"Liste / bearbeiten Fragen";
#elif L==3
	"List / edit questions";
#elif L==4
	"Listar / editar preguntas";
#elif L==5
	"Lister / modifier questions";
#elif L==6
	"Listar / editar preguntas";	// Okoteve traduccin
#elif L==7
	"Elencare / modificare domande";
#elif L==8
	"Wy&sacute;wietli&cacute; / edytowa&cacute; pytania";
#elif L==9
	"listar / editar quest&otilde;es";
#endif

const char *Txt_List_of_detailed_clicks =
#if   L==1
	"Listado de clics detallados";		// Necessita traduccio
#elif L==2
	"List of detailed clicks";		// Need bersetzung
#elif L==3
	"List of detailed clicks";
#elif L==4
	"Listado de clics detallados";
#elif L==5
	"List of detailed clicks";		// Besoin de traduction
#elif L==6
	"Listado de clics detallados";		// Okoteve traduccin
#elif L==7
	"Lista di scatti dettagliati";
#elif L==8
	"Lista szczeg&oacute;lowa klikniec";
#elif L==9
	"List of detailed clicks";		// Necessita de traduo
#endif

const char *Txt_List_of_nicks_emails_or_IDs =
#if   L==1
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";	// Necessita traduccio
#elif L==2
	"Liste von @Benutzernamen, Emails oder Ausweis-Nr.";
#elif L==3
	"List of @nicks, emails or IDs";
#elif L==4
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";
#elif L==5
	"Liste de num&eacute;ros d'identit&eacute;";
#elif L==6
	"Lista de @apodos, correos o ID (DNI/c&eacute;dulas)";	// Okoteve traduccin
#elif L==7
	"Lista di @nomi-utenti, email o numero carte d'identit&agrave;";
#elif L==8
	"List of @nicks, emails or IDs";			// Potrzebujesz tlumaczenie
#elif L==9
	"Lista de @alcunhas, emails ou n&ordm;s. identif.";
#endif

const char *Txt_Links =
#if   L==1
	"Enlla&ccedil;os";
#elif L==2
	"Links";
#elif L==3
	"Links";
#elif L==4
	"Enlaces";
#elif L==5
	"Liens";
#elif L==6
	"Enlaces";	// Okoteve traduccin
#elif L==7
	"Link";
#elif L==8
	"Linki";
#elif L==9
	"Liga&ccedil;&otilde;es";
#endif

const char *Txt_Links_copied =
#if   L==1
	"Enlla&ccedil;os copiats";
#elif L==2
	"Links kopiert";
#elif L==3
	"Links copied";
#elif L==4
	"Enlaces copiados";
#elif L==5
	"Liens copi&eacute;s";
#elif L==6
	"Enlaces copiados";	// Okoteve traduccin
#elif L==7
	"Link copiati";
#elif L==8
	"Linki kopiowane";
#elif L==9
	"Liga&ccedil;&otilde;es copiadas";
#endif

const char *Txt_Links_removed =
#if   L==1
	"Enlla&ccedil;os eliminats";
#elif L==2
	"Links entfernt";
#elif L==3
	"Links removed";
#elif L==4
	"Enlaces eliminados";
#elif L==5
	"Liens supprim&eacute;s";
#elif L==6
	"Enlaces eliminados";	// Okoteve traduccin
#elif L==7
	"Link rimossi";
#elif L==8
	"Linki usuni&eogon;te";
#elif L==9
	"Liga&ccedil;&otilde;es removidas";
#endif

const char *Txt_Local_address =
#if   L==1
	"Domicilio local";	// Necessita traduccio
#elif L==2
	"Lokale Adresse";
#elif L==3
	"Local address";
#elif L==4
	"Domicilio local";
#elif L==5
	"Adresse locale";
#elif L==6
	"Domicilio local";	// Okoteve traduccin
#elif L==7
	"Indirizzo locale";
#elif L==8
	"Adres lokalny";
#elif L==9
	"Local Address";
#endif

const char *Txt_Location =
#if   L==1
	"Ubicaci&oacute;";
#elif L==2
	"Standort";
#elif L==3
	"Location";
#elif L==4
	"Ubicaci&oacute;n";
#elif L==5
	"Emplacement";
#elif L==6
	"Ubicaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Posizione";
#elif L==8
	"Lokacja";
#elif L==9
	"Localiza&ccedil;&atilde;o";
#endif

const char *Txt_Locations =
#if   L==1
	"Ubicacions";
#elif L==2
	"Standorte";
#elif L==3
	"Locations";
#elif L==4
	"Ubicaciones";
#elif L==5
	"Emplacements";
#elif L==6
	"Ubicaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Posizioni";
#elif L==8
	"Lokalizacje";
#elif L==9
	"Localiza&ccedil;&otilde;es";
#endif

const char *Txt_LOCATIONS_HELP_ORDER[2] =
   {
#if   L==1
	 "Ordenar por fecha inicial"	// Necessita traduccio
#elif L==2
	 "Sortieren nach Startdatum"
#elif L==3
	 "Sort by start date"
#elif L==4
	 "Ordenar por fecha inicial"
#elif L==5
	 "Trier par date initiale"
#elif L==6
	 "Ordenar por fecha inicial"	// Okoteve traduccin
#elif L==7
	 "Ordina per data iniziale"
#elif L==8
	 "Sortuj wedlug daty rozpoczecia"
#elif L==9
	 "Classificar por data de in&iacute;cio"
#endif
	 ,
#if   L==1
	 "Ordenar por fecha final"	// Necessita traduccio
#elif L==2
	 "Sortieren nach Enddatum"
#elif L==3
	 "Sort by end date"
#elif L==4
	 "Ordenar por fecha final"
#elif L==5
	 "Trier par date finale"
#elif L==6
	 "Ordenar por fecha final"	// Okoteve traduccin
#elif L==7
	 "Ordina per data finale"
#elif L==8
	 "Sortuj wedlug daty koncowa"
#elif L==9
	 "Classificar por data final"
#endif
	};

const char *Txt_LOG_More_info =
#if   L==1
	"Observaciones";	// Necessita traduccio
#elif L==2
	"Weitere Informationen";
#elif L==3
	"More info";
#elif L==4
	"Observaciones";
#elif L==5
	"Plus d'informations";
#elif L==6
	"Observaciones";	// Okoteve traduccin
#elif L==7
	"Maggiori informazioni";
#elif L==8
	"Wiecej informacji";
#elif L==9
	"Mais informa&ccedil;&atilde;o";
#endif

const char *Txt_Log_in =
#if   L==1
	"Iniciar sessi&oacute;";
#elif L==2
	"Anmelden";
#elif L==3
	"Log in";
#elif L==4
	"Iniciar sesi&oacute;n";
#elif L==5
	"Entrer";
#elif L==6
	"Tera&ntilde;emboguapy";
#elif L==7
	"Inizia sessione";
#elif L==8
	"Zaloguj si&eogon;";
#elif L==9
	"Entrar";
#endif

const char *Txt_Log_out =
#if   L==1
	"Tancar sessi&oacute;";
#elif L==2
	"Abmelden";
#elif L==3
	"Log out";
#elif L==4
	"Cerrar sesi&oacute;n";
#elif L==5
	"Fermer session";
#elif L==6
	"S&#7869;";
#elif L==7
	"Chiudi sessione";
#elif L==8
	"Wyloguj si&eogon;";
#elif L==9
	"Fechar sess&atilde;o";
#endif

const char *Txt_logged[Usr_NUM_SEXS] =
   {
#if   L==1	// Usr_SEX_UNKNOWN
	 "identificado/a"	// Necessita traduccio
#elif L==2
	 "angemeldet"
#elif L==3
	 "logged"
#elif L==4
	 "identificado/a"
#elif L==5
	 "connect&eacute;/e"
#elif L==6
	 "identificado/a"	// Okoteve traduccin
#elif L==7
	 "loggato/a"
#elif L==8
	 "zalogowany"
#elif L==9
	 "logado/a"
#endif
	 ,
#if   L==1	// Usr_SEX_FEMALE
	 "identificada"	// Necessita traduccio
#elif L==2
	 "angemeldet"
#elif L==3
	 "logged"
#elif L==4
	 "identificada"
#elif L==5
	 "connect&eacute;e"
#elif L==6
	 "identificada"	// Okoteve traduccin
#elif L==7
	 "loggata"
#elif L==8
	 "zalogowana"
#elif L==9
	 "logada"
#endif
	 ,
#if   L==1	// Usr_SEX_MALE
	 "identificado"	// Necessita traduccio
#elif L==2
	 "angemeldet"
#elif L==3
	 "logged"
#elif L==4
	 "identificado"
#elif L==5
	 "connect&eacute;"
#elif L==6
	 "identificado"	// Okoteve traduccin
#elif L==7
	 "loggato"
#elif L==8
	 "zalogowany"
#elif L==9
	 "logado"
#endif
	 ,
#if   L==1	// Usr_SEX_ALL
	 "identificado/a"	// Necessita traduccio
#elif L==2
	 "angemeldet"
#elif L==3
	 "logged"
#elif L==4
	 "identificado/a"
#elif L==5
	 "connect&eacute;/e"
#elif L==6
	 "identificado/a"	// Okoteve traduccin
#elif L==7
	 "loggato/a"
#elif L==8
	 "zalogowani"
#elif L==9
	 "logado/a"
#endif
	};

const char *Txt_Logo =
#if   L==1
	"Logo";	// Necessita traduccio
#elif L==2
	"Logo";
#elif L==3
	"Logo";
#elif L==4
	"Logo";
#elif L==5
	"Logo";
#elif L==6
	"Logo";	// Okoteve traduccin
#elif L==7
	"Logo";
#elif L==8
	"Logo";
#elif L==9
	"Logotipo";
#endif

const char *Txt_Mandatory_enrollment =		// (to a type of group)
#if   L==1
	"&iquest;Adscripci&oacute;n obligatoria?";	// Necessita traduccio
#elif L==2
	"Obligatorische beitreten?";
#elif L==3
	"Mandatory enrollment?";
#elif L==4
	"&iquest;Adscripci&oacute;n obligatoria?";
#elif L==5
	"Inscription obligatoire?";
#elif L==6
	"&iquest;Adscripci&oacute;n obligatoria?";	// Okoteve traduccin
#elif L==7
	"Iscrizione obbligatoria?";
#elif L==8
	"Obowiazkowa rejestracja?";
#elif L==9
	"Inscri&ccedil;&atilde;o obrigat&oacute;ria?";
#endif

const char *Txt_Mark_all_NOTIFICATIONS_as_read =
#if   L==1
	"Marca totes com llegides";
#elif L==2
	"Markieren Sie alle als gelesen";
#elif L==3
	"Mark all as read";
#elif L==4
	"Marcar todas como le&iacute;das";
#elif L==5
	"Marquer tous comme lus";
#elif L==6
	"Marcar todas como le&iacute;das";	// Okoteve traduccin
#elif L==7
	"Segnare tutte come gi&grave; letti";
#elif L==8
	"Oznacz wszystko jako czytasz";
#elif L==9
	"Marcar todas como lidas";
#endif

const char *Txt_Mark_all_notifications_as_read =
#if   L==1
	"Marca totes les notificacions com llegides";
#elif L==2
	"Markieren Sie alle Anmeldungen als gelesen";
#elif L==3
	"Mark all notifications as read";
#elif L==4
	"Marcar todas las notificaciones como le&iacute;das";
#elif L==5
	"Marquer tous les notifications comme lus";
#elif L==6
	"Marcar todas las notificaciones como le&iacute;das";	// Okoteve traduccin
#elif L==7
	"Segnare tutte le notifiche come gi&grave; letti";
#elif L==8
	"Mark all notifications as read";			// Potrzebujesz tlumaczenie
#elif L==9
	"Marcar todas as notifica&ccedil;&otilde;es como lidas";
#endif

const char *Txt_Mark_as_favourite =
#if   L==1
	"Marca com favorit";
#elif L==2
	"Markieren als Favorit";
#elif L==3
	"Mark as favourite";
#elif L==4
	"Marcar como favorito";
#elif L==5
	"Marquer comme favori";
#elif L==6
	"Marcar como favorito";	// Okoteve traduccin
#elif L==7
	"Segnare come preferito";
#elif L==8
	"Oznacz jako ulubiony";
#elif L==9
	"Marcar como favorito";
#endif

const char *Txt_Marks_management_area =
#if   L==1
	"Zona d'administraci&oacute; de qualificacions";
#elif L==2
	"Bewertungs-Verwaltung";
#elif L==3
	"Marks management area";
#elif L==4
	"Zona de administraci&oacute;n de calificaciones";
#elif L==5
	"Zone de gestion de notes";
#elif L==6
	"Zona de administraci&oacute;n de calificaciones";	// Okoteve traduccin
#elif L==7
	"Area di gestione dei risultati";
#elif L==8
	"Obszar zarzadzania znak&oacute;w";
#elif L==9
	"Zona de gest&atilde;o de notas";
#endif

const char *Txt_marks_management_area =
#if   L==1
	"zona de administraci&oacute;n de calificaciones";	// Necessita traduccio
#elif L==2
	"Bewertungs-Verwaltung";
#elif L==3
	"marks management area";
#elif L==4
	"zona de administraci&oacute;n de calificaciones";
#elif L==5
	"zone de gestion de notes";
#elif L==6
	"zona de administraci&oacute;n de calificaciones";	// Okoteve traduccin
#elif L==7
	"area di gestione dei risultati";
#elif L==8
	"obszar zarzadzania znak&oacute;w";
#elif L==9
	"zona de gest&atilde;o de notas";
#endif

const char *Txt_Marks_area =
#if   L==1
	"Zona de qualificacions";
#elif L==2
	"Bewertungs-Bereich";
#elif L==3
	"Marks area";
#elif L==4
	"Zona de calificaciones";
#elif L==5
	"Zone de notes";
#elif L==6
	"Zona de calificaciones";	// Okoteve traduccin
#elif L==7
	"Area risultati";
#elif L==8
	"Powierzchnia znak&oacute;w";
#elif L==9
	"Zona de notas";
#endif

const char *Txt_Materials =
#if   L==1
	"Materiales";	// Necessita traduccio
#elif L==2
	"Material";
#elif L==3
	"Materials";
#elif L==4
	"Materiales";
#elif L==5
	"Mat&eacute;riaux";
#elif L==6
	"Materiales";	// Okoteve traduccin
#elif L==7
	"Materiale";
#elif L==8
	"Materialy";
#elif L==9
	"Materiais";
#endif

const char *Txt_Max_levels =
#if   L==1
	"M&aacute;x. niveles";	// Necessita traduccio
#elif L==2
	"Max. Ebenen";
#elif L==3
	"Max. levels";
#elif L==4
	"M&aacute;x. niveles";
#elif L==5
	"Max. niveaux";
#elif L==6
	"M&aacute;x. niveles";	// Okoteve traduccin
#elif L==7
	"Max. livelli";
#elif L==8
	"Max. poziom&oacute;w";
#elif L==9
	"M&aacute;x. n&iacute;veis";
#endif

const char *Txt_Max_BR_students =
#if   L==1
	"M&agrave;xim<br />d'estud.";
#elif L==2
	"Max.<br />Stud.";
#elif L==3
	"Max.<br />stud.";
#elif L==4
	"M&aacute;x.<br />estud.";
#elif L==5
	"Max.<br />&eacute;tud.";
#elif L==6
	"M&aacute;x.<br />estud.";	// Okoteve traduccin
#elif L==7
	"Max.<br />stud.";
#elif L==8
	"Max.<br />stud.";
#elif L==9
	"M&aacute;x.<br />estud.";
#endif

const char *Txt_maximum =
#if   L==1
	"m&aacute;ximo";	// Necessita traduccio
#elif L==2
	"Maximum";
#elif L==3
	"maximum";
#elif L==4
	"m&aacute;ximo";
#elif L==5
	"maximum";
#elif L==6
	"m&aacute;ximo";	// Okoteve traduccin
#elif L==7
	"massimo";
#elif L==8
	"maksimum";
#elif L==9
	"m&aacute;ximo";
#endif

const char *Txt_Menu =
#if   L==1
	"Men&uacute;";
#elif L==2
	"Men&uuml;";
#elif L==3
	"Menu";
#elif L==4
	"Men&uacute;";
#elif L==5
	"Menu";
#elif L==6
	"Men&uacute;";	// Okoteve traduccin
#elif L==7
	"Menu";
#elif L==8
	"Menu";
#elif L==9
	"Menu";
#endif

const char *Txt_MENU_NAMES[Mnu_NUM_MENUS] =
   {
#if   L==1
	 "Horitzontal"
#elif L==2
	 "Horizontal"
#elif L==3
	 "Horizontal"
#elif L==4
	 "Horizontal"
#elif L==5
	 "Horizontal"
#elif L==6
	 "Horizontal"	// Okoteve traduccin
#elif L==7
	 "Orizzontale"
#elif L==8
	 "Poziomy"
#elif L==9
	 "Horizontal"
#endif
	 ,
#if   L==1
	 "Vertical"
#elif L==2
	 "Vertikal"
#elif L==3
	 "Vertical"
#elif L==4
	 "Vertical"
#elif L==5
	 "Vertical"
#elif L==6
	 "Vertical"		// Okoteve traduccin
#elif L==7
	 "Verticale"
#elif L==8
	 "Pionowy"
#elif L==9
	 "Vertical"
#endif
	};

const char *Txt_MENU_TITLE[Tab_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB] =
{
	{
	// TabUnk ****************************************************
	NULL,	//  0
	NULL,	//  1
	NULL,	//  2
	NULL,	//  3
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabSys *****************************************************
	//  0: ActSysReqSch
#if   L==1
	"Cercar"
#elif L==2
	"Suchen"
#elif L==3
	"Search"
#elif L==4
	"Buscar"
#elif L==5
	"Chercher"
#elif L==6
	"Heka"
#elif L==7
	"Cercare"
#elif L==8
	"Szukaj"
#elif L==9
	"Procurar"
#endif
	,
	//  1: ActSeeCty
#if   L==1
	"Pa&iuml;sos"
#elif L==2
	"L&auml;nder"
#elif L==3
	"Countries"
#elif L==4
	"Pa&iacute;ses"
#elif L==5
	"Pays"
#elif L==6
	"Tet&atilde;"
#elif L==7
	"Paesi"
#elif L==8
	"Kraje"
#elif L==9
	"Pa&iacute;ses"
#endif
	,
	//  2: ActSeePen
#if   L==1
	"Pendents"
#elif L==2
	"Anh&auml;ngig"
#elif L==3
	"Pending"
#elif L==4
	"Pendientes"
#elif L==5
	"En attente"
#elif L==6
	"Pendientes"	// Okoteve traduccin
#elif L==7
	"In sospeso"
#elif L==8
	"Oczekuj&aogon;cy"
#elif L==9
	"Pendentes"
#endif
	,
	//  3: ActReqRemOldCrs
#if   L==1
	"Antics"
#elif L==2
	"Fr&uuml;here"
#elif L==3
	"Old"
#elif L==4
	"Antiguas"
#elif L==5
	"Anciens"
#elif L==6
	"Antiguas"	// Okoteve traduccin
#elif L==7
	"Vecchi"
#elif L==8
	"Starych"
#elif L==9
	"Antigas"
#endif
	,
	//  4: ActSeeDegTyp
#if   L==1
	"Estudis"
#elif L==2
	"Studium"
#elif L==3
	"Studies"
#elif L==4
	"Estudios"
#elif L==5
	"&Eacute;tudes"
#elif L==6
	"Estudios"	// Okoteve traduccin
#elif L==7
	"Studi"
#elif L==8
	"Studia"
#elif L==9
	"Estudos"
#endif
	,
	//  5: ActSeeMai
#if   L==1
	"Dominis"
#elif L==2
	"Dom&auml;nen"
#elif L==3
	"Domains"
#elif L==4
	"Dominios"
#elif L==5
	"Domaines"
#elif L==6
	"Dominios"	// Okoteve traduccin
#elif L==7
	"Campi"
#elif L==8
	"Domen"
#elif L==9
	"Dom&iacute;nios"
#endif
	,
	//  6: ActSeeBan
#if   L==1
	"Banners"
#elif L==2
	"Werbebanners"
#elif L==3
	"Banners"
#elif L==4
	"Banners"
#elif L==5
	"Banni&eacute;res"
#elif L==6
	"Banners"
#elif L==7
	"Banners"
#elif L==8
	"Banners"
#elif L==9
	"Banners"
#endif
	,
	//  7: ActSeeLnk
#if   L==1
	"Enlla&ccedil;os"
#elif L==2
	"Links"
#elif L==3
	"Links"
#elif L==4
	"Enlaces"
#elif L==5
	"Liens"
#elif L==6
	"Ambu&eacute;re"
#elif L==7
	"Link"
#elif L==8
	"Linki"
#elif L==9
	"Liga&ccedil;&otilde;es"
#endif
	,
	//  8: ActLstPlg
#if   L==1
	"Complements"
#elif L==2
	"Plugins"
#elif L==3
	"Plugins"
#elif L==4
	"Complementos"
#elif L==5
	"Plugins"
#elif L==6
	"Complementos"	// Okoteve traduccin
#elif L==7
	"Plugin"
#elif L==8
	"Plugin&oacute;w"
#elif L==9
	"Plugins"
#endif
	,
	//  9: ActSetUp
#if   L==1
	"Instal lar"
#elif L==2
	"Einrichten"
#elif L==3
	"Set up"
#elif L==4
	"Instalar"
#elif L==5
	"Installer"
#elif L==6
	"Instalar"	// Okoteve traduccin
#elif L==7
	"Setup"
#elif L==8
	"Set up"
#elif L==9
	"Instalar"
#endif
	,
	// 10: ActSeeCalSys
#if   L==1
	"Calendari"
#elif L==2
	"Kalender"
#elif L==3
	"Calendar"
#elif L==4
	"Calendario"
#elif L==5
	"Calendrier"
#elif L==6
	"Arapapaha"
#elif L==7
	"Calendario"
#elif L==8
	"Kalendarz"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	// 11
	},
	{
	// TabCty *****************************************************
	//  0: ActCtyReqSch
#if   L==1
	"Cercar"
#elif L==2
	"Suchen"
#elif L==3
	"Search"
#elif L==4
	"Buscar"
#elif L==5
	"Chercher"
#elif L==6
	"Heka"
#elif L==7
	"Cercare"
#elif L==8
	"Szukaj"
#elif L==9
	"Procurar"
#endif
	,
	//  1: ActSeeCtyInf
#if   L==1
	"Informaci&oacute;"
#elif L==2
	"Information"
#elif L==3
	"Information"
#elif L==4
	"Informaci&oacute;n"
#elif L==5
	"Information"
#elif L==6
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Informazione"
#elif L==8
	"Informacja"
#elif L==9
	"Informa&ccedil;&atilde;o"
#endif
	,
	//  2: ActSeeIns
#if   L==1
	"Institucions"
#elif L==2
	"Hochschulen"
#elif L==3
	"Institutions"
#elif L==4
	"Instituciones"
#elif L==5
	"&Eacute;tablissements"
#elif L==6
	"Mbo'ehao"
#elif L==7
	"Istituzioni"
#elif L==8
	"Instytucje"
#elif L==9
	"Institu&ccedil;&otilde;es"
#endif
	,
	//  3: ActSeeCalCty
#if   L==1
	"Calendari"
#elif L==2
	"Kalender"
#elif L==3
	"Calendar"
#elif L==4
	"Calendario"
#elif L==5
	"Calendrier"
#elif L==6
	"Arapapaha"
#elif L==7
	"Calendario"
#elif L==8
	"Kalendarz"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabIns *****************************************************
	//  0: ActInsReqSch
#if   L==1
	"Cercar"
#elif L==2
	"Suchen"
#elif L==3
	"Search"
#elif L==4
	"Buscar"
#elif L==5
	"Chercher"
#elif L==6
	"Heka"
#elif L==7
	"Cercare"
#elif L==8
	"Szukaj"
#elif L==9
	"Procurar"
#endif
	,
	//  1: ActSeeInsInf
#if   L==1
	"Informaci&oacute;"
#elif L==2
	"Information"
#elif L==3
	"Information"
#elif L==4
	"Informaci&oacute;n"
#elif L==5
	"Information"
#elif L==6
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Informazione"
#elif L==8
	"Informacja"
#elif L==9
	"Informa&ccedil;&atilde;o"
#endif
	,
	//  2: ActSeeCtr
#if   L==1
	"Centres"
#elif L==2
	"Lehrinstitute"
#elif L==3
	"Centres"
#elif L==4
	"Centros"
#elif L==5
	"Centres"
#elif L==6
	"Mbo'ehao"
#elif L==7
	"Centri"
#elif L==8
	"Centra"
#elif L==9
	"Centros"
#endif
	,
	//  3: ActSeeDpt
#if   L==1
	"Departaments"
#elif L==2
	"Abteilungen"
#elif L==3
	"Departments"
#elif L==4
	"Departamentos"
#elif L==5
	"D&eacute;partements"
#elif L==6
	"Peh&#7869;"
#elif L==7
	"Dipartimenti"
#elif L==8
	"Wydzia&lstrok;y"
#elif L==9
	"Departamentos"
#endif
	,
	//  4: ActSeePlc
#if   L==1
	"Llocs"
#elif L==2
	"Standorte"
#elif L==3
	"Places"
#elif L==4
	"Lugares"
#elif L==5
	"Emplacements"
#elif L==6
	"Tenda"
#elif L==7
	"Localit&agrave;"
#elif L==8
	"Miejsca"
#elif L==9
	"Localiza&ccedil;oes"
#endif
	,
	//  5: ActSeeHld
#if   L==1
	"Festius"
#elif L==2
	"Feiertage"
#elif L==3
	"Holidays"
#elif L==4
	"Festivos"
#elif L==5
	"F&eacute;ri&eacute;s"
#elif L==6
	"Arete"
#elif L==7
	"Vacanze"
#elif L==8
	"Noclegi"
#elif L==9
	"F&eacute;rias"
#endif
	,
	//  6: ActSeeCalIns
#if   L==1
	"Calendari"
#elif L==2
	"Kalender"
#elif L==3
	"Calendar"
#elif L==4
	"Calendario"
#elif L==5
	"Calendrier"
#elif L==6
	"Arapapaha"
#elif L==7
	"Calendario"
#elif L==8
	"Kalendarz"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabCtr *****************************************************
	//  0: ActCtrReqSch
#if   L==1
	"Cercar"
#elif L==2
	"Suchen"
#elif L==3
	"Search"
#elif L==4
	"Buscar"
#elif L==5
	"Chercher"
#elif L==6
	"Heka"
#elif L==7
	"Cercare"
#elif L==8
	"Szukaj"
#elif L==9
	"Procurar"
#endif
	,
	//  1: ActSeeCtrInf
#if   L==1
	"Informaci&oacute;"
#elif L==2
	"Information"
#elif L==3
	"Information"
#elif L==4
	"Informaci&oacute;n"
#elif L==5
	"Information"
#elif L==6
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Informazione"
#elif L==8
	"Informacja"
#elif L==9
	"Informa&ccedil;&atilde;o"
#endif
	,
	//  2: ActSeeDeg
#if   L==1
	"Titulacions"
#elif L==2
	"Studiengang"
#elif L==3
	"Degrees"
#elif L==4
	"Titulaciones"
#elif L==5
	"&Eacute;tudes"
#elif L==6
	"Arandur&atilde;"
#elif L==7
	"Lauree"
#elif L==8
	"Stopnie"
#elif L==9
	"Titula&ccedil;&atilde;os"
#endif
	,
	//  3: ActSeeCalCtr
#if   L==1
	"Calendari"
#elif L==2
	"Kalender"
#elif L==3
	"Calendar"
#elif L==4
	"Calendario"
#elif L==5
	"Calendrier"
#elif L==6
	"Arapapaha"
#elif L==7
	"Calendario"
#elif L==8
	"Kalendarz"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabDeg *****************************************************
	//  0: ActDegReqSch
#if   L==1
	"Cercar"
#elif L==2
	"Suchen"
#elif L==3
	"Search"
#elif L==4
	"Buscar"
#elif L==5
	"Chercher"
#elif L==6
	"Heka"
#elif L==7
	"Cercare"
#elif L==8
	"Szukaj"
#elif L==9
	"Procurar"
#endif
	,
	//  1: ActSeeDegInf
#if   L==1
	"Informaci&oacute;"
#elif L==2
	"Information"
#elif L==3
	"Information"
#elif L==4
	"Informaci&oacute;n"
#elif L==5
	"Information"
#elif L==6
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Informazione"
#elif L==8
	"Informacja"
#elif L==9
	"Informa&ccedil;&atilde;o"
#endif
	,
	//  2: ActSeeCrs
#if   L==1
	"Assignatures"
#elif L==2
	"Kursen"
#elif L==3
	"Courses"
#elif L==4
	"Asignaturas"
#elif L==5
	"Mati&egrave;res"
#elif L==6
	"Mbo'esyry"
#elif L==7
	"Corsi"
#elif L==8
	"Kurs&oacute;w"
#elif L==9
	"Disciplinas"
#endif
	,
	//  3: ActSeeCalDeg
#if   L==1
	"Calendari"
#elif L==2
	"Kalender"
#elif L==3
	"Calendar"
#elif L==4
	"Calendario"
#elif L==5
	"Calendrier"
#elif L==6
	"Arapapaha"
#elif L==7
	"Calendario"
#elif L==8
	"Kalendarz"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabCrs *****************************************************
	//  0: ActCtrReqSch
#if   L==1
	"Cercar"
#elif L==2
	"Suchen"
#elif L==3
	"Search"
#elif L==4
	"Buscar"
#elif L==5
	"Chercher"
#elif L==6
	"Heka"
#elif L==7
	"Cercare"
#elif L==8
	"Szukaj"
#elif L==9
	"Procurar"
#endif
	,
	//  1: ActSeeCrsInf
#if   L==1
	"Informaci&oacute;"
#elif L==2
	"Information"
#elif L==3
	"Information"
#elif L==4
	"Informaci&oacute;n"
#elif L==5
	"Information"
#elif L==6
	"Informaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Informazione"
#elif L==8
	"Informacja"
#elif L==9
	"Informa&ccedil;&atilde;o"
#endif
	,
	//  2: ActSeeTchGui
#if   L==1
	"Guia"
#elif L==2
	"Hinweise"
#elif L==3
	"Guide"
#elif L==4
	"Gu&iacute;a"
#elif L==5
	"Guide"
#elif L==6
	"Gu&iacute;a"	// Okoteve traduccin
#elif L==7
	"Guida"
#elif L==8
	"Podr&eogon;cznik"
#elif L==9
	"Guia"
#endif
	,
	//  3: ActSeeSyl
#if   L==1
	"Programa"
#elif L==2
	"Studienplan"
#elif L==3
	"Syllabus"
#elif L==4
	"Programa"
#elif L==5
	"Plan"
#elif L==6
	"Programa"	// Okoteve traduccin
#elif L==7
	"Programma"
#elif L==8
	"Syllabus"
#elif L==9
	"Programa"
#endif
	,
	//  4: ActSeeBib
#if   L==1
	"Bibliografia"
#elif L==2
	"Literatur"
#elif L==3
	"Bibliography"
#elif L==4
	"Bibliograf&iacute;a"
#elif L==5
	"Bibliographie"
#elif L==6
	"Arandukapurupyre"
#elif L==7
	"Bibliografia"
#elif L==8
	"Bibliografia"
#elif L==9
	"Bibliografia"
#endif
	,
	//  5: ActSeeFAQ
#if   L==1
	"FAQ"	// Necessita traduccio
#elif L==2
	"FAQ"
#elif L==3
	"FAQ"
#elif L==4
	"FAQ"
#elif L==5
	"FAQ"
#elif L==6
	"FAQ"	// Okoteve traduccin
#elif L==7
	"FAQ"
#elif L==8
	"FAQ"
#elif L==9
	"FAQ"
#endif
	,
	//  6: ActSeeCrsLnk
#if   L==1
	"Enlla&ccedil;os"
#elif L==2
	"Links"
#elif L==3
	"Links"
#elif L==4
	"Enlaces"
#elif L==5
	"Liens"
#elif L==6
	"Ambu&eacute;re"
#elif L==7
	"Link"
#elif L==8
	"Linki"
#elif L==9
	"Liga&ccedil;&otilde;es"
#endif
	,
	//  7: ActSeeCrsTT
#if   L==1
	"Horari"
#elif L==2
	"Stundenplan"
#elif L==3
	"Timetable"
#elif L==4
	"Horario"
#elif L==5
	"Horaire"
#elif L==6
	"Horario"	// Okoteve traduccin
#elif L==7
	"Orario"
#elif L==8
	"Plan"
#elif L==9
	"Hor&aacute;rio"
#endif
	,
	//  8: ActSeeCalCrs
#if   L==1
	"Calendari"
#elif L==2
	"Kalender"
#elif L==3
	"Calendar"
#elif L==4
	"Calendario"
#elif L==5
	"Calendrier"
#elif L==6
	"Arapapaha"
#elif L==7
	"Calendario"
#elif L==8
	"Kalendarz"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabAss *****************************************************
	//  0: ActSeeAss
#if   L==1
	 "Sistema"
#elif L==2
	 "System"
#elif L==3
	 "System"
#elif L==4
	 "Sistema"
#elif L==5
	 "Syst&egrave;me"
#elif L==6
	 "Sistema"	// Okoteve traduccin
#elif L==7
	 "Sistema"
#elif L==8
	 "System"
#elif L==9
	 "Sistema"
#endif
	,
	//  1: ActSeeAsg
#if   L==1
	"Activitats"
#elif L==2
	"Aufgaben"
#elif L==3
	"Assignments"
#elif L==4
	"Actividades"
#elif L==5
	"Activit&eacute;s"
#elif L==6
	"Tar&eacute;a"
#elif L==7
	"Attivit&agrave;"
#elif L==8
	"Zadania"
#elif L==9
	"Atividades"
#endif
	,
	//  2: ActReqTst
#if   L==1
	"Tests"
#elif L==2
	"Tests"
#elif L==3
	"Tests"
#elif L==4
	"Tests"
#elif L==5
	"Tests"
#elif L==6
	"Tests"	// Okoteve traduccin
#elif L==7
	"Test"
#elif L==8
	"Test&oacute;w"
#elif L==9
	"Testes"
#endif
	,
	//  3: ActSeeAllExaAnn
#if   L==1
	"Convocat&ograve;ries"
#elif L==2
	"Pr&uuml;fungsaufrufe"
#elif L==3
	"Announcements"
#elif L==4
	"Convocatorias"
#elif L==5
	"Convocations"
#elif L==6
	"Convocatorias"	// Okoteve traduccin
#elif L==7
	"Appelli"
#elif L==8
	"Ogloszenia"
#elif L==9
	"Chamadas"
#endif
	,
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabFil *****************************************************
	//  0: ActSeeAdmDocIns
#if   L==1
	"Documents"
#elif L==2
	"Dokumente"
#elif L==3
	"Documents"
#elif L==4
	"Documentos"
#elif L==5
	"Documents"
#elif L==6
	"Kuatia"
#elif L==7
	"Documenti"
#elif L==8
	"Dokument&oacute;w"
#elif L==9
	"Documentos"
#endif
	,
	//  1: ActAdmShaIns
#if   L==1
	"Compartits"
#elif L==2
	"Freigegebene"
#elif L==3
	"Shared"
#elif L==4
	"Compartidos"
#elif L==5
	"Partag&eacute;s"
#elif L==6
	"Compartidos"	// Okoteve traduccin
#elif L==7
	"Condivisi"
#elif L==8
	"Udost&eogon;pnione"
#elif L==9
	"Compartilhados"
#endif
	,
	//  2: ActSeeAdmDocCtr
#if   L==1
	"Documents"
#elif L==2
	"Dokumente"
#elif L==3
	"Documents"
#elif L==4
	"Documentos"
#elif L==5
	"Documents"
#elif L==6
	"Kuatia"
#elif L==7
	"Documenti"
#elif L==8
	"Dokument&oacute;w"
#elif L==9
	"Documentos"
#endif
	,
	//  3: ActAdmShaCtr
#if   L==1
	"Compartits"
#elif L==2
	"Freigegebene"
#elif L==3
	"Shared"
#elif L==4
	"Compartidos"
#elif L==5
	"Partag&eacute;s"
#elif L==6
	"Compartidos"	// Okoteve traduccin
#elif L==7
	"Condivisi"
#elif L==8
	"Udost&eogon;pnione"
#elif L==9
	"Compartilhados"
#endif
	,
	//  4: ActSeeAdmDocDeg
#if   L==1
	"Documents"
#elif L==2
	"Dokumente"
#elif L==3
	"Documents"
#elif L==4
	"Documentos"
#elif L==5
	"Documents"
#elif L==6
	"Kuatia"
#elif L==7
	"Documenti"
#elif L==8
	"Dokument&oacute;w"
#elif L==9
	"Documentos"
#endif
	,
	//  5: ActAdmShaDeg
#if   L==1
	"Compartits"
#elif L==2
	"Freigegebene"
#elif L==3
	"Shared"
#elif L==4
	"Compartidos"
#elif L==5
	"Partag&eacute;s"
#elif L==6
	"Compartidos"	// Okoteve traduccin
#elif L==7
	"Condivisi"
#elif L==8
	"Udost&eogon;pnione"
#elif L==9
	"Compartilhados"
#endif
	,
	//  6: ActSeeAdmDocCrsGrp
#if   L==1
	"Documents"
#elif L==2
	"Dokumente"
#elif L==3
	"Documents"
#elif L==4
	"Documentos"
#elif L==5
	"Documents"
#elif L==6
	"Kuatia"
#elif L==7
	"Documenti"
#elif L==8
	"Dokument&oacute;w"
#elif L==9
	"Documentos"
#endif
	,
	//  7: ActAdmTchCrsGrp
#if   L==1
	"Privats"
#elif L==2
	"Private"
#elif L==3
	"Private"
#elif L==4
	"Privados"
#elif L==5
	"Priv&eacute;s"
#elif L==6
	"Privados"	// Okoteve traduccin
#elif L==7
	"Privati"
#elif L==8
	"Prywatne"
#elif L==9
	"Privados"
#endif
	,
	//  8: ActAdmShaCrsGrp
#if   L==1
	"Compartits"
#elif L==2
	"Freigegebene"
#elif L==3
	"Shared"
#elif L==4
	"Compartidos"
#elif L==5
	"Partag&eacute;s"
#elif L==6
	"Compartidos"	// Okoteve traduccin
#elif L==7
	"Condivisi"
#elif L==8
	"Udost&eogon;pnione"
#elif L==9
	"Compartilhados"
#endif
	,
	//  9: ActAdmAsgWrkUsr
#if   L==1
	"Treballs"
#elif L==2
	"Hausarbeiten"
#elif L==3
	"Homework"
#elif L==4
	"Trabajos"
#elif L==5
	"Travaux"
#elif L==6
	"Tembiapo"
#elif L==7
	"Lavori"
#elif L==8
	"Zadanie"
#elif L==9
	"Trabalhos"
#endif
	,
	//  10: ActReqAsgWrkCrs
#if   L==1
	"Treballs"
#elif L==2
	"Hausarbeiten"
#elif L==3
	"Homework"
#elif L==4
	"Trabajos"
#elif L==5
	"Travaux"
#elif L==6
	"Tembiapo"
#elif L==7
	"Lavori"
#elif L==8
	"Zadanie"
#elif L==9
	"Trabalhos"
#endif
	,
	//  11: ActSeeMrk
#if   L==1
	"Qualificacions"
#elif L==2
	"Bewertungen"
#elif L==3
	"Marks"
#elif L==4
	"Calificaciones"
#elif L==5
	"Notes"
#elif L==6
	"Calificaciones"	// Okoteve traduccin
#elif L==7
	"Risultati"
#elif L==8
	"Marks"
#elif L==9
	"Notas"
#endif
	},
	{
	// TabUsr *****************************************************
	//  0: ActReqSelGrp
#if   L==1
	"Grups"
#elif L==2
	"Gruppen"
#elif L==3
	"Groups"
#elif L==4
	"Grupos"
#elif L==5
	"Groupes"
#elif L==6
	"Aty"
#elif L==7
	"Gruppi"
#elif L==8
	"Grupy"
#elif L==9
	"Grupos"
#endif
	,
	//  1: ActLstStd
#if   L==1
	"Estudiants"
#elif L==2
	"Studenten"
#elif L==3
	"Students"
#elif L==4
	"Estudiantes"
#elif L==5
	"&Eacute;tudiants"
#elif L==6
	"Estudiantes"	// Okoteve traduccin
#elif L==7
	"Studenti"
#elif L==8
	"Student&oacute;w"
#elif L==9
	"Estudantes"
#endif
	,
	//  2: ActLstTch
#if   L==1
	"Professors"
#elif L==2
	"Lehrkraft"
#elif L==3
	"Teachers"
#elif L==4
	"Profesores"
#elif L==5
	"Enseignants"
#elif L==6
	"Mbo'eh&aacute;ra"
#elif L==7
	"Professori"
#elif L==8
	"Nauczyciele"
#elif L==9
	"Professores"
#endif
	,
	//  3: ActLstOth
#if   L==1
	"Altres"
#elif L==2
	"Andere"
#elif L==3
	"Others"
#elif L==4
	"Otros"
#elif L==5
	"Autres"
#elif L==6
	"Otros"		// Okoteve traduccin
#elif L==7
	"Altri"
#elif L==8
	"Inne"
#elif L==9
	"Outros"
#endif
	,
	//  4: ActSeeAtt
#if   L==1
	"Assist&egrave;ncia"
#elif L==2
	"Anwesenheit"
#elif L==3
	"Attendance"
#elif L==4
	"Asistencia"
#elif L==5
	"Pr&eacute;sence"
#elif L==6
	"Asistencia"	// Okoteve traduccin
#elif L==7
	"Frequenza"
#elif L==8
	"Frekwencji"
#elif L==9
	"Presen&ccedil;a"
#endif
	,
	//  5: ActReqSignUp
#if   L==1
	"Inscriure"
#elif L==2
	"Anmeldung"
#elif L==3
	"Sign up"
#elif L==4
	"Inscribirme"
#elif L==5
	"Signer"
#elif L==6
	"Inscribirme"	// Okoteve traduccin
#elif L==7
	"Iscrivi"
#elif L==8
	"Zapisa&cacute;"
#elif L==9
	"Inscrever"
#endif
	,
	//  6: ActSeeSignUpReq
#if   L==1
	"Peticions"
#elif L==2
	"Ersuchen"
#elif L==3
	"Requests"
#elif L==4
	"Solicitudes"
#elif L==5
	"Demandes"
#elif L==6
	"Solicitudes"// Okoteve traduccin
#elif L==7
	"Richieste"
#elif L==8
	"Upraszanie"
#elif L==9
	"Solicita&ccedil;&otilde;es"
#endif
	,
	//  7: ActLstCon
#if   L==1
	"Connectats"
#elif L==2
	"Angemeldet"
#elif L==3
	"Connected"
#elif L==4
	"Conectados"
#elif L==5
	"Connect&eacute;s"
#elif L==6
	"Conectados"	// Okoteve traduccin
#elif L==7
	"Connessi"
#elif L==8
	"Online"
#elif L==9
	"Conectados"
#endif
	,
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabSoc *****************************************************
	//  0: ActSeeSocTmlGbl
#if   L==1
	 "Activitat"
#elif L==2
	 "Aktivit&auml;t"
#elif L==3
	 "Activity"
#elif L==4
	 "Actividad"
#elif L==5
	 "Activit&eacute;"
#elif L==6
	 "Actividad"	// Okoteve traduccin
#elif L==7
	 "Attivit&agrave;"
#elif L==8
	 "Aktywno&sacute;&cacute;"
#elif L==9
	 "Atividade"
#endif
	,
	//  1: ActSeeSocPrf
#if   L==1
	 "Perfils"
#elif L==2
	 "Profiles"
#elif L==3
	 "Profiles"
#elif L==4
	 "Perfiles"
#elif L==5
	 "Profils"
#elif L==6
	 "Che"
#elif L==7
	 "Profili"
#elif L==8
	 "Profile"
#elif L==9
	 "Perfiles"
#endif
	,
	//  2: ActSeeFor
#if   L==1
	"F&ograve;rums"
#elif L==2
	"Foren"
#elif L==3
	"Forums"
#elif L==4
	"Foros"
#elif L==5
	"Forums"
#elif L==6
	"Foros"	// Okoteve traduccin
#elif L==7
	"Forum"
#elif L==8
	"Forum"
#elif L==9
	"F&oacute;runs"
#endif
	,
	//  3: ActSeeChtRms
#if   L==1
	"Xat"
#elif L==2
	"Chat"
#elif L==3
	"Chat"
#elif L==4
	"Chat"
#elif L==5
	"Chat"
#elif L==6
	"&Ntilde;omongeta"
#elif L==7
	"Chat"
#elif L==8
	"Chat"
#elif L==9
	"Chat"
#endif
	,
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabMsg *****************************************************
	//  0: ActSeeNtf
#if   L==1
	"Notificacions"
#elif L==2
	"Anmeldungen"
#elif L==3
	"Notifications"
#elif L==4
	"Notificaciones"
#elif L==5
	"Notifications"
#elif L==6
	"Momarandu"
#elif L==7
	"Notifiche"
#elif L==8
	"Powiadomienia"
#elif L==9
	"Notifica&ccedil;&otilde;es"
#endif
	,
	//  1: ActSeeAnn
#if   L==1
	"Anuncis"
#elif L==2
	"Bekanntmachungen"
#elif L==3
	"Announcements"
#elif L==4
	"Anuncios"
#elif L==5
	"Annonces"
#elif L==6
	"Anuncios"	// Okoteve traduccin
#elif L==7
	"Annunci"
#elif L==8
	"Og&lstrok;oszenia"
#elif L==9
	"An&uacute;ncios"
#endif
	,
	//  2: ActSeeAllNot
#if   L==1
	"Avisos"
#elif L==2
	"Ank&uuml;ndigungen"
#elif L==3
	"Notices"
#elif L==4
	"Avisos"
#elif L==5
	"Avis"
#elif L==6
	"Marandu"
#elif L==7
	"Avvisi"
#elif L==8
	"Ogloszenia"
#elif L==9
	"Avisos"
#endif
	,
	//  3: ActReqMsgUsr
#if   L==1
	"Redactar"
#elif L==2
	"Schreiben"
#elif L==3
	"Write"
#elif L==4
	"Redactar"
#elif L==5
	"Ecrire"
#elif L==6
	"Redactar"	// Okoteve traduccin
#elif L==7
	"Scrivi"
#elif L==8
	"Napisz"
#elif L==9
	"Escrever"
#endif
	,
	//  4: ActSeeRcvMsg
#if   L==1
	"Rebuts"
#elif L==2
	"Empfangene"
#elif L==3
	"Received"
#elif L==4
	"Recibidos"
#elif L==5
	"Re&ccedil;us"
#elif L==6
	"Recibidos"	// Okoteve traduccin
#elif L==7
	"Ricevuti"
#elif L==8
	"Odebrane"
#elif L==9
	"Recebidas"
#endif
	,
	//  5: ActSeeSntMsg
#if   L==1
	"Enviats"
#elif L==2
	"Gesendete"
#elif L==3
	"Sent"
#elif L==4
	"Enviados"
#elif L==5
	"Envoy&eacute;s"
#elif L==6
	"Enviados"	// Okoteve traduccin
#elif L==7
	"Inviati"
#elif L==8
	"Wysylane"
#elif L==9
	"Enviadas"
#endif
	,
	//  6: ActMaiStd
#if   L==1
	"Correu"
#elif L==2
	"E-Mail"
#elif L==3
	"Email"
#elif L==4
	"Correo"
#elif L==5
	"Courrier"
#elif L==6
	"Correo"	// Okoteve traduccin
#elif L==7
	"Email"
#elif L==8
	"Email"
#elif L==9
	"Email"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabSta **********************************************************************
	//  0: ActSeeAllSvy
#if   L==1
	"Enquestes"
#elif L==2
	"Umfragen"
#elif L==3
	"Surveys"
#elif L==4
	"Encuestas"
#elif L==5
	"Sondages"
#elif L==6
	"Encuestas"	// Okoteve traduccin
#elif L==7
	"Sondaggi"
#elif L==8
	"Ankiety"
#elif L==9
	"Inqu&eacute;ritos"
#endif
	,
	//  1: ActReqUseGbl
#if   L==1
	"Xifres"
#elif L==2
	"Zahlen"
#elif L==3
	"Figures"
#elif L==4
	"Cifras"
#elif L==5
	"Chiffres"
#elif L==6
	"Cifras"	// Okoteve traduccin
#elif L==7
	"Cifre"
#elif L==8
	"Cyfry"
#elif L==9
	"N&uacute;meros"
#endif
	,
	//  2: ActSeePhoDeg
#if   L==1
	"Titulacions"
#elif L==2
	"Studiengang"
#elif L==3
	"Degrees"
#elif L==4
	"Titulaciones"
#elif L==5
	"&Eacute;tudes"
#elif L==6
	"Arandur&atilde;"
#elif L==7
	"Lauree"
#elif L==8
	"Stopnie"
#elif L==9
	"Titula&ccedil;&atilde;os"
#endif
	,
	//  3: ActReqStaCrs
#if   L==1
	"Indicadors"
#elif L==2
	"Anzeigen"
#elif L==3
	"Indicators"
#elif L==4
	"Indicadores"
#elif L==5
	"Indicateurs"
#elif L==6
	"Indicadores"	// Okoteve traduccin
#elif L==7
	"Indicatori"
#elif L==8
	"Wska&zacute;niki"
#elif L==9
	"Indicadores"
#endif
	,
	//  4: ActReqAccGbl
#if   L==1
	"Accessos"
#elif L==2
	"Anmeldungen"
#elif L==3
	"Visits"
#elif L==4
	"Accesos"
#elif L==5
	"Visites"
#elif L==6
	"Accesos"	// Okoteve traduccin
#elif L==7
	"Accessi"
#elif L==8
	"Logowanie"
#elif L==9
	"Visitas"
#endif
	,
	//  5: ActReqMyUsgRep
#if   L==1
	"Informe"
#elif L==2
	"Nutzungsbericht"
#elif L==3
	"Report"
#elif L==4
	"Informe"
#elif L==5
	"Rapport"
#elif L==6
	"Informe"	// Okoteve traduccin
#elif L==7
	"Rapporto"
#elif L==8
	"Raport"
#elif L==9
	"Relat&oacute;rio"
#endif
	,
	//  6: ActMFUAct
#if   L==1
	"Freq&uuml;ents"
#elif L==2
	"H&auml;ufige"
#elif L==3
	"Frequent"
#elif L==4
	"Frecuentes"
#elif L==5
	"Frequent"
#elif L==6
	"Frecuentes"	// Okoteve traduccin
#elif L==7
	"Frequenti"
#elif L==8
	"Cz&eogon;ste"
#elif L==9
	"Freq&uuml;entes"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabPrf *****************************************************
	//  0: ActFrmLogIn
#if   L==1
	"Entrar"
#elif L==2
	"Einloggen"
#elif L==3
	"Log in"
#elif L==4
	"Entrar"
#elif L==5
	"S'identifier"
#elif L==6
	"Entrar"	// Okoteve traduccin
#elif L==7
	"Accesso"
#elif L==8
	"Zaloguj Si&eogon;"
#elif L==9
	"Entrar"
#endif
	,
	//  1: ActFrmRolSes
#if   L==1
	"Sessi&oacute;"
#elif L==2
	"Sitzung"
#elif L==3
	"Session"
#elif L==4
	"Sesi&oacute;n"
#elif L==5
	"Session"
#elif L==6
	"Sesi&oacute;n"	// Okoteve traduccin
#elif L==7
	"Sessione"
#elif L==8
	"Sesja"
#elif L==9
	"Sess&atilde;o"
#endif
	,
	//  2: ActMyCrs
#if   L==1
	"Assignatures"
#elif L==2
	"Kursen"
#elif L==3
	"Courses"
#elif L==4
	"Asignaturas"
#elif L==5
	"Mati&egrave;res"
#elif L==6
	"Mbo'esyry"
#elif L==7
	"Corsi"
#elif L==8
	"Kurs&oacute;w"
#elif L==9
	"Disciplinas"
#endif
	,
	//  3: ActSeeMyTT
#if   L==1
	"Horari"
#elif L==2
	"Stundenplan"
#elif L==3
	"Timetable"
#elif L==4
	"Horario"
#elif L==5
	"Horaire"
#elif L==6
	"Horario"	// Okoteve traduccin
#elif L==7
	"Orario"
#elif L==8
	"Plan"
#elif L==9
	"Hor&aacute;rio"
#endif
	,
	//  4: ActSeeMyAgd
#if   L==1
	"Agenda"
#elif L==2
	"Organizer"
#elif L==3
	"Agenda"
#elif L==4
	"Agenda"
#elif L==5
	"Agenda"
#elif L==6
	"Aporyr&atilde;"
#elif L==7
	"Agenda"
#elif L==8
	"Pami&eogon;tnik"
#elif L==9
	"Agenda"
#endif
	,
	//  5: ActFrmMyAcc
#if   L==1
	"Compte"
#elif L==2
	"Benutzerkonto"
#elif L==3
	"Account"
#elif L==4
	"Cuenta"
#elif L==5
	"Compte"
#elif L==6
	"Cuenta"	// Okoteve traduccin
#elif L==7
	"Account"
#elif L==8
	"Konto"
#elif L==9
	"Conta"
#endif
	,
	//  6: ActReqEdiRecCom
#if   L==1
	"Fitxa"
#elif L==2
	"Profil"
#elif L==3
	"Record"
#elif L==4
	"Ficha"
#elif L==5
	"Carte"
#elif L==6
	"Ficha"	// Okoteve traduccin
#elif L==7
	"Scheda"
#elif L==8
	"Karta"
#elif L==9
	"Cart&atilde;o"
#endif
	,
	//  7: ActEdiPrf
#if   L==1
	"Prefer&egrave;ncies"
#elif L==2
	"Einstellungen"
#elif L==3
	"Preferences"
#elif L==4
	"Preferencias"
#elif L==5
	"Pr&eacute;f&eacute;rences"
#elif L==6
	"Mbohoryha"
#elif L==7
	"Preferenze"
#elif L==8
	"Preferencje"
#elif L==9
	"Prefer&ecirc;ncias"
#endif
	,
	//  8: ActAdmBrf
#if   L==1
	"Malet&iacute;"
#elif L==2
	"Aktentasche"
#elif L==3
	"Briefcase"
#elif L==4
	"Malet&iacute;n"
#elif L==5
	"Portedocuments"
#elif L==6
	"Malet&iacute;n"	// Okoteve traduccin
#elif L==7
	"Cartella"
#elif L==8
	"Teczka"
#elif L==9
	"Pasta"
#endif
	,
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	}
};

const char *Txt_MENU_SUBTITLE[Tab_NUM_TABS][Act_MAX_OPTIONS_IN_MENU_PER_TAB] =
{
	{
	// TabUnk ****************************************************
	NULL,	//  0
	NULL,	//  1
	NULL,	//  2
	NULL,	//  3
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabSys *****************************************************
	//  0: ActSysReqSch
#if   L==1
	"Cerca d'assignatures, professors i documents"
#elif L==2
	"Suche Kursen, Lehrer und Dokumente"
#elif L==3
	"Search subjects, teachers and documents"
#elif L==4
	"B&uacute;squeda de asignaturas, profesores y documentos"
#elif L==5
	"Rechercher des mati&egrave;res, des enseignants et des documents"
#elif L==6
	"B&uacute;squeda de asignaturas, profesores y documentos"// Okoteve traduccin
#elif L==7
	"Cercare corsi, professori e documenti"
#elif L==8
	"Szukaj przedmiot&oacute;w, nauczycieli i dokumenty"
#elif L==9
	"Procurar disciplinas, professores e documentos"
#endif
	,
	//  1: ActSeeCty
#if   L==1
	"Llista de pa&iuml;sos i del nombre d'usuaris de cada pa&iacute;s"
#elif L==2
	"L&auml;nder"
#elif L==3
	"Countries"
#elif L==4
	"Lista de pa&iacute;ses y del n&uacute;mero de usuarios de cada pais"
#elif L==5
	"Pays"
#elif L==6
	"Lista de pa&iacute;ses y del n&uacute;mero de usuarios de cada pais"// Okoteve traduccin
#elif L==7
	"Paesi"
#elif L==8
	"Kraje"
#elif L==9
	"Pa&iacute;ses"
#endif
	,
	//  2: ActSeePen
#if   L==1
	"Institutions, centres, titulacions i assignatures pendents d'activar"
#elif L==2
	"Anh&auml;ngig Hochschulen, Lehrinstitute, Studiengang und Kursen"
#elif L==3
	"Pending institutions, centres, degrees and courses"
#elif L==4
	"Instituciones, centros, titulaciones y asignaturas pendientes de activar"
#elif L==5
	"&Eacute;tablissements, centres, &eacute;tudes et mati&egrave;res en attente"
#elif L==6
	"Instituciones, centros, titulaciones y asignaturas pendientes de activar"	// Okoteve traduccin
#elif L==7
	"Istituzioni, centri, lauree e corsi in sospeso"
#elif L==8
	"Instytucje, centra, stopnie i kurs&otilde;w oczekuj&aogon;cy"
#elif L==9
	"Institu&ccedil;&otilde;es, centros, titula&ccedil;&otilde;es e disciplinas pendentes"
#endif
	,
	//  3: ActReqRemOldCrs
#if   L==1
	"Eliminar assignatures antics"
#elif L==2
	"Fr&uuml;here Kursen entfernen"
#elif L==3
	"Eliminate old courses"
#elif L==4
	"Eliminar asignaturas antiguas"
#elif L==5
	"&Eacute;liminer anciens mati&egrave;res"
#elif L==6
	"Eliminar asignaturas antiguas"	// Okoteve traduccin
#elif L==7
	"Rimuovere vecchi corsi"
#elif L==8
	"Wyeleminuj starych kurs&oacute;w"
#elif L==9
	"Eliminar antigas disciplinas"
#endif
	,
	//  4: ActSeeDegTyp
#if   L==1
	"Tipus de titulaci&oacute;"
#elif L==2
	"Abschlussarten"
#elif L==3
	"Degree types"
#elif L==4
	"Tipos de titulaci&oacute;n"
#elif L==5
	"Types d'&eacute;tude"
#elif L==6
	"Arandur&atilde; l&aacute;ja"
#elif L==7
	"Tipi di laurea"
#elif L==8
	"Typy stopnia"
#elif L==9
	"Tipos de titula&ccedil;&atilde;o"
#endif
	,
	//  5: ActSeeMai
#if   L==1
	"Llista de dominis de correu permesos"
	" per a l'enviament de notificacions"
#elif L==2
	"E-Mail-Dom&auml;nen f&uuml;r Benachrichtigungen"
#elif L==3
	"Email domains for notifications"
#elif L==4
	"Lista de dominios de correo permitidos"
	" para el env&iacute;o de notificaciones"
#elif L==5
	"Domaines de messagerie pour notifications"
#elif L==6
	"Lista de dominios de correo permitidos"
	" para el env&iacute;o de notificaciones"// Okoteve traduccin
#elif L==7
	"Campi mail per invio di notifiche"
#elif L==8
	"Domen Mail powiadomien"
#elif L==9
	"Dom&iacute;nios de email"
	  " para notifica&ccedil;&otilde;es"
#endif
	,
	//  6: ActSeeBan
#if   L==1
	"Llista de banners publicitaris"
#elif L==2
	"Werbebanners Liste"
#elif L==3
	"Banners list"
#elif L==4
	"Lista de banners publicitarios"
#elif L==5
	"Liste des banni&eacute;res publicitaires"
#elif L==6
	"Lista de banners publicitarios"// Okoteve traduccin
#elif L==7
	"Lista di banners"
#elif L==8
	"Lista banery"
#elif L==9
	"Lista de banners publicit&aacute;rios"
#endif
	,
	//  7: ActSeeLnk
#if   L==1
	"Llista d'enlla&ccedil;os institucionals"
#elif L==2
	"Offizielle Links Liste"
#elif L==3
	"Institutional links list"
#elif L==4
	"Lista de enlaces institucionales"
#elif L==5
	"Liens institutionnels"
#elif L==6
	"Lista de enlaces institucionales"// Okoteve traduccin
#elif L==7
	"Link istituzionali"
#elif L==8
	"Instytucjonalne linki"
#elif L==9
	"Liga&ccedil;&otilde;es institucionais"
#endif
	,
	//  8: ActLstPlg
#if   L==1
	"Llista de complements (versi&oacute; beta)"
#elif L==2
	"Plugin-Liste"
#elif L==3
	"List of plugins"
#elif L==4
	"Lista de complementos (versi&oacute;n beta)"
#elif L==5
	"Liste de plugins"
#elif L==6
	"Lista de complementos (versi&oacute;n beta)"// Okoteve traduccin
#elif L==7
	"Lista di plugin"
#elif L==8
	"Lista plugin&oacute;w"
#elif L==9
	"Lista de plugins"
#endif
	,
	//  9: ActSetUp
#if   L==1
	"Instal laci&oacute; de la plataforma (en proves)"
#elif L==2
	"Einrichten"
#elif L==3
	"Set up"
#elif L==4
	"Instalaci&oacute;n de la plataforma (en pruebas)"
#elif L==5
	"Installer"
#elif L==6
	"Instalaci&oacute;n de la plataforma (en pruebas)"// Okoteve traduccin
#elif L==7
	"Setup"
#elif L==8
	"Set up"
#elif L==9
	"Instalar"
#endif
	,
	// 10: ActSeeCalSys
#if   L==1
	"Calendari acad&egrave;mic"
#elif L==2
	"Kalender"
#elif L==3
	"Academic calendar"
#elif L==4
	"Calendario acad&eacute;mico"
#elif L==5
	"Calendrier"
#elif L==6
	"Calendario acad&eacute;mico"// Okoteve traduccin
#elif L==7
	"Calendario del corso"
#elif L==8
	"Kalendarz akademicki"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	// 11
	},
	{
	// TabCty *****************************************************
	//  0: ActCtyReqSch
#if   L==1
	"Cerca d'assignatures, professors i documents"
#elif L==2
	"Suche Kursen, Lehrer und Dokumente"
#elif L==3
	"Search subjects, teachers and documents"
#elif L==4
	"B&uacute;squeda de asignaturas, profesores y documentos"
#elif L==5
	"Rechercher des mati&egrave;res, des enseignants et des documents"
#elif L==6
	"B&uacute;squeda de asignaturas, profesores y documentos"// Okoteve traduccin
#elif L==7
	"Cercare corsi, professori e documenti"
#elif L==8
	"Szukaj przedmiot&oacute;w, nauczycieli i dokumenty"
#elif L==9
	"Procurar disciplinas, professores e documentos"
#endif
	,
	//  1: ActSeeCtyInf
#if   L==1
	"Informaci&oacute; d'aquest pa&iacute;s"
#elif L==2
	"Information dieses Land"
#elif L==3
	"Information on this country"
#elif L==4
	"Informaci&oacute;n sobre este pa&iacute;s"
#elif L==5
	"Information de cet pays"
#elif L==6
	"Informaci&oacute;n sobre este pa&iacute;s"// Okoteve traduccin
#elif L==7
	"Informazione di questo paese"
#elif L==8
	"Informacja na ten kraj"
#elif L==9
	"Informa&ccedil;&atilde;o deste pa&iacute;s"
#endif
	,
	//  2: ActSeeIns
#if   L==1
	"Llista d'institucions"
#elif L==2
	"Hochschulen"
#elif L==3
	"List of institutions of the current country"
#elif L==4
	"Lista de instituciones (universidades, institutos&hellip;) de este pa&iacute;s"
#elif L==5
	"&Eacute;tablissement"
#elif L==6
	"Lista de instituciones (universidades, institutos&hellip;) de este pa&iacute;s"// Okoteve traduccin
#elif L==7
	"Istituzioni"
#elif L==8
	"Instytucje"
#elif L==9
	"Institu&ccedil;&otilde;es"
#endif
	,
	//  3: ActSeeCalCty
#if   L==1
	"Calendari acad&egrave;mic"
#elif L==2
	"Kalender"
#elif L==3
	"Academic calendar"
#elif L==4
	"Calendario acad&eacute;mico"
#elif L==5
	"Calendrier"
#elif L==6
	"Calendario acad&eacute;mico"// Okoteve traduccin
#elif L==7
	"Calendario del corso"
#elif L==8
	"Kalendarz akademicki"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabIns *****************************************************
	//  0: ActInsReqSch
#if   L==1
	"Cerca d'assignatures, professors i documents"
#elif L==2
	"Suche Kursen, Lehrer und Dokumente"
#elif L==3
	"Search subjects, teachers and documents"
#elif L==4
	"B&uacute;squeda de asignaturas, profesores y documentos"
#elif L==5
	"Rechercher des mati&egrave;res, des enseignants et des documents"
#elif L==6
	"B&uacute;squeda de asignaturas, profesores y documentos"// Okoteve traduccin
#elif L==7
	"Cercare corsi, professori e documenti"
#elif L==8
	"Szukaj przedmiot&oacute;w, nauczycieli i dokumenty"
#elif L==9
	"Procurar disciplinas, professores e documentos"
#endif
	,
	//  1: ActSeeInsInf
#if   L==1
	"Informaci&oacute; d'aquesta instituci&oacute;"
#elif L==2
	"Information dieses Hochschule"
#elif L==3
	"Information on this institution"
#elif L==4
	"Informaci&oacute;n sobre esta instituci&oacute;n"
#elif L==5
	"Information de cet &eacute;tablissement"
#elif L==6
	"Informaci&oacute;n sobre esta instituci&oacute;n"// Okoteve traduccin
#elif L==7
	"Informazione di questa istituzione"
#elif L==8
	"Informacja na ten instytucja"
#elif L==9
	"Informa&ccedil;&atilde;o desta institu&ccedil;&atilde;o"
#endif
	,
	//  2: ActSeeCtr
#if   L==1
	"Llista de centres"
#elif L==2
	"Lehrinstitute"
#elif L==3
	"List of centres of the current institution"
#elif L==4
	"Lista de centros (facultades, escuelas&hellip;) de esta instituci&oacute;n"
#elif L==5
	"Centres"
#elif L==6
	"Lista de centros (facultades, escuelas&hellip;) de esta instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Centri"
#elif L==8
	"Centra"
#elif L==9
	"Centros"
#endif
	,
	//  3: ActSeeDpt
#if   L==1
	"Llista de tots els departaments d'aquesta instituci&oacute;"
#elif L==2
	"Abteilungen"
#elif L==3
	"List all departments of this institution"
#elif L==4
	"Lista de todos los departamentos de esta instituci&oacute;n"
#elif L==5
	"D&eacute;partements"
#elif L==6
	"Lista de todos los departamentos de esta instituci&oacute;n"// Okoteve traduccin
#elif L==7
	"Dipartimenti"
#elif L==8
	"Wydzia&lstrok;y"
#elif L==9
	"Departamentos"
#endif
	,
	//  4: ActSeePlc
#if   L==1
	"Llista d'empla&ccedil;aments d'aquesta instituci&oacute;"
#elif L==2
	"Standorte"
#elif L==3
	"List of places of this institution"
#elif L==4
	"Lista de emplazamientos de esta instituci&oacute;n"
#elif L==5
	"Emplacements"
#elif L==6
	"Lista de emplazamientos de esta instituci&oacute;n"// Okoteve traduccin
#elif L==7
	"Localit&agrave;"
#elif L==8
	"Miejsca"
#elif L==9
	"Localiza&ccedil;oes"
#endif
	,
	//  5: ActSeeHld
#if   L==1
	"Dies festius i per&iacute;odes no lectius d'aquesta instituci&oacute;"
#elif L==2
	"Urlaub und vorlesungsfreie Zeit"
#elif L==3
	"Holidays and non school days of this institution"
#elif L==4
	"D&iacute;as festivos y per&iacute;odos no lectivos de esta instituci&oacute;n"
#elif L==5
	"Jours f&eacute;ri&eacute;s et sans &eacute;cole"
#elif L==6
	"D&iacute;as festivos y per&iacute;odos no lectivos de esta instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Festivit&agrave; e giorni senza lezione"
#elif L==8
	"Noclegi i innych dni zajec szkolnych"
#elif L==9
	"Feriados e dias sem escola"
#endif
	,
	//  6: ActSeeCalIns
#if   L==1
	"Calendari acad&egrave;mic"
#elif L==2
	"Kalender"
#elif L==3
	"Academic calendar"
#elif L==4
	"Calendario acad&eacute;mico"
#elif L==5
	"Calendrier"
#elif L==6
	"Calendario acad&eacute;mico"// Okoteve traduccin
#elif L==7
	"Calendario del corso"
#elif L==8
	"Kalendarz akademicki"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabCtr *****************************************************
	//  0: ActCtrReqSch
#if   L==1
	"Cerca d'assignatures, professors i documents"
#elif L==2
	"Suche Kursen, Lehrer und Dokumente"
#elif L==3
	"Search subjects, teachers and documents"
#elif L==4
	"B&uacute;squeda de asignaturas, profesores y documentos"
#elif L==5
	"Rechercher des mati&egrave;res, des enseignants et des documents"
#elif L==6
	"B&uacute;squeda de asignaturas, profesores y documentos"// Okoteve traduccin
#elif L==7
	"Cercare corsi, professori e documenti"
#elif L==8
	"Szukaj przedmiot&oacute;w, nauczycieli i dokumenty"
#elif L==9
	"Procurar disciplinas, professores e documentos"
#endif
	,
	//  1: ActSeeCtrInf
#if   L==1
	"Informaci&oacute; d'aquest centre"
#elif L==2
	"Information dieses Lehrinstitut"
#elif L==3
	"Information on this centre"
#elif L==4
	"Informaci&oacute;n sobre este centro"
#elif L==5
	"Information de cet centre"
#elif L==6
	"Informaci&oacute;n sobre este centro"// Okoteve traduccin
#elif L==7
	"Informazione di questo centro"
#elif L==8
	"Informacja na ten centrum"
#elif L==9
	"Informa&ccedil;&atilde;o deste centro"
#endif
	,
	//  2: ActSeeDeg
#if   L==1
	"Titulacions d'aquest centre"
#elif L==2
	"Studiengang"
#elif L==3
	"List of degrees of the current centre"
#elif L==4
	"Lista de titulaciones (carreras, m&aacute;steres&hellip;) de este centro"
#elif L==5
	"&Eacute;tudes"
#elif L==6
	"Lista de titulaciones (carreras, m&aacute;steres&hellip;) de este centro"// Okoteve traduccin
#elif L==7
	"Lauree"
#elif L==8
	"Stopnie"
#elif L==9
	"Titula&ccedil;&atilde;os"
#endif
	,
	//  3: ActSeeCalCtr
#if   L==1
	"Calendari acad&egrave;mic (els dies festius"
	" corresponen al lloc del centre seleccionada)"
#elif L==2
	"Kalender"
#elif L==3
	"Academic calendar"
#elif L==4
	"Calendario acad&eacute;mico (los d&iacute;as festivos"
	" corresponden al lugar del centro seleccionado)"
#elif L==5
	"Calendrier"
#elif L==6
	"Calendario acad&eacute;mico (los d&iacute;as festivos"
	" corresponden al lugar del centro seleccionado)"// Okoteve traduccin
#elif L==7
	"Calendario del corso"
#elif L==8
	"Kalendarz akademicki"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabDeg *****************************************************
	//  0: ActDegReqSch
#if   L==1
	"Cerca d'assignatures, professors i documents"
#elif L==2
	"Suche Kursen, Lehrer und Dokumente"
#elif L==3
	"Search subjects, teachers and documents"
#elif L==4
	"B&uacute;squeda de asignaturas, profesores y documentos"
#elif L==5
	"Rechercher des mati&egrave;res, des enseignants et des documents"
#elif L==6
	"B&uacute;squeda de asignaturas, profesores y documentos"// Okoteve traduccin
#elif L==7
	"Cercare corsi, professori e documenti"
#elif L==8
	"Szukaj przedmiot&oacute;w, nauczycieli i dokumenty"
#elif L==9
	"Procurar disciplinas, professores e documentos"
#endif
	,
	//  1: ActSeeDegInf
#if   L==1
	"Informaci&oacute; d'aquesta titulaci&oacute;"
#elif L==2
	"Information dieses Studiengang"
#elif L==3
	"Information on this degree"
#elif L==4
	"Informaci&oacute;n sobre esta titulaci&oacute;n"
#elif L==5
	"Information de cet &eacute;tude"
#elif L==6
	"Informaci&oacute;n sobre esta titulaci&oacute;n"// Okoteve traduccin
#elif L==7
	"Informazione di questa laurea"
#elif L==8
	"Informacja na ten stopie&nacute;"
#elif L==9
	"Informa&ccedil;&atilde;o desta titula&ccedil;&atilde;o"
#endif
	,
	//  2: ActSeeCrs
#if   L==1
	"Assignatures d'aquesta titulaci&oacute;"
#elif L==2
	"Kursen in diesem Studiengang"
#elif L==3
	"List of ourses of the current degree"
#elif L==4
	"Lista de asignaturas de esta titulaci&oacute;n"
#elif L==5
	"Mati&egrave;res dans cet &eacute;tude"
#elif L==6
	"Lista de asignaturas de esta titulaci&oacute;n"// Okoteve traduccin
#elif L==7
	"Corsi di questa laurea"
#elif L==8
	"Kurs&oacute;w w tym stopie&nacute;"
#elif L==9
	"Disciplinas nesta titula&ccedil;&atilde;o"
#endif
	,
	//  3: ActSeeCalDeg
#if   L==1
	"Calendari acad&egrave;mic (els dies festius"
	" corresponen al lloc del centre seleccionada)"
#elif L==2
	"Kalender"
#elif L==3
	"Academic calendar"
#elif L==4
	"Calendario acad&eacute;mico (los d&iacute;as festivos"
	" corresponden al lugar del centro seleccionado)"
#elif L==5
	"Calendrier"
#elif L==6
	"Calendario acad&eacute;mico (los d&iacute;as festivos"
	" corresponden al lugar del centro seleccionado)"// Okoteve traduccin
#elif L==7
	"Calendario del corso"
#elif L==8
	"Kalendarz akademicki"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabCrs *****************************************************
	//  0: ActCrsReqSch
#if   L==1
	"Cerca de professors i documents"
#elif L==2
	"Suche Lehrer und Dokumente"
#elif L==3
	"Search teachers and documents"
#elif L==4
	"B&uacute;squeda de profesores y documentos"
#elif L==5
	"Rechercher des enseignants et des documents"
#elif L==6
	"B&uacute;squeda de profesores y documentos"// Okoteve traduccin
#elif L==7
	"Cercare professori e documenti"
#elif L==8
	"Szukaj nauczycieli i dokumenty"
#elif L==9
	"Procurar professores e documentos"
#endif
	,
	//  1: ActSeeCrsInf
#if   L==1
	"Informaci&oacute; d'aquesta assignatura"
#elif L==2
	"Information dieses Kurs"
#elif L==3
	"Information on this course"
#elif L==4
	"Informaci&oacute;n sobre esta asignatura"
#elif L==5
	"Information de cette mati&egrave;re"
#elif L==6
	"Informaci&oacute;n sobre esta asignatura"// Okoteve traduccin
#elif L==7
	"Informazione di questo corso"
#elif L==8
	"Informacja na ten kurs"
#elif L==9
	"Informa&ccedil;&atilde;o desta disciplina"
#endif
	,
	//  2: ActSeeTchGui
#if   L==1
	"Guia docent de l'assignatura"
#elif L==2
	"Hinweise zur Kurs"
#elif L==3
	"Teaching guide"
#elif L==4
	"Gu&iacute;a docente de la asignatura"
#elif L==5
	"Guide p&eacute;dagogique de la mati&egrave;re"
#elif L==6
	"Gu&iacute;a docente de la asignatura"// Okoteve traduccin
#elif L==7
	"Guida didattica del corso"
#elif L==8
	"Nauczanie przypomnienie"
#elif L==9
	"Guia de ensino"
#endif
	,
	//  3: ActSeeSyl
#if   L==1
	"Programa de l'assignatura"
#elif L==2
	"Studienplan"
#elif L==3
	"Syllabus"
#elif L==4
	"Programa de la asignatura"
#elif L==5
	"Plan"
#elif L==6
	"Programa de la asignatura"// Okoteve traduccin
#elif L==7
	"Programma"
#elif L==8
	"Syllabus"
#elif L==9
	"Programa"
#endif
	,
	//  4: ActSeeBib
#if   L==1
	"Bibliografia relacionada amb els continguts de l'assignatura"
#elif L==2
	"Literatur"
#elif L==3
	"Bibliography"
#elif L==4
	"Bibliograf&iacute;a relacionada con los contenidos de la asignatura"
#elif L==5
	"Bibliographie"
#elif L==6
	"Bibliograf&iacute;a relacionada con los contenidos de la asignatura"// Okoteve traduccin
#elif L==7
	"Bibliografia"
#elif L==8
	"Bibliografia"	// Potrzebujesz tlumaczenie
#elif L==9
	"Bibliografia"
#endif
	,
	//  5: ActSeeFAQ
#if   L==1
	"FAQ (preguntes freq&uuml;entment formulades i les seves respostes)"
#elif L==2
	"FAQ"
#elif L==3
	"FAQ (Frequently Asked Questions)"
#elif L==4
	"FAQ (preguntas frecuentemente formuladas y sus respuestas)"
#elif L==5
	"FAQ"
#elif L==6
	"FAQ (preguntas frecuentemente formuladas y sus respuestas)"// Okoteve traduccin
#elif L==7
	"FAQ"
#elif L==8
	"FAQ (Frequently Asked Questions)"
#elif L==9
	"FAQ"
#endif
	,
	//  6: ActSeeCrsLnk
#if   L==1
	"Enlla&ccedil;os relacionats amb els continguts de l'assignatura"
#elif L==2
	"Links"
#elif L==3
	"Links"
#elif L==4
	"Enlaces relacionados con los contenidos de la asignatura"
#elif L==5
	"Liens"
#elif L==6
	"Enlaces relacionados con los contenidos de la asignatura"// Okoteve traduccin
#elif L==7
	"Link"
#elif L==8
	"Linki"
#elif L==9
	"Liga&ccedil;&otilde;es"
#endif
	,
	//  7: ActSeeCrsTT
#if   L==1
	"Horari de classes de l'assignatura"
#elif L==2
	"Stundenplan"
#elif L==3
	"Class timetable"
#elif L==4
	"Horario de clases de la asignatura"
#elif L==5
	"Horaire"
#elif L==6
	"Horario de clases de la asignatura"	// Okoteve traduccin
#elif L==7
	"Orario del corso"
#elif L==8
	"Harmonogram klasy"
#elif L==9
	"Hor&aacute;rio"
#endif
	,
	//  8: ActSeeCalCrs
#if   L==1
	"Calendari acad&egrave;mic (els dies festius"
	" corresponen al lloc del centre seleccionada)"
#elif L==2
	"Kalender"
#elif L==3
	"Academic calendar"
#elif L==4
	"Calendario acad&eacute;mico (los d&iacute;as festivos"
	" corresponden al lugar del centro seleccionado)"
#elif L==5
	"Calendrier"
#elif L==6
	"Calendario acad&eacute;mico (los d&iacute;as festivos"
	" corresponden al lugar del centro seleccionado)"// Okoteve traduccin
#elif L==7
	"Calendario del corso"
#elif L==8
	"Kalendarz akademicki"
#elif L==9
	"Calend&aacute;rio"
#endif
	,
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabAss *****************************************************
	//  0: ActSeeAss
#if   L==1
	"Sistema o criteris d'avaluaci&oacute;"
	" de l'estudiant en aquesta assignatura"
#elif L==2
	"Bewertungssystem"
#elif L==3
	"Assessment system"
#elif L==4
	"Sistema o criterios de evaluaci&oacute;n"
	" del estudiante en esta asignatura"
#elif L==5
	"Syst&egrave;me d'&eacute;valuation"
#elif L==6
	"Sistema o criterios de evaluaci&oacute;n"
	" del estudiante en esta asignatura"// Okoteve traduccin
#elif L==7
	"Sistema di valutazione"
#elif L==8
	"System oceny"
#elif L==9
	"Sistema de avalia&ccedil;&atilde;o"
#endif
	,
	//  1: ActSeeAsg
#if   L==1
	"Activitats obligat&ograve;ries o recomanades en aquesta assignatura"
#elif L==2
	"Aufgaben"
#elif L==3
	"Assignments"
#elif L==4
	"Actividades obligatorias o recomendadas en esta asignatura"
#elif L==5
	"Activit&eacute;s"
#elif L==6
	"Actividades obligatorias o recomendadas en esta asignatura"// Okoteve traduccin
#elif L==7
	"Attivit&agrave;"
#elif L==8
	"Zadania"
#elif L==9
	"Atividades"
#endif
	,
	//  2: ActReqTst
#if   L==1
	"Tests sobre els continguts d'aquesta assignatura"
#elif L==2
	"Tests"
#elif L==3
	"Tests on the contents of this course"
#elif L==4
	"Tests sobre los contenidos de esta asignatura"
#elif L==5
	"Tests sur le contenu de cette mati&egrave;re"
#elif L==6
	"Tests sobre los contenidos de esta asignatura"// Okoteve traduccin
#elif L==7
	"Test sui contenuti del corso"
#elif L==8
	"Test&oacute;w"
#elif L==9
	"Testes"
#endif
	,
	//  3: ActSeeAllExaAnn
#if   L==1
	"Llistat de convocat&ograve;ries d'ex&agrave;mens d'aquesta assignatura"
#elif L==2
	"Pr&uuml;fungsaufruf"
#elif L==3
	"Announcements of exams"
#elif L==4
	"Listado de convocatorias de ex&aacute;menes de esta asignatura"
#elif L==5
	"Convocations &agrave; examens"
#elif L==6
	"Listado de convocatorias de ex&aacute;menes de esta asignatura"// Okoteve traduccin
#elif L==7
	"Appelli d'esame"
#elif L==8
	"Ogloszenia egzamin&oacute;w"
#elif L==9
	"Chamadas para exames"
#endif
	,
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabFil *****************************************************
	//  0: ActSeeAdmDocIns
#if   L==1
	"Documents"
#elif L==2
	"Dokumente"
#elif L==3
	"Documents of the institution uploaded by administrators"
#elif L==4
	"Documentos de la instituci&oacute;n subidos por administradores"
#elif L==5
	"Documents"
#elif L==6
	"Documentos de la instituci&oacute;n subidos por administradores"	// Okoteve traduccin
#elif L==7
	"Area documenti"
#elif L==8
	"Obszar Dokument&oacute;w"
#elif L==9
	"Documentos"
#endif
	,
	//  1: ActAdmShaIns
#if   L==1
	"Arxius compartits per professors i estudiants"
	" de l'instituci&oacute;"
#elif L==2
	"Freigegebene Dateien"
#elif L==3
	"Files shared by teachers and students"
	" of the institution"
#elif L==4
	"Archivos compartidos por profesores y estudiantes"
	" de la instituci&oacute;n"
#elif L==5
	"Fichiers partag&eacute;s"
#elif L==6
	"Archivos compartidos por profesores y estudiantes"
	" de la instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	"File condivisi"
#elif L==8
	"Pliki udost&eogon;pnione"
#elif L==9
	"Arquivos compartilhados"
#endif
	,
	//  2: ActSeeAdmDocCtr
#if   L==1
	"Documents"
#elif L==2
	"Dokumente"
#elif L==3
	"Documents of the centre uploaded by administrators"
#elif L==4
	"Documentos del centro subidos por administradores"
#elif L==5
	"Documents"
#elif L==6
	"Documentos del centro subidos por administradores"	// Okoteve traduccin
#elif L==7
	"Area documenti"
#elif L==8
	"Obszar Dokument&oacute;w"
#elif L==9
	"Documentos"
#endif
	,
	//  3: ActAdmShaCtr
#if   L==1
	"Arxius compartits per professors i estudiants"
	" del centre"
#elif L==2
	"Freigegebene Dateien"
#elif L==3
	"Files shared by teachers and students"
	" of the centre"
#elif L==4
	"Archivos compartidos por profesores y estudiantes"
	" del centro"
#elif L==5
	"Fichiers partag&eacute;s"
#elif L==6
	"Archivos compartidos por profesores y estudiantes"
	" del centro"	// Okoteve traduccin
#elif L==7
	"File condivisi"
#elif L==8
	"Pliki udost&eogon;pnione"
#elif L==9
	"Arquivos compartilhados"
#endif
	,
	//  4: ActSeeAdmDocDeg
#if   L==1
	"Documents"
#elif L==2
	"Dokumente"
#elif L==3
	"Documents of the degree uploaded by administrators"
#elif L==4
	"Documentos de la titulaci&oacute;n subidos por administradores"
#elif L==5
	"Documents"
#elif L==6
	"Documentos de la titulaci&oacute;n subidos por administradores"	// Okoteve traduccin
#elif L==7
	"Area documenti"
#elif L==8
	"Obszar Dokument&oacute;w"
#elif L==9
	"Documentos"
#endif
	,
	//  5: ActAdmShaDeg
#if   L==1
	"Arxius compartits per professors i estudiants"
	" de la titulaci&oacute;"
#elif L==2
	"Freigegebene Dateien"
#elif L==3
	"Files shared by teachers and students"
	" of the degree"
#elif L==4
	"Archivos compartidos por profesores y estudiantes"
	" de la titulaci&oacute;n"
#elif L==5
	"Fichiers partag&eacute;s"
#elif L==6
	"Archivos compartidos por profesores y estudiantes"
	" de la titulaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"File condivisi"
#elif L==8
	"Pliki udost&eogon;pnione"
#elif L==9
	"Arquivos compartilhados"
#endif
	,
	//  6: ActSeeAdmDocCrsGrp
#if   L==1
	"Documents de l'assignatura o d'un grup enviats per professors"
#elif L==2
	"Dokumente"
#elif L==3
	"Documents of the course or a group uploaded by teachers"
#elif L==4
	"Documentos de la asignatura o de un grupo subidos por profesores"
#elif L==5
	"Documents"
#elif L==6
	"Documentos de la asignatura o de un grupo subidos por profesores"// Okoteve traduccin
#elif L==7
	"Area documenti"
#elif L==8
	"Obszar Dokument&oacute;w"
#elif L==9
	"Documentos"
#endif
	,
	//  7: ActAdmTchCrsGrp
#if   L==1
	"Arxius privats, nom&eacute;s per als professors de l'assignatura"
#elif L==2
	"Privat-Dateien, nur f&uuml;r die Lehrer des Kurses"
#elif L==3
	"Private files, only for teachers of the course"
#elif L==4
	"Archivos privados, solo para los profesores de la asignatura"
#elif L==5
	"Fichiers priv&eacute;s, seulement pour les enseignants du cours"
#elif L==6
	"Archivos privados, solo para los profesores de la asignatura"// Okoteve traduccin
#elif L==7
	"File privati, solo per gli insegnanti del corso"
#elif L==8
	"Pliki prywatne, tylko dla nauczycieli przedmiotu"
#elif L==9
	"Arquivos privados, apenas para os professores do curso"
#endif
	,
	//  8: ActAdmShaCrsGrp
#if   L==1
	"Arxius compartits per professors i estudiants"
	" de l'assignatura o d'un grup"
#elif L==2
	"Freigegebene Dateien"
#elif L==3
	"Files shared by teachers and students"
	" of the course or group"
#elif L==4
	"Archivos compartidos por profesores y estudiantes"
	" de la asignatura o de un grupo"
#elif L==5
	"Fichiers partag&eacute;s"
#elif L==6
	"Archivos compartidos por profesores y estudiantes"
	" de la asignatura o de un grupo"// Okoteve traduccin
#elif L==7
	"File condivisi"
#elif L==8
	"Pliki udost&eogon;pnione"
#elif L==9
	"Arquivos compartilhados"
#endif
	,
	//  9: ActAdmAsgWrkUsr
#if   L==1
	"Els meus documents d'activitats i altres treballs en aquesta assignatura"
#elif L==2
	"Eingereichte Hausarbeiten"
#elif L==3
	"My homework"
#elif L==4
	"Mis documentos de actividades y otros trabajos en esta asignatura"
#elif L==5
	"Mes travaux"
#elif L==6
	"Mis documentos de actividades y otros trabajos en esta asignatura"// Okoteve traduccin
#elif L==7
	"Miei lavori"
#elif L==8
	"Moje zadanie domowe"
#elif L==9
	"Os meus trabalhos"
#endif
	,
	// 10: ActReqAsgWrkCrs
#if   L==1
	"Activitats i altres treballs enviats pels usuaris d'aquesta assignatura"
#elif L==2
	"Hausarbeiten der Benutzer"
#elif L==3
	"Users' homework"
#elif L==4
	"Actividades y otros trabajos enviados por los usuarios de esta asignatura"
#elif L==5
	"Travails d'utilisateurs"
#elif L==6
	"Actividades y otros trabajos enviados por los usuarios de esta asignatura"// Okoteve traduccin
#elif L==7
	"Lavori di utenti"
#elif L==8
	"Uzytkownik&oacute;w domowych"
#elif L==9
	"Trabalhos de utilizadores"
#endif
	,
	// 11: ActSeeMrk
#if   L==1
	"Consulta individualitzada de qualificacions"
	" (cada estudiant nom&eacute;s veu les seves qualificacions)"
#elif L==2
	"Individuelle Beratung von Bewertungen"
	" (jeder Student sieht nur seine Bewertungen)"
#elif L==3
	"Individualized consultation of marks"
	"(each student only sees her/his marks)"
#elif L==4
	"Consulta individualizada de calificaciones"
	" (cada estudiante solo ve sus calificaciones)"
#elif L==5
	"Consultation individualis&eacute; des notes"
	" (chaque &eacute;tudiant ne voit que ses notes)"
#elif L==6
	"Consulta individualizada de calificaciones"
	" (cada estudiante solo ve sus calificaciones)"	// Okoteve traduccin
#elif L==7
	"Consultazione individualizzata dei risultati"
	" (ogni studente vede solo i suoi risultati)"
#elif L==8
	"Zindywidualizowane konsultacje znak&oacute;w"
	" (ka&zdot;dy student widzi tylko jego &sacute;lady)"
#elif L==9
	"Consulta individualizada de notas"
	" (somente cada aluno v&ecirc; suas notas)"
#endif
	},
	{
	// TabUsr *****************************************************
	//  0: ActReqSelGrp
#if   L==1
	"Grups en aquesta assignatura"
#elif L==2
	"Gruppen"
#elif L==3
	"Groups"
#elif L==4
	"Grupos en esta asignatura"
#elif L==5
	"Groupes"
#elif L==6
	"Grupos en esta asignatura"	// Okoteve traduccin
#elif L==7
	"Gruppi"
#elif L==8
	"Grupy"
#elif L==9
	"Grupos"
#endif
	,
	//  1: ActLstStd
#if   L==1
	"Llistat i fitxes dels estudiants"
#elif L==2
	"Studentenverzeichnis und profile"
#elif L==3
	"Students' list and records"
#elif L==4
	"Listado y fichas de los estudiantes"
#elif L==5
	"Liste et cartes d'&eacute;tudiants"
#elif L==6
	"Listado y fichas de los estudiantes"// Okoteve traduccin
#elif L==7
	"Elenco e schede studenti"
#elif L==8
	"Listy i rekordy student&oacute;w"
#elif L==9
	"Lista e cart&otilde;es estudantes"
#endif
	,
	//  2: ActLstTch
#if   L==1
	"Llistat i fitxes de professors"
#elif L==2
	"Lehrkr&auml;fteverzeichnis und profile"
#elif L==3
	"Teachers' list and records"
#elif L==4
	"Listado y fichas de profesores"
#elif L==5
	"Liste et cartes d'enseignants"
#elif L==6
	"Listado y fichas de profesores"// Okoteve traduccin
#elif L==7
	"Elenco e schede professori"
#elif L==8
	"Lista i rekordy nauczycieli"
#elif L==9
	"Lista e cart&otilde;es professores"
#endif
	,
	//  3: ActLstOth
#if   L==1
	"Llistat i fitxes d'altres usuaris"
#elif L==2
	"Eingeladenverzeichnis und profile von anderen Benutzer"
#elif L==3
	"Other users' list and records"
#elif L==4
	"Listado y fichas de otros usuarios"
#elif L==5
	"Liste et cartes d'autres utilisateurs"
#elif L==6
	"Listado y fichas de otros usuarios"	// Okoteve traduccin
#elif L==7
	"Elenco e schede da altri utenti"
#elif L==8
	"Listy i rekordy innych u&zdot;ytkownik&oacute;w"
#elif L==9
	"Lista e cart&otilde;es de outros usu&aacute;rios"
#endif
	,
	//  4: ActSeeAtt
#if   L==1
	"Control d'assist&egrave;ncia dels estudiants"
	" a les classes d'aquesta assignatura"
#elif L==2
	"Kontrolle der Studierenden Anwesenheit"
#elif L==3
	"Control of students' attendance"
	" to classes in this course"
#elif L==4
	"Control de asistencia de los estudiantes"
	" a las clases de esta asignatura"
#elif L==5
	"Contr&ocirc;le de pr&eacute;sence des &eacute;tudiants"
#elif L==6
	"Control de asistencia de los estudiantes"
	" a las clases de esta asignatura"// Okoteve traduccin
#elif L==7
	"Controllo di frequenza degli studenti"
#elif L==8
	"Kontrola frekwencji"
#elif L==9
	"Lista de presen&ccedil;a"
#endif
	,
	//  5: ActReqSignUp
#if   L==1
	"Sol&middot;licitar inscripci&oacute;"
#elif L==2
	"Registrierung beantragen"
#elif L==3
	"Sign up in this course"
#elif L==4
	"Solicitar mi inscripci&oacute;n en esta asignatura"
#elif L==5
	"Demander inscription"
#elif L==6
	"Solicitar mi inscripci&oacute;n en esta asignatura"// Okoteve traduccin
#elif L==7
	"Richiedere iscrizione"
#elif L==8
	"Wniosku rejestracj&eogon;"
#elif L==9
	"Solicitar inscri&ccedil;&atilde;o"
#endif
	,
	//  6: ActSeeSignUpReq
#if   L==1
	"Peticions d'inscripci&oacute;"
#elif L==2
	"Registrierungsanforderungen"
#elif L==3
	"Enrollment requests in this course pending verification"
#elif L==4
	"Peticiones de inscripci&oacute;n pendientes de verificar"
#elif L==5
	"Demandes d'inscription"
#elif L==6
	"Peticiones de inscripci&oacute;n pendientes de verificar"	// Okoteve traduccin
#elif L==7
	"Richieste di iscrizione"
#elif L==8
	"&Zdot;&aogon;dania rejestrowania"
#elif L==9
	"Solicita&ccedil;&otilde;es de inscri&ccedil;&atilde;o"
#endif
	,
	//  7: ActLstCon
#if   L==1
	"Llista d'usuaris connectats en aquest moment"
#elif L==2
	"Angemeldet"
#elif L==3
	"Connected"
#elif L==4
	"Lista de usuarios conectados en este momento"
#elif L==5
	"Connect&eacute;s"
#elif L==6
	"Lista de usuarios conectados en este momento"// Okoteve traduccin
#elif L==7
	"Connessi"
#elif L==8
	"Online"
#elif L==9
	"Conectados"
#endif
	,
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabSoc *****************************************************
	//  0: ActSeeSocTmlGbl
#if   L==1
	 "Activitat p&uacute;blica dels usuaris que segueixo"
#elif L==2
	 "&Ouml;ffentliche Aktivit&auml;t der Nutzer Ich folge"
#elif L==3
	 "Public activity of the users I follow"
#elif L==4
	 "Actividad p&uacute;blica de los usuarios que sigo"
#elif L==5
	 "Activit&eacute; publique des utilisateurs je suivre"
#elif L==6
	 "Actividad p&uacute;blica de los usuarios que sigo"	// Okoteve traduccin
#elif L==7
	 "Attivit&agrave; pubblica degli utenti che seguo"
#elif L==8
	 "Aktywno&sacute;&cacute; publiczna u&zdot;ytkownik&oacute;w &sacute;ledz&eogon;"
#elif L==9
	 "Atividade p&uacute;blica dos usu&aacute;rios que eu sigo"
#endif
	,
	 //  1: ActSeeSocPrf
#if   L==1
	 "Perfils p&uacute;blics"
#elif L==2
	 "&Ouml;ffentliche Profile"
#elif L==3
	 "Public profiles"
#elif L==4
	 "Perfiles p&uacute;blicos de usuarios"
#elif L==5
	 "Profils publics"
#elif L==6
	 "Perfiles p&uacute;blicos de usuarios"	// Okoteve traduccin
#elif L==7
	 "Profili pubblici"
#elif L==8
	 "Profile publiczne"
#elif L==9
	 "Perfis p&uacute;blicos"
#endif
	,
	//  2: ActSeeFor
#if   L==1
	"F&ograve;rums de discussi&oacute;, distribu&iuml;ts"
	" segons la tem&agrave;tica (escolliu el f&ograve;rum adequat)"
#elif L==2
	"Foren"
#elif L==3
	"Forums"
#elif L==4
	"Foros de discusi&oacute;n, distribuidos"
	" seg&uacute;n la tem&aacute;tica (elija el foro adecuado)"
#elif L==5
	"Forums"
#elif L==6
	"Foros de discusi&oacute;n, distribuidos"
	" seg&uacute;n la tem&aacute;tica (elija el foro adecuado)"// Okoteve traduccin
#elif L==7
	"Forum"
#elif L==8
	"Forum"
#elif L==9
	"F&oacute;runs"
#endif
	,
	//  3: ActSeeChtRms
#if   L==1
	"Conversa en temps real amb altres usuaris"
#elif L==2
	"Chat"
#elif L==3
	"Chat"
#elif L==4
	"Conversaci&oacute;n en tiempo real con otros usuarios"
#elif L==5
	"Chat"
#elif L==6
	"Conversaci&oacute;n en tiempo real con otros usuarios"	// Okoteve traduccin
#elif L==7
	"Chat"
#elif L==8
	"Chat"
#elif L==9
	"Chat"
#endif
	,
	NULL,	//  4
	NULL,	//  5
	NULL,	//  6
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabMsg *****************************************************
	//  0: ActSeeNtf
#if   L==1
	"Esdeveniments recents que han estat o seran notificats"
#elif L==2
	"Die j&uuml;ngsten Ereignisse wurden oder werden benachrichtigt"
#elif L==3
	"Recent events that have been or will be notified"
#elif L==4
	"Eventos recientes que han sido o ser&aacute;n notificados"
#elif L==5
	"Les &eacute;v&eacute;nements r&eacute;cents qui ont &eacute;t&eacute; ou seront avis&eacute;s"
#elif L==6
	"Eventos recientes que han sido o ser&aacute;n notificados"// Okoteve traduccin
#elif L==7
	"Recenti eventi che sono stati o saranno notificati"
#elif L==8
	"Ostatnie wydarzenia, kt&oacute;re zostaly lub zostana powiadomieni"
#elif L==9
	"Os recentes acontecimentos que foram ou ser&atilde;o notificados"
#endif
	,
	//  1: ActSeeAnn
#if   L==1
	"Anuncis globals mostrat en iniciar sessi&oacute;"
#elif L==2
	"Globale Bekanntmachungen bei der Anmeldung sichtbar"
#elif L==3
	"Global announcements visible at login"
#elif L==4
	"Anuncios globales visibles al iniciar sesi&oacute;n"
#elif L==5
	"Annonces globales visibles lors de la connexion"
#elif L==6
	"Anuncios globales visibles al iniciar sesi&oacute;n"	// Okoteve traduccin
#elif L==7
	"Annunci globali visibili al login"
#elif L==8
	"Globalne og&lstrok;oszenia widoczne przy logowaniu"
#elif L==9
	"An&uacute;ncios globais vis&iacute;veis no in&iacute;cio da sess&atilde;o"
#endif
	,
	//  2: ActSeeAllNot
#if   L==1
	"Breus avisos relacionats amb aquesta assignatura,"
	" visibles per qualsevol visitant, encara que no estigui identificat"
#elif L==2
	"Ank&uuml;ndigungen"
#elif L==3
	"Notices"
#elif L==4
	"Breves avisos relacionados con esta asignatura,"
	" visibles por cualquier visitante, aunque no est&eacute; identificado"
#elif L==5
	"Avis"
#elif L==6
	"Breves avisos relacionados con esta asignatura,"
	" visibles por cualquier visitante, aunque no est&eacute; identificado"	// Okoteve traduccin
#elif L==7
	"Avvisi"
#elif L==8
	"Ogloszenia"
#elif L==9
	"Avisos"
#endif
	,
	//  3: ActReqMsgUsr
#if   L==1
	"Enviament d'un nou missatge privat a altres usuaris"
#elif L==2
	"Neue Nachricht"
#elif L==3
	"New message"
#elif L==4
	"Env&iacute;o de un nuevo mensaje privado a otros usuarios"
#elif L==5
	"Nouveau message"
#elif L==6
	"Env&iacute;o de un nuevo mensaje privado a otros usuarios"// Okoteve traduccin
#elif L==7
	"Nuovo messaggio"
#elif L==8
	"Nowe wiadomosci"
#elif L==9
	"Nova mensagem"
#endif
	,
	//  4: ActSeeRcvMsg
#if   L==1
	"Missatges privats rebuts d'altres usuaris"
#elif L==2
	"Empfangene Nachrichten"
#elif L==3
	"Messages received"
#elif L==4
	"Mensajes privados recibidos de otros usuarios"
#elif L==5
	"Messages re&ccedil;us"
#elif L==6
	"Mensajes privados recibidos de otros usuarios"	// Okoteve traduccin
#elif L==7
	"Messaggi ricevuti"
#elif L==8
	"Wiadomo&sacute;ci odebrane"
#elif L==9
	"Mensagens recebidas"
#endif
	,
	//  5: ActSeeSntMsg
#if   L==1
	"Missatges privats enviats per mi a altres usuaris"
#elif L==2
	"Gesendete Nachrichten"
#elif L==3
	"Messages sent"
#elif L==4
	"Mensajes privados enviados por m&iacute; a otros usuarios"
#elif L==5
	"Messages envoy&eacute;s"
#elif L==6
	"Mensajes privados enviados por m&iacute; a otros usuarios"// Okoteve traduccin
#elif L==7
	"Messaggi inviati"
#elif L==8
	"Wiadomo&sacute;ci odebrane"
#elif L==9
	"Mensagens enviadas"
#endif
	,
	//  6: ActMaiStd
#if   L==1
	"Creaci&oacute; d'un missatge de correu"
	" per enviar-lo a estudiants d'aquesta assignatura"
	" utilitzant el programa client de correu predeterminat en el meu sistema"
#elif L==2
	"E-Mail"
#elif L==3
	"Email"
#elif L==4
	"Creaci&oacute;n de un mensaje de correo"
	" para enviarlo a estudiantes de esta asignatura"
	" usando el programa cliente de correo predeterminado en mi sistema"
#elif L==5
	"Courrier &eacute;lectronique"
#elif L==6
	"Creaci&oacute;n de un mensaje de correo"
	" para enviarlo a estudiantes de esta asignatura"
	" usando el programa cliente de correo predeterminado en mi sistema"// Okoteve traduccin
#elif L==7
	"Email"
#elif L==8
	"Email"
#elif L==9
	"Email"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabSta *****************************************************
	//  0: ActSeeAllSvy
#if   L==1
	"Enquestes an&ograve;nimes"
#elif L==2
	"Umfragen"
#elif L==3
	"Surveys"
#elif L==4
	"Encuestas an&oacute;nimas"
#elif L==5
	"Sondages"
#elif L==6
	"Encuestas an&oacute;nimas"	// Okoteve traduccin
#elif L==7
	"Sondaggi"
#elif L==8
	"Ankiety"
#elif L==9
	"Inqu&eacute;ritos"
#endif
	,
	//  1: ActReqUseGbl
#if   L==1
	"Grans nombres sobre l'&uacute;s actual de tota la plataforma,"
	" o d'aquesta titulaci&oacute; o assignatura"
#elif L==2
	"Nutzung von SWAD"
#elif L==3
	"Use of SWAD"
#elif L==4
	"Grandes n&uacute;meros sobre el uso actual de toda la plataforma,"
	" o de esta titulaci&oacute;n o asignatura"
#elif L==5
	"Utilisation de SWAD"
#elif L==6
	"Grandes n&uacute;meros sobre el uso actual de toda la plataforma,"
	" o de esta titulaci&oacute;n o asignatura"// Okoteve traduccin
#elif L==7
	"Uso di SWAD"
#elif L==8
	"Korzystanie z SWAD"
#elif L==9
	"Utiliza&ccedil;&atilde;o de SWAD"
#endif
	,
	//  2: ActSeePhoDeg
#if   L==1
	"Estad&iacute;stiques del nombre d'estudiants en cada titulaci&oacute;"
	" representades per les fotos mitjana dels estudiants"
#elif L==2
	"Mischbilder der Studieng&auml;nge"
#elif L==3
	"Photos of degrees"
#elif L==4
	"Estad&iacute;sticas del n&uacute;mero de estudiantes en cada titulaci&oacute;n"
	" representadas por las fotos promedio de los estudiantes"
#elif L==5
	"Photographie &eacute;tudes"
#elif L==6
	"Estad&iacute;sticas del n&uacute;mero de estudiantes en cada titulaci&oacute;n"
	" representadas por las fotos promedio de los estudiantes"// Okoteve traduccin
#elif L==7
	"Foto di lauree"
#elif L==8
	"Zdjecia stopni"
#elif L==9
	"Fotografia titula&ccedil;oes"
#endif
	,
	//  3: ActReqStaCrs
#if   L==1
	"Indicadors d'assignatures per al contracte-programa"
#elif L==2
	"Anzeigen von Kursen f&uuml;r den Vertrag Programm"
#elif L==3
	"Indicators of courses for the contract program"
#elif L==4
	"Indicadores de asignaturas para el contrato-programa"
#elif L==5
	"Indicateurs de mati&egrave;res pour le contrat-programme"
#elif L==6
	"Indicadores de asignaturas para el contrato-programa"// Okoteve traduccin
#elif L==7
	"Indicatori di corsi per il contratto programma"
#elif L==8
	"Wska&zacute;niki kurs&oacute;w dla umowy programu"
#elif L==9
	"Indicadores de disciplinas para o contrato programa"
#endif
	,
	//  4: ActReqAccGbl
#if   L==1
	"Estad&iacute;stiques d'accesso a la plataforma"
#elif L==2
	"Zugriffsstatistik auf die Plattform"
#elif L==3
	"Statistics of access to the platform"
#elif L==4
	"Estad&iacute;sticas de acceso a la plataforma"
#elif L==5
	"Statistiques de connexion &agrave; la plate-forme"
#elif L==6
	"Estad&iacute;sticas de acceso a la plataforma"	// Okoteve traduccin
#elif L==7
	"Statistiche di accesso alla piattaforma"
#elif L==8
	"Statystyki dost&eogon;p do platformy"
#elif L==9
	"Estat&iacute;sticas de acesso &agrave; plataforma"
#endif
	,
	//  5: ActReqMyUsgRep
#if   L==1
	"Informe sobre el meu &uacute;s de la plataforma"
#elif L==2
	"Bericht &uuml;ber meine Nutzung der Plattform"
#elif L==3
	"Report on my use of the platform"
#elif L==4
	"Informe sobre mi uso de la plataforma"
#elif L==5
	"Rapport sur mon utilisation de la plate-forme"
#elif L==6
	"Informe sobre mi uso de la plataforma"	// Okoteve traduccin
#elif L==7
	"Relazione sulla mia utilizzo della piattaforma"
#elif L==8
	"Raport o moim korzystania z platformy"
#elif L==9
	"Relat&oacute;rio sobre o meu uso da plataforma"
#endif
	,
	//  6: ActMFUAct
#if   L==1
	"Acc&eacute;s directe a les meves accions m&eacute;s freq&uuml;ents"
#elif L==2
	"Direkter Zugriff auf meine h&auml;ufige Aktionen"
#elif L==3
	"Direct access to my most frequent actions"
#elif L==4
	"Acceso directo a mis acciones m&aacute;s frecuentes"
#elif L==5
	"Acc&egrave;s direct &agrave; mes actions les plus fr&eacute;quentes"
#elif L==6
	"Acceso directo a mis acciones m&aacute;s frecuentes"// Okoteve traduccin
#elif L==7
	"Accesso diretto alle mie azioni pi&ugrave; frequenti"
#elif L==8
	"Bezpo&sacute;redni dost&eogon;p do najcz&eogon;stszych dzia&lstrok;a&nacute;"
#elif L==9
	"Acesso direto das minhas a&ccedil;&otilde;es mais freq&uuml;entes"
#endif
	,
	NULL,	//  7
	NULL,	//  8
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	},
	{
	// TabPrf *****************************************************
	//  0: ActFrmLogIn
#if   L==1
	"Rol en aquesta sessi&oacute;"
#elif L==2
	"Rolle in dieser Sitzung"
#elif L==3
	"Role in this session"
#elif L==4
	"Rol en esta sesi&oacute;n"
#elif L==5
	"R&ocirc;le dans cette session"
#elif L==6
	"Rol en esta sesi&oacute;n"	// Okoteve traduccin
#elif L==7
	"Ruolo in questa sessione"
#elif L==8
	"Rola w tej sesji"
#elif L==9
	"Role nesta sess&atilde;o"
#endif
	,
	//  1: ActFrmRolSes
#if   L==1
	"Rol en aquesta sessi&oacute;"
#elif L==2
	"Rolle in dieser Sitzung"
#elif L==3
	"Role in this session"
#elif L==4
	"Rol en esta sesi&oacute;n"
#elif L==5
	"R&ocirc;le dans cette session"
#elif L==6
	"Rol en esta sesi&oacute;n"	// Okoteve traduccin
#elif L==7
	"Ruolo in questa sessione"
#elif L==8
	"Rola w tej sesji"
#elif L==9
	"Role nesta sess&atilde;o"
#endif
	,
	//  2: ActMyCrs
#if   L==1
	"Assignatures a les quals pertanyo"
#elif L==2
	"Kursen der ich angeh&ouml;re"
#elif L==3
	"Courses to which I belong"
#elif L==4
	"Asignaturas a las que pertenezco"
#elif L==5
	"Mati&egrave;res &agrave; laquelle je appartiens"
#elif L==6
	"Asignaturas a las que pertenezco"	// Okoteve traduccin
#elif L==7
	"Corsi a cui appartengo"
#elif L==8
	"Kursy do kt&oacute;rego nale&zdot;&eogon;"
#elif L==9
	"Disciplinas a que perten&ccedil;o"
#endif
	,
	//  3: ActSeeMyTT
#if   L==1
	"Horari conjunt de totes les meves assignatures a la plataforma"
#elif L==2
	"Stundenplan"
#elif L==3
	"My timetable"
#elif L==4
	"Horario conjunto de todas mis asignaturas en la plataforma"
#elif L==5
	"Horaire mes mati&egrave;res"
#elif L==6
	"Horario conjunto de todas mis asignaturas en la plataforma"// Okoteve traduccin
#elif L==7
	"Orario dei miei corsi"
#elif L==8
	"M&oacute;j plan lekcji"
#elif L==9
	"Hor&aacute;rio minha disciplinas"
#endif
	,
	//  4: ActSeeMyAgd
#if   L==1
	"Agenda"
#elif L==2
	"Organizer"
#elif L==3
	"Agenda (personal organizer)"
#elif L==4
	"Agenda"
#elif L==5
	"Agenda"
#elif L==6
	"Aporyr&atilde;"
#elif L==7
	"Agenda"
#elif L==8
	"Pami&eogon;tnik"
#elif L==9
	"Agenda"
#endif
	,
	//  5: ActFrmMyAcc
#if   L==1
	"Creaci&oacute; o edici&oacute; del meu compte d'usuari"
#elif L==2
	"Erstellen oder Bearbeiten meinem Benutzerkonto"
#elif L==3
	"Create or edit my user account"
#elif L==4
	"Creaci&oacute;n o edici&oacute;n de mi cuenta de usuario/a"
#elif L==5
	"Cr&eacute;er ou modifier mon compte d'utilisateur"
#elif L==6
	"Creaci&oacute;n o edici&oacute;n de mi cuenta de usuario/a"// Okoteve traduccin
#elif L==7
	"Creare o modificare il mio account utente"
#elif L==8
	"Utworzy&cacute; lub edytowa&cacute; konto u&zdot;ytkownika"
#elif L==9
	"Criar ou editar minha conta de usu&aacute;rio"
#endif
	,
	//  6: ActReqEdiRecCom
#if   L==1
	"Dades personals, comuns per a totes les assignatures"
#elif L==2
	"Profil"
#elif L==3
	"Record"
#elif L==4
	"Datos personales, comunes para todas las asignaturas"
#elif L==5
	"Carte"
#elif L==6
	"Datos personales, comunes para todas las asignaturas"// Okoteve traduccin
#elif L==7
	"Scheda"
#elif L==8
	"Karta"
#elif L==9
	"Cart&atilde;o"
#endif
	,
	//  7: ActEdiPrf
#if   L==1
	"Les meves prefer&egrave;ncies (idioma, colors, notificacions &hellip;)"
#elif L==2
	"Einstellungen"
#elif L==3
	"Preferences"
#elif L==4
	"Mis preferencias (idioma, colores, notificaciones&hellip;)"
#elif L==5
	"Pr&eacute;f&eacute;rences"
#elif L==6
	"Mis preferencias (idioma, colores, notificaciones&hellip;)"	// Okoteve traduccin
#elif L==7
	"Preferenze"
#elif L==8
	"Preferencje"
#elif L==9
	"Prefer&ecirc;ncias"
#endif
	,
	//  8: ActAdmBrf
#if   L==1
	"La meva zona privada d'emmagatzematge de fitxers"
#elif L==2
	"Aktentasche"
#elif L==3
	"Virtual pendrive"
#elif L==4
	"Mi zona privada de almacenamiento de archivos"
#elif L==5
	"Portedocuments"
#elif L==6
	"Mi zona privada de almacenamiento de archivos"	// Okoteve traduccin
#elif L==7
	"Penna USB"
#elif L==8
	"Pendrive wirtualne"
#elif L==9
	"Pasta"
#endif
	,
	NULL,	//  9
	NULL,	// 10
	NULL,	// 11
	}
};

const char *Txt_message =
#if   L==1
	"missatge";
#elif L==2
	"Nachricht";
#elif L==3
	"message";
#elif L==4
	"mensaje";
#elif L==5
	"message";
#elif L==6
	"marandu";
#elif L==7
	"messaggio";
#elif L==8
	"wiadomosc";
#elif L==9
	"mensagem";
#endif

const char *Txt_Message_deleted =
#if   L==1
	"Mensaje eliminado.";		// Necessita traduccio
#elif L==2
	"Nachricht entfernt.";
#elif L==3
	"Message deleted.";
#elif L==4
	"Mensaje eliminado.";
#elif L==5
	"Message deleted.";		// Besoin de traduction
#elif L==6
	"Mensaje eliminado.";		// Okoteve traduccin
#elif L==7
	"Messaggio rimosso.";
#elif L==8
	"Wiadomo&sacute;&cacute; usunieta.";
#elif L==9
	"Message deleted.";		// Necessita de traduo
#endif

const char *Txt_message_not_sent_to_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Missatge no enviat a %s.";
#elif L==2
	"Nachricht nicht gesendet, um %s.";
#elif L==3
	"Message not sent to %s.";
#elif L==4
	"Mensaje no enviado a %s.";
#elif L==5
	"Message non envoy&eacute; &agrave; %s.";
#elif L==6
	"Mensaje no enviado a %s.";	// Okoteve traduccin
#elif L==7
	"Messaggio non inviato a %s.";
#elif L==8
	"Wiadomo&sacute;&cacute; nie zostala wyslana do %s.";
#elif L==9
	"Mensagem n&atilde;o enviada para %s.";
#endif

const char *Txt_message_sent_to_X_not_notified_by_email =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Mensaje enviado a %s (no notificado por email).";	// Necessita traduccio
#elif L==2
	"Message sent to %s (not notified by email).";	// Need bersetzung
#elif L==3
	"Message sent to %s (not notified by email).";
#elif L==4
	"Mensaje enviado a %s (no notificado por email).";
#elif L==5
	"Message sent to %s (not notified by email).";	// Besoin de traduction
#elif L==6
	"Mensaje enviado a %s (no notificado por email).";	// Okoteve traduccin
#elif L==7
	"Messaggio inviato a %s (non notificato per email).";
#elif L==8
	"Wiadomo&sacute;&cacute; wyslana do %s (nie zgloszonych przez email).";	//
#elif L==9
	"Message sent to %s (not notified by email).";	// Necessita de traduo
#endif

const char *Txt_message_sent_to_X_notified_by_email =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Mensaje enviado a %s (notificado por email).";	// Necessita traduccio
#elif L==2
	"Message sent to %s (notified by email).";	// Need bersetzung
#elif L==3
	"Message sent to %s (notified by email).";
#elif L==4
	"Mensaje enviado a %s (notificado por email).";
#elif L==5
	"Message sent to %s (notified by email).";	// Besoin de traduction
#elif L==6
	"Mensaje enviado a %s (notificado por email).";	// Okoteve traduccin
#elif L==7
	"Messaggio inviato a %s (notificato per email).";
#elif L==8
	"Wiadomo&sacute;&cacute; wyslana do %s (zgloszone przez email).";
#elif L==9
	"Message sent to %s (notified by email).";	// Necessita de traduo
#endif

const char *Txt_Messages =
#if   L==1
	"Missatges";
#elif L==2
	"Nachrichten";
#elif L==3
	"Messages";
#elif L==4
	"Mensajes";
#elif L==5
	"Messages";
#elif L==6
	"Marandu";
#elif L==7
	"Messaggi";
#elif L==8
	"Wiadomo&sacute;ci";
#elif L==9
	"Mensagens";
#endif

const char *Txt_messages =
#if   L==1
	"missatges";
#elif L==2
	"Nachrichten";
#elif L==3
	"messages";
#elif L==4
	"mensajes";
#elif L==5
	"messages";
#elif L==6
	"marandu";
#elif L==7
	"messaggi";
#elif L==8
	"wiadomo&sacute;ci";
#elif L==9
	"mensagens";
#endif

const char *Txt_X_messages_have_been_deleted =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Se han eliminado %u mensajes.";	// Necessita traduccio
#elif L==2
	"%u messages have been deleted.";	// Need bersetzung
#elif L==3
	"%u messages have been deleted.";
#elif L==4
	"Se han eliminado %u mensajes.";
#elif L==5
	"%u messages have been deleted.";	// Besoin de traduction
#elif L==6
	"Se han eliminado %u mensajes.";	// Okoteve traduccin
#elif L==7
	"%u messaggi sono stati rimossi.";
#elif L==8
	"wiadomosci% u zostaly usuniete.";
#elif L==9
	"%u messages have been deleted.";	// Necessita de traduo
#endif

const char *Txt_Messages_of_THE_USER_X_have_been_deleted =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Se han eliminado los mensajes de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Messages of <strong>%s</strong> have been deleted.";	// Need bersetzung
#elif L==3
	"Messages of <strong>%s</strong> have been deleted.";
#elif L==4
	"Se han eliminado los mensajes de <strong>%s</strong>.";
#elif L==5
	"Messages of <strong>%s</strong> have been deleted.";	// Besoin de traduction
#elif L==6
	"Se han eliminado los mensajes de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Messaggi de <strong>%s</strong> sono stati rimossi.";
#elif L==8
	"Wiadomo&sacute;ci od <strong>%s</strong> zostaly usuniete.";
#elif L==9
	"Messages of <strong>%s</strong> have been deleted.";	// Necessita de traduo
#endif

const char *Txt_message_received =
#if   L==1
	"missatge rebut";
#elif L==2
	"Nachricht empfangen";
#elif L==3
	"message received";
#elif L==4
	"mensaje recibido";
#elif L==5
	"message re&ccedil;u";
#elif L==6
	"mensaje recibido";	// Okoteve traduccin
#elif L==7
	"messaggio ricevuto";
#elif L==8
	"odebranej wiadomo&sacute;ci";
#elif L==9
	"mensagem recebida";
#endif

const char *Txt_messages_received =
#if   L==1
	"missatges rebuts";
#elif L==2
	"empfangene Nachrichten";
#elif L==3
	"messages received";
#elif L==4
	"mensajes recibidos";
#elif L==5
	"messages re&ccedil;us";
#elif L==6
	"mensajes recibidos";	// Okoteve traduccin
#elif L==7
	"messaggi ricevuti";
#elif L==8
	"wiadomo&sacute;ci odebrane";
#elif L==9
	"mensagens recebidas";
#endif

const char *Txt_Messages_received_from_A_COURSE =
#if   L==1
	"Missatges rebuts des de";
#elif L==2
	"Empfangene Nachrichten aus";
#elif L==3
	"Messages received from";
#elif L==4
	"Mensajes recibidos desde";
#elif L==5
	"Messages re&ccedil;us de";
#elif L==6
	"Mensajes recibidos desde";	// Okoteve traduccin
#elif L==7
	"Messaggi ricevuti da";
#elif L==8
	"Wiadomo&sacute;ci odebrane";
#elif L==9
	"Mensagens recebidas de";
#endif

const char *Txt_message_sent =
#if   L==1
	"missatge enviat";
#elif L==2
	"Nachricht gesendet";
#elif L==3
	"message sent";
#elif L==4
	"mensaje enviado";
#elif L==5
	"message envoy&eacute;";
#elif L==6
	"mensaje enviado";	// Okoteve traduccin
#elif L==7
	"messaggio inviato";
#elif L==8
	"wiadomo&sacute;&cacute; wys&lstrok;ana";
#elif L==9
	"mensagem enviada";
#endif

const char *Txt_messages_sent =
#if   L==1
	"missatges enviats";
#elif L==2
	"gesendete Nachrichten";
#elif L==3
	"messages sent";
#elif L==4
	"mensajes enviados";
#elif L==5
	"messages envoy&eacute;s";
#elif L==6
	"mensajes enviados";	// Okoteve traduccin
#elif L==7
	"messaggi inviati";
#elif L==8
	"wiadomo&sacute;ci wysylane";
#elif L==9
	"mensagens enviadas";
#endif

const char *Txt_Messages_sent =
#if   L==1
	"Missatges enviats";
#elif L==2
	"Gesendete Nachrichten";
#elif L==3
	"Messages sent";
#elif L==4
	"Mensajes enviados";
#elif L==5
	"Messages envoy&eacute;s";
#elif L==6
	"Mensajes enviados";	// Okoteve traduccin
#elif L==7
	"Messaggi inviati";
#elif L==8
	"Wiadomo&sacute;ci wysylane";
#elif L==9
	"Mensagens enviadas";
#endif

const char *Txt_Messages_sent_by_teachers =
#if   L==1
	"Missatges enviats per professors";
#elif L==2
	"Gesendete Nachrichten, die von Lehrkr&auml;ften";
#elif L==3
	"Messages sent by teachers";
#elif L==4
	"Mensajes enviados por profesores";
#elif L==5
	"Messages envoy&eacute;s par enseignants";
#elif L==6
	"Mensajes enviados por profesores";	// Okoteve traduccin
#elif L==7
	"Messaggi inviati dai professori";
#elif L==8
	"Wiadomo&sacute;ci wysylane przez nauczycieli";
#elif L==9
	"Mensagens enviadas por professores";
#endif

const char *Txt_Messages_sent_from_A_COURSE =
#if   L==1
	"Missatges enviats des de";
#elif L==2
	"Gesendete Nachrichten aus";
#elif L==3
	"Messages sent from";
#elif L==4
	"Mensajes enviados desde";
#elif L==5
	"Messages envoy&eacute;s de";
#elif L==6
	"Mensajes enviados desde";	// Okoteve traduccin
#elif L==7
	"Messaggi inviati da";
#elif L==8
	"Wiadomo&sacute;ci wyslane z";
#elif L==9
	"Mensagens enviadas de";
#endif

const char *Txt_minimum =
#if   L==1
	"m&iacute;nimo";	// Necessita traduccio
#elif L==2
	"Mindest";
#elif L==3
	"minimum";
#elif L==4
	"m&iacute;nimo";
#elif L==5
	"minimum";
#elif L==6
	"m&iacute;nimo";	// Okoteve traduccin
#elif L==7
	"minimo";
#elif L==8
	"minimum";
#elif L==9
	"m&iacute;nimo";
#endif

const char *Txt_Minimum_time_seconds_per_question_between_two_tests =
#if   L==1
	"Tiempo m&iacute;nimo (segundos)<br />por pregunta<br />entre dos tests";	// Necessita traduccio
#elif L==2
	"Minimale Zeit (Sekunden)<br />pro Frage<br />zwischen zwei Tests";
#elif L==3
	"Minimum time (seconds)<br />per question<br />between two tests";
#elif L==4
	"Tiempo m&iacute;nimo (segundos)<br />por pregunta<br />entre dos tests";
#elif L==5
	"Minimum de temps (en secondes)<br />par question<br />entre deux tests";
#elif L==6
	"Tiempo m&iacute;nimo (segundos)<br />por pregunta<br />entre dos tests";	// Okoteve traduccin
#elif L==7
	"Minimo tempo (in secondi)<br />per domanda<br />tra due test";
#elif L==8
	"Minimalny czas (w sekundach)<br />na pytanie miedzy<br />dwoma badaniami";
#elif L==9
	"Tempo m&iacute;nimo (segundos)<br />por quest&atilde;o<br />entre dois testes";
#endif

const char *Txt_minute =
#if   L==1
	"minuto";	// Necessita traduccio
#elif L==2
	"Minute";
#elif L==3
	"minute";
#elif L==4
	"minuto";
#elif L==5
	"minute";
#elif L==6
	"aravo'i";
#elif L==7
	"minuto";
#elif L==8
	"minut&eogon;";
#elif L==9
	"minuto";
#endif

const char *Txt_minutes =
#if   L==1
	"minutos";	// Necessita traduccio
#elif L==2
	"Minute";
#elif L==3
	"minutes";
#elif L==4
	"minutos";
#elif L==5
	"minutes";
#elif L==6
	"aravo'i";
#elif L==7
	"minuti";
#elif L==8
	"minuty";
#elif L==9
	"minutos";
#endif

const char *Txt_Modify_me_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Modificarme en l'assignatura <strong>%s</strong>";
#elif L==2
	"&Auml;ndern Sie mir im Kurs <strong>%s</strong>";
#elif L==3
	"Modify me in the course <strong>%s</strong>";
#elif L==4
	"Modificarme en la asignatura <strong>%s</strong>";
#elif L==5
	"Modifier-moi dans la mati&egrave;re <strong>%s</strong>";
#elif L==6
	"Modificarme en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Modificami utente nel corso <strong>%s</strong>";
#elif L==8
	"Modyfikacja mnie w kurs <strong>%s</strong>";
#elif L==9
	"Modificar me na disciplina <strong>%s</strong>";
#endif

const char *Txt_Modify_user_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Modificar usuari en l'assignatura <strong>%s</strong>";
#elif L==2
	"&Auml;ndern Benutzer im Kurs <strong>%s</strong>";
#elif L==3
	"Modify user in the course <strong>%s</strong>";
#elif L==4
	"Modificar usuario en la asignatura <strong>%s</strong>";
#elif L==5
	"Modifier utilisateur dans la mati&egrave;re <strong>%s</strong>";
#elif L==6
	"Modificar usuario en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Modifica utente nel corso <strong>%s</strong>";
#elif L==8
	"Modyfikacja u&zdot;ytkownika w kurs <strong>%s</strong>";
#elif L==9
	"Modificar utilizador na disciplina <strong>%s</strong>";
#endif

const char *Txt_Month =
#if   L==1
	"Mes";		// Necessita traduccio
#elif L==2
	"Monat";
#elif L==3
	"Month";
#elif L==4
	"Mes";
#elif L==5
	"Mois";
#elif L==6
	"Jasy";
#elif L==7
	"Mese";
#elif L==8
	"Miesiac";
#elif L==9
	"M&ecirc;s";
#endif

const char *Txt_MONTHS_CAPS[12] =
   {
#if   L==1
	 "GENER"
#elif L==2
	 "JANUAR"
#elif L==3
	 "JANUARY"
#elif L==4
	 "ENERO"
#elif L==5
	 "JANVIER"
#elif L==6
	 "JASYTE&#296;"
#elif L==7
	 "GENNAIO"
#elif L==8
	 "STYCZNIA"
#elif L==9
	 "JANEIRO"
#endif
	 ,
#if   L==1
	 "FEBRER"
#elif L==2
	 "FEBRUAR"
#elif L==3
	 "FEBRUARY"
#elif L==4
	 "FEBRERO"
#elif L==5
	 "F&Eacute;VRIER"
#elif L==6
	 "JASYK&Otilde;I"
#elif L==7
	 "FEBBRAIO"
#elif L==8
	 "LUTEGO"
#elif L==9
	 "FEVEREIRO"
#endif
	 ,
#if   L==1
	 "MAR&Ccedil;"
#elif L==2
	 "M&Auml;RZ"
#elif L==3
	 "MARCH"
#elif L==4
	 "MARZO"
#elif L==5
	 "MARS"
#elif L==6
	 "JASYAPY"
#elif L==7
	 "MARZO"
#elif L==8
	 "MARCA"
#elif L==9
	 "MAR&Ccedil;O"
#endif
	 ,
#if   L==1
	 "ABRIL"
#elif L==2
	 "APRIL"
#elif L==3
	 "APRIL"
#elif L==4
	 "ABRIL"
#elif L==5
	 "AVRIL"
#elif L==6
	 "JASYRUNDY"
#elif L==7
	 "APRILE"
#elif L==8
	 "KWIETNIA"
#elif L==9
	 "ABRIL"
#endif
	 ,
#if   L==1
	 "MAIG"
#elif L==2
	 "MAI"
#elif L==3
	 "MAY"
#elif L==4
	 "MAYO"
#elif L==5
	 "MAI"
#elif L==6
	 "JASYPO"
#elif L==7
	 "MAGGIO"
#elif L==8
	 "MAJA"
#elif L==9
	 "MAIO"
#endif
	 ,
#if   L==1
	 "JUNY"
#elif L==2
	 "JUNI"
#elif L==3
	 "JUNE"
#elif L==4
	 "JUNIO"
#elif L==5
	 "JUIN"
#elif L==6
	 "JASYPOTE&#296;"
#elif L==7
	 "GIUGNO"
#elif L==8
	 "CZERWCA"
#elif L==9
	 "JUNHO"
#endif
	 ,
#if   L==1
	 "JULIOL"
#elif L==2
	 "JULI"
#elif L==3
	 "JULY"
#elif L==4
	 "JULIO"
#elif L==5
	 "JUILLET"
#elif L==6
	 "JASYPOK&Otilde;I"
#elif L==7
	 "LUGLIO"
#elif L==8
	 "LIPCA"
#elif L==9
	 "JULHO"
#endif
	 ,
#if   L==1
	 "AGOST"
#elif L==2
	 "AUGUST"
#elif L==3
	 "AUGUST"
#elif L==4
	 "AGOSTO"
#elif L==5
	 "AO&Ucirc;T"
#elif L==6
	 "JASYPOAPY"
#elif L==7
	 "AGOSTO"
#elif L==8
	 "SIERPNIA"
#elif L==9
	 "AGOSTO"
#endif
	 ,
#if   L==1
	 "SETEMBRE"
#elif L==2
	 "SEPTEMBER"
#elif L==3
	 "SEPTEMBER"
#elif L==4
	 "SEPTIEMBRE"
#elif L==5
	 "SEPTEMBRE"
#elif L==6
	 "JASYPORUNDY"
#elif L==7
	 "SETTEMBRE"
#elif L==8
	 "WRZE&Sacute;NIA"
#elif L==9
	 "SETEMBRO"
#endif
	 ,
#if   L==1
	 "OCTUBRE"
#elif L==2
	 "OKTOBER"
#elif L==3
	 "OCTOBER"
#elif L==4
	 "OCTUBRE"
#elif L==5
	 "OCTOBRE"
#elif L==6
	 "JASYPA"
#elif L==7
	 "OTTOBRE"
#elif L==8
	 "PA&Zacute;DZIERNIKA"
#elif L==9
	 "OUTUBRO"
#endif
	 ,
#if   L==1
	 "NOVEMBRE"
#elif L==2
	 "NOVEMBER"
#elif L==3
	 "NOVEMBER"
#elif L==4
	 "NOVIEMBRE"
#elif L==5
	 "NOVEMBRE"
#elif L==6
	 "JASYPATE&#296;"
#elif L==7
	 "NOVEMBRE"
#elif L==8
	 "LISTOPADA"
#elif L==9
	 "NOVEMBRO"
#endif
	 ,
#if   L==1
	 "DESEMBRE"
#elif L==2
	 "DEZEMBER"
#elif L==3
	 "DECEMBER"
#elif L==4
	 "DICIEMBRE"
#elif L==5
	 "D&Eacute;CEMBRE"
#elif L==6
	 "JASYPAK&Otilde;I"
#elif L==7
	 "DICEMBRE"
#elif L==8
	 "GRUDNIA"
#elif L==9
	 "DEZEMBRO"
#endif
	};

const char *Txt_MONTHS_SMALL[12] =
   {
#if   L==1
	 "gener"
#elif L==2
	 "Januar"
#elif L==3
	 "January"
#elif L==4
	 "enero"
#elif L==5
	 "janvier"
#elif L==6
	 "jasyte&#297;"
#elif L==7
	 "gennaio"
#elif L==8
	 "Stycznia"
#elif L==9
	 "janeiro"
#endif
	 ,
#if   L==1
	 "febrer"
#elif L==2
	 "Februar"
#elif L==3
	 "February"
#elif L==4
	 "febrero"
#elif L==5
	 "f&eacute;vrier"
#elif L==6
	 "jasyk&otilde;i"
#elif L==7
	 "febbraio"
#elif L==8
	 "Lutego"
#elif L==9
	 "fevereiro"
#endif
	 ,
#if   L==1
	 "mar&ccedil;"
#elif L==2
	 "M;&auml;rz"
#elif L==3
	 "March"
#elif L==4
	 "marzo"
#elif L==5
	 "mars"
#elif L==6
	 "jasyapy"
#elif L==7
	 "marzo"
#elif L==8
	 "Marca"
#elif L==9
	 "mar&ccedil;o"
#endif
	 ,
#if   L==1
	 "abril"
#elif L==2
	 "April"
#elif L==3
	 "April"
#elif L==4
	 "abril"
#elif L==5
	 "avril"
#elif L==6
	 "jasyrundy"
#elif L==7
	 "aprile"
#elif L==8
	 "Kwietnia"
#elif L==9
	 "abril"
#endif
	 ,
#if   L==1
	 "maig"
#elif L==2
	 "Mai"
#elif L==3
	 "May"
#elif L==4
	 "mayo"
#elif L==5
	 "mai"
#elif L==6
	 "jasypo"
#elif L==7
	 "maggio"
#elif L==8
	 "Maja"
#elif L==9
	 "maio"
#endif
	 ,
#if   L==1
	 "juny"
#elif L==2
	 "Juni"
#elif L==3
	 "June"
#elif L==4
	 "junio"
#elif L==5
	 "juin"
#elif L==6
	 "jasypote&#297;"
#elif L==7
	 "giugno"
#elif L==8
	 "Czerwca"
#elif L==9
	 "junho"
#endif
	 ,
#if   L==1
	 "juliol"
#elif L==2
	 "Juli"
#elif L==3
	 "July"
#elif L==4
	 "julio"
#elif L==5
	 "juillet"
#elif L==6
	 "jasypok&otilde;i"
#elif L==7
	 "luglio"
#elif L==8
	 "Lipca"
#elif L==9
	 "julho"
#endif
	 ,
#if   L==1
	 "agost"
#elif L==2
	 "August"
#elif L==3
	 "August"
#elif L==4
	 "agosto"
#elif L==5
	 "ao&ucirc;t"
#elif L==6
	 "jasypoapy"
#elif L==7
	 "agosto"
#elif L==8
	 "Sierpnia"
#elif L==9
	 "agosto"
#endif
	 ,
#if   L==1
	 "setembre"
#elif L==2
	 "September"
#elif L==3
	 "September"
#elif L==4
	 "septiembre"
#elif L==5
	 "septembre"
#elif L==6
	 "jasyporundy"
#elif L==7
	 "settembre"
#elif L==8
	 "Wrze&sacute;nia"
#elif L==9
	 "setembro"
#endif
	 ,
#if   L==1
	 "octubre"
#elif L==2
	 "Oktober"
#elif L==3
	 "October"
#elif L==4
	 "octubre"
#elif L==5
	 "octobre"
#elif L==6
	 "jasypa"
#elif L==7
	 "ottobre"
#elif L==8
	 "Pa&zacute;dziernika"
#elif L==9
	 "outubro"
#endif
	 ,
#if   L==1
	 "novembre"
#elif L==2
	 "November"
#elif L==3
	 "November"
#elif L==4
	 "noviembre"
#elif L==5
	 "novembre"
#elif L==6
	 "jasypate&#297;"
#elif L==7
	 "novembre"
#elif L==8
	 "Listopada"
#elif L==9
	 "novembro"
#endif
	 ,
#if   L==1
	 "desembre"
#elif L==2
	 "Dezember"
#elif L==3
	 "December"
#elif L==4
	 "diciembre"
#elif L==5
	 "d&eacute;cembre"
#elif L==6
	 "jasypak&otilde;i"
#elif L==7
	 "dicembre"
#elif L==8
	 "Grudnia"
#elif L==9
	 "dezembro"
#endif
	};

const char *Txt_MONTHS_SMALL_SHORT[12] =
   {
#if   L==1
	 "gen"
#elif L==2
	 "Jan"
#elif L==3
	 "Jan"
#elif L==4
	 "ene"
#elif L==5
	 "jan"
#elif L==6
	 "ene"	// Okoteve traduccin
#elif L==7
	 "gen"
#elif L==8
	 "Sty"
#elif L==9
	 "jan"
#endif
	 ,
#if   L==1
	 "feb"
#elif L==2
	 "Feb"
#elif L==3
	 "Feb"
#elif L==4
	 "feb"
#elif L==5
	 "f&eacute;v"
#elif L==6
	 "feb"	// Okoteve traduccin
#elif L==7
	 "feb"
#elif L==8
	 "Lut"
#elif L==9
	 "fev"
#endif
	 ,
#if   L==1
	 "mar"
#elif L==2
	 "M;&auml;r"
#elif L==3
	 "Mar"
#elif L==4
	 "mar"
#elif L==5
	 "mar"
#elif L==6
	 "mar"	// Okoteve traduccin
#elif L==7
	 "mar"
#elif L==8
	 "Mar"
#elif L==9
	 "mar"
#endif
	 ,
#if   L==1
	 "abr"
#elif L==2
	 "Apr"
#elif L==3
	 "Apr"
#elif L==4
	 "abr"
#elif L==5
	 "avr"
#elif L==6
	 "abr"	// Okoteve traduccin
#elif L==7
	 "apr"
#elif L==8
	 "Kwi"
#elif L==9
	 "abr"
#endif
	 ,
#if   L==1
	 "mai"
#elif L==2
	 "Mai"
#elif L==3
	 "May"
#elif L==4
	 "may"
#elif L==5
	 "mai"
#elif L==6
	 "may"	// Okoteve traduccin
#elif L==7
	 "mag"
#elif L==8
	 "Maj"
#elif L==9
	 "mai"
#endif
	 ,
#if   L==1
	 "jun"
#elif L==2
	 "Jun"
#elif L==3
	 "Jun"
#elif L==4
	 "jun"
#elif L==5
	 "jui"
#elif L==6
	 "jun"	// Okoteve traduccin
#elif L==7
	 "giu"
#elif L==8
	 "Cze"
#elif L==9
	 "jun"
#endif
	 ,
#if   L==1
	 "jul"
#elif L==2
	 "Jul"
#elif L==3
	 "Jul"
#elif L==4
	 "jul"
#elif L==5
	 "jui"
#elif L==6
	 "jul"	// Okoteve traduccin
#elif L==7
	 "lug"
#elif L==8
	 "Lip"
#elif L==9
	 "jul"
#endif
	 ,
#if   L==1
	 "ago"
#elif L==2
	 "Aug"
#elif L==3
	 "Aug"
#elif L==4
	 "ago"
#elif L==5
	 "ao&ucirc;"
#elif L==6
	 "ago"	// Okoteve traduccin
#elif L==7
	 "ago"
#elif L==8
	 "Sie"
#elif L==9
	 "ago"
#endif
	 ,
#if   L==1
	 "set"
#elif L==2
	 "Sep"
#elif L==3
	 "Sep"
#elif L==4
	 "sep"
#elif L==5
	 "sep"
#elif L==6
	 "sep"	// Okoteve traduccin
#elif L==7
	 "set"
#elif L==8
	 "Wrz"
#elif L==9
	 "set"
#endif
	 ,
#if   L==1
	 "oct"
#elif L==2
	 "Okt"
#elif L==3
	 "Oct"
#elif L==4
	 "oct"
#elif L==5
	 "oct"
#elif L==6
	 "oct"	// Okoteve traduccin
#elif L==7
	 "ott"
#elif L==8
	 "Pa&zacute;"
#elif L==9
	 "out"
#endif
	 ,
#if   L==1
	 "nov"
#elif L==2
	 "Nov"
#elif L==3
	 "Nov"
#elif L==4
	 "nov"
#elif L==5
	 "nov"
#elif L==6
	 "nov"	// Okoteve traduccin
#elif L==7
	 "nov"
#elif L==8
	 "Lis"
#elif L==9
	 "nov"
#endif
	 ,
#if   L==1
	 "des"
#elif L==2
	 "Dez"
#elif L==3
	 "Dec"
#elif L==4
	 "dic"
#elif L==5
	 "d&eacute;c"
#elif L==6
	 "dic"	// Okoteve traduccin
#elif L==7
	 "dic"
#elif L==8
	 "Gru"
#elif L==9
	 "dez"
#endif
	};

const char *Txt_Move_up_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Subir %s";		// Necessita traduccio
#elif L==2
	"Move up %s";		// Need bersetzung
#elif L==3
	"Move up %s";
#elif L==4
	"Subir %s";
#elif L==5
	"Move up %s";		// Besoin de traduction
#elif L==6
	"Subir %s";		// Okoteve traduccin
#elif L==7
	"Muovi su %s";
#elif L==8
	"Przesun w g&oacute;re %s";
#elif L==9
	"Move up %s";		// Necessita de traduo
#endif

const char *Txt_Move_up_X_and_its_subsections =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Subir %s y sus subapartados";			// Necessita traduccio
#elif L==2
	"Move up %s and its subsections";		// Need bersetzung
#elif L==3
	"Move up %s and its subsections";
#elif L==4
	"Subir %s y sus subapartados";
#elif L==5
	"Move up %s and its subsections";		// Besoin de traduction
#elif L==6
	"Subir %s y sus subapartados";			// Okoteve traduccin
#elif L==7
	"Muovi su %s e le sue sottosezioni";
#elif L==8
	"Przenies sie %s, a jego podrozdzialy";
#elif L==9
	"Move up %s and its subsections";		// Necessita de traduo
#endif

const char *Txt_Move_down_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Bajar %s";		// Necessita traduccio
#elif L==2
	"Move down %s";		// Need bersetzung
#elif L==3
	"Move down %s";
#elif L==4
	"Bajar %s";
#elif L==5
	"Move down %s";		// Besoin de traduction
#elif L==6
	"Bajar %s";		// Okoteve traduccin
#elif L==7
	"Muovi gi&ugrave; %s";
#elif L==8
	"Opuszczac %s";
#elif L==9
	"Move down %s";		// Necessita de traduo
#endif

const char *Txt_Move_down_X_and_its_subsections =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Bajar %s y sus subapartados";			// Necessita traduccio
#elif L==2
	"Move down %s and its subsections";		// Need bersetzung
#elif L==3
	"Move down %s and its subsections";
#elif L==4
	"Bajar %s y sus subapartados";
#elif L==5
	"Move down %s and its subsections";		// Besoin de traduction
#elif L==6
	"Bajar %s y sus subapartados";			// Okoteve traduccin
#elif L==7
	"Muovi gi&ugrave; %s e le sue sottosezioni";
#elif L==8
	"Przesun w d&oacute;l %s, a jego podrozdzialy";
#elif L==9
	"Move down %s and its subsections";		// Necessita de traduo
#endif

const char *Txt_Move_thread =
#if   L==1
	"Mover discusi&oacute;n";	// Necessita traduccio
#elif L==2
	"Verschieben Thread";
#elif L==3
	"Move thread";
#elif L==4
	"Mover discusi&oacute;n";
#elif L==5
	"D&eacute;placer fil";
#elif L==6
	"Mover discusi&oacute;n";	// Okoteve traduccin
#elif L==7
	"Sposta discussione";
#elif L==8
	"Przenies watku";
#elif L==9
	"Mover thread";
#endif

const char *Txt_Movement_not_allowed =
#if   L==1
	"Moviment no perm&egrave;s";
#elif L==2
	"Bewegung nicht erlaubt";
#elif L==3
	"Move up not allowed";
#elif L==4
	"Movimiento no permitido";
#elif L==5
	"Mouvement pas autoris&eacute;";
#elif L==6
	"Movimiento no permitido";	// Okoteve traduccin
#elif L==7
	"Movimento non consentito";
#elif L==8
	"Ruch nie jest dozwolone";
#elif L==9
	"Movimento n&atilde;o permitido";
#endif

const char *Txt_MSG_Deleted_without_opening =
#if   L==1
	"Eliminado sin abrir";		// Necessita traduccio
#elif L==2
	"Gel&ouml;scht ohne &Ouml;ffnen";
#elif L==3
	"Deleted without opening";
#elif L==4
	"Eliminado sin abrir";
#elif L==5
	"Supprim&eacute; sans ouvrir";
#elif L==6
	"Eliminado sin abrir";		// Okoteve traduccin
#elif L==7
	"Rimosso senza aprire";
#elif L==8
	"Usuniete bez otwierania";
#elif L==9
	"Suprimida sem abrir";
#endif

const char *Txt_MSG_From =
#if   L==1
	"De";		// Necessita traduccio
#elif L==2
	"Von";
#elif L==3
	"From";
#elif L==4
	"De";
#elif L==5
	"De";
#elif L==6
	"De";		// Okoteve traduccin
#elif L==7
	"Da";
#elif L==8
	"z";
#elif L==9
	"De";
#endif

const char *Txt_MSG_from =
#if   L==1
	"from";		// Necessita traduccio
#elif L==2
	"von";
#elif L==3
	"from";
#elif L==4
	"de";
#elif L==5
	"de";
#elif L==6
	"de";	// Okoteve traduccin
#elif L==7
	"da";
#elif L==8
	"z";
#elif L==9
	"de";
#endif

const char *Txt_MSG_Message =
#if   L==1
	"Missatge";
#elif L==2
	"Nachricht";
#elif L==3
	"Message";
#elif L==4
	"Mensaje";
#elif L==5
	"Message";
#elif L==6
	"Marandu";
#elif L==7
	"Messaggio";
#elif L==8
	"Wiadomo&sacute;&cacute;";
#elif L==9
	"Mensagem";
#endif

const char *Txt_MSG_New =
#if   L==1
	"Nuevo";	// Necessita traduccio
#elif L==2
	"Neue";
#elif L==3
	"New";
#elif L==4
	"Nuevo";
#elif L==5
	"Nouveau";
#elif L==6
	"Nuevo";	// Okoteve traduccin
#elif L==7
	"Nuovo";
#elif L==8
	"nowe";
#elif L==9
	"Nova";
#endif

const char *Txt_MSG_Not_replied =
#if   L==1
	"No respondido";	// Necessita traduccio
#elif L==2
	"Nicht geantwortet";
#elif L==3
	"Not replied";
#elif L==4
	"No respondido";
#elif L==5
	"Pas r&eacute;pondu";
#elif L==6
	"No respondido";	// Okoteve traduccin
#elif L==7
	"Non risposto";
#elif L==8
	"Nie odpowiedzial";
#elif L==9
	"N&atilde;o respondida";
#endif

const char *Txt_MSG_Open =
#if   L==1
	"Abierto";	// Necessita traduccio
#elif L==2
	"Ge&ouml;ffneten";
#elif L==3
	"Open";
#elif L==4
	"Abierto";
#elif L==5
	"Ouvert";
#elif L==6
	"Abierto";	// Okoteve traduccin
#elif L==7
	"Aperto";
#elif L==8
	"otwarte";
#elif L==9
	"Aberta";
#endif

const char *Txt_MSG_Open_and_deleted =
#if   L==1
	"Abierto y eliminado";	// Necessita traduccio
#elif L==2
	"Ge&ouml;ffneten und gel&ouml;scht";
#elif L==3
	"Open and deleted";
#elif L==4
	"Abierto y eliminado";
#elif L==5
	"Ouvert et supprim&eacute;";
#elif L==6
	"Abierto y eliminado";	// Okoteve traduccin
#elif L==7
	"Aperto e rimosso";
#elif L==8
	"Open i usuniete";
#elif L==9
	"Aberta e suprimida";
#endif

const char *Txt_MSG_Replied =
#if   L==1
	"Respondido";	// Necessita traduccio
#elif L==2
	"Geantwortet";
#elif L==3
	"Replied";
#elif L==4
	"Respondido";
#elif L==5
	"R&eacute;pondu";
#elif L==6
	"Respondido";	// Okoteve traduccin
#elif L==7
	"Risposto";
#elif L==8
	"Odpowiedziano";
#elif L==9
	"Respondida";
#endif

const char *Txt_MSG_Sent =
#if   L==1
	"Enviado";	// Necessita traduccio
#elif L==2
	"Gesendet";
#elif L==3
	"Sent";
#elif L==4
	"Enviado";
#elif L==5
	"Envoy&eacute;";
#elif L==6
	"Enviado";	// Okoteve traduccin
#elif L==7
	"Inviato";
#elif L==8
	"Wyslane";
#elif L==9
	"Enviada";
#endif

const char *Txt_MSG_Sent_and_deleted =
#if   L==1
	"Enviado y eliminado";	// Necessita traduccio
#elif L==2
	"Gesendet und gel&ouml;scht";
#elif L==3
	"Sent and deleted";
#elif L==4
	"Enviado y eliminado";
#elif L==5
	"Envoy&eacute; et supprim&eacute;";
#elif L==6
	"Enviado y eliminado";	// Okoteve traduccin
#elif L==7
	"Inviato e rimosso";
#elif L==8
	"Wyslane i usuniete";
#elif L==9
	"Enviada e suprimida";
#endif

const char *Txt_MSG_Subject =
#if   L==1
	"Assumpte";
#elif L==2
	"Betreff";
#elif L==3
	"Subject";
#elif L==4
	"Asunto";
#elif L==5
	"Objet";
#elif L==6
	"Asunto";	// Okoteve traduccin
#elif L==7
	"Oggetto";
#elif L==8
	"Z zastrzezeniem";
#elif L==9
	"Assunto";
#endif

const char *Txt_MSG_To =
#if   L==1
	"Per";
#elif L==2
	"An";
#elif L==3
	"To";
#elif L==4
	"Para";
#elif L==5
	"Pour";
#elif L==6
	"Para";	// Okoteve traduccin
#elif L==7
	"A";
#elif L==8
	"Aby";
#elif L==9
	"Para";
#endif

const char *Txt_MSG_Unopened =
#if   L==1
	"Sin abrir";	// Necessita traduccio
#elif L==2
	"Unge&ouml;ffnet";
#elif L==3
	"Unopened";
#elif L==4
	"Sin abrir";
#elif L==5
	"Non ouvert";
#elif L==6
	"Sin abrir";	// Okoteve traduccin
#elif L==7
	"Non aperto";
#elif L==8
	"Otwarte";
#elif L==9
	"Fechada";
#endif

const char *Txt_MSGS_Deleted =
#if   L==1
	"Eliminados";	// Necessita traduccio
#elif L==2
	"Gel&ouml;scht";
#elif L==3
	"Deleted";
#elif L==4
	"Eliminados";
#elif L==5
	"Supprim&eacute;s";
#elif L==6
	"Eliminados";	// Okoteve traduccin
#elif L==7
	"Rimossi";
#elif L==8
	"Usuniete";
#elif L==9
	"Suprimidas";
#endif

const char *Txt_MSGS_Not_deleted =
#if   L==1
	"No eliminados";	// Necessita traduccio
#elif L==2
	"Nicht Gel&ouml;scht";
#elif L==3
	"Not deleted";
#elif L==4
	"No eliminados";
#elif L==5
	"Non supprim&eacute;s";
#elif L==6
	"No eliminados";	// Okoteve traduccin
#elif L==7
	"Non rimossi";
#elif L==8
	"Nie usuniete";
#elif L==9
	"N&atilde;o suprimidas";
#endif

const char *Txt_MSGS_Received =
#if   L==1
	"Rebuts";
#elif L==2
	"Empfangene";
#elif L==3
	"Received";
#elif L==4
	"Recibidos";
#elif L==5
	"Re&ccedil;us";
#elif L==6
	"Recibidos";	// Okoteve traduccin
#elif L==7
	"Ricevuti";
#elif L==8
	"Odebrane";
#elif L==9
	"Recebidas";
#endif

const char *Txt_MSGS_Sent =
#if   L==1
	"Enviats";
#elif L==2
	"Gesendete";
#elif L==3
	"Sent";
#elif L==4
	"Enviados";
#elif L==5
	"Envoy&eacute;s";
#elif L==6
	"Enviados";	// Okoteve traduccin
#elif L==7
	"Inviati";
#elif L==8
	"Wyslane";
#elif L==9
	"Enviadas";
#endif

const char *Txt_Multiple_enrollment =		// (to a type of group)
#if   L==1
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";	// Necessita traduccio
#elif L==2
	"Mehrere beitreten?";
#elif L==3
	"Multiple enrollment?";
#elif L==4
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";
#elif L==5
	"Inscription multiple?";
#elif L==6
	"&iquest;Adscripci&oacute;n m&uacute;ltiple?";	// Okoteve traduccin
#elif L==7
	"Registrazione multipla?";
#elif L==8
	"Rejestracja wielu element&oacute;w?";
#elif L==9
	"Inscri&ccedil;&atilde;o m&uacute;ltiplas?";
#endif

const char *Txt_My_agenda =
#if   L==1
	"La meva agenda";
#elif L==2
	"Mein Organizer";
#elif L==3
	"My agenda";
#elif L==4
	"Mi agenda";
#elif L==5
	"Mon agenda";
#elif L==6
	"Che aporyr&atilde;";
#elif L==7
	"La mia agenda";
#elif L==8
	"Moja pami&eogon;tnik";
#elif L==9
	"Minha agenda";
#endif

const char *Txt_My_courses =
#if   L==1
	"Les meves assignatures";
#elif L==2
	"Meine Kursen";
#elif L==3
	"My courses";
#elif L==4
	"Mis asignaturas";
#elif L==5
	"Mes mati&egrave;res";
#elif L==6
	"Che mbo'esyry";
#elif L==7
	"Miei corsi";
#elif L==8
	"Moje kurs&oacute;w";
#elif L==9
	"As minha disciplinas";
#endif

const char *Txt_my_documents =
#if   L==1
	"els meus documents";
#elif L==2
	"meine Dokumente";
#elif L==3
	"my documents";
#elif L==4
	"mis documentos";
#elif L==5
	"mes documents";
#elif L==6
	"mis documentos";	// Okoteve traduccin
#elif L==7
	"i miei documenti";
#elif L==8
	"moje dokumenty";
#elif L==9
	"meus documentos";
#endif

const char *Txt_My_frequent_actions =
#if   L==1
	"Les meves accions freq&uuml;ents";
#elif L==2
	"Meine h&auml;ufigen Handlungen";
#elif L==3
	"My frequent actions";
#elif L==4
	"Mis acciones frecuentes";
#elif L==5
	"Mes actions fr&eacute;quentes";
#elif L==6
	"Mis acciones frecuentes";	// Okoteve traduccin
#elif L==7
	"Le mie azioni frequenti";
#elif L==8
	"Moi cz&eogon;ste dzia&lstrok;ania";
#elif L==9
	"Minhas a&ccedil;&otilde;es freq&uuml;entes";
#endif

const char *Txt_My_groups =
#if   L==1
	"Meus grups";
#elif L==2
	"Meine Gruppen";
#elif L==3
	"My groups";
#elif L==4
	"Mis grupos";
#elif L==5
	"Mes groupes";
#elif L==6
	"Mis grupos";	// Okoteve traduccin
#elif L==7
	"Miei gruppi";
#elif L==8
	"Moje grupy";
#elif L==9
	"Meus grupos";
#endif

const char *Txt_My_public_profile =
#if   L==1
	"El meu perfil p&uacute;blic";
#elif L==2
	"Mein &ouml;ffentliches Profil";
#elif L==3
	"My public profile";
#elif L==4
	"Mi perfil p&uacute;blico";
#elif L==5
	"Mon profil public";
#elif L==6
	"Mi perfil p&uacute;blico";	// Okoteve traduccin
#elif L==7
	"Il mio profilo pubblico";
#elif L==8
	"M&oacute;j profil publiczny";
#elif L==9
	"Meu perfil p&uacute;blico";
#endif

const char *Txt_My_views =
#if   L==1
	"Els meus accessos";
#elif L==2
	"Meine Ansichten";
#elif L==3
	"My views";
#elif L==4
	"Mis accesos";
#elif L==5
	"Mes vues";
#elif L==6
	"Mis accesos";	// Okoteve traduccin
#elif L==7
	"Miei visualizzazioni";
#elif L==8
	"Moje ods&lstrok;on";
#elif L==9
	"Meus acessos";
#endif

const char *Txt_Name =
#if   L==1
	"Nom";
#elif L==2
	"Name";
#elif L==3
	"Name";
#elif L==4
	"Nombre";
#elif L==5
	"Nom";
#elif L==6
	"T&eacute;ra";
#elif L==7
	"Nome";
#elif L==8
	"Nazwa";
#elif L==9
	"Nome";
#endif

const char *Txt_New_assignment =
#if   L==1
	"Nova activitat";
#elif L==2
	"Neue Aufgabe";
#elif L==3
	"New assignment";
#elif L==4
	"Nueva actividad";
#elif L==5
	"Nouvelle activit&eacute;";
#elif L==6
	"Nueva actividad";	// Okoteve traduccin
#elif L==7
	"Nuova attivit&agrave;";
#elif L==8
	"Nowe zadania";
#elif L==9
	"Nova atividade";
#endif

const char *Txt_New_announcement =
#if   L==1
	"Nou anunci";
#elif L==2
	"Neue Bekanntmachung";
#elif L==3
	"New announcement";
#elif L==4
	"Nuevo anuncio";
#elif L==5
	"Nouvel annonce";
#elif L==6
	"Nuevo anuncio";	// Okoteve traduccin
#elif L==7
	"Nuovo annuncio";
#elif L==8
	"Nowe og&lstrok;oszenia";
#elif L==9
	"Novo an&uacute;ncio";
#endif

const char *Txt_New_announcement_OF_EXAM =
#if   L==1
	"Nova convocat&ograve;ria";
#elif L==2
	"Neuer Aufruf";
#elif L==3
	"New announcement";
#elif L==4
	"Nueva convocatoria";
#elif L==5
	"Nouvelle convocation";
#elif L==6
	"Nueva convocatoria";	// Okoteve traduccin
#elif L==7
	"Nuova sessione";
#elif L==8
	"Nowe ogloszenie";
#elif L==9
	"Nova chamada";
#endif

const char *Txt_New_banner =
#if   L==1
	"Nuevo banner";		// Necessita traduccio
#elif L==2
	"Neue Werbebanner";
#elif L==3
	"New banner";
#elif L==4
	"Nuevo banner";
#elif L==5
	"Nouvelle banni&eacute;re";
#elif L==6
	"Nuevo banner";		// Okoteve traduccin
#elif L==7
	"Nuovo banner";
#elif L==8
	"Nowy banner";
#elif L==9
	"Novo banner";
#endif

const char *Txt_New_centre_of_INSTITUTION_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Nou centre (facultat, escola, institut, divisi&oacute;, edifici...) de %s";
#elif L==2
	"Neues Lehrinstitut (Fakult&auml;t, Schule, Hochschule, Abteilung, Geb&auml;ude...) von %s";
#elif L==3
	"New centre (faculty, school, college, division, building...) of %s";
#elif L==4
	"Nuevo centro (facultad, escuela, instituto, divisi&oacute;n, edificio...) de %s";
#elif L==5
	"Nouveau centre (facult&eacute;, &eacute;cole, universit&eacute;, division, b&acirc;timent...) de %s";
#elif L==6
	"Nuevo centro (facultad, escuela, instituto, divisi&oacute;n, edificio...) de %s";	// Okoteve traduccin
#elif L==7
	"Nuovo centro (facolt&agrave;, scuola, istituto, divisione, costruzione...) di %s";
#elif L==8
	"Nowe centrum (wydzia&lstrok;, szko&lstrok;a, uczelnia, podzia&lstrok;, budowa...) %s";
#elif L==9
	"Novo centro (faculdade, escola, faculdade, divis&atilde;o, constru&ccedil;&atilde;o...) de %s";
#endif

const char *Txt_New_country =
#if   L==1
	"Nuevo pa&iacute;s";	// Necessita traduccio
#elif L==2
	"Neues Land";
#elif L==3
	"New country";
#elif L==4
	"Nuevo pa&iacute;s";
#elif L==5
	"Nouveau pays";
#elif L==6
	"Nuevo pa&iacute;s";	// Okoteve traduccin
#elif L==7
	"Nuovo paese";
#elif L==8
	"Nowy kraj";
#elif L==9
	"Novo pa&iacute;s";
#endif

const char *Txt_New_course_of_DEGREE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Nova assignatura (assignatura, curs, mat&egrave;ria, disciplina...) de %s";
#elif L==2
	"Neue Kurs (Kurs, Thema, Disziplin...) der %s";
#elif L==3
	"New course (course, subject, discipline...) of %s";
#elif L==4
	"Nueva asignatura (asignatura, curso, materia, disciplina...) de %s";
#elif L==5
	"Nouvelle mati&egrave;re (mati&egrave;re, cours, discipline...) de %s";
#elif L==6
	"Nueva asignatura (asignatura, curso, materia, disciplina...) de %s";	// Okoteve traduccin
#elif L==7
	"Nuovo corso (corso, materia...) di %s";
#elif L==8
	"Nowy kurs %s";
#elif L==9
	"Nova disciplina (disciplina, curso...) de %s";
#endif

const char *Txt_New_degree_of_CENTRE_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Nova titulaci&oacute; (carrera, llicenciatura, diplomatura, grau, postgrau, m&agrave;ster, cicle, nivell...) de %s";
#elif L==2
	"Neuer Studiengang (Karriere, Diplom, Grad, Graduierten, Master, Zyklus, Niveau...) von %s";
#elif L==3
	"New degree (career, diploma, degree, postgraduate, master, cycle, level...) of %s";
#elif L==4
	"Nueva titulaci&oacute;n (carrera, licenciatura, diplomatura, grado, posgrado, m&aacute;ster, ciclo, nivel...) de %s";
#elif L==5
	"Nouvel &eacute;tude (carri&egrave;re, grade, dipl&ocirc;me, degr&eacute;, troisi&agrave;me cycle, master, cycle, niveau...) de %s";
#elif L==6
	"Nueva titulaci&oacute;n (carrera, licenciatura, diplomatura, grado, posgrado, m&aacute;ster, ciclo, nivel...) de %s";	// Okoteve traduccin
#elif L==7
	"Nuova laurea (carriera, diploma, laurea, post-laurea, master, ciclo, livello...) di %s";
#elif L==8
	"Nowy stopie&nacute; (zawodowe, stopie&nacute;, dyplom, magisterskie, podyplomowe, magisterskie, cykl, poziom...) %s";
#elif L==9
	"Nova titula&ccedil;&atilde;o (carreira, diploma, grau, p&oacute;s-grau, mestrado, ciclo, n&iacute;vel...) de %s";
#endif

const char *Txt_New_department =
#if   L==1
	"Nuevo departamento";	// Necessita traduccio
#elif L==2
	"Neue Abteilung";
#elif L==3
	"New department";
#elif L==4
	"Nuevo departamento";
#elif L==5
	"Nouveau d&eacute;partement";
#elif L==6
	"Nuevo departamento";	// Okoteve traduccin
#elif L==7
	"Nuovo dipartimento";
#elif L==8
	"Nowy dzia&lstrok;";
#elif L==9
	"Novo departamento";
#endif

const char *Txt_New_email =
#if   L==1
	"Nou correu";
#elif L==2
	"Neues E-Mail";
#elif L==3
	"New email";
#elif L==4
	"Nuevo correo";
#elif L==5
	"Nouveau courrier";
#elif L==6
	"Nuevo correo";	// Okoteve traduccin
#elif L==7
	"Nuovo email";
#elif L==8
	"Nowy email";
#elif L==9
	"Novo email";
#endif

const char *Txt_New_email_domain =
#if   L==1
	"Nuevo dominio de correo";	// Necessita traduccio
#elif L==2
	"Neue E-Mail-Dom&auml;ne";
#elif L==3
	"New email domain";
#elif L==4
	"Nuevo dominio de correo";
#elif L==5
	"Nouveau domaine de messagerie";
#elif L==6
	"Nuevo dominio de correo";	// Okoteve traduccin
#elif L==7
	"Nuovo campo mail";
#elif L==8
	"Nowe domeny mail";
#elif L==9
	"Novo dom&iacute;nio de email";
#endif

const char *Txt_New_event =
#if   L==1
	"Nou esdeveniment";
#elif L==2
	"Neue Ereignis";
#elif L==3
	"New event";
#elif L==4
	"Nuevo evento";
#elif L==5
	"Nouveau &eacute;v&eacute;nement";
#elif L==6
	"Nuevo evento";	// Okoteve traduccin
#elif L==7
	"Nuovo evento";
#elif L==8
	"Nowe wydarzenie";
#elif L==9
	"Novo evento";
#endif

const char *Txt_New_FILE_OR_FOLDER =
#if   L==1
	"Nou";
#elif L==2
	"Neue";
#elif L==3
	"New";
#elif L==4
	"Nuevo";
#elif L==5
	"Nouveau";
#elif L==6
	"Nuevo";	// Okoteve traduccin
#elif L==7
	"Nuovo";
#elif L==8
	"Nowe";
#elif L==9
	"Novo";
#endif

const char *Txt_New_group =
#if   L==1
	"Nuevo grupo";	// Necessita traduccio
#elif L==2
	"Neue Gruppe";
#elif L==3
	"New group";
#elif L==4
	"Nuevo grupo";
#elif L==5
	"Nouveau groupe";
#elif L==6
	"Nuevo grupo";	// Okoteve traduccin
#elif L==7
	"Nuovo gruppo";
#elif L==8
	"Nowa grupa";
#elif L==9
	"Novo grupo";
#endif

const char *Txt_New_holiday =
#if   L==1
	"Nova festivitat";
#elif L==2
	"Neue Feiertag";
#elif L==3
	"New holiday";
#elif L==4
	"Nueva festividad";
#elif L==5
	"Nouvelle f&ecirc;te";
#elif L==6
	"Nueva festividad";	// Okoteve traduccin
#elif L==7
	"Nuova festivit&agrave;";
#elif L==8
	"Nowe swieto";
#elif L==9
	"Nova f&eacute;ria";
#endif

const char *Txt_New_institution_of_COUNTRY_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Nova instituci&oacute;n (universitat, institut, escola, acad&egrave;mia, organitzaci&oacute;, empresa...) de %s";
#elif L==2
	"Neue Hochschule (Universit&auml;t, Hochschule, Schule, Hochschule, Organisation, Unternehmen...) von %s";
#elif L==3
	"New institution (university, college, school, academy, organization, company...) of %s";
#elif L==4
	"Nueva instituci&oacute;n (universidad, instituto, colegio, academia, organizaci&oacute;n, empresa...) de %s";
#elif L==5
	"Nouvel &eacute;tablissement (universit&eacute;, coll&egrave;ge, &eacute;cole, acad&eacute;mie, organisation, entreprise...) de %s";
#elif L==6
	"Nueva instituci&oacute;n (universidad, instituto, colegio, academia, organizaci&oacute;n, empresa...) de %s";	// Okoteve traduccin
#elif L==7
	"Nuova istituzione (universit&agrave;, istituto, scuola, accademia, organizzazione, societ&agrave;...) di %s";
#elif L==8
	"Nowa instytucja (uniwersytet, uczelnia, szko&lstrok;a, akademia, organizacja, firma...) %s";
#elif L==9
	"Nova institu&ccedil;&atilde;o (universidade, faculdade, escola, academia, organiza&ccedil;&atilde;o, empresa...) de %s";
#endif

const char *Txt_New_link =
#if   L==1
	"Nuevo enlace";		// Necessita traduccio
#elif L==2
	"Neue Link";
#elif L==3
	"New link";
#elif L==4
	"Nuevo enlace";
#elif L==5
	"Nouveau lien";
#elif L==6
	"Nuevo enlace";		// Okoteve traduccin
#elif L==7
	"Nuovo link";
#elif L==8
	"Nowy link";
#elif L==9
	"Nova liga&ccedil;&atilde;o";
#endif

const char *Txt_New_message =
#if   L==1
	"Nou missatge";
#elif L==2
	"Neue Nachricht";
#elif L==3
	"New message";
#elif L==4
	"Nuevo mensaje";
#elif L==5
	"Nouveau message";
#elif L==6
	"Nuevo mensaje";	// Okoteve traduccin
#elif L==7
	"Nuovo messaggio";
#elif L==8
	"Nowa wiadomosc";
#elif L==9
	"Nova mensagem";
#endif

const char *Txt_New_nickname =
#if   L==1
	"Nou sobrenom";
#elif L==2
	"Neues Benutzername";
#elif L==3
	"New nickname";
#elif L==4
	"Nuevo apodo";
#elif L==5
	"Nouveau surnom";
#elif L==6
	"Nuevo apodo";	// Okoteve traduccin
#elif L==7
	"Nuovo nome utente";
#elif L==8
	"Nowy nick";
#elif L==9
	"Nova alcunha";
#endif

const char *Txt_New_notice =
#if   L==1
	"Nou av&iacute;s";
#elif L==2
	"Neue Ank&uuml;ndigung";
#elif L==3
	"New notice";
#elif L==4
	"Nuevo aviso";
#elif L==5
	"Nouvel avis";
#elif L==6
	"Nuevo aviso";	// Okoteve traduccin
#elif L==7
	"Nuovo avviso";
#elif L==8
	"Nowe obwieszczenie";
#elif L==9
	"Novo aviso";
#endif

const char *Txt_New_on_PLATFORM_Sign_up =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No t&eacute; compte a %s? Registra't ara";
#elif L==2
	"Neu bei %s? Registriere Dich!";
#elif L==3
	"New on %s? Sign up";
#elif L==4
	"&iquest;No tiene cuenta en %s? Reg&iacute;strese";
#elif L==5
	"Nouveau sur %s? Inscrivez-vous";
#elif L==6
	"&iquest;No tiene cuenta en %s? Reg&iacute;strese";	// Okoteve traduccin
#elif L==7
	"Nuovo su %s? Iscriviti";
#elif L==8
	"Pierwszy raz na %s? Zarejestruj si&eogon;";
#elif L==9
	"Novo no %s? Inscreva-se";
#endif

const char *Txt_New_password =
#if   L==1
	"Nova contrasenya";
#elif L==2
	"Neues Passwort";
#elif L==3
	"New password";
#elif L==4
	"Nueva contrase&ntilde;a";
#elif L==5
	"Nouveau mot de passe";
#elif L==6
	"Nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==7
	"Nuova password";
#elif L==8
	"Nowe has&lstrok;o";
#elif L==9
	"Nova senha";
#endif

const char *Txt_New_place =
#if   L==1
	"Nuevo lugar";	// Necessita traduccio
#elif L==2
	"Neue Standort";
#elif L==3
	"New place";
#elif L==4
	"Nuevo lugar";
#elif L==5
	"Nouvel emplacement";
#elif L==6
	"Nuevo lugar";	// Okoteve traduccin
#elif L==7
	"Nuova localit&grave;";
#elif L==8
	"Nowe miejsce";
#elif L==9
	"Nova localiza&ccedil;&atilde;o";
#endif

const char *Txt_New_plugin =
#if   L==1
	"Nuevo complemento";	// Necessita traduccio
#elif L==2
	"Neue Plugin";
#elif L==3
	"New plugin";
#elif L==4
	"Nuevo complemento";
#elif L==5
	"Nouveau plugin";
#elif L==6
	"Nuevo complemento";	// Okoteve traduccin
#elif L==7
	"Nuovo plugin";
#elif L==8
	"Nowy plugin";
#elif L==9
	"Novo plugin";
#endif

const char *Txt_New_question =
#if   L==1
	"Nova pregunta";
#elif L==2
	"Neuer Frage";
#elif L==3
	"New question";
#elif L==4
	"Nueva pregunta";
#elif L==5
	"Nouvelle question";
#elif L==6
	"Nueva pregunta";	// Okoteve traduccin
#elif L==7
	"Nuova domanda";
#elif L==8
	"Nowe pytanie";
#elif L==9
	"Nova quest&atilde;o";
#endif

const char *Txt_New_record_field =
#if   L==1
	"Nuevo campo de ficha";		// Necessita traduccio
#elif L==2
	"Neues Datenfeld";
#elif L==3
	"New record field";
#elif L==4
	"Nuevo campo de ficha";
#elif L==5
	"Nouveau champ de carte";
#elif L==6
	"Nuevo campo de ficha";		// Okoteve traduccin
#elif L==7
	"Nuovo campo di scheda";
#elif L==8
	"Nowy rekord pola";
#elif L==9
	"Novo campo de cart&atilde;o";
#endif

const char *Txt_New_SOCIAL_comment =
#if   L==1
	"Nou comentari";
#elif L==2
	"Neue Kommentar";
#elif L==3
	"New comment";
#elif L==4
	"Nuevo comentario";
#elif L==5
	"Nouveau commentaire";
#elif L==6
	"Nuevo comentario";	// Okoteve traduccin
#elif L==7
	"Nuovo commento";
#elif L==8
	"Nowy komentarz";
#elif L==9
	"Novo coment&aacute;rio";
#endif

const char *Txt_New_SOCIAL_post =
#if   L==1
	"Novo missatge";
#elif L==2
	"Neue Beitrag";
#elif L==3
	"New post";
#elif L==4
	"Nuevo mensaje";
#elif L==5
	"Nouveau message";
#elif L==6
	"Nuevo mensaje";	// Okoteve traduccin
#elif L==7
	"Nuovo post";
#elif L==8
	"Nowy post";
#elif L==9
	"Novo post";
#endif

const char *Txt_New_survey =
#if   L==1
	"Nova enquesta";
#elif L==2
	"Neuer Umfrage";
#elif L==3
	"New survey";
#elif L==4
	"Nueva encuesta";
#elif L==5
	"Nouveau sondage";
#elif L==6
	"Nueva encuesta";	// Okoteve traduccin
#elif L==7
	"Nuovo sondaggio";
#elif L==8
	"Nowe badania";
#elif L==9
	"Novo inqu&eacute;rito";
#endif

const char *Txt_new_tag =		// Tag used in test
#if   L==1
	"nuevo descriptor";	// Necessita traduccio
#elif L==2
	"neuen Tag";
#elif L==3
	"new tag";
#elif L==4
	"nuevo descriptor";
#elif L==5
	"nouveau descripteur";
#elif L==6
	"nuevo descriptor";	// Okoteve traduccin
#elif L==7
	"nuova etichetta";
#elif L==8
	"nowy tag";
#elif L==9
	"novo descritor";
#endif

const char *Txt_New_thread =
#if   L==1
	"Nueva discusi&oacute;n";	// Necessita traduccio
#elif L==2
	"Neuen Thread";
#elif L==3
	"New thread";
#elif L==4
	"Nueva discusi&oacute;n";
#elif L==5
	"Nouveau fil";
#elif L==6
	"Nueva discusi&oacute;n";	// Okoteve traduccin
#elif L==7
	"Nuova discussione";
#elif L==8
	"Nowy temat";
#elif L==9
	"Novo thread";
#endif

const char *Txt_New_type_of_degree =
#if   L==1
	"Nuevo tipo de titulaci&oacute;n";	// Necessita traduccio
#elif L==2
	"Neue Abschlussart";
#elif L==3
	"New type of degree";
#elif L==4
	"Nuevo tipo de titulaci&oacute;n";
#elif L==5
	"Nouveau type d'&eacute;tude";
#elif L==6
	"Nuevo tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Nuovo tipo di laurea";
#elif L==8
	"Nowy rodzaj stopnia";
#elif L==9
	"Novo tipo de titula&ccedil;&atilde;o";
#endif

const char *Txt_New_type_of_group =
#if   L==1
	"Nuevo tipo de grupo";		// Necessita traduccio
#elif L==2
	"Neue Art der Gruppe";
#elif L==3
	"New type of group";
#elif L==4
	"Nuevo tipo de grupo";
#elif L==5
	"Nouveau type de groupe";
#elif L==6
	"Nuevo tipo de grupo";		// Okoteve traduccin
#elif L==7
	"Nuovo tipo di gruppo";
#elif L==8
	"Nowy typ grupy";
#elif L==9
	"Novo tipo de grupo";
#endif

const char *Txt_NEW_YEAR_GREETING =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"&iexcl;Feliz %u!";	// Necessita traduccio
#elif L==2
	"Happy %u!";
#elif L==3
	"Happy %u!";
#elif L==4
	"&iexcl;Feliz %u!";
#elif L==5
	"Happy %u!";
#elif L==6
	"&iexcl;Feliz %u!";	// Okoteve traduccin
#elif L==7
	"Felice %u!";
#elif L==8
	"Happy %u!";
#elif L==9
	"Feliz %u!";
#endif

const char *Txt_nick_email_or_ID =
#if   L==1
	"@sobrenom, correu o ID (DNI/c&egrave;dula)";
#elif L==2
	"@Benutzername, Email oder Ausweis-Nr.";
#elif L==3
	"@nick, email or ID";
#elif L==4
	"@apodo, correo o ID (DNI/c&eacute;dula)";
#elif L==5
	"@surnom, courrier ou num&eacute;ro d'identit&eacute;";
#elif L==6
	"@apodo, correo o ID (DNI/c&eacute;dula)";	// Okoteve traduccin
#elif L==7
	"@nome-utente, email o numero carta d'identit&agrave;";
#elif L==8
	"@pseudonim, email lub ID";
#elif L==9
	"@alcunha, email ou n&ordm; identif.";
#endif

const char *Txt_nicks_emails_or_IDs_separated_by_commas =
#if   L==1
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";	// Necessita traduccio
#elif L==2
	"@Benutzernamen, Emails oder Ausweis-Nr. durch Kommas getrennt";
#elif L==3
	"@nicks, emails or IDs separated by commas";
#elif L==4
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";
#elif L==5
	"@surnoms, courriers ou num&eacute;ros d'identit&eacute; s&eacute;par&eacute;s par des virgules";
#elif L==6
	"@apodos, correos o ID (DNI/c&eacute;dulas) separados por comas";	// Okoteve traduccin
#elif L==7
	"@nomi-utenti, email o numero carte d'identit&agrave; separati da virgole";
#elif L==8
	"@nicks, emails or IDs separated by commas";	// Potrzebujesz tlumaczenie
#elif L==9
	"@alcunhas, emails ou n&ordm;s. identif. separados por v&iacute;rgulas";
#endif

const char *Txt_Nickname =
#if   L==1
	"Sobrenom";
#elif L==2
	"Benutzername";
#elif L==3
	"Nickname";
#elif L==4
	"Apodo";
#elif L==5
	"Surnom";
#elif L==6
	"Apodo";	// Okoteve traduccin
#elif L==7
	"Nome-utente";
#elif L==8
	"Pseudonim";
#elif L==9
	"Alcunha";
#endif

const char *Txt_Nickname_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Sobrenom <strong>@%s</strong> eliminat.";
#elif L==2
	"Benutzername <strong>@%s</strong> entfernt.";
#elif L==3
	"Nickname <strong>@%s</strong> removed.";
#elif L==4
	"Apodo <strong>@%s</strong> eliminado.";
#elif L==5
	"Surnom <strong>@%s</strong> supprim&eacute;.";
#elif L==6
	"Apodo <strong>@%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Nome utente <strong>@%s</strong> rimosso.";
#elif L==8
	"Pseudonim <strong>@%s</strong> usuniete.";
#elif L==9
	"Alcunha <strong>@%s</strong> removido.";
#endif

const char *Txt_NO =
#if   L==1
	"NO";
#elif L==2
	"NEIN";
#elif L==3
	"NO";
#elif L==4
	"NO";
#elif L==5
	"NON";
#elif L==6
	"NAH&aACUTE;NIRI";
#elif L==7
	"NO";
#elif L==8
	"NO";
#elif L==9
	"N&Atilde;O";
#endif

const char *Txt_No =
#if   L==1
	"No";
#elif L==2
	"Nein";
#elif L==3
	"No";
#elif L==4
	"No";
#elif L==5
	"Non";
#elif L==6
	"Nah&aacute;niri";
#elif L==7
	"No";
#elif L==8
	"No";
#elif L==9
	"N&atilde;o";
#endif

const char *Txt_No_announcements =
#if   L==1
	"No hi ha anuncis";
#elif L==2
	"Keine Bekanntmachungen";
#elif L==3
	"No announcements";
#elif L==4
	"No hay anuncios";
#elif L==5
	"Il n'y a pas d'annonces";
#elif L==6
	"No hay anuncios";	// Okoteve traduccin
#elif L==7
	"Non ci sono annunci";
#elif L==8
	"Brak og&lstrok;oszenia";
#elif L==9
	"N&atilde;o h&aacute; an&uacute;ncios";
#endif

const char *Txt_No_announcements_of_exams_of_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No hay convocatorias de examen de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Keine Aufrufe f&uuml;r Pr&uuml;fung von <strong>%s</strong>.";
#elif L==3
	"No announcements of exams of <strong>%s</strong>.";
#elif L==4
	"No hay convocatorias de examen de <strong>%s</strong>.";
#elif L==5
	"Aucune convocation &agrave; un examen de <strong>%s</strong>.";
#elif L==6
	"No hay convocatorias de examen de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Nessun appello d'esame di <strong>%s</strong>.";
#elif L==8
	"Brak ogloszen egzamin&oacute;w <strong>%s</strong>.";
#elif L==9
	"N&atilde;o chamadas para exame de <strong>%s</strong>.";
#endif

const char *Txt_No_assignments =
#if   L==1
	"No hi ha activitats.";
#elif L==2
	"Keine Aufgaben.";
#elif L==3
	"No assignments.";
#elif L==4
	"No hay actividades.";
#elif L==5
	"Il n'y a pas d'activit&eacute;s.";
#elif L==6
	"No hay actividades.";	// Okoteve traduccin
#elif L==7
	"Non ci sono attivit&agrave;.";
#elif L==8
	"Brak zadania.";
#elif L==9
	"N&atilde;o h&aacute; atividades.";
#endif

const char *Txt_no_course_of_origin =	// Means: "message sent from any course"
#if   L==1
	"sin asignatura de origen";	// Necessita traduccio
#elif L==2
	"no course of origin";		// Need bersetzung
#elif L==3
	"no course of origin";
#elif L==4
	"sin asignatura de origen";
#elif L==5
	"no course of origin";		// Besoin de traduction
#elif L==6
	"sin asignatura de origen";	// Okoteve traduccin
#elif L==7
	"senza corso di origine";
#elif L==8
	"kurs nie pochodzenia";
#elif L==9
	"no course of origin";		// Necessita de traduo
#endif

const char *Txt_no_course_selected =
#if   L==1
	"cap assignatura seleccionada";
#elif L==2
	"kein Kurs ausgew&auml;hlt";
#elif L==3
	"no course selected";
#elif L==4
	"ninguna asignatura seleccionada";
#elif L==5
	"pas de mati&egrave;re choisi";
#elif L==6
	"ninguna asignatura seleccionada";
#elif L==7
	"nessun corso scelto";
#elif L==8
	"kurs nie jest wybrany";
#elif L==9
	"nenhuma disciplina seleccionada";
#endif

const char *Txt_No_centres =
#if   L==1
	"No hi ha centres.";
#elif L==2
	"Keine Lehrinstitute.";
#elif L==3
	"No centres.";
#elif L==4
	"No hay centros.";
#elif L==5
	"Il n'y a pas de centres.";
#elif L==6
	"No hay centros.";	// Okoteve traduccin
#elif L==7
	"Non ci sono centri.";
#elif L==8
	"Brak centra.";
#elif L==9
	"N&atilde;o h&aacute; centros.";
#endif

const char *Txt_No_countries_have_been_created =
#if   L==1
	"No se han creado pa&iacute;ses.";	// Necessita traduccio
#elif L==2
	"No countries have been created.";	// Need bersetzung
#elif L==3
	"No countries have been created.";
#elif L==4
	"No se han creado pa&iacute;ses.";
#elif L==5
	"No countries have been created.";	// Besoin de traduction
#elif L==6
	"No se han creado pa&iacute;ses.";	// Okoteve traduccin
#elif L==7
	"No countries have been created.";	// Bisogno di traduzione
#elif L==8
	"No countries have been created.";	// Potrzebujesz tlumaczenie
#elif L==9
	"No countries have been created.";	// Necessita de traduo
#endif

const char *Txt_No_courses =
#if   L==1
	"No hi ha assignatures.";
#elif L==2
	"Keine Kursen.";
#elif L==3
	"No courses.";
#elif L==4
	"No hay asignaturas.";
#elif L==5
	"Il n'y a pas de mati&egrave;res.";
#elif L==6
	"No hay asignaturas.";	// Okoteve traduccin
#elif L==7
	"Non ci sono laurea.";
#elif L==8
	"Brak kurs&oacute;w.";
#elif L==9
	"N&atilde;o h&aacute; disciplinas.";
#endif

const char *Txt_No_degrees =
#if   L==1
	"No hi ha titulacions.";
#elif L==2
	"Keine Studieng&auml;nge.";
#elif L==3
	"No degrees.";
#elif L==4
	"No hay titulaciones.";
#elif L==5
	"Il n'y a pas d'&eacute;tudes.";
#elif L==6
	"No hay titulaciones.";	// Okoteve traduccin
#elif L==7
	"Non ci sono corsi.";
#elif L==8
	"Brak stopni.";
#elif L==9
	"N&atilde;o h&aacute; titula&ccedil;oes.";
#endif

const char *Txt_No_enrollment_requests =
#if   L==1
	"No hay solicitudes de inscripci&oacute;n.";	// Necessita traduccio
#elif L==2
	"No enrollment requests.";			// Need bersetzung
#elif L==3
	"No enrollment requests.";
#elif L==4
	"No hay solicitudes de inscripci&oacute;n.";
#elif L==5
	"No enrollment requests.";			// Besoin de traduction
#elif L==6
	"No hay solicitudes de inscripci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"No enrollment requests.";			// Bisogno di traduzione
#elif L==8
	"No enrollment requests.";			// Potrzebujesz tlumaczenie
#elif L==9
	"No enrollment requests.";			// Necessita de traduo
#endif

const char *Txt_No_events =
#if   L==1
	"No hi ha esdeveniments.";
#elif L==2
	"Keine Ereignisse.";
#elif L==3
	"No events.";
#elif L==4
	"No hay eventos.";
#elif L==5
	"Il n'y a pas d'&eacute;v&eacute;nements.";
#elif L==6
	"No hay eventos.";	// Okoteve traduccin
#elif L==7
	"Non ci sono eventi.";
#elif L==8
	"Brak wydarzen.";
#elif L==9
	"N&atilde;o h&aacute; eventos.";
#endif

const char *Txt_No_file_index_html_index_htm_found_within_the_ZIP_file =
#if   L==1
	"No se ha encontrado un archivo <em>index.html</em> / <em>index.htm</em>"
	" dentro del archivo <em>ZIP</em>.";		// Necessita traduccio
#elif L==2
	"No file <em>index.html</em> / <em>index.htm</em> found"
	" within the <em>ZIP</em> file.";		// Need bersetzung
#elif L==3
	"No file <em>index.html</em> / <em>index.htm</em> found"
	" within the <em>ZIP</em> file.";
#elif L==4
	"No se ha encontrado un archivo <em>index.html</em> / <em>index.htm</em>"
	" dentro del archivo <em>ZIP</em>.";
#elif L==5
	"No file <em>index.html</em> / <em>index.htm</em> found"
	" within the <em>ZIP</em> file.";		// Besoin de traduction
#elif L==6
	"No se ha encontrado un archivo <em>index.html</em> / <em>index.htm</em>"
	" dentro del archivo <em>ZIP</em>.";		// Okoteve traduccin
#elif L==7
	"Nessun documento <em>index.html</em> / <em>index.htm</em> trovato"
	" nel file <em>ZIP</em>.";
#elif L==8
	"Nr pliku <em>indeks.html</em> / <em>indeks.htm</em> znalezionych"
	" w <em>ZIP</em> plik.";
#elif L==9
	"No file <em>index.html</em> / <em>index.htm</em> found"
	" within the <em>ZIP</em> file.";		// Necessita de traduo
#endif

const char *Txt_No_groups_have_been_created_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No se han creado grupos en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"No groups have been created in the course <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"No groups have been created in the course <strong>%s</strong>.";
#elif L==4
	"No se han creado grupos en la asignatura <strong>%s</strong>.";
#elif L==5
	"No groups have been created in the course <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"No se han creado grupos en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Nessun gruppo &egrave; stato creato nel corso <strong>%s</strong>.";
#elif L==8
	"Nr grupy zostaly utworzone w ramach <strong>%s</strong>.";
#elif L==9
	"No groups have been created in the course <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_No_groups_have_been_created_in_the_course_X_Therefore_ =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No se han creado grupos en la asignatura <strong>%s</strong>."
	" Por tanto, los usuarios especificados ser&aacute;n inscritos/eliminados en/de la asignatura.";	// Necessita traduccio
#elif L==2
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users will be enrolled/removed in/from the course.";				// Need bersetzung
#elif L==3
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users will be enrolled/removed in/from the course.";
#elif L==4
	"No se han creado grupos en la asignatura <strong>%s</strong>."
	" Por tanto, los usuarios especificados ser&aacute;n inscritos/eliminados en/de la asignatura.";
#elif L==5
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users will be enrolled/removed in/from the course.";				// Besoin de traduction
#elif L==6
	"No se han creado grupos en la asignatura <strong>%s</strong>."
	" Por tanto, los usuarios especificados ser&aacute;n inscritos/eliminados en/de la asignatura.";	// Okoteve traduccin
#elif L==7
	"Nessun gruppo &egrave; stato creato nel corso <strong>%s</strong>."
	"Dunque, gli utenti specificati saranno registrati/rimossi in/dal corso.";
#elif L==8
	"Nr grupy zostaly utworzone w ramach <strong>%s</strong>."
	"Dlatego okreslonych uzytkownik&oacute;w zostanie zarejestrowany / usuniete w / z kursu.";
#elif L==9
	"No groups have been created in the course <strong>%s</strong>."
	"Therefore, the specified users will be enrolled/removed in/from the course.";				// Necessita de traduo
#endif

const char *Txt_No_image =	// Without any image
#if   L==1
	"Sense imatge";
#elif L==2
	"Ohne Abbild";
#elif L==3
	"No image";
#elif L==4
	"Sin imagen";
#elif L==5
	"Pas d'image";
#elif L==6
	"Sin imagen";	// Okoteve traduccin
#elif L==7
	"Nessuna immagine";
#elif L==8
	"Brak obrazka";
#elif L==9
	"Sem imagem";
#endif

const char *Txt_No_INDEX =	// Short version of "Number" (as an index)...
#if   L==1
	"N&ordm;";	// Necessita traduccio
#elif L==2
	"Anzahl";
#elif L==3
	"#";
#elif L==4
	"N&ordm;";
#elif L==5
	"Nombre";
#elif L==6
	"Papanga";
#elif L==7
	"Numero";
#elif L==8
	"Numer";
#elif L==9
	"N&ordm;";
#endif

const char *Txt_No_information =
#if   L==1
	"No hi ha informaci&oacute;.";
#elif L==2
	"Keine Informationen.";
#elif L==3
	"No information.";
#elif L==4
	"No hay informaci&oacute;n.";
#elif L==5
	"Aucune information.";
#elif L==6
	"No hay informaci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Nessuna informazione.";
#elif L==8
	"Brak informacji.";
#elif L==9
	"N&atilde;o h&aacute; informa&ccedil;&atilde;o.";
#endif

const char *Txt_No_institutions =
#if   L==1
	"No hi ha institucions.";
#elif L==2
	"Keine Hochschulen.";
#elif L==3
	"No institutions.";
#elif L==4
	"No hay instituciones.";
#elif L==5
	"Il n'y a pas d'&eacute;tablissements.";
#elif L==6
	"No hay instituciones.";	// Okoteve traduccin
#elif L==7
	"Non ci sono istituzioni.";
#elif L==8
	"Brak instytucje.";
#elif L==9
	"N&atilde;o h&aacute; institu&ccedil;&otilde;es.";
#endif

const char *Txt_No_BR_msgs =
#if   L==1
	"N&ordm;<br />mens.";	// Necessita traduccio
#elif L==2
	"Anzahl<br />Nachr.";
#elif L==3
	"No.<br />msgs.";
#elif L==4
	"N&ordm;<br />mens.";
#elif L==5
	"Nombre<br />msgs.";
#elif L==6
	"N&ordm;<br />mens.";	// Okoteve traduccin
#elif L==7
	"Num.<br />msg.";
#elif L==8
	"Numer<br />msgs.";
#elif L==9
	"N&ordm;<br />mens.";
#endif

const char *Txt_No_new_posts =
#if   L==1
	"No hay mensajes nuevos";	// Necessita traduccio
#elif L==2
	"Keine neuen Nachrichten";
#elif L==3
	"No new posts";
#elif L==4
	"No hay mensajes nuevos";
#elif L==5
	"Pas de nouveaux messages";
#elif L==6
	"No hay mensajes nuevos";	// Okoteve traduccin
#elif L==7
	"Nessun nuovo post";
#elif L==8
	"Brak nowych post&oacute;w";
#elif L==9
	"Sem posts novos";
#endif

const char *Txt_No_notices =
#if   L==1
	"No hay avisos.";	// Necessita traduccio
#elif L==2
	"Keine Ank&uuml;ndigungen.";
#elif L==3
	"No notices.";
#elif L==4
	"No hay avisos.";
#elif L==5
	"Il n'y a pas d'avis.";
#elif L==6
	"No hay avisos.";	// Okoteve traduccin
#elif L==7
	"Non ci sono avvisi.";
#elif L==8
	"Brak ogloszen.";
#elif L==9
	"N&atilde;o h&aacute; avisos.";
#endif

const char *Txt_No_of_files =
#if   L==1
	"Nombre d'arxius";
#elif L==2
	"Anzahl der Dateien";
#elif L==3
	"Number of files";
#elif L==4
	"N&ordm; de archivos";
#elif L==5
	"Nombre de fichiers";
#elif L==6
	"N&ordm; de archivos";	// Okoteve traduccin
#elif L==7
	"Numero di file";
#elif L==8
	"Liczba pliki";
#elif L==9
	"N&ordm; de arquivos";
#endif

const char *Txt_No_of_files_in_SHARE_zones =
#if   L==1
	"Nombre d'arxius compartits";
#elif L==2
	"Anzahl der freigegebene Dateien";
#elif L==3
	"Number of shared files";
#elif L==4
	"N&ordm; de archivos compartidos";
#elif L==5
	"Nombre de fichiers partag&eacute;s";
#elif L==6
	"N&ordm; de archivos compartidos";		// Okoteve traduccin
#elif L==7
	"Numero di file condivisi";
#elif L==8
	"Liczba pliki udost&eogon;pnione";
#elif L==9
	"N&ordm; de arquivos compartilhados";
#endif

const char *Txt_No_of_files_in_DOCUM_zones =
#if   L==1
	"N&ordm; de archivos en zonas de documentos";	// Necessita traduccio
#elif L==2
	"Anzahl der Dateien im Dokumente-Bereich";
#elif L==3
	"Number of files in documents area";
#elif L==4
	"N&ordm; de archivos en zonas de documentos";
#elif L==5
	"Nombre de fichiers dans les zones de documents";
#elif L==6
	"N&ordm; de archivos en zonas de documentos";	// Okoteve traduccin
#elif L==7
	"Numero di file nell'area documenti";
#elif L==8
	"Number of files in documents area";	// Potrzebujesz tlumaczenie
#elif L==9
	"N&ordm; de arquivos em zonas de documentos";
#endif

const char *Txt_No_of_forums =
#if   L==1
	"N&ordm; de foros";	// Necessita traduccio
#elif L==2
	"Anzahl der Foren";
#elif L==3
	"No. of forums";
#elif L==4
	"N&ordm; de foros";
#elif L==5
	"Nombre de forums";
#elif L==6
	"N&ordm; de foros";	// Okoteve traduccin
#elif L==7
	"Numero di forum";
#elif L==8
	"Liczba forum";
#elif L==9
	"N&ordm; de f&oacute;runs";
#endif

const char *Txt_No_of_BR_groups =
#if   L==1
	"N&ordm; de<br />grupos";	// Necessita traduccio
#elif L==2
	"Anzahl der<br />Gruppen";
#elif L==3
	"No. of<br />groups";
#elif L==4
	"N&ordm; de<br />grupos";
#elif L==5
	"Nombre de<br />groupes";
#elif L==6
	"N&ordm; de<br />grupos";	// Okoteve traduccin
#elif L==7
	"Numero di<br />gruppi";
#elif L==8
	"Liczba z<br />grupy";
#elif L==9
	"N&ordm; de<br />grupos";
#endif

const char *Txt_No_of_indicators =
#if   L==1
	"N&ordm; de indicadores";	// Necessita traduccio
#elif L==2
	"Anzahl der Indikatoren";
#elif L==3
	"No. of indicators";
#elif L==4
	"N&ordm; de indicadores";
#elif L==5
	"Nombre d'indicateurs";
#elif L==6
	"N&ordm; de indicadores";	// Okoteve traduccin
#elif L==7
	"Numero di indicatori";
#elif L==8
	"Liczba z wskazniki";
#elif L==9
	"N&ordm; de indocadores";
#endif

const char *Txt_No_of_BR_lines =
#if   L==1
	"N&ordm; de<br />l&iacute;neas";	// Necessita traduccio
#elif L==2
	"Anzahl der<br />Zeilen";
#elif L==3
	"No. of<br />lines";
#elif L==4
	"N&ordm; de<br />l&iacute;neas";
#elif L==5
	"Nombre de<br />lignes";
#elif L==6
	"N&ordm; de<br />l&iacute;neas";	// Okoteve traduccin
#elif L==7
	"Numero di<br />linee";
#elif L==8
	"Liczba z<br />linie";
#elif L==9
	"N&ordm; de<br />linhas";
#endif

const char *Txt_No_of_posts =
#if   L==1
	"N&ordm; de mensajes";	// Necessita traduccio
#elif L==2
	"Anzahl der Nachrichten";
#elif L==3
	"No. of posts";
#elif L==4
	"N&ordm; de mensajes";
#elif L==5
	"Nombre de messages";
#elif L==6
	"N&ordm; de mensajes";	// Okoteve traduccin
#elif L==7
	"Numero di post";
#elif L==8
	"Liczba wiadomosci";
#elif L==9
	"N&ordm; de mensagens";
#endif

const char *Txt_No_of_posts_BR_per_forum =
#if   L==1
	"N&ordm; de mensajes<br />por foro";	// Necessita traduccio
#elif L==2
	"Anzahl der Nachrichten<br />pro Forum";
#elif L==3
	"No. of posts<br />per forum";
#elif L==4
	"N&ordm; de mensajes<br />por foro";
#elif L==5
	"Nombre de messages<br />par forum";
#elif L==6
	"N&ordm; de mensajes<br />por foro";	// Okoteve traduccin
#elif L==7
	"Numero di post<br />per forum";
#elif L==8
	"Liczba wiadomosci<br />per forum";
#elif L==9
	"N&ordm; de mensagens<br />por f&oacute;rum";
#endif

const char *Txt_No_of_posts_BR_per_user =
#if   L==1
	"N&ordm; de mensajes<br />por usuari";	// Necessita traduccio
#elif L==2
	"Anzahl der Nachrichten<br />pro Benutzer";
#elif L==3
	"No. of posts<br />per user";
#elif L==4
	"N&ordm; de mensajes<br />por usuario";
#elif L==5
	"Nombre de messages<br />par utilisateur";
#elif L==6
	"N&ordm; de mensajes<br />por puruh&aacute;ra";	// Okoteve traduccin
#elif L==7
	"Numero di post<br />per utente";
#elif L==8
	"Liczba wiadomosci<br />per u&zdot;ytkownik";
#elif L==9
	"N&ordm; de mensagens<br />por utilizador";
#endif

const char *Txt_No_of_posts_BR_per_thread =
#if   L==1
	"N&ordm; de mensajes<br />por discusi&oacute;n";	// Necessita traduccio
#elif L==2
	"Anzahl der Nachrichten<br />pro Thread";
#elif L==3
	"No. of posts<br />per thread";
#elif L==4
	"N&ordm; de mensajes<br />por usuario";
#elif L==5
	"Nombre de messages<br />par fil";
#elif L==6
	"N&ordm; de mensajes<br />por discusi&oacute;n";	// Okoteve traduccin
#elif L==7
	"Numero di post<br />per discussione";
#elif L==8
	"Liczba wiadomosci<br />per watku";
#elif L==9
	"N&ordm; de mensagens<br />por thread";
#endif

const char *Txt_No_of_private_files =
#if   L==1
	"Nombre d'arxius privats";
#elif L==2
	"Anzahl der private Dateien";
#elif L==3
	"Number of private files";
#elif L==4
	"N&ordm; de archivos privados";
#elif L==5
	"Nombre de fichiers priv&eacute;s";
#elif L==6
	"N&ordm; de archivos privados";	// Okoteve traduccin
#elif L==7
	"Numero di file privati";
#elif L==8
	"Liczba prywatne pliki";
#elif L==9
	"N&ordm; de arquivos privados";
#endif

const char *Txt_No_of_public_files =
#if   L==1
	"Nombre d'arxius p&uacute;blics";
#elif L==2
	"Anzahl der &ouml;ffentliche Dateien";
#elif L==3
	"Number of public files";
#elif L==4
	"N&ordm; de archivos p&uacute;blicos";
#elif L==5
	"Nombre de fichiers publics";
#elif L==6
	"N&ordm; de archivos p&uacute;blicos";	// Okoteve traduccin
#elif L==7
	"Numero di file pubblici";
#elif L==8
	"Liczba publicznych pliki";
#elif L==9
	"N&ordm; de arquivos p&uacute;blicos";
#endif

const char *Txt_No_of_questions =
#if   L==1
	"Nombre de preguntes";
#elif L==2
	"Anzahl der Fragen";
#elif L==3
	"No. of questions";
#elif L==4
	"N&ordm; de preguntas";
#elif L==5
	"Nombre de questions";
#elif L==6
	"N&ordm; de preguntas";	// Okoteve traduccin
#elif L==7
	"Numero di domande";
#elif L==8
	"Liczba pyta&nacute;";
#elif L==9
	"N&ordm; de quest&otilde;es";
#endif

const char *Txt_No_of_social_posts =
#if   L==1
	"Nombre de missatges socials";
#elif L==2
	"Anzahl der sozialen Beitr&auml;ge";
#elif L==3
	"No. of social posts";
#elif L==4
	"N&ordm; de mensajes sociales";
#elif L==5
	"Nombre de messages sociaux";
#elif L==6
	"N&ordm; de mensajes sociales";	// Okoteve traduccin
#elif L==7
	"Numero di post sociali";
#elif L==8
	"Liczba post spo&lstrok;ecznej";
#elif L==9
	"N&ordm; de post sociais";
#endif

const char *Txt_No_of_threads =
#if   L==1
	"N&ordm; de discusiones";	// Necessita traduccio
#elif L==2
	"Anzahl der Threads";
#elif L==3
	"No. of threads";
#elif L==4
	"N&ordm; de discusiones";
#elif L==5
	"Nombre de fils";
#elif L==6
	"N&ordm; de discusiones";	// Okoteve traduccin
#elif L==7
	"Numero di discussioni";
#elif L==8
	"Liczba watk&oacute;w";
#elif L==9
	"N&ordm; de threads";
#endif

const char *Txt_No_of_threads_BR_per_forum =
#if   L==1
	"N&ordm; de discusiones<br />por foro";	// Necessita traduccio
#elif L==2
	"Anzahl der Threads<br />pro Forum";
#elif L==3
	"No. of threads<br />per forum";
#elif L==4
	"N&ordm; de discusiones<br />por foro";
#elif L==5
	"Nombre de fils<br />par forum";
#elif L==6
	"N&ordm; de discusiones<br />por foro";	// Okoteve traduccin
#elif L==7
	"Numero di discussioni<br />per forum";
#elif L==8
	"Liczba watk&oacute;w<br />per forum";
#elif L==9
	"N&ordm; de threads<br />por f&oacute;rum";
#endif

const char *Txt_Number_of_BR_times_that_BR_questions_BR_have_been_BR_responded =
#if   L==1
	"N&ordm; de<br />veces<br />que se ha<br />respondido";	// Necessita traduccio
#elif L==2
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Need bersetzung
#elif L==3
	"Number of<br />times that<br />questions<br />have been<br />responded";
#elif L==4
	"N&ordm; de<br />veces<br />que se ha<br />respondido";
#elif L==5
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Besoin de traduction
#elif L==6
	"N&ordm; de<br />veces<br />que se ha<br />respondido";	// Okoteve traduccin
#elif L==7
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Bisogno di traduzione
#elif L==8
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Potrzebujesz tlumaczenie
#elif L==9
	"Number of<br />times that<br />questions<br />have been<br />responded";	// Necessita de traduo
#endif

const char *Txt_No_of_users =
#if   L==1
	"Nombre d'usuaris";	// Necessita traduccio
#elif L==2
	"Anzahl der Benutzer";
#elif L==3
	"No. of users";
#elif L==4
	"N&ordm; de usuarios";
#elif L==5
	"Nombre d'utilisateurs";
#elif L==6
	"N&ordm; de usuarios";	// Okoteve traduccin
#elif L==7
	"Numero di utenti";
#elif L==8
	"Liczba uzytkownik&oacute;w";
#elif L==9
	"N&ordm; de utilizadores";
#endif

const char *Txt_No_of_users_who_will_be_notified_by_email =
#if   L==1
	"N&ordm; de usuarios que ser&aacute;n notificados por correo";	// Necessita traduccio
#elif L==2
	"Anzahl der Benutzer, um benachrichtigt zu werden per E-Mail";
#elif L==3
	"No. of users who will be notified by email";
#elif L==4
	"N&ordm; de usuarios que ser&aacute;n notificados por correo";
#elif L==5
	"Nombre d'utilisateurs qui seront notifi&eacute;s par email";
#elif L==6
	"N&ordm; de usuarios que ser&aacute;n notificados por correo";	// Okoteve traduccin
#elif L==7
	"Numero di utenti che saranno notificati via email";
#elif L==8
	"Liczba uzytkownik&oacute;w, kt&oacute;rzy zostana powiadomieni przez email";
#elif L==9
	"N&ordm; de utilizadores que ser&atilde;o notificados por email";
#endif

const char *Txt_No_questions_found_matching_your_search_criteria =
#if   L==1
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.";	// Necessita traduccio
#elif L==2
	"Keine Testfragen gefunden, die mit Ihrer Suchanfrage entsprechen.";
#elif L==3
	"No questions found matching your search criteria.";
#elif L==4
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.";
#elif L==5
	"Aucune question trouv&eacute;e correspondant &agrave; vos crit&egrave;res de recherche.";
#elif L==6
	"No hay preguntas con el criterio de b&uacute;squeda seleccionado.";	// Okoteve traduccin
#elif L==7
	"Non ci sono domande trovate con il criterio di ricerca selezionato.";
#elif L==8
	"Brak pyta&nacute; spelniajacych zadane kryteria wyszukiwania.";
#elif L==9
	"N&atilde;o quest&otilde;es com os seus crit&eacute;rios de pesquisa selecionados.";
#endif

const char *Txt_No_results =
#if   L==1
	"No hi ha resultats.";
#elif L==2
	"Keine Ergebnisse.";
#elif L==3
	"No results.";
#elif L==4
	"No hay resultados.";
#elif L==5
	"Pas de r&eacute;sultats.";
#elif L==6
	"No hay resultados.";		// Okoteve traduccin
#elif L==7
	"Nessun risultato.";
#elif L==8
	"Brak wynik&oacute;w.";
#elif L==9
	"Nenhum resultado.";
#endif

const char *Txt_no_subject =
#if   L==1
	"sin asunto";	// Necessita traduccio
#elif L==2
	"kein Betreff";
#elif L==3
	"no subject";
#elif L==4
	"sin asunto";
#elif L==5
	"pas de sujet";
#elif L==6
	"sin asunto";	// Okoteve traduccin
#elif L==7
	"nessun oggetto";
#elif L==8
	"bez tematu";
#elif L==9
	"sem assunto";
#endif

const char *Txt_No_surveys =
#if   L==1
	"No hi ha enquestes.";
#elif L==2
	"Keine Umfragen.";
#elif L==3
	"No surveys.";
#elif L==4
	"No hay encuestas.";
#elif L==5
	"Il n'y a pas de sondages.";
#elif L==6
	"No hay encuestas.";	// Okoteve traduccin
#elif L==7
	"Non ci sono sondaggi.";
#elif L==8
	"Brak badania.";
#elif L==9
	"N&atilde;o h&aacute; inqu&eacute;ritos.";
#endif

const char *Txt_no_tags =		// Tags used in test
#if   L==1
	"sin descriptores";	// Necessita traduccio
#elif L==2
	"keine Tags";
#elif L==3
	"no tags";
#elif L==4
	"sin descriptores";
#elif L==5
	"pas de descripteurs";
#elif L==6
	"sin descriptores";	// Okoteve traduccin
#elif L==7
	"senza etichette";
#elif L==8
	"zadnych tag&oacute;w";
#elif L==9
	"sem descritores";
#endif

const char *Txt_No_teachers_or_students_found =
#if   L==1
	"No hay profesores ni estudiantes.";	// Necessita traduccio
#elif L==2
	"Keine Lehrer oder Studenten gefunden.";
#elif L==3
	"No teachers or students found.";
#elif L==4
	"No hay profesores ni estudiantes.";
#elif L==5
	"Aucun enseignant ou &eacute;tudiant trouv&eacute;.";
#elif L==6
	"No hay profesores ni estudiantes.";	// Okoteve traduccin
#elif L==7
	"Nessun professore o studente trovato.";
#elif L==8
	"Nie znaleziono nauczycieli i uczni&oacute;w.";
#elif L==9
	"Nenhum professor ou estudante encontrado.";
#endif

const char *Txt_No_test_questions =
#if   L==1
	"No hay preguntas de test.";	// Necessita traduccio
#elif L==2
	"Keine Testfragen.";
#elif L==3
	"No test questions.";
#elif L==4
	"No hay preguntas de test.";
#elif L==5
	"Il n'y a pas de questions de test.";
#elif L==6
	"No hay preguntas de test.";	// Okoteve traduccin
#elif L==7
	"Non ci sono domande di test.";
#elif L==8
	"Brak pyta&nacute; testowych.";
#elif L==9
	"N&atilde;o existem quest&otilde;es de test.";
#endif

const char *Txt_No_user_has_been_eliminated =
#if   L==1
	"No se ha eliminado ning&uacute;n usuario.";	// Necessita traduccio
#elif L==2
	"No user has been eliminated.";			// Need bersetzung
#elif L==3
	"No user has been eliminated.";
#elif L==4
	"No se ha eliminado ning&uacute;n usuario.";
#elif L==5
	"No user has been eliminated.";			// Besoin de traduction
#elif L==6
	"No se ha eliminado ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7
	"Nessun utente &egrave; stato rimosso.";
#elif L==8
	"Zaden uzytkownik nie zostal wyeliminowany.";
#elif L==9
	"No user has been eliminated.";			// Necessita de traduo
#endif

const char *Txt_No_user_has_been_enrolled =
#if   L==1
	"No se ha inscrito ning&uacute;n usuario.";	// Necessita traduccio
#elif L==2
	"No user has been enrolled.";			// Need bersetzung
#elif L==3
	"No user has been enrolled.";
#elif L==4
	"No se ha inscrito ning&uacute;n usuario.";
#elif L==5
	"No user has been enrolled.";			// Besoin de traduction
#elif L==6
	"No se ha inscrito ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7
	"Nessun utente &egrave; stato registrato.";
#elif L==8
	"Zaden uzytkownik nie zostal zarejestrowany.";
#elif L==9
	"No user has been enrolled.";			// Necessita de traduo
#endif

const char *Txt_No_user_has_been_removed =
#if   L==1
	"No se ha eliminado ning&uacute;n usuario.";	// Necessita traduccio
#elif L==2
	"No user has been removed.";			// Need bersetzung
#elif L==3
	"No user has been removed.";
#elif L==4
	"No se ha eliminado ning&uacute;n usuario.";
#elif L==5
	"No user has been removed.";			// Besoin de traduction
#elif L==6
	"No se ha eliminado ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7
	"Nessun utente &egrave; stato rimosso.";
#elif L==8
	"Zaden uzytkownik nie zostal usuniety.";
#elif L==9
	"No user has been removed.";			// Necessita de traduo
#endif

const char *Txt_No_user_to_whom_you_can_follow_Try_again_later =
#if   L==1
	"No hi ha cap usuari a qui vost&egrave; pugui seguir."
	" Intenteu-ho m&eacute;s endavant.";
#elif L==2
	"Kein Benutzer dem Sie folgen k&ouml;nnen."
	" Versuchen Sie es sp&auml;ter noch einmal.";
#elif L==3
	"No user to whom you can follow."
	" Try again later.";
#elif L==4
	"No hay ning&uacute;n usuario a quien usted pueda seguir."
	" Int&eacute;ntelo m&aacute;s adelante.";
#elif L==5
	"Aucun utilisateur auquel vous pouvez suivre."
	" R&eacute;essayez plus tard.";
#elif L==6
	"No hay ning&uacute;n usuario a quien usted pueda seguir."
	" Int&eacute;ntelo m&aacute;s adelante.";	// Okoteve traduccin
#elif L==7
	"Nessun utente a cui &egrave; possibile seguire."
	" Riprovare pi&ugrave; tardi.";
#elif L==8
	"No u&zdot;ytkownika, do kt&oacute;rego mo&zdot;na &sacute;ledzi&cacute;."
	" Spr&oacute;buj ponownie p&oacute;&zacute;niej.";
#elif L==9
	"Nenhum usu&aacute;rio a quem voc&ecirc; pode seguir."
	" Tente novamente mais tarde.";
#endif

const char *Txt_No_users_found[Rol_NUM_ROLES] =
   {
#if   L==1	// Rol_ROLE_UNKNOWN
	 "No hay usuarios."	// Necessita traduccio
#elif L==2
	 "Keine Benutzer gefunden."
#elif L==3
	 "No users found."
#elif L==4
	 "No hay usuarios."
#elif L==5
	 "Aucun utilisateur trouv&eacute;."
#elif L==6
	 "No hay usuarios."	// Okoteve traduccin
#elif L==7
	 "Nessun utente trovato."
#elif L==8
	 "Nie znaleziono uzytkownik&oacute;w."
#elif L==9
	 "Nenhum utilizador encontrado."
#endif
	 ,
#if   L==1	// Rol_ROLE_GUEST__
	 "No hi ha convidats."
#elif L==2
	 "Keine Eingeladen gefunden."
#elif L==3
	 "No guests found."
#elif L==4
	 "No hay invitados."
#elif L==5
	 "Aucun invit&eacute; trouv&eacute;."
#elif L==6
	 "No hay invitados."	// Okoteve traduccin
#elif L==7
	 "Nessun ospite trovato."
#elif L==8
	 "Nie znaleziono zaproszeni."
#elif L==9
	 "Nenhum convidado encontrado."
#endif
	 ,
#if   L==1	// Rol_ROLE_VISITOR
	 "No hay visitantes."	// Necessita traduccio
#elif L==2
	 "Keine Besucher gefunden."
#elif L==3
	 "No visitors found."
#elif L==4
	 "No hay visitantes."
#elif L==5
	 "Aucun visiteur trouv&eacute;."
#elif L==6
	 "No hay visitantes."	// Okoteve traduccin
#elif L==7
	 "Nessun visitatore trovato."
#elif L==8
	 "No visitors found."	// Potrzebujesz tlumaczenie
#elif L==9
	 "Nenhum visitante encontrado."
#endif
	 ,
#if   L==1	// Rol_ROLE_STUDENT
	 "No hay estudiantes."	// Necessita traduccio
#elif L==2
	 "Keine Studenten gefunden."
#elif L==3
	 "No students found."
#elif L==4
	 "No hay estudiantes."
#elif L==5
	 "Aucun &eacute;tudiant trouv&eacute;."
#elif L==6
	 "No hay estudiantes."	// Okoteve traduccin
#elif L==7
	 "Non ci sono studenti."
#elif L==8
	 "No students found."	// Potrzebujesz tlumaczenie
#elif L==9
	 "Nenhum estudante encontrado."
#endif
	 ,
#if   L==1	// Rol_ROLE_TEACHER
	 "No hay profesores."	// Necessita traduccio
#elif L==2
	 "Keine Lehrkr&auml;ften gefunden."
#elif L==3
	 "No teachers found."
#elif L==4
	 "No hay profesores."
#elif L==5
	 "Aucun enseignant trouv&eacute;."
#elif L==6
	 "No hay profesores."	// Okoteve traduccin
#elif L==7
	 "Non ci sono professori."
#elif L==8
	 "Nie znaleziono nauczycieli."
#elif L==9
	 "Nenhum professor encontrado."
#endif
	 ,
#if   L==1	// Rol_ROLE_DEG_ADM
	 "No hay administradores."	// Necessita traduccio
#elif L==2
	 "Keine Administratoren gefunden."
#elif L==3
	 "No administrators found."
#elif L==4
	 "No hay administradores."
#elif L==5
	 "Aucun administrateur trouv&eacute;."
#elif L==6
	 "No hay administradores."	// Okoteve traduccin
#elif L==7
	 "Non ci sono amministratori."
#elif L==8
	 "Nie znaleziono administrator&oacute;w."
#elif L==9
	 "Nenhum administrador encontrado."
#endif
	 ,
#if   L==1	// Rol_ROLE_CTR_ADM
	 "No hay administradores."	// Necessita traduccio
#elif L==2
	 "Keine Administratoren gefunden."
#elif L==3
	 "No administrators found."
#elif L==4
	 "No hay administradores."
#elif L==5
	 "Aucun administrateur trouv&eacute;."
#elif L==6
	 "No hay administradores."	// Okoteve traduccin
#elif L==7
	 "Non ci sono amministratori."
#elif L==8
	 "Nie znaleziono administrator&oacute;w."
#elif L==9
	 "Nenhum administrador encontrado."
#endif
	 ,
#if   L==1	// Rol_ROLE_INS_ADM
	 "No hay administradores."	// Necessita traduccio
#elif L==2
	 "Keine Administratoren gefunden."
#elif L==3
	 "No administrators found."
#elif L==4
	 "No hay administradores."
#elif L==5
	 "Aucun administrateur trouv&eacute;."
#elif L==6
	 "No hay administradores."	// Okoteve traduccin
#elif L==7
	 "Non ci sono amministratori."
#elif L==8
	 "Nie znaleziono administrator&oacute;w."
#elif L==9
	 "Nenhum administrador encontrado."
#endif
	 ,
#if   L==1	// Rol_ROLE_SYS_ADM
	 "No hay superusuarios."	// Necessita traduccio
#elif L==2
	 "Keine Superuser gefunden."
#elif L==3
	 "No superusers found."
#elif L==4
	 "No hay superusuarios."
#elif L==5
	 "Aucun superutilisateur trouv&eacute;."
#elif L==6
	 "No hay superusuarios."	// Okoteve traduccin
#elif L==7
	 "Non ci sono superutenti."
#elif L==8
	 "Nie znaleziono superu&zdot;ytkownika."
#elif L==9
	 "Nenhum superutilizador encontrado."
#endif
	};

const char *Txt_nobody_else_can_access_this_content =
#if   L==1
	"ning&uacute; m&eacute;s pot accedir a aquest contingut";
#elif L==2
	"Niemand sonst kann auf diese Inhalte";
#elif L==3
	"nobody else can access this content";
#elif L==4
	"nadie m&aacute;s puede acceder a este contenido";
#elif L==5
	"personne d'autre ne peut acc&eacute;der &agrave; ce contenu";
#elif L==6
	"nadie m&aacute;s puede acceder a este contenido";	// Okoteve traduccin
#elif L==7
	"nessun altro pu&ograve; accedere a questo contenuto";
#elif L==8
	"nikt inny nie moze uzyskac dostepu do tej zawartosci";
#elif L==9
	"ningu&eacute;m mais pode acessar este conte&uacute;do";
#endif

const char *Txt_non_blank_QUESTIONS =
#if   L==1
	"contestades";
#elif L==2
	"Non-blank";
#elif L==3
	"non-blank";
#elif L==4
	"contestadas";
#elif L==5
	"non vide";
#elif L==6
	"contestadas";	// Okoteve traduccin
#elif L==7
	"risposte";
#elif L==8
	"niepustych";
#elif L==9
	"respondidas";
#endif

const char *Txt_Non_blank_BR_questions =
#if   L==1
	"Preguntes<br />contestades";
#elif L==2
	"Non-blank<br />Fragen";
#elif L==3
	"Non-blank<br />questions";
#elif L==4
	"Preguntas<br />contestadas";
#elif L==5
	"Questions<br />non vide";
#elif L==6
	"Preguntas<br />contestadas";	// Okoteve traduccin
#elif L==7
	"Domande<br />risposte";
#elif L==8
	"Niepustych<br />pytania";
#elif L==9
	"Quest&otilde;es<br />respondidas";
#endif

const char *Txt_Not_applicable =
#if   L==1
	"No aplicable";
#elif L==2
	"Nicht anwendbar";
#elif L==3
	"Not applicable";
#elif L==4
	"No aplicable";
#elif L==5
	"Non applicable";
#elif L==6
	"No aplicable";		// Okoteve traduccin
#elif L==7
	"Non applicabile";
#elif L==8
	"Nie dotyczy";
#elif L==9
	"N&atilde;o aplic&aacute;vel";
#endif

const char *Txt_not_available =
#if   L==1
	"no disponible";
#elif L==2
	"Nicht verf&uuml;gbar";
#elif L==3
	"not available";
#elif L==4
	"no disponible";
#elif L==5
	"pas disponible";
#elif L==6
	"no disponible";		// Okoteve traduccin
#elif L==7
	"non disponibile";
#elif L==8
	"niedost&eogon;pne";
#elif L==9
	"n&atilde;o dispon&iacute;vel";
#endif

const char *Txt_Not_duplicated =
#if   L==1
	"No &eacute;s duplicat";
#elif L==2
	"Nicht dupliziert";
#elif L==3
	"Not duplicated";
#elif L==4
	"No es duplicado";
#elif L==5
	"Pas dupliqu&eacute;";
#elif L==6
	"No es duplicado";		// Okoteve traduccin
#elif L==7
	"Non duplicato";
#elif L==8
	"Nie powiela&cacute;";
#elif L==9
	"N&atilde;o duplicado";
#endif

const char *Txt_Nothing_has_been_pasted_because_the_clipboard_is_empty_ =
#if   L==1
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o."
	" Trate de copiar y pegar de nuevo.";	// Necessita traduccio
#elif L==2
	"Es konnten keine Daten eingef&uuml;gt werden. Der Zwischenspeicher ist leer."
	"Versuchen Sie den Kopieren-Einf&uuml;gen-Vorgang bitte erneut.";
#elif L==3
	"Nothing has been pasted because the clipboard is empty."
	"Try copy-paste again.";
#elif L==4
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o."
	" Trate de copiar y pegar de nuevo.";
#elif L==5
	"Nothing has been pasted because the clipboard is empty."
	"Try copy-paste again.";		// Besoin de traduction
#elif L==6
	"No se pegado nada porque el portapapeles est&aacute; vac&iacute;o."
	" Trate de copiar y pegar de nuevo.";	// Okoteve traduccin
#elif L==7
	"Niente &egrave; stato copiato perch&egrave; la bacheca &egrave; piena."
	"Prova copia-incolla nuovamente.";
#elif L==8
	"Nic nie zostal wklejony bo schowek jest pusty."
	"Spr&oacute;buj ponownie kopiuj-wklej.";
#elif L==9
	"Nothing has been pasted because the clipboard is empty."
	"Try copy-paste again.";		// Necessita de traduo
#endif

const char *Txt_NOTICE_Active_SINGULAR =
#if   L==1
	"Activo";	// Necessita traduccio
#elif L==2
	"Aktive";
#elif L==3
	"Active";
#elif L==4
	"Activo";
#elif L==5
	"Active";
#elif L==6
	"Activo";	// Okoteve traduccin
#elif L==7
	"Attivo";
#elif L==8
	"Active";
#elif L==9
	"Ativo";
#endif

const char *Txt_NOTICE_Active_BR_notices =
#if   L==1
	"Avisos<br />activos";	// Necessita traduccio
#elif L==2
	"Aktive<br />Ank&uuml;ndigungen";
#elif L==3
	"Active<br />notices";
#elif L==4
	"Avisos<br />activos";
#elif L==5
	"Avis<br />actives";
#elif L==6
	"Avisos<br />activos";	// Okoteve traduccin
#elif L==7
	"Avvisi<br />attivi";
#elif L==8
	"Active<br />Ogloszenia";
#elif L==9
	"Avisos<br />ativos";
#endif

const char *Txt_NOTICE_Active_Mark_as_obsolete =
#if   L==1
	"Actiu; marcar com obsolet";
#elif L==2
	"Aktive; marke als veraltet";
#elif L==3
	"Active; mark as obsolete";
#elif L==4
	"Activo; marcar como obsoleto";
#elif L==5
	"Active; marquer comme obsol&egrave;te";
#elif L==6
	"Activo; marcar como obsoleto";	// Okoteve traduccin
#elif L==7
	"Attivo; segnare come obsoleto";
#elif L==8
	"Active; zaznaczy jako przestarzale";
#elif L==9
	"Ativo; marcar como obsoleto";
#endif

const char *Txt_NOTICE_Deleted_BR_notices =
#if   L==1
	"Avisos<br />eliminados";	// Necessita traduccio
#elif L==2
	"Gel&ouml;scht<br />Ank&uuml;ndigungen";
#elif L==3
	"Deleted<br />notices";
#elif L==4
	"Avisos<br />eliminados";
#elif L==5
	"Avis<br />supprim&eacute;s";
#elif L==6
	"Avisos<br />eliminados";	// Okoteve traduccin
#elif L==7
	"Avvisi<br />rimossi";
#elif L==8
	"Usuniete<br />notices";
#elif L==9
	"Avisos<br />suprimidos";
#endif

const char *Txt_NOTICE_Obsolete_SINGULAR =
#if   L==1
	"Obsoleto";	// Necessita traduccio
#elif L==2
	"Veraltet";
#elif L==3
	"Obsolete";
#elif L==4
	"Obsoleto";
#elif L==5
	"Obsol&egrave;te";
#elif L==6
	"Obsoleto";	// Okoteve traduccin
#elif L==7
	"Obsoleto";
#elif L==8
	"Przestarzaly";
#elif L==9
	"Obsoleto";
#endif

const char *Txt_NOTICE_Obsolete_BR_notices =
#if   L==1
	"Avisos<br />obsoletos";	// Necessita traduccio
#elif L==2
	"Veraltet<br />Ank&uuml;ndigungen";
#elif L==3
	"Obsolete<br />notices";
#elif L==4
	"Avisos<br />obsoletos";
#elif L==5
	"Avis<br />obsol&egrave;tes";
#elif L==6
	"Avisos<br />obsoletos";	// Okoteve traduccin
#elif L==7
	"Avvisi<br />obsoleti";
#elif L==8
	"Przestarzaly<br />Ogloszenia";
#elif L==9
	"Avisos<br />obsoletos";
#endif

const char *Txt_NOTICE_Obsolete_Mark_as_active =
#if   L==1
	"Obsolet; marcar com actiu";
#elif L==2
	"Veraltet; marke als aktive";
#elif L==3
	"Obsolete; mark as active";
#elif L==4
	"Obsoleto; marcar como activo";
#elif L==5
	"Obsol&egrave;te; marquer comme active";
#elif L==6
	"Obsoleto; marcar como activo";	// Okoteve traduccin
#elif L==7
	"Obsoleto; segnare come attivo";
#elif L==8
	"Przestarzaly; znak towarowy jako aktywnych";
#elif L==9
	"Obsoleto; marcar como ativo";
#endif

const char *Txt_Notice =
#if   L==1
	"Av&iacute;s";
#elif L==2
	"Ank&uuml;ndigung";
#elif L==3
	"Notice";
#elif L==4
	"Aviso";
#elif L==5
	"Avis";
#elif L==6
	"Marandu";
#elif L==7
	"Avviso";
#elif L==8
	"Informacja";
#elif L==9
	"Aviso";
#endif

const char *Txt_Notice_created =
#if   L==1
	"Av&iacute;s creat.";
#elif L==2
	"Ank&uuml;ndigung erstellt.";
#elif L==3
	"Notice created.";
#elif L==4
	"Aviso creado.";
#elif L==5
	"Avis cr&eacute;&eacute;.";
#elif L==6
	"Aviso creado.";	// Okoteve traduccin
#elif L==7
	"Avviso creato.";
#elif L==8
	"Zawiadomienie stworzony.";
#elif L==9
	"Aviso criado.";
#endif

const char *Txt_Notice_removed =
#if   L==1
	"Av&iacute;s eliminat.";
#elif L==2
	"Ank&uuml;ndigung entfernt.";
#elif L==3
	"Notice removed.";
#elif L==4
	"Aviso eliminado";
#elif L==5
	"Avis supprim&eacute;.";
#elif L==6
	"Aviso eliminado.";	// Okoteve traduccin
#elif L==7
	"Avviso rimosso.";
#elif L==8
	"Informacja usuniete.";
#elif L==9
	"Aviso removido.";
#endif

const char *Txt_Notices =
#if   L==1
	 "Avisos";
#elif L==2
	 "Ank&uuml;ndigungen";
#elif L==3
	 "Notices";
#elif L==4
	 "Avisos";
#elif L==5
	 "Avis";
#elif L==6
	 "Marandu";
#elif L==7
	 "Avvisi";
#elif L==8
	 "Ogloszenia";
#elif L==9
	 "Avisos";
#endif

const char *Txt_NOTIF_new_SINGULAR =
#if   L==1
	"nueva";	// Necessita traduccio
#elif L==2
	"neue";
#elif L==3
	"new";
#elif L==4
	"nueva";
#elif L==5
	"nouvelle";
#elif L==6
	"nueva";	// Okoteve traduccin
#elif L==7
	"nuova";
#elif L==8
	"nowy";
#elif L==9
	"nova";
#endif

const char *Txt_NOTIF_new_PLURAL =
#if   L==1
	"nuevas";	// Necessita traduccio
#elif L==2
	"neue";
#elif L==3
	"new";
#elif L==4
	"nuevas";
#elif L==5
	"nouvelles";
#elif L==6
	"nuevas";	// Okoteve traduccin
#elif L==7
	"nuove";
#elif L==8
	"nowy";
#elif L==9
	"novas";
#endif

const char *Txt_NOTIFICATION_STATUS[Ntf_NUM_STATUS_TXT] =
   {
#if   L==1	// Ntf_STATUS_NO_EMAIL
	 "No"	// Necessita traduccio
#elif L==2
	 "Nein"
#elif L==3
	 "No"
#elif L==4
	 "No"
#elif L==5
	 "Non"
#elif L==6
	 "Nah&aacute;niri"
#elif L==7
	 "No"
#elif L==8
	 "No"
#elif L==9
	 "N&atilde;o"
#endif
	 ,
#if   L==1	// Ntf_STATUS_EMAIL_PENDING
	 "Pendiente"	// Necessita traduccio
#elif L==2
	 "Offen"
#elif L==3
	 "Pending"
#elif L==4
	 "Pendiente"
#elif L==5
	 "En attente"
#elif L==6
	 "Pendiente"	// Okoteve traduccin
#elif L==7
	 "In attesa"
#elif L==8
	 "Do czasu"
#elif L==9
	 "Pendente"
#endif
	 ,
#if   L==1	// Ntf_STATUS_EMAIL_CANCELLED
	 "Cancelado"	// Necessita traduccio
#elif L==2
	 "Abgebrochen"
#elif L==3
	 "Cancelled"
#elif L==4
	 "Cancelado"
#elif L==5
	 "Annul&eacute;"
#elif L==6
	 "Cancelado"	// Okoteve traduccin
#elif L==7
	 "Rimosso"
#elif L==8
	 "Odwolany"
#elif L==9
	 "Cancelado"
#endif
	 ,
#if   L==1	// Ntf_STATUS_EMAIL_SENT
	 "Enviado"	// Necessita traduccio
#elif L==2
	 "Gesendet"
#elif L==3
	 "Sent"
#elif L==4
	 "Enviado"
#elif L==5
	 "Envoy&eacute;e"
#elif L==6
	 "Enviado"	// Okoteve traduccin
#elif L==7
	 "Inviata"
#elif L==8
	 "Wyslane"
#elif L==9
	 "Enviada"
#endif
	};

const char *Txt_notification =
#if   L==1
	"notificaci&oacute;";
#elif L==2
	"Anmeldung";
#elif L==3
	"notification";
#elif L==4
	"notificaci&oacute;n";
#elif L==5
	"notification";
#elif L==6
	"momarandu";
#elif L==7
	"notifica";
#elif L==8
	"powiadomienie";
#elif L==9
	"notifica&ccedil;&atilde;o";
#endif

const char *Txt_Notifications =
#if   L==1
	"Notificacions";
#elif L==2
	"Anmeldungen";
#elif L==3
	"Notifications";
#elif L==4
	"Notificaciones";
#elif L==5
	"Notifications";
#elif L==6
	"Momarandu";
#elif L==7
	"Notifiche";
#elif L==8
	"Powiadomienia";
#elif L==9
	"Notifica&ccedil;&otilde;es";
#endif

const char *Txt_notifications =
#if   L==1
	"notificacions";
#elif L==2
	"Anmeldungen";
#elif L==3
	"notifications";
#elif L==4
	"notificaciones";
#elif L==5
	"notifications";
#elif L==6
	"momarandu";
#elif L==7
	"notifiche";
#elif L==8
	"powiadomienia";
#elif L==9
	"notifica&ccedil;&otilde;es";
#endif

const char *Txt_NOTIFY_EVENTS_PLURAL[Ntf_NUM_NOTIFY_EVENTS] =
   {
#if   L==1	// Ntf_EVENT_UNKNOWN
	 "Esdeveniments desconeguts"
#elif L==2
	 "Unbekannte Ereignisse"
#elif L==3
	 "Unknown events"
#elif L==4
	 "Eventos desconocidos"
#elif L==5
	 "&Eacute;v&eacute;nements inconnus"
#elif L==6
	 "Eventos desconocidos"	// Okoteve traduccin
#elif L==7
	 "Eventi sconosciuto"
#elif L==8
	 "Brak wydarzen"
#elif L==9
	 "Eventos desconhecidos"
#endif
	 ,
#if   L==1	// Ntf_EVENT_DOCUMENT_FILE
	 "Nous documents"
#elif L==2
	 "Neue Dokumentdateien"
#elif L==3
	 "New document files"
#elif L==4
	 "Nuevos documentos"
#elif L==5
	 "Nouveaux fichiers de documents"
#elif L==6
	 "Nuevos documentos"	// Okoteve traduccin
#elif L==7
	 "Nuovi documenti"
#elif L==8
	 "Nowy plik dokumentu"
#elif L==9
	 "Novos arquivos de documento"
#endif
	 ,
#if   L==1	// Ntf_EVENT_TEACHERS_FILE
	 "Nous arxiu de professors"
#elif L==2
	 "New Lehrer Dateien"
#elif L==3
	 "New teachers' files"
#elif L==4
	 "Nuevos archivos de profesores"
#elif L==5
	 "Nouveaux fichiers d'enseignants"
#elif L==6
	 "Nuevos archivos de profesores"	// Okoteve traduccin
#elif L==7
	 "Nuovi file di professori"
#elif L==8
	 "Nowy Nauczyciela plik"
#elif L==9
	 "Novos arquivos dos professores"
#endif
	 ,
#if   L==1	// Ntf_EVENT_SHARED_FILE
	 "Nous arxiu compartit"
#elif L==2
	 "New freigegebene Dateien"
#elif L==3
	 "New shared files"
#elif L==4
	 "Nuevos archivos compartidos"
#elif L==5
	 "Nouveaux fichiers partag&eacute;s"
#elif L==6
	 "Nuevos archivos compartidos"	// Okoteve traduccin
#elif L==7
	 "Nuovi file condiviso"
#elif L==8
	 "Nowy Udost&eogon;pniony plik"
#elif L==9
	 "Novos arquivos compartilhado"
#endif
	 ,
#if   L==1	// Ntf_EVENT_ASSIGNMENT
	 "Noves activitats"
#elif L==2
	 "Neue Aufgaben"
#elif L==3
	 "New assignments"
#elif L==4
	 "Nuevas actividades"
#elif L==5
	 "Nouvelles activit&eacute;s"
#elif L==6
	 "Nuevas actividades"	// Okoteve traduccin
#elif L==7
	 "Nuove attivit&agrave;"
#elif L==8
	 "Nowe zadania"
#elif L==9
	 "Novas atividades"
#endif
	 ,
#if   L==1	// Ntf_EVENT_EXAM_ANNOUNCEMENT
	 "Noves convocat&ograve;ries d'examen"
#elif L==2
	 "Neue Aufrufe f&uuml;r Pr&uuml;fung"
#elif L==3
	 "New announcements of exam"
#elif L==4
	 "Nuevas convocatorias de examen"
#elif L==5
	 "Nouvelles convocations &agrave; un examen"
#elif L==6
	 "Nuevas convocatorias de examen"	// Okoteve traduccin
#elif L==7
	 "Nuovi appelli d'esame"
#elif L==8
	 "Nowe anonse egzamin"
#elif L==9
	 "Novas chamadas para exame"
#endif
	 ,
#if   L==1	// Ntf_EVENT_MARKS_FILE
	 "Nous fitxers de qualificacions"
#elif L==2
	 "Neue Dateien mit Bewertung"
#elif L==3
	 "New files with marks"
#elif L==4
	 "Nuevos archivos de calificaciones"
#elif L==5
	 "Nouveaux fichiers avec des notes"
#elif L==6
	 "Nuevos archivos de calificaciones"	// Okoteve traduccin
#elif L==7
	 "Nuovi file con risultati"
#elif L==8
	 "Nowe pliki ze znakami"
#elif L==9
	 "Novos arquivos de notas"
#endif
	 ,
#if   L==1	// Ntf_EVENT_ENROLLMENT_STUDENT
	 "Novas inscripci&oacute;s com a estudiant"
#elif L==2
	 "New Einschreibungen als Student"
#elif L==3
	 "New enrollments as a student"
#elif L==4
	 "Nuevas inscripciones como estudiante"
#elif L==5
	 "Nouvelles inscriptions en tant qu'&eacute;tudiant"
#elif L==6
	 "Nuevas inscripciones como estudiante"	// Okoteve traduccin
#elif L==7
	 "Nuova iscrizione come studente"
#elif L==8
	 "Nowe zapisy jako student"
#elif L==9
	 "Novas inscri&ccedil;&otilde;es como estudante"
#endif
	 ,
#if   L==1	// Ntf_EVENT_ENROLLMENT_TEACHER
	 "Novas inscripci&oacute;s com a professor"
#elif L==2
	 "New Einschreibungen als Lehrkraft"
#elif L==3
	 "New enrollments as a teacher"
#elif L==4
	 "Nuevas inscripciones como profesor/a"
#elif L==5
	 "Nouvelles inscriptions en tant qu'enseignant"
#elif L==6
	 "Nuevas inscripciones como profesor/a"	// Okoteve traduccin
#elif L==7
	 "Nuova iscrizione come professore"
#elif L==8
	 "Nowe zapisy jako nauczyciel"
#elif L==9
	 "Novas inscri&ccedil;&otilde;es como professor"
#endif
	 ,
#if   L==1	// Ntf_EVENT_ENROLLMENT_REQUEST
	 "Novas petici&oacute;s d'inscripci&oacute;"
#elif L==2
	 "New Registrierungsanforderungen"
#elif L==3
	 "New enrollment requests"
#elif L==4
	 "Nuevas peticiones de inscripci&oacute;n"
#elif L==5
	 "Nouvelles demandes d'inscription"
#elif L==6
	 "Nuevas peticiones de inscripci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Nuova richiesta di iscrizione"
#elif L==8
	 "Nowe pro&sacute;by o rejestracji"
#elif L==9
	 "Novos pedidos de inscri&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// Ntf_EVENT_TIMELINE_COMMENT
	 "Nous comentaris als meus missatges socials"
#elif L==2
	 "Neue Kommentare zu meinem sozialen Beitr&auml;ge"
#elif L==3
	 "New comments to my social posts"
#elif L==4
	 "Nuevos comentarios a mis mensajes sociales"
#elif L==5
	 "Nouveaux commentaires &agrave; mes messages sociaux"
#elif L==6
	 "Nuevos comentarios a mis mensajes sociales"	// Okoteve traduccin
#elif L==7
	 "Nuovi commenti ai miei post sociali"
#elif L==8
	 "Nowe komentarze do moich post spo&lstrok;ecznej"
#elif L==9
	 "Novos coment&aacute;rio &agrave;s minhos post sociais"
#endif
	 ,
#if   L==1	// Ntf_EVENT_TIMELINE_FAV
	 "Nous favorits als meus missatges socials"
#elif L==2
	 "Neue Favorit zu meinem sozialen Beitr&auml;ge"
#elif L==3
	 "New favourite to my social posts"
#elif L==4
	 "Nuevos favoritos a mis mensajes sociales"
#elif L==5
	 "Nouveaux favori &agrave; mes messages sociaux"
#elif L==6
	 "Nuevos favoritos a mis mensajes sociales"	// Okoteve traduccin
#elif L==7
	 "Nuovi preferito ai miei post sociali"
#elif L==8
	 "Nowe ulubiony do moich post spo&lstrok;ecznej"
#elif L==9
	 "Novos favoritos &agrave;s minhos post sociais"
#endif
	 ,
#if   L==1	// Ntf_EVENT_TIMELINE_SHARE
	 "Noves comparticions dels meus missatges socials"
#elif L==2
	 "Neue gemeinsame Nutzung zu meinem sozialen Beitr&auml;ge"
#elif L==3
	 "New sharings of my social posts"
#elif L==4
	 "Nuevas comparticiones de mis mensajes sociales"
#elif L==5
	 "Nouveaux partages de mes messages sociaux"
#elif L==6
	 "Nuevas comparticiones de mis mensajes sociales"	// Okoteve traduccin
#elif L==7
	 "Nuove condivisioni dei miei post sociali"
#elif L==8
	 "Nowe podzial moich post spo&lstrok;ecznej"
#elif L==9
	 "Novas partilhas de minhos post sociais"
#endif
	 ,
#if   L==1	// Ntf_EVENT_TIMELINE_MENTION
	 "Noves mencions"
#elif L==2
	 "Neue Erw&auml;hnungen"
#elif L==3
	 "New mentions"
#elif L==4
	 "Nuevas menciones"
#elif L==5
	 "Nouvelles mentions"
#elif L==6
	 "Nuevas menciones"	// Okoteve traduccin
#elif L==7
	 "Nuovi menzioni"
#elif L==8
	 "Nowe wzmianki"
#elif L==9
	 "Novas men&ccedil;&otilde;es"
#endif
	 ,
#if   L==1	// Ntf_EVENT_FOLLOWER
	 "Nous seguidors"
#elif L==2
	 "Neue Anh&auml;nger"
#elif L==3
	 "New followers"
#elif L==4
	 "Nuevos seguidores"
#elif L==5
	 "Nouveaux suiveurs"
#elif L==6
	 "Nuevos seguidores"	// Okoteve traduccin
#elif L==7
	 "Nuovi followers"
#elif L==8
	 "Nowe obserwuj&aogon;"
#elif L==9
	 "Novos seguidores"
#endif
	 ,
#if   L==1	// Ntf_EVENT_FORUM_POST_COURSE
	 "Novos missatges en f&ograve;rums"
#elif L==2
	 "Neue Beitr&auml;ge in den Foren"
#elif L==3
	 "New posts in course forums"
#elif L==4
	 "Nuevos mensajes en foros de asignaturas"
#elif L==5
	 "Nouveaux messages dans les forums"
#elif L==6
	 "Nuevos mensajes en foros de asignaturas"	// Okoteve traduccin
#elif L==7
	 "Nuovi post nei forum"
#elif L==8
	 "Nowe post&oacute;w na forum"
#elif L==9
	 "Novos posts em f&oacute;runs"
#endif
	 ,
#if   L==1	// Ntf_EVENT_FORUM_REPLY
	 "Noves respostes als meus missatges en f&ograve;rums"
#elif L==2
	 "Neue Antworten auf meine Beitr&auml;ge in den Foren"
#elif L==3
	 "New replies to my posts in forums"
#elif L==4
	 "Nuevas respuestas a mis mensajes en foros"
#elif L==5
	 "Nouvelles r&eacute;ponses &agrave; mes messages dans les forums"
#elif L==6
	 "Nuevas respuestas a mis mensajes en foros"	// Okoteve traduccin
#elif L==7
	 "Nuove risposte ai miei post nei forum"
#elif L==8
	 "Nowe podejscie do moich post&oacute;w na forum"
#elif L==9
	 "Novas respostas aos meus posts em f&oacute;runs"
#endif
	 ,
#if   L==1	// Ntf_EVENT_NOTICE
	 "Nous avisos"
#elif L==2
	 "Neue Ank&uuml;ndigungen"
#elif L==3
	 "New notices"
#elif L==4
	 "Nuevos avisos"
#elif L==5
	 "Nouveaux avis"
#elif L==6
	 "Nuevos avisos"	// Okoteve traduccin
#elif L==7
	 "Nuovi avvisi"
#elif L==8
	 "Nowe ogloszenia"
#elif L==9
	 "Novos avisos"
#endif
	 ,
#if   L==1	// Ntf_EVENT_MESSAGE
	 "Nous missatges"
#elif L==2
	 "Neue Nachrichten"
#elif L==3
	 "New messages"
#elif L==4
	 "Nuevos mensajes"
#elif L==5
	 "Nouveaux messages"
#elif L==6
	 "Nuevos mensajes"	// Okoteve traduccin
#elif L==7
	 "Nuovi messaggi"
#elif L==8
	 "Nowa wiadomosc"
#elif L==9
	 "Novas mensagens"
#endif
	 ,
#if   L==1	// Ntf_EVENT_SURVEY
	 "Noves enquestes"
#elif L==2
	 "Neue Umfragen"
#elif L==3
	 "New surveys"
#elif L==4
	 "Nuevas encuestas"
#elif L==5
	 "Nouveaux sondages"
#elif L==6
	 "Nuevas encuestas"	// Okoteve traduccin
#elif L==7
	 "Nuove sondaggi"
#elif L==8
	 "Nowe badania"
#elif L==9
	 "Novos inqu&eacute;ritos"
#endif
	};

const char *Txt_NOTIFY_EVENTS_SINGULAR[Ntf_NUM_NOTIFY_EVENTS] =
   {
#if   L==1	// Ntf_EVENT_UNKNOWN
	 "Esdeveniment desconegut"
#elif L==2
	 "Unbekannt Ereignis"
#elif L==3
	 "Unknown event"
#elif L==4
	 "Evento desconocido"
#elif L==5
	 "&Eacute;v&eacute;nement inconnu"
#elif L==6
	 "Evento desconocido"	// Okoteve traduccin
#elif L==7
	 "Evento sconosciuto"
#elif L==8
	 "Nieznane zdarzenia"
#elif L==9
	 "Evento desconhecido"
#endif
	 ,
#if   L==1	// Ntf_EVENT_DOCUMENT_FILE
	 "Document"
#elif L==2
	 "Dokumentdatei"
#elif L==3
	 "Document file"
#elif L==4
	 "Documento"
#elif L==5
	 "Fichier de document"
#elif L==6
	 "Documento"	// Okoteve traduccin
#elif L==7
	 "Documento"
#elif L==8
	 "Plik dokumentu"
#elif L==9
	 "Arquivo de documento"
#endif
	 ,
#if   L==1	// Ntf_EVENT_TEACHERS_FILE
	 "Arxiu de professors"
#elif L==2
	 "Lehrer Datei"
#elif L==3
	 "Teachers' file"
#elif L==4
	 "Archivo de profesores"
#elif L==5
	 "Fichier d'enseignants"
#elif L==6
	 "Archivo de profesores"	// Okoteve traduccin
#elif L==7
	 "File di professori"
#elif L==8
	 "Nauczyciela plik"
#elif L==9
	 "Arquivo dos professores"
#endif
	 ,
#if   L==1	// Ntf_EVENT_SHARED_FILE
	 "Arxiu compartit"
#elif L==2
	 "Freigegebene Datei"
#elif L==3
	 "Shared file"
#elif L==4
	 "Archivo compartido"
#elif L==5
	 "Fichier partag&eacute;"
#elif L==6
	 "Archivo compartido"	// Okoteve traduccin
#elif L==7
	 "File condiviso"
#elif L==8
	 "Udost&eogon;pniony plik"
#elif L==9
	 "Arquivo compartilhado"
#endif
	 ,
#if   L==1	// Ntf_EVENT_ASSIGNMENT
	 "Activitat"
#elif L==2
	 "Aufgabe"
#elif L==3
	 "Assignment"
#elif L==4
	 "Actividad"
#elif L==5
	 "Activit&eacute;"
#elif L==6
	 "Tar&eacute;a"
#elif L==7
	 "Attivit&agrave;"
#elif L==8
	 "Przyporzadkowanie"
#elif L==9
	 "Atividade"
#endif
	 ,
#if   L==1	// Ntf_EVENT_EXAM_ANNOUNCEMENT
	 "Convocat&ograve;ria d'examen"
#elif L==2
	 "Aufrufe fr Pr&uuml;fung"
#elif L==3
	 "Announcement of exam"
#elif L==4
	 "Convocatoria de examen"
#elif L==5
	 "Convocation &agrave; un examen"
#elif L==6
	 "Convocatoria de examen"	// Okoteve traduccin
#elif L==7
	 "Appello d'esame"
#elif L==8
	 "Ogloszenie egzamin"
#elif L==9
	 "Chamada para exame"
#endif
	 ,
#if   L==1	// Ntf_EVENT_MARKS_FILE
	 "Fitxer de qualificacions"
#elif L==2
	 "Datei mit Bewertung"
#elif L==3
	 "File with marks"
#elif L==4
	 "Archivo de calificaciones"
#elif L==5
	 "Fichier avec des notes"
#elif L==6
	 "Archivo de calificaciones"	// Okoteve traduccin
#elif L==7
	 "File con risultati"
#elif L==8
	 "Plik ze znakami"
#elif L==9
	 "Arquivo de notas"
#endif
	 ,
#if   L==1	// Ntf_EVENT_ENROLLMENT_STUDENT
	 "Inscripci&oacute; com a estudiant"
#elif L==2
	 "Einschreibung als Student"
#elif L==3
	 "Enrollment as a student"
#elif L==4
	 "Inscripci&oacute;n como estudiante"
#elif L==5
	 "Inscription en tant qu'&eacute;tudiant"
#elif L==6
	 "Inscripci&oacute;n como estudiante"	// Okoteve traduccin
#elif L==7
	 "Iscrizione come studente"
#elif L==8
	 "Zapisy jako student"
#elif L==9
	 "Inscri&ccedil;&atilde;o como estudante"
#endif
	 ,
#if   L==1	// Ntf_EVENT_ENROLLMENT_TEACHER
	 "Inscripci&oacute; com a professor"
#elif L==2
	 "Einschreibung als Lehrkraft"
#elif L==3
	 "Enrollment as a teacher"
#elif L==4
	 "Inscripci&oacute;n como profesor/a"
#elif L==5
	 "Inscription en tant qu'enseignant"
#elif L==6
	 "Inscripci&oacute;n como profesor/a"	// Okoteve traduccin
#elif L==7
	 "Iscrizione come professore"
#elif L==8
	 "Zapisy jako nauczyciel"
#elif L==9
	 "Inscri&ccedil;&atilde;o como professor"
#endif
	 ,
#if   L==1	// Ntf_EVENT_ENROLLMENT_REQUEST
	 "Petici&oacute; d'inscripci&oacute;"
#elif L==2
	 "Registrierungsanforderung"
#elif L==3
	 "Enrollment request"
#elif L==4
	 "Petici&oacute;n de inscripci&oacute;n"
#elif L==5
	 "Demande d'inscription"
#elif L==6
	 "Petici&oacute;n de inscripci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Richiesta di iscrizione"
#elif L==8
	 "&Zdot;&aogon;danie rejestracji"
#elif L==9
	 "Pedido de inscri&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// Ntf_EVENT_TIMELINE_COMMENT
	 "Comentari a un missatge social"
#elif L==2
	 "Kommentar zu soziale Beitrag"
#elif L==3
	 "Comment to social post"
#elif L==4
	 "Comentario a un mensaje social"
#elif L==5
	 "Commentaire &agrave; message social"
#elif L==6
	 "Comentario a un mensaje social"	// Okoteve traduccin
#elif L==7
	 "Commento a un post sociale"
#elif L==8
	 "Komentarz do post spo&lstrok;eczna"
#elif L==9
	 "Coment&aacute;rio a um post sociais"
#endif
	 ,
#if   L==1	// Ntf_EVENT_TIMELINE_FAV
	 "Missatge social marcat com favorit"
#elif L==2
	 "Social Beitrag als Favorit markiert"
#elif L==3
	 "Social post marked as favourite"
#elif L==4
	 "Mensaje social marcado como favorito"
#elif L==5
	 "Message social marqu&eacute;e comme favori"
#elif L==6
	 "Mensaje social marcado como favorito"	// Okoteve traduccin
#elif L==7
	 "Post sociale segnato come preferito"
#elif L==8
	 "Post spo&lstrok;ecznej oznaczone jako ulubiony"
#elif L==9
	 "Post sociais marcado como favorito"
#endif
	 ,
#if   L==1	// Ntf_EVENT_TIMELINE_SHARE
	 "Missatge social compartit"
#elif L==2
	 "Social Beitrag gemeinsamer"
#elif L==3
	 "Social post shared"
#elif L==4
	 "Mensaje social compartido"
#elif L==5
	 "Message social partag&eacute;"
#elif L==6
	 "Mensaje social compartido"	// Okoteve traduccin
#elif L==7
	 "Post sociale condiviso"
#elif L==8
	 "Post spo&lstrok;ecznej wsp&oacute;lne"
#elif L==9
	 "Post sociais compartido"
#endif
	 ,
#if   L==1	// Ntf_EVENT_TIMELINE_MENTION
	 "Menci&oacute;"
#elif L==2
	 "Erw&auml;hnung"
#elif L==3
	 "Mention"
#elif L==4
	 "Menci&oacute;n"
#elif L==5
	 "Mention"
#elif L==6
	 "Menci&oacute;n"			// Okoteve traduccin
#elif L==7
	 "Menzione"
#elif L==8
	 "Wzmianka"
#elif L==9
	 "Men&ccedil;&atilde;o"
#endif
	 ,
#if   L==1	// Ntf_EVENT_FOLLOWER
	"Seguidor"
#elif L==2
	"Anh&auml;nger"
#elif L==3
	"Follower"
#elif L==4
	"Seguidor"
#elif L==5
	"Suiveur"
#elif L==6
	"Seguidor"	// Okoteve traduccin
#elif L==7
	"Follower"
#elif L==8
	"Obserwuj&aogon;"
#elif L==9
	"Seguidor"
#endif
	 ,
#if   L==1	// Ntf_EVENT_FORUM_POST_COURSE
	 "Missatge en un f&ograve;rum"
#elif L==2
	 "Beitr&auml;ge in einem Forum"
#elif L==3
	 "Post in a forum"
#elif L==4
	 "Mensaje en un foro"
#elif L==5
	 "Message dans un forum"
#elif L==6
	 "Mensaje en un foro"	// Okoteve traduccin
#elif L==7
	 "Post in un forum"
#elif L==8
	 "Post na forum"
#elif L==9
	 "Post em um f&oacute;rum"
#endif
	 ,
#if   L==1	// Ntf_EVENT_FORUM_REPLY
	 "Resposta en un f&ograve;rum"
#elif L==2
	 "Antwort in einem Forum"
#elif L==3
	 "Response in a forum"
#elif L==4
	 "Respuesta en un foro"
#elif L==5
	 "R&eacute;ponse dans un forum"
#elif L==6
	 "Respuesta en un foro"	// Okoteve traduccin
#elif L==7
	 "Risposta in un forum"
#elif L==8
	 "Odpowiedz na forum"
#elif L==9
	 "Resposta em um f&oacute;rum"
#endif
	 ,
#if   L==1	// Ntf_EVENT_NOTICE
	 "Av&iacute;s"
#elif L==2
	 "Ank&uuml;ndigungen"
#elif L==3
	 "Notice"
#elif L==4
	 "Aviso"
#elif L==5
	 "Avis"
#elif L==6
	 "Marandu"
#elif L==7
	 "Avviso"
#elif L==8
	 "Informacja"
#elif L==9
	 "An&uacute;ncio"
#endif
	 ,
#if   L==1	// Ntf_EVENT_MESSAGE
	 "Missatge"
#elif L==2
	 "Nachricht"
#elif L==3
	 "Message"
#elif L==4
	 "Mensaje"
#elif L==5
	 "Message"
#elif L==6
	 "Marandu"
#elif L==7
	 "Messaggio"
#elif L==8
	 "Wiadomo&sacute;&cacute;"
#elif L==9
	 "Mensagem"
#endif
	 ,
#if   L==1	// Ntf_EVENT_SURVEY
	 "Enquesta"
#elif L==2
	 "Umfrage"
#elif L==3
	 "Survey"
#elif L==4
	 "Encuesta"
#elif L==5
	 "Sondage"
#elif L==6
	 "Encuesta"	// Okoteve traduccin
#elif L==7
	 "Sondaggio"
#elif L==8
	 "Badania"
#elif L==9
	 "Inqu&eacute;rito"
#endif
	};

const char *Txt_Notify_me_BR_by_email =
#if   L==1
	"Avisar-me<br />per correu";
#elif L==2
	"Benachrichtigt mich<br />per E-Mail";
#elif L==3
	"Notify me<br />by email";
#elif L==4
	"Avisarme<br />por correo";
#elif L==5
	"Avertissez-moi<br />par email";
#elif L==6
	"Avisarme<br />por correo";		// Okoteve traduccin
#elif L==7
	"Notificatami<br />via email";
#elif L==8
	"Informuj<br />mnie emailem";
#elif L==9
	"Notifique-me<br />por email";
#endif

const char *Txt_Now_each_student_can_belong_to_multiple_groups_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Ahora cada estudiante puede pertenecer a varios grupos de tipo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";		//
#elif L==3
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";
#elif L==4
	"Ahora cada estudiante puede pertenecer a varios grupos de tipo <strong>%s</strong>.";
#elif L==5
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";		//
#elif L==6
	"Ahora cada estudiante puede pertenecer a varios grupos de tipo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Ora ogni studente pu&ograve; appartenere a gruppi multipli di tipo <strong>%s</strong>.";
#elif L==8
	"Teraz kazdy student moze nalezec do wielu grup typu <strong>%s</strong>.";
#elif L==9
	"Now each student can belong to multiple groups of type <strong>%s</strong>.";		//
#endif

const char *Txt_Now_each_student_can_only_belong_to_a_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Ahora cada estudiante solo puede pertenecer a un grupo de tipo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Now each student can only belong to a group of type <strong>%s</strong>.";
#elif L==3
	"Now each student can only belong to a group of type <strong>%s</strong>.";
#elif L==4
	"Ahora cada estudiante solo puede pertenecer a un grupo de tipo <strong>%s</strong>.";
#elif L==5
	"Now each student can only belong to a group of type <strong>%s</strong>.";		//
#elif L==6
	"Ahora cada estudiante solo puede pertenecer a un grupo de tipo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Ora ogni studente pu&ograve; solo appartenere a un gruppo di tipo <strong>%s</strong>.";
#elif L==8
	"Teraz kazdy student moze nalezec tylko do grupy typu <strong>%s</strong>.";
#elif L==9
	"Now each student can only belong to a group of type <strong>%s</strong>.";		//
#endif

const char *Txt_Number_of_BR_assignments =
#if   L==1
	"Nombre<br />d'activitats";
#elif L==2
	"Anzahl der<br />Aufgaben";
#elif L==3
	"Number of<br />assignments";
#elif L==4
	"N&ordm; de<br />actividades";
#elif L==5
	"Nombre<br />d'activit&eacute;s";
#elif L==6
	"N&ordm; de<br />actividades";	// Okoteve traduccin
#elif L==7
	"Numero di<br />attivit&agrave;";
#elif L==8
	"Number of<br />assignments";	// Potrzebujesz tlumaczenie
#elif L==9
	"N&ordm; de<br />atividades";
#endif

const char *Txt_Number_of_BR_courses_with_BR_assignments =
#if   L==1
	"Nombre<br />d'assignatures<br />amb activitats";
#elif L==2
	"Anzahl der<br />Kursen<br />mit Aufgaben";
#elif L==3
	"Number of<br />courses with<br />assignments";
#elif L==4
	"N&ordm; de<br />asignaturas<br />con actividades";
#elif L==5
	"Nombre de<br />mati&egrave;res<br />avec activit&eacute;s";
#elif L==6
	"N&ordm; de<br />asignaturas<br />con actividades";	// Okoteve traduccin
#elif L==7
	"Numero di<br />corsi con<br />attivit&agrave;";
#elif L==8
	"Liczba<br />kurs&oacute;w z<br />zadania";
#elif L==9
	"N&ordm; de<br />disciplinas<br />com atividades";
#endif

const char *Txt_Number_of_BR_courses_with_BR_exportable_BR_test_BR_questions =
#if   L==1
	"Nombre<br />d'assignatures<br />amb preguntes<br />de test<br />exportables";
#elif L==2
	"Anzahl der<br />Kursen<br />mit exportierbar<br />Testfragen";
#elif L==3
	"Number of<br />courses with<br />exportable<br />test<br />questions";
#elif L==4
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test<br />exportables";
#elif L==5
	"Nombre de<br />mati&egrave;res<br />avec questions<br />de test<br />exportables";
#elif L==6
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test<br />exportables";	// Okoteve traduccin
#elif L==7
	"Numero di<br />corsi con<br />domande<br />de test<br />esportabili";
#elif L==8
	"Liczba<br />kurs&oacute;w<br />z eksportowane<br />pyta&nacute;<br />testowych";
#elif L==9
	"N&ordm; de<br />disciplinas<br />com quest&otilde;es<br />de test<br />export&aacute;veis";
#endif

const char *Txt_Number_of_BR_courses_BR_with_test_BR_questions =
#if   L==1
	"Nombre<br />d'assignatures<br />amb preguntes<br />de test";
#elif L==2
	"Anzahl der<br />Kursen<br />mit Testfragen";
#elif L==3
	"Number of<br />courses<br />with test<br />questions";
#elif L==4
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test";
#elif L==5
	"Nombre de<br />mati&egrave;res<br />avec questions<br />de test";
#elif L==6
	"N&ordm; de<br />asignaturas<br />con preguntas<br />de test";	// Okoteve traduccin
#elif L==7
	"Numero di<br />corsi con<br />domande<br />de test";
#elif L==8
	"Liczba<br />kurs&oacute;w<br />z pyta&nacute;<br />testowych";
#elif L==9
	"N&ordm; de<br />disciplinas<br />com quest&otilde;es<br />de test";
#endif

const char *Txt_Number_of_BR_courses_with_BR_surveys =
#if   L==1
	"Nombre<br />d'assignatures<br />amb enquestes";
#elif L==2
	"Anzahl der<br />Kursen<br />mit Umfragen";
#elif L==3
	"Number of<br />courses with<br />surveys";
#elif L==4
	"N&ordm; de<br />asignaturas<br />con encuestas";
#elif L==5
	"Nombre de<br />mati&egrave;res<br />avec sondages";
#elif L==6
	"N&ordm; de<br />asignaturas<br />con encuestas";	// Okoteve traduccin
#elif L==7
	"Numero di<br />corsi con<br />sondaggi";
#elif L==8
	"Liczba<br />kurs&oacute;w z<br />badania";
#elif L==9
	"N&ordm; de<br />disciplinas<br />com inqu&eacute;ritos";
#endif

const char *Txt_Number_of_BR_events =
#if   L==1
	"N&ordm; de<br />eventos";	// Necessita traduccio
#elif L==2
	"Anzahl der<br />Ereignisse";
#elif L==3
	"Number of<br />events";
#elif L==4
	"N&ordm; de<br />eventos";
#elif L==5
	"Nombre<br />d'&eacute;v&eacute;nements";
#elif L==6
	"N&ordm; de<br />eventos";	// Okoteve traduccin
#elif L==7
	"Numero di<br />eventi";
#elif L==8
	"Liczba of<br />zdarzenia";
#elif L==9
	"N&uacute;mero de<br />eventos";
#endif

const char *Txt_Number_of_BR_emails =
#if   L==1
	"N&ordm; de<br />correos";	// Necessita traduccio
#elif L==2
	"Anzahl der<br />E-Mails";
#elif L==3
	"Number of<br />emails";
#elif L==4
	"N&ordm; de<br />correos";
#elif L==5
	"Nombre<br />d'emails";
#elif L==6
	"N&ordm; de<br />correos";	// Okoteve traduccin
#elif L==7
	"Numero di<br />email";
#elif L==8
	"Liczba<br />email";
#elif L==9
	"N&uacute;mero de<br />emails";
#endif

const char *Txt_Number_of_BR_notifications =
#if   L==1
	"Nombre de<br />notificacions";
#elif L==2
	"Anzahl der<br />Anmeldungen";
#elif L==3
	"Number of<br />notifications";
#elif L==4
	"N&ordm; de<br />notificaciones";
#elif L==5
	"Nombre de<br />notifications";
#elif L==6
	"N&ordm; de<br />notificaciones";	// Okoteve traduccin
#elif L==7
	"Numero di<br />notifiche";
#elif L==8
	"Liczba<br />zgloszen";
#elif L==9
	"N&uacute;mero de<br />notifica&ccedil;&otilde;es";
#endif

const char *Txt_Number_of_students =
#if   L==1
	"Nombre d'estudiants";
#elif L==2
	"Anzahl der Studenten";
#elif L==3
	"Number of students";
#elif L==4
	"N&ordm; de estudiantes";
#elif L==5
	"Nombre d'&eacute;tudiants";
#elif L==6
	"N&ordm; de estudiantes";	// Okoteve traduccin
#elif L==7
	"Numero di studenti";
#elif L==8
	"Liczba studenci";
#elif L==9
	"N&ordm; de estudantes";
#endif

const char *Txt_Number_of_BR_surveys =
#if   L==1
	"Nombre<br />d'enquestes";
#elif L==2
	"Anzahl der<br />Umfragen";
#elif L==3
	"Number of<br />surveys";
#elif L==4
	"N&ordm; de<br />encuestas";
#elif L==5
	"Nombre de<br />sondages";
#elif L==6
	"N&ordm; de<br />encuestas";	// Okoteve traduccin
#elif L==7
	"Numero di<br />sondaggi";
#elif L==8
	"Liczba<br />badania";
#elif L==9
	"N&ordm; de<br />inqu&eacute;ritos";
#endif

const char *Txt_Number_BR_of_test_BR_questions =
#if   L==1
	"Nombre de<br />preguntes<br />de test";
#elif L==2
	"Anzahl der<br />Testfragen";
#elif L==3
	"Number<br />of test<br />questions";
#elif L==4
	"N&ordm; de<br />preguntas<br />de test";
#elif L==5
	"Nombre de<br />questions<br />de test";
#elif L==6
	"N&ordm; de<br />preguntas<br />de test";	// Okoteve traduccin
#elif L==7
	"Numero di<br />domande<br />de test";
#elif L==8
	"Liczba<br />pyta&nacute;<br />testowych";
#elif L==9
	"N&ordm; de<br />quest&otilde;es<br />de test";
#endif

const char *Txt_Numeric_BR_code_BR_ISO_3166_1 =
#if   L==1
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";	// Necessita traduccio
#elif L==2
	"Numerischen<br />Code<br />ISO 3166-1";
#elif L==3
	"Numeric<br />code<br />ISO 3166-1";
#elif L==4
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";
#elif L==5
	"Code<br />num&eacute;rique<br />ISO 3166-1";
#elif L==6
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";	// Okoteve traduccin
#elif L==7
	"Codice<br />numerico<br />ISO 3166-1";
#elif L==8
	"Nnumerycznych<br />kod<br />ISO 3166-1";
#elif L==9
	"C&oacute;d.<br />num&eacute;rico<br />ISO 3166-1";
#endif

const char *Txt_of_PART_OF_A_TOTAL =	// Example: transferred 2 of 3 files
#if   L==1
	"de";	// Necessita traduccio
#elif L==2
	"von";
#elif L==3
	"of";
#elif L==4
	"de";
#elif L==5
	"de";
#elif L==6
	"de";	// Okoteve traduccin
#elif L==7
	"di";
#elif L==8
	"z";
#elif L==9
	"de";
#endif

const char *Txt_Office =
#if   L==1
	"Despatx";
#elif L==2
	"Raum";
#elif L==3
	"Office";
#elif L==4
	"Despacho";
#elif L==5
	"Bureau";
#elif L==6
	"Mba'apoha";
#elif L==7
	"Ufficio";
#elif L==8
	"Biuro";
#elif L==9
	"Gabinete";
#endif

const char *Txt_One_message_has_been_deleted =
#if   L==1
	"Se ha eliminado un mensaje.";			// Necessita traduccio
#elif L==2
	"One message has been deleted.";		// Need bersetzung
#elif L==3
	"One message has been deleted.";
#elif L==4
	"Se ha eliminado un mensaje.";
#elif L==5
	"One message has been deleted.";		// Besoin de traduction
#elif L==6
	"Se ha eliminado un mensaje.";			// Okoteve traduccin
#elif L==7
	"Un messaggio &egrave; stato rimosso.";
#elif L==8
	"Jedna wiadomosc zostala usunieta.";
#elif L==9
	"One message has been deleted.";		// Necessita de traduo
#endif

const char *Txt_One_user_has_been_eliminated =
#if   L==1
	"Se ha eliminado un usuario.";			// Necessita traduccio
#elif L==2
	"One user has been eliminated.";		// Need bersetzung
#elif L==3
	"One user has been eliminated.";
#elif L==4
	"Se ha eliminado un usuario.";
#elif L==5
	"One user has been eliminated.";		// Besoin de traduction
#elif L==6
	"Se ha eliminado un usuario.";			// Okoteve traduccin
#elif L==7
	"Un utente &egrave; stato rimosso.";
#elif L==8
	"Jeden uzytkownik zostal usuniety.";
#elif L==9
	"One user has been eliminated.";		// Necessita de traduo
#endif

const char *Txt_One_user_has_been_enrolled =
#if   L==1
	"Se ha inscrito un usuario.";		// Necessita traduccio
#elif L==2
	"One user has been enrolled.";		// Need bersetzung
#elif L==3
	"One user has been enrolled.";
#elif L==4
	"Se ha inscrito un usuario.";
#elif L==5
	"One user has been enrolled.";		// Besoin de traduction
#elif L==6
	"Se ha inscrito un usuario.";		// Okoteve traduccin
#elif L==7
	"Un utente &egrave; stato registrato.";
#elif L==8
	"Jeden uzytkownik zostal zarejestrowany.";
#elif L==9
	"One user has been enrolled.";		// Necessita de traduo
#endif

const char *Txt_One_user_has_been_removed =
#if   L==1
	"Se ha eliminado un usuario.";		// Necessita traduccio
#elif L==2
	"One user has been removed.";		// Need bersetzung
#elif L==3
	"One user has been removed.";
#elif L==4
	"Se ha eliminado un usuario.";
#elif L==5
	"One user has been removed.";		// Besoin de traduction
#elif L==6
	"Se ha eliminado un usuario.";		// Okoteve traduccin
#elif L==7
	"Un utente &egrave; stato rimosso.";
#elif L==8
	"Jeden z uzytkownik&oacute;w zostaly usuniete.";
#elif L==9
	"One user has been removed.";		// Necessita de traduo
#endif

const char *Txt_Online_tutoring =
#if   L==1
	"Tutor&iacute;as virtuales";	// Necessita traduccio
#elif L==2
	"Virtual Anleitungen";
#elif L==3
	"Online tutoring";
#elif L==4
	"Tutor&iacute;as virtuales";
#elif L==5
	"Tutorat en ligne";
#elif L==6
	"Tutor&iacute;as virtuales";	// Okoteve traduccin
#elif L==7
	"Tutoraggio on-line";
#elif L==8
	"Korepetycje online";
#elif L==9
	"Tutoria on-line";
#endif

const char *Txt_Only_courses_with_more_than_X_clicks_are_shown =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Nom&eacute;s es mostren les assignatures amb m&eacute;s de %u clics.";
#elif L==2
	"Nur Kurse mit mehr als %u Klicks angezeigt.";
#elif L==3
	"Only courses with more than %u clicks are shown.";
#elif L==4
	"Solo se muestran las asignaturas con m&aacute;s de %u clics.";
#elif L==5
	"Seuls les cours avec plus de %u clics sont pr&eacute;sent&eacute;s.";
#elif L==6
	"Solo se muestran las asignaturas con m&aacute;s de %u clics.";	// Okoteve traduccin
#elif L==7
	"Solo i corsi con pi&ugrave; di %u scatti sono mostrati.";
#elif L==8
	"Tylko kursy z ponad 100 klikni&eogon;&cacute; s&aogon; pokazane.";
#elif L==9
	"Somente disciplinas com mais de %u cliques s&atilde;o mostradas.";
#endif

const char *Txt_only_if_the_scope_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"nom&eacute;s si l'&agrave;mbit &eacute;s %s";
#elif L==2
	"nur dann, wenn der Geltungsbereich ist %s";
#elif L==3
	"only if the scope is %s";
#elif L==4
	"solo si el &aacute;mbito es %s";
#elif L==5
	"seulement si le port&eacute;e est %s";
#elif L==6
	"solo si el &aacute;mbito es %s";	// Okoteve traduccin
#elif L==7
	"solo se l'ambito &egrave; %s";
#elif L==8
	"tylko wtedy, gdy zakres jest %s";
#elif L==9
	"somente se o &acirc;mbito &eacute; %s";
#endif

const char *Txt_only_students =
#if   L==1
	", solo estudiantes";	// Necessita traduccio
#elif L==2
	", nur Studenten";
#elif L==3
	", only students";
#elif L==4
	", solo estudiantes";
#elif L==5
	", seulement &eacute;tudiants";
#elif L==6
	", solo estudiantes";	// Okoteve traduccin
#elif L==7
	", solo studenti";
#elif L==8
	", tylko studenci";
#elif L==9
	", apenas estudantes";
#endif

const char *Txt_only_teachers =
#if   L==1
	", professors";
#elif L==2
	", Lehrkr&auml;fte";
#elif L==3
	", teachers";
#elif L==4
	", profesores";
#elif L==5
	", enseignants";
#elif L==6
	", profesores";	// Okoteve traduccin
#elif L==7
	", professori";
#elif L==8
	", nauczycieli";
#elif L==9
	", professores";
#endif

const char *Txt_only_unread_messages =
#if   L==1
	"nom&eacute;s els missatges no llegits";
#elif L==2
	"nur ungelesene Nachrichten";
#elif L==3
	"only unread messages";
#elif L==4
	"&uacute;nicamente los mensajes no le&iacute;dos";
#elif L==5
	"uniquement les messages non lus";
#elif L==6
	"&uacute;nicamente los mensajes no le&iacute;dos";	// Okoteve traduccin
#elif L==7
	"solo i messaggi non letti";
#elif L==8
	"only unread messages";					// Potrzebujesz tlumaczenie
#elif L==9
	"apenas mensagens n&atilde;o lidas";
#endif

const char *Txt_open_document =
#if   L==1
	"document obert";
#elif L==2
	"ge&ouml;ffnet Dokumente";
#elif L==3
	"open document";
#elif L==4
	"documento abierto";
#elif L==5
	"document ouvert";
#elif L==6
	"documento abierto";	// Okoteve traduccin
#elif L==7
	"documento aperto";
#elif L==8
	"otwarty dokument";
#elif L==9
	"documento aberto";
#endif

const char *Txt_open_documents =
#if   L==1
	"documents oberts";
#elif L==2
	"ge&ouml;ffnete Dokumente";
#elif L==3
	"open documents";
#elif L==4
	"documentos abiertos";
#elif L==5
	"documents ouverts";
#elif L==6
	"documentos abiertos";	// Okoteve traduccin
#elif L==7
	"documenti aperti";
#elif L==8
	"otwarte dokumenty";
#elif L==9
	"documentos abertos";
#endif

const char *Txt_Open_survey =
#if   L==1
	"Enquesta oberta";
#elif L==2
	"Ge&ouml;ffneten Umfrage";
#elif L==3
	"Open survey";
#elif L==4
	"Encuesta abierta";
#elif L==5
	"Sondage ouvert";
#elif L==6
	"Encuesta abierta";	// Okoteve traduccin
#elif L==7
	"Sondaggio aperto";
#elif L==8
	"otwarte badania";
#elif L==9
	"Inqu&eacute;rito aberta";
#endif

const char *Txt_Opening_of_groups =
#if   L==1
	"Obertura de grups";
#elif L==2
	"Opening of groups";		// Need bersetzung
#elif L==3
	"Opening of groups";
#elif L==4
	"Apertura de grupos";
#elif L==5
	"Opening of groups";		// Besoin de traduction
#elif L==6
	"Apertura de grupos";		// Okoteve traduccin
#elif L==7
	"Apertura di gruppi";
#elif L==8
	"Opening of groups";		// Potrzebujesz tlumaczenie
#elif L==9
	"Opening of groups";		// Necessita de traduo
#endif

const char *Txt_Option_under_development =
#if   L==1
	"Opci&oacute; en desenvolupament.";
#elif L==2
	"Option in der Entwicklung.";
#elif L==3
	"Option under development.";
#elif L==4
	"Opci&oacute;n en desarrollo.";
#elif L==5
	"Option en cours de d&eacute;veloppement.";
#elif L==6
	"Opci&oacute;n en desarrollo.";			// Okoteve traduccin
#elif L==7
	"Opzione in fase di sviluppo.";
#elif L==8
	"Opcja w fazie rozwoju.";
#elif L==9
	"Op&ccedil;&atilde;o em desenvolvimento.";
#endif

const char *Txt_optional =
#if   L==1
	"opcional";
#elif L==2
	"wahlweise";
#elif L==3
	"optional";
#elif L==4
	"opcional";
#elif L==5
	"en option";
#elif L==6
	"opcional";			// Okoteve traduccin
#elif L==7
	"opzionale";
#elif L==8
	"do wyboru";
#elif L==9
	"opcional";
#endif

const char *Txt_or_you_can_create_a_new_link_inside_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&hellip;o pot crear un novo enlla&ccedil;"
	" dins de la carpeta <strong>%s</strong>:";
#elif L==2
	"&hellip;Oder Sie k&ouml;nnen ein neues Link"
	" im Verzeichnis <strong>%s</strong> anlegen:";
#elif L==3
	"&hellip;or you can create a new link"
	" inside the folder <strong>%s</strong>:";
#elif L==4
	"&hellip;o puede crear un nuevo enlace"
	" dentro de la carpeta <strong>%s</strong>:";
#elif L==5
	"&hellip;ou vous pouvez cr&eacute;er un nouveau lien"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6
	"&hellip;o puede crear un nuevo enlace"
	" dentro de la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7
	"&hellip;oppure &egrave; possibile creare una nuova cartella"
	" all'interno della cartella <strong>%s</strong>:";
#elif L==8
	"&hellip;czy mo&zdot;esz utworzy&cacute; nowy link"
	" wewn&aogon;trz folderu <strong>%s</strong>:";
#elif L==9
	"&hellip;ou voc&ecirc; pode criar uma nova liga&ccedil;&atilde;o"
	" dentro do diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_or_you_can_make_a_file_copy_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&hellip;o pot realitzar una c&ograve;pia d'arxius"
	" a la carpeta <strong>%s</strong>:";
#elif L==2
	"&hellip;Oder Sie k&ouml;nnen die Kopie einer Datei"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==3
	"&hellip;or you can make a file copy"
	" to the folder <strong>%s</strong>:";
#elif L==4
	"&hellip;o puede realizar una copia de archivos"
	" a la carpeta <strong>%s</strong>:";
#elif L==5
	"&hellip;ou vous pouvez faire une copie de fichiers"
	" vers le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6
	"&hellip;o puede realizar una copia de archivos"
	" a la carpeta <strong>%s</strong>:";		// Okoteve traduccin
#elif L==7
	"&hellip;oppure &egrave; possibile effettuare una copia dei file"
	" nella cartella <strong>%s</strong>:";
#elif L==8
	"&hellip;czy mozna zrobic kopie pliku"
	" do folderu <strong>%s</strong>:";
#elif L==9
	"&hellip;ou voc&ecirc; pode fazer uma c&oacute;pia de arquivo"
	" na diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_or_you_can_upload_a_new_file_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&hellip;o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";	// Necessita traduccio
#elif L==2
	"&hellip;oder senden Sie eine neue Datei"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==3
	"&hellip;or you can upload a new file"
	" to the folder <strong>%s</strong>:";
#elif L==4
	"&hellip;o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";
#elif L==5
	"&hellip;ou vous pouvez envoyer un nouveau fichier"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6
	"&hellip;o puede subir un nuevo archivo"
	" a la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7
	"&hellip;oppure si pu&ograve; inviare un nuovo file"
	" nella cartella <strong>%s</strong>:";
#elif L==8
	"&hellip;lub wyslac nowy plik"
	" do folderu <strong>%s</strong>:";
#elif L==9
	"&hellip;ou voc&ecirc; pode enviar um novo arquivo"
	" para o diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_or_you_can_upload_new_files_to_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"&hellip;o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";	// Necessita traduccio
#elif L==2
	"&hellip;oder senden Sie neue Dateien"
	" in das Verzeichnis <strong>%s</strong>:";
#elif L==3
	"&hellip;or you can upload new files"
	" to the folder <strong>%s</strong>:";
#elif L==4
	"&hellip;o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";
#elif L==5
	"&hellip;ou vous pouvez envoyer des nouveaux fichiers"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6
	"&hellip;o puede subir nuevos archivos"
	" a la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7
	"&hellip;oppure si pu&ograve; inviare nuovi file"
	" nella cartella <strong>%s</strong>:";
#elif L==8
	"&hellip;lub wyslac nowy plik"
	" do folderu <strong>%s</strong>:";
#elif L==9
	"&hellip;ou voc&ecirc; pode enviar novos arquivos"
	" para o diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_Original_message =
#if   L==1
	"Mensaje original";	// Necessita traduccio
#elif L==2
	"Original-Nachricht";
#elif L==3
	"Original message";
#elif L==4
	"Mensaje original";
#elif L==5
	"Message d'origine";
#elif L==6
	"Mensaje original";	// Okoteve traduccin
#elif L==7
	"Messaggio originale";
#elif L==8
	"Oryginalna wiadomo&sacute;&cacute;";
#elif L==9
	"Mensagem original";
#endif

const char *Txt_Other_actions =
#if   L==1
	"Altres accions";
#elif L==2
	"Andere Handlungen";
#elif L==3
	"Other actions";
#elif L==4
	"Otras acciones";
#elif L==5
	"Autres actions";
#elif L==6
	"Otras acciones";	// Okoteve traduccin
#elif L==7
	"Altri azioni";
#elif L==8
	"Inne dzia&lstrok;ania";
#elif L==9
	"Outras a&ccedil;&otilde;es";
#endif

const char *Txt_Other_countries =
#if   L==1
	"Altres pa&iuml;sos";
#elif L==2
	"Andere L&auml;nder";
#elif L==3
	"Other countries";
#elif L==4
	"Otros pa&iacute;ses";
#elif L==5
	"Autres pays";
#elif L==6
	"Otros pa&iacute;ses";	// Okoteve traduccin
#elif L==7
	"Altri paesi";
#elif L==8
	"Inne kraje";
#elif L==9
	"Outros pa&iacute;ses";
#endif

const char *Txt_Other_departments =
#if   L==1
	"Altres departaments";
#elif L==2
	"Andere Abteilung";
#elif L==3
	"Other departments";
#elif L==4
	"Otros departamentos";
#elif L==5
	"Autres d&eacute;partements";
#elif L==6
	"Otros departamentos";	// Okoteve traduccin
#elif L==7
	"Altri dipartimenti";
#elif L==8
	"Inne wydzialy";
#elif L==9
	"Outros departamentos";
#endif

const char *Txt_Other_emails =
#if   L==1
	"Altres correus";
#elif L==2
	"Andere E-Mails";
#elif L==3
	"Other emails";
#elif L==4
	"Otros correos";
#elif L==5
	"Autres emails";
#elif L==6
	"Otros correos";	// Okoteve traduccin
#elif L==7
	"Altre email";
#elif L==8
	"Inne email";
#elif L==9
	"Outros emails";
#endif

const char *Txt_Other_nicknames =
#if   L==1
	"Altres sobrenoms";
#elif L==2
	"Andere Benutzername";
#elif L==3
	"Other nicknames";
#elif L==4
	"Otros apodos";
#elif L==5
	"Autres surnoms";
#elif L==6
	"Otros apodos";	// Okoteve traduccin
#elif L==7
	"Altri nomi utenti";
#elif L==8
	"Inne przydomki";
#elif L==9
	"Outras alcunha";
#endif

const char *Txt_Other_places =
#if   L==1
	"Altres llocs";
#elif L==2
	"Andere Standorte";
#elif L==3
	"Other places";
#elif L==4
	"Otros lugares";
#elif L==5
	"Autres emplacements";
#elif L==6
	"Otros lugares";	// Okoteve traduccin
#elif L==7
	"Altre localit&agrave;";
#elif L==8
	"Inne miejsca";
#elif L==9
	"Outras localiza&ccedil;oes";
#endif

const char *Txt_Other_recipients =
#if   L==1
	"Altres destinataris";
#elif L==2
	"Andere Empf&auml;nger";
#elif L==3
	"Other recipients";
#elif L==4
	"Otros destinatarios";
#elif L==5
	"Autres destinataires";
#elif L==6
	"Otros destinatarios";	// Okoteve traduccin
#elif L==7
	"Altri destinatari";
#elif L==8
	"Inne odbiorc&oacute;w";
#elif L==9
	"Outros destinat&aacute;rios";
#endif

const char *Txt_out_of_PART_OF_A_SCORE =	// Example: score is 2 out of 10
#if   L==1
	"sobre";	// Necessita traduccio
#elif L==2
	"von";
#elif L==3
	"out of";
#elif L==4
	"sobre";
#elif L==5
	"sur";
#elif L==6
	"sobre";	// Okoteve traduccin
#elif L==7
	"su";
#elif L==8
	"obecnie";
#elif L==9
	"out of";
#endif

const char *Txt_Page =
#if   L==1
	"P&agrave;g.";
#elif L==2
	"Page";
#elif L==3
	"Page";
#elif L==4
	"P&aacute;g.";
#elif L==5
	"Page";
#elif L==6
	"Kuatiarogue";
#elif L==7
	"Pag.";
#elif L==8
	"Strona";
#elif L==9
	"P&aacute;g.";
#endif

const char *Txt_page =
#if   L==1
	"p&agrave;g.";
#elif L==2
	"Page";
#elif L==3
	"page";
#elif L==4
	"p&aacute;g.";
#elif L==5
	"page";
#elif L==6
	"kuatiarogue";
#elif L==7
	"pag.";
#elif L==8
	"strona";
#elif L==9
	"p&aacute;g.";
#endif

const char *Txt_PAGE1_Page_generated_in =
#if   L==1
	"P&agrave;gina generada en";
#elif L==2
	"Page generated in";
#elif L==3
	"Page generated in";
#elif L==4
	"P&aacute;gina generada en";
#elif L==5
	"Page g&eacute;n&eacute;r&eacute;e en";
#elif L==6
	"P&aacute;gina generada en";	// Okoteve traduccin
#elif L==7
	"Pagina generata in";
#elif L==8
	"Strona wygenerowana w";
#elif L==9
	"P&aacute;gina gerada em";
#endif

const char *Txt_PAGE2_and_sent_in =
#if   L==1
	"i enviat en";
#elif L==2
	"gesendet und in";
#elif L==3
	"and sent in";
#elif L==4
	"y enviada en";
#elif L==5
	"et envoy&eacute;e en";
#elif L==6
	"y enviada en";	// Okoteve traduccin
#elif L==7
	"e inviata in";
#elif L==8
	"i wysylane w";
#elif L==9
	"e enviada em";
#endif

const char *Txt_PAGES_Previous =
#if   L==1
	"Anteriores";	// Necessita traduccio
#elif L==2
	"Zur&uuml;ck";
#elif L==3
	"Previous";
#elif L==4
	"Anteriores";
#elif L==5
	"Pr&eacute;c&eacute;dentes";
#elif L==6
	"Anteriores";	// Okoteve traduccin
#elif L==7
	"Precedenti";
#elif L==8
	"Poprzedni";
#elif L==9
	"Anteriores";
#endif

const char *Txt_PAGES_Next =
#if   L==1
	"Siguientes";	// Necessita traduccio
#elif L==2
	"N&auml;chste";
#elif L==3
	"Next";
#elif L==4
	"Siguientes";
#elif L==5
	"Suivantes";
#elif L==6
	"Siguientes";	// Okoteve traduccin
#elif L==7
	"Seguenti";
#elif L==8
	"Nastepny";
#elif L==9
	"Seguintes";
#endif

const char *Txt_Password =
#if   L==1
	"Contrasenya";
#elif L==2
	"Passwort";
#elif L==3
	"Password";
#elif L==4
	"Contrase&ntilde;a";
#elif L==5
	"Mot de passe";
#elif L==6
	"Tera&ntilde;emi";
#elif L==7
	"Password";
#elif L==8
	"Has&lstrok;o";
#elif L==9
	"Senha";
#endif

const char *Txt_password =
#if   L==1
	"contrasenya";
#elif L==2
	"Passwort";
#elif L==3
	"password";
#elif L==4
	"contrase&ntilde;a";
#elif L==5
	"mot de passe";
#elif L==6
	"tera&ntilde;emi";
#elif L==7
	"password";
#elif L==8
	"has&lstrok;o";
#elif L==9
	"senha";
#endif

const char *Txt_Paste =
#if   L==1
	"Pegar";	// Necessita traduccio
#elif L==2
	"Einf&uuml;gen";
#elif L==3
	"Paste";
#elif L==4
	"Pegar";
#elif L==5
	"Coller";
#elif L==6
	"Pegar";	// Okoteve traduccin
#elif L==7
	"Incolla";
#elif L==8
	"Wklej";
#elif L==9
	"Colar";
#endif

const char *Txt_Paste_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Pegar en %s";	// Necessita traduccio
#elif L==2
	"Einf&uuml;gen an %s";
#elif L==3
	"Paste in %s";
#elif L==4
	"Pegar en %s";
#elif L==5
	"Coller dans %s";
#elif L==6
	"Pegar en %s";	// Okoteve traduccin
#elif L==7
	"Incolla in %s";
#elif L==8
	"Wklej w %s";
#elif L==9
	"Colar em %s";
#endif

const char *Txt_Paste_thread =
#if   L==1
	"Pegar discusi&oacute;n";	// Necessita traduccio
#elif L==2
	"Einf&uuml;gen Thread";
#elif L==3
	"Paste thread";
#elif L==4
	"Pegar discusi&oacute;n";
#elif L==5
	"Coller fil";
#elif L==6
	"Pegar discusi&oacute;n";	// Okoteve traduccin
#elif L==7
	"Incolla discussione";
#elif L==8
	"Wklej watku";
#elif L==9
	"Colar thread";
#endif

const char *Txt_PERCENT_of_users =
#if   L==1
	"% de usuarios";	// Necessita traduccio
#elif L==2
	"% der Benutzer";
#elif L==3
	"% of users";
#elif L==4
	"% de usuarios";
#elif L==5
	"% d'utilisateurs";
#elif L==6
	"% de usuarios";	// Okoteve traduccin
#elif L==7
	"% di utenti";
#elif L==8
	"% uzytkownik&oacute;w";
#elif L==9
	"% de utilizadores";
#endif

const char *Txt_Permalink =
#if   L==1
	"Enlla&ccedil; permanent";
#elif L==2
	"Permalink";
#elif L==3
	"Permalink";
#elif L==4
	"Permalink";
#elif L==5
	"Permalien";
#elif L==6
	"Permalink";	// Okoteve traduccin
#elif L==7
	"Permalink";
#elif L==8
	"Odno&sacute;nik bezpo&sacute;redni";
#elif L==9
	"Liga&ccedil;&atilde;o permanente";
#endif

const char *Txt_Personal_information =
#if   L==1
	"Dades personals";
#elif L==2
	"Pers&ouml;nliche Angaben";
#elif L==3
	"Personal information";
#elif L==4
	"Datos personales";
#elif L==5
	"Informations personnelles";
#elif L==6
	"Datos personales";	// Okoteve traduccin
#elif L==7
	"Informazione personale";
#elif L==8
	"Informacje osobiste";
#elif L==9
	"Informa&ccedil;&atilde;o pessoal";
#endif

const char *Txt_Phone =
#if   L==1
	"Tel&egrave;fon";
#elif L==2
	"Telefon";
#elif L==3
	"Phone";
#elif L==4
	"Tel&eacute;fono";
#elif L==5
	"T&eacute;l&eacute;phone";
#elif L==6
	"Pumbyry";
#elif L==7
	"Telefono";
#elif L==8
	"Telefon";
#elif L==9
	"Telefone";
#endif

const char *Txt_Photo =
#if   L==1
	"Foto";
#elif L==2
	"Foto";
#elif L==3
	"Photo";
#elif L==4
	"Foto";
#elif L==5
	"Photo";
#elif L==6
	"Ta'anga";
#elif L==7
	"Foto";
#elif L==8
	"Zdj&eogon;cie";
#elif L==9
	"Foto";
#endif

const char *Txt_Photo_has_been_updated =
#if   L==1
	"La fotograf&iacute;a se ha actualizado.";	// Necessita traduccio
#elif L==2
	"Photo has been updated.";			// Need bersetzung
#elif L==3
	"Photo has been updated.";
#elif L==4
	"La fotograf&iacute;a se ha actualizado.";
#elif L==5
	"Photo has been updated.";			// Besoin de traduction
#elif L==6
	"La fotograf&iacute;a se ha actualizado.";	// Okoteve traduccin
#elif L==7
	"La foto &egrave; stata aggiornata.";
#elif L==8
	"Zdjecie zostalo zaktualizowane.";
#elif L==9
	"Photo has been updated.";			// Necessita de traduo
#endif

const char *Txt_Photo_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Se ha eliminado la foto de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Photo of <strong>%s</strong> has been removed.";	// Need bersetzung
#elif L==3
	"Photo of <strong>%s</strong> has been removed.";
#elif L==4
	"Se ha eliminado la foto de <strong>%s</strong>.";
#elif L==5
	"Photo of <strong>%s</strong> has been removed.";	// Besoin de traduction
#elif L==6
	"Se ha eliminado la foto de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"La foto de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==8
	"Zdjecie <strong>%s</strong> zostalo usuniete.";
#elif L==9
	"Photo of <strong>%s</strong> has been removed.";	// Necessita de traduo
#endif

const char *Txt_PHOTO_PROCESSING_CAPTIONS[3] =
   {
#if   L==1
	 "Rostro seleccionado"		// Necessita traduccio
#elif L==2
	 "Face selected"			// Need bersetzung
#elif L==3
	 "Face selected"
#elif L==4
	 "Rostro seleccionado"
#elif L==5
	 "Face selected"			// Besoin de traduction
#elif L==6
	 "Rostro seleccionado"		// Okoteve traduccin
#elif L==7
	 "Viso selezionato"
#elif L==8
	 "Twarze wybranych"
#elif L==9
	 "Face selected"			// Necessita de traduo
#endif
	 ,
#if   L==1
	 "Contraste y saturaci&oacute;n mejorados"	// Necessita traduccio
#elif L==2
	 "Contrast and saturation enhanced"		// Need bersetzung
#elif L==3
	 "Contrast and saturation enhanced"
#elif L==4
	 "Contraste y saturaci&oacute;n mejorados"
#elif L==5
	 "Contrast and saturation enhanced"		// Besoin de traduction
#elif L==6
	 "Contraste y saturaci&oacute;n mejorados"	// Okoteve traduccin
#elif L==7
	 "Contrasto e saturazione migliorata"
#elif L==8
	 "Zwiekszenie kontrastu i nasycenia"
#elif L==9
	 "Contrast and saturation enhanced"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "Balance de blancos realizado"		// Necessita traduccio
#elif L==2
	 "White balance made"			// Need bersetzung
#elif L==3
	 "White balance made"
#elif L==4
	 "Balance de blancos realizado"
#elif L==5
	 "White balance made"			// Besoin de traduction
#elif L==6
	 "Balance de blancos realizado"		// Okoteve traduccin
#elif L==7
	 "Bilanciamento di bianco realizzato"
#elif L==8
	 "Balans bieli sie"
#elif L==9
	 "White balance made"			// Necessita de traduo
#endif
	};

const char *Txt_Photo_removed =
#if   L==1
	"Fotograf&iacute;a eliminada.";	// Necessita traduccio
#elif L==2
	"Foto entfernt.";
#elif L==3
	"Photo removed.";
#elif L==4
	"Fotograf&iacute;a eliminada.";
#elif L==5
	"Photo removed.";		// Besoin de traduction
#elif L==6
	"Fotograf&iacute;a eliminada.";	// Okoteve traduccin
#elif L==7
	"Foto rimossa.";
#elif L==8
	"Zdjecie usuniete.";
#elif L==9
	"Photo removed.";		// Necessita de traduo
#endif

const char *Txt_photos =
#if   L==1
	"fotos";
#elif L==2
	"Fotos";
#elif L==3
	"photos";
#elif L==4
	"fotos";
#elif L==5
	"photos";
#elif L==6
	"fotos";	// Okoteve traduccin
#elif L==7
	"foto";
#elif L==8
	"zdjecia";
#elif L==9
	"fotos";
#endif

const char *Txt_XxY_pixels_or_higher =
#if   L==1
	"p&iacute;xels o m&eacute;s gran";
#elif L==2
	"Pixel oder h&ouml;her";
#elif L==3
	"pixels or higher";
#elif L==4
	"p&iacute;xeles o mayor";
#elif L==5
	"pixels ou plus";
#elif L==6
	"p&iacute;xeles o mayor";	// Okoteve traduccin
#elif L==7
	"pixel o superiore";
#elif L==8
	"pikseli lub wy&zdot;szej";
#elif L==9
	"pixels ou superior";
#endif

const char *Txt_Place =		// A particular spot or area
#if   L==1
	"Lloc";
#elif L==2
	"Standort";
#elif L==3
	"Place";
#elif L==4
	"Lugar";
#elif L==5
	"Emplacement";
#elif L==6
	"Tenda";
#elif L==7
	"Luogo";
#elif L==8
	"Miejsce";
#elif L==9
	"Localiza&ccedil;&atilde;o";
#endif

const char *Txt_Place_of_origin =
#if   L==1
	"Lugar de origen";	// Necessita traduccio
#elif L==2
	"Herkunftsort";
#elif L==3
	"Place of origin";
#elif L==4
	"Lugar de origen";
#elif L==5
	"Lieu d'origine";
#elif L==6
	"Lugar de origen";	// Okoteve traduccin
#elif L==7
	"Luogo d'origine";
#elif L==8
	"Miejsce pochodzenia";
#elif L==9
	"Lugar de origem";
#endif

const char *Txt_Place_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Lugar <strong>%s</strong> eliminado.";		// Necessita traduccio
#elif L==2
	"Standort <strong>%s</strong> entfernt.";
#elif L==3
	"Place <strong>%s</strong> removed.";
#elif L==4
	"Lugar <strong>%s</strong> eliminado.";
#elif L==5
	"Emplacement <strong>%s</strong> supprim&eacute;.";
#elif L==6
	"Lugar <strong>%s</strong> eliminado.";		// Okoteve traduccin
#elif L==7
	"Luogo <strong>%s</strong> rimosso.";
#elif L==8
	"Miejsce <strong>%s</strong> usuniete.";
#elif L==9
	"Place <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Place_unspecified =
#if   L==1
	"Lloc sense especificar";
#elif L==2
	"Standort nicht spezifiziert";
#elif L==3
	"Place unspecified";
#elif L==4
	"Lugar sin especificar";
#elif L==5
	"Emplacement non sp&eacute;cifi&eacute;";
#elif L==6
	"Lugar sin especificar";	// Okoteve traduccin
#elif L==7
	"Luogo non specificato";
#elif L==8
	"Miejsce nieokreslone";
#elif L==9
	"Localiza&ccedil;&atilde;o indeterminada";
#endif

const char *Txt_Places =
#if   L==1
	"Llocs";
#elif L==2
	"Standorte";
#elif L==3
	"Places";
#elif L==4
	"Lugares / sedes";
#elif L==5
	"Emplacements";
#elif L==6
	"Tenda";
#elif L==7
	"Localit&grave;";
#elif L==8
	"Miejsca";
#elif L==9
	"Localiza&ccedil;oes";
#endif

const char *Txt_PLACES_HELP_ORDER[2] =
   {
#if   L==1
	 "Ordenar por lugar"	// Necessita traduccio
#elif L==2
	 "Nach Standort sortieren"
#elif L==3
	 "Sort by place"
#elif L==4
	 "Ordenar por lugar"
#elif L==5
	 "Trier par emplacement"
#elif L==6
	 "Ordenar por lugar"	// Okoteve traduccin
#elif L==7
	 "Ordina per localit&grave;"
#elif L==8
	 "Sortuj wedlug miejsca"
#elif L==9
	 "Classificar por localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "Ordenar por n&ordm; de titulaciones"	// Necessita traduccio
#elif L==2
	 "Nach Anzahl der Studieng&auml;nge sortieren"
#elif L==3
	 "Sort by no. of degrees"
#elif L==4
	 "Ordenar por n&ordm; de titulaciones"
#elif L==5
	 "Trier par nombre d'&eacute;tudes"
#elif L==6
	 "Ordenar por n&ordm; de titulaciones"	// Okoteve traduccin
#elif L==7
	 "Ordina per numero di lauree"
#elif L==8
	 "Sortuj wedlug stopni"
#elif L==9
	 "Classificar por n&ordm; de titula&ccedil;oes"
#endif
	};

const char *Txt_PLACES_ORDER[2] =
   {
#if   L==1
	 "Lloc"
#elif L==2
	 "Standort"
#elif L==3
	 "Place"
#elif L==4
	 "Lugar"
#elif L==5
	 "Emplacement"
#elif L==6
	 "Tenda"
#elif L==7
	 "Luogo"
#elif L==8
	 "Miejsce"
#elif L==9
	 "Localiza&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	"Centres"
#elif L==2
	"Lehrinstitute"
#elif L==3
	"Centres"
#elif L==4
	"Centros"
#elif L==5
	"Centres"
#elif L==6
	"Mbo'ehao"
#elif L==7
	"Centri"
#elif L==8
	"Centra"
#elif L==9
	"Centros"
#endif
	};

const char *Txt_Please_check_and_confirm_your_email_address =
#if   L==1
	"Si us plau, comproveu i confirmeu la seva adre&ccedil;a de correu.";
#elif L==2
	"Bitte &uuml;berpr&uuml;fen Sie und Ihre E-Mail-Adresse best&auml;tigen.";
#elif L==3
	"Please check and confirm your email address.";
#elif L==4
	"Por favor, compruebe y confirme su direcci&oacute;n de correo.";
#elif L==5
	"S'il vous pla&iacute;t, v&eacute;rifiez et confirmez votre adresse email.";
#elif L==6
	"Por favor, compruebe y confirme su direcci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7
	"Per favore, verifica e conferma il tuo indirizzo email.";
#elif L==8
	"Prosz&eogon; sprawdzi&cacute; i potwierdzi&cacute; sw&oacute;j adres email.";
#elif L==9
	"Por favor, verificar e confirmar o seu endere&ccedil;o de email.";
#endif

const char *Txt_Please_enter_your_ID =
#if   L==1
	"Si us plau, escriviu el seu ID (DNI/c&egrave;dula).";
#elif L==2
	"Bitte geben Sie Ihre Ausweis-Nr. &auml;nder.";
#elif L==3
	"Please enter your ID.";
#elif L==4
	"Por favor, escriba su ID (DNI/c&eacute;dula).";
#elif L==5
	"S'il vous pla&iacute;t, entrez votre ID.";
#elif L==6
	"Por favor, escriba su ID (DNI/c&eacute;dula).";		// Okoteve traduccin
#elif L==7
	"Per favore, inserisci il tuo ID.";
#elif L==8
	"Prosz&eogon;, poda&cacute; swoje zmie&nacute; identyfikatory.";
#elif L==9
	"Por favor, digite n&ordm; de identifica&ccedil;&atilde;o.";
#endif

const char *Txt_Please_fill_in_your_centre_and_department =
#if   L==1
	"Si us plau, ompli el seu centre i department.";
#elif L==2
	"Bitte geben Sie Ihre Zentrum und Abteilung zu f&uuml;llen.";
#elif L==3
	"Please fill in your centre and department.";
#elif L==4
	"Por favor, rellene su centro y departamento.";
#elif L==5
	"S'il vous pla&iacute;t, indiquez votre centre et d&eacute;partement.";
#elif L==6
	"Por favor, rellene su centro y departamento.";	// Okoteve traduccin
#elif L==7
	"Per favore, inserisci il tuo centro e dipartimento.";
#elif L==8
	"Prosz&eogon;, wpisz swoje tym centrum i dzia&lstrok;u.";
#elif L==9
	"Por favor, preencha o seu centro e departamento.";
#endif

const char *Txt_Please_fill_in_your_email_address =
#if   L==1
	"Si us plau, ompli la seva adre&ccedil;a de correu.";
#elif L==2
	"Bitte geben Sie Ihre E-Mail-Adresse ein.";
#elif L==3
	"Please fill in your email address.";
#elif L==4
	"Por favor, rellene su direcci&oacute;n de correo.";
#elif L==5
	"S'il vous pla&iacute;t, indiquez votre adresse email.";
#elif L==6
	"Por favor, rellene su direcci&oacute;n de correo.";	// Okoteve traduccin
#elif L==7
	"Per favore, inserisci il tuo indirizzo email.";
#elif L==8
	"Prosz&eogon;, wpisz swoje w tym adresu email.";
#elif L==9
	"Por favor, preencha o seu endere&ccedil;o de email.";
#endif

const char *Txt_Please_fill_in_your_ID =
#if   L==1
	"Si us plau, ompli el seu ID (DNI/c&egrave;dula)."
	" &Eacute;s una dada opcional,"
	" per&ograve; &eacute;s recomanable que l'ompli"
        " perqu&egrave; facilita el proc&eacute;s d'inscripci&oacute;"
        " dels usuaris en les assignatures.";
#elif L==2
	"Bitte geben Sie Ihre Ausweis-Nr. &auml;nder."
	" Es ist eine optionale Daten,"
	" aber es wird empfohlen, sie zu f&uuml;llen,"
	" weil sie die Registrierung"
	" der Benutzer in den Kursen erleichtert.";
#elif L==3
	"Please fill in your ID."
	" It is an optional data,"
	" but it is recommended that you fill it"
	" because it facilitates the registration process"
	" of users in the courses.";
#elif L==4
	"Por favor, rellene su ID (DNI/c&eacute;dula)."
	" Es un dato opcional,"
	" pero es recomendable que lo rellene"
	" porque facilita el proceso de inscripci&oacute;n"
	" de los usuarios en las asignaturas.";
#elif L==5
	"S'il vous pla&iacute;t, indiquez votre ID."
	" C'est une donn&eacute;e facultative,"
	" mais il est recommand&eacute; que vous la remplissez"
	" parce qu'il facilite le processus d'enregistrement"
	" des utilisateurs dans les mati&egrave;res.";
#elif L==6
	"Por favor, rellene su ID (DNI/c&eacute;dula)."
	" Es un dato opcional,"
	" pero es recomendable que lo rellene"
	" porque facilita el proceso de inscripci&oacute;n"
	" de los usuarios en las asignaturas.";		// Okoteve traduccin
#elif L==7
	"Per favore, inserisci il tuo ID."
	" Si tratta di un dato facoltativo,"
	" ma si raccomanda che si riempie"
	" perch&eacute; facilita il processo di registrazione"
	" degli utenti ai corsi.";
#elif L==8
	"Prosz&eogon;, wpisz swoje zmie&nacute; identyfikatory."
	" Jest to opcja danych, ale zaleca si&eogon;,"
	" aby wype&lstrok;ni&cacute; go, poniewa&zdot; u&lstrok;atwia proces rejestracji"
	" u&zdot;ytkownik&oacute;w w kursach.";
#elif L==9
	"Por favor, preencha seu n&ordm; de identifica&ccedil;&atilde;o"
	" &Eacute; uma dada opcional,"
	" mas &eacute; recomend&aacute;vel que voc&ecirc; preench&ecirc;-lo,"
	" pois facilita o processo de registro"
	" de usu&aacute;rios nas disciplinas.";
#endif

const char *Txt_Please_fill_in_your_institution =
#if   L==1
	"Si us plau, ompli la seva instituci&oacute;.";
#elif L==2
	"Bitte geben Sie Ihre Einrichtung zu f&uuml;llen.";
#elif L==3
	"Please fill in your institution.";
#elif L==4
	"Por favor, rellene su instituci&oacute;n.";
#elif L==5
	"S'il vous pla&iacute;t, indiquez votre &eacute;tablissement.";
#elif L==6
	"Por favor, rellene su instituci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Per favore, inserisci la tua istituzione accademica.";
#elif L==8
	"Prosz&eogon;, wpisz swoje tym instytucji.";
#elif L==9
	"Por favor, preencha sua institui&ccedil;&atilde;o.";
#endif

const char *Txt_Please_fill_in_your_record_card_including_your_country_nationality =
#if   L==1
	"Si us plau, ompli la seva fitxa,"
	" incloent el seu pais (nacionalitat).";
#elif L==2
	"Bitte geben Sie Ihre Karteikarte zu f&uuml;llen,"
	" einschlie&szlig;lich Ihrem Land (Nationalit&auml;t).";
#elif L==3
	"Please fill in your country (nationality).";
#elif L==4
	"Por favor, rellene su ficha,"
	" incluyendo su pa&iacute;s (nacionalidad).";
#elif L==5
	"S'il vous pla&iacute;t, remplissez votre carte d'enregistrement,"
	" y compris votre pays (nationalit&eacute;).";
#elif L==6
	"Por favor, rellene su ficha,"
	" incluyendo su pa&iacute;s (nacionalidad).";	// Okoteve traduccin
#elif L==7
	"Per favore, compila la tua scheda,"
	" compreso il tuo paese (nazionalit&agrave;).";
#elif L==8
	"Prosz&eogon;, wpisz swoje karte zapisu,"
	" w tym kraju (obywatelstwo";
#elif L==9
	"Por favor, preencha o seu cart&atilde;o,"
	" incluindo seu pa&iacute;s (nacionalidade).";
#endif

const char *Txt_Please_fill_in_your_record_card_including_your_name =
#if   L==1
	"Si us plau, ompli la seva fitxa,"
	" incloent el seu nom i cognom(s).";
#elif L==2
	"Bitte geben Sie Ihre Karteikarte zu f&uuml;llen,"
	" mit Ihrem Namen.";
#elif L==3
	"Please fill in your record card,"
	" including your name.";
#elif L==4
	"Por favor, rellene su ficha,"
	" incluyendo su nombre y apellido(s).";
#elif L==5
	"S'il vous pla&iacute;t, remplissez votre carte d'enregistrement,"
	" y compris votre nom.";
#elif L==6
	"Por favor, rellene su ficha,"
	" incluyendo su nombre y apellido(s).";		// Okoteve traduccin
#elif L==7
	"Per favore, compila la tua scheda,"
	" compresi il tuo nome e cognome.";
#elif L==8
	"Prosz&eogon;, wpisz swoje karte zapisu,"
	" w tym imie i nazwisko.";
#elif L==9
	"Por favor, preencha o cart&atilde;o,"
	" incluindo seu nome.";
#endif

const char *Txt_Please_fill_in_your_record_card_including_your_sex =
#if   L==1
	"Si us plau, ompli la seva fitxa,"
	" incloent el sexe.";
#elif L==2
	"Bitte geben Sie Ihre Karteikarte zu f&uuml;llen,"
	" einschlie&szlig;lich Ihrer Sex.";
#elif L==3
	"Please fill in your record card,"
	" including your sex.";
#elif L==4
	"Por favor, rellene su ficha,"
	" incluyendo el sexo.";
#elif L==5
	"S'il vous pla&iacute;t, remplissez votre carte d'enregistrement,"
	" y compris votre sexe.";
#elif L==6
	"Por favor, rellene su ficha,"
	" incluyendo el sexo.";				// Okoteve traduccin
#elif L==7
	"Per favore, compila la tua scheda,"
	" compreso il sesso.";
#elif L==8
	"Prosz&eogon;, wpisz swoje karte zapisu,"
	" w tym ple&cacute;.";
#elif L==9
	"Por favor, preencha o cart&atilde;o,"
	" incluindo o seu sexo.";
#endif

const char *Txt_Please_review_your_privacy_preferences =
#if   L==1
	"Si us plau, revisi les seves prefer&egrave;ncies de privacitat.";
#elif L==2
	"Bitte &uuml;berpr&uuml;fen Sie Ihre Datenschutzeinstellungen.";
#elif L==3
	"Please review your privacy preferences.";
#elif L==4
	"Por favor, revise sus preferencias de privacidad.";
#elif L==5
	"S'il vous pla&icirc;t revoir vos pr&eacute;f&eacute;rences de privacit&eacute;.";
#elif L==6
	"Por favor, revise sus preferencias de privacidad.";	// Okoteve traduccin
#elif L==7
	"Si prega di rivedere le vostre preferenze di privatezza.";
#elif L==8
	"Prosz&eogon; sprawdzi&cacute; ustawienia prywatno&sacute;ci.";
#elif L==9
	"Por favor, revise suas prefer&ecirc;ncias de privacidade.";
#endif

const char *Txt_Please_select_the_country_of_your_institution =
#if   L==1
	"Si us plau, seleccioneu el pa&iacute;s de la seva instituci&oacute;.";
#elif L==2
	"Bitte w&auml;hlen Sie das Land Ihrer Einrichtung aus";
#elif L==3
	"Please select the country of your institution.";
#elif L==4
	"Por favor, seleccione el pa&iacute;s de su instituci&oacute;n.";
#elif L==5
	"S'il vous pla&iacute;t, s&eacute;lectionnez le pays de votre &eacute;tablissement.";
#elif L==6
	"Por favor, seleccione el pa&iacute;s de su instituci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Per favore, selezionare il paese di la tua istituzione accademica.";
#elif L==8
	"Prosz&eogon;, wybra&cacute; kraj swojej instytucji.";
#elif L==9
	"Por favor, selecione o pa&iacute;s da sua institui&ccedil;&atilde;o.";
#endif

const char *Txt_Please_specify_if_you_allow_access_to_test_questions_from_mobile_applications =
#if   L==1
	"Por favor, especifique si permite el acceso"
	" a las preguntas de test desde aplicaciones m&oacute;viles.";	// Necessita traduccio
#elif L==2
	"Please specify if you allow access"
	" to test questions from mobile applications.";	// Need bersetzung
#elif L==3
	"Please specify if you allow access"
	" to test questions from mobile applications.";
#elif L==4
	"Por favor, especifique si permite el acceso"
	" a las preguntas de test desde aplicaciones m&oacute;viles.";
#elif L==5
	"Please specify if you allow access"
	" to test questions from mobile applications.";	// Besoin de traduction
#elif L==6
	"Por favor, especifique si permite el acceso"
	" a las preguntas de test desde aplicaciones m&oacute;viles.";	// Okoteve traduccin
#elif L==7
	"Please specify if you allow access"
	" to test questions from mobile applications.";	// Bisogno di traduzione
#elif L==8
	"Please specify if you allow access"
	" to test questions from mobile applications.";	// Potrzebujesz tlumaczenie
#elif L==9
	"Please specify if you allow access"
	" to test questions from mobile applications.";	// Necessita de traduo
#endif

const char *Txt_Plugin =
#if   L==1
	"Complement";
#elif L==2
	"Plugin";
#elif L==3
	"Plugin";
#elif L==4
	"Complemento";
#elif L==5
	"Plugin";
#elif L==6
	"Complemento";	// Okoteve traduccin
#elif L==7
	"Plugin";
#elif L==8
	"Wtyczka";
#elif L==9
	"Plugin";
#endif

const char *Txt_Plugin_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Complemento <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2
	"Plugin <strong>%s</strong> entfernt.";
#elif L==3
	"Plugin <strong>%s</strong> removed.";
#elif L==4
	"Complemento <strong>%s</strong> eliminado.";
#elif L==5
	"Plugin <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6
	"Complemento <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Plugin <strong>%s</strong> rimosso.";
#elif L==8
	"Wtyczka <strong>%s</strong> usunieta.";
#elif L==9
	"Plugin <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Plugins =
#if   L==1
	"Complements";
#elif L==2
	"Plugins";
#elif L==3
	"Plugins";
#elif L==4
	"Complementos";
#elif L==5
	"Plugins";
#elif L==6
	"Complementos";		// Okoteve traduccin
#elif L==7
	"Plugin";
#elif L==8
	"wtyczki";
#elif L==9
	"Plugins";
#endif

const char *Txt_Possibly_duplicate_users =
#if   L==1
	"Usuaris possiblement duplicats";
#elif L==2
	"M&ouml;gliche doppelte Benutzer";
#elif L==3
	"Possibly duplicate users";
#elif L==4
	"Usuarios posiblemente duplicados";
#elif L==5
	"Possibles utilisateurs en double";
#elif L==6
	"Usuarios posiblemente duplicados";	// Okoteve traduccin
#elif L==7
	"Possibili utenti duplicati";
#elif L==8
	"Mo&zdot;liwe duplikaty u&zdot;ytkownik&oacute;w";
#elif L==9
	"Usu&aacute;rios possivelmente duplicados";
#endif

const char *Txt_Post_X_allowed =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Mensaje %u permitido.";	// Necessita traduccio
#elif L==2
	"Post %u allowed.";		// Need bersetzung
#elif L==3
	"Post %u allowed.";
#elif L==4
	"Mensaje %u permitido.";
#elif L==5
	"Post %u allowed.";		// Besoin de traduction
#elif L==6
	"Mensaje %u permitido.";	// Okoteve traduccin
#elif L==7
	"Post %u autorizzato.";
#elif L==8
	"Post %u dozwolone.";
#elif L==9
	"Post %u allowed.";		// Necessita de traduo
#endif

const char *Txt_Post_X_allowed_Click_to_ban_it =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Mensaje %u permitido. Pulse para censurarlo.";	// Necessita traduccio
#elif L==2
	"Post %u allowed. Click to ban it.";		// Need bersetzung
#elif L==3
	"Post %u allowed. Click to ban it.";
#elif L==4
	"Mensaje %u permitido. Pulse para censurarlo.";
#elif L==5
	"Post %u allowed. Click to ban it.";		// Besoin de traduction
#elif L==6
	"Mensaje %u permitido. Pulse para censurarlo.";	// Okoteve traduccin
#elif L==7
	"Post %u autorizzato. Clicca per censurarlo.";
#elif L==8
	"Post% u dozwolone. Kliknij, aby to zakazac.";
#elif L==9
	"Post %u allowed. Click to ban it.";		// Necessita de traduo
#endif

const char *Txt_Post_and_thread_removed =
#if   L==1
	"Mensaje y discusi&oacute;n eliminados.";	// Necessita traduccio
#elif L==2
	"Nachricht und Faden entfernt.";
#elif L==3
	"Post and thread removed.";
#elif L==4
	"Mensaje y discusi&oacute;n eliminados.";
#elif L==5
	"Post and thread removed.";			// Besoin de traduction
#elif L==6
	"Mensaje y discusi&oacute;n eliminados.";	// Okoteve traduccin
#elif L==7
	"Post e discussione rimossi.";
#elif L==8
	"Post watku usuniety.";
#elif L==9
	"Post and thread removed.";			// Necessita de traduo
#endif

const char *Txt_Post_banned =
#if   L==1
	"Mensaje no permitido";		// Necessita traduccio
#elif L==2
	"Post banned.";			// Need bersetzung
#elif L==3
	"Post banned.";
#elif L==4
	"Mensaje no permitido";
#elif L==5
	"Post banned.";			// Besoin de traduction
#elif L==6
	"Mensaje no permitido";		// Okoteve traduccin
#elif L==7
	"Post censurato.";
#elif L==8
	"Post zakazany.";
#elif L==9
	"Post banned.";			// Necessita de traduo
#endif

const char *Txt_Post_X_banned =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Mensaje %u no permitido";	// Necessita traduccio
#elif L==2
	"Post %u banned.";		// Need bersetzung
#elif L==3
	"Post %u banned.";
#elif L==4
	"Mensaje %u no permitido";
#elif L==5
	"Post %u banned.";		// Besoin de traduction
#elif L==6
	"Mensaje %u no permitido";	// Okoteve traduccin
#elif L==7
	"Post %u censurato.";
#elif L==8
	"Post %u zakazany.";
#elif L==9
	"Post %u banned.";		// Necessita de traduo
#endif

const char *Txt_Post_X_banned_Click_to_unban_it =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Mensaje %u no permitido. Pulse para permitirlo.";	// Necessita traduccio
#elif L==2
	"Post %u banned. Click to unban it.";		// Need bersetzung
#elif L==3
	"Post %u banned. Click to unban it.";
#elif L==4
	"Mensaje %u no permitido. Pulse para permitirlo.";
#elif L==5
	"Post %u banned. Click to unban it.";		// Besoin de traduction
#elif L==6
	"Mensaje %u no permitido. Pulse para permitirlo.";	// Okoteve traduccin
#elif L==7
	"Post %u censurato. Clicca per autorizzarlo.";
#elif L==8
	"Post %u zakazany.Kliknij, aby to odbanowac.";
#elif L==9
	"Post %u banned. Click to unban it.";		// Necessita de traduo
#endif

const char *Txt_Post_sent =
#if   L==1
	"Mensaje enviado.";	// Necessita traduccio
#elif L==2
	"Post sent.";		// Need bersetzung
#elif L==3
	"Post sent.";
#elif L==4
	"Mensaje enviado.";
#elif L==5
	"Post sent.";		// Besoin de traduction
#elif L==6
	"Mensaje enviado.";	// Okoteve traduccin
#elif L==7
	"Post inviato.";
#elif L==8
	"Post wyslany.";
#elif L==9
	"Post sent.";		// Necessita de traduo
#endif

const char *Txt_Post_removed =
#if   L==1
	"Missatge eliminat.";
#elif L==2
	"Nachricht entfernt.";
#elif L==3
	"Post removed.";
#elif L==4
	"Mensaje eliminado.";
#elif L==5
	"Message supprim&eacute;.";
#elif L==6
	"Mensaje eliminado.";		// Okoteve traduccin
#elif L==7
	"Post rimosso.";
#elif L==8
	"Post usuniety.";
#elif L==9
	"Post removido.";
#endif

const char *Txt_Post_unbanned =
#if   L==1
	"Mensaje permitido";		// Necessita traduccio
#elif L==2
	"Post unbanned.";		// Need bersetzung
#elif L==3
	"Post unbanned";
#elif L==4
	"Mensaje permitido";
#elif L==5
	"Post unbanned.";		// Besoin de traduction
#elif L==6
	"Mensaje permitido";		// Okoteve traduccin
#elif L==7
	"Post autorizzato.";
#elif L==8
	"Post odbanowany";
#elif L==9
	"Post unbanned.";		// Necessita de traduo
#endif

const char *Txt_Preferences =
#if   L==1
	"Prefer&egrave;ncies";
#elif L==2
	"Einstellungen";
#elif L==3
	"Preferences";
#elif L==4
	"Preferencias";
#elif L==5
	"Pr&eacute;f&eacute;rences";
#elif L==6
	"Preferencias";	// Okoteve traduccin
#elif L==7
	"Preferenze";
#elif L==8
	"Preferencje";
#elif L==9
	"Prefer&ecirc;ncias";
#endif

const char *Txt_Present =
#if   L==1
	"Present";
#elif L==2
	"Vorhanden";
#elif L==3
	"Present";
#elif L==4
	"Presente";
#elif L==5
	"Pr&eacute;sent";
#elif L==6
	"Presente";	// Okoteve traduccin
#elif L==7
	"Presente";
#elif L==8
	"Obecny";
#elif L==9
	"Presente";
#endif

const char *Txt_Presents =
#if   L==1
	"Presents";
#elif L==2
	"Vorhanden";
#elif L==3
	"Presents";
#elif L==4
	"Presentes";
#elif L==5
	"Pr&eacute;sents";
#elif L==6
	"Presentes";	// Okoteve traduccin
#elif L==7
	"Presenti";
#elif L==8
	"Obecny";
#elif L==9
	"Presentes";
#endif

const char *Txt_Print =
#if   L==1
	"Imprimir";
#elif L==2
	"Ausdrucken";
#elif L==3
	"Print";
#elif L==4
	"Imprimir";
#elif L==5
	"Imprimer";
#elif L==6
	"Imprimir";	// Okoteve traduccin
#elif L==7
	"Stampare";
#elif L==8
	"Wydrukuj";
#elif L==9
	"Imprimir";
#endif

const char *Txt_Privacy =
#if   L==1
	"Privacitat";
#elif L==2
	"Geheimhaltung";
#elif L==3
	"Privacy";
#elif L==4
	"Privacidad";
#elif L==5
	"Privacit&eacute;";
#elif L==6
	"Privacidad";	// Okoteve traduccin
#elif L==7
	"Privatezza";
#elif L==8
	"Prywatno&sacute;&cacute;";
#elif L==9
	"Privacidade";
#endif

const char *Txt_PRIVACY_OPTIONS[Pri_NUM_OPTIONS_PRIVACY] =
   {		// Pri_VISIBILITY_UNKNOWN
#if   L==1
	 "Desconeguda"
#elif L==2
	 "Unbekannt"
#elif L==3
	 "Unknown"
#elif L==4
	 "Desconocida"
#elif L==5
	 "Inconnu"
#elif L==6
	 "Desconocida"	// Okoteve traduccin
#elif L==7
	 "Sconosciuta"
#elif L==8
	 "Nieznany"
#elif L==9
	 "Desconhecida"
#endif
	 ,	// Pri_VISIBILITY_USER
#if   L==1
	 "Visible por m&iacute; y mis profesores / mis estudiantes"	// Necessita traduccio
#elif L==2
	 "Visible by me and my teachers / my students"			// Need bersetzung
#elif L==3
	 "Visible by me and my teachers / my students"
#elif L==4
	 "Visible por m&iacute; y mis profesores / mis estudiantes"
#elif L==5
	 "Visible by me and my teachers / my students"			// Besoin de traduction
#elif L==6
	 "Visible por m&iacute; y mis profesores / mis estudiantes"	// Okoteve traduccin
#elif L==7
	 "Visible by me and my teachers / my students"			// Bisogno di traduzione
#elif L==8
	 "Visible by me and my teachers / my students"			// Potrzebujesz tlumaczenie
#elif L==9
	 "Visible by me and my teachers / my students"			// Necessita de traduo
#endif
	 ,	// Pri_VISIBILITY_COURSE
#if   L==1
	 "Visible por usuarios con los que comparto asignaturas"	// Necessita traduccio
#elif L==2
	 "Visible by users with whom I share courses"			// Need bersetzung
#elif L==3
	 "Visible by users with whom I share courses"
#elif L==4
	 "Visible por usuarios con los que comparto asignaturas"
#elif L==5
	 "Visible by users with whom I share courses"			// Besoin de traduction
#elif L==6
	 "Visible por usuarios con los que comparto asignaturas"	// Okoteve traduccin
#elif L==7
	 "Visible by users with whom I share courses"			// Bisogno di traduzione
#elif L==8
	 "Visible by users with whom I share courses"			// Potrzebujesz tlumaczenie
#elif L==9
	 "Visible by users with whom I share courses"			// Necessita de traduo
#endif
	 ,	// Pri_VISIBILITY_SYSTEM
#if   L==1
	 "Visible por cualquier usuario identificado"	// Necessita traduccio
#elif L==2
	 "Visible by any logged user"			// Need bersetzung
#elif L==3
	 "Visible by any logged user"
#elif L==4
	 "Visible por cualquier usuario identificado"
#elif L==5
	 "Visible by any logged user"			// Besoin de traduction
#elif L==6
	 "Visible por cualquier usuario identificado"	// Okoteve traduccin
#elif L==7
	 "Visible by any logged user"			// Bisogno di traduzione
#elif L==8
	 "Visible by any logged user"			// Potrzebujesz tlumaczenie
#elif L==9
	 "Visible by any logged user"			// Necessita de traduo
#endif
	 ,	// Pri_VISIBILITY_WORLD
#if   L==1
	 "Visible por todo el mundo"	// Necessita traduccio
#elif L==2
	 "Visible by everyone"		// Need bersetzung
#elif L==3
	 "Visible by everyone"
#elif L==4
	 "Visible por todo el mundo"
#elif L==5
	 "Visible by everyone"		// Besoin de traduction
#elif L==6
	 "Visible por todo el mundo"	// Okoteve traduccin
#elif L==7
	 "Visible by everyone"		// Bisogno di traduzione
#elif L==8
	 "Visible by everyone"		// Potrzebujesz tlumaczenie
#elif L==9
	 "Visible by everyone"		// Necessita de traduo
#endif
	};

const char *Txt_Private_available_to_certain_users_identified =
#if   L==1
	"Privat, disponible per a certs usuaris identificats";
#elif L==2
	"Private, verf&uuml;gbar auf bestimmte Benutzer identifiziert";
#elif L==3
	"Private, available to certain users identified";
#elif L==4
	"Privado, disponible para ciertos usuarios identificados";
#elif L==5
	"Priv&eacute;, &agrave; la disposition de certains utilisateurs identifi&eacute;s";
#elif L==6
	"Privado, disponible para ciertos usuarios identificados";	// Okoteve traduccin
#elif L==7
	"Privato, a disposizione degli utenti individuati alcuni";
#elif L==8
	"Prywatne, dost&eogon;pne dla okre&sacute;lonych u&zdot;ytkownik&oacute;w zidentyfikowanych";
#elif L==9
	"Privado, dispon&iacute;vel para alguns usu&aacute;rios identificados";
#endif

const char *Txt_Post =	// Publish (verb)
#if   L==1
	"Publicar";
#elif L==2
	"Ver&ouml;ffentlichen";
#elif L==3
	"Post";
#elif L==4
	"Publicar";
#elif L==5
	"Publier";
#elif L==6
	"Publicar";	// Okoteve traduccin
#elif L==7
	"Pubblicare";
#elif L==8
	"Publikowa&cacute;";
#elif L==9
	"Publicar";
#endif

const char *Txt_post =
#if   L==1
	"missatge";
#elif L==2
	"Nachricht";
#elif L==3
	"post";
#elif L==4
	"mensaje";
#elif L==5
	"message";
#elif L==6
	"marandu";
#elif L==7
	"post";
#elif L==8
	"post";
#elif L==9
	"post";
#endif

const char *Txt_Posts =
#if   L==1
	"Missatges";
#elif L==2
	"Nachrichten";
#elif L==3
	"Posts";
#elif L==4
	"Mensajes";
#elif L==5
	"Messages";
#elif L==6
	"Marandu";
#elif L==7
	"Post";
#elif L==8
	"Posty";
#elif L==9
	"Posts";
#endif

const char *Txt_posts =
#if   L==1
	"missatges";
#elif L==2
	"Nachrichten";
#elif L==3
	"posts";
#elif L==4
	"mensajes";
#elif L==5
	"messages";
#elif L==6
	"marandu";
#elif L==7
	"post";
#elif L==8
	"posty";
#elif L==9
	"posts";
#endif

const char *Txt_Public_activity =
#if   L==1
	 "Activitat p&uacute;blica";
#elif L==2
	 "&Ouml;ffentliche Aktivit&auml;t";
#elif L==3
	 "Public activity";
#elif L==4
	 "Actividad p&uacute;blica";
#elif L==5
	 "Activit&eacute; public";
#elif L==6
	 "Actividad p&uacute;blica";	// Okoteve traduccin
#elif L==7
	 "Attivit&agrave; pubblica";
#elif L==8
	 "Aktywno&sacute;&cacute; publiczne";
#elif L==9
	 "Atividade p&uacute;blica";
#endif

const char *Txt_Public_activity_OF_A_USER =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	 "Activitat p&uacute;blica de %s";
#elif L==2
	 "%s &ouml;ffentliche Aktivit&auml;t";
#elif L==3
	 "%s public activity";
#elif L==4
	 "Actividad p&uacute;blica de %s";
#elif L==5
	 "Activit&eacute; public de %s";
#elif L==6
	 "Actividad p&uacute;blica de %s";	// Okoteve traduccin
#elif L==7
	 "Attivit&agrave; pubblica di %s";
#elif L==8
	 "Aktywno&sacute;&cacute; publiczna %s";
#elif L==9
	 "Atividade p&uacute;blica de %s";
#endif

const char *Txt_Public_agenda_USER =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Agenda p&uacute;blica de %s";
#elif L==2
	"&Ouml;ffentliche Organizer von %s";
#elif L==3
	"%s's public agenda";
#elif L==4
	"Agenda p&uacute;blica de %s";
#elif L==5
	"Agenda publique de %s";
#elif L==6
	"Agenda p&uacute;blica de %s";	// Okoteve traduccin
#elif L==7
	"Agenda pubblica di %s";
#elif L==8
	"Pami&eogon;tnik publiczny %s";
#elif L==9
	"Agenda p&uacute;blica de %s";
#endif

const char *Txt_public_FILES =
#if   L==1
	"p&uacute;blics";
#elif L==2
	"&ouml;ffentliche";
#elif L==3
	"public";
#elif L==4
	"p&uacute;blicos";
#elif L==5
	"publics";
#elif L==6
	"p&uacute;blicos";	// Okoteve traduccin
#elif L==7
	"pubblici";
#elif L==8
	"publicznych";
#elif L==9
	"p&uacute;blicos";
#endif

const char *Txt_Public_open_educational_resource_OER_for_everyone =
#if   L==1
	"P&uacute;blic, recurs educatiu obert (OER) per a tothom";
#elif L==2
	"&Ouml;ffentliche, Open Educational Resource (OER) f&uuml;r jedermann";
#elif L==3
	"Public, open educational resource for everyone";
#elif L==4
	"P&uacute;blico, recurso educativo abierto (OER) para todo el mundo";
#elif L==5
	"Public, ressource &eacute;ducatif libre (REL) pour tout le monde";
#elif L==6
	"P&uacute;blico, recurso educativo abierto (OER) para todo el mundo";	// Okoteve traduccin
#elif L==7
	"Pubblico, risorse didattici aperto (OER) per tutti";
#elif L==8
	"Publiczne, otwarte zasoby edukacyjne (OER) dla ka&zdot;dego";
#elif L==9
	"P&uacute;blico, recurso educacional aberto (OER) para todos";
#endif

const char *Txt_Public_profile =
#if   L==1
	"Perfil p&uacute;blic";
#elif L==2
	"&Ouml;ffentliches Benutzer-Profile";
#elif L==3
	"Public profile";
#elif L==4
	"Perfil p&uacute;blico";
#elif L==5
	"Profil public";
#elif L==6
	"Perfil p&uacute;blico";	// Okoteve traduccin
#elif L==7
	"Profilo pubblico";
#elif L==8
	"Profil publiczny";
#elif L==9
	"Perfil p&uacute;blico";
#endif

const char *Txt_Public_views =
#if   L==1
	"Accessos p&uacute;blics";
#elif L==2
	"&Ouml;ffentliche Ansichten";
#elif L==3
	"Public views";
#elif L==4
	"Accesos p&uacute;blicos";
#elif L==5
	"Vues publics";
#elif L==6
	"Accesos p&uacute;blicos";	// Okoteve traduccin
#elif L==7
	"Visualizzazioni pubblici";
#elif L==8
	"Publiczne ods&lstrok;on";
#elif L==9
	"Acessos p&uacute;blicos";
#endif

const char *Txt_Publish_announcement_OF_EXAM =
#if   L==1
	"Publicar convocat&ograve;ria";
#elif L==2
	"Ver&ouml;ffentlichen Aufruf";
#elif L==3
	"Publish announcement";
#elif L==4
	"Publicar convocatoria";
#elif L==5
	"Publier convocation";
#elif L==6
	"Publicar convocatoria";	// Okoteve traduccin
#elif L==7
	"Pubblicare sessione";
#elif L==8
	"Publikuje zaproszenie";
#elif L==9
	"Publicar chamada";
#endif

const char *Txt_QR_code =
#if   L==1
	"Codi QR";
#elif L==2
	"QR-Code";
#elif L==3
	"QR code";
#elif L==4
	"C&oacute;digo QR";
#elif L==5
	"QR Code";
#elif L==6
	"C&oacute;digo QR";	// Okoteve traduccin
#elif L==7
	"Codice QR";
#elif L==8
	"QR Code";
#elif L==9
	"C&oacute;digo QR";
#endif

const char *Txt_Question =
#if   L==1
	"Pregunta";	// Necessita traduccio
#elif L==2
	"Frage";
#elif L==3
	"Question";
#elif L==4
	"Pregunta";
#elif L==5
	"Question";
#elif L==6
	"Pregunta";	// Okoteve traduccin
#elif L==7
	"Domanda";
#elif L==8
	"Pytanie";
#elif L==9
	"Quest&atilde;o";
#endif

const char *Txt_Question_code_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Pregunta amb codi %ld";
#elif L==2
	"Frage Code %ld";
#elif L==3
	"Code %ld";
#elif L==4
	"Pregunta con c&oacute;digo %ld";
#elif L==5
	"Code de question %ld";
#elif L==6
	"Pregunta con c&oacute;digo %ld";	// Okoteve traduccin
#elif L==7
	"Codice di domanda %ld";
#elif L==8
	"Kodeks pytanie %ld";
#elif L==9
	"Quest&atilde;o com c&oacute;digo %ld";
#endif

const char *Txt_Question_modified =
#if   L==1
	"Pregunta modificada.";
#elif L==2
	"Frage modifiziert.";
#elif L==3
	"Question modified.";
#elif L==4
	"Pregunta modificada.";
#elif L==5
	"Question modifi&eacute;e.";
#elif L==6
	"Pregunta modificada.";		// Okoteve traduccin
#elif L==7
	"Domanda modificata.";
#elif L==8
	"Pytanie zmodyfikowane.";
#elif L==9
	"Quest&atilde;o modificada.";
#endif

const char *Txt_Question_removed =
#if   L==1
	"Pregunta eliminada.";
#elif L==2
	"Frage entfernt.";
#elif L==3
	"Question removed.";
#elif L==4
	"Pregunta eliminada.";
#elif L==5
	"Question supprim&eacute;e.";
#elif L==6
	"Pregunta eliminada.";		// Okoteve traduccin
#elif L==7
	"Domanda rimossa.";
#elif L==8
	"Pytanie usuniete.";
#elif L==9
	"Quest&atilde;o removioda.";
#endif

const char *Txt_Questions =
#if   L==1
	"Preguntes";
#elif L==2
	"Fragen";
#elif L==3
	"Questions";
#elif L==4
	"Preguntas";
#elif L==5
	"Questions";
#elif L==6
	"Preguntas";	// Okoteve traduccin
#elif L==7
	"Domande";
#elif L==8
	"Pyta&nacute;";
#elif L==9
	"Quest&otilde;es";
#endif

const char *Txt_Questions_and_problems =
#if   L==1
	"Consultes i problemes";
#elif L==2
	"Fragen und Probleme";
#elif L==3
	"Questions and problems";
#elif L==4
	"Consultas y problemas";
#elif L==5
	"Questions et probl&egrave;mes";
#elif L==6
	"Consultas y problemas";		// Okoteve traduccin
#elif L==7
	"Domande e problemi";
#elif L==8
	"Pytania i problemy";
#elif L==9
	"Quest&otilde;es e problemas";
#endif

const char *Txt_Quota_exceeded =
#if   L==1
	"&iexcl;Cuota excedida!";	// Necessita traduccio
#elif L==2
	"Kontingent &uuml;berschritten!";
#elif L==3
	"Quota exceeded!";
#elif L==4
	"&iexcl;Cuota excedida!";
#elif L==5
	"Quota d&eacute;pass&eacute;!";
#elif L==6
	"&iexcl;Cuota excedida!";	// Okoteve traduccin
#elif L==7
	"Quota superata!";
#elif L==8
	"Przekroczono przydzial!";
#elif L==9
	"Quota excedida!";
#endif

const char *Txt_ReaBRders =
#if   L==1
	"Lec-<br />tores";	// Necessita traduccio
#elif L==2
	"Leser";
#elif L==3
	"Rea-<br />ders";
#elif L==4
	"Lec-<br />tores";
#elif L==5
	"Lec-<br />teurs";
#elif L==6
	"Lec-<br />tores";	// Okoteve traduccin
#elif L==7
	"Le-<br />ttori";
#elif L==8
	"Czytel-<br />nik&oacute;w";
#elif L==9
	"Lei-<br />tores";
#endif

const char *Txt_Real_number_between_A_and_B_1 =
#if   L==1
	"Nombre real entre";
#elif L==2
	"Reelle Zahl zwischen";
#elif L==3
	"Real number between";
#elif L==4
	"N&ordm; real entre";
#elif L==5
	"Nombre r&eacute;el entre";
#elif L==6
	"N&ordm; real entre";	// Okoteve traduccin
#elif L==7
	"Numero reale compreso tra";
#elif L==8
	"Prawdziwa liczbe miedzy";
#elif L==9
	"N&ordm; real entre";
#endif

const char *Txt_Real_number_between_A_and_B_2 =	// Must start by space
#if   L==1
	" i";
#elif L==2
	" und";
#elif L==3
	" and";
#elif L==4
	" y";
#elif L==5
	" et";
#elif L==6
	" y";	// Okoteve traduccin
#elif L==7
	" e";
#elif L==8
	" i";
#elif L==9
	" e";
#endif

const char *Txt_Recommended_aspect_ratio =
#if   L==1
	"Relaci&oacute; d'aspecte recomanada";
#elif L==2
	"Empfohlene Seitenverh&auml;ltnisr";
#elif L==3
	"Recommended aspect ratio";
#elif L==4
	"Relaci&oacute;n de aspecto recomendada";
#elif L==5
	"Rapport d'aspect recommand&eacute;";
#elif L==6
	"Relaci&oacute;n de aspecto recomendada";	// Okoteve traduccin
#elif L==7
	"Rapporto aspetto consigliato";
#elif L==8
	"Zalecane proporcje obrazu";
#elif L==9
	"Propor&ccedil;&atilde;o recomendada";
#endif

const char *Txt_Recommended_resolution =
#if   L==1
	"Resoluci&oacute; recomanada";
#elif L==2
	"Empfohlene Aufl&ouml;sung";
#elif L==3
	"Recommended resolution";
#elif L==4
	"Resoluci&oacute;n recomendada";
#elif L==5
	"R&eacute;solution recommand&eacute;e";
#elif L==6
	"Resoluci&oacute;n recomendada";	// Okoteve traduccin
#elif L==7
	"Risoluzione consigliata";
#elif L==8
	"Zalecana rozdzielczo&sacute;&cacute;";
#elif L==9
	"Resolu&ccedil;&atilde;o recomendada";
#endif

const char *Txt_Record_card_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Se ha eliminado la ficha de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Record card of <strong>%s</strong> has been removed.";	// Need bersetzung
#elif L==3
	"Record card of <strong>%s</strong> has been removed.";
#elif L==4
	"Se ha eliminado la ficha de <strong>%s</strong>.";
#elif L==5
	"Record card of <strong>%s</strong> has been removed.";	// Besoin de traduction
#elif L==6
	"Se ha eliminado la ficha de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"La scheda de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==8
	"Karta Rekord <strong>%s</strong> zostala usunieta.";
#elif L==9
	"Record card of <strong>%s</strong> has been removed.";	// Necessita de traduo
#endif

const char *Txt_record_cards_per_page =
#if   L==1
	"fichas por p&aacute;gina";		// Necessita traduccio
#elif L==2
	"record cards per page";		// Need bersetzung
#elif L==3
	"record cards per page";
#elif L==4
	"fichas por p&aacute;gina";
#elif L==5
	"record cards per page";		// Besoin de traduction
#elif L==6
	"fichas por p&aacute;gina";		// Okoteve traduccin
#elif L==7
	"scheda per pagina";
#elif L==8
	"karty zapisu na stronie";
#elif L==9
	"record cards per page";		// Necessita de traduo
#endif

const char *Txt_Record_field_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Campo de ficha <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2
	"Datenfeld <strong>%s</strong> entfernt.";
#elif L==3
	"Record field <strong>%s</strong> removed.";
#elif L==4
	"Campo de ficha <strong>%s</strong> eliminado.";
#elif L==5
	"Record field <strong>%s</strong> removed.";	// Besoin de traduction
#elif L==6
	"Campo de ficha <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Campo scheda <strong>%s</strong> rimosso.";
#elif L==8
	"Pole rekordu <strong>%s</strong> usuniete.";
#elif L==9
	"Record field <strong>%s</strong> removed.";	// Necessita de traduo
#endif

const char *Txt_RECORD_FIELD_VISIBILITY_MENU[Rec_NUM_TYPES_VISIBILITY] =
   {
#if   L==1
	 "Oculto"	// Necessita traduccio
#elif L==2
	 "Versteckt"
#elif L==3
	 "Hidden"
#elif L==4
	 "Oculto"
#elif L==5
	 "Cach&eacute;"
#elif L==6
	 "Oculto"	// Okoteve traduccin
#elif L==7
	 "Nascosto"
#elif L==8
	 "Ukryty"
#elif L==9
	 "Oculto"
#endif
	 ,
#if   L==1
	 "Visible pero no editable"	// Necessita traduccio
#elif L==2
	 "Sichtbar, aber nicht bearbeitet werden"
#elif L==3
	 "Visible but not editable"
#elif L==4
	 "Visible pero no editable"
#elif L==5
	 "Visible mais non modifiable"
#elif L==6
	 "Visible pero no editable"	// Okoteve traduccin
#elif L==7
	 "Visibile ma non modificabile"
#elif L==8
	 "Widoczne, ale nie mozna edytowa&cacute;"
#elif L==9
	 "Vis&iacute;vel, mas n&atilde;o edit&aacute;vel"
#endif
	 ,
#if   L==1
	 "Visible y editable"	// Necessita traduccio
#elif L==2
	 "Sichtbar und bearbeitet werde"
#elif L==3
	 "Visible and editable"
#elif L==4
	 "Visible y editable"
#elif L==5
	 "Visible et modifiable"
#elif L==6
	 "Visible y editable"	// Okoteve traduccin
#elif L==7
	 "Visibile e modificabile"
#elif L==8
	 "Widzialne i mozna edytowa&cacute;"
#elif L==9
	 "Vis&iacute;vel e edit&aacute;vel"
#endif
	};

const char *Txt_RECORD_FIELD_VISIBILITY_MSG[Rec_NUM_TYPES_VISIBILITY] =	// Warning: it is very important to include %s in the following sentences
	{
#if   L==1
	"A partir de ahora los estudiantes"
	" no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."// Necessita traduccio
#elif L==2
	"Von nun an k&ouml;nnen die Studenten"
	" nicht das Feld <strong>%s</strong> der Profile."
#elif L==3
	"From now on students"
	" can not see the field <strong>%s</strong> of the records."
#elif L==4
	"A partir de ahora los estudiantes"
	" no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."
#elif L==5
	"A partir de maintenant, les &eacute;tudiants"
	" ne peuvent pas voir le champ <strong>%s</strong> de les cartes."
#elif L==6
	"A partir de ahora los estudiantes"
	" no podr&aacute;n ver el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==7
	"Da ora in poi gli studenti"
	" non possono visualizzare il campo <strong>%s</strong> delle schede."
#elif L==8
	"Od teraz studenci"
	" nie widac pola <strong>%s</strong> zapis&oacute;w."
#elif L==9
	"De agora em diante,"
		 " os alunos n&atilde;o podem ver o campo <strong>%s</strong> dos cart&otilde;es."
#endif
	,
#if   L==1
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."// Necessita traduccio
#elif L==2
	"Von nun an k&ouml;nnen sich Studenten,"
	" aber nicht bearbeiten das Feld <strong>%s</strong> der Profile."
#elif L==3
	"From now on students"
	" can view but not edit the field <strong>%s</strong> of the records."
#elif L==4
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."
#elif L==5
	"A partir de maintenant, les &eacute;tudiants"
	" peuvent consulter mais pas modifier le champ <strong>%s</strong> de les cartes."
#elif L==6
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver pero no editar el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==7
	"Da ora in poi gli studenti"
	" possono visualizzare ma non modificare il campo <strong>%s</strong> delle schede."
#elif L==8
	"Od teraz studenci"
	" moga przegladac, ale nie edytowa&cacute; pole <strong>%s</strong> zapis&oacute;w."
#elif L==9
	"De agora em diante,"
		 " os alunos podem visualizar, mas n&atilde;o editar o campo <strong>%s</strong> dos cart&otilde;es."
#endif
	,
#if   L==1
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."// Necessita traduccio
#elif L==2
	"Von nun an k&ouml;nnen die Studenten"
	" anzuzeigen und zu bearbeiten das Feld <strong>%s</strong> der Profile."
#elif L==3
	"From now on students"
	" can view and edit the field <strong>%s</strong> of the records."
#elif L==4
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."
#elif L==5
	"A partir de maintenant,"
	" les &eacute;tudiants peuvent consulter et modifier le champ <strong>%s</strong> de les cartes."
#elif L==6
	"A partir de ahora los estudiantes"
	" podr&aacute;n ver y editar el campo <strong>%s</strong> de las fichas."// Okoteve traduccin
#elif L==7
	"Da ora in poi"
	" gli studenti possono visualizzare e modificare il campo <strong>%s</strong> delle schede."
#elif L==8
	"Od teraz studenci"
	" moga ogladac i edytowa&cacute; pola <strong>%s</strong> zapis&oacute;w."
#elif L==9
	"De agora em diante,"
		 " os alunos podem visualizar e editar o campo <strong>%s</strong> dos cart&otilde;es."
#endif
	};

const char *Txt_RECORD_FIELD_VISIBILITY_RECORD[Rec_NUM_TYPES_VISIBILITY] =
   {
#if   L==1
	 "oculto"	// Necessita traduccio
#elif L==2
	 "versteckt"
#elif L==3
	 "hidden"
#elif L==4
	 "oculto"
#elif L==5
	 "cach&eacute;"
#elif L==6
	 "oculto"	// Okoteve traduccin
#elif L==7
	 "nascosto"
#elif L==8
	 "ukryty"
#elif L==9
	 "oculto"
#endif
	 ,
#if   L==1
	 "visible"	// Necessita traduccio
#elif L==2
	 "sichtbar"
#elif L==3
	 "visible"
#elif L==4
	 "visible"
#elif L==5
	 "visible"
#elif L==6
	 "visible"	// Okoteve traduccin
#elif L==7
	 "visibile"
#elif L==8
	 "widoczny"
#elif L==9
	 "vis&iacute;vel"
#endif
	 ,
#if   L==1
	 "editable"	// Necessita traduccio
#elif L==2
	 "bearbeitet werde"
#elif L==3
	 "editable"
#elif L==4
	 "editable"
#elif L==5
	 "modifiable"
#elif L==6
	 "editable"	// Okoteve traduccin
#elif L==7
	 "modificabile"
#elif L==8
	 "edytowalne"
#elif L==9
	 "edit&aacute;vel"
#endif
	};

const char *Txt_Record_fields =
#if   L==1
	"Camps de fitxes";
#elif L==2
	"Record fields";	// Need bersetzung
#elif L==3
	"Record fields";
#elif L==4
	"Campos de fichas";
#elif L==5
	"Record fields";	// Besoin de traduction
#elif L==6
	"Campos de fichas";	// Okoteve traduccin
#elif L==7
	"Campo schede";
#elif L==8
	"Rekord w dziedzinie";
#elif L==9
	"Record fields";	// Necessita de traduo
#endif

const char *Txt_Recipients =
#if   L==1
	"Destinatarios";	// Necessita traduccio
#elif L==2
	"Empf&auml;nger";
#elif L==3
	"Recipients";
#elif L==4
	"Destinatarios";
#elif L==5
	"Destinataires";
#elif L==6
	"Destinatarios";	// Okoteve traduccin
#elif L==7
	"Destinatari";
#elif L==8
	"Odbiorcy";
#elif L==9
	"Destinat&aacute;rios";
#endif

const char *Txt_Register =
#if   L==1
	"Inscriure";
#elif L==2
	"Anmeldung";
#elif L==3
	"Register";
#elif L==4
	"Inscribir";
#elif L==5
	"Inscrire";
#elif L==6
	"Inscribir";	// Okoteve traduccin
#elif L==7
	"Registrare";
#elif L==8
	"Zarejestruj sie";
#elif L==9
	"Registrar";
#endif

const char *Txt_Register_students =
#if   L==1
	"Inscriure estudiants";
#elif L==2
	"Anmeldung Studenten";
#elif L==3
	"Register students";
#elif L==4
	"Inscribir estudiantes";
#elif L==5
	"Inscrire &eacute;tudiants";
#elif L==6
	"Inscribir estudiantes";	// Okoteve traduccin
#elif L==7
	"Registrare studenti";
#elif L==8
	"Rejestracja student&oacute;w";
#elif L==9
	"Registrar estudantes";
#endif

const char *Txt_Register_students_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Inscriure estudiants en l'assignatura <strong>%s</strong>";
#elif L==2
	"Anmeldung Studenten im Kurs <strong>%s</strong>";
#elif L==3
	"Register students in the course <strong>%s</strong>";
#elif L==4
	"Inscribir estudiantes en la asignatura <strong>%s</strong>";
#elif L==5
	"Inscrire &eacute;tudiants dans la mati&egrave;re <strong>%s</strong>";
#elif L==6
	"Inscribir estudiantes en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Registrare studenti nel corso <strong>%s</strong>";
#elif L==8
	"Rejestracja student&oacute;w w kurs <strong>%s</strong>";
#elif L==9
	"Registrar estudantes na disciplina <strong>%s</strong>";
#endif

const char *Txt_Register_the_users_indicated_in_step_1 =
#if   L==1
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";	// Necessita traduccio
#elif L==2
	"Anmeldung Sie die Benutzer, die auf der Liste";
#elif L==3
	"Add (register the users indicated in step 1)";
#elif L==4
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";
#elif L==5
	"Inscrire les utilisateurs qui sont sur la liste";
#elif L==6
	"A&ntilde;adir (inscribir los usuarios indicados en el paso 1)";	// Okoteve traduccin
#elif L==7
	"Registrare gli utenti che sono in lista";
#elif L==8
	"Rejestracja uzytkownik&oacute;w, kt&oacute;rzy sa na liscie";
#elif L==9
	"Registrar os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Register_the_users_indicated_in_step_1_and_remove_the_users_not_indicated =
#if   L==1
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";	// Necessita traduccio
#elif L==2
	"Entfernen Sie die Benutzer, die <strong>nicht</strong> auf der Liste,"
	" und Anmeldung Sie die Benutzer, die auf der Liste";
#elif L==3
	"Update (register the users indicated in step 1"
	" and remove the users <strong>not</strong> indicated)";
#elif L==4
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";
#elif L==5
	"Enlever les utilisateurs qui <strong>ne</strong> sont <strong>pas</strong> sur la liste,"
	" et inscrire les utilisateurs qui sont sur la liste";
#elif L==6
	"Actualizar (inscribir los usuarios indicados en el paso 1"
	" y eliminar los <strong>no</strong> indicados)";	// Okoteve traduccin
#elif L==7
	"Rimuovere gli utenti che <strong>non</strong> sono sulla lista,"
	" e registrare gli utenti che invece lo sono";
#elif L==8
	"Usun u&zdot;ytkownikami, kt&oacute;rzy sa <strong>nie</strong> na liscie,"
	"i  zarejestrowac uzytkownik&oacute;w, kt&oacute;rzy sa na liscie";
#elif L==9
	"Tirar os utilizadores que <strong>n&atilde;o</strong> est&atilde;o na lista,"
	" e registrar os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Register_teacher =
#if   L==1
	"Inscriure professor/a";
#elif L==2
	"Anmeldung Lehrkraft";
#elif L==3
	"Register teacher";
#elif L==4
	"Inscribir profesor/a";
#elif L==5
	"Inscrire enseignant/e";
#elif L==6
	"Inscribir profesor/a";	// Okoteve traduccin
#elif L==7
	"Registrare professore/ssa";
#elif L==8
	"Rejestracja nauczyciel";
#elif L==9
	"Registrar professor/a";
#endif

const char *Txt_Register_user_IN_A_COURSE_OR_DEGREE =
#if   L==1
	"Inscriure usuari";
#elif L==2
	"Anmeldung Benutzer";
#elif L==3
	"Register user";
#elif L==4
	"Inscribir usuario";
#elif L==5
	"Inscrire utilisateur";
#elif L==6
	"Inscribir usuario";	// Okoteve traduccin
#elif L==7
	"Registrare utente";
#elif L==8
	"Rejestracja u&zdot;ytkownika";
#elif L==9
	"Registrar utilizador";
#endif

const char *Txt_Register_me_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Inscriureme en <strong>%s</strong>";
#elif L==2
	"Anmeldung Sie mir im <strong>%s</strong>";
#elif L==3
	"Register me in <strong>%s</strong>";
#elif L==4
	"Inscribirme en <strong>%s</strong>";
#elif L==5
	"Inscrire-moi dans <strong>%s</strong>";
#elif L==6
	"Inscribirme en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Registrami in <strong>%s</strong>";
#elif L==8
	"Rejestracja mnie w <strong>%s</strong>";
#elif L==9
	"Registrar me em <strong>%s</strong>";
#endif

const char *Txt_Register_USER_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Inscriure en l'assignatura <strong>%s</strong>";
#elif L==2
	"Anmeldung im Kurs <strong>%s</strong>";
#elif L==3
	"Register in the course <strong>%s</strong>";
#elif L==4
	"Inscribir en la asignatura <strong>%s</strong>";
#elif L==5
	"Inscrire dans la mati&egrave;re <strong>%s</strong>";
#elif L==6
	"Inscribir en la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Registrare nel corso <strong>%s</strong>";
#elif L==8
	"Rejestracja w kurs <strong>%s</strong>";
#elif L==9
	"Registrar na disciplina <strong>%s</strong>";
#endif

const char *Txt_Register_USER_as_an_administrator_of_the_centre_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Inscriure com a administrador"
	" del centre <strong>%s</strong>";
#elif L==2
	"Anmeldung als Administrator"
	" des Lehrinstitut <strong>%s</strong>";
#elif L==3
	"Register as an administrator"
	" of the centre <strong>%s</strong>";
#elif L==4
	"Inscribir como administrador"
	" del centro <strong>%s</strong>";
#elif L==5
	"Inscrire en tant qu'administrateur"
	" du centre <strong>%s</strong>";
#elif L==6
	"Inscribir como administrador"
	" del centro <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Registrare come amministratore"
	" del centro <strong>%s</strong>";
#elif L==8
	"Rejestracja jako administrator"
	" centrum <strong>%s</strong>";
#elif L==9
	"Registrar como administrador"
	" do centro <strong>%s</strong>";
#endif

const char *Txt_Register_USER_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Inscriure com a administrador"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==2
	"Anmeldung als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==3
	"Register as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==4
	"Inscribir como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==5
	"Inscrire en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==6
	"Inscribir como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Registrare come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==8
	"Rejestracja jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==9
	"Registrar como administrador"
	" da titula&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Register_USER_as_an_administrator_of_the_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Inscriure com a administrador"
	" de la instituci&oacute; <strong>%s</strong>";
#elif L==2
	"Anmeldung als Administrator"
	" des Hochschule <strong>%s</strong>";
#elif L==3
	"Register as an administrator"
	" of the institution <strong>%s</strong>";
#elif L==4
	"Inscribir como administrador"
	" de la instituci&oacute;n <strong>%s</strong>";
#elif L==5
	"Inscrire en tant qu'administrateur"
	" du &eacute;tablissement <strong>%s</strong>";
#elif L==6
	"Inscribir como administrador"
	" de la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Registrare come amministratore"
	" della istituzione <strong>%s</strong>";
#elif L==8
	"Rejestracja jako administrator"
	" instytucje <strong>%s</strong>";
#elif L==9
	"Registrar como administrador"
	" da institui&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Reject =
#if   L==1
	"Rebutjar";
#elif L==2
	"Ablehnen";
#elif L==3
	"Reject";
#elif L==4
	"Rechazar";
#elif L==5
	"Rejeter";
#elif L==6
	"Rechazar";	// Okoteve traduccin
#elif L==7
	"Rifiutare";
#elif L==8
	"Odrzuca&cacute;";
#elif L==9
	"Rejeitar";
#endif

const char *Txt_Removal_not_allowed =
#if   L==1
	"Eliminaci&oacute; no perm&egrave;s";
#elif L==2
	"Entfernen nicht erlaubt";
#elif L==3
	"Removal not allowed";
#elif L==4
	"Eliminaci&oacute;n no permitida";
#elif L==5
	"Suppression pas autoris&eacute;";
#elif L==6
	"Eliminaci&oacute;n no permitida";	// Okoteve traduccin
#elif L==7
	"Rimozione non ammessi";
#elif L==8
	"Usuwanie nie wolno";
#elif L==9
	"Remo&ccedil;&atilde;o n&atilde;o permitida";
#endif

const char *Txt_Remove =
#if   L==1
	"Eliminar";
#elif L==2
	"Entfernen";
#elif L==3
	"Remove";
#elif L==4
	"Eliminar";
#elif L==5
	"Supprimer";
#elif L==6
	"Mbogue";
#elif L==7
	"Rimuovere";
#elif L==8
	"Usu&nacute;";
#elif L==9
	"Remover";
#endif

const char *Txt_Remove_account =
#if   L==1
	"Eliminar compte";
#elif L==2
	"Benutzerkonto entfernen";
#elif L==3
	"Remove account";
#elif L==4
	"Eliminar cuenta";
#elif L==5
	"Supprimer compte";
#elif L==6
	"Eliminar cuenta";	// Okoteve traduccin
#elif L==7
	"Rimuovere account";
#elif L==8
	"Usu&nacute; konto";
#elif L==9
	"Remover conta";
#endif

const char *Txt_Remove_all_students =
#if   L==1
	"Eliminar tots estudiants";
#elif L==2
	"Alle Studenten entfernen";
#elif L==3
	"Remove all students";
#elif L==4
	"Eliminar todos los estudiantes";
#elif L==5
	"Enlever tous &eacute;tudiants";
#elif L==6
	"Eliminar todos los estudiantes";	// Okoteve traduccin
#elif L==7
	"Rimuovere tutti gli stud.";
#elif L==8
	"Usu&nacute; wszystkich student&oacute;w";
#elif L==9
	"Tirar todos estudantes";
#endif

const char *Txt_Remove_assignment =
#if   L==1
	"Eliminar activitat";
#elif L==2
	"Aufgabe entfernen";
#elif L==3
	"Remove assignment";
#elif L==4
	"Eliminar actividad";
#elif L==5
	"Supprimer activit&eacute;";
#elif L==6
	"Eliminar actividad";	// Okoteve traduccin
#elif L==7
	"Rimuovere attivit&agrave;";
#elif L==8
	"Usu&nacute; przypisanie";
#elif L==9
	"Remover atividade";
#endif

const char *Txt_Remove_event =
#if   L==1
	"Eliminar evento";	// Necessita traduccio
#elif L==2
	"Ereignis Entfernen";
#elif L==3
	"Remove event";
#elif L==4
	"Eliminar evento";
#elif L==5
	"Supprimer &eacute;v&eacute;nement";
#elif L==6
	"Eliminar evento";	// Okoteve traduccin
#elif L==7
	"Rimuovere evento";
#elif L==8
	"Usu&nacute; wydarzenie";
#elif L==9
	"Remover evento";
#endif

const char *Txt_Remove_file =
#if   L==1
	"Eliminar fitxer";
#elif L==2
	"Datei Entfernen";
#elif L==3
	"Remove file";
#elif L==4
	"Eliminar archivo";
#elif L==5
	"Supprimer fichier";
#elif L==6
	"Eliminar archivo";	// Okoteve traduccin
#elif L==7
	"Rimuovere file";
#elif L==8
	"Usu&nacute; plik";
#elif L==9
	"Remover arquivo";
#endif

const char *Txt_Remove_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Eliminar %s";
#elif L==2
	"Datei %s";
#elif L==3
	"Remove %s";
#elif L==4
	"Eliminar %s";
#elif L==5
	"Supprimer %s";
#elif L==6
	"Eliminar %s";	// Okoteve traduccin
#elif L==7
	"Rimuovere %s";
#elif L==8
	"Usu&nacute; %s";
#elif L==9
	"Remover %s";
#endif

const char *Txt_Remove_files_older_than_PART_1_OF_2 =
#if   L==1
	"Eliminar els arxius amb m&eacute;s de";
#elif L==2
	"Dateien zu entfernen, die &auml;lter als";
#elif L==3
	"Remove files older than";
#elif L==4
	"Eliminar los archivos con m&aacute;s de";
#elif L==5
	"Supprimer les fichiers de plus de";
#elif L==6
	"Eliminar los archivos con m&aacute;s de";	// Okoteve traduccin
#elif L==7
	"Rimuovere i file pi&ugrave; vecchi di";
#elif L==8
	"Remove files older than";			// Potrzebujesz tlumaczenie
#elif L==9
	"Remover arquivos com mais de";
#endif

const char *Txt_Remove_files_older_than_PART_2_OF_2 =
#if   L==1
	"mesos d'antiguitat.";
#elif L==2
	"Monate.";
#elif L==3
	"months.";
#elif L==4
	"meses de antig&uuml;edad.";
#elif L==5
	"mois.";
#elif L==6
	"meses de antig&uuml;edad.";	// Okoteve traduccin
#elif L==7
	"mesi.";
#elif L==8
	"months.";			// Potrzebujesz tlumaczenie
#elif L==9
	"meses.";
#endif

const char *Txt_Remove_folder =
#if   L==1
	"Eliminar carpeta";
#elif L==2
	"Verzeichnis entfernen";
#elif L==3
	"Remove folder";
#elif L==4
	"Eliminar carpeta";
#elif L==5
	"Supprimer r&eacute;pertoire";
#elif L==6
	"Eliminar carpeta";	// Okoteve traduccin
#elif L==7
	"Rimuovere cartella";
#elif L==8
	"Usu&nacute; folder";
#elif L==9
	"Remover diret&oacute;rio";
#endif

const char *Txt_Remove_folder_X = // Warning: it is very important to include %s in the following sentences
#if   L==1
	"Eliminar carpeta %s";
#elif L==2
	"Verzeichnis entfernen %s";
#elif L==3
	"Remove folder %s";
#elif L==4
	"Eliminar carpeta %s";
#elif L==5
	"Supprimer r&eacute;pertoire %s";
#elif L==6
	"Eliminar carpeta %s";	// Okoteve traduccin
#elif L==7
	"Rimuovere cartella %s";
#elif L==8
	"Usu&nacute; folder %s";
#elif L==9
	"Remover diret&oacute;rio %s";
#endif

const char *Txt_Remove_group =
#if   L==1
	"Eliminar grupo";	// Necessita traduccio
#elif L==2
	"Entfernen Gruppe";
#elif L==3
	"Remove group";
#elif L==4
	"Eliminar grupo";
#elif L==5
	"Supprimer groupe";
#elif L==6
	"Eliminar grupo";	// Okoteve traduccin
#elif L==7
	"Rimuovere gruppo";
#elif L==8
	"Usu&nacute; gruper";
#elif L==9
	"Remover grupo";
#endif

const char *Txt_Remove_link =
#if   L==1
	"Eliminar enlla&ccedil;";
#elif L==2
	"Entfernen Link";
#elif L==3
	"Remove link";
#elif L==4
	"Eliminar enlace";
#elif L==5
	"Supprimer lien";
#elif L==6
	"Eliminar enlace";	// Okoteve traduccin
#elif L==7
	"Rimuovere link";
#elif L==8
	"Usu&nacute; link";
#elif L==9
	"Remover liga&ccedil;&atilde;o";
#endif

const char *Txt_Remove_logo =
#if   L==1
	"Eliminar logo";
#elif L==2
	"Entfernen Logo";
#elif L==3
	"Remove logo";
#elif L==4
	"Eliminar logo";
#elif L==5
	"Supprimer logo";
#elif L==6
	"Eliminar logo";	// Okoteve traduccin
#elif L==7
	"Rimuovere logo";
#elif L==8
	"Usu&nacute; logo";
#elif L==9
	"Remover logotipo";
#endif

const char *Txt_Remove_me =
#if   L==1
	"Eliminarme";
#elif L==2
	"Entfernen Sie";
#elif L==3
	"Remove me";
#elif L==4
	"Eliminarme";
#elif L==5
	"Enlever-moi";
#elif L==6
	"Eliminarme";	// Okoteve traduccin
#elif L==7
	"Rimuovimi";
#elif L==8
	"Usu&ntilde;";
#elif L==9
	"Tirar me";
#endif

const char *Txt_Remove_me_as_an_administrator =
#if   L==1
	"Eliminarme com a administrador";
#elif L==2
	"Entfernen Sie mir als Administrator";
#elif L==3
	"Remove me as an administrator";
#elif L==4
	"Eliminarme como administrador";
#elif L==5
	"Enlever-moi en tant qu'administrateur";
#elif L==6
	"Eliminarme como administrador";	// Okoteve traduccin
#elif L==7
	"Rimuovimi come amministratore";
#elif L==8
	"Usu&nacute; mnie jako administrator";
#elif L==9
	"Tirar me como administrador";
#endif

const char *Txt_Remove_me_as_an_administrator_of_the_centre_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Eliminarme com a administrador"
	" del centre <strong>%s</strong>";
#elif L==2
	"Entfernen Sie mir als Administrator"
	" des Lehrinstitute <strong>%s</strong>";
#elif L==3
	"Remove me as an administrator"
	" of the centre <strong>%s</strong>";
#elif L==4
	"Eliminarme como administrador"
	" del centro <strong>%s</strong>";
#elif L==5
	"Enlever-moi en tant qu'administrateur"
	" du centre <strong>%s</strong>";
#elif L==6
	"Eliminarme como administrador"
	" del centro <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Rimuovimi come amministratore"
	" del centro <strong>%s</strong>";
#elif L==8
	"Usu&nacute; mnie jako administrator"
	" centrum <strong>%s</strong>";
#elif L==9
	"Tirar me como administrador"
	" do centro <strong>%s</strong>";
#endif

const char *Txt_Remove_me_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Eliminarme com a administrador"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==2
	"Entfernen Sie mir als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==3
	"Remove me as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==4
	"Eliminarme como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==5
	"Enlever-moi en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==6
	"Eliminarme como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Rimuovimi come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==8
	"Usu&nacute; mnie jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==9
	"Tirar me como administrador"
	" da titula&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Remove_me_as_an_administrator_of_the_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Eliminarme com a administrador"
	" de la instituci&oacute; <strong>%s</strong>";
#elif L==2
	"Entfernen Sie mir als Administrator"
	" des Hochschule <strong>%s</strong>";
#elif L==3
	"Remove me as an administrator"
	" of the institution <strong>%s</strong>";
#elif L==4
	"Eliminarme como administrador"
	" de la instituci&oacute;n <strong>%s</strong>";
#elif L==5
	"Enlever-moi en tant qu'administrateur"
	" du &eacute;tablissement <strong>%s</strong>";
#elif L==6
	"Eliminarme como administrador"
	" de la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Rimuovimi come amministratore"
	" della istituzione <strong>%s</strong>";
#elif L==8
	"Usu&nacute; mnie jako administrator"
	" instytucji <strong>%s</strong>";
#elif L==9
	"Tirar me como administrador"
	" da institu&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Remove_me_from_this_course =
#if   L==1
	"Eliminarme d'aquesta assignatura";
#elif L==2
	"Entfernen Sie diesen Kurs";
#elif L==3
	"Remove me from this course";
#elif L==4
	"Eliminarme de esta asignatura";
#elif L==5
	"Enlever-moi de cette mati&egrave;re";
#elif L==6
	"Eliminarme de esta asignatura";	// Okoteve traduccin
#elif L==7
	"Rimuovimi da questo corso";
#elif L==8
	"Usu&ntilde; z kursu";
#elif L==9
	"Tirar me desta disciplina";
#endif

const char *Txt_Remove_me_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Eliminarme de l'assignatura <strong>%s</strong>";
#elif L==2
	"Entfernen Sie mir aus dem Kurs <strong>%s</strong>";
#elif L==3
	"Remove me from the course <strong>%s</strong>";
#elif L==4
	"Eliminarme de la asignatura <strong>%s</strong>";
#elif L==5
	"Enlever-moi de la mati&egrave;re <strong>%s</strong>";
#elif L==6
	"Eliminarme de la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Rimuovimi dal corso <strong>%s</strong>";
#elif L==8
	"Usu&nacute; mnie w kurs <strong>%s</strong>";
#elif L==9
	"Tirar me da disciplina <strong>%s</strong>";
#endif

const char *Txt_Remove_old_files =
#if   L==1
	"Eliminar arxius antics";
#elif L==2
	"Entfernen Sie alte Dateien";
#elif L==3
	"Remove old files";
#elif L==4
	"Eliminar archivos antiguos";
#elif L==5
	"Supprimer anciens fichiers";
#elif L==6
	"Eliminar archivos antiguos";	// Okoteve traduccin
#elif L==7
	"Rimuovere vecchi file";
#elif L==8
	"Usu&nacute; stare pliki";
#elif L==9
	"Remover arquivos antigos";
#endif

const char *Txt_Remove_photo =
#if   L==1
	"Eliminar foto";
#elif L==2
	"Entfernen Foto";
#elif L==3
	"Remove photo";
#elif L==4
	"Eliminar foto";
#elif L==5
	"Supprimer photo";
#elif L==6
	"Eliminar foto";	// Okoteve traduccin
#elif L==7
	"Rimuovere foto";
#elif L==8
	"Usu&nacute; zdjecie";
#elif L==9
	"Remover foto";
#endif

const char *Txt_Remove_question =
#if   L==1
	"Eliminar pregunta";
#elif L==2
	"Entfernen Frage";
#elif L==3
	"Remove question";
#elif L==4
	"Eliminar pregunta";
#elif L==5
	"Supprimer question";
#elif L==6
	"Eliminar pregunta";	// Okoteve traduccin
#elif L==7
	"Rimuovere domanda";
#elif L==8
	"Usu&nacute; pytanie";
#elif L==9
	"Remover quest&atilde;o";
#endif

const char *Txt_Remove_record_field =
#if   L==1
	"Eliminar campo de ficha";	// Necessita traduccio
#elif L==2
	"Entfernen Datenfeld";
#elif L==3
	"Remove record field";
#elif L==4
	"Eliminar campo de ficha";
#elif L==5
	"Supprimer champ de carte";
#elif L==6
	"Eliminar campo de ficha";	// Okoteve traduccin
#elif L==7
	"Rimuovere campo di scheda";
#elif L==8
	"Usu&nacute; pola rekordu";
#elif L==9
	"Remover campo de cart&atilde;o";
#endif

const char *Txt_Remove_survey =
#if   L==1
	"Eliminar encuesta";	// Necessita traduccio
#elif L==2
	"Entfernen Umfrage";
#elif L==3
	"Remove survey";
#elif L==4
	"Eliminar encuesta";
#elif L==5
	"Supprimer sondage";
#elif L==6
	"Eliminar encuesta";	// Okoteve traduccin
#elif L==7
	"Rimuovere sondaggio";
#elif L==8
	"Usu&nacute; badania";
#elif L==9
	"Remover inqu&eacute;rito";
#endif

const char *Txt_Remove_this_message =
#if   L==1
	"Eliminar aquest missatge";
#elif L==2
	"Diese Nachricht entfernen";
#elif L==3
	"Remove this message";
#elif L==4
	"Eliminar este mensaje";
#elif L==5
	"Supprimer ce message";
#elif L==6
	"Eliminar este mensaje";	// Okoteve traduccin
#elif L==7
	"Rimuovere questo messaggio";
#elif L==8
	"Usu&nacute; t&eogon; wiadomo&sacute;&cacute;";
#elif L==9
	"Remover esta mensagem";
#endif

const char *Txt_Remove_thread =
#if   L==1
	"Eliminar discusi&oacute;n";	// Necessita traduccio
#elif L==2
	"Entfernen Thread";
#elif L==3
	"Remove thread";
#elif L==4
	"Eliminar discusi&oacute;n";
#elif L==5
	"Supprimer fil";
#elif L==6
	"Eliminar discusi&oacute;n";	// Okoteve traduccin
#elif L==7
	"Rimuovere discussione";
#elif L==8
	"Usu&nacute; watkek";
#elif L==9
	"Remover thread";
#endif

const char *Txt_Remove_the_users_not_indicated_in_step_1 =
#if   L==1
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";	// Necessita traduccio
#elif L==2
	"Entfernen Sie die Benutzer, die <strong>nicht</strong> auf der Liste";
#elif L==3
	"Remove the users <strong>not</strong> indicated in step 1";
#elif L==4
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";
#elif L==5
	"Enlever les utilisateurs qui <strong>ne</strong> sont <strong>pas</strong> sur la liste";
#elif L==6
	"Eliminar los usuarios <strong>no</strong> indicados en el paso 1";	// Okoteve traduccin
#elif L==7
	"Rimuovere gli utenti che <strong>non</strong> sono sulla lista";
#elif L==8
	"Usu&nacute; u&zdot;ytkownikami, kt&oacute;rzy sa <strong>nie</strong> na liscie";
#elif L==9
	"Tirar os utilizadores que <strong>n&atilde;o</strong> est&atilde;o na lista";
#endif

const char *Txt_Remove_the_users_indicated_in_step_1 =
#if   L==1
	"Eliminar los usuarios indicados en el paso 1";	// Necessita traduccio
#elif L==2
	"Entfernen Sie die Benutzer, die auf der Liste";
#elif L==3
	"Remove the users indicated in step 1";
#elif L==4
	"Eliminar los usuarios indicados en el paso 1";
#elif L==5
	"Enlever les utilisateurs qui sont sur la liste";
#elif L==6
	"Eliminar los usuarios indicados en el paso 1";	// Okoteve traduccin
#elif L==7
	"Rimuovere gli utenti che sono in lista";
#elif L==8
	"Usu&nacute; u&zdot;ytkownikami, kt&oacute;rzy sa na liscie";
#elif L==9
	"Tirar os utilizadores que est&atilde;o na lista";
#endif

const char *Txt_Remove_these_messages =
#if   L==1
	"Eliminar aquests missatges";
#elif L==2
	"Diese Nachrichten entfernen";
#elif L==3
	"Remove these messages";
#elif L==4
	"Eliminar estos mensajes";
#elif L==5
	"Supprimer ces messages";
#elif L==6
	"Eliminar estos mensajes";	// Okoteve traduccin
#elif L==7
	"Rimuovere questi messaggi";
#elif L==8
	"Usu&nacute; te wiadomo&sacute;ci";
#elif L==9
	"Remover estas mensagens";
#endif

const char *Txt_Remove_type_of_group =
#if   L==1
	"Eliminar tipo de grupo";	// Necessita traduccio
#elif L==2
	"Entfernen Art der Gruppe";
#elif L==3
	"Remove type of group";
#elif L==4
	"Eliminar tipo de grupo";
#elif L==5
	"Supprimer type de groupe";
#elif L==6
	"Eliminar tipo de grupo";	// Okoteve traduccin
#elif L==7
	"Rimuovere tipo di gruppo";
#elif L==8
	"Usu&nacute; typ grupy";
#elif L==9
	"Remover tipo de grupo";
#endif

const char *Txt_Remove_user_from_this_course =
#if   L==1
	"Eliminar usuari d'aquesta assignatura";
#elif L==2
	"Entfernen Benutzer Sie diesen Kurs";
#elif L==3
	"Remove user from this course";
#elif L==4
	"Eliminar usuario de esta asignatura";
#elif L==5
	"Enlever utilisateur de cette mati&egrave;re";
#elif L==6
	"Eliminar usuario de esta asignatura";	// Okoteve traduccin
#elif L==7
	"Rimuovi utente da questo corso";
#elif L==8
	"Usu&ntilde; u&zdot;ytkownika z kursu";
#elif L==9
	"Tirar utilizador desta disciplina";
#endif

const char *Txt_Remove_USER_as_an_administrator =
#if   L==1
	"Eliminar com a administrador";
#elif L==2
	"Entfernen als Administrator";
#elif L==3
	"Remove as an administrator";
#elif L==4
	"Eliminar como administrador";
#elif L==5
	"Enlever en tant qu'administrateur";
#elif L==6
	"Eliminar como administrador";	// Okoteve traduccin
#elif L==7
	"Rimuovi come amministratore";
#elif L==8
	"Usu&nacute; jako administrator";
#elif L==9
	"Tirar como administrador";
#endif

const char *Txt_Remove_USER_as_an_administrator_of_the_centre_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Eliminar com a administrador"
	" del centre <strong>%s</strong>";
#elif L==2
	"Entfernen als Administrator"
	" des Lehrinstitute <strong>%s</strong>";
#elif L==3
	"Remove as an administrator"
	" of the centre <strong>%s</strong>";
#elif L==4
	"Eliminar como administrador"
	" del centro <strong>%s</strong>";
#elif L==5
	"Enlever en tant qu'administrateur"
	" du centre <strong>%s</strong>";
#elif L==6
	"Eliminar como administrador"
	" del centro <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Rimuovi come amministratore"
	" del centro <strong>%s</strong>";
#elif L==8
	"Usu&nacute; jako administrator"
	" centrum <strong>%s</strong>";
#elif L==9
	"Tirar como administrador"
	" do centro <strong>%s</strong>";
#endif

const char *Txt_Remove_USER_as_an_administrator_of_the_degree_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Eliminar com a administrador"
	" de la titulaci&oacute; <strong>%s</strong>";
#elif L==2
	"Entfernen als Administrator"
	" des Studiengang <strong>%s</strong>";
#elif L==3
	"Remove as an administrator"
	" of the degree <strong>%s</strong>";
#elif L==4
	"Eliminar como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";
#elif L==5
	"Enlever en tant qu'administrateur"
	" du &eacute;tude <strong>%s</strong>";
#elif L==6
	"Eliminar como administrador"
	" de la titulaci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Rimuovi come amministratore"
	" della laurea <strong>%s</strong>";
#elif L==8
	"Usu&nacute; jako administrator"
	" stopnia <strong>%s</strong>";
#elif L==9
	"Tirar como administrador"
	" da titula&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Remove_USER_as_an_administrator_of_the_institution_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Eliminar com a administrador"
	" de la instituci&oacute; <strong>%s</strong>";
#elif L==2
	"Entfernen als Administrator"
	" des Hochschule <strong>%s</strong>";
#elif L==3
	"Remove as an administrator"
	" of the institution <strong>%s</strong>";
#elif L==4
	"Eliminar como administrador"
	" de la instituci&oacute;n <strong>%s</strong>";
#elif L==5
	"Enlever en tant qu'administrateur"
	" du &eacute;tablissement <strong>%s</strong>";
#elif L==6
	"Eliminar como administrador"
	" de la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Rimuovi come amministratore"
	" della istituzione <strong>%s</strong>";
#elif L==8
	"Usu&nacute; jako administrator"
	" instytucji <strong>%s</strong>";
#elif L==9
	"Tirar como administrador"
	" da institu&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Remove_USER_from_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Eliminar de l'assignatura <strong>%s</strong>";
#elif L==2
	"Entfernen aus dem Kurs <strong>%s</strong>";
#elif L==3
	"Remove from the course <strong>%s</strong>";
#elif L==4
	"Eliminar de la asignatura <strong>%s</strong>";
#elif L==5
	"Enlever de la mati&egrave;re <strong>%s</strong>";
#elif L==6
	"Eliminar de la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Rimuovi dal corso <strong>%s</strong>";
#elif L==8
	"Usu&nacute; w kurs <strong>%s</strong>";
#elif L==9
	"Tirar da disciplina <strong>%s</strong>";
#endif

const char *Txt_Reply =
#if   L==1
	"Respondre";
#elif L==2
	"Antworten";
#elif L==3
	"Reply";
#elif L==4
	"Responder";
#elif L==5
	"R&eacute;pondre";
#elif L==6
	"Responder";	// Okoteve traduccin
#elif L==7
	"Rispondere";
#elif L==8
	"Odpowiedz";
#elif L==9
	"Responder";
#endif

const char *Txt_Reply_again =
#if   L==1
	"Respondre de nou";
#elif L==2
	"Antworten wieder";
#elif L==3
	"Reply again";
#elif L==4
	"Responder otra vez";
#elif L==5
	"R&eacute;pondre &agrave; nouveau";
#elif L==6
	"Responder otra vez";		// Okoteve traduccin
#elif L==7
	"Rispondere nuovamente";
#elif L==8
	"Odpowiedz ponownie";
#elif L==9
	"Responder novamente";
#endif

const char *Txt_Reply_message =	// "Reply" as a noun, not as a verb
#if   L==1
	"Missatge de resposta";
#elif L==2
	"Antwortnachricht";
#elif L==3
	"Reply message";
#elif L==4
	"Mensaje de respuesta";
#elif L==5
	"Message de r&eacute;ponse";
#elif L==6
	"Mensaje de respuesta";	// Okoteve traduccin
#elif L==7
	"Messaggio di risposta";
#elif L==8
	"Wiadomo&sacute;&cacute; odpowiedz";
#elif L==9
	"Mensagem de resposta";
#endif

const char *Txt_Report =
#if   L==1
	"Informe";
#elif L==2
	"Nutzungsbericht";
#elif L==3
	"Report";
#elif L==4
	"Informe";
#elif L==5
	"Rapport";
#elif L==6
	"Informe";	// Okoteve traduccin
#elif L==7
	"Rapporto";
#elif L==8
	"Raport";
#elif L==9
	"Relat&oacute;rio";
#endif

const char *Txt_Report_of_use_of_PLATFORM =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Informe d'&uacute;s de %s";
#elif L==2
	"Bericht der Nutzung der %s";
#elif L==3
	"Report of use of %s";
#elif L==4
	"Informe de uso de %s";
#elif L==5
	"Rapport d'utilisation de %s";
#elif L==6
	"Informe de uso de %s";	// Okoteve traduccin
#elif L==7
	"Rapporto di utilizzo de %s";
#elif L==8
	"Sprawozdanie z wykorzystaniem %s";
#elif L==9
	"Relat&oacute;rio de utiliza&ccedil;&atilde;o de %s";
#endif

const char *Txt_Report_possible_duplicate_user =
#if   L==1
	"Informar possible usuari duplicat";
#elif L==2
	"Melden Sie die doppelte Benutzer";
#elif L==3
	"Report possible duplicate user";
#elif L==4
	"Informar de posible usuario duplicado";
#elif L==5
	"Signaler possible utilisateur en double";
#elif L==6
	"Informar de posible usuario duplicado";	// Okoteve traduccin
#elif L==7
	"Segnalare eventuali utente duplicato";
#elif L==8
	"Zg&lstrok;o&sacute; mo&zdot;liwy duplikat u&zdot;ytkownika";
#elif L==9
	"Denunciar poss&iacute;vel usu&aacute;rio duplicado";
#endif

const char *Txt_Requester =
#if   L==1
	"Sol&middot;licitant";
#elif L==2
	"Requester";
#elif L==3
	"Requester";
#elif L==4
	"Solicitante";
#elif L==5
	"Demandeur";
#elif L==6
	"Solicitante";	// Okoteve traduccin
#elif L==7
	"Richiedente";
#elif L==8
	"&Zdot;&aogon;daj&aogon;cego";
#elif L==9
	"Requerente";
#endif

const char *Txt_Required_reading =
#if   L==1
	"Lectura obligat&ograve;ria";
#elif L==2
	"Erforderliches Lesen";
#elif L==3
	"Required reading";
#elif L==4
	"Lectura obligatoria";
#elif L==5
	"Lecture obligatoire";
#elif L==6
	"Lectura obligatoria";	// Okoteve traduccin
#elif L==7
	"Lettura obbligatoria";
#elif L==8
	"Obowi&aogon;zkowa lektura";
#elif L==9
	"Leitura obrigat&oacute;ria";
#endif

const char *Txt_Reset =
#if   L==1
	"Posar a zero";
#elif L==2
	"Reset";
#elif L==3
	"Reset";
#elif L==4
	"Poner a cero";
#elif L==5
	"Reset";
#elif L==6
	"Poner a cero";	// Okoteve traduccin
#elif L==7
	"Resettare";
#elif L==8
	"Reset";
#elif L==9
	"Reiniciar";
#endif

const char *Txt_Reset_survey =
#if   L==1
	"Posar enquesta a zero";
#elif L==2
	"Reset Umfrage";
#elif L==3
	"Reset survey";
#elif L==4
	"Poner encuesta a cero";
#elif L==5
	"Reset sondage";
#elif L==6
	"Poner encuesta a cero";	// Okoteve traduccin
#elif L==7
	"Resettare sondaggio";
#elif L==8
	"Reset badania";
#elif L==9
	"Reiniciar inqu&eacute;rito";
#endif

const char *Txt_results_per_page =
#if   L==1
	"resultats per p&agrave;gina";
#elif L==2
	"Ergebnisse pro Seite";
#elif L==3
	"results per page";
#elif L==4
	"resultados por p&aacute;gina";
#elif L==5
	"r&eacute;sultats par page";
#elif L==6
	"resultados por p&aacute;gina";	// Okoteve traduccin
#elif L==7
	"risultati per pagina";
#elif L==8
	"wynik&oacute;w na stronie";
#elif L==9
	"resultados por p&aacute;gina";
#endif

const char *Txt_Retype_new_password =
#if   L==1
	"Reescriviu la nova contrasenya";
#elif L==2
	"Wiederholung neues Passwort";
#elif L==3
	"Retype new password";
#elif L==4
	"Reescriba la nueva contrase&ntilde;a";
#elif L==5
	"Confirmez votre nouveau mot de passe";
#elif L==6
	"Reescriba la nueva contrase&ntilde;a";	// Okoteve traduccin
#elif L==7
	"Riscrivi la nuova password";
#elif L==8
	"Powt&oacute;rz nowe has&lstrok;o";
#elif L==9
	"Redigite a nova senha";
#endif

const char *Txt_Role =
#if   L==1
	"Rol";
#elif L==2
	"Rolle";
#elif L==3
	"Role";
#elif L==4
	"Rol";
#elif L==5
	"R&ocirc;le";
#elif L==6
	"Rol";	// Okoteve traduccin
#elif L==7
	"Ruolo";
#elif L==8
	"Rol&eogon;";
#elif L==9
	"Papel";
#endif

const char *Txt_ROLE_STATS[Sta_NUM_ROLES_STAT] =
   {	// Sta_IDENTIFIED_USRS
#if   L==1
	"identificats"
#elif L==2
	"Identifiziert"
#elif L==3
	"identified"
#elif L==4
	"identificados"
#elif L==5
	"identifi&eacute;s"
#elif L==6
	"identificados"	// Okoteve traduccin
#elif L==7
	"identificati"
#elif L==8
	"zidentyfikowane"
#elif L==9
	"identificados"
#endif
	,	// Sta_ALL_USRS
#if   L==1
	"identificats o desconeguts"
#elif L==2
	"Identifiziert oder Unbekannte"
#elif L==3
	"identified or unknown"
#elif L==4
	"identificados o desconocidos"
#elif L==5
	"identifi&eacute;s ou inconnus"
#elif L==6
	"identificados o desconocidos"	// Okoteve traduccin
#elif L==7
	"identificati o sconosciuti"
#elif L==8
	"zidentyfikowanej lub nieznane"
#elif L==9
	"identificados ou desconhecidos"
#endif
	,	// Sta_INS_ADMINS
#if   L==1
	"administradors d'institucions"
#elif L==2
	"Administratoren von Hochschulen"
#elif L==3
	"administrators of institutions"
#elif L==4
	"administradores de instituciones"
#elif L==5
	"administrateurs d'&eacute;tablissements"
#elif L==6
	"administradores de instituciones"	// Okoteve traduccin
#elif L==7
	"amministratori di istituzioni"
#elif L==8
	"Administratorzy instytucje"
#elif L==9
	"administradores de institu&ccedil;&otilde;es"
#endif
	,	// Sta_CTR_ADMINS
#if   L==1
	"administradors de centres"
#elif L==2
	"Administratoren von Lehrinstitute"
#elif L==3
	"administrators of centres"
#elif L==4
	"administradores de centros"
#elif L==5
	"administrateurs de centres"
#elif L==6
	"administradores de centros"	// Okoteve traduccin
#elif L==7
	"amministratori di centri"
#elif L==8
	"Administratorzy centra"
#elif L==9
	"administradores de centros"
#endif
	,	// Sta_DEG_ADMINS
#if   L==1
	"administradors de titulacions"
#elif L==2
	"Administratoren von Studieng&auml;nge"
#elif L==3
	"administrators of degrees"
#elif L==4
	"administradores de titulaciones"
#elif L==5
	"administrateurs de degr&eacute;s"
#elif L==6
	"administradores de titulaciones"	// Okoteve traduccin
#elif L==7
	"amministratori di laurea"
#elif L==8
	"Administratorzy stopni"
#elif L==9
	"administradores de graus"
#endif
	,	// Sta_TEACHERS
#if   L==1
	"professors"
#elif L==2
	"Lehrkr&auml;fte"
#elif L==3
	"teachers"
#elif L==4
	"profesores"
#elif L==5
	"enseignants"
#elif L==6
	"mbo'eh&aacute;ra"
#elif L==7
	"professori"
#elif L==8
	"nauczycieli"
#elif L==9
	"professores"
#endif
	,	// Sta_STUDENTS
#if   L==1
	"estudiants"
#elif L==2
	"Studenten"
#elif L==3
	"students"
#elif L==4
	"estudiantes"
#elif L==5
	"&eacute;tudiants"
#elif L==6
	"temimbo'e"
#elif L==7
	"studenti"
#elif L==8
	"studenci"
#elif L==9
	"estudantes"
#endif
	,	// Sta_VISITORS
#if   L==1
	"visitants"
#elif L==2
	"Besucher"
#elif L==3
	"visitors"
#elif L==4
	"visitantes"
#elif L==5
	"visiteurs"
#elif L==6
	"visitantes"	// Okoteve traduccin
#elif L==7
	"visitatori"
#elif L==8
	"odwiedzaj&aogon;cy"
#elif L==9
	"visitantes"
#endif
	,	// Sta_GUESTS
#if   L==1
	"convidats (sense assignatures)"
#elif L==2
	"Eingeladen (ohne Kursen)"
#elif L==3
	"guests (without courses)"
#elif L==4
	"invitados (sin asignaturas)"
#elif L==5
	"invit&eacute;s (sans mati&egrave;res)"
#elif L==6
	"invitados (sin asignaturas)"	// Okoteve traduccin
#elif L==7
	"ospiti (senza corsi)"
#elif L==8
	"zaproszeny (bez kurs&oacute;w)"
#elif L==9
	"convidados (sem disciplinas)"
#endif
	,	// Sta_UNKNOWN_USRS
#if   L==1
	"desconeguts"
#elif L==2
	"Unbekannte"
#elif L==3
	"unknown"
#elif L==4
	"desconocidos"
#elif L==5
	"inconnus"
#elif L==6
	"desconocidos"	// Okoteve traduccin
#elif L==7
	"sconosciuti"
#elif L==8
	"nieznany"
#elif L==9
	"desconhecidos"
#endif
	,	// Sta_ME
#if   L==1
	"jo"
#elif L==2
	"Mich"
#elif L==3
	"me"
#elif L==4
	"yo"
#elif L==5
	"moi"
#elif L==6
	"che"
#elif L==7
	"me"
#elif L==8
	"ja"
#elif L==9
	"eu"
#endif
	};

const char *Txt_ROLES_SINGUL_Abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
   {
	{
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_UNKNOWN
	"Desconocido/a"	// Necessita traduccio
#elif L==2
	"Unbekannter/e"
#elif L==3
	"Unknown"
#elif L==4
	"Desconocido/a"
#elif L==5
	"Inconnu/e"
#elif L==6
	"Desconocido/a"	// Okoteve traduccin
#elif L==7
	"Sconosciuto/a"
#elif L==8
	"Nieznany"
#elif L==9
	"Desconhecido/a"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_FEMALE
	"Desconocida"	// Necessita traduccio
#elif L==2
	"Unbekannte"
#elif L==3
	"Unknown"
#elif L==4
	"Desconocida"
#elif L==5
	"Inconnue"
#elif L==6
	"Desconocida"	// Okoteve traduccin
#elif L==7
	"Sconosciuta"
#elif L==8
	"Nieznana"
#elif L==9
	"Desconhecida"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_MALE
	"Desconocido"	// Necessita traduccio
#elif L==2
	"Unbekannter"
#elif L==3
	"Unknown"
#elif L==4
	"Desconocido"
#elif L==5
	"Inconnu"
#elif L==6
	"Desconocido"	// Okoteve traduccin
#elif L==7
	"Sconosciuto"
#elif L==8
	"Nieznany"
#elif L==9
	"Desconhecido"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_ALL
	"Desconocido/a"	// Necessita traduccio
#elif L==2
	"Unbekannter/e"
#elif L==3
	"Unknown"
#elif L==4
	"Desconocido/a"
#elif L==5
	"Inconnu/e"
#elif L==6
	"Desconocido/a"	// Okoteve traduccin
#elif L==7
	"Sconosciuto/a"
#elif L==8
	"Nieznani"
#elif L==9
	"Desconhecido/a"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_UNKNOWN
	"Invitado/a"	// Necessita traduccio
#elif L==2
	"Gast"
#elif L==3
	"Guest"
#elif L==4
	"Invitado/a"
#elif L==5
	"Invit&eacute;/e"
#elif L==6
	"Invitado/a"	// Okoteve traduccin
#elif L==7
	"Ospite"
#elif L==8
	"Go&sacute;&cacute;"
#elif L==9
	"Convidado/a"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_FEMALE
	"Invitada"	// Necessita traduccio
#elif L==2
	"Gast"
#elif L==3
	"Guest"
#elif L==4
	"Invitada"
#elif L==5
	"Invit&eacute;e"
#elif L==6
	"Invitada"	// Okoteve traduccin
#elif L==7
	"Ospite"
#elif L==8
	"Opinia"
#elif L==9
	"Convidada"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_MALE
	"Invitado"	// Necessita traduccio
#elif L==2
	"Gast"
#elif L==3
	"Guest"
#elif L==4
	"Invitado"
#elif L==5
	"Invit&eacute;"
#elif L==6
	"Invitado"	// Okoteve traduccin
#elif L==7
	"Ospite"
#elif L==8
	"Opinia"
#elif L==9
	"Convidado"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_ALL
	"Invitado/a"	// Necessita traduccio
#elif L==2
	"Gast"
#elif L==3
	"Guest"
#elif L==4
	"Invitado/a"
#elif L==5
	"Invit&eacute;/e"
#elif L==6
	"Invitado/a"	// Okoteve traduccin
#elif L==7
	"Ospite"
#elif L==8
	"Opinie"
#elif L==9
	"Convidado/a"
#endif
	},
	{
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_UNKNOWN
	"Visitant"
#elif L==2
	"Besucher"
#elif L==3
	"Visitor"
#elif L==4
	"Visitante"
#elif L==5
	"Visiteur"
#elif L==6
	"Visitante"	// Okoteve traduccin
#elif L==7
	"Visitatore"
#elif L==8
	"Go&sacute;&cacute;"
#elif L==9
	"Visitante"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_FEMALE
	"Visitant"
#elif L==2
	"Besucher"
#elif L==3
	"Visitor"
#elif L==4
	"Visitante"
#elif L==5
	"Visiteur"
#elif L==6
	"Visitante"	// Okoteve traduccin
#elif L==7
	"Visitatore"
#elif L==8
	"Go&sacute;&cacute;"
#elif L==9
	"Visitante"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_MALE
	"Visitant"
#elif L==2
	"Besucher"
#elif L==3
	"Visitor"
#elif L==4
	"Visitante"
#elif L==5
	"Visiteur"
#elif L==6
	"Visitante"	// Okoteve traduccin
#elif L==7
	"Visitatore"
#elif L==8
	"Go&sacute;&cacute;"
#elif L==9
	"Visitante"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_ALL
	"Visitant"
#elif L==2
	"Besucher"
#elif L==3
	"Visitor"
#elif L==4
	"Visitante"
#elif L==5
	"Visiteur"
#elif L==6
	"Visitante"	// Okoteve traduccin
#elif L==7
	"Visitatore"
#elif L==8
	"Go&sacute;&cacute;"
#elif L==9
	"Visitante"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_UNKNOWN
	"Estudiant"
#elif L==2
	"Student"
#elif L==3
	"Student"
#elif L==4
	"Estudiante"
#elif L==5
	"&Eacute;tudiant/e"
#elif L==6
	"Temimbo'e"
#elif L==7
	"Studente"
#elif L==8
	"Student"
#elif L==9
	"Estudante"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_FEMALE
	"Estudiant"
#elif L==2
	"Student"
#elif L==3
	"Student"
#elif L==4
	"Estudiante"
#elif L==5
	"&Eacute;tudiante"
#elif L==6
	"Temimbo'e"
#elif L==7
	"Studentessa"
#elif L==8
	"Studentka"
#elif L==9
	"Estudante"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_MALE
	"Estudiant"
#elif L==2
	"Student"
#elif L==3
	"Student"
#elif L==4
	"Estudiante"
#elif L==5
	"&Eacute;tudiant"
#elif L==6
	"Temimbo'e"
#elif L==7
	"Studente"
#elif L==8
	"Student"
#elif L==9
	"Estudante"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_ALL
	"Estudiant"
#elif L==2
	"Student"
#elif L==3
	"Student"
#elif L==4
	"Estudiante"
#elif L==5
	"&Eacute;tudiant/e"
#elif L==6
	"Temimbo'e"
#elif L==7
	"Studente"
#elif L==8
	"Studenci"
#elif L==9
	"Estudante"
#endif
	},
	{
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_UNKNOWN
	"Professor/a"
#elif L==2
	"Lehrkraft"
#elif L==3
	"Teacher"
#elif L==4
	"Profesor/a"
#elif L==5
	"Enseignant/e"
#elif L==6
	"Mbo'eh&aacute;ra"
#elif L==7
	"Professore/ssa"
#elif L==8
	"Nauczyciel"
#elif L==9
	"Professor/a"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_FEMALE
	"Professora"
#elif L==2
	"Lehrkraft"
#elif L==3
	"Teacher"
#elif L==4
	"Profesora"
#elif L==5
	"Enseignante"
#elif L==6
	"Mbo'eh&aacute;ra"
#elif L==7
	"Professoressa"
#elif L==8
	"Nauczycielka"
#elif L==9
	"Professora"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_MALE
	"Professor"
#elif L==2
	"Lehrkraft"
#elif L==3
	"Teacher"
#elif L==4
	"Profesor"
#elif L==5
	"Enseignant"
#elif L==6
	"Mbo'eh&aacute;ra"
#elif L==7
	"Professore"
#elif L==8
	"Nauczyciel"
#elif L==9
	"Professor"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_ALL
	"Professor/a"
#elif L==2
	"Lehrkraft"
#elif L==3
	"Teacher"
#elif L==4
	"Profesor/a"
#elif L==5
	"Enseignant/e"
#elif L==6
	"Mbo'eh&aacute;ra"
#elif L==7
	"Professore/ssa"
#elif L==8
	"Nauczyciele"
#elif L==9
	"Professor/a"
#endif
	},
	{
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_UNKNOWN
	"Admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Studiengang"
#elif L==3
	"Admin. degree"
#elif L==4
	"Admin. titulaci&oacute;n"
#elif L==5
	"Admin. &eacute;tude"
#elif L==6
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Ammin. laurea"
#elif L==8
	"Admin. stopien"
#elif L==9
	"Admin. titula&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_FEMALE
	"Admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Studiengang"
#elif L==3
	"Admin. degree"
#elif L==4
	"Admin. titulaci&oacute;n"
#elif L==5
	"Admin. &eacute;tude"
#elif L==6
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Ammin. laurea"
#elif L==8
	"Admin. stopien"
#elif L==9
	"Admin. titula&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_MALE
	"Admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Studiengang"
#elif L==3
	"Admin. degree"
#elif L==4
	"Admin. titulaci&oacute;n"
#elif L==5
	"Admin. &eacute;tude"
#elif L==6
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Ammin. laurea"
#elif L==8
	"Admin. stopien"
#elif L==9
	"Admin. titula&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_ALL
	"Admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Studiengang"
#elif L==3
	"Admin. degree"
#elif L==4
	"Admin. titulaci&oacute;n"
#elif L==5
	"Admin. &eacute;tude"
#elif L==6
	"Admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Ammin. laurea"
#elif L==8
	"Admin. stopien"
#elif L==9
	"Admin. titula&ccedil;&atilde;o"
#endif
	},
	{
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_UNKNOWN
	"Admin. centro"	// Necessita traduccio
#elif L==2
	"Admin. Lehrinstitut"
#elif L==3
	"Admin. centre"
#elif L==4
	"Admin. centro"
#elif L==5
	"Admin. centre"
#elif L==6
	"Admin. centro"	// Okoteve traduccin
#elif L==7
	"Ammin. centro"
#elif L==8
	"Admin. centrum"
#elif L==9
	"Admin. centro"
#endif
	,
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_FEMALE
	"Admin. centro"	// Necessita traduccio
#elif L==2
	"Admin. Lehrinstitut"
#elif L==3
	"Admin. centre"
#elif L==4
	"Admin. centro"
#elif L==5
	"Admin. centre"
#elif L==6
	"Admin. centro"	// Okoteve traduccin
#elif L==7
	"Ammin. centro"
#elif L==8
	"Admin. centrum"
#elif L==9
	"Admin. centro"
#endif
	,
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_MALE
	"Admin. centro"	// Necessita traduccio
#elif L==2
	"Admin. Lehrinstitut"
#elif L==3
	"Admin. centre"
#elif L==4
	"Admin. centro"
#elif L==5
	"Admin. centre"
#elif L==6
	"Admin. centro"	// Okoteve traduccin
#elif L==7
	"Ammin. centro"
#elif L==8
	"Admin. centrum"
#elif L==9
	"Admin. centro"
#endif
	,
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_ALL
	"Admin. centro"	// Necessita traduccio
#elif L==2
	"Admin. Lehrinstitut"
#elif L==3
	"Admin. centre"
#elif L==4
	"Admin. centro"
#elif L==5
	"Admin. centre"
#elif L==6
	"Admin. centro"	// Okoteve traduccin
#elif L==7
	"Ammin. centro"
#elif L==8
	"Admin. centrum"
#elif L==9
	"Admin. centro"
#endif
	},
	{
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_UNKNOWN
	"Admin. instituci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Hochschule"
#elif L==3
	"Admin. institution"
#elif L==4
	"Admin. instituci&oacute;n"
#elif L==5
	"Admin. &eacute;tablissement"
#elif L==6
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Ammin. istituzione"
#elif L==8
	"Admin. instytucji"
#elif L==9
	"Admin. institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_FEMALE
	"Admin. instituci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Hochschule"
#elif L==3
	"Admin. institution"
#elif L==4
	"Admin. instituci&oacute;n"
#elif L==5
	"Admin. &eacute;tablissement"
#elif L==6
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Ammin. istituzione"
#elif L==8
	"Admin. instytucji"
#elif L==9
	"Admin. institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_MALE
	"Admin. instituci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Hochschule"
#elif L==3
	"Admin. institution"
#elif L==4
	"Admin. instituci&oacute;n"
#elif L==5
	"Admin. &eacute;tablissement"
#elif L==6
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Ammin. istituzione"
#elif L==8
	"Admin. instytucji"
#elif L==9
	"Admin. institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_ALL
	"Admin. instituci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Hochschule"
#elif L==3
	"Admin. institution"
#elif L==4
	"Admin. instituci&oacute;n"
#elif L==5
	"Admin. &eacute;tablissement"
#elif L==6
	"Admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	"Ammin. istituzione"
#elif L==8
	"Admin. instytucji"
#elif L==9
	"Admin. institu&ccedil;&atilde;o"
#endif
	},
	{
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_UNKNOWN
	"Superusuari/&agrave;ria"
#elif L==2
	"Superuser"
#elif L==3
	"Superuser"
#elif L==4
	"Superusuario/a"
#elif L==5
	"Superutilisateur/se"
#elif L==6
	"Superusuario/a"	// Okoteve traduccin
#elif L==7
	"Superutente"
#elif L==8
	"Superuzytkownik"
#elif L==9
	"Superutilizador/a"
#endif
	,
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_FEMALE
	"Superusuaria"	// Necessita traduccio
#elif L==2
	"Superuser"
#elif L==3
	"Superuser"
#elif L==4
	"Superusuaria"
#elif L==5
	"Superutilisateuse"
#elif L==6
	"Superusuaria"	// Okoteve traduccin
#elif L==7
	"Superutente"
#elif L==8
	"Superuzytkownik"
#elif L==9
	"Superutilizadora"
#endif
	,
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_MALE
	"Superusuari"
#elif L==2
	"Superuser"
#elif L==3
	"Superuser"
#elif L==4
	"Superusuario"
#elif L==5
	"Superutilisateur"
#elif L==6
	"Superusuario"	// Okoteve traduccin
#elif L==7
	"Superutente"
#elif L==8
	"Superuzytkownik"
#elif L==9
	"Superutilizador"
#endif
	,
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_ALL
	"Superusuari/&agrave;ria"	// Necessita traduccio
#elif L==2
	"Superuser"
#elif L==3
	"Superuser"
#elif L==4
	"Superusuario/a"
#elif L==5
	"Superutilisateur/se"
#elif L==6
	"Superusuario/a"		// Okoteve traduccin
#elif L==7
	"Superutente"
#elif L==8
	"Superuzytkownicy"
#elif L==9
	"Superutilizador/a"
#endif
	 } };

const char *Txt_ROLES_SINGUL_abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
   {
	{
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_UNKNOWN
	"desconocido/a"		// Necessita traduccio
#elif L==2
	"Unbekannter/e"
#elif L==3
	"unknown"
#elif L==4
	"desconocido/a"
#elif L==5
	"inconnu/e"
#elif L==6
	"desconocido/a"		// Okoteve traduccin
#elif L==7
	"sconosciuto/a"
#elif L==8
	"nieznany"
#elif L==9
	"desconhecido/a"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_FEMALE
	"desconocida"	// Necessita traduccio
#elif L==2
	"Unbekannte"
#elif L==3
	"unknown"
#elif L==4
	"desconocida"
#elif L==5
	"inconnue"
#elif L==6
	"desconocida"	// Okoteve traduccin
#elif L==7
	"sconosciuta"
#elif L==8
	"nieznana"
#elif L==9
	"desconhecida"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_MALE
	"desconocido"	// Necessita traduccio
#elif L==2
	"Unbekannter"
#elif L==3
	"unknown"
#elif L==4
	"desconocido"
#elif L==5
	"inconnu"
#elif L==6
	"desconocido"	// Okoteve traduccin
#elif L==7
	"sconosciuto"
#elif L==8
	"nieznany"
#elif L==9
	"desconhecido"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_ALL
	"desconocido/a"		// Necessita traduccio
#elif L==2
	"Unbekannter/e"
#elif L==3
	"unknown"
#elif L==4
	"desconocido/a"
#elif L==5
	"inconnu/e"
#elif L==6
	"desconocido/a"		// Okoteve traduccin
#elif L==7
	"sconosciuto/a"
#elif L==8
	"nieznani"
#elif L==9
	"desconhecido/a"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_UNKNOWN
	"invitado/a"	// Necessita traduccio
#elif L==2
	"Gast"
#elif L==3
	"guest"
#elif L==4
	"invitado/a"
#elif L==5
	"invit&eacute;/e"
#elif L==6
	"invitado/a"	// Okoteve traduccin
#elif L==7
	"ospite"
#elif L==8
	"go&sacute;&cacute;"
#elif L==9
	"convidado/a"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_FEMALE
	"invitada"	// Necessita traduccio
#elif L==2
	"Gast"
#elif L==3
	"guest"
#elif L==4
	"invitada"
#elif L==5
	"invit&eacute;e"
#elif L==6
	"invitada"	// Okoteve traduccin
#elif L==7
	"ospite"
#elif L==8
	"goscinna"
#elif L==9
	"convidada"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_MALE
	"invitado"	// Necessita traduccio
#elif L==2
	"Gast"
#elif L==3
	"guest"
#elif L==4
	"invitado"
#elif L==5
	"invit&eacute;"
#elif L==6
	"invitado"	// Okoteve traduccin
#elif L==7
	"ospite"
#elif L==8
	"goscinny"
#elif L==9
	"convidado"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_ALL
	"invitado/a"	// Necessita traduccio
#elif L==2
	"Gast"
#elif L==3
	"guest"
#elif L==4
	"invitado/a"
#elif L==5
	"invit&eacute;/e"
#elif L==6
	"invitado/a"	// Okoteve traduccin
#elif L==7
	"ospite"
#elif L==8
	"goscinni"
#elif L==9
	"convidado/a"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_UNKNOWN
	"visitant"
#elif L==2
	"Besucher"
#elif L==3
	"visitor"
#elif L==4
	"visitante"
#elif L==5
	"visiteur"
#elif L==6
	"visitante"	// Okoteve traduccin
#elif L==7
	"visitatore"
#elif L==8
	"go&sacute;&cacute;"
#elif L==9
	"Visitante"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_FEMALE
	"visitant"
#elif L==2
	"Besucher"
#elif L==3
	"visitor"
#elif L==4
	"visitante"
#elif L==5
	"visiteur"
#elif L==6
	"visitante"	// Okoteve traduccin
#elif L==7
	"visitatore"
#elif L==8
	"go&sacute;&cacute;"
#elif L==9
	"Visitante"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_MALE
	"visitant"
#elif L==2
	"Besucher"
#elif L==3
	"visitor"
#elif L==4
	"visitante"
#elif L==5
	"visiteur"
#elif L==6
	"visitante"	// Okoteve traduccin
#elif L==7
	"visitatore"
#elif L==8
	"go&sacute;&cacute;"
#elif L==9
	"Visitante"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_ALL
	"visitant"
#elif L==2
	"Besucher"
#elif L==3
	"visitor"
#elif L==4
	"visitante"
#elif L==5
	"visiteur"
#elif L==6
	"visitante"	// Okoteve traduccin
#elif L==7
	"visitatore"
#elif L==8
	"go&sacute;&cacute;"
#elif L==9
	"Visitante"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_UNKNOWN
	"estudiant"
#elif L==2
	"Student"
#elif L==3
	"student"
#elif L==4
	"estudiante"
#elif L==5
	"&eacute;tudiant/e"
#elif L==6
	"temimbo'e"
#elif L==7
	"studente"
#elif L==8
	"student"
#elif L==9
	"estudante"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_FEMALE
	"estudiant"
#elif L==2
	"Student"
#elif L==3
	"student"
#elif L==4
	"estudiante"
#elif L==5
	"&eacute;tudiante"
#elif L==6
	"temimbo'e"
#elif L==7
	"studentessa"
#elif L==8
	"studentka"
#elif L==9
	"estudante"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_MALE
	"estudiant"
#elif L==2
	"Student"
#elif L==3
	"student"
#elif L==4
	"estudiante"
#elif L==5
	"&eacute;tudiant"
#elif L==6
	"temimbo'e"
#elif L==7
	"studente"
#elif L==8
	"student"
#elif L==9
	"estudante"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_ALL
	"estudiant"
#elif L==2
	"Student"
#elif L==3
	"student"
#elif L==4
	"estudiante"
#elif L==5
	"&eacute;tudiant/e"
#elif L==6
	"temimbo'e"
#elif L==7
	"studente/ssa"
#elif L==8
	"studenci"
#elif L==9
	"estudante"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_UNKNOWN
	"professor/a"
#elif L==2
	"Lehrkraft"
#elif L==3
	"teacher"
#elif L==4
	"profesor/a"
#elif L==5
	"enseignant/e"
#elif L==6
	"mbo'eh&aacute;ra"
#elif L==7
	"Professore/ssa"
#elif L==8
	"nauczyciel"
#elif L==9
	"professor/a"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_FEMALE
	"professora"
#elif L==2
	"Lehrkraft"
#elif L==3
	"teacher"
#elif L==4
	"profesora"
#elif L==5
	"enseignante"
#elif L==6
	"mbo'eh&aacute;ra"
#elif L==7
	"Professoressa"
#elif L==8
	"nauczycielka"
#elif L==9
	"professora"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_MALE
	"professor"
#elif L==2
	"Lehrkraft"
#elif L==3
	"teacher"
#elif L==4
	"profesor"
#elif L==5
	"enseignant"
#elif L==6
	"mbo'eh&aacute;ra"
#elif L==7
	"professore"
#elif L==8
	"nauczyciel"
#elif L==9
	"professor"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_ALL
	"professor/a"
#elif L==2
	"Lehrkraft"
#elif L==3
	"teacher"
#elif L==4
	"profesor/a"
#elif L==5
	"enseignant/e"
#elif L==6
	"mbo'eh&aacute;ra"
#elif L==7
	"professore/ssa"
#elif L==8
	"nauczyciele"
#elif L==9
	"professor/a"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_UNKNOWN
	"admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Studiengang"
#elif L==3
	"admin. degree"
#elif L==4
	"admin. titulaci&oacute;n"
#elif L==5
	"admin. &eacute;tude"
#elif L==6
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"ammin. laurea"
#elif L==8
	"admin. stopien"
#elif L==9
	"admin. titula&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_FEMALE
	"admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Studiengang"
#elif L==3
	"admin. degree"
#elif L==4
	"admin. titulaci&oacute;n"
#elif L==5
	"admin. &eacute;tude"
#elif L==6
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"ammin. laurea"
#elif L==8
	"admin. stopien"
#elif L==9
	"admin. titula&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_MALE
	"admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Studiengang"
#elif L==3
	"admin. degree"
#elif L==4
	"admin. titulaci&oacute;n"
#elif L==5
	"admin. &eacute;tude"
#elif L==6
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"ammin. laurea"
#elif L==8
	"admin. stopien"
#elif L==9
	"admin. titula&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_ALL
	"admin. titulaci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Studiengang"
#elif L==3
	"admin. degree"
#elif L==4
	"admin. titulaci&oacute;n"
#elif L==5
	"admin. &eacute;tude"
#elif L==6
	"admin. titulaci&oacute;n"	// Okoteve traduccin
#elif L==7
	"ammin. laurea"
#elif L==8
	"admin. stopien"
#elif L==9
	"admin. titula&ccedil;&atilde;o"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_UNKNOWN
	"admin. centro"	// Necessita traduccio
#elif L==2
	"Admin. Lehrinstitut"
#elif L==3
	"admin. centre"
#elif L==4
	"admin. centro"
#elif L==5
	"admin. centre"
#elif L==6
	"admin. centro"	// Okoteve traduccin
#elif L==7
	"ammin. centro"
#elif L==8
	"admin. centrum"
#elif L==9
	"admin. centro"
#endif
	,
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_FEMALE
	"admin. centro"	// Necessita traduccio
#elif L==2
	"Admin. Lehrinstitut"
#elif L==3
	"admin. centre"
#elif L==4
	"admin. centro"
#elif L==5
	"admin. centre"
#elif L==6
	"admin. centro"	// Okoteve traduccin
#elif L==7
	"ammin. centro"
#elif L==8
	"admin. centrum"
#elif L==9
	"admin. centro"
#endif
	,
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_MALE
	"admin. centro"	// Necessita traduccio
#elif L==2
	"Admin. Lehrinstitut"
#elif L==3
	"admin. centre"
#elif L==4
	"admin. centro"
#elif L==5
	"admin. centre"
#elif L==6
	"admin. centro"	// Okoteve traduccin
#elif L==7
	"ammin. centro"
#elif L==8
	"admin. centrum"
#elif L==9
	"admin. centro"
#endif
	,
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_ALL
	"admin. centro"	// Necessita traduccio
#elif L==2
	"Admin. Lehrinstitut"
#elif L==3
	"admin. centre"
#elif L==4
	"admin. centro"
#elif L==5
	"admin. centre"
#elif L==6
	"admin. centro"	// Okoteve traduccin
#elif L==7
	"ammin. centro"
#elif L==8
	"admin. centrum"
#elif L==9
	"admin. centro"
#endif
	},
	{
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_UNKNOWN
	"admin. instituci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Hochschule"
#elif L==3
	"admin. institution"
#elif L==4
	"admin. instituci&oacute;n"
#elif L==5
	"admin. &eacute;tablissement"
#elif L==6
	"admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	"ammin. istituzione"
#elif L==8
	"admin. instytucji"
#elif L==9
	"admin. institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_FEMALE
	"admin. instituci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Hochschule"
#elif L==3
	"admin. institution"
#elif L==4
	"admin. instituci&oacute;n"
#elif L==5
	"admin. &eacute;tablissement"
#elif L==6
	"admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	"ammin. istituzione"
#elif L==8
	"admin. instytucji"
#elif L==9
	"admin. institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_MALE
	"administrador de instituci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Hochschule"
#elif L==3
	"admin. institution"
#elif L==4
	"administrador de instituci&oacute;n"
#elif L==5
	"administrateur d'&eacute;tablissement"
#elif L==6
	"administrador de instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	"ammin.istituzione"
#elif L==8
	"administrator instytucji"
#elif L==9
	"administrador de institu&ccedil;&atilde;o"
#endif
	,
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_ALL
	"admin. instituci&oacute;n"	// Necessita traduccio
#elif L==2
	"Admin. Hochschule"
#elif L==3
	"admin. institution"
#elif L==4
	"admin. instituci&oacute;n"
#elif L==5
	"admin. &eacute;tablissement"
#elif L==6
	"admin. instituci&oacute;n"	// Okoteve traduccin
#elif L==7
	"ammin.istituzione"
#elif L==8
	"administratorzy instytucji"
#elif L==9
	"admin. institu&ccedil;&atilde;o"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_UNKNOWN
	"superusuari/&agrave;ria"
#elif L==2
	"Superuser"
#elif L==3
	"superuser"
#elif L==4
	"superusuario/a"
#elif L==5
	"superutilisateur/se"
#elif L==6
	"superusuario/a"	// Okoteve traduccin
#elif L==7
	"superutente"
#elif L==8
	"superuzytkownik"
#elif L==9
	"superutilizador/a"
#endif
	,
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_FEMALE
	"superusuaria"		// Necessita traduccio
#elif L==2
	"Superuser"
#elif L==3
	"superuser"
#elif L==4
	"superusuaria"
#elif L==5
	"superutilisateuse"
#elif L==6
	"superusuaria"		// Okoteve traduccin
#elif L==7
	"superutente"
#elif L==8
	"superuzytkowniczka"
#elif L==9
	"superutilizadora"
#endif
	,
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_MALE
	"superusuari"
#elif L==2
	"Superuser"
#elif L==3
	"superuser"
#elif L==4
	"superusuario"
#elif L==5
	"superutilisateur"
#elif L==6
	"superusuario"	// Okoteve traduccin
#elif L==7
	"superutente"
#elif L==8
	"superu&zdot;ytkownika"
#elif L==9
	"superutilizador"
#endif
	,
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_ALL
	"superusuari/&agrave;ria"
#elif L==2
	"Superuser"
#elif L==3
	"superuser"
#elif L==4
	"superusuario/a"
#elif L==5
	"superutilisateur/se"
#elif L==6
	"superusuario/a"	// Okoteve traduccin
#elif L==7
	"superutente"
#elif L==8
	"superuzytkownicy"
#elif L==9
	"superutilizador/a"
#endif
	 }
   };

const char *Txt_ROLES_PLURAL_Abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
   {
	{
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_UNKNOWN
	"Desconocidos"	// Necessita traduccio
#elif L==2
	"Unbekannte"
#elif L==3
	"Unknown"
#elif L==4
	"Desconocidos"
#elif L==5
	"Inconnus"
#elif L==6
	"Desconocidos"	// Okoteve traduccin
#elif L==7
	"Sconosciuti"
#elif L==8
	"Nieznany"
#elif L==9
	"Desconhecidos"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_FEMALE
	"Desconocidas"	// Necessita traduccio
#elif L==2
	"Unbekannte"
#elif L==3
	"Unknown"
#elif L==4
	"Desconocidas"
#elif L==5
	"Inconnues"
#elif L==6
	"Desconocidas"	// Okoteve traduccin
#elif L==7
	"Sconosciuti"
#elif L==8
	"Nieznana"
#elif L==9
	"Desconhecidas"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_MALE
	"Desconocidos"	// Necessita traduccio
#elif L==2
	"Unbekannte"
#elif L==3
	"Unknown"
#elif L==4
	"Desconocidos"
#elif L==5
	"Inconnus"
#elif L==6
	"Desconocidos"	// Okoteve traduccin
#elif L==7
	"Sconosciuti"
#elif L==8
	"Nieznany"
#elif L==9
	"Desconhecidos"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_ALL
	"Desconocidos"	// Necessita traduccio
#elif L==2
	"Unbekannte"
#elif L==3
	"Unknown"
#elif L==4
	"Desconocidos"
#elif L==5
	"Inconnus"
#elif L==6
	"Desconocidos"	// Okoteve traduccin
#elif L==7
	"Sconosciuti"
#elif L==8
	"Nieznani"
#elif L==9
	"Desconhecidos"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_UNKNOWN
	"Convidats"
#elif L==2
	"Eingeladen"
#elif L==3
	"Guests"
#elif L==4
	"Invitados"
#elif L==5
	"Invit&eacute;s"
#elif L==6
	"Invitados"	// Okoteve traduccin
#elif L==7
	"Ospiti"
#elif L==8
	"Zaproszenie"
#elif L==9
	"Convidados"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_FEMALE
	"Invitadas"	// Necessita traduccio
#elif L==2
	"Eingeladen"
#elif L==3
	"Guests"
#elif L==4
	"Invitadas"
#elif L==5
	"Invit&eacute;es"
#elif L==6
	"Invitadas"	// Okoteve traduccin
#elif L==7
	"Ospiti"
#elif L==8
	"Zaproszona"
#elif L==9
	"Convidadas"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_MALE
	"Convidats"
#elif L==2
	"Eingeladen"
#elif L==3
	"Guests"
#elif L==4
	"Invitados"
#elif L==5
	"Invit&eacute;s"
#elif L==6
	"Invitados"	// Okoteve traduccin
#elif L==7
	"Ospiti"
#elif L==8
	"Zaproszony"
#elif L==9
	"Convidados"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_ALL
	"Convidats"
#elif L==2
	"Eingeladen"
#elif L==3
	"Guests"
#elif L==4
	"Invitados"
#elif L==5
	"Invit&eacute;s"
#elif L==6
	"Invitados"	// Okoteve traduccin
#elif L==7
	"Ospiti"
#elif L==8
	"Zaproszeni"
#elif L==9
	"Convidados"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_UNKNOWN
	"Visitants"
#elif L==2
	"Besucher"
#elif L==3
	"Visitors"
#elif L==4
	"Visitantes"
#elif L==5
	"Visiteurs"
#elif L==6
	"Visitantes"	// Okoteve traduccin
#elif L==7
	"Visitatori"
#elif L==8
	"Odwiedzaj&aogon;cy"
#elif L==9
	"Visitantes"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_FEMALE
	"Visitants"
#elif L==2
	"Besucher"
#elif L==3
	"Visitors"
#elif L==4
	"Visitantes"
#elif L==5
	"Visiteurs"
#elif L==6
	"Visitantes"	// Okoteve traduccin
#elif L==7
	"Visitatori"
#elif L==8
	"Odwiedzaj&aogon;cy"
#elif L==9
	"Visitantes"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_MALE
	"Visitants"
#elif L==2
	"Besucher"
#elif L==3
	"Visitors"
#elif L==4
	"Visitantes"
#elif L==5
	"Visiteurs"
#elif L==6
	"Visitantes"	// Okoteve traduccin
#elif L==7
	"Visitatori"
#elif L==8
	"Odwiedzaj&aogon;cy"
#elif L==9
	"Visitantes"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_ALL
	"Visitants"
#elif L==2
	"Besucher"
#elif L==3
	"Visitors"
#elif L==4
	"Visitantes"
#elif L==5
	"Visiteurs"
#elif L==6
	"Visitantes"	// Okoteve traduccin
#elif L==7
	"Visitatori"
#elif L==8
	"Odwiedzaj&aogon;cy"
#elif L==9
	"Visitantes"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_UNKNOWN
	"Estudiants"
#elif L==2
	"Studenten"
#elif L==3
	"Students"
#elif L==4
	"Estudiantes"
#elif L==5
	"&Eacute;tudiants"
#elif L==6
	"Temimbo'e"
#elif L==7
	"Studenti"
#elif L==8
	"Studenci"
#elif L==9
	"Estudantes"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_FEMALE
	"Estudiants"
#elif L==2
	"Studenten"
#elif L==3
	"Students"
#elif L==4
	"Estudiantes"
#elif L==5
	"&Eacute;tudiantes"
#elif L==6
	"Temimbo'e"
#elif L==7
	"Studentesse"
#elif L==8
	"Studentki"
#elif L==9
	"Estudantes"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_MALE
	"Estudiants"
#elif L==2
	"Studenten"
#elif L==3
	"Students"
#elif L==4
	"Estudiantes"
#elif L==5
	"&Eacute;tudiants"
#elif L==6
	"Temimbo'e"
#elif L==7
	"Studenti"
#elif L==8
	"Studenci"
#elif L==9
	"Estudantes"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_ALL
	"Estudiants"
#elif L==2
	"Studenten"
#elif L==3
	"Students"
#elif L==4
	"Estudiantes"
#elif L==5
	"&Eacute;tudiants"
#elif L==6
	"Temimbo'e"
#elif L==7
	"Studenti"
#elif L==8
	"Studenci"
#elif L==9
	"Estudantes"
#endif
	},
	{
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_UNKNOWN
	"Professors"
#elif L==2
	"Lehrkraft"
#elif L==3
	"Teachers"
#elif L==4
	"Profesores"
#elif L==5
	"Enseignants"
#elif L==6
	"Mbo'eh&aacute;ra"
#elif L==7
	"Professori"
#elif L==8
	"Nauczyciele"
#elif L==9
	"Professores"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_FEMALE
	"Professores"
#elif L==2
	"Lehrkraft"
#elif L==3
	"Teachers"
#elif L==4
	"Profesoras"
#elif L==5
	"Enseignantes"
#elif L==6
	"Mbo'eh&aacute;ra"
#elif L==7
	"Professoresse"
#elif L==8
	"Nauczycielki"
#elif L==9
	"Professoras"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_MALE
	"Professors"
#elif L==2
	"Lehrkraft"
#elif L==3
	"Teachers"
#elif L==4
	"Profesores"
#elif L==5
	"Enseignants"
#elif L==6
	"Mbo'eh&aacute;ra"
#elif L==7
	"Professori"
#elif L==8
	"Nauczyciele"
#elif L==9
	"Professores"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_ALL
	"Professors"
#elif L==2
	"Lehrkraft"
#elif L==3
	"Teachers"
#elif L==4
	"Profesores"
#elif L==5
	"Enseignants"
#elif L==6
	"Mbo'eh&aacute;ra"
#elif L==7
	"Professori"
#elif L==8
	"Nauczyciele"
#elif L==9
	"Professores"
#endif
	},
	{
#if   L==1	// Rol_DEG_ADM - Usr_SEX_UNKNOWN
	"Administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradores"
#elif L==5
	"Administrateurs"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorzy"
#elif L==9
	"Administradores"
#endif
	,
#if   L==1	// Rol_DEG_ADM - Usr_SEX_FEMALE
	"Administradoras"	// Necessita traduccio
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradoras"
#elif L==5
	"Administrateuses"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorki"
#elif L==9
	"Administradoras"
#endif
	,
#if   L==1	// Rol_DEG_ADM - Usr_SEX_MALE
	"Administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradores"
#elif L==5
	"Administrateurs"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorzy"
#elif L==9
	"Administradores"
#endif
	,
#if   L==1	// Rol_DEG_ADM - Usr_SEX_ALL
	"Administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradores"
#elif L==5
	"Administrateurs"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorzy"
#elif L==9
	"Administradores"
#endif
	 },
	 {
#if   L==1	// Rol_CTR_ADM - Usr_SEX_UNKNOWN
	"Administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradores"
#elif L==5
	"Administrateurs"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorzy"
#elif L==9
	"Administradores"
#endif
	,
#if   L==1	// Rol_CTR_ADM - Usr_SEX_FEMALE
	"Administradoras"	// Necessita traduccio
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradoras"
#elif L==5
	"Administrateuses"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorki"
#elif L==9
	"Administradoras"
#endif
	,
#if   L==1	// Rol_CTR_ADM - Usr_SEX_MALE
	"Administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradores"
#elif L==5
	"Administrateurs"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorzy"
#elif L==9
	"Administradores"
#endif
	,
#if   L==1	// Rol_CTR_ADM - Usr_SEX_ALL
	"Administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradores"
#elif L==5
	"Administrateurs"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorzy"
#elif L==9
	"Administradores"
#endif
	 },
	 {
#if   L==1	// Rol_INS_ADM - Usr_SEX_UNKNOWN
	"Administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradores"
#elif L==5
	"Administrateurs"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorzy"
#elif L==9
	"Administradores"
#endif
	,
#if   L==1	// Rol_INS_ADM - Usr_SEX_FEMALE
	"Administradoras"	// Necessita traduccio
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradoras"
#elif L==5
	"Administrateuses"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorki"
#elif L==9
	"Administradoras"
#endif
	,
#if   L==1	// Rol_INS_ADM - Usr_SEX_MALE
	"Administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradores"
#elif L==5
	"Administrateurs"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorzy"
#elif L==9
	"Administradores"
#endif
	,
#if   L==1	// Rol_INS_ADM - Usr_SEX_ALL
	"Administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"Administrators"
#elif L==4
	"Administradores"
#elif L==5
	"Administrateurs"
#elif L==6
	"&Ntilde;angarekoh&aacute;ra"
#elif L==7
	"Amministratori"
#elif L==8
	"Administratorzy"
#elif L==9
	"Administradores"
#endif
	},
	{
#if   L==1	// Rol_SYS_ADM - Usr_SEX_UNKNOWN
	"Superusuaris"
#elif L==2
	"Superuser"
#elif L==3
	"Superusers"
#elif L==4
	"Superusuarios"
#elif L==5
	"Superutilisateurs"
#elif L==6
	"Superusuarios"	// Okoteve traduccin
#elif L==7
	"Superutenti"
#elif L==8
	"Superu&zdot;ytkownika"
#elif L==9
	"Superutilizadores"
#endif
	,
#if   L==1	// Rol_SYS_ADM - Usr_SEX_FEMALE
	"Superusuarias"	// Necessita traduccio
#elif L==2
	"Superuser"
#elif L==3
	"Superusers"
#elif L==4
	"Superusuarias"
#elif L==5
	"Superutilisateuses"
#elif L==6
	"Superusuarias"	// Okoteve traduccin
#elif L==7
	"Superutenti"
#elif L==8
	"Superuzytkowniczka"
#elif L==9
	"Superutilizadoras"
#endif
	,
#if   L==1	// Rol_SYS_ADM - Usr_SEX_MALE
	"Superusuaris"
#elif L==2
	"Superuser"
#elif L==3
	"Superusers"
#elif L==4
	"Superusuarios"
#elif L==5
	"Superutilisateurs"
#elif L==6
	"Superusuarios"	// Okoteve traduccin
#elif L==7
	"Superutenti"
#elif L==8
	"Superuzytkownik"
#elif L==9
	"Superutilizadores"
#endif
	,
#if   L==1	// Rol_SYS_ADM - Usr_SEX_ALL
	"Superusuaris"
#elif L==2
	"Superuser"
#elif L==3
	"Superusers"
#elif L==4
	"Superusuarios"
#elif L==5
	"Superutilisateurs"
#elif L==6
	"Superusuarios"	// Okoteve traduccin
#elif L==7
	"Superutenti"
#elif L==8
	"Superuzytkownicy"
#elif L==9
	"Superutilizadores"
#endif
	 } };

const char *Txt_ROLES_PLURAL_abc[Rol_NUM_ROLES][Usr_NUM_SEXS] =
   {
	{
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_UNKNOWN
	"desconocidos"	// Necessita traduccio
#elif L==2
	"Unbekannte"
#elif L==3
	"unknown"
#elif L==4
	"desconocidos"
#elif L==5
	"inconnus"
#elif L==6
	"desconocidos"	// Okoteve traduccin
#elif L==7
	"sconosciuti"
#elif L==8
	"nieznany"
#elif L==9
	"desconhecidos"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_FEMALE
	"desconocidas"	// Necessita traduccio
#elif L==2
	"Unbekannte"
#elif L==3
	"unknown"
#elif L==4
	"desconocidas"
#elif L==5
	"inconnues"
#elif L==6
	"desconocidas"	// Okoteve traduccin
#elif L==7
	"sconosciuti"
#elif L==8
	"nieznana"
#elif L==9
	"desconhecidas"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_MALE
	"desconocidos"	// Necessita traduccio
#elif L==2
	"Unbekannte"
#elif L==3
	"unknown"
#elif L==4
	"desconocidos"
#elif L==5
	"inconnus"
#elif L==6
	"desconocidos"	// Okoteve traduccin
#elif L==7
	"sconosciuti"
#elif L==8
	"nieznany"
#elif L==9
	"desconhecidos"
#endif
	,
#if   L==1	// Rol_ROLE_UNKNOWN - Usr_SEX_ALL
	"desconocidos"	// Necessita traduccio
#elif L==2
	"Unbekannte"
#elif L==3
	"unknown"
#elif L==4
	"desconocidos"
#elif L==5
	"inconnus"
#elif L==6
	"desconocidos"	// Okoteve traduccin
#elif L==7
	"sconosciuti"
#elif L==8
	"nieznani"
#elif L==9
	"desconhecidos"
#endif
	},
	{
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_UNKNOWN
	"convidats"
#elif L==2
	"Eingeladen"
#elif L==3
	"guests"
#elif L==4
	"invitados"
#elif L==5
	"invit&eacute;s"
#elif L==6
	"invitados"	// Okoteve traduccin
#elif L==7
	"ospiti"
#elif L==8
	"zaproszenie"
#elif L==9
	"convidados"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_FEMALE
	"invitadas"	// Necessita traduccio
#elif L==2
	"Eingeladen"
#elif L==3
	"guests"
#elif L==4
	"invitadas"
#elif L==5
	"invit&eacute;es"
#elif L==6
	"invitadas"	// Okoteve traduccin
#elif L==7
	"ospiti"
#elif L==8
	"zaproszona"
#elif L==9
	"convidadas"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_MALE
	"convidats"
#elif L==2
	"Eingeladen"
#elif L==3
	"guests"
#elif L==4
	"invitados"
#elif L==5
	"invit&eacute;s"
#elif L==6
	"invitados"	// Okoteve traduccin
#elif L==7
	"ospiti"
#elif L==8
	"zaproszony"
#elif L==9
	"convidados"
#endif
	,
#if   L==1	// Rol_ROLE_GUEST__ - Usr_SEX_ALL
	"convidats"
#elif L==2
	"Eingeladen"
#elif L==3
	"guests"
#elif L==4
	"invitados"
#elif L==5
	"invit&eacute;s"
#elif L==6
	"invitados"	// Okoteve traduccin
#elif L==7
	"ospiti"
#elif L==8
	"zaproszeni"
#elif L==9
	"convidados"
#endif
	},
	{
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_UNKNOWN
	"visitants"
#elif L==2
	"Besucher"
#elif L==3
	"visitors"
#elif L==4
	"visitantes"
#elif L==5
	"visiteurs"
#elif L==6
	"visitantes"	// Okoteve traduccin
#elif L==7
	"visitatori"
#elif L==8
	"odwiedzaj&aogon;cy"
#elif L==9
	"visitantes"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_FEMALE
	"visitants"
#elif L==2
	"Besucher"
#elif L==3
	"visitors"
#elif L==4
	"visitantes"
#elif L==5
	"visiteurs"
#elif L==6
	"visitantes"	// Okoteve traduccin
#elif L==7
	"visitatori"
#elif L==8
	"odwiedzaj&aogon;cy"
#elif L==9
	"visitantes"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_MALE
	"visitants"
#elif L==2
	"Besucher"
#elif L==3
	"visitors"
#elif L==4
	"visitantes"
#elif L==5
	"visiteurs"
#elif L==6
	"visitantes"	// Okoteve traduccin
#elif L==7
	"visitatori"
#elif L==8
	"odwiedzaj&aogon;cy"
#elif L==9
	"visitantes"
#endif
	,
#if   L==1	// Rol_ROLE_VISITOR - Usr_SEX_ALL
	"visitants"
#elif L==2
	"Besucher"
#elif L==3
	"visitors"
#elif L==4
	"visitantes"
#elif L==5
	"visiteurs"
#elif L==6
	"visitantes"	// Okoteve traduccin
#elif L==7
	"visitatori"
#elif L==8
	"odwiedzaj&aogon;cy"
#elif L==9
	"visitantes"
#endif
	},
	{
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_UNKNOWN
	"estudiants"
#elif L==2
	"Studenten"
#elif L==3
	"students"
#elif L==4
	"estudiantes"
#elif L==5
	"&eacute;tudiants"
#elif L==6
	"temimbo'e"
#elif L==7
	"studenti"
#elif L==8
	"studenci"
#elif L==9
	"estudantes"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_FEMALE
	"estudiants"
#elif L==2
	"Studenten"
#elif L==3
	"students"
#elif L==4
	"estudiantes"
#elif L==5
	"&eacute;tudiantes"
#elif L==6
	"temimbo'e"
#elif L==7
	"studentesse"
#elif L==8
	"studenka"
#elif L==9
	"estudantes"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_MALE
	"estudiants"
#elif L==2
	"Studenten"
#elif L==3
	"students"
#elif L==4
	"estudiantes"
#elif L==5
	"&eacute;tudiants"
#elif L==6
	"temimbo'e"
#elif L==7
	"studenti"
#elif L==8
	"studentci"
#elif L==9
	"estudantes"
#endif
	,
#if   L==1	// Rol_ROLE_STUDENT - Usr_SEX_ALL
	"estudiants"
#elif L==2
	"Studenten"
#elif L==3
	"students"
#elif L==4
	"estudiantes"
#elif L==5
	"&eacute;tudiants"
#elif L==6
	"temimbo'e"
#elif L==7
	"studenti"
#elif L==8
	"studentci"
#elif L==9
	"estudantes"
#endif
	},
	{
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_UNKNOWN
	"professors"
#elif L==2
	"Lehrkr&auml;fte"
#elif L==3
	"teachers"
#elif L==4
	"profesores"
#elif L==5
	"enseignants"
#elif L==6
	"mbo'eh&aacute;ra"
#elif L==7
	"professori"
#elif L==8
	"nauczyciele"
#elif L==9
	"professores"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_FEMALE
	"professores"
#elif L==2
	"Lehrkr&auml;fte"
#elif L==3
	"teachers"
#elif L==4
	"profesoras"
#elif L==5
	"enseignantes"
#elif L==6
	"mbo'eh&aacute;ra"
#elif L==7
	"professoresse"
#elif L==8
	"nauczycielki"
#elif L==9
	"professoras"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_MALE
	"professors"
#elif L==2
	"Lehrkr&auml;fte"
#elif L==3
	"teachers"
#elif L==4
	"profesores"
#elif L==5
	"enseignants"
#elif L==6
	"mbo'eh&aacute;ra"
#elif L==7
	"professori"
#elif L==8
	"nauczyciele"
#elif L==9
	"professores"
#endif
	,
#if   L==1	// Rol_ROLE_TEACHER - Usr_SEX_ALL
	"professors"
#elif L==2
	"Lehrkr&auml;fte"
#elif L==3
	"teachers"
#elif L==4
	"profesores"
#elif L==5
	"enseignants"
#elif L==6
	"mbo'eh&aacute;ra"
#elif L==7
	"professori"
#elif L==8
	"nauczyciele"
#elif L==9
	"professores"
#endif
	},
	{
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_UNKNOWN
	"administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradores"
#elif L==5
	"administrateurs"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorzy"
#elif L==9
	"administradores"
#endif
	,
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_FEMALE
	"administradoras"	// Necessita traduccio
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradoras"
#elif L==5
	"administrateuses"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorki"
#elif L==9
	"administradoras"
#endif
	,
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_MALE
	"administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradores"
#elif L==5
	"administrateurs"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorzy"
#elif L==9
	"administradores"
#endif
	,
#if   L==1	// Rol_ROLE_DEG_ADM - Usr_SEX_ALL
	"administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradores"
#elif L==5
	"administrateurs"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorzy"
#elif L==9
	"administradores"
#endif
	},
	{
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_UNKNOWN
	"administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradores"
#elif L==5
	"administrateurs"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorzy"
#elif L==9
	"administradores"
#endif
	,
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_FEMALE
	"administradoras"	// Necessita traduccio
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradoras"
#elif L==5
	"administrateuses"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorki"
#elif L==9
	"administradoras"
#endif
	,
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_MALE
	"administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradores"
#elif L==5
	"administrateurs"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorzy"
#elif L==9
	"administradores"
#endif
	,
#if   L==1	// Rol_ROLE_CTR_ADM - Usr_SEX_ALL
	"administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradores"
#elif L==5
	"administrateurs"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorzy"
#elif L==9
	"administradores"
#endif
	 },
	 {
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_UNKNOWN
	"administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradores"
#elif L==5
	"administrateurs"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorzy"
#elif L==9
	"administradores"
#endif
	,
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_FEMALE
	"administradoras"	// Necessita traduccio
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradoras"
#elif L==5
	"administrateuses"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorki"
#elif L==9
	"administradoras"
#endif
	,
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_MALE
	"administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradores"
#elif L==5
	"administrateurs"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorzy"
#elif L==9
	"administradores"
#endif
	,
#if   L==1	// Rol_ROLE_INS_ADM - Usr_SEX_ALL
	"administradors"
#elif L==2
	"Administratoren"
#elif L==3
	"administrators"
#elif L==4
	"administradores"
#elif L==5
	"administrateurs"
#elif L==6
	"&ntilde;angarekoh&aacute;ra"
#elif L==7
	"amministratori"
#elif L==8
	"administratorzy"
#elif L==9
	"administradores"
#endif
	},
	{
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_UNKNOWN
	"superusuaris"
#elif L==2
	"Superuser"
#elif L==3
	"superusers"
#elif L==4
	"superusuarios"
#elif L==5
	"superutilisateurs"
#elif L==6
	"superusuarios"		// Okoteve traduccin
#elif L==7
	"superutenti"
#elif L==8
	"superusers"			// Potrzebujesz tlumaczenie
#elif L==9
	"superutilizadores"
#endif
	,
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_FEMALE
	"superusuarias"		// Necessita traduccio
#elif L==2
	"Superuser"
#elif L==3
	"superusers"
#elif L==4
	"superusuarias"
#elif L==5
	"superutilisateuses"
#elif L==6
	"superusuarias"		// Okoteve traduccin
#elif L==7
	"superutenti"
#elif L==8
	"superusers"			// Potrzebujesz tlumaczenie
#elif L==9
	"superutilizadoras"
#endif
	,
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_MALE
	"superusuaris"
#elif L==2
	"Superuser"
#elif L==3
	"superusers"
#elif L==4
	"superusuarios"
#elif L==5
	"superutilisateurs"
#elif L==6
	"superusuarios"		// Okoteve traduccin
#elif L==7
	"superutenti"
#elif L==8
	"superusers"			// Potrzebujesz tlumaczenie
#elif L==9
	"superutilizadores"
#endif
	,
#if   L==1	// Rol_ROLE_SYS_ADM - Usr_SEX_ALL
	"superusuaris"
#elif L==2
	"Superuser"
#elif L==3
	"superusers"
#elif L==4
	"superusuarios"
#elif L==5
	"superutilisateurs"
#elif L==6
	"superusuarios"		// Okoteve traduccin
#elif L==7
	"superutenti"
#elif L==8
	"superusers"			// Potrzebujesz tlumaczenie
#elif L==9
	"superutilizadores"
#endif
	 } };

const char *Txt_Rooms_with_users =
#if   L==1
	"Salas con usuarios";		// Necessita traduccio
#elif L==2
	"Rooms with users";		// Need bersetzung
#elif L==3
	"Rooms with users";
#elif L==4
	"Salas con usuarios";
#elif L==5
	"Rooms with users";		// Besoin de traduction
#elif L==6
	"Salas con usuarios";		// Okoteve traduccin
#elif L==7
	"Rooms con utenti";
#elif L==8
	"Rooms with users";		// Potrzebujesz tlumaczenie
#elif L==9
	"Rooms with users";		// Necessita de traduo
#endif

const char *Txt_ROOT_FOLDER_EXTERNAL_NAMES[Brw_NUM_TYPES_FILE_BROWSER] =
   {
	 // Brw_UNKNOWN
	 "",
	 // Brw_SHOW_DOCUM_CRS
#if   L==1
	 "documents"
#elif L==2
	 "Dokumente"
#elif L==3
	 "documents"
#elif L==4
	 "documentos"
#elif L==5
	 "documents"
#elif L==6
	 "kuatia"
#elif L==7
	 "documenti"
#elif L==8
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==9
	 "documentos"
#endif
	 ,
	 // Brw_SHOW_MARKS_CRS
#if   L==1
	 "qualificacions"
#elif L==2
	 "Bewertungen"
#elif L==3
	 "marks"
#elif L==4
	 "calificaciones"
#elif L==5
	 "notes"
#elif L==6
	 "calificaciones"	// Okoteve traduccin
#elif L==7
	 "risultati"
#elif L==8
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==9
	 "notas"
#endif
	 ,
	 // Brw_ADMI_DOCUM_CRS
#if   L==1
	 "documents"
#elif L==2
	 "Dokumente"
#elif L==3
	 "documents"
#elif L==4
	 "documentos"
#elif L==5
	 "documents"
#elif L==6
	 "kuatia"
#elif L==7
	 "documenti"
#elif L==8
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==9
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_SHARE_CRS
#if   L==1
	 "compartits"
#elif L==2
	 "Freigegebene"
#elif L==3
	 "shared"
#elif L==4
	 "compartidos"
#elif L==5
	 "partages"
#elif L==6
	 "compartidos"			// Okoteve traduccin
#elif L==7
	 "condivisi"
#elif L==8
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==9
	 "comum"
#endif
	 ,
	 // Brw_ADMI_SHARE_GRP
#if   L==1
	 "compartits"
#elif L==2
	 "Freigegebene"
#elif L==3
	 "shared"
#elif L==4
	 "compartidos"
#elif L==5
	 "partages"
#elif L==6
	 "compartidos"			// Okoteve traduccin
#elif L==7
	 "condivisi"
#elif L==8
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==9
	 "comum"
#endif
	 ,
	 // Brw_ADMI_WORKS_USR
#if   L==1
	 "treballs"
#elif L==2
	 "Hausarbeiten"
#elif L==3
	 "homework"
#elif L==4
	 "trabajos"
#elif L==5
	 "travaux"
#elif L==6
	 "tembiapo"
#elif L==7
	 "lavori"
#elif L==8
	 "homework"		// Potrzebujesz tlumaczenie
#elif L==9
	 "trabalhos"
#endif
	 ,
	 // Brw_ADMI_WORKS_CRS
#if   L==1
	 "treballs"
#elif L==2
	 "Hausarbeiten"
#elif L==3
	 "homework"
#elif L==4
	 "trabajos"
#elif L==5
	 "travaux"
#elif L==6
	 "tembiapo"
#elif L==7
	 "lavori"
#elif L==8
	 "homework"		// Potrzebujesz tlumaczenie
#elif L==9
	 "trabalhos"
#endif
	 ,
	 // Brw_ADMI_MARKS_CRS
#if   L==1
	 "qualificacions"
#elif L==2
	 "Bewertungen"
#elif L==3
	 "marks"
#elif L==4
	 "calificaciones"
#elif L==5
	 "notes"
#elif L==6
	 "calificaciones"	// Okoteve traduccin
#elif L==7
	 "risultati"
#elif L==8
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==9
	 "notas"
#endif
	 ,
	 // Brw_ADMI_BRIEF_USR
#if   L==1
	 "maleti"
#elif L==2
	 "Aktentasche"
#elif L==3
	 "briefcase"
#elif L==4
	 "maletin"
#elif L==5
	 "porte-documents"
#elif L==6
	 "maletin"		// Okoteve traduccin
#elif L==7
	 "cartella"
#elif L==8
	 "briefcase"		// Potrzebujesz tlumaczenie
#elif L==9
	 "pasta"
#endif
	 ,
	 // Brw_SHOW_DOCUM_GRP
#if   L==1
	 "documents"
#elif L==2
	 "Dokumente"
#elif L==3
	 "documents"
#elif L==4
	 "documentos"
#elif L==5
	 "documents"
#elif L==6
	 "kuatia"
#elif L==7
	 "documenti"
#elif L==8
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==9
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_DOCUM_GRP
#if   L==1
	 "documents"
#elif L==2
	 "Dokumente"
#elif L==3
	 "documents"
#elif L==4
	 "documentos"
#elif L==5
	 "documents"
#elif L==6
	 "kuatia"
#elif L==7
	 "documenti"
#elif L==8
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==9
	 "documentos"
#endif
	 ,
	 // Brw_SHOW_MARKS_GRP
#if   L==1
	 "qualificacions"	// Necessita traduccio
#elif L==2
	 "Bewertungen"
#elif L==3
	 "marks"
#elif L==4
	 "calificaciones"
#elif L==5
	 "notes"
#elif L==6
	 "calificaciones"	// Okoteve traduccin
#elif L==7
	 "risultati"
#elif L==8
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==9
	 "notas"
#endif
	 ,
	 // Brw_ADMI_MARKS_GRP
#if   L==1
	 "qualificacions"
#elif L==2
	 "Bewertungen"
#elif L==3
	 "marks"
#elif L==4
	 "calificaciones"
#elif L==5
	 "notes"
#elif L==6
	 "calificaciones"	// Okoteve traduccin
#elif L==7
	 "risultati"
#elif L==8
	 "marks"		// Potrzebujesz tlumaczenie
#elif L==9
	 "notas"
#endif
	 ,
	 // Brw_ADMI_ASSIG_USR
#if   L==1
	 "activitats"
#elif L==2
	 "Aufgaben"
#elif L==3
	 "assignments"
#elif L==4
	 "actividades"
#elif L==5
	 "activites"
#elif L==6
	 "tar&eacute;a"
#elif L==7
	 "attivit&agrave;"
#elif L==8
	 "assignments"		// Potrzebujesz tlumaczenie
#elif L==9
	 "atividades"
#endif
	 ,
	 // Brw_ADMI_ASSIG_CRS
#if   L==1
	 "activitats"
#elif L==2
	 "Aufgaben"
#elif L==3
	 "assignments"
#elif L==4
	 "actividades"
#elif L==5
	 "activites"
#elif L==6
	 "tar&eacute;a"
#elif L==7
	 "attivit&agrave;"
#elif L==8
	 "assignments"		// Potrzebujesz tlumaczenie
#elif L==9
	 "atividades"
#endif
	 ,
	 // Brw_SHOW_DOCUM_DEG
#if   L==1
	 "documents"
#elif L==2
	 "Dokumente"
#elif L==3
	 "documents"
#elif L==4
	 "documentos"
#elif L==5
	 "documents"
#elif L==6
	 "kuatia"
#elif L==7
	 "documenti"
#elif L==8
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==9
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_DOCUM_DEG
#if   L==1
	 "documents"
#elif L==2
	 "Dokumente"
#elif L==3
	 "documents"
#elif L==4
	 "documentos"
#elif L==5
	 "documents"
#elif L==6
	 "kuatia"
#elif L==7
	 "documenti"
#elif L==8
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==9
	 "documentos"
#endif
	 ,
	 // Brw_SHOW_DOCUM_CTR
#if   L==1
	 "documents"
#elif L==2
	 "Dokumente"
#elif L==3
	 "documents"
#elif L==4
	 "documentos"
#elif L==5
	 "documents"
#elif L==6
	 "kuatia"
#elif L==7
	 "documenti"
#elif L==8
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==9
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_DOCUM_CTR
#if   L==1
	 "documents"
#elif L==2
	 "Dokumente"
#elif L==3
	 "documents"
#elif L==4
	 "documentos"
#elif L==5
	 "documents"
#elif L==6
	 "kuatia"
#elif L==7
	 "documenti"
#elif L==8
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==9
	 "documentos"
#endif
	 ,
	 // Brw_SHOW_DOCUM_INS
#if   L==1
	 "documents"
#elif L==2
	 "Dokumente"
#elif L==3
	 "documents"
#elif L==4
	 "documentos"
#elif L==5
	 "documents"
#elif L==6
	 "kuatia"
#elif L==7
	 "documenti"
#elif L==8
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==9
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_DOCUM_INS
#if   L==1
	 "documents"
#elif L==2
	 "Dokumente"
#elif L==3
	 "documents"
#elif L==4
	 "documentos"
#elif L==5
	 "documents"
#elif L==6
	 "kuatia"
#elif L==7
	 "documenti"
#elif L==8
	 "documents"		// Potrzebujesz tlumaczenie
#elif L==9
	 "documentos"
#endif
	 ,
	 // Brw_ADMI_SHARE_DEG
#if   L==1
	 "compartits"
#elif L==2
	 "Freigegebene"
#elif L==3
	 "shared"
#elif L==4
	 "compartidos"
#elif L==5
	 "partages"
#elif L==6
	 "compartidos"			// Okoteve traduccin
#elif L==7
	 "condivisi"
#elif L==8
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==9
	 "comum"
#endif
	 ,
	 // Brw_ADMI_SHARE_CTR
#if   L==1
	 "compartits"
#elif L==2
	 "Freigegebene"
#elif L==3
	 "shared"
#elif L==4
	 "compartidos"
#elif L==5
	 "partages"
#elif L==6
	 "compartidos"			// Okoteve traduccin
#elif L==7
	 "condivisi"
#elif L==8
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==9
	 "comum"
#endif
	 ,
	 // Brw_ADMI_SHARE_INS
#if   L==1
	 "compartits"
#elif L==2
	 "Freigegebene"
#elif L==3
	 "shared"
#elif L==4
	 "compartidos"
#elif L==5
	 "partages"
#elif L==6
	 "compartidos"			// Okoteve traduccin
#elif L==7
	 "condivisi"
#elif L==8
	 "shared"			// Potrzebujesz tlumaczenie
#elif L==9
	 "comum"
#endif
	 ,
	 // Brw_ADMI_TEACH_CRS
#if   L==1
	"privats"
#elif L==2
	"Private"
#elif L==3
	"private"
#elif L==4
	"privados"
#elif L==5
	"priv&eacute;s"
#elif L==6
	"privados"	// Okoteve traduccin
#elif L==7
	"privati"
#elif L==8
	"prywatne"
#elif L==9
	"privados"
#endif
	 ,
	 // Brw_ADMI_TEACH_GRP
#if   L==1
	"privats"
#elif L==2
	"Private"
#elif L==3
	"private"
#elif L==4
	"privados"
#elif L==5
	"priv&eacute;s"
#elif L==6
	"privados"	// Okoteve traduccin
#elif L==7
	"privati"
#elif L==8
	"prywatne"
#elif L==9
	"privados"
#endif
	};

const char *Txt_Save =
#if   L==1
	"Guardar";
#elif L==2
	"Speichern";
#elif L==3
	"Save";
#elif L==4
	"Guardar";
#elif L==5
	"Enregistrer";
#elif L==6
	"&Ntilde;ongatu";
#elif L==7
	"Salvare";
#elif L==8
	"Zapisa&cacute;";
#elif L==9
	"Salvar";
#endif

const char *Txt_Save_as =
#if   L==1
	"Guardar com";
#elif L==2
	"Speichern unter";
#elif L==3
	"Save as";
#elif L==4
	"Guardar como";
#elif L==5
	"Enregistrer sous";
#elif L==6
	"Guardar como";	// Okoteve traduccin
#elif L==7
	"Salva come";
#elif L==8
	"Zapisz jako";
#elif L==9
	"Salvar como";
#endif

const char *Txt_Save_changes =
#if   L==1
	"Desar canvis";
#elif L==2
	"&Auml;nderungen speichern";
#elif L==3
	"Save changes";
#elif L==4
	"Guardar cambios";
#elif L==5
	"Enregistrer les modifications";
#elif L==6
	"&Ntilde;ongatu";
#elif L==7
	"Salva modifiche";
#elif L==8
	"Zapisz zmiany";
#elif L==9
	"Salvar altera&ccedil;&otilde;es";
#endif

const char *Txt_Save_file_properties =
#if   L==1
	"Guardar les propietats del fitxer";
#elif L==2
	"Speichern Dateieigenschaften";
#elif L==3
	"Save file properties";
#elif L==4
	"Guardar propiedades del archivo";
#elif L==5
	"Enregistrer les propri&eacute;t&eacute;s du fichier";
#elif L==6
	"Guardar propiedades del archivo";	// Okoteve traduccin
#elif L==7
	"Salva le propriet&agrave; dei file";
#elif L==8
	"Zapisz w&lstrok;a&sacute;ciwo&sacute;ci pliku";
#elif L==9
	"Salvar as propriedades do arquivo";
#endif

const char *Txt_Scope =
#if   L==1
	"&Agrave;mbit";
#elif L==2
	"Geltungsbereich";
#elif L==3
	"Scope";
#elif L==4
	"&Aacute;mbito";
#elif L==5
	"Port&eacute;e";
#elif L==6
	"&Aacute;mbito";	// Okoteve traduccin
#elif L==7
	"Ambito";
#elif L==8
	"Scope";		// Potrzebujesz tlumaczenie
#elif L==9
	"&Acirc;mbito";
#endif

const char *Txt_Score =
#if   L==1
	"Puntuaci&oacute;n";	// Necessita traduccio
#elif L==2
	"Ergebnis";
#elif L==3
	"Score";
#elif L==4
	"Puntuaci&oacute;n";
#elif L==5
	"Note";
#elif L==6
	"Puntuaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Punteggio";
#elif L==8
	"Score";		// Potrzebujesz tlumaczenie
#elif L==9
	"Pontua&ccedil;&atilde;o";
#endif

const char *Txt_Search =
#if   L==1
	"Cercar";
#elif L==2
	"Suchen";
#elif L==3
	"Search";
#elif L==4
	"Buscar";
#elif L==5
	"Chercher";
#elif L==6
	"Heka";
#elif L==7
	"Cercare";
#elif L==8
	"Szukaj";
#elif L==9
	"Procurar";
#endif

const char *Txt_Search_courses =
#if   L==1
	"Cercar assignatures";
#elif L==2
	"Suchen Kursen";
#elif L==3
	"Search courses";
#elif L==4
	"Buscar asignaturas";
#elif L==5
	"Chercher mati&egrave;res";
#elif L==6
	"Heka mbo'esyry";
#elif L==7
	"Cercare corsi";
#elif L==8
	"Szukaj kurs&oacute;w";
#elif L==9
	"Procurar disciplinas";
#endif

const char *Txt_See_full_notice =
#if   L==1
	"Veure avs complet";
#elif L==2
	"Siehe voll Ank&uuml;ndigung";
#elif L==3
	"See full notice";
#elif L==4
	"Ver aviso completo";
#elif L==5
	"Voir l'avis complet";
#elif L==6
	"Ver aviso completo";	// Okoteve traduccin
#elif L==7
	"Vedi avviso completo ";
#elif L==8
	"Wiadomo&sacute;ci ca&lstrok;y obwieszczenie";
#elif L==9
	"Veja aviso completo";
#endif

const char *Txt_See_message =
#if   L==1
	"Veure missatge";
#elif L==2
	"Siehe Nachricht";
#elif L==3
	"See message";
#elif L==4
	"Ver mensaje";
#elif L==5
	"Voir message";
#elif L==6
	"Ver mensaje";	// Okoteve traduccin
#elif L==7
	"Vedi messaggio";
#elif L==8
	"See message";	// Potrzebujesz tlumaczenie
#elif L==9
	"Veja mensagem";
#endif

const char *Txt_See_messages_received =
#if   L==1
	"Veure missatges rebuts";
#elif L==2
	"Siehe Empfangene Nachrichten";
#elif L==3
	"See messages received";
#elif L==4
	"Ver mensajes recibidos";
#elif L==5
	"Voir messages re&ccedil;us";
#elif L==6
	"Ver mensajes recibidos";	// Okoteve traduccin
#elif L==7
	"Vedi messaggi ricevuti";
#elif L==8
	"Wiadomo&sacute;ci odebrane";
#elif L==9
	"Veja mensagens recebidas";
#endif

const char *Txt_See_more =
#if   L==1
	 "Veure m&eacute;s";
#elif L==2
	 "Weitere";
#elif L==3
	 "See more";
#elif L==4
	 "Ver m&aacute;s";
#elif L==5
	 "Voir plus";
#elif L==6
	 "Ver m&aacute;s";	// Okoteve traduccin
#elif L==7
	 "Vedere pi&ugrave;";
#elif L==8
	 "Zobacz wi&eogon;cej";
#elif L==9
	 "Veja mais";
#endif

const char *Txt_See_new_activity =
#if   L==1
	 "Veure nova activitat";
#elif L==2
	 "Siehe neue Aktivit&auml;t";
#elif L==3
	 "See new activity";
#elif L==4
	 "Ver nueva actividad";
#elif L==5
	 "Voir nouvelle activit&eacute;";
#elif L==6
	 "Ver nueva actividad";	// Okoteve traduccin
#elif L==7
	 "Vedere nuova attivit&agrave;";
#elif L==8
	 "Zobacz now&aogon; aktywno&sacute;&cacute;";
#elif L==9
	 "Veja nova atividade";
#endif

const char *Txt_See_notifications =
#if   L==1
	"Ver notificaciones";	// Necessita traduccio
#elif L==2
	"Siehe Anmeldungen";
#elif L==3
	"See notifications";
#elif L==4
	"Ver notificaciones";
#elif L==5
	"Voir notifications";
#elif L==6
	"Ver notificaciones";	// Okoteve traduccin
#elif L==7
	"Vedi notifiche";
#elif L==8
	"See notifications";	// Potrzebujesz tlumaczenie
#elif L==9
	"Veja notifica&ccedil;&otilde;es";
#endif

const char *Txt_See_page_X_of_Y =	// Warning: it is very important to include two %u in the following sentences
#if   L==1
	"Ver p&aacute;gina %u de %u";	// Necessita traduccio
#elif L==2
	"Siehe Seite %u von %u";
#elif L==3
	"See page %u of %u";
#elif L==4
	"Ver p&aacute;gina %u de %u";
#elif L==5
	"Voir page %u de %u";
#elif L==6
	"Ver p&aacute;gina %u de %u";	// Okoteve traduccin
#elif L==7
	"Vedi pagina %u di %u";
#elif L==8
	"See page %u of %u";		// Potrzebujesz tlumaczenie
#elif L==9
	"Veja p&aacute;gina %u de %u";
#endif

const char *Txt_Select_another_country =
#if   L==1
	"Seleccionar un altre pas";
#elif L==2
	"W&auml;hlen Sie eine anderes Land";
#elif L==3
	"Select another country";
#elif L==4
	"Seleccionar otro pa&iacute;s";
#elif L==5
	"S&eacute;lectionnez un autre pays";
#elif L==6
	"Seleccionar otro pa&iacute;s";		// Okoteve traduccin
#elif L==7
	"Selezionare un altro paese";
#elif L==8
	"Wybierz inny kraj";
#elif L==9
	"Selecionar outro pa&iacute;s";
#endif

const char *Txt_Select_one_country =
#if   L==1
	"Seleccionar un pa&iacute;s";
#elif L==2
	"W&auml;hlen Sie ein Land";
#elif L==3
	"Select one country";
#elif L==4
	"Seleccionar un pa&iacute;s";
#elif L==5
	"S&eacute;lectionnez un pays";
#elif L==6
	"Seleccionar un pa&iacute;s";		// Okoteve traduccin
#elif L==7
	"Selezionare un paese";
#elif L==8
	"Wybierz jeden kraj";
#elif L==9
	"Selecionar um pa&iacute;s";
#endif

const char *Txt_Select_or_create_another_centre_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Seleccionar o crear un altre centre en la instituci&oacute; <strong>%s</strong>";
#elif L==2
	"W&auml;hlen oder erstellen Sie einen anderen Lehrinstitut der Hochschule <strong>%s</strong>";
#elif L==3
	"Select or create another centre in the institution <strong>%s</strong>";
#elif L==4
	"Seleccionar o crear otro centro en la instituci&oacute;n <strong>%s</strong>";
#elif L==5
	"S&eacute;lectionnez ou cr&eacute;ez un autre &eacute;tude du &eacute;tablissement <strong>%s</strong>";
#elif L==6
	"Seleccionar o crear otro centro en la instituci&oacute;n <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Selezionare o creare un altro centro di istituzione <strong>%s</strong>";
#elif L==8
	"Select or create another centre in the institution <strong>%s</strong>";	// Potrzebujesz tlumaczenie
#elif L==9
	"Selecionar ou criar um outro centro da institu&ccedil;&atilde;o <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_another_course_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Seleccionar o crear una altra assignatura en <strong>%s</strong>";
#elif L==2
	"W&auml;hlen oder erstellen Sie einen anderen Kurs von <strong>%s</strong>";
#elif L==3
	"Select or create another course in <strong>%s</strong>";
#elif L==4
	"Seleccionar o crear otra asignatura en <strong>%s</strong>";
#elif L==5
	"S&eacute;lectionnez ou cr&eacute;ez une autre mati&egrave;re dans <strong>%s</strong>";
#elif L==6
	"Seleccionar o crear otra asignatura en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Selezionare o creare un altro corso di <strong>%s</strong>";
#elif L==8
	"Select or create another course in <strong>%s</strong>";	// Potrzebujesz tlumaczenie
#elif L==9
	"Selecionar ou criar uma outra disciplina de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_another_degree_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Seleccionar o crear una altra titulaci&oacute; en <strong>%s</strong>";
#elif L==2
	"W&auml;hlen oder erstellen Sie einen anderen Studiengang von <strong>%s</strong>";
#elif L==3
	"Select or create another degree in <strong>%s</strong>";
#elif L==4
	"Seleccionar o crear otra titulaci&oacute;n en <strong>%s</strong>";
#elif L==5
	"S&eacute;lectionnez ou cr&eacute;ez un autre &eacute;tude dans <strong>%s</strong>";
#elif L==6
	"Seleccionar o crear otra titulaci&oacute;n en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Selezionare o creare una altra laurea di <strong>%s</strong>";
#elif L==8
	"Select or create another degree in <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==9
	"Selecionar ou criar uma outra titula&ccedil;&atilde;o de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_another_institution_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Seleccionar o crear una altra instituci&oacute; en <strong>%s</strong>";
#elif L==2
	"W&auml;hlen oder erstellen Sie einen anderen Hochschule von <strong>%s</strong>";
#elif L==3
	"Select or create another institution in <strong>%s</strong>";
#elif L==4
	"Seleccionar o crear otra instituci&oacute;n en <strong>%s</strong>";
#elif L==5
	"S&eacute;lectionnez ou cr&eacute;ez un autre &eacute;tablissement de <strong>%s</strong>";
#elif L==6
	"Seleccionar o crear otra instituci&oacute;n en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Selezionare o creare una altra istituzione di <strong>%s</strong>";
#elif L==8
	"Select or create another institution in <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==9
	"Selecionar ou criar uma outra institu&ccedil;&atilde;o de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_one_centre_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Seleccionar o crear un centre en <strong>%s</strong>";
#elif L==2
	"W&auml;hlen oder erstellen Sie einen Lehrinstitut von <strong>%s</strong>";
#elif L==3
	"Select or create one centre in <strong>%s</strong>";
#elif L==4
	"Seleccionar o crear un centro en <strong>%s</strong>";
#elif L==5
	"S&eacute;lectionnez ou cr&eacute;ez un &eacute;tude dans <strong>%s</strong>";
#elif L==6
	"Seleccionar o crear un centro en <strong>%s</strong>";		// Okoteve traduccin
#elif L==7
	"Selezionare o creare un centro di <strong>%s</strong>";
#elif L==8
	"Select or create one degree in <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==9
	"Selecionar ou criar um centro de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_one_course_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Seleccionar o crear una assignatura en <strong>%s</strong>";
#elif L==2
	"W&auml;hlen oder erstellen Sie eine Kurs von <strong>%s</strong>";
#elif L==3
	"Select or create one course in <strong>%s</strong>";
#elif L==4
	"Seleccionar o crear una asignatura en <strong>%s</strong>";
#elif L==5
	"S&eacute;lectionnez ou cr&eacute;ez une mati&egrave;re dans <strong>%s</strong>";
#elif L==6
	"Seleccionar o crear una asignatura en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Selezionare o creare un corso di <strong>%s</strong>";
#elif L==8
	"Select or create one course in <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==9
	"Selecionar ou criar uma disciplina de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_one_degree_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Seleccionar o crear una titulaci&oacute; en <strong>%s</strong>";
#elif L==2
	"W&auml;hlen oder erstellen Sie einen Studiengang von <strong>%s</strong>";
#elif L==3
	"Select or create one degree in <strong>%s</strong>";
#elif L==4
	"Seleccionar o crear una titulaci&oacute;n en <strong>%s</strong>";
#elif L==5
	"S&eacute;lectionnez ou cr&eacute;ez un &eacute;tude dans <strong>%s</strong>";
#elif L==6
	"Seleccionar o crear una titulaci&oacute;n en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Selezionare o creare una laurea di <strong>%s</strong>";
#elif L==8
	"Select or create one degree in <strong>%s</strong>";			// Potrzebujesz tlumaczenie
#elif L==9
	"Selecionar ou criar uma titula&ccedil;&atilde;o de <strong>%s</strong>";
#endif

const char *Txt_Select_or_create_one_institution_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Seleccionar o crear una instituci&oacute; en <strong>%s</strong>";
#elif L==2
	"W&auml;hlen oder erstellen Sie einen Hochschule von <strong>%s</strong>";
#elif L==3
	"Select or create one institution in <strong>%s</strong>";
#elif L==4
	"Seleccionar o crear una instituci&oacute;n en <strong>%s</strong>";
#elif L==5
	"S&eacute;lectionnez ou cr&eacute;ez un &eacute;tablissement de <strong>%s</strong>";
#elif L==6
	"Seleccionar o crear una instituci&oacute;n en <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Selezionare o creare una istituzione di <strong>%s</strong>";
#elif L==8
	"Select or create one institution in <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==9
	"Selecionar ou criar uma institu&ccedil;&atilde;o de <strong>%s</strong>";
#endif

const char *Txt_Select_one_or_more_files_from_your_computer_or_drag_and_drop_here =
#if   L==1
	"Seleccioneu un o m&eacute;s fitxers del seu ordinador<br />"
	"o arrossegueu i deixeu-los anar aqu&iacute;";
#elif L==2
	"W&auml;hlen Sie eine oder mehrere Dateien von Ihrem Computer<br />"
	"oder per Drag & Drop hier";
#elif L==3
	"Select one or more files from your computer<br />"
	"or drag and drop here";
#elif L==4
	"Seleccione uno o m&aacute;s archivos de su ordenador<br />"
	"o arr&aacute;strelos y su&eacute;ltelos aqu&iacute;";
#elif L==5
	"S&eacute;lectionnez un ou plusieurs fichiers &agrave; partir de votre ordinateur<br />"
	"ou glisser-d&eacute;poser ici";
#elif L==6
	"Seleccione uno o m&aacute;s archivos de su ordenador<br />"
	"o arr&aacute;strelos y su&eacute;ltelos aqu&iacute;";	// Okoteve traduccin
#elif L==7
	"Selezionare uno o pi&ugrave; file dal tuo computer<br />"
	"o trascinare e rilasciare qui";
#elif L==8
	"Wybierz jeden lub wi&eogon;cej plik&oacute;w z komputera<br />"
	"lub przeci&aogon;gnij i upu&sacute;&cacute; tutaj";
#elif L==9
	"Selecione um ou mais arquivos de seu computador<br />"
	"ou arrastar e soltar aqui";
#endif

const char *Txt_Select_the_groups_in_from_which_you_want_to_register_remove_users_ =
#if   L==1
	"Seleccione los grupos en/de los que quiere inscribir/eliminar usuarios."
	" Solo puede seleccionar aquellos grupos a los que usted pertenezca.<br />"
	"Si selecciona grupos, los usuarios ser&aacute;n inscritos en la asignatura y en los grupos seleccionados,"
	" o bien eliminados de los grupos seleccionados pero no de la asignatura.<br />"
	"Si no selecciona ning&uacute;n grupo, los usuarios ser&aacute;n inscritos en la asignatura pero no en grupos concretos,"
	" o bien eliminados de la asignatura y de todos los grupos.";	// Necessita traduccio
#elif L==2
	"Select the groups in/from which you want to register/remove users."
	"You can only select those groups to which you belong.<br />"
	"If you select any groups, users will be enrolled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enrolled in the course but not in specific groups,"
	" or removed from the course and from all groups.";	// Need bersetzung
#elif L==3
	"Select the groups in/from which you want to register/remove users."
	"You can only select those groups to which you belong.<br />"
	"If you select any groups, users will be enrolled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enrolled in the course but not in specific groups,"
	" or removed from the course and from all groups.";
#elif L==4
	"Seleccione los grupos en/de los que quiere inscribir/eliminar usuarios."
	" Solo puede seleccionar aquellos grupos a los que usted pertenezca.<br />"
	"Si selecciona grupos, los usuarios ser&aacute;n inscritos en la asignatura y en los grupos seleccionados,"
	" o bien eliminados de los grupos seleccionados pero no de la asignatura.<br />"
	"Si no selecciona ning&uacute;n grupo, los usuarios ser&aacute;n inscritos en la asignatura pero no en grupos concretos,"
	" o bien eliminados de la asignatura y de todos los grupos.";
#elif L==5
	"Select the groups in/from which you want to register/remove users."
	"You can only select those groups to which you belong.<br />"
	"If you select any groups, users will be enrolled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enrolled in the course but not in specific groups,"
	" or removed from the course and from all groups.";	// Besoin de traduction
#elif L==6
	"Seleccione los grupos en/de los que quiere inscribir/eliminar usuarios."
	" Solo puede seleccionar aquellos grupos a los que usted pertenezca.<br />"
	"Si selecciona grupos, los usuarios ser&aacute;n inscritos en la asignatura y en los grupos seleccionados,"
	" o bien eliminados de los grupos seleccionados pero no de la asignatura.<br />"
	"Si no selecciona ning&uacute;n grupo, los usuarios ser&aacute;n inscritos en la asignatura pero no en grupos concretos,"
	" o bien eliminados de la asignatura y de todos los grupos.";	// Okoteve traduccin
#elif L==7
	"Seleziona i gruppi nel/dal quale vuoi registrare/rimuovere utenti."
	"Puoi solo selezionare quei gruppi ai quali appartieni.<br />"
	"Se selezioni qualche gruppo, gli utenti saranno registrati nel corso e nel gruppo selezionato,"
	" o rimossi dai gruppi selezionati ma non dal corso.<br />"
	"Se non selezioni alcun gruppo, gli utenti saranno registrati nel corso ma non in gruppi specifici,"
	" o rimossi dal corso e da tutti i gruppi.";
#elif L==8
	"Select the groups in/from which you want to register/remove users."
	"You can only select those groups to which you belong.<br />"
	"If you select any groups, users will be enrolled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enrolled in the course but not in specific groups,"
	" or removed from the course and from all groups.";	// Potrzebujesz tlumaczenie
#elif L==9
	"Select the groups in/from which you want to register/remove users."
	"You can only select those groups to which you belong.<br />"
	"If you select any groups, users will be enrolled in the course and the selected groups,"
	" or removed from the selected groups but not from the course.<br />"
	"If you do not select any groups, users will be enrolled in the course but not in specific groups,"
	" or removed from the course and from all groups.";	// Necessita de traduo
#endif

const char *Txt_Send =
#if   L==1
	"Enviar";
#elif L==2
	"Senden";
#elif L==3
	"Send";
#elif L==4
	"Enviar";
#elif L==5
	"Envoyer";
#elif L==6
	"Mondo";
#elif L==7
	"Invia";
#elif L==8
	"Send";		// Potrzebujesz tlumaczenie
#elif L==9
	"Enviar";
#endif

const char *Txt_Set_password =
#if   L==1
	"Establir contrasenya";
#elif L==2
	"Passwort festlegen";
#elif L==3
	"Set password";
#elif L==4
	"Establecer contrase&ntilde;a";
#elif L==5
	"D&eacute;finir mot de passe";
#elif L==6
	"Establecer contrase&ntilde;a";	// Okoteve traduccin
#elif L==7
	"Impostare password";
#elif L==8
	"Ustaw has&lstrok;o";
#elif L==9
	"Definir senha";
#endif

const char *Txt_Send_message =
#if   L==1
	"Enviar missatge";
#elif L==2
	"Senden Nachricht";
#elif L==3
	"Send message";
#elif L==4
	"Enviar mensaje";
#elif L==5
	"Envoyer message";
#elif L==6
	"Enviar mensaje";	// Okoteve traduccin
#elif L==7
	"Invia messaggio";
#elif L==8
	"Wy&sacute;lij wiadomo&sacute;&cacute;";
#elif L==9
	"Enviar mensagem";
#endif

const char *Txt_Send_survey =
#if   L==1
	"Enviar encuesta";	// Necessita traduccio
#elif L==2
	"Senden Umfrage";
#elif L==3
	"Send survey";
#elif L==4
	"Enviar encuesta";
#elif L==5
	"Envoyer sondage";
#elif L==6
	"Enviar encuesta";	// Okoteve traduccin
#elif L==7
	"Invia sondaggio";
#elif L==8
	"Send survey";		// Potrzebujesz tlumaczenie
#elif L==9
	"Enviar inqu&eacute;rito";
#endif

const char *Txt_Send_URL =
#if   L==1
	"Enviar URL";		// Necessita traduccio
#elif L==2
	"Senden URL";
#elif L==3
	"Send URL";
#elif L==4
	"Enviar URL";
#elif L==5
	"Envoyer URL";
#elif L==6
	"Enviar URL";		// Okoteve traduccin
#elif L==7
	"Invia URL";
#elif L==8
	"Send URL";		// Potrzebujesz tlumaczenie
#elif L==9
	"Enviar URL";
#endif

const char *Txt_Sender_banned_click_to_unban_him =
#if   L==1
	"Remitente bloqueado/a; pulse para desbloquearlo/a";	// Necessita traduccio
#elif L==2
	"Sender banned; click to unban her/him";		// Need bersetzung
#elif L==3
	"Sender banned; click to unban her/him";
#elif L==4
	"Remitente bloqueado/a; pulse para desbloquearlo/a";
#elif L==5
	"Sender banned; click to unban her/him";		// Besoin de traduction
#elif L==6
	"Remitente bloqueado/a; pulse para desbloquearlo/a";	// Okoteve traduccin
#elif L==7
	"Sender banned; click to unban her/him";		// Bisogno di traduzione
#elif L==8
	"Sender banned; click to unban her/him";		// Potrzebujesz tlumaczenie
#elif L==9
	"Sender banned; click to unban her/him";		// Necessita de traduo
#endif

const char *Txt_Sender_permitted_click_to_ban_him =
#if   L==1
	"Remitente permitido/a; pulse para bloquearlo/a";	// Necessita traduccio
#elif L==2
	"Sender permitted; click to ban her/him";		// Need bersetzung
#elif L==3
	"Sender permitted; click to ban her/him";
#elif L==4
	"Remitente permitido/a; pulse para bloquearlo/a";
#elif L==5
	"Sender permitted; click to ban her/him";		// Besoin de traduction
#elif L==6
	"Remitente permitido/a; pulse para bloquearlo/a";	// Okoteve traduccin
#elif L==7
	"Sender permitted; click to ban her/him";		// Bisogno di traduzione
#elif L==8
	"Sender permitted; click to ban her/him";		// Potrzebujesz tlumaczenie
#elif L==9
	"Sender permitted; click to ban her/him";		// Necessita de traduo
#endif

const char *Txt_session =
#if   L==1
	"sesi&oacute;n";	// Necessita traduccio
#elif L==2
	"Sitzung";
#elif L==3
	"session";
#elif L==4
	"sesi&oacute;n";
#elif L==5
	"session";
#elif L==6
	"sesi&oacute;n";	// Okoteve traduccin
#elif L==7
	"sessione";
#elif L==8
	"sesja";
#elif L==9
	"sess&atilde;o";
#endif

const char *Txt_sessions =
#if   L==1
	"sessions";
#elif L==2
	"Sitzungen";
#elif L==3
	"sessions";
#elif L==4
	"sesiones";
#elif L==5
	"sessions";
#elif L==6
	"sesiones";	// Okoteve traduccin
#elif L==7
	"sessioni";
#elif L==8
	"sessions";	// Potrzebujesz tlumaczenie
#elif L==9
	"sess&otilde;es";
#endif

const char *Txt_Sex =
#if   L==1
	"Sexe";
#elif L==2
	"Sex";
#elif L==3
	"Sex";
#elif L==4
	"Sexo";
#elif L==5
	"Sexe";
#elif L==6
	"Me&ntilde;a";
#elif L==7
	"Sesso";
#elif L==8
	"Ple&cacute;";
#elif L==9
	"Sexo";
#endif

const char *Txt_SEX_PLURAL_Abc[Usr_NUM_SEXS] =
   {
#if   L==1	// Usr_SEX_UNKNOWN
	 "Desconocidos"	// Necessita traduccio
#elif L==2
	 "Unbekannt"
#elif L==3
	 "Unknown"
#elif L==4
	 "Desconocidos"
#elif L==5
	 "Inconnus"
#elif L==6
	 "Desconocidos"	// Okoteve traduccin
#elif L==7
	 "Sconosciuti"
#elif L==8
	 "Nieznany"
#elif L==9
	 "Desconhecidos"
#endif
	 ,
#if   L==1	// Usr_SEX_FEMALE
	 "Dones"	// Necessita traduccio
#elif L==2
	 "Frauen"
#elif L==3
	 "Women"
#elif L==4
	 "Mujeres"
#elif L==5
	 "Femmes"
#elif L==6
	 "Mujeres"	// Okoteve traduccin
#elif L==7
	 "Donne"
#elif L==8
	 "Kobiety"
#elif L==9
	 "Mulheres"
#endif
	 ,
#if   L==1	// Usr_SEX_MALE
	 "Homes"
#elif L==2
	 "M&auml;nner"
#elif L==3
	 "Men"
#elif L==4
	 "Hombres"
#elif L==5
	 "Hommes"
#elif L==6
	 "Hombres"	// Okoteve traduccin
#elif L==7
	 "Uomini"
#elif L==8
	 "Mezczyzni"
#elif L==9
	 "Homens"
#endif
	 ,
#if   L==1	// Usr_SEX_ALL
	 "Todos"	// Necessita traduccio
#elif L==2
	 "Alle"
#elif L==3
	 "All"
#elif L==4
	 "Todos"
#elif L==5
	 "Tous"
#elif L==6
	 "Todos"	// Okoteve traduccin
#elif L==7
	 "Tutti"
#elif L==8
	 "Wszyscy"
#elif L==9
	 "Todos"
#endif
	};

const char *Txt_SEX_PLURAL_abc[Usr_NUM_SEXS] =
   {
#if   L==1	// Usr_SEX_UNKNOWN
	 "desconocidos"	// Necessita traduccio
#elif L==2
	 "unbekannt"
#elif L==3
	 "unknown"
#elif L==4
	 "desconocidos"
#elif L==5
	 "inconnus"
#elif L==6
	 "desconocidos"	// Okoteve traduccin
#elif L==7
	 "sconosciuti"
#elif L==8
	 "nieznany"
#elif L==9
	 "desconhecidos"
#endif
	 ,
#if   L==1	// Usr_SEX_FEMALE
	 "dones"
#elif L==2
	 "Frauen"
#elif L==3
	 "women"
#elif L==4
	 "mujeres"
#elif L==5
	 "femmes"
#elif L==6
	 "mujeres"	// Okoteve traduccin
#elif L==7
	 "donne"
#elif L==8
	 "kobiety"
#elif L==9
	 "mulheres"
#endif
	 ,
#if   L==1	// Usr_SEX_MALE
	 "homes"
#elif L==2
	 "M&auml;nner"
#elif L==3
	 "men"
#elif L==4
	 "hombres"
#elif L==5
	 "hommes"
#elif L==6
	 "hombres"	// Okoteve traduccin
#elif L==7
	 "uomini"
#elif L==8
	 "mezczyzni"
#elif L==9
	 "homens"
#endif
	 ,
#if   L==1	// Usr_SEX_ALL
	 "todos"	// Necessita traduccio
#elif L==2
	 "alle"
#elif L==3
	 "all"
#elif L==4
	 "todos"
#elif L==5
	 "tous"
#elif L==6
	 "todos"	// Okoteve traduccin
#elif L==7
	 "tutti"
#elif L==8
	 "wszyscy"
#elif L==9
	 "todos"
#endif
	};

const char *Txt_SEX_SINGULAR_Abc[Usr_NUM_SEXS] =
   {
#if   L==1	// Usr_SEX_UNKNOWN
	 "Desconocido"	// Necessita traduccio
#elif L==2
	 "Unbekannt"
#elif L==3
	 "Unknown"
#elif L==4
	 "Desconocido"
#elif L==5
	 "Inconnu"
#elif L==6
	 "Desconocido"	// Okoteve traduccin
#elif L==7
	 "Sconosciuto"
#elif L==8
	 "Nieznany"
#elif L==9
	 "Desconhecido"
#endif
	 ,
#if   L==1	// Usr_SEX_FEMALE
	 "Dona"
#elif L==2
	 "Frau"
#elif L==3
	 "Woman"
#elif L==4
	 "Mujer"
#elif L==5
	 "Femme"
#elif L==6
	 "Ku&ntilde;a"
#elif L==7
	 "Donna"
#elif L==8
	 "Kobieta"
#elif L==9
	 "Mulher"
#endif
	 ,
#if   L==1	// Usr_SEX_MALE
	 "Home"
#elif L==2
	 "Mann"
#elif L==3
	 "Man"
#elif L==4
	 "Hombre"
#elif L==5
	 "Homme"
#elif L==6
	 "Kuimba'e"
#elif L==7
	 "Uomo"
#elif L==8
	 "M&eogon;&zdot;czyzna"
#elif L==9
	 "Homem"
#endif
	 ,
#if   L==1	// Usr_SEX_ALL
	 "Todos"		// Necessita traduccio
#elif L==2
	 "Alle"
#elif L==3
	 "All"
#elif L==4
	 "Todos"
#elif L==5
	 "Tous"
#elif L==6
	 "Todos"		// Okoteve traduccin
#elif L==7
	 "Tutti"
#elif L==8
	 "Wwszyscy"
#elif L==9
	 "Todos"
#endif
	};

const char *Txt_Share =
#if   L==1
	"Compartir";
#elif L==2
	"Teilen";
#elif L==3
	"Share";
#elif L==4
	"Compartir";
#elif L==5
	"Partager";
#elif L==6
	"Compartir";	// Okoteve traduccin
#elif L==7
	"Condividere";
#elif L==8
	"Dzieli&cacute;";
#elif L==9
	"Compartilhar";
#endif

const char *Txt_Shared_files_area =
#if   L==1
	"Zona d'arxius compartits";
#elif L==2
	"Freigegebene Dateien-Bereich";
#elif L==3
	"Shared files area";
#elif L==4
	"Zona de archivos compartidos";
#elif L==5
	"Zone de fichiers partag&eacute;s";
#elif L==6
	"Zona de archivos compartidos";	// Okoteve traduccin
#elif L==7
	"Area comune";
#elif L==8
	"Wsp&oacute;lnej dla";
#elif L==9
	"Zona comum";
#endif

const char *Txt_shared_files_area =
#if   L==1
	"zona d'arxius compartits";
#elif L==2
	"Freigegebene Dateien-Bereich";
#elif L==3
	"shared files area";
#elif L==4
	"zona de archivos compartidos";
#elif L==5
	"zone de fichiers partag&eacute;s";
#elif L==6
	"zona de archivos compartidos";	// Okoteve traduccin
#elif L==7
	"zona di file condivisi";
#elif L==8
	"Powierzchnia udost&eogon;pnionych plik&oacute;w";
#elif L==9
	"zona de arquivos compartilhados";
#endif

const char *Txt_Short_name =
#if   L==1
	"Nom breu";
#elif L==2
	"Kurzname";
#elif L==3
	"Short name";
#elif L==4
	"Nombre breve";
#elif L==5
	"Nom abr&eacute;g&eacute;";
#elif L==6
	"Nombre breve";		// Okoteve traduccin
#elif L==7
	"Nome breve";
#elif L==8
	"Nazwa skr&oacute;cona";
#elif L==9
	"Nome abreviado";
#endif

const char *Txt_Short_name_of_the_degree =
#if   L==1
	"Nom breu de la titulaci&oacute;";
#elif L==2
	"Kurzname der Studiengang";
#elif L==3
	"Short name of the degree";
#elif L==4
	"Nombre breve de la titulaci&oacute;n";
#elif L==5
	"Nom abr&eacute;g&eacute; du &eacute;tude";
#elif L==6
	"Nombre breve de la titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Nome breve della laurea";
#elif L==8
	"Skr&oacute;cona nazwa stopie&nacute;";
#elif L==9
	"Nome abreviado da titula&ccedil;&atilde;o";
#endif

const char *Txt_Short_name_of_the_centre =
#if   L==1
	"Nom breu del centre";
#elif L==2
	"Kurzname der Lehrinstitut";
#elif L==3
	"Short name of the centre";
#elif L==4
	"Nombre breve del centro";
#elif L==5
	"Nom abr&eacute;g&eacute; du centre";
#elif L==6
	"Nombre breve del centro";	// Okoteve traduccin
#elif L==7
	"Nome breve del centro";
#elif L==8
	"Skr&oacute;cona nazwa centrum";
#elif L==9
	"Nome abreviado do centro";
#endif

const char *Txt_Short_name_of_the_course =
#if   L==1
	"Nom breu de l'assignatura";
#elif L==2
	"Kurzname der Kurs";
#elif L==3
	"Short name of the course";
#elif L==4
	"Nombre breve de la asignatura";
#elif L==5
	"Nom abr&eacute;g&eacute; de la mati&egrave;re";
#elif L==6
	"Nombre breve de la asignatura";	// Okoteve traduccin
#elif L==7
	"Nome breve del corso";
#elif L==8
	"Skr&oacute;cona nazwa kurs";
#elif L==9
	"Nome abreviado da disciplina";
#endif

const char *Txt_Short_name_of_the_institution =
#if   L==1
	"Nom breu de la instituci&oacute;";
#elif L==2
	"Kurzname der Institution";
#elif L==3
	"Short name of the institution";
#elif L==4
	"Nombre breve de la instituci&oacute;n";
#elif L==5
	"Nom abr&eacute;g&eacute; de l'institution";
#elif L==6
	"Nombre breve de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Nome breve dell'istituzione";
#elif L==8
	"Skr&oacute;cona nazwa instytucji";
#elif L==9
	"Nome abreviado da institui&ccedil;&atilde;o";
#endif

const char *Txt_Shortcut =
#if   L==1
	"Acc&eacute;s directe";
#elif L==2
	"Direktzugriff";
#elif L==3
	"Shortcut";
#elif L==4
	"Acceso directo";
#elif L==5
	"Acc&egrave;s direct";
#elif L==6
	"Acceso directo";	// Okoteve traduccin
#elif L==7
	"Accesso diretto";
#elif L==8
	"Bezpo&sacute;redni dost&eogon;p";
#elif L==9
	"Acesso directo";
#endif

const char *Txt_Show =
#if   L==1
	"Mostra";
#elif L==2
	"Zeige";
#elif L==3
	"Show";
#elif L==4
	"Mostrar";
#elif L==5
	"Voir";
#elif L==6
	"Mostrar";	// Okoteve traduccin
#elif L==7
	"Visualizza";
#elif L==8
	"Pokaza&cacute;";
#elif L==9
	"Mostrar";
#endif

const char *Txt_Show_FOLDER_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Mostra %s";
#elif L==2
	"Zeige %s";
#elif L==3
	"Show %s";
#elif L==4
	"Mostrar %s";
#elif L==5
	"Voir %s";
#elif L==6
	"Mostrar %s";	// Okoteve traduccin
#elif L==7
	"Visualizza %s";
#elif L==8
	"Pokaza&cacute; %s";
#elif L==9
	"Mostrar %s";
#endif

const char *Txt_Show_all_data =
#if   L==1
	"Mostrar totes les dades";
#elif L==2
	"Alle Daten";
#elif L==3
	"Show all data";
#elif L==4
	"Mostrar todos los datos";
#elif L==5
	"Voir toutes les donn&eacute;es";
#elif L==6
	"Mostrar todos los datos";	// Okoteve traduccin
#elif L==7
	"Visualizza tutti i dati";
#elif L==8
	"Show all data";		// Potrzebujesz tlumaczenie
#elif L==9
	"Mostrar todos os dados";
#endif

const char *Txt_Show_all_files =
#if   L==1
	"Mostrar tots els fitxers";
#elif L==2
	"Alle Dateien anzeigen";
#elif L==3
	"Show all files";
#elif L==4
	"Mostrar todos los archivos";
#elif L==5
	"Afficher tous les fichiers";
#elif L==6
	"Mostrar todos los archivos";	// Okoteve traduccin
#elif L==7
	"Mostra tutti i file";
#elif L==8
	"Poka&zdot; wszystkie pliki";
#elif L==9
	"Mostrar todos os arquivos";
#endif

const char *Txt_Show_all_NOTIFICATIONS =
#if   L==1
	"Mostrar tots";
#elif L==2
	"Alle anzeigen";
#elif L==3
	"Show all";
#elif L==4
	"Mostrar todas";
#elif L==5
	"Afficher tous";
#elif L==6
	"Mostrar todas";	// Okoteve traduccin
#elif L==7
	"Mostra tutti";
#elif L==8
	"Poka&zdot; wszystkie";
#elif L==9
	"Mostrar todas";
#endif

const char *Txt_Show_all_notifications =
#if   L==1
	"Mostrar tots les notificacions";
#elif L==2
	"Alle Anmeldungen anzeigen";
#elif L==3
	"Show all notifications";
#elif L==4
	"Mostrar todas las notificaciones";
#elif L==5
	"Afficher tous les notifications";
#elif L==6
	"Mostrar todas las notificaciones";	// Okoteve traduccin
#elif L==7
	"Mostra tutti le notifiche";
#elif L==8
	"Poka&zdot; wszystkie powiadomienia";
#elif L==9
	"Mostrar todas as notifica&ccedil;&otilde;es";
#endif

const char *Txt_Show_agenda =
#if   L==1
	"Mostra agenda";
#elif L==2
	"Organizer anzeigen";
#elif L==3
	"Show agenda";
#elif L==4
	"Ver agenda";
#elif L==5
	"Afficher agenda";
#elif L==6
	"Ver agenda";	// Okoteve traduccin
#elif L==7
	"Mostra agenda";
#elif L==8
	"Poka&zdot; pami&eogon;tnik";
#elif L==9
	"Mostrar agenda";
#endif

const char *Txt_Show_anyway =
#if   L==1
	"Mostra de totes maneres";
#elif L==2
	"Zeigesowieso";
#elif L==3
	"Show anyway";
#elif L==4
	"Mostrar de todos modos";
#elif L==5
	"Voir de toute faon";
#elif L==6
	"Mostrar de todos modos";	// Okoteve traduccin
#elif L==7
	"Visualizza comunque";
#elif L==8
	"Show anyway";			// Potrzebujesz tlumaczenie
#elif L==9
	"Mostrar de qualquer maneira";
#endif

const char *Txt_Show_calendar =
#if   L==1
	"Mostra calendari";
#elif L==2
	"Kalender anzeigen";
#elif L==3
	"Show calendar";
#elif L==4
	"Ver calendario";
#elif L==5
	"Afficher calendrier";
#elif L==6
	"Ver calendario";	// Okoteve traduccin
#elif L==7
	"Mostra calendario";
#elif L==8
	"Poka&zdot; kalendarz";
#elif L==9
	"Mostrar calend&aacute;rio";
#endif

const char *Txt_Show_hits =	// hits = visits, clicks, page views...
#if   L==1
	"Veure accessos";
#elif L==2
	"Zeige Anmeldungen";
#elif L==3
	"Show hits";
#elif L==4
	"Ver accesos";
#elif L==5
	"Voir visites";
#elif L==6
	"Ver accesos";		// Okoteve traduccin
#elif L==7
	"Visualizza visite";
#elif L==8
	"Show hits";		// Potrzebujesz tlumaczenie
#elif L==9
	"Mostrar visitas";
#endif

const char *Txt_Show_list =
#if   L==1
	"Mostra la llista";
#elif L==2
	"Liste anzeigen";
#elif L==3
	"Show list";
#elif L==4
	"Ver lista";
#elif L==5
	"Afficher la liste";
#elif L==6
	"Ver lista";	// Okoteve traduccin
#elif L==7
	"Mostra elenco";
#elif L==8
	"Poka&zdot; list&eogon;";
#elif L==9
	"Mostrar lista";
#endif

const char *Txt_Show_WHICH_events[2] =
   {
#if   L==1
	 "Tots els esdeveniments"
#elif L==2
	 "Alle Ereignisse"
#elif L==3
	 "All events"
#elif L==4
	 "Todos los eventos"
#elif L==5
	 "Tous les &eacute;v&eacute;nements"
#elif L==6
	 "Todos los eventos"	// Okoteve traduccin
#elif L==7
	 "Tutti i eventi"
#elif L==8
	 "Wszystkie imprezy"
#elif L==9
	 "Todos os eventos"
#endif
	 ,
#if   L==1
	 "Nom&eacute;s els esdeveniments p&uacute;blics i visibles"
#elif L==2
	 "Nur &ouml;ffentliche und sichtbare Ereignisse"
#elif L==3
	 "Only public, visible events"
#elif L==4
	 "Solo eventos p&uacute;blicos y visibles"
#elif L==5
	 "Uniquement les &eacute;v&eacute;nements publics et visibles"
#elif L==6
	 "Solo eventos p&uacute;blicos y visibles"	// Okoteve traduccin
#elif L==7
	 "Solo eventi pubblici e visibili"
#elif L==8
	 "Tylko wydarzenia publiczne i widoczne"
#elif L==9
	 "Apenas eventos p&uacute;blicos e vis&iacute;veis"
#endif
	};

const char *Txt_Show_WHICH_groups[2] =
   {
#if   L==1
	 "Nom&eacute;s els meus grups"
#elif L==2
	 "Nur meine Gruppen"
#elif L==3
	 "Only my groups"
#elif L==4
	 "Solo mis grupos"
#elif L==5
	 "Uniquement mes groupes"
#elif L==6
	 "Solo mis grupos"	// Okoteve traduccin
#elif L==7
	 "Solo i miei gruppi"
#elif L==8
	 "Only my groups"			// Potrzebujesz tlumaczenie
#elif L==9
	 "Apenas os meus grupos"
#endif
	 ,
#if   L==1
	 "Tots els grups"
#elif L==2
	 "Alle Gruppen"
#elif L==3
	 "All groups"
#elif L==4
	 "Todos los grupos"
#elif L==5
	 "Tous les groupes"
#elif L==6
	 "Todos los grupos"	// Okoteve traduccin
#elif L==7
	 "Tutti i gruppi"
#elif L==8
	 "All groups"		// Potrzebujesz tlumaczenie
#elif L==9
	 "Todos os grupos"
#endif
	};

const char *Txt_Show_more_details =
#if   L==1
	"Mostra m&eacute;s detalls";
#elif L==2
	"Mehr Details";
#elif L==3
	"Show more details";
#elif L==4
	"Mostrar m&aacute;s detalles";
#elif L==5
	"Montrer plus de d&eacute;tails";
#elif L==6
	"Mostrar m&aacute;s detalles";	// Okoteve traduccin
#elif L==7
	"Mostra pi&ugrave; dettagli";
#elif L==8
	"Poka&zdot; wi&eogon;cej szczeg&oacute;&lstrok;&oacute;w";
#elif L==9
	"Mostrar mais detalhes";
#endif

const char *Txt_Show_more_recipients =
#if   L==1
	"Veure m&eacute;s recipients";
#elif L==2
	"Mehrere Empf&auml;nger anzeigen";
#elif L==3
	"Show more recipients";
#elif L==4
	"Ver m&aacute;s destinatarios";
#elif L==5
	"Montrer plus de destinataires";
#elif L==6
	"Ver m&aacute;s destinatarios";	// Okoteve traduccin
#elif L==7
	"Mostra pi&ugrave; destinatari";
#elif L==8
	"Poka&zdot; wi&eogon;cej odbiorc&oacute;w";
#elif L==9
	"Ver mais destinat&aacute;rios";
#endif

const char *Txt_Show_next_X_clicks =	// Warning: it is very important to include %lu in the following sentences
#if   L==1
	"Mostrar %lu clics posteriores";	// Necessita traduccio
#elif L==2
	"Show next %lu clicks";			// Need bersetzung
#elif L==3
	"Show next %lu clicks";
#elif L==4
	"Mostrar %lu clics posteriores";
#elif L==5
	"Show next %lu clicks";			// Besoin de traduction
#elif L==6
	"Mostrar %lu clics posteriores";	// Okoteve traduccin
#elif L==7
	"Mostra %lu scatti seguenti";
#elif L==8
	"Show next %lu clicks";			// Potrzebujesz tlumaczenie
#elif L==9
	"Show next %lu clicks";			// Necessita de traduo
#endif

const char *Txt_Show_office_hours =
#if   L==1
	"Mostra horaris tutories";
#elif L==2
	"Gesch&auml;ftszeiten anzeigen";
#elif L==3
	"Show office hours";
#elif L==4
	"Ver horarios tutor&iacute;as";
#elif L==5
	"Voir horaires tuteur";
#elif L==6
	"Ver horarios tutor&iacute;as";	// Okoteve traduccin
#elif L==7
	"Mostra orari tutoraggi";
#elif L==8
	"Poka&zdot; godzin pracy biura";
#elif L==9
	"Mostrar hor&aacute;rios tutor";
#endif

const char *Txt_Show_previous_X_clicks =	// Warning: it is very important to include %lu in the following sentences
#if   L==1
	"Mostrar %lu clics anteriores";		// Necessita traduccio
#elif L==2
	"Show previous %lu clicks";		// Need bersetzung
#elif L==3
	"Show previous %lu clicks";
#elif L==4
	"Mostrar %lu clics anteriores";
#elif L==5
	"Show previous %lu clicks";		// Besoin de traduction
#elif L==6
	"Mostrar %lu clics anteriores";		// Okoteve traduccin
#elif L==7
	"Mostra %lu scatti precedenti";
#elif L==8
	"Show previous %lu clicks";		// Potrzebujesz tlumaczenie
#elif L==9
	"Show previous %lu clicks";		// Necessita de traduo
#endif

const char *Txt_Show_questions =
#if   L==1
	"Mostrar preguntes";
#elif L==2
	"Zeige Testfragen";
#elif L==3
	"Show questions";
#elif L==4
	"Mostrar preguntas";
#elif L==5
	"Voir les questions";
#elif L==6
	"Mostrar preguntas";	// Okoteve traduccin
#elif L==7
	"Visualizza le domande";
#elif L==8
	"Show questions";	// Potrzebujesz tlumaczenie
#elif L==9
	"Mostrar quest&otilde;es";
#endif

const char *Txt_Show_records =
#if   L==1
	"Mostrar fichas";	// Necessita traduccio
#elif L==2
	"Zeige Profile";
#elif L==3
	"Show records";
#elif L==4
	"Mostrar fichas";
#elif L==5
	"Voir les cartes";
#elif L==6
	"Mostrar fichas";	// Okoteve traduccin
#elif L==7
	"Visualizza le schede";
#elif L==8
	"Show records";		// Potrzebujesz tlumaczenie
#elif L==9
	"Mostrar cart&otilde;es";
#endif

const char *Txt_Show_statistic =
#if   L==1
	"Mostrar estad&iacute;stica";
#elif L==2
	"Zeige Statistik";
#elif L==3
	"Show statistic";
#elif L==4
	"Mostrar estad&iacute;stica";
#elif L==5
	"Voir statistique";
#elif L==6
	"Mostrar estad&iacute;stica";	// Okoteve traduccin
#elif L==7
	"Visualizza statistica";
#elif L==8
	"Show statistic";		// Potrzebujesz tlumaczenie
#elif L==9
	"Mostrar estat&iacute;stica";
#endif

const char *Txt_Show_timetable =
#if   L==1
	"Veure horari";
#elif L==2
	"Zeige Stundenplan";
#elif L==3
	"Show timetable";
#elif L==4
	"Ver horario";
#elif L==5
	"Voir horaire";
#elif L==6
	"Ver horario";	// Okoteve traduccin
#elif L==7
	"Vedi orario";
#elif L==8
	"Wiadomo&sacute;ci plan lekcji";
#elif L==9
	"Veja hor&aacute;rio";
#endif

const char *Txt_Shuffle =
#if   L==1
	"Barrejar";
#elif L==2
	"Shuffle";
#elif L==3
	"Shuffle";
#elif L==4
	"Barajar";
#elif L==5
	"M&eacute;langer";
#elif L==6
	"Barajar";	// Okoteve traduccin
#elif L==7
	"Mischiare";
#elif L==8
	"Shuffle";	// Potrzebujesz tlumaczenie
#elif L==9
	"Baralhar";
#endif

const char *Txt_Sign_up =
#if   L==1
	"Sol&middot;licitar inscripci&oacute;";
#elif L==2
	"Registrierung beantragen";
#elif L==3
	"Sign up";
#elif L==4
	"Solicitar inscripci&oacute;n";
#elif L==5
	"Demander inscription";
#elif L==6
	"Solicitar inscripci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Richiedere iscrizione";
#elif L==8
	"Wniosku rejestracj&eogon;";
#elif L==9
	"Solicitar inscri&ccedil;&atilde;o";
#endif

const char *Txt_Similar_users =
#if   L==1
	"Usuaris similars";
#elif L==2
	"&Auml;hnliche Benutzer";
#elif L==3
	"Similar users";
#elif L==4
	"Usuarios similares";
#elif L==5
	"Utilisateurs similaires";
#elif L==6
	"Usuarios similares";	// Okoteve traduccin
#elif L==7
	"Utenti simili";
#elif L==8
	"Podobne u&zdot;ytkownik&oacute;w";
#elif L==9
	"Usu&aacute;rios similares";
#endif

const char *Txt_Size =		// ...of a file zone
#if   L==1
	"Mida";
#elif L==2
	"Gr&ouml;&szlig;e";
#elif L==3
	"Size";
#elif L==4
	"Tama&ntilde;o";
#elif L==5
	"Taille";
#elif L==6
	"Tama&ntilde;o";	// Okoteve traduccin
#elif L==7
	"Dimensione";
#elif L==8
	"Size";			// Potrzebujesz tlumaczenie
#elif L==9
	"Tamanho";
#endif

const char *Txt_Size_of_photos =
#if   L==1
	"Mida de les fotos";
#elif L==2
	"Gr&ouml;&szlig;e des Fotos";
#elif L==3
	"Size of photos";
#elif L==4
	"Tama&ntilde;o de las fotos";
#elif L==5
	"Taille des photos";
#elif L==6
	"Tama&ntilde;o de las fotos";	// Okoteve traduccin
#elif L==7
	"Dimensioni delle foto";
#elif L==8
	"Rozmiar zdj&eogon;&cacute;";
#elif L==9
	"Tamanho das fotos";
#endif

const char *Txt_SOCIAL_NOTE[Soc_NUM_NOTE_TYPES] =
	{
#if   L==1	// Soc_NOTE_UNKNOWN
	 "Esdeveniment desconegut"
#elif L==2
	 "Unbekannt Ereignis"
#elif L==3
	 "Unknown event"
#elif L==4
	 "Evento desconocido"
#elif L==5
	 "&Eacute;v&eacute;nement inconnu"
#elif L==6
	 "Evento desconocido"	// Okoteve traduccin
#elif L==7
	 "Evento sconosciuto"
#elif L==8
	 "Nieznane zdarzenia"
#elif L==9
	 "Evento desconhecido"
#endif
	 ,
#if   L==1	// Soc_NOTE_INS_DOC_PUB_FILE
	 "Document p&uacute;blic"
#elif L==2
	 "&Ouml;ffentliche Dokumentdatei"
#elif L==3
	 "Public document file"
#elif L==4
	 "Documento p&uacute;blico"
#elif L==5
	 "Fichier de document public"
#elif L==6
	 "Documento p&uacute;blico"	// Okoteve traduccin
#elif L==7
	 "Documento pubblico"
#elif L==8
	 "Plik dokumentu publiczne"
#elif L==9
	 "Arquivo de documento p&uacute;blico"
#endif
	 ,
#if   L==1	// Soc_NOTE_INS_SHA_PUB_FILE
	 "Arxiu compartit p&uacute;blic"
#elif L==2
	 "&Ouml;ffentliche Freigegebene Datei"
#elif L==3
	 "Public shared file"
#elif L==4
	 "Archivo compartido p&uacute;blico"
#elif L==5
	 "Fichier partag&eacute; public"
#elif L==6
	 "Archivo compartido p&uacute;blico"	// Okoteve traduccin
#elif L==7
	 "File condiviso pubblico"
#elif L==8
	 "Udost&eogon;pniony plik publiczne"
#elif L==9
	 "Arquivo compartilhado p&uacute;blico"
#endif
	 ,
#if   L==1	// Soc_NOTE_CTR_DOC_PUB_FILE
	 "Document p&uacute;blic"
#elif L==2
	 "&Ouml;ffentliche Dokumentdatei"
#elif L==3
	 "Public document file"
#elif L==4
	 "Documento p&uacute;blico"
#elif L==5
	 "Fichier de document public"
#elif L==6
	 "Documento p&uacute;blico"	// Okoteve traduccin
#elif L==7
	 "Documento pubblico"
#elif L==8
	 "Plik dokumentu publiczne"
#elif L==9
	 "Arquivo de documento p&uacute;blico"
#endif
	 ,
#if   L==1	// Soc_NOTE_CTR_SHA_PUB_FILE
	 "Arxiu compartit p&uacute;blic"
#elif L==2
	 "&Ouml;ffentliche Freigegebene Datei"
#elif L==3
	 "Public shared file"
#elif L==4
	 "Archivo compartido p&uacute;blico"
#elif L==5
	 "Fichier partag&eacute; public"
#elif L==6
	 "Archivo compartido p&uacute;blico"	// Okoteve traduccin
#elif L==7
	 "File condiviso pubblico"
#elif L==8
	 "Udost&eogon;pniony plik publiczne"
#elif L==9
	 "Arquivo compartilhado p&uacute;blico"
#endif
	 ,
#if   L==1	// Soc_NOTE_DEG_DOC_PUB_FILE
	 "Document p&uacute;blic"
#elif L==2
	 "&Ouml;ffentliche Dokumentdatei"
#elif L==3
	 "Public document file"
#elif L==4
	 "Documento p&uacute;blico"
#elif L==5
	 "Fichier de document public"
#elif L==6
	 "Documento p&uacute;blico"	// Okoteve traduccin
#elif L==7
	 "Documento pubblico"
#elif L==8
	 "Plik dokumentu publiczne"
#elif L==9
	 "Arquivo de documento p&uacute;blico"
#endif
	 ,
#if   L==1	// Soc_NOTE_DEG_SHA_PUB_FILE
	 "Arxiu compartit p&uacute;blic"
#elif L==2
	 "&Ouml;ffentliche Freigegebene Datei"
#elif L==3
	 "Public shared file"
#elif L==4
	 "Archivo compartido p&uacute;blico"
#elif L==5
	 "Fichier partag&eacute; public"
#elif L==6
	 "Archivo compartido p&uacute;blico"	// Okoteve traduccin
#elif L==7
	 "File condiviso pubblico"
#elif L==8
	 "Udost&eogon;pniony plik publiczne"
#elif L==9
	 "Arquivo compartilhado p&uacute;blico"
#endif
	 ,
#if   L==1	// Soc_NOTE_CRS_DOC_PUB_FILE
	 "Document p&uacute;blic"
#elif L==2
	 "&Ouml;ffentliche Dokumentdatei"
#elif L==3
	 "Public document file"
#elif L==4
	 "Documento p&uacute;blico"
#elif L==5
	 "Fichier de document public"
#elif L==6
	 "Documento p&uacute;blico"	// Okoteve traduccin
#elif L==7
	 "Documento pubblico"
#elif L==8
	 "Plik dokumentu publiczne"
#elif L==9
	 "Arquivo de documento p&uacute;blico"
#endif
	 ,
#if   L==1	// Soc_NOTE_CRS_SHA_PUB_FILE
	 "Arxiu compartit p&uacute;blic"
#elif L==2
	 "&Ouml;ffentliche Freigegebene Datei"
#elif L==3
	 "Public shared file"
#elif L==4
	 "Archivo compartido p&uacute;blico"
#elif L==5
	 "Fichier partag&eacute; public"
#elif L==6
	 "Archivo compartido p&uacute;blico"	// Okoteve traduccin
#elif L==7
	 "File condiviso pubblico"
#elif L==8
	 "Udost&eogon;pniony plik publiczne"
#elif L==9
	 "Arquivo compartilhado p&uacute;blico"
#endif
	 ,
#if   L==1	// Soc_NOTE_EXAM_ANNOUNCEMENT
	 "Convocat&ograve;ria d'examen"
#elif L==2
	 "Aufrufe fr Pr&uuml;fung"
#elif L==3
	 "Announcement of exam"
#elif L==4
	 "Convocatoria de examen"
#elif L==5
	 "Convocation &agrave; un examen"
#elif L==6
	 "Convocatoria de examen"	// Okoteve traduccin
#elif L==7
	 "Appello d'esame"
#elif L==8
	 "Ogloszenie egzamin"
#elif L==9
	 "Chamada para exame"
#endif
	 ,
#if   L==1	// Soc_NOTE_SOCIAL_POST
	 "Missatge social"
#elif L==2
	 "Social Beitrag"
#elif L==3
	 "Social post"
#elif L==4
	 "Mensaje social"
#elif L==5
	 "Message social"
#elif L==6
	 "Mensaje social"	// Okoteve traduccin
#elif L==7
	 "Post sociale"
#elif L==8
	 "Post spo&lstrok;ecznej"
#elif L==9
	 "Post sociais"
#endif
	 ,
#if   L==1	// Soc_NOTE_FORUM_POST
	 "Missatge en un f&ograve;rum"
#elif L==2
	 "Beitr&auml;ge in einem Forum"
#elif L==3
	 "Post in a forum"
#elif L==4
	 "Mensaje en un foro"
#elif L==5
	 "Message dans un forum"
#elif L==6
	 "Mensaje en un foro"	// Okoteve traduccin
#elif L==7
	 "Post in un forum"
#elif L==8
	 "Post na forum"
#elif L==9
	 "Post em um f&oacute;rum"
#endif
	 ,
#if   L==1	// Soc_NOTE_NOTICE
	 "Av&iacute;s"
#elif L==2
	 "Ank&uuml;ndigungen"
#elif L==3
	 "Notice"
#elif L==4
	 "Aviso"
#elif L==5
	 "Avis"
#elif L==6
	 "Marandu"
#elif L==7
	 "Avviso"
#elif L==8
	 "Informacja"
#elif L==9
	 "An&uacute;ncio"
#endif
	};

const char *Txt_SOCIAL_NOTE_Favourite =
#if   L==1
	"Favorit";
#elif L==2
	"Favorit";
#elif L==3
	"Favourite";
#elif L==4
	"Favorito";
#elif L==5
	"Favori";
#elif L==6
	"Favorito";	// Okoteve traduccin
#elif L==7
	"Preferito";
#elif L==8
	"Ulubiony";
#elif L==9
	"Favorito";
#endif

const char *Txt_SOCIAL_NOTE_Favourited_by_X_USERS =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Marcat com favorit per %u";
#elif L==2
	"Von %u vorgemerkt";
#elif L==3
	"Favourited by %u";
#elif L==4
	"Marcado como favorito por %u";
#elif L==5
	"Ajout&eacute; aux favoris par %u";
#elif L==6
	"Marcado como favorito por %u";	// Okoteve traduccin
#elif L==7
	"Preferiti da %u";
#elif L==8
	"Dodane do ulubionych przez %u";
#elif L==9
	"Marcado como favorito por %u";
#endif

const char *Txt_SOCIAL_NOTE_Shared =
#if   L==1
	"Compartit";
#elif L==2
	"Geteilt";
#elif L==3
	"Shared";
#elif L==4
	"Compartido";
#elif L==5
	"Partag&eacute;";
#elif L==6
	"Compartido";	// Okoteve traduccin
#elif L==7
	"Condiviso";
#elif L==8
	"Udost&eogon;pnione";
#elif L==9
	"Compartilhado";
#endif

const char *Txt_SOCIAL_NOTE_Shared_by_X_USERS =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Compartit per %u";
#elif L==2
	"Von %u geteilt";
#elif L==3
	"Shared by %u";
#elif L==4
	"Compartido por %u";
#elif L==5
	"Partag&eacute; par %u";
#elif L==6
	"Compartido por %u";	// Okoteve traduccin
#elif L==7
	"Condiviso da %u";
#elif L==8
	"Udost&eogon;pnione przez %u";
#elif L==9
	"Compartilhado por %u";
#endif

const char *Txt_SOCIAL_NOTE_Not_favourited_by_anyone =	// No longer shared
#if   L==1
	"No marcat com favorit per ning&uacute;";
#elif L==2
	"Niemand hat sie als Favorit markiert";
#elif L==3
	"Not favourited by anyone";
#elif L==4
	"No marcado como favorito por nadie";
#elif L==5
	"Personne n'a marqu&eacute; comme favori";
#elif L==6
	"No marcado como favorito por nadie";	// Okoteve traduccin
#elif L==7
	"Non preferiti da chiunque";
#elif L==8
	"Nie przez nikogo ulubionych";
#elif L==9
	"N&atilde;o marcado como favorito por qualquer pessoa";
#endif

const char *Txt_SOCIAL_NOTE_Not_shared_by_anyone =	// No longer shared
#if   L==1
	"No compartit per ning&uacute;";
#elif L==2
	"Nicht durch Dritte freigegebenen";
#elif L==3
	"Not shared by anyone";
#elif L==4
	"No compartido por nadie";
#elif L==5
	"Personne n'a partag&eacute;";
#elif L==6
	"No compartido por nadie";	// Okoteve traduccin
#elif L==7
	"Non condiviso da chiunque";
#elif L==8
	"Nie jest dzielona przez nikogo";
#elif L==9
	"N&atilde;o compartilhado por qualquer pessoa";
#endif

const char *Txt_SOCIAL_NOTE_TOP_MESSAGES[Soc_NUM_TOP_MESSAGES] =
	{
	// Soc_TOP_MESSAGE_NONE
	""
	,
	// Soc_TOP_MESSAGE_COMMENTED:
#if   L==1
	"ha comentat"
#elif L==2
	"hat kommentiert"
#elif L==3
	"has commented"
#elif L==4
	"ha comentado"
#elif L==5
	"a comment&eacute;"
#elif L==6
	"ha comentado"	// Okoteve traduccin
#elif L==7
	"ha commentato"
#elif L==8
	"skomentowa&lstrok;"
#elif L==9
	"comentou"
#endif
	,
	// Soc_TOP_MESSAGE_FAVED:
#if   L==1
	"ha marcat com favorit"
#elif L==2
	"hat als Favorit markiert"
#elif L==3
	"has marked as favourite"
#elif L==4
	"ha marcado como favorito"
#elif L==5
	"a marqu&eacute; en tant que favori"
#elif L==6
	"ha marcado como favorito"	// Okoteve traduccin
#elif L==7
	"ha segnato come preferito"
#elif L==8
	"oznaczone jako ulubiony"
#elif L==9
	"marcou como favorito"
#endif
	,
	// Soc_TOP_MESSAGE_UNFAVED:
#if   L==1
	"ha desmarcat com favorit"
#elif L==2
	"hat als Favorit unmarkiert"
#elif L==3
	"has unmarked as favourite"
#elif L==4
	"ha desmarcado como favorito"
#elif L==5
	"a pas marqu&eacute; en tant que favori"
#elif L==6
	"ha desmarcado como favorito"	// Okoteve traduccin
#elif L==7
	"ha smarcato come preferito"
#elif L==8
	"nieoznaczonych jako ulubiony"
#elif L==9
	"desmarcou como favorito"
#endif
	,
	// Soc_TOP_MESSAGE_SHARED:
#if   L==1
	"ha compartit"
#elif L==2
	"hat sich geteilt"
#elif L==3
	"has shared"
#elif L==4
	"ha compartido"
#elif L==5
	"a partag&eacute;"
#elif L==6
	"ha compartido"	// Okoteve traduccin
#elif L==7
	"ha condiviso"
#elif L==8
	"podzieli&lstrok;"
#elif L==9
	"compartilhou"
#endif
	,
	// Soc_TOP_MESSAGE_UNSHARED:
#if   L==1
	"ha deixat de compartir"
#elif L==2
	"hat Sharing gestoppt"
#elif L==3
	"has stopped sharing"
#elif L==4
	"ha dejado de compartir"
#elif L==5
	"a cess&eacute; de partage"
#elif L==6
	"ha dejado de compartir"	// Okoteve traduccin
#elif L==7
	"ha smesso di condividere"
#elif L==8
	"przesta&lstrok; udost&eogon;pniania"
#elif L==9
	"parou compartilhar"
#endif
	,
	// Soc_TOP_MESSAGE_MENTIONED:
#if   L==1
	"li ha esmentat"
#elif L==2
	"hat Sie erw&auml;hnt"
#elif L==3
	"has mentioned you"
#elif L==4
	"le ha mencionado"
#elif L==5
	"vous a mentionn&eacute;"
#elif L==6
	"le ha mencionado"	// Okoteve traduccin
#elif L==7
	"ti ha menzionato"
#elif L==8
	"wspomnia&lstrok; o Tobie"
#elif L==9
	"mencionou voc&ecirc;"
#endif
	};

const char *Txt_Sort_degrees_by =
#if   L==1
	"Ordenar titulacions per";
#elif L==2
	"Sortieren Studieng&auml;nge nach";
#elif L==3
	"Sort degrees by";
#elif L==4
	"Ordenar titulaciones por";
#elif L==5
	"Trier &eacute;tudes par";
#elif L==6
	"Ordenar titulaciones por";	// Okoteve traduccin
#elif L==7
	"Ordina laurea per";
#elif L==8
	"Sortuj stopni wed&lstrok;ug";
#elif L==9
	"Classificar titula&ccedil;oes por";
#endif

const char *Txt_Source_of_information =
#if   L==1
	"Font d'informaci&oacute;";
#elif L==2
	"Quelle von Informationen";
#elif L==3
	"Source of information";
#elif L==4
	"Fuente de informaci&oacute;n";
#elif L==5
	"Source d'information";
#elif L==6
	"Fuente de informaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Fonte delle informazioni";
#elif L==8
	"&Zacute;r&oacute;d&lstrok;o informacji";
#elif L==9
	"Fonte de informa&ccedil;&atilde;o";
#endif

const char *Txt_STAT_CLICKS_GROUPED_BY[Sta_NUM_CLICKS_GROUPED_BY] =
   {
#if   L==1
	 "clics detallats"
#elif L==2
	 "detaillierte Klicks"
#elif L==3
	 "detailed clicks"
#elif L==4
	 "clics detallados"
#elif L==5
	 "clics d&eacute;taill&eacute;es"
#elif L==6
	 "clics detallados"	// Okoteve traduccin
#elif L==7
	 "dettagli di scatto"
#elif L==8
	 "szczeg&oacute;&lstrok;owe klikni&eogon;&cacute;"
#elif L==9
	 "detalhadamente cliques"
#endif
	 ,
#if   L==1
	 "usuari"
#elif L==2
	 "Benutzer"
#elif L==3
	 "user"
#elif L==4
	 "usuario"
#elif L==5
	 "utilisateur"
#elif L==6
	 "puruh&aacute;ra"
#elif L==7
	 "utente"
#elif L==8
	 "u&zdot;ytkownik"
#elif L==9
	 "utilizador"
#endif
	 ,
#if   L==1
	 "dia"
#elif L==2
	 "Tag"
#elif L==3
	 "day"
#elif L==4
	 "d&iacute;a"
#elif L==5
	 "jour"
#elif L==6
	 "d&iacute;a"	// Okoteve traduccin
#elif L==7
	 "giorno"
#elif L==8
	 "dzie&nacute;"
#elif L==9
	 "dia"
#endif
	 ,
#if   L==1
	 "dia i hora"
#elif L==2
	 "Tag und Stunde"
#elif L==3
	 "day and hour"
#elif L==4
	 "d&iacute;a y hora"
#elif L==5
	 "jour et heure"
#elif L==6
	 "d&iacute;a y hora"	// Okoteve traduccin
#elif L==7
	 "giorno e ora"
#elif L==8
	 "dzie&nacute; i godzin&eogon;"
#elif L==9
	 "dia e hora"
#endif
	 ,
#if   L==1
	 "setmana"
#elif L==2
	 "Woche"
#elif L==3
	 "week"
#elif L==4
	 "semana"
#elif L==5
	 "semaine"
#elif L==6
	 "arapok&otilde;indy"
#elif L==7
	 "settimana"
#elif L==8
	 "tydzie&nacute;"
#elif L==9
	 "semana"
#endif
	 ,
#if   L==1
	 "mes"
#elif L==2
	 "Monat"
#elif L==3
	 "month"
#elif L==4
	 "mes"
#elif L==5
	 "mois"
#elif L==6
	 "jasy"
#elif L==7
	 "mese"
#elif L==8
	 "miesi&aogon;c"
#elif L==9
	 "m&ecirc;s"
#endif
	 ,
#if   L==1
	 "hora"
#elif L==2
	 "Stunde"
#elif L==3
	 "hour"
#elif L==4
	 "hora"
#elif L==5
	 "heure"
#elif L==6
	 "aravo"
#elif L==7
	 "ora"
#elif L==8
	 "godzina"
#elif L==9
	 "hora"
#endif
	 ,
#if   L==1
	 "minut"
#elif L==2
	 "Minute"
#elif L==3
	 "minute"
#elif L==4
	 "minuto"
#elif L==5
	 "minute"
#elif L==6
	 "aravo'i"
#elif L==7
	 "minuto"
#elif L==8
	 "minut&eogon;"
#elif L==9
	 "minuto"
#endif
	 ,
#if   L==1
	 "acci&oacute;"
#elif L==2
	 "Handlung"
#elif L==3
	 "action"
#elif L==4
	 "acci&oacute;n"
#elif L==5
	 "action"
#elif L==6
	 "tembiapo"
#elif L==7
	 "azione"
#elif L==8
	 "dzia&lstrok;anie"
#elif L==9
	 "a&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "dia"
#elif L==2
	 "Tag"
#elif L==3
	 "day"
#elif L==4
	 "d&iacute;a"
#elif L==5
	 "jour"
#elif L==6
	 "d&iacute;a"	// Okoteve traduccin
#elif L==7
	 "giorno"
#elif L==8
	 "dzie&nacute;"
#elif L==9
	 "dia"
#endif
	 ,
#if   L==1
	 "dia i hora"
#elif L==2
	 "Tag und Stunde"
#elif L==3
	 "day and hour"
#elif L==4
	 "d&iacute;a y hora"
#elif L==5
	 "jour et heure"
#elif L==6
	 "d&iacute;a y hora"	// Okoteve traduccin
#elif L==7
	 "giorno e ora"
#elif L==8
	 "dzie&nacute; i godzin&eogon;"
#elif L==9
	 "dia e hora"
#endif
	 ,
#if   L==1
	 "setmana"
#elif L==2
	 "Woche"
#elif L==3
	 "week"
#elif L==4
	 "semana"
#elif L==5
	 "semaine"
#elif L==6
	 "arapok&otilde;indy"
#elif L==7
	 "settimana"
#elif L==8
	 "tydzie&nacute;"
#elif L==9
	 "semana"
#endif
	 ,
#if   L==1
	 "mes"
#elif L==2
	 "Monat"
#elif L==3
	 "month"
#elif L==4
	 "mes"
#elif L==5
	 "mois"
#elif L==6
	 "jasy"
#elif L==7
	 "mese"
#elif L==8
	 "miesi&aogon;c"
#elif L==9
	 "m&ecirc;s"
#endif
	 ,
#if   L==1
	 "hora"
#elif L==2
	 "Stunde"
#elif L==3
	 "hour"
#elif L==4
	 "hora"
#elif L==5
	 "heure"
#elif L==6
	 "aravo"
#elif L==7
	 "ora"
#elif L==8
	 "godzina"
#elif L==9
	 "hora"
#endif
	 ,
#if   L==1
	 "minut"
#elif L==2
	 "Minute"
#elif L==3
	 "minute"
#elif L==4
	 "minuto"
#elif L==5
	 "minute"
#elif L==6
	 "aravo'i"
#elif L==7
	 "minuto"
#elif L==8
	 "minut&eogon;"
#elif L==9
	 "minuto"
#endif
	 ,
#if   L==1
	 "acci&oacute;"
#elif L==2
	 "Handlung"
#elif L==3
	 "action"
#elif L==4
	 "acci&oacute;n"
#elif L==5
	 "action"
#elif L==6
	 "tembiapo"
#elif L==7
	 "azione"
#elif L==8
	 "dzia&lstrok;anie"
#elif L==9
	 "a&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "complement"
#elif L==2
	 "Plugin"
#elif L==3
	 "plugin"
#elif L==4
	 "complemento"
#elif L==5
	 "plugin"
#elif L==6
	 "complemento"	// Okoteve traduccin
#elif L==7
	 "plugin"
#elif L==8
	 "wtyczka"
#elif L==9
	 "plugin"
#endif
	 ,
#if   L==1
	 "funci&oacute;"
#elif L==2
	 "Funktion"
#elif L==3
	 "function"
#elif L==4
	 "funci&oacute;n"
#elif L==5
	 "fonction"
#elif L==6
	 "funci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "funzione"
#elif L==8
	 "funkcja"
#elif L==9
	 "fun&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "banner"
#elif L==2
	 "Werbebanner"
#elif L==3
	 "banner"
#elif L==4
	 "banner"
#elif L==5
	 "banni&eacute;re"
#elif L==6
	 "banner"	// Okoteve traduccin
#elif L==7
	 "banner"
#elif L==8
	 "banner"
#elif L==9
	 "banner"
#endif
	 ,
#if   L==1
	"pa&iacute;s"
#elif L==2
	"Land"
#elif L==3
	"country"
#elif L==4
	"pa&iacute;s"
#elif L==5
	"pays;"
#elif L==6
	"tet&atilde;"
#elif L==7
	"paese"
#elif L==8
	"kraj"
#elif L==9
	"pa&iacute;s"
#endif
	 ,
#if   L==1
	"instituci&oacute;"
#elif L==2
	"Hochschule"
#elif L==3
	"institution"
#elif L==4
	"instituci&oacute;n"
#elif L==5
	"&eacute;tablissement"
#elif L==6
	"mbo'ehao"
#elif L==7
	"istituzione"
#elif L==8
	"instytucji"
#elif L==9
	"institu&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	"centre"
#elif L==2
	"Lehrinstitut"
#elif L==3
	"centre"
#elif L==4
	"centro"
#elif L==5
	"centre"
#elif L==6
	"mbo'ehao"
#elif L==7
	"centro"
#elif L==8
	"centrum"
#elif L==9
	"centro"
#endif
	 ,
#if   L==1
	 "titulaci&oacute;"
#elif L==2
	 "Studiengang"
#elif L==3
	 "degree"
#elif L==4
	 "titulaci&oacute;n"
#elif L==5
	 "&eacute;tude"
#elif L==6
	 "arandur&atilde;"
#elif L==7
	 "laurea"
#elif L==8
	 "stopie&nacute;"
#elif L==9
	 "titula&ccedil;&atilde;o"
#endif
	 ,
#if   L==1
	 "assignatura"
#elif L==2
	 "Kurs"
#elif L==3
	 "course"
#elif L==4
	 "asignatura"
#elif L==5
	 "mati&egrave;re"
#elif L==6
	 "mbo'esyry"
#elif L==7
	 "corso"
#elif L==8
	 "kurs"
#elif L==9
	 "disciplina"
#endif
	};

const char *Txt_STAT_COLOR_TYPES[Sta_NUM_COLOR_TYPES] =
   {
#if   L==1
	 "color"
#elif L==2
	 "Farbe"
#elif L==3
	 "color"
#elif L==4
	 "color"
#elif L==5
	 "couleur"
#elif L==6
	 "color"	// Okoteve traduccin
#elif L==7
	 "colore"
#elif L==8
	 "kolor"
#elif L==9
	 "cor"
#endif
	 ,
#if   L==1
	 "de negre a blanc"
#elif L==2
	 "schwarz zu wei&szlig;"
#elif L==3
	 "black to white"
#elif L==4
	 "de negro a blanco"
#elif L==5
	 "du noir au blanc"
#elif L==6
	 "de negro a blanco"	// Okoteve traduccin
#elif L==7
	 "dal nero al bianco"
#elif L==8
	 "czarny na bia&lstrok;y"
#elif L==9
	 "do preto ao branco"
#endif
	 ,
#if   L==1
	 "de blanc a negre"
#elif L==2
	 "wei&szlig; zu schwarz"
#elif L==3
	 "white to black"
#elif L==4
	 "de blanco a negro"
#elif L==5
	 "du blanc au noir"
#elif L==6
	 "de blanco a negro"	// Okoteve traduccin
#elif L==7
	 "dal bianco al nero"
#elif L==8
	 "bia&lstrok;y do czarny"
#elif L==9
	 "do branco ao preto"
#endif
	};

const char *Txt_STAT_COURSE_FILE_ZONES[Sta_NUM_STAT_CRS_FILE_ZONES] =	// Use &nbsp; instead of space
	 {
#if   L==1	// Brw_ADMI_DOCUM_CRS
	 "Documents&nbsp;(ass.)"
#elif L==2
	 "Dokumente&nbsp;(Ver.)"
#elif L==3
	 "Documents&nbsp;(crs.)"
#elif L==4
	 "Documentos&nbsp;(asg.)"
#elif L==5
	 "Documents&nbsp;(mat.)"
#elif L==6
	 "Documentos&nbsp;(asg.)"		// Okoteve traduccin
#elif L==7
	 "Documenti&nbsp;(crs.)"
#elif L==8
	 "Dokumenty&nbsp;(kurs)"
#elif L==9
	"Documentos&nbsp;(crs.)"
#endif
	,
#if   L==1	// Brw_ADMI_DOCUM_GRP
	"Documents&nbsp;(grp.)"
#elif L==2
	"Dokumente&nbsp;(Grp.)"
#elif L==3
	"Documents&nbsp;(grp.)"
#elif L==4
	"Documentos&nbsp;(grp.)"
#elif L==5
	"Documents&nbsp;(grp.)"
#elif L==6
	"Documentos&nbsp;(grp.)"	// Okoteve traduccin
#elif L==7
	"Documenti&nbsp;(grp.)"
#elif L==8
	"Dokumenty&nbsp;(grp.)"
#elif L==9
	"Documentos&nbsp;(grp.)"
#endif
	,
#if   L==1	// Brw_ADMI_TEACH_CRS
	"Privats&nbsp;(ass.)"
#elif L==2
	"Private&nbsp;(Ver.)"
#elif L==3
	"Private&nbsp;(crs.)"
#elif L==4
	"Privados&nbsp;(asg.)"
#elif L==5
	"Priv&eacute;s&nbsp;(mat.)"
#elif L==6
	"Privados&nbsp;(asg.)"	// Okoteve traduccin
#elif L==7
	"Privati&nbsp;(crs.)"
#elif L==8
	"Prywatne&nbsp;(kurs)"
#elif L==9
	"Privadoss&nbsp;(crs.)"
#endif
	,
#if   L==1	// Brw_ADMI_TEACH_GRP
	"Privats&nbsp;(grp.)"
#elif L==2
	"Private&nbsp;(Grp.)"
#elif L==3
	"Private&nbsp;(grp.)"
#elif L==4
	"Privados&nbsp;(grp.)"
#elif L==5
	"Priv&eacute;s&nbsp;(grp.)"
#elif L==6
	"Privados&nbsp;(grp.)"	// Okoteve traduccin
#elif L==7
	"Privati&nbsp;(grp.)"
#elif L==8
	"Prywatne&nbsp;(grp)"
#elif L==9
	"Privadoss&nbsp;(grp.)"
#endif
	,
#if   L==1	// Brw_ADMI_SHARE_CRS
	"Compartits&nbsp;(asg.)"
#elif L==2
	"Freigegebene&nbsp;(Ver.)"
#elif L==3
	"Shared&nbsp;(crs.)"
#elif L==4
	"Compartidos&nbsp;(asg.)"
#elif L==5
	"Partag&eacute;s&nbsp;(mat.)"
#elif L==6
	"Compartidos&nbsp;(asg.)"	// Okoteve traduccin
#elif L==7
	"Condivisi&nbsp;(crs.)"
#elif L==8
	"Wsp&oacute;lna&nbsp;(kurs)"
#elif L==9
	"Compartilhados&nbsp;(crs.)"
#endif
	,
#if   L==1	// Brw_ADMI_SHARE_GRP
	"Compartits&nbsp;(grp.)"
#elif L==2
	"Freigegebene&nbsp;(Grp.)"
#elif L==3
	"Shared&nbsp;(grp.)"
#elif L==4
	"Compartidos&nbsp;(grp.)"
#elif L==5
	"Partag&eacute;s&nbsp;(grp.)"
#elif L==6
	"Compartidos&nbsp;(grp.)"	// Okoteve traduccin
#elif L==7
	"Condivisi&nbsp;(grp.)"
#elif L==8
	"Wsp&oacute;lna&nbsp;(grp.)"
#elif L==9
	"Compartilhados&nbsp;(grp.)"
#endif
	,
#if   L==1	// Brw_ADMI_MARKS_CRS
	"Qualificacions&nbsp;(ass.)"
#elif L==2
	"Bewertungen&nbsp;(Ver.)"
#elif L==3
	"Marks&nbsp;(crs.)"
#elif L==4
	"Calificaciones&nbsp;(asg.)"
#elif L==5
	"Notes&nbsp;(mat.)"
#elif L==6
	"Calificaciones&nbsp;(asg.)"		// Okoteve traduccin
#elif L==7
	"Risultati&nbsp;(crs.)"
#elif L==8
	"Marks&nbsp;(crs.)"			// Potrzebujesz tlumaczenie
#elif L==9
	"Notas&nbsp;(crs.)"
#endif
	,
#if   L==1	// Brw_ADMI_MARKS_GRP
	"Qualificacions&nbsp;(grp.)"
#elif L==2
	"Bewertungen&nbsp;(Grp.)"
#elif L==3
	"Marks&nbsp;(grp.)"
#elif L==4
	"Calificaciones&nbsp;(grp.)"
#elif L==5
	"Notes&nbsp;(grp.)"
#elif L==6
	"Calificaciones&nbsp;(grp.)"		// Okoteve traduccin
#elif L==7
	"Risultati&nbsp;(grp.)"
#elif L==8
	"Marks&nbsp;(grp.)"			// Potrzebujesz tlumaczenie
#elif L==9
	"Notas&nbsp;(grp.)"
#endif
	,
#if   L==1	// Brw_ADMI_ASSIG_USR
	"Activitats"
#elif L==2
	"Aufgaben"
#elif L==3
	"Assignments"
#elif L==4
	"Actividades"
#elif L==5
	"Activit&eacute;s"
#elif L==6
	"Tar&eacute;a"
#elif L==7
	"Attivit&agrave;"
#elif L==8
	"Zadania"
#elif L==9
	"Atividades"
#endif
	,
#if   L==1	// Brw_ADMI_WORKS_USR
	"Treballs"
#elif L==2
	"Hausarbeiten"
#elif L==3
	"Homework"
#elif L==4
	"Trabajos"
#elif L==5
	"Travails"
#elif L==6
	"Tembiapo"
#elif L==7
	"Lavori"
#elif L==8
	"Praca domowa"
#elif L==9
	"Trabalhos"
#endif
	,
#if   L==1	// Brw_UNKNOWN
	"Total"
#elif L==2
	"Gesamtzahl"
#elif L==3
	"Total"
#elif L==4
	"Total"
#elif L==5
	"Total"
#elif L==6
	"Total"		// Okoteve traduccin
#elif L==7
	"Totale"
#elif L==8
	"Ca&lstrok;kowity"
#elif L==9
	"Total"
#endif
	};

const char *Txt_STAT_DEGREE_PHOTO_SIZE[Pho_NUM_HOW_COMPUTE_PHOTO_SIZES] =
   {
#if   L==1
	 "proporcional al n&ordm; de estudiantes"	// Necessita traduccio
#elif L==2
	 "proportional to the number of students"	// Need bersetzung
#elif L==3
	 "proportional to the number of students"
#elif L==4
	 "proporcional al n&ordm; de estudiantes"
#elif L==5
	 "proportional to the number of students"	// Besoin de traduction
#elif L==6
	 "proporcional al n&ordm; de estudiantes"	// Okoteve traduccin
#elif L==7
	 "proporzionale al numero di studenti"
#elif L==8
	 "proportional to the number of students"	// Potrzebujesz tlumaczenie
#elif L==9
	 "proportional to the number of students"	// Necessita de traduo
#endif
	 ,
#if   L==1
	 "proporcional al n&ordm; de estudiantes con fotograf&iacute;a"	// Necessita traduccio
#elif L==2
	 "proportional to the number of students with photo"		// Need bersetzung
#elif L==3
	 "proportional to the number of students with photo"
#elif L==4
	 "proporcional al n&ordm; de estudiantes con fotograf&iacute;a"
#elif L==5
	 "proportional to the number of students with photo"		// Besoin de traduction
#elif L==6
	 "proporcional al n&ordm; de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7
	 "proporzionale al numero di studenti con foto"
#elif L==8
	 "proportional to the number of students with photo"		// Potrzebujesz tlumaczenie
#elif L==9
	 "proportional to the number of students with photo"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "proporcional al porcentaje de estudiantes con fotograf&iacute;a"	// Necessita traduccio
#elif L==2
	 "proportional to the percentage of students with photo"		// Need bersetzung
#elif L==3
	 "proportional to the percentage of students with photo"
#elif L==4
	 "proporcional al porcentaje de estudiantes con fotograf&iacute;a"
#elif L==5
	 "proportional to the percentage of students with photo"		// Besoin de traduction
#elif L==6
	 "proporcional al porcentaje de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7
	 "proporzionale alla percentuale di studenti con foto"
#elif L==8
	 "proportional to the percentage of students with photo"		// Potrzebujesz tlumaczenie
#elif L==9
	 "proportional to the percentage of students with photo"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "igual para todas las titulaciones"	// Necessita traduccio
#elif L==2
	 "same for all degrees"			// Need bersetzung
#elif L==3
	 "same for all degrees"
#elif L==4
	 "igual para todas las titulaciones"
#elif L==5
	 "same for all degrees"			// Besoin de traduction
#elif L==6
	 "igual para todas las titulaciones"	// Okoteve traduccin
#elif L==7
	 "uguale per tutte le lauree"
#elif L==8
	 "same for all degrees"			// Potrzebujesz tlumaczenie
#elif L==9
	 "same for all degrees"			// Necessita de traduo
#endif
	};

const char *Txt_STAT_DEGREE_PHOTO_ORDER[Pho_NUM_HOW_ORDER_DEGREES] =
   {
#if   L==1
	 "nombre d'estudiants"
#elif L==2
	 "Anzahl der Studenten"
#elif L==3
	 "number of students"
#elif L==4
	 "n&ordm; de estudiantes"
#elif L==5
	 "nombre d'&eacute;tudiants"
#elif L==6
	 "n&ordm; de estudiantes"	// Okoteve traduccin
#elif L==7
	 "numero di studenti"
#elif L==8
	 "liczba student&oacute;w"
#elif L==9
	 "n&uacute;mero de estudantes"
#endif
	 ,
#if   L==1
	 "nombre d'estudiants amb fotografia"
#elif L==2
	 "Anzahl der Studenten mit Foto"
#elif L==3
	 "number of students with photo"
#elif L==4
	 "n&ordm; de estudiantes con fotograf&iacute;a"
#elif L==5
	 "nombre d'&eacute;tudiants avec photo"
#elif L==6
	 "n&ordm; de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7
	 "numero di studenti con foto"
#elif L==8
	 "liczba student&oacute;w ze zdj&eogon;ciem"
#elif L==9
	 "n&uacute;mero de estudantes com foto"
#endif
	 ,
#if   L==1
	 "porcentaje de estudiantes con fotograf&iacute;a"	// Necessita traduccio
#elif L==2
	 "percentage of students with photo"			// Need bersetzung
#elif L==3
	 "percentage of students with photo"
#elif L==4
	 "porcentaje de estudiantes con fotograf&iacute;a"
#elif L==5
	 "percentage of students with photo"			// Besoin de traduction
#elif L==6
	 "porcentaje de estudiantes con fotograf&iacute;a"	// Okoteve traduccin
#elif L==7
	 "percentuale di studenti con foto"
#elif L==8
	 "percentage of students with photo"			// Potrzebujesz tlumaczenie
#elif L==9
	 "percentage of students with photo"			// Necessita de traduo
#endif
	 ,
#if   L==1
	 "nombre de titulaci&oacute;n"	// Necessita traduccio
#elif L==2
	 "degree name"			// Need bersetzung
#elif L==3
	 "degree name"
#elif L==4
	 "nombre de titulaci&oacute;n"
#elif L==5
	 "degree name"			// Besoin de traduction
#elif L==6
	 "nombre de titulaci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "nome laurea"
#elif L==8
	 "degree name"			// Potrzebujesz tlumaczenie
#elif L==9
	 "degree name"			// Necessita de traduo
#endif
	};

const char *Txt_STAT_TYPE_COUNT_CAPS[Sta_NUM_COUNT_TYPES] =
   {
#if   L==1
	 "N&ordm; total de clics"	// Necessita traduccio
#elif L==2
	 "Gesamtzahl der Klicks"
#elif L==3
	 "Total number of clicks"
#elif L==4
	 "N&ordm; total de clics"
#elif L==5
	 "Nombre total de clics"
#elif L==6
	 "N&ordm; total de clics"	// Okoteve traduccin
#elif L==7
	 "Numero totale di scatti"
#elif L==8
	 "Total number of clicks"	// Potrzebujesz tlumaczenie
#elif L==9
	 "N&uacute;mero total de cliques"
#endif
	 ,
#if   L==1
	 "N&ordm; de usuarios distintos"		// Necessita traduccio
#elif L==2
	 "Anzahl der unterschiedlichen Benutzer"
#elif L==3
	 "Number of distinct users"
#elif L==4
	 "N&ordm; de usuarios distintos"
#elif L==5
	 "Nombre des diff&eacute;rents utilisateurs"
#elif L==6
	 "N&ordm; de usuarios distintos"		// Okoteve traduccin
#elif L==7
	 "Numero di utenti distinti"
#elif L==8
	 "Number of distinct users"			// Potrzebujesz tlumaczenie
#elif L==9
	 "N&uacute;mero de utilizadores diferentes"
#endif
	 ,
#if   L==1
	 "N&ordm; de clics por usuario"		// Necessita traduccio
#elif L==2
	 "Anzahl der Klicks pro Benutzer"
#elif L==3
	 "Number of clicks per user"
#elif L==4
	 "N&ordm; de clics por usuario"
#elif L==5
	 "Nombre de clics par utilisateur"
#elif L==6
	 "N&ordm; de clics por usuario"		// Okoteve traduccin
#elif L==7
	 "Numero di scatti per utente"
#elif L==8
	 "Number of clicks per user"		// Potrzebujesz tlumaczenie
#elif L==9
	 "n&uacute;mero de cliques por utilizador"
#endif
	 ,
#if   L==1
	 "Tiempo de generaci&oacute;n de p&aacute;gina"	// Necessita traduccio
#elif L==2
	 "Zeit zur Erzeugung der Seite"
#elif L==3
	 "Time to generate page"
#elif L==4
	 "Tiempo de generaci&oacute;n de p&aacute;gina"
#elif L==5
	 "Temps pour g&eacute;n&eacute;rer la page"
#elif L==6
	 "Tiempo de generaci&oacute;n de p&aacute;gina"	// Okoteve traduccin
#elif L==7
	 "Tempo per generare pagina"
#elif L==8
	 "Time to generate page"			// Potrzebujesz tlumaczenie
#elif L==9
	 "Tempo para gerar p&aacute;gina"
#endif
	 ,
#if   L==1
	 "Tiempo de env&iacute;o de p&aacute;gina"	// Necessita traduccio
#elif L==2
	 "Zeit, um die Seite zu senden"
#elif L==3
	 "Time to send page"
#elif L==4
	 "Tiempo de env&iacute;o de p&aacute;gina"
#elif L==5
	 "Temps pour envoyer la page"
#elif L==6
	 "Tiempo de env&iacute;o de p&aacute;gina"	// Okoteve traduccin
#elif L==7
	 "Tempo d'invio della pagina"
#elif L==8
	 "Time to send page"				// Potrzebujesz tlumaczenie
#elif L==9
	 "Tempo para enviar a p&aacute;gina"
#endif
	};

const char *Txt_STAT_TYPE_COUNT_SMALL[Sta_NUM_COUNT_TYPES] =
   {
#if   L==1
	 "n&ordm; total de clics"	// Necessita traduccio
#elif L==2
	 "Gesamtzahl der Klicks"
#elif L==3
	 "total number of clicks"
#elif L==4
	 "n&ordm; total de clics"
#elif L==5
	 "nombre total de clics"
#elif L==6
	 "n&ordm; total de clics"	// Okoteve traduccin
#elif L==7
	 "numero totale di scatti"
#elif L==8
	 "total number of clicks"	// Potrzebujesz tlumaczenie
#elif L==9
	 "n&uacute;mero total de cliques"
#endif
	 ,
#if   L==1
	 "n&ordm; de usuarios distintos"		// Necessita traduccio
#elif L==2
	 "Anzahl der unterschiedliche Benutzer"
#elif L==3
	 "number of distinct users"
#elif L==4
	 "n&ordm; de usuarios distintos"
#elif L==5
	 "nombre des diff&eacute;rents utilisateurs"
#elif L==6
	 "n&ordm; de usuarios distintos"		// Okoteve traduccin
#elif L==7
	 "numero di utenti distinti"
#elif L==8
	 "number of distinct users"			// Potrzebujesz tlumaczenie
#elif L==9
	 "n&uacute;mero de utilizadores diferentes"
#endif
	 ,
#if   L==1
	 "n&ordm; de clics por usuario"		// Necessita traduccio
#elif L==2
	 "Anzahl der Klicks pro Benutzer"
#elif L==3
	 "number of clicks per user"
#elif L==4
	 "n&ordm; de clics por usuario"
#elif L==5
	 "nombre de clics par utilisateur"
#elif L==6
	 "n&ordm; de clics por usuario"		// Okoteve traduccin
#elif L==7
	 "numero di scatti per utente"
#elif L==8
	 "number of clicks per user"		// Potrzebujesz tlumaczenie
#elif L==9
	 "n&uacute;mero de cliques por utilizador"
#endif
	 ,
#if   L==1
	 "tiempo de generaci&oacute;n"		// Necessita traduccio
#elif L==2
	 "Zeit zur Erzeugung"
#elif L==3
	 "time to generate"
#elif L==4
	 "tiempo de generaci&oacute;n"
#elif L==5
	 "temps pour g&eacute;n&eacute;rer"
#elif L==6
	 "tiempo de generaci&oacute;n"		// Okoteve traduccin
#elif L==7
	 "tempo per generare"
#elif L==8
	 "time to generate"			// Potrzebujesz tlumaczenie
#elif L==9
	 "tempo para gerar"
#endif
	 ,
#if   L==1
	 "tiempo de env&iacute;o"	// Necessita traduccio
#elif L==2
	 "Zeit, um Seite"
#elif L==3
	 "time to send"
#elif L==4
	 "tiempo de env&iacute;o"
#elif L==5
	 "temps pour envoyer"
#elif L==6
	 "tiempo de env&iacute;o"	// Okoteve traduccin
#elif L==7
	 "tempo d'invio"
#elif L==8
	 "time to send"			// Potrzebujesz tlumaczenie
#elif L==9
	 "Tempo para enviar"
#endif
	};

const char *Txt_STAT_USE_STAT_TYPES[Sta_NUM_FIGURES] =
   {
#if   L==1				// Sta_USERS
	 "Usuaris"
#elif L==2
	 "Benutzer"
#elif L==3
	 "Users"
#elif L==4
	 "Usuarios"
#elif L==5
	 "Utilisateurs"
#elif L==6
	 "Puruh&aacute;ra"
#elif L==7
	 "Utenti"
#elif L==8
	 "U&zdot;ytkownicy"
#elif L==9
	 "Utilizadores"
#endif
	 ,
#if   L==1				// Sta_USERS_RANKING
	 "R&agrave;nquing"
#elif L==2
	 "Rang"
#elif L==3
	 "Ranking"
#elif L==4
	 "Ranking"
#elif L==5
	 "Classement"
#elif L==6
	 "Ranking"	// Okoteve traduccin
#elif L==7
	 "Posto"
#elif L==8
	 "Ranking"
#elif L==9
	 "Posi&ccedil;&atilde;o"
#endif
	 ,
#if   L==1				// Sta_HIERARCHY
	 "Jerarquia"
#elif L==2
	 "Hierarchie"
#elif L==3
	 "Hierarchy"
#elif L==4
	 "Jerarqu&iacute;a"
#elif L==5
	 "Hi&eacute;rarchie"
#elif L==6
	 "Jerarqu&iacute;a"		// Okoteve traduccin
#elif L==7
	 "Gerarchia"
#elif L==8
	 "Hierarchia"
#elif L==9
	 "Hierarquia"
#endif
	 ,
#if   L==1				// Sta_INSTITUTIONS
	"Institucions"
#elif L==2
	"Hochschulen"
#elif L==3
	"Institutions"
#elif L==4
	"Instituciones"
#elif L==5
	"&Eacute;tablissements"
#elif L==6
	"Mbo'ehao"
#elif L==7
	"Istituzioni"
#elif L==8
	"Instytucje"
#elif L==9
	"Institu&ccedil;&otilde;es"
#endif
	 ,
#if   L==1				// Sta_FOLDERS_AND_FILES
	 "Carpetes i fitxers"
#elif L==2
	 "Verzeichnisse und Dateien"
#elif L==3
	 "Folders and files"
#elif L==4
	 "Carpetas y archivos"
#elif L==5
	 "R&eacute;pertoires et fichiers"
#elif L==6
	 "Carpetas y archivos"		// Okoteve traduccin
#elif L==7
	 "Cartelle e file"
#elif L==8
	 "Folders and files"		// Potrzebujesz tlumaczenie
#elif L==9
	 "Diret&oacute;rios e arquivos"
#endif
	 ,
#if   L==1				// Sta_OER
	 "Recursos Educatius Oberts (OER)"
#elif L==2
	 "Open Educational Resources (OER)"
#elif L==3
	 "Open Educational Resources (OER)"
#elif L==4
	 "Recursos Educativos Abiertos (OER)"
#elif L==5
	 "Ressources &Eacute;ducatives Libres (REL)"
#elif L==6
	 "Recursos Educativos Abiertos (OER)"	// Okoteve traduccin
#elif L==7
	 "Risorse Didattiche Aperte (OER)"
#elif L==8
	 "Otwarte Zasoby Edukacyjne (OER)"
#elif L==9
	 "Recursos Educacionais Abertos (OER)"
#endif
	 ,
#if   L==1				// Sta_ASSIGNMENTS
	 "Activitats"
#elif L==2
	 "Aufgaben"
#elif L==3
	 "Assignments"
#elif L==4
	 "Actividades"
#elif L==5
	 "Activit&eacute;s"
#elif L==6
	 "Tar&eacute;a"
#elif L==7
	 "Attivit&agrave;"
#elif L==8
	 "Assignments"	// Potrzebujesz tlumaczenie
#elif L==9
	 "Atividades"
#endif
	 ,
#if   L==1				// Sta_TESTS
	 "Tests"
#elif L==2
	 "Tests"
#elif L==3
	 "Tests"
#elif L==4
	 "Tests"
#elif L==5
	 "Tests"
#elif L==6
	 "Tests"		// Okoteve traduccin
#elif L==7
	 "Test"
#elif L==8
	 "Tests"		// Potrzebujesz tlumaczenie
#elif L==9
	 "Testes"
#endif
	 ,
#if   L==1				// Sta_SOCIAL_ACTIVITY
	 "Activitat p&uacute;blica"
#elif L==2
	 "&Ouml;ffentliche Aktivit&auml;t"
#elif L==3
	 "Public activity"
#elif L==4
	 "Actividad p&uacute;blica"
#elif L==5
	 "Activit&eacute; public"
#elif L==6
	 "Actividad p&uacute;blica"	// Okoteve traduccin
#elif L==7
	 "Attivit&agrave; pubblica"
#elif L==8
	 "Aktywno&sacute;&cacute; publiczne"
#elif L==9
	 "Atividade p&uacute;blica"
#endif
	 ,
#if   L==1				// Sta_FOLLOW
	"Seguits / Seguidors"
#elif L==2
	"Gefolgt / Anh&auml;nger"
#elif L==3
	"Followed / Followers"
#elif L==4
	"Seguidos / Seguidores"
#elif L==5
	"Suivis / Suiveurs"
#elif L==6
	"Seguidos / Seguidores"	// Okoteve traduccin
#elif L==7
	"Seguiti / Seguaci"
#elif L==8
	"Obserwowani / Obserwuj&aogon;"
#elif L==9
	"Seguidos / Seguidores"
#endif
	 ,
#if   L==1				// Sta_FORUMS
	 "F&ograve;rums"
#elif L==2
	 "Foren"
#elif L==3
	 "Forums"
#elif L==4
	 "Foros"
#elif L==5
	 "Forums"
#elif L==6
	 "Foros"		// Okoteve traduccin
#elif L==7
	 "Forum"
#elif L==8
	 "Forums"		// Potrzebujesz tlumaczenie
#elif L==9
	 "F&oacute;runs"
#endif
	 ,
#if   L==1				// Sta_NOTIFY_EVENTS
	"Notificacions enviades per correu"
#elif L==2
	"Anmeldungen per E-Mail gesendet"
#elif L==3
	"Notifications sent by email"
#elif L==4
	"Notificaciones enviadas por correo"
#elif L==5
	"Notifications envoy&eacute;es par courriel"
#elif L==6
	"Notificaciones enviadas por correo"	// Okoteve traduccin
#elif L==7
	"Notifiche inviate via email"
#elif L==8
	"Powiadomienia wysy&lstrok;ane przez email"
#elif L==9
	"Notifica&ccedil;&otilde;es enviadas por email"
#endif
	 ,
#if   L==1				// Sta_NOTICES
	 "Avisos"
#elif L==2
	 "Ank&uuml;ndigungen"
#elif L==3
	 "Notices"
#elif L==4
	 "Avisos"
#elif L==5
	 "Avis"
#elif L==6
	 "Marandu"
#elif L==7
	 "Avvisi"
#elif L==8
	 "Ogloszenia"
#elif L==9
	 "Avisos"
#endif
	 ,
#if   L==1				// Sta_MESSAGES
	 "Missatges"
#elif L==2
	 "Nachrichten"
#elif L==3
	 "Messages"
#elif L==4
	 "Mensajes"
#elif L==5
	 "Messages"
#elif L==6
	 "Marandu"
#elif L==7
	 "Messaggi"
#elif L==8
	 "Wiadomo&sacute;ci"
#elif L==9
	 "Mensagens"
#endif
	 ,
#if   L==1				// Sta_SURVEYS
	 "Enquestes"
#elif L==2
	 "Umfragen"
#elif L==3
	 "Surveys"
#elif L==4
	 "Encuestas"
#elif L==5
	 "Sondages"
#elif L==6
	 "Encuestas"	// Okoteve traduccin
#elif L==7
	 "Sondaggi"
#elif L==8
	 "Badania"
#elif L==9
	 "Inqu&eacute;ritos"
#endif
	 ,
#if   L==1				// Sta_SOCIAL_NETWORKS
	 "Webs / xarxes socials"
#elif L==2
	 "Webs / Social Networks"
#elif L==3
	 "Webs / social networks"
#elif L==4
	 "Webs / redes sociales"
#elif L==5
	 "Webs / r&eacute;seaux sociaux"
#elif L==6
	 "Webs / redes sociales"	// Okoteve traduccin
#elif L==7
	 "Webs / reti sociali"
#elif L==8
	 "Webs / sieci spo&lstrok;eczne"
#elif L==9
	 "Webs / redes sociais"
#endif
	 ,
#if   L==1				// Sta_LANGUAGES
	 "Idioma"
#elif L==2
	 "Sprache"
#elif L==3
	 "Language"
#elif L==4
	 "Idioma"
#elif L==5
	 "Langue"
#elif L==6
	 "&Ntilde;e'&#7869;"
#elif L==7
	 "Lingua"
#elif L==8
	 "Language"	// Potrzebujesz tlumaczenie
#elif L==9
	 "Lingua"
#endif
	 ,
#if   L==1				// Sta_ICON_SETS
	 "Icones"
#elif L==2
	 "Symbole"
#elif L==3
	 "Icons"
#elif L==4
	 "Iconos"
#elif L==5
	 "Ic&ocirc;nes"
#elif L==6
	 "Iconos"		// Okoteve traduccin
#elif L==7
	 "Icone"
#elif L==8
	 "Ikony"
#elif L==9
	 "&Iacute;cones"
#endif
	 ,
#if   L==1				// Sta_MENUS
	"Men&uacute;"
#elif L==2
	"Men&uuml;"
#elif L==3
	"Menu"
#elif L==4
	"Men&uacute;"
#elif L==5
	"Menu"
#elif L==6
	"Men&uacute;"	// Okoteve traduccin
#elif L==7
	"Menu"
#elif L==8
	"Menu"
#elif L==9
	"Menu"
#endif
	 ,
#if   L==1				// Sta_FIRST_DAY_OF_WEEK
	"Calendari"
#elif L==2
	"Kalender"
#elif L==3
	"Calendar"
#elif L==4
	"Calendario"
#elif L==5
	"Calendrier"
#elif L==6
	"Arapapaha"
#elif L==7
	"Calendario"
#elif L==8
	"Kalendarz"
#elif L==9
	"Calend&aacute;rio"
#endif
	 ,
#if   L==1				// Sta_THEMES
	 "Tema (colors)"
#elif L==2
	 "Skin"
#elif L==3
	 "Theme"
#elif L==4
	 "Tema (colores)"
#elif L==5
	 "Th&egrave;me"
#elif L==6
	 "Tema (colores)"	// Okoteve traduccin
#elif L==7
	 "Tema"
#elif L==8
	 "Temat"
#elif L==9
	 "Skin"
#endif
	 ,
#if   L==1				// Sta_SIDE_COLUMNS
	 "Columnes"
#elif L==2
	 "Spalten"
#elif L==3
	 "Columns"
#elif L==4
	 "Columnas"
#elif L==5
	 "Colonnes"
#elif L==6
	 "Columnas"	// Okoteve traduccin
#elif L==7
	 "Colonne"
#elif L==8
	 "Kolumny"
#elif L==9
	 "Colunas"
#endif
	 ,
#if   L==1				// Sta_PRIVACY
	"Privacitat"
#elif L==2
	"Geheimhaltung"
#elif L==3
	"Privacy"
#elif L==4
	"Privacidad"
#elif L==5
	"Privacit&eacute;"
#elif L==6
	"Privacidad"	// Okoteve traduccin
#elif L==7
	"Privatezza"
#elif L==8
	"Prywatno&sacute;&cacute;"
#elif L==9
	"Privacidade"
#endif
	};

const char *Txt_Statistic =
#if   L==1
	 "Estad&iacute;stique";
#elif L==2
	 "Statistik";
#elif L==3
	 "Statistic";
#elif L==4
	 "Estad&iacute;stica";
#elif L==5
	 "Statistique";
#elif L==6
	 "Papy'aporeko";
#elif L==7
	 "Statistica";
#elif L==8
	 "Statystyki";
#elif L==9
	 "Estat&iacute;sticas";
#endif

const char *Txt_Statistics_of_all_visits =
#if   L==1
	"Estad&iacute;stica de todos los accesos";	// Necessita traduccio
#elif L==2
	"Statistics of all visits";			// Need bersetzung
#elif L==3
	"Statistics of all visits";
#elif L==4
	"Estad&iacute;stica de todos los accesos";
#elif L==5
	"Statistics of all visits";			// Besoin de traduction
#elif L==6
	"Estad&iacute;stica de todos los accesos";	// Okoteve traduccin
#elif L==7
	"Statistiche di tutte gli accessi";
#elif L==8
	"Statistics of all visits";			// Potrzebujesz tlumaczenie
#elif L==9
	"Statistics of all visits";			// Necessita de traduo
#endif

const char *Txt_Statistics_of_visits_to_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Estad&iacute;stica d'accessos a l'assignatura <strong>%s</strong>";
#elif L==2
	"Statistics of visits to the course <strong>%s</strong>";		// Need bersetzung
#elif L==3
	"Statistics of visits to the course <strong>%s</strong>";
#elif L==4
	"Estad&iacute;stica de accesos a la asignatura <strong>%s</strong>";
#elif L==5
	"Statistics of visits to the course <strong>%s</strong>";		// Besoin de traduction
#elif L==6
	"Estad&iacute;stica de accesos a la asignatura <strong>%s</strong>";	// Okoteve traduccin
#elif L==7
	"Statistiche degli accessi al corso <strong>%s</strong>";
#elif L==8
	"Statistics of visits to the course <strong>%s</strong>";		// Potrzebujesz tlumaczenie
#elif L==9
	"Statistics of visits to the course <strong>%s</strong>";		// Necessita de traduo
#endif

const char *Txt_Status =
#if   L==1
	"Estado";	// Necessita traduccio
#elif L==2
	"Status";
#elif L==3
	"Status";
#elif L==4
	"Estado";
#elif L==5
	"Statut";
#elif L==6
	"O&#297; h&aacute;icha";
#elif L==7
	"Stato";
#elif L==8
	"Status";	// Potrzebujesz tlumaczenie
#elif L==9
	"Status";
#endif

const char *Txt_Stay_connected_with_SWADroid =
#if   L==1
	"Seguiu connectat/da amb <strong>SWADroid</strong>.";
#elif L==2
	"Stay connected with <strong>SWADroid</strong>.";		// Need bersetzung
#elif L==3
	"Stay connected with <strong>SWADroid</strong>.";
#elif L==4
	"Siga conectado/a con <strong>SWADroid</strong>.";
#elif L==5
	"Stay connected with <strong>SWADroid</strong>.";		// Besoin de traduction
#elif L==6
	"Siga conectado/a con <strong>SWADroid</strong>.";		// Okoteve traduccin
#elif L==7
	"Resta in contatto con <strong>SWADroid</strong>.";
#elif L==8
	"Stay connected with <strong>SWADroid</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Stay connected with <strong>SWADroid</strong>.";		// Necessita de traduo
#endif

const char *Txt_Stem =		// Stem of a test question
#if   L==1
	"Enunciat";
#elif L==2
	"Stamm";
#elif L==3
	"Stem";
#elif L==4
	"Enunciado";
#elif L==5
	"Question";
#elif L==6
	"Enunciado";	// Okoteve traduccin
#elif L==7
	"Domanda";
#elif L==8
	"Stem";		// Potrzebujesz tlumaczenie
#elif L==9
	"Quest&atilde;o";
#endif

const char *Txt_Step_1_Provide_a_list_of_users =
#if   L==1
	"Paso 1: Proporcione una lista de usuarios";	// Necessita traduccio
#elif L==2
	"Step 1: Provide a list of users";		// Need bersetzung
#elif L==3
	"Step 1: Provide a list of users";
#elif L==4
	"Paso 1: Proporcione una lista de usuarios";
#elif L==5
	"Step 1: Provide a list of users";		// Besoin de traduction
#elif L==6
	"Paso 1: Proporcione una lista de usuarios";	// Okoteve traduccin
#elif L==7
	"Passo 1: Fornisci una lista di utenti";
#elif L==8
	"Step 1: Provide a list of users";		// Potrzebujesz tlumaczenie
#elif L==9
	"Step 1: Provide a list of users";		// Necessita de traduo
#endif

const char *Txt_Step_2_Select_the_desired_action =
#if   L==1
	"Paso 2: Seleccione la acci&oacute;n que desea realizar";	// Necessita traduccio
#elif L==2
	"Step 2: Select the desired action";				// Need bersetzung
#elif L==3
	"Step 2: Select the desired action";
#elif L==4
	"Paso 2: Seleccione la acci&oacute;n que desea realizar";
#elif L==5
	"Step 2: Select the desired action";				// Besoin de traduction
#elif L==6
	"Paso 2: Seleccione la acci&oacute;n que desea realizar";	// Okoteve traduccin
#elif L==7
	"Passo 2: Seleziona l'azione desiderata";
#elif L==8
	"Step 2: Select the desired action";				// Potrzebujesz tlumaczenie
#elif L==9
	"Step 2: Select the desired action";				// Necessita de traduo
#endif

const char *Txt_Step_3_Optionally_select_groups =
#if   L==1
	"Paso 3: Opcionalmente, seleccione grupos";	// Necessita traduccio
#elif L==2
	"Step 3: Optionally, select groups";		// Need bersetzung
#elif L==3
	"Step 3: Optionally, select groups";
#elif L==4
	"Paso 3: Opcionalmente, seleccione grupos";
#elif L==5
	"Step 3: Optionally, select groups";		// Besoin de traduction
#elif L==6
	"Paso 3: Opcionalmente, seleccione grupos";	// Okoteve traduccin
#elif L==7
	"Passo 3: A scelta, seleziona i gruppi";
#elif L==8
	"Step 3: Optionally, select groups";		// Potrzebujesz tlumaczenie
#elif L==9
	"Step 3: Optionally, select groups";		// Necessita de traduo
#endif

const char *Txt_Step_4_Confirm_the_enrollment_removing =
#if   L==1
	"Paso 4: Confirme la inscripci&oacute;n / eliminaci&oacute;n";	// Necessita traduccio
#elif L==2
	"Step 4: Confirm the enrollment / removing";			// Need bersetzung
#elif L==3
	"Step 4: Confirm the enrollment / removing";
#elif L==4
	"Paso 4: Confirme la inscripci&oacute;n / eliminaci&oacute;n";
#elif L==5
	"Step 4: Confirm the enrollment / removing";			// Besoin de traduction
#elif L==6
	"Paso 4: Confirme la inscripci&oacute;n / eliminaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Passo 4: Conferma la registrazione / rimozione";
#elif L==8
	"Step 4: Confirm the enrollment / removing";			// Potrzebujesz tlumaczenie
#elif L==9
	"Step 4: Confirm the enrollment / removing";			// Necessita de traduo
#endif

const char *Txt_Student_comment =
#if   L==1
	"Comentari de l'estudiant";
#elif L==2
	"Student-Kommentar";
#elif L==3
	"Student's comment";
#elif L==4
	"Comentario del estudiante";
#elif L==5
	"Commentaire de l'&eacute;tudiant";
#elif L==6
	"Comentario del estudiante";	// Okoteve traduccin
#elif L==7
	"Commento di student";
#elif L==8
	"Komentarz studenta";
#elif L==9
	"Coment&aacute;rio do estudante";
#endif

const char *Txt_Students_ABBREVIATION =	// Abbreviation of "Students"
#if   L==1
	"Estu.";
#elif L==2
	"Stud.";
#elif L==3
	"Stud.";
#elif L==4
	"Estu.";
#elif L==5
	"&Eacute;tu.";
#elif L==6
	"Temi.";
#elif L==7
	"Stud.";
#elif L==8
	"Stud.";	// Potrzebujesz tlumaczenie
#elif L==9
	"Estu.";
#endif

const char *Txt_students_ABBREVIATION =	// Abbreviation of "students"
#if   L==1
	"estu.";
#elif L==2
	"Stud.";
#elif L==3
	"stud.";
#elif L==4
	"estu.";
#elif L==5
	"&eacute;tu.";
#elif L==6
	"Temi.";
#elif L==7
	"stud.";
#elif L==8
	"stud.";	// Potrzebujesz tlumaczenie
#elif L==9
	"estu.";
#endif

const char *Txt_Student_record_card_in_this_course_has_been_updated =
#if   L==1
	"La ficha del estudiante en la asignatura se ha actualizado.";	// Necessita traduccio
#elif L==2
	"Student record card in this course has been updated.";		// Need bersetzung
#elif L==3
	"Student record card in this course has been updated.";
#elif L==4
	"La ficha del estudiante en la asignatura se ha actualizado.";
#elif L==5
	"Student record card in this course has been updated.";		// Besoin de traduction
#elif L==6
	"La ficha del estudiante en la asignatura se ha actualizado.";	// Okoteve traduccin
#elif L==7
	"La scheda dello/a studente/ssa in questo corso &egrave; stata aggiornata.";
#elif L==8
	"Student record card in this course has been updated.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Student record card in this course has been updated.";		// Necessita de traduo
#endif

const char *Txt_Students_are_no_longer_obliged_to_read_this_information =
#if   L==1
	"Los estudiantes ya no est&aacute;n obligados a leer esta informaci&oacute;n.";	// Necessita traduccio
#elif L==2
	"Students are no longer obliged to read this information.";			// Need bersetzung
#elif L==3
	"Students are no longer obliged to read this information.";
#elif L==4
	"Los estudiantes ya no est&aacute;n obligados a leer esta informaci&oacute;n.";
#elif L==5
	"Students are no longer obliged to read this information.";			// Besoin de traduction
#elif L==6
	"Los estudiantes ya no est&aacute;n obligados a leer esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Gli studenti non sono obbligati a leggere questa informazione.";
#elif L==8
	"Students are no longer obliged to read this information.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Students are no longer obliged to read this information.";			// Necessita de traduo
#endif

const char *Txt_Students_now_are_required_to_read_this_information =
#if   L==1
	"Los estudiantes ahora est&aacute;n obligados a leer esta informaci&oacute;n.";	// Necessita traduccio
#elif L==2
	"Students now are required to read this information.";				// Need bersetzung
#elif L==3
	"Students now are required to read this information.";
#elif L==4
	"Los estudiantes ahora est&aacute;n obligados a leer esta informaci&oacute;n.";
#elif L==5
	"Students now are required to read this information.";				// Besoin de traduction
#elif L==6
	"Los estudiantes ahora est&aacute;n obligados a leer esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Gli studenti ora sono obbligati a leggere questa informazione.";
#elif L==8
	"Students now are required to read this information.";				// Potrzebujesz tlumaczenie
#elif L==9
	"Students now are required to read this information.";				// Necessita de traduo
#endif

const char *Txt_X_students_who_have_email =	// Warning: it is very important to include %u and %f in the following sentences
#if   L==1
	"%u estudiantes con correo (%.1f%% de %u estudiantes)";	// Necessita traduccio
#elif L==2
	"%u Studenten mit E-Mail (%.1f%% von %u Studenten)";
#elif L==3
	"%u students who have email (%.1f%% of %u students)";
#elif L==4
	"%u estudiantes con correo (%.1f%% de %u estudiantes)";
#elif L==5
	"%u &eacute;tudiants qui ont l'email (%.1f%% de %u &eacute;tudiants)";
#elif L==6
	"%u estudiantes con correo (%.1f%% de %u estudiantes)";	// Okoteve traduccin
#elif L==7
	"%u studenti che hanno email (%.1f%% di %u studenti)";
#elif L==8
	"%u students who have email (%.1f%% of %u students)";				// Potrzebujesz tlumaczenie
#elif L==9
	"%u estudantes que t&ecirc;m email (%.1f%% dos %u estudantes)";
#endif

const char *Txt_Students_who_have_accepted_and_who_have_email =
#if   L==1
	"Estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo";	// Necessita traduccio
#elif L==2
	"Studenten die akzeptiert wurden"
	" und die E-Mail";
#elif L==3
	"Students who have accepted"
	" and who have email";
#elif L==4
	"Estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo";
#elif L==5
	"&Eacute;tudiants qui ont accept&eacute;"
	" et qui ont l'email";
#elif L==6
	"Estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo";	// Okoteve traduccin
#elif L==7
	"Studenti che hanno aderito"
	" e che hanno email";
#elif L==8
	"Students who have accepted"
	" and who have email";				// Potrzebujesz tlumaczenie
#elif L==9
	"Estudantes que aceitaram"
	" e que t&ecirc;m email";
#endif

const char *Txt_X_students_who_have_accepted_and_who_have_email =	// Warning: it is very important to include %u and %f in the following sentences
#if   L==1
	"%u estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo (%.1f%% de %u estudiantes)";	// Necessita traduccio
#elif L==2
	"%u Studenten die akzeptiert wurden"
	" und die E-Mail (%.1f%% von %u Studenten)";
#elif L==3
	"%u students who have accepted"
	" and who have email (%.1f%% of %u students)";
#elif L==4
	"%u estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo (%.1f%% de %u estudiantes)";
#elif L==5
	"%u &eacute;tudiants qui ont accept&eacute;"
	" et qui ont l'email (%.1f%% de %u &eacute;tudiants)";
#elif L==6
	"%u estudiantes que han aceptado la inscripci&oacute;n"
	" y que tienen correo (%.1f%% de %u estudiantes)";	// Okoteve traduccin
#elif L==7
	"%u studenti che hanno aderito"
	" e che hanno email (%.1f%% di %u studenti)";
#elif L==8
	"%u students who have accepted"
	" and who have email (%.1f%% of %u students)";				// Potrzebujesz tlumaczenie
#elif L==9
	"%u estudantes que aceitaram"
	" e que t&ecirc;m email (%.1f%% dos %u estudantes)";
#endif

const char *Txt_students_with_no_group =
#if   L==1
	"estudiantes en ning&uacute;n grupo";	// Necessita traduccio
#elif L==2
	"Studente mit nicht-Gruppe";
#elif L==3
	"students with no group";
#elif L==4
	"estudiantes en ning&uacute;n grupo";
#elif L==5
	"&eacute;tudiants avec aucun groupe";
#elif L==6
	"estudiantes en ning&uacute;n grupo";	// Okoteve traduccin
#elif L==7
	"studenti con nessun gruppo";
#elif L==8
	"students with no group";		// Potrzebujesz tlumaczenie
#elif L==9
	"estudantes com nenhum grupo";
#endif

const char *Txt_Surname_1 =
#if   L==1
	"Primer cognom";
#elif L==2
	"Nachname 1";
#elif L==3
	"Surname 1";
#elif L==4
	"Primer apellido";
#elif L==5
	"Nom de famille 1";
#elif L==6
	"Terajoapy 1";
#elif L==7
	"Primo cognome";
#elif L==8
	"Nazwisko 1";
#elif L==9
	"Primeiro apelido";
#endif

const char *Txt_Surname_2 =
#if   L==1
	"Segon cognom";
#elif L==2
	"Nachname 2";
#elif L==3
	"Surname 2";
#elif L==4
	"Segundo apellido";
#elif L==5
	"Nom de famille 2";
#elif L==6
	"Terajoapy 2";
#elif L==7
	"Secondo cognome";
#elif L==8
	"Nazwisko 2";
#elif L==9
	"Segundo apelido";
#endif

const char *Txt_Survey =
#if   L==1
	"Encuesta";	// Necessita traduccio
#elif L==2
	"Umfrage";
#elif L==3
	"Survey";
#elif L==4
	"Encuesta";
#elif L==5
	"Sondage";
#elif L==6
	"Encuesta";	// Okoteve traduccin
#elif L==7
	"Sondaggio";
#elif L==8
	"Survey";	// Potrzebujesz tlumaczenie
#elif L==9
	"Inqu&eacute;rito";
#endif

const char *Txt_SURVEY_You_belong_to_the_scope_of_the_survey =
#if   L==1
	"Pertany a l'&agrave;mbit de l'enquesta";
#elif L==2
	"Sie geh&ouml;ren zum Umfang der Umfrage";
#elif L==3
	"You belong to the scope of the survey";
#elif L==4
	"Pertenece al &aacute;mbito de la encuesta";
#elif L==5
	"Vous appartenez &agrave; la port&eacute;e de l'enqu&ecirc;te";
#elif L==6
	"Pertenece al &aacute;mbito de la encuesta";	// Okoteve traduccin
#elif L==7
	"Voi appartenete al campo d'indagine";
#elif L==8
	"Nale&zdot;ysz do zakresu badania";
#elif L==9
	"Voc&ecirc; pertence ao &acirc;mbito do inqu&eacute;rito";
#endif

const char *Txt_SURVEY_You_dont_belong_to_the_scope_of_the_survey =
#if   L==1
	"No pertany a l'&agrave;mbit de l'enquesta";
#elif L==2
	"Sie geh&ouml;ren nicht zum Umfang der Befragung";
#elif L==3
	"You don't belong to the scope of the survey)";
#elif L==4
	"No pertenece al &aacute;mbito de la encuesta";
#elif L==5
	"Vous n'appartenez pas &agrave; la port&eacute;e de l'enqu&ecirc;te";
#elif L==6
	"No pertenece al &aacute;mbito de la encuesta";	// Okoteve traduccin
#elif L==7
	"Voi non appartieni al campo d'indagine";
#elif L==8
	"Nie nale&zdot;&aogon; do zakresu badania";
#elif L==9
	"Voc&ecirc; n&atilde;o pertence ao &acirc;mbito do inqu&eacute;rito";
#endif

const char *Txt_SURVEY_You_have_already_answered =
#if   L==1
	"Ya ha respondido";		// Necessita traduccio
#elif L==2
	"You have already answered";	// Need bersetzung
#elif L==3
	"You have already answered";
#elif L==4
	"Ya ha respondido";
#elif L==5
	"You have already answered";	// Besoin de traduction
#elif L==6
	"Ya ha respondido";		// Okoteve traduccin
#elif L==7
	"Hai gi&agrave; risposto";
#elif L==8
	"You have already answered";	// Potrzebujesz tlumaczenie
#elif L==9
	"You have already answered";	// Necessita de traduo
#endif

const char *Txt_SURVEY_You_have_not_answered =
#if   L==1
	"No ha respondido";		// Necessita traduccio
#elif L==2
	"You have not answered";	// Need bersetzung
#elif L==3
	"You have not answered";
#elif L==4
	"No ha respondido";
#elif L==5
	"You have not answered";	// Besoin de traduction
#elif L==6
	"No ha respondido";		// Okoteve traduccin
#elif L==7
	"Non hai risposto";
#elif L==8
	"You have not answered";	// Potrzebujesz tlumaczenie
#elif L==9
	"You have not answered";	// Necessita de traduo
#endif

const char *Txt_SURVEY_Type_of_user_allowed =
#if   L==1
	"Tipo de usuario permitido";	// Necessita traduccio
#elif L==2
	"Type of user allowed";		// Need bersetzung
#elif L==3
	"Type of user allowed";
#elif L==4
	"Tipo de usuario permitido";
#elif L==5
	"Type of user allowed";		// Besoin de traduction
#elif L==6
	"Tipo de usuario permitido";	// Okoteve traduccin
#elif L==7
	"Tipo di utente autorizzato";
#elif L==8
	"Type of user allowed";		// Potrzebujesz tlumaczenie
#elif L==9
	"Type of user allowed";		// Necessita de traduo
#endif

const char *Txt_SURVEY_Type_of_user_not_allowed =
#if   L==1
	"Tipo de usuario no permitido";		// Necessita traduccio
#elif L==2
	"Type of user not allowed";		// Need bersetzung
#elif L==3
	"Type of user not allowed";
#elif L==4
	"Tipo de usuario no permitido";
#elif L==5
	"Type of user not allowed";		// Besoin de traduction
#elif L==6
	"Tipo de usuario no permitido";		// Okoteve traduccin
#elif L==7
	"Tipo di utente non autorizzato";
#elif L==8
	"Type of user not allowed";		// Potrzebujesz tlumaczenie
#elif L==9
	"Type of user not allowed";		// Necessita de traduo
#endif

const char *Txt_Survey_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La encuesta <strong>%s</strong> ahora est&aacute; oculta.";	// Necessita traduccio
#elif L==2
	"Die Umfrage <strong>%s</strong> ist jetzt ausgeblendet.";
#elif L==3
	"Survey <strong>%s</strong> is now hidden.";
#elif L==4
	"La encuesta <strong>%s</strong> ahora est&aacute; oculta.";
#elif L==5
	"Le sondage <strong>%s</strong> est maintenant cach&eacute;.";
#elif L==6
	"La encuesta <strong>%s</strong> ahora est&aacute; oculta.";	// Okoteve traduccin
#elif L==7
	"Il sondaggio <strong>%s</strong> &egrave; ora nascosto.";
#elif L==8
	"Survey <strong>%s</strong> is now hidden.";			// Potrzebujesz tlumaczenie
#elif L==9
	"O inqu&eacute;rito <strong>%s</strong> &eacute; agora oculta.";
#endif

const char *Txt_Survey_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La encuesta <strong>%s</strong> ahora est&aacute; visible.";	// Necessita traduccio
#elif L==2
	"Die Umfrage <strong>%s</strong> ist jetzt sichtbar.";
#elif L==3
	"Survey <strong>%s</strong> is now visible.";
#elif L==4
	"La encuesta <strong>%s</strong> ahora est&aacute; visible.";
#elif L==5
	"Le sondage <strong>%s</strong> est maintenant visible.";
#elif L==6
	"La encuesta <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7
	"Il sondaggio <strong>%s</strong> &egrave; ora visibile.";
#elif L==8
	"Survey <strong>%s</strong> is now visible.";			// Potrzebujesz tlumaczenie
#elif L==9
	"O inqu&eacute;rito <strong>%s</strong> &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_Survey_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Encuesta <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2
	"Umfrage <strong>%s</strong> entfernt.";
#elif L==3
	"Survey <strong>%s</strong> removed.";
#elif L==4
	"Encuesta <strong>%s</strong> eliminada.";
#elif L==5
	"Sondage <strong>%s</strong> supprim&eacute;.";
#elif L==6
	"Encuesta <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7
	"Sondaggio <strong>%s</strong> rimosso.";
#elif L==8
	"Survey <strong>%s</strong> removed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Inqu&eacute;rito <strong>%s</strong> removido.";
#endif

const char *Txt_Survey_X_reset =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Encuesta <strong>%s</strong> puesta a cero.";	// Necessita traduccio
#elif L==2
	"Umfrage <strong>%s</strong> zur&uuml;ckgesetzt.";
#elif L==3
	"Survey <strong>%s</strong> reset.";
#elif L==4
	"Encuesta <strong>%s</strong> puesta a cero.";
#elif L==5
	"Sondage <strong>%s</strong> remis &agrave; z&eacute;ro.";
#elif L==6
	"Encuesta <strong>%s</strong> puesta a cero.";	// Okoteve traduccin
#elif L==7
	"Sondaggio <strong>%s</strong> resettato.";
#elif L==8
	"Survey <strong>%s</strong> reset.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Inqu&eacute;rito <strong>%s</strong> fixado em zero.";
#endif

const char *Txt_SURVEY_STR_ANSWER_TYPES[Svy_NUM_ANS_TYPES] =
   {
#if   L==1
	 "Elecci&oacute; &uacute;nica"
#elif L==2
	 "Unique-Choice"
#elif L==3
	 "Unique choice"
#elif L==4
	 "Elecci&oacute;n &uacute;nica"
#elif L==5
	 "Choix unique"
#elif L==6
	 "Elecci&oacute;n &uacute;nica"		// Okoteve traduccin
#elif L==7
	 "Scelta unica"
#elif L==8
	 "Unique choice"			// Potrzebujesz tlumaczenie
#elif L==9
	 "&Uacute;nica escolha"
#endif
	 ,
#if   L==1
	 "Elecci&oacute; m&uacute;ltiple"
#elif L==2
	 "Multiple-Choice"
#elif L==3
	 "Multiple choice"
#elif L==4
	 "Elecci&oacute;n m&uacute;ltiple"
#elif L==5
	 "Choix multiple"
#elif L==6
	 "Elecci&oacute;n m&uacute;ltiple"	// Okoteve traduccin
#elif L==7
	 "Scelta multipla"
#elif L==8
	 "Multiple choice"			// Potrzebujesz tlumaczenie
#elif L==9
	 "M&uacute;ltipla escolha"
#endif
	};

const char *Txt_Surveys =
#if   L==1
	"Enquestes";
#elif L==2
	"Umfragen";
#elif L==3
	"Surveys";
#elif L==4
	"Encuestas";
#elif L==5
	"Sondages";
#elif L==6
	"Encuestas";	// Okoteve traduccin
#elif L==7
	"Sondaggi";
#elif L==8
	"Ankiety";
#elif L==9
	"Inqu&eacute;ritos";
#endif

const char *Txt_Syllabus =
#if   L==1
	"Programa";
#elif L==2
	"Studienplan";
#elif L==3
	"Syllabus";
#elif L==4
	"Programa";
#elif L==5
	"Plan";
#elif L==6
	"Programa";	// Okoteve traduccin
#elif L==7
	"Programma";
#elif L==8
	"Syllabus";	// Potrzebujesz tlumaczenie
#elif L==9
	"Programa";
#endif

const char *Txt_Syllabus_of_the_course =
#if   L==1
	"Programa de la asignatura";	// Necessita traduccio
#elif L==2
	"Studienplan der Kurs";
#elif L==3
	"Syllabus of the course";
#elif L==4
	"Programa de la asignatura";
#elif L==5
	"Plan de la mati&egrave;re";
#elif L==6
	"Programa de la asignatura";	// Okoteve traduccin
#elif L==7
	"Programma del corso";
#elif L==8
	"Syllabus of the course";	// Potrzebujesz tlumaczenie
#elif L==9
	"Programa da disciplina";
#endif

const char *Txt_SYLLABUS_WHICH_SYLLABUS[Syl_NUM_WHICH_SYLLABUS] =
	 {
	 // Lectures
#if   L==1
	 "Teoria"
#elif L==2
	 "Vorlesung"
#elif L==3
	 "Lectures"
#elif L==4
	 "Teor&iacute;a"
#elif L==5
	 "Th&eacute;orie"
#elif L==6
	 "Teor&iacute;a"		// Okoteve traduccin
#elif L==7
	 "Teoria"
#elif L==8
	 "Wyklady"
#elif L==9
	 "Teoria"
#endif
	 ,
	 // Practicals
#if   L==1
	 "Pr&agrave;ctiques"
#elif L==2
	 "&Uuml;bung"
#elif L==3
	 "Practicals"
#elif L==4
	 "Pr&aacute;cticas"
#elif L==5
	 "Pratique"
#elif L==6
	 "Pr&aacute;cticas"		// Okoteve traduccin
#elif L==7
	 "Pratica"
#elif L==8
	 "Praktyki"
#elif L==9
	 "Pr&aacute;tica"
#endif
	 ,
	 };

const char *Txt_System =
#if   L==1
	"Plataforma";
#elif L==2
	"Plattform";
#elif L==3
	"System";
#elif L==4
	"Plataforma";
#elif L==5
	"Plate-forme";
#elif L==6
	"Ypykatu";
#elif L==7
	"Piattaforma";
#elif L==8
	"System";
#elif L==9
	"Sistema";
#endif

const char *Txt_TABLE_Footer =
#if   L==1
	"Pie";		// Necessita traduccio
#elif L==2
	"Fu&szlig;";
#elif L==3
	"Footer";
#elif L==4
	"Pie";
#elif L==5
	"Pied";
#elif L==6
	"Pie";		// Okoteve traduccin
#elif L==7
	"Piede";
#elif L==8
	"Footer";	// Potrzebujesz tlumaczenie
#elif L==9
	"P&eacute;";
#endif

const char *Txt_TABLE_Header =
#if   L==1
	"Cabecera";	// Necessita traduccio
#elif L==2
	"Header";
#elif L==3
	"Header";
#elif L==4
	"Cabecera";
#elif L==5
	"T&ecirc;te";
#elif L==6
	"Cabecera";	// Okoteve traduccin
#elif L==7
	"Intestazione";
#elif L==8
	"Header";	// Potrzebujesz tlumaczenie
#elif L==9
	"Cabe&ccedil;alho";
#endif

const char *Txt_Table_not_found_in_the_file_of_marks =
#if   L==1
	"No hay ninguna tabla en el archivo de calificaciones.";	// Necessita traduccio
#elif L==2
	"Table not found in the file of marks.";			// Need bersetzung
#elif L==3
	"Table not found in the file of marks.";
#elif L==4
	"No hay ninguna tabla en el archivo de calificaciones.";
#elif L==5
	"Table not found in the file of marks.";			// Besoin de traduction
#elif L==6
	"No hay ninguna tabla en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==7
	"Tabella non trovata nel file dei risultati.";
#elif L==8
	"Table not found in the file of marks.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Table not found in the file of marks.";			// Necessita de traduo
#endif

const char *Txt_TABS_TXT[Tab_NUM_TABS] =
   {
	 // TabUnk
	 NULL,
	 // TabSys
#if   L==1
	 "Plataforma"
#elif L==2
	 "Plattform"
#elif L==3
	 "System"
#elif L==4
	 "Plataforma"
#elif L==5
	 "Plate-forme"
#elif L==6
	 "Ypykatu"
#elif L==7
	 "Piattaforma"
#elif L==8
	 "System"
#elif L==9
	 "Sistema"
#endif
	 ,
	 // TabCty
#if   L==1
	 "Pa&iacute;s"
#elif L==2
	 "Land"
#elif L==3
	 "Country"
#elif L==4
	 "Pa&iacute;s"
#elif L==5
	 "Pays"
#elif L==6
	 "Tet&atilde;"
#elif L==7
	 "Paese"
#elif L==8
	 "Kraj"
#elif L==9
	 "Pa&iacute;s"
#endif
	 ,
	 // TabIns
#if   L==1
	 "Instituci&oacute;"
#elif L==2
	 "Hochschule"
#elif L==3
	 "Institution"
#elif L==4
	 "Instituci&oacute;n"
#elif L==5
	 "&Eacute;tablissement"
#elif L==6
	 "Mbo'ehao"
#elif L==7
	 "Istituzione"
#elif L==8
	 "Instytucji"
#elif L==9
	 "Institu&ccedil;&atilde;o"
#endif
	 ,
	 // TabCtr
#if   L==1
	 "Centre"
#elif L==2
	 "Lehrinstitut"
#elif L==3
	 "Centre"
#elif L==4
	 "Centro"
#elif L==5
	 "Centre"
#elif L==6
	 "Mbo'ehao"
#elif L==7
	 "Centro"
#elif L==8
	 "Centrum"
#elif L==9
	 "Centro"
#endif
	 ,
	 // TabDeg
#if   L==1
	 "Titulaci&oacute;"
#elif L==2
	 "Studiengang"
#elif L==3
	 "Degree"
#elif L==4
	 "Titulaci&oacute;n"
#elif L==5
	 "&Eacute;tude"
#elif L==6
	 "Arandur&atilde;"
#elif L==7
	 "Laurea"
#elif L==8
	 "Stopien"
#elif L==9
	 "Titula&ccedil;&atilde;o"
#endif
	 ,
	 // TabCrs
#if   L==1
	 "Assignatura"
#elif L==2
	 "Kurs"
#elif L==3
	 "Course"
#elif L==4
	 "Asignatura"
#elif L==5
	 "Mati&egrave;re"
#elif L==6
	 "Mbo'esyry"
#elif L==7
	 "Corso"
#elif L==8
	 "Kurs"
#elif L==9
	 "Disciplina"
#endif
	 ,
	 // TabAss
#if   L==1
	 "Avaluaci&oacute;"
#elif L==2
	 "Bewertung"
#elif L==3
	 "Assessment"
#elif L==4
	 "Evaluaci&oacute;n"
#elif L==5
	 "&Eacute;valuation"
#elif L==6
	 "Mbohepy"
#elif L==7
	 "Valutazione"
#elif L==8
	 "Ocena"
#elif L==9
	 "Avalia&ccedil;&atilde;o"
#endif
	 ,
	 // TabFil
#if   L==1
	"Fitxers"
#elif L==2
	"Dateien"
#elif L==3
	"Files"
#elif L==4
	"Archivos"
#elif L==5
	"Fichiers"
#elif L==6
	"Archivos"	// Okoteve traduccin
#elif L==7
	"File"
#elif L==8
	"Pliki"
#elif L==9
	"Arquivos"
#endif
	 ,
	 // TabUsr
#if   L==1
	 "Usuaris"
#elif L==2
	 "Benutzer"
#elif L==3
	 "Users"
#elif L==4
	 "Usuarios"
#elif L==5
	 "Utilisateurs"
#elif L==6
	 "Puruh&aacute;ra"
#elif L==7
	 "Utenti"
#elif L==8
	 "U&zdot;ytkownicy"
#elif L==9
	 "Utilizadores"
#endif
	 ,
	 // TabSoc
#if   L==1
	 "Social"
#elif L==2
	 "Soziales"
#elif L==3
	 "Social"
#elif L==4
	 "Social"
#elif L==5
	 "Social"
#elif L==6
	 "Social"	// Okoteve traduccin
#elif L==7
	 "Social"
#elif L==8
	 "Spo&lstrok;eczna"
#elif L==9
	 "Social"
#endif
	 ,
	 // TabMsg
#if   L==1
	 "Missatges"
#elif L==2
	 "Nachrichten"
#elif L==3
	 "Messages"
#elif L==4
	 "Mensajes"
#elif L==5
	 "Messages"
#elif L==6
	 "Marandu"
#elif L==7
	 "Messaggi"
#elif L==8
	 "Wiadomo&sacute;ci"
#elif L==9
	 "Mensagens"
#endif
	 ,
	 // TabSta
#if   L==1
	 "Estad&iacute;stiques"
#elif L==2
	 "Statistik"
#elif L==3
	 "Statistics"
#elif L==4
	 "Estad&iacute;sticas"
#elif L==5
	 "Statistiques"
#elif L==6
	 "Papy'aporeko"
#elif L==7
	 "Statistiche"
#elif L==8
	 "Statystyki"
#elif L==9
	 "Estat&iacute;sticas"
#endif
	 ,
	 // TabPrf
#if   L==1
	 "Perfil"
#elif L==2
	 "Profil"
#elif L==3
	 "Profile"
#elif L==4
	 "Perfil"
#elif L==5
	 "Profil"
#elif L==6
	 "Che"
#elif L==7
	 "Profilo"
#elif L==8
	 "Profil"
#elif L==9
	 "Perfil"
#endif
	};

const char *Txt_Tag_allowed =
#if   L==1
	"Descriptor permitido";	// Necessita traduccio
#elif L==2
	"Tag erlaubt";
#elif L==3
	"Tag allowed";
#elif L==4
	"Descriptor permitido";
#elif L==5
	"Descripteur admis";
#elif L==6
	"Descriptor permitido";	// Okoteve traduccin
#elif L==7
	"Etichetta ammessa";
#elif L==8
	"Tag allowed";		// Potrzebujesz tlumaczenie
#elif L==9
	"Descritor permitido";
#endif

const char *Txt_Tag_X_allowed_Click_to_disable_it =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Descriptor %s permitido. Pulse para prohibirlo.";	// Necessita traduccio
#elif L==2
	"Tag %s allowed. Click to disable it.";			// Need bersetzung
#elif L==3
	"Tag %s allowed. Click to disable it.";
#elif L==4
	"Descriptor %s permitido. Pulse para prohibirlo.";
#elif L==5
	"Tag %s allowed. Click to disable it.";			// Besoin de traduction
#elif L==6
	"Descriptor %s permitido. Pulse para prohibirlo.";	// Okoteve traduccin
#elif L==7
	"Etichetta %s autorizzata. Clicca per censurarla.";
#elif L==8
	"Tag %s allowed. Click to disable it.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Tag %s allowed. Click to disable it.";			// Necessita de traduo
#endif

const char *Txt_Tag_not_allowed =
#if   L==1
	"Descriptor no permitido";	// Necessita traduccio
#elif L==2
	"Tag nicht erlaubt";
#elif L==3
	"Tag not allowed";
#elif L==4
	"Descriptor no permitido";
#elif L==5
	"Descripteur non admis";
#elif L==6
	"Descriptor no permitido";	// Okoteve traduccin
#elif L==7
	"Etichetta non autorizzata";
#elif L==8
	"Tag not allowed";		// Potrzebujesz tlumaczenie
#elif L==9
	"Descritor n&atilde;o permitido";
#endif

const char *Txt_Tag_X_not_allowed_Click_to_allow_it =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Descriptor %s no permitido. Pulse para permitirlo.";	// Necessita traduccio
#elif L==2
	"Tag %s not allowed. Click to allow it.";		// Need bersetzung
#elif L==3
	"Tag %s not allowed. Click to allow it.";
#elif L==4
	"Descriptor %s no permitido. Pulse para permitirlo.";
#elif L==5
	"Tag %s not allowed. Click to allow it.";		// Besoin de traduction
#elif L==6
	"Descriptor %s no permitido. Pulse para permitirlo.";	// Okoteve traduccin
#elif L==7
	"Etichetta %s censurata. Clicca per autorizzarla.";
#elif L==8
	"Tag %s not allowed. Click to allow it.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Tag %s not allowed. Click to allow it.";		// Necessita de traduo
#endif

#if defined LOCALHOST_UBUNTU		// localhost (Ubuntu) *****************

const char *Txt_TAGLINE =
# if   L==1
	"plataforma de suport a la doc&egrave;ncia";
# elif L==2
	"Plattform zur Unterst&uuml;tzung der Lehre";
# elif L==3
	"platform to support teaching";
# elif L==4
	"plataforma de apoyo a la docencia";
# elif L==5
	"plate-forme pour appuyer l'enseignement";
# elif L==6
	"plataforma de apoyo a la docencia";	// Okoteve traduccin
# elif L==7
	"piattaforma per sostenere l'insegnamento";
# elif L==8
	"platforma do wspierania nauczania";
# elif L==9
	"plataforma de apoio ao ensino";
# endif

const char *Txt_TAGLINE_BR =
# if   L==1
	"plataforma de suport<br />a la doc&egrave;ncia";
# elif L==2
	"Plattform zur<br />Unterst&uuml;tzung der Lehre";
# elif L==3
	"platform to<br />support teaching";
# elif L==4
	"plataforma de apoyo<br />a la docencia";
# elif L==5
	"plate-forme pour<br />appuyer l'enseignement";
# elif L==6
	"plataforma de apoyo<br />a la docencia";	// Okoteve traduccin
# elif L==7
	"piattaforma per<br />sostenere l'insegnamento";
# elif L==8
	"platforma do<br />wspierania nauczania";
# elif L==9
	"plataforma de<br />apoio ao ensino";
# endif

#elif defined OPENSWAD_ORG		// openswad.org ***********************

const char *Txt_TAGLINE =
# if   L==1
	"plataforma de suport a la doc&egrave;ncia";
# elif L==2
	"Plattform zur Unterst&uuml;tzung der Lehre";
# elif L==3
	"platform to support teaching";
# elif L==4
	"plataforma de apoyo a la docencia";
# elif L==5
	"plate-forme pour appuyer l'enseignement";
# elif L==6
	"plataforma de apoyo a la docencia";	// Okoteve traduccin
# elif L==7
	"piattaforma per sostenere l'insegnamento";
# elif L==8
	"platforma do wspierania nauczania";
# elif L==9
	"plataforma de apoio ao ensino";
# endif

const char *Txt_TAGLINE_BR =
# if   L==1
	"plataforma de suport<br />a la doc&egrave;ncia";
# elif L==2
	"Plattform zur<br />Unterst&uuml;tzung der Lehre";
# elif L==3
	"platform to<br />support teaching";
# elif L==4
	"plataforma de apoyo<br />a la docencia";
# elif L==5
	"plate-forme pour<br />appuyer l'enseignement";
# elif L==6
	"plataforma de apoyo<br />a la docencia";	// Okoteve traduccin
# elif L==7
	"piattaforma per<br />sostenere l'insegnamento";
# elif L==8
	"platforma do<br />wspierania nauczania";
# elif L==9
	"plataforma de<br />apoio ao ensino";
# endif

#elif defined SWAD_UGR_ES		// swad.ugr.es ************************

const char *Txt_TAGLINE =
# if   L==1
	"plataforma de suport a la doc&egrave;ncia (UGR)";
# elif L==2
	"Plattform zur Unterst&uuml;tzung der Lehre (UGR)";
# elif L==3
	"platform to support teaching (UGR)";
# elif L==4
	"plataforma de apoyo a la docencia (UGR)";
# elif L==5
	"plate-forme pour appuyer l'enseignement (UGR)";
# elif L==6
	"plataforma de apoyo a la docencia (UGR)";	// Okoteve traduccin
# elif L==7
	"piattaforma per sostenere l'insegnamento (UGR)";
# elif L==8
	"platforma do wspierania nauczania (UGR)";
# elif L==9
	"plataforma de apoio ao ensino (UGR)";
# endif

const char *Txt_TAGLINE_BR =
# if   L==1
	"plataforma de suport<br />a la doc&egrave;ncia (UGR)";
# elif L==2
	"Plattform zur Unterst&uuml;tzung<br />der Lehre (UGR)";
# elif L==3
	"platform to support<br />teaching (UGR)";
# elif L==4
	"plataforma de apoyo<br />a la docencia (UGR)";
# elif L==5
	"plate-forme pour appuyer<br />l'enseignement (UGR)";
# elif L==6
	"plataforma de apoyo<br />a la docencia (UGR)";	// Okoteve traduccin
# elif L==7
	"piattaforma per sostenere<br />l'insegnamento (UGR)";
# elif L==8
	"platforma do wspierania<br />nauczania (UGR)";
# elif L==9
	"plataforma de apoio<br />ao ensino (UGR)";
# endif

#elif defined SWADBERRY_UGR_ES		// swadberry.ugr.es *******************

const char *Txt_TAGLINE =
# if   L==1
	"plataforma de suport a la doc&egrave;ncia (Raspberry Pi)";
# elif L==2
	"Plattform zur Unterst&uuml;tzung der Lehre (Raspberry Pi)";
# elif L==3
	"platform to support teaching (Raspberry Pi)";
# elif L==4
	"plataforma de apoyo a la docencia (Raspberry Pi)";
# elif L==5
	"plate-forme pour appuyer l'enseignement (Raspberry Pi)";
# elif L==6
	"plataforma de apoyo a la docencia (Raspberry Pi)";	// Okoteve traduccin
# elif L==7
	"piattaforma per sostenere l'insegnamento (Raspberry Pi)";
# elif L==8
	"platforma do wspierania nauczania (Raspberry Pi)";
# elif L==9
	"plataforma de apoio ao ensino (Raspberry Pi)";
# endif

const char *Txt_TAGLINE_BR =
# if   L==1
	"plataforma de suport a la<br />doc&egrave;ncia (Raspberry Pi)";
# elif L==2
	"Plattform zur Unterst&uuml;tzung<br />der Lehre (Raspberry Pi)";
# elif L==3
	"platform to support<br />teaching (Raspberry Pi)";
# elif L==4
	"plataforma de apoyo a la<br />docencia (Raspberry Pi)";
# elif L==5
	"plate-forme pour appuyer<br />l'enseignement (Raspberry Pi)";
# elif L==6
	"plataforma de apoyo a la<br />docencia (Raspberry Pi)";	// Okoteve traduccin
# elif L==7
	"piattaforma per sostenere<br />l'insegnamento (Raspberry Pi)";
# elif L==8
	"platforma do wspierania<br />nauczania (Raspberry Pi)";
# elif L==9
	"plataforma de apoio<br />ao ensino (Raspberry Pi)";
# endif

#endif

const char *Txt_Tags =		// Tags used in test
#if   L==1
	"Descriptors";
#elif L==2
	"Tags";
#elif L==3
	"Tags";
#elif L==4
	"Descriptores";
#elif L==5
	"Descripteurs";
#elif L==6
	"Descriptores";	// Okoteve traduccin
#elif L==7
	"Etichette";
#elif L==8
	"Tags";		// Potrzebujesz tlumaczenie
#elif L==9
	"Descritores";
#endif

const char *Txt_Take_a_test =
#if   L==1
	"Fer un test";
#elif L==2
	"F&uuml;hren Sie einen Test";
#elif L==3
	"Take a test";
#elif L==4
	"Hacer un test";
#elif L==5
	"Passer un test";
#elif L==6
	"Hacer un test";	// Okoteve traduccin
#elif L==7
	"Fare un test";
#elif L==8
	"Zr&oacute;b test";
#elif L==9
	"Fazer um teste";
#endif

const char *Txt_Teachers_ABBREVIATION =	// Abbreviation of "Teachers"
#if   L==1
	"Prof.";
#elif L==2
	"Lehr.";
#elif L==3
	"Teac.";
#elif L==4
	"Prof.";
#elif L==5
	"Ense.";
#elif L==6
	"Prof.";	// Okoteve traduccin
#elif L==7
	"Prof.";
#elif L==8
	"Nauc.";
#elif L==9
	"Prof.";
#endif

const char *Txt_teachers_ABBREVIATION =	// Abbreviation of "Teachers"
#if   L==1
	"prof.";
#elif L==2
	"Lehr.";
#elif L==3
	"teac.";
#elif L==4
	"prof.";
#elif L==5
	"ense.";
#elif L==6
	"prof.";	// Okoteve traduccin
#elif L==7
	"prof.";
#elif L==8
	"nauc.";
#elif L==9
	"prof.";
#endif

const char *Txt_Teachers_files_area =
#if   L==1
	"Zona d'arxius de professors";
#elif L==2
	"Lehrer Dateien-Bereich";
#elif L==3
	"Teachers' files area";
#elif L==4
	"Zona de archivos de profesores";
#elif L==5
	"Zone de fichiers d'enseignants";
#elif L==6
	"Zona de archivos de profesores";	// Okoteve traduccin
#elif L==7
	"Zona di file di professori";
#elif L==8
	"Nauczyciela udost&eogon;pnionych plik&oacute;w";
#elif L==9
	"Zona de arquivos dos professores";
#endif

const char *Txt_teachers_files_area =
#if   L==1
	"zona d'arxius de professors";
#elif L==2
	"Lehrer Dateien-Bereich";
#elif L==3
	"teachers' files area";
#elif L==4
	"zona de archivos de profesores";
#elif L==5
	"zone de fichiers d'enseignants";
#elif L==6
	"zona de archivos de profesores";	// Okoteve traduccin
#elif L==7
	"zona di file di professori";
#elif L==8
	"nauczyciela udost&eogon;pnionych plik&oacute;w";
#elif L==9
	"zona de arquivos dos professores";
#endif

const char *Txt_Teaching_platform =
#if   L==1
	"Plataforma docent";
#elif L==2
	"Lernplattform ";
#elif L==3
	"Teaching platform";
#elif L==4
	"Plataforma docente";
#elif L==5
	"Plate-forme d'enseignement";
#elif L==6
	"Plataforma docente";	// Okoteve traduccin
#elif L==7
	"Piattaforma didattica";
#elif L==8
	"Platforma nauczania";
#elif L==9
	"Plataforma de ensino";
#endif

const char *Txt_Temporary_private_storage_area =
#if   L==1
	"Zona d'emmagatzematge privada";
#elif L==2
	"Privater Speicherbereich";
#elif L==3
	"Temporary private storage area";
#elif L==4
	"Zona de almacenamiento privada temporal";
#elif L==5
	"Zone de stockage priv&eacute;e";
#elif L==6
	"Zona de almacenamiento privada temporal";	// Okoteve traduccin
#elif L==7
	"Area di archiviazione privata";
#elif L==8
	"Prywatny magazyn";
#elif L==9
	"Zona privada de armazenamento";
#endif

const char *Txt_temporary_private_storage_area =
#if   L==1
	"zona d'emmagatzematge privada";
#elif L==2
	"privater Speicherbereich";
#elif L==3
	"temporary private storage area";
#elif L==4
	"zona de almacenamiento privada temporal";
#elif L==5
	"zone de stockage priv&eacute;e";
#elif L==6
	"zona de almacenamiento privada temporal";	// Okoteve traduccin
#elif L==7
	"area di archiviazione privata";
#elif L==8
	"obszar prywatnego przechowywania";
#elif L==9
	"zona privada de armazenamento";
#endif

const char *Txt_Test =
#if   L==1
	"Test";
#elif L==2
	"Test";
#elif L==3
	"Test";
#elif L==4
	"Test";
#elif L==5
	"Test";
#elif L==6
	"Test";	// Okoteve traduccin
#elif L==7
	"Test";
#elif L==8
	"Test";
#elif L==9
	"Teste";
#endif

const char *Txt_TEST_Correct_answer =
#if   L==1
	"Resposta correcta";
#elif L==2
	"Correct answer";		// Need bersetzung
#elif L==3
	"Correct answer";
#elif L==4
	"Respuesta correcta";
#elif L==5
	"Correct answer";		// Besoin de traduction
#elif L==6
	"Respuesta correcta";		// Okoteve traduccin
#elif L==7
	"Risposta corretta";
#elif L==8
	"Prawidlowa odpowiedz";
#elif L==9
	"Correct answer";		// Necessita de traduo
#endif

const char *Txt_TEST_User_answer =
#if   L==1
	"Resposta usuari";
#elif L==2
	"User's answer";		// Need bersetzung
#elif L==3
	"User's answer";
#elif L==4
	"Respuesta usuario";
#elif L==5
	"User's answer";		// Besoin de traduction
#elif L==6
	"Respuesta usuario";		// Okoteve traduccin
#elif L==7
	"Risposta utente";
#elif L==8
	"User's answer";		// Potrzebujesz tlumaczenie
#elif L==9
	"User's answer";		// Necessita de traduo
#endif

const char *Txt_Test_No_X_that_you_make_in_this_course =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Test n&ordm; %u que realiza usted en esta asignatura";	// Necessita traduccio
#elif L==2
	"Der %u. Test, den Sie in dieser Kurs machen";
#elif L==3
	"Test No. %u that you make in this course";
#elif L==4
	"Test n&ordm; %u que realiza usted en esta asignatura";
#elif L==5
	"Test n&ordm; %u que vous faites dans cette mati&egrave;re";
#elif L==6
	"Test n&ordm; %u que realiza usted en esta asignatura";	// Okoteve traduccin
#elif L==7
	"Test n. %u che fai in questo corso";
#elif L==8
	"Test No. %u that you make in this course";		// Potrzebujesz tlumaczenie
#elif L==9
	"Teste n&ordm; %u que voc&ecirc; faz nesta disciplina";
#endif

const char *Txt_Test_result =
#if   L==1
	"Resultat del test";
#elif L==2
	"Testergebnis";
#elif L==3
	"Test result";
#elif L==4
	"Resultado del test";
#elif L==5
	"R&eacute;sultat du test";
#elif L==6
	"Resultado del test";	// Okoteve traduccin
#elif L==7
	"Risultato del test";
#elif L==8
	"Wynik testu";
#elif L==9
	"Resultado do teste";
#endif

const char *Txt_Test_results =
#if   L==1
	"Resultats de tests";
#elif L==2
	"Testergebnisse";
#elif L==3
	"Test results";
#elif L==4
	"Resultados de tests";
#elif L==5
	"R&eacute;sultats des tests";
#elif L==6
	"Resultados de tests";	// Okoteve traduccin
#elif L==7
	"Risultati dei test";
#elif L==8
	"Wyniki test&oacute;w";
#elif L==9
	"Resultados dos testes";
#endif

const char *Txt_Text =
#if   L==1
	"Text";
#elif L==2
	"Text";
#elif L==3
	"Text";
#elif L==4
	"Texto";
#elif L==5
	"Texte";
#elif L==6
	"Texto";	// Okoteve traduccin
#elif L==7
	"Testo";
#elif L==8
	"Tekst";
#elif L==9
	"Texto";
#endif

const char *Txt_TEXT_plain =
#if   L==1
	 "pla";
#elif L==2
	 "Plain";
#elif L==3
	 "plain";
#elif L==4
	 "plano";
#elif L==5
	 "plain";
#elif L==6
	 "plano";	// Okoteve traduccin
#elif L==7
	 "normale";
#elif L==8
	 "zwykly";
#elif L==9
	 "plano";
#endif

const char *Txt_Thank_you_for_reporting_a_possible_duplicate_user =
#if   L==1
	"Gr&agrave;cies per informar d'un usuari possiblement duplicat.";
#elif L==2
	"Vielen Dank fr eine m&ouml;gliche doppelte Benutzer berichten.";
#elif L==3
	"Thank you for reporting a possible duplicate user.";
#elif L==4
	"Gracias por informar de un usuario posiblemente duplicado.";
#elif L==5
	"Merci d'avoir signal&eacute; un &eacute;ventuel utilisateur en double.";
#elif L==6
	"Gracias por informar de un usuario posiblemente duplicado.";	// Okoteve traduccin
#elif L==7
	"Grazie per aver segnalato un possibile utente duplicato.";
#elif L==8
	"Dzi&eogon;kujemy za zg&lstrok;oszenie ewentualnego duplikat u&zdot;ytkownika.";
#elif L==9
	"Obrigado por relatar um possvel usu&aacute;rio duplicado.";
#endif

const char *Txt_Thanks_for_answering_the_survey =
#if   L==1
	"Gracias por responder la encuesta.";	// Necessita traduccio
#elif L==2
	"Vielen Dank f&uuml;r die Beantwortung der Umfrage.";
#elif L==3
	"Thanks for answering the survey.";
#elif L==4
	"Gracias por responder la encuesta.";
#elif L==5
	"Merci de r&eacute;pondre au sondage.";
#elif L==6
	"Gracias por responder la encuesta.";	// Okoteve traduccin
#elif L==7
	"Grazie per la risposta al sondaggio.";
#elif L==8
	"Thanks for answering the survey.";	// Potrzebujesz tlumaczenie
#elif L==9
	"Obrigado por responder o inqu&eacute;rito.";
#endif

const char *Txt_The_alphabetical_code_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s ya existe.";	// Necessita traduccio
#elif L==2
	"The ISO 3166-1 alphabetical code %s already exists.";		// Need bersetzung
#elif L==3
	"The ISO 3166-1 alphabetical code %s already exists.";
#elif L==4
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s ya existe.";
#elif L==5
	"The ISO 3166-1 alphabetical code %s already exists.";		// Besoin de traduction
#elif L==6
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s ya existe.";	// Okoteve traduccin
#elif L==7
	"Il codice alfabetico ISO 3166-1 %s gi&agrave; esiste.";
#elif L==8
	"The ISO 3166-1 alphabetical code %s already exists.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The ISO 3166-1 alphabetical code %s already exists.";		// Necessita de traduo
#endif

const char *Txt_The_alphabetical_code_X_is_not_correct =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s no es correcto.";	// Necessita traduccio
#elif L==2
	"The ISO 3166-1 alphabetical code %s is not correct.";			// Need bersetzung
#elif L==3
	"The ISO 3166-1 alphabetical code %s is not correct.";
#elif L==4
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s no es correcto.";
#elif L==5
	"The ISO 3166-1 alphabetical code %s is not correct.";			// Besoin de traduction
#elif L==6
	"El c&oacute;digo alfab&eacute;tico ISO 3166-1 %s no es correcto.";	// Okoteve traduccin
#elif L==7
	"Il codice alfabetico ISO 3166-1 %s non &egrave; corretto.";
#elif L==8
	"The ISO 3166-1 alphabetical code %s is not correct.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The ISO 3166-1 alphabetical code %s is not correct.";			// Necessita de traduo
#endif

const char *Txt_The_announcement_of_exam_has_been_successfully_updated =
#if   L==1
	"La convocat&ograve;ria d'examen s'ha actualitzat correctament.";
#elif L==2
	"Der Aufruf zur Pr&uuml;fung wurde erfolgreich aktualisiert.";
#elif L==3
	"The announcement of exam has been successfully updated.";
#elif L==4
	"La convocatoria de examen se ha actualizado correctamente.";
#elif L==5
	"La convocation pour l'examen a &eacute;t&eacute; mis &agrave; jour avec succ&egrave;s.";
#elif L==6
	"La convocatoria de examen se ha actualizado correctamente.";	// Okoteve traduccin
#elif L==7
	"L'apello d'esame &egrave; stata aggiornata con successo.";
#elif L==8
	"Ogloszenie egzamin zosta&lstrok; pomy&sacute;lnie zaktualizowany.";
#elif L==9
	"A chamada para exame foi atualizada com sucesso.";
#endif

const char *Txt_The_announcement_of_exam_is_now_hidden =
#if   L==1
	"La convocat&ograve;ria d'examen ara est&agrave; oculta.";
#elif L==2
	"Der Aufruf zur Pr&uuml;fung ist jetzt ausgeblendet.";
#elif L==3
	"The announcement of exam is now hidden.";
#elif L==4
	"La convocatoria de examen ahora est&aacute; oculta.";
#elif L==5
	"La convocation pour l'examen est maintenant cach&eacute;.";
#elif L==6
	"La convocatoria de examen ahora est&aacute; oculta.";	// Okoteve traduccin
#elif L==7
	"L'apello d'esame &egrave; ora nascosto.";
#elif L==8
	"Ogloszenie egzamin jest teraz ukryte.";
#elif L==9
	"A chamada para exame &eacute; agora oculta.";
#endif

const char *Txt_The_announcement_of_exam_is_now_visible =
#if   L==1
	"La convocat&ograve;ria d'examen ara est&agrave; visible.";
#elif L==2
	"Der Aufruf zur Pr&uuml;fung ist jetzt sichtbar.";
#elif L==3
	"The announcement of exam is now visible.";
#elif L==4
	"La convocatoria de examen ahora est&aacute; visible.";
#elif L==5
	"La convocation pour l'examen est maintenant visible.";
#elif L==6
	"La convocatoria de examen ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7
	"L'apello d'esame &egrave; ora visibile.";
#elif L==8
	"Ogloszenie egzamin jest teraz widoczny.";
#elif L==9
	"A chamada para exame &eacute; agora vis&iacute;vel.";
#endif

const char *Txt_The_answers_of_the_question_with_code_X_will_appear_shuffled =	// Warning: it is very important to include %ld in the following sentences
#if   L==1
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n barajadas.";	// Necessita traduccio
#elif L==2
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// Need bersetzung
#elif L==3
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";
#elif L==4
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n barajadas.";
#elif L==5
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// Besoin de traduction
#elif L==6
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n barajadas.";	// Okoteve traduccin
#elif L==7
	"Le risposte alla domande con il codice <strong>%ld</strong>"
	" appariranno mischiate.";
#elif L==8
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The answers of the question with code <strong>%ld</strong>"
	" will appear shuffled.";		// Necessita de traduo
#endif

const char *Txt_The_answers_of_the_question_with_code_X_will_appear_without_shuffling =	// Warning: it is very important to include %ld in the following sentences
#if   L==1
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n sin barajar.";	// Necessita traduccio
#elif L==2
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// Need bersetzung
#elif L==3
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";
#elif L==4
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n sin barajar.";
#elif L==5
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// Besoin de traduction
#elif L==6
	"Las respuestas de la pregunta con c&oacute;digo <strong>%ld</strong>"
	" aparecer&aacute;n sin barajar.";	// Okoteve traduccin
#elif L==7
	"Le risposte della domanda con il codice <strong>%ld</strong>"
	" appariranno senza essere mischiate.";
#elif L==8
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The answers of the question with code <strong>%ld</strong>"
	" will appear without shuffling.";	// Necessita de traduo
#endif

const char *Txt_The_assignment_has_been_modified =
#if   L==1
	"La actividad ha sido modificada.";	// Necessita traduccio
#elif L==2
	"Die Aufgabe wurde ge&auml;ndert.";
#elif L==3
	"The assignment has been modified.";
#elif L==4
	"La actividad ha sido modificada.";
#elif L==5
	"L'activit&eacute; a &eacute;t&eacute; modifi&eacute;e.";
#elif L==6
	"La actividad ha sido modificada.";	// Okoteve traduccin
#elif L==7
	"L'attivit&agrave; &egrave; stata modificata.";
#elif L==8
	"The assignment has been modified.";	// Potrzebujesz tlumaczenie
#elif L==9
	"A atividade foi modificada.";
#endif

const char *Txt_The_banner_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El banner <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==2
	"Der Werbebanner <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3
	"The banner <strong>%s</strong> already exists.";
#elif L==4
	"El banner <strong>%s</strong> ya existe.";
#elif L==5
	"The banner <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"El banner <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7
	"Il banner <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The banner <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The banner <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_banner_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El banner <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The Werbebanner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El banner <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"El banner <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il banner <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The banner <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_banner_X_is_now_hidden =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El banner <strong>%s</strong> ara est&agrave; ocult.";
#elif L==2
	"Der Werbebanner <strong>%s</strong> ist jetzt ausgeblendet.";	// Need bersetzung
#elif L==3
	"The banner <strong>%s</strong> is now hidden.";
#elif L==4
	"El banner <strong>%s</strong> ahora est&aacute; oculto.";
#elif L==5
	"Le banner <strong>%s</strong> est maintenant cach&eacute;.";
#elif L==6
	"El banner <strong>%s</strong> ahora est&aacute; oculto.";	// Okoteve traduccin
#elif L==7
	"Il banner <strong>%s</strong> &egrave; ora nascosto.";
#elif L==8
	"The banner <strong>%s</strong> jest ukryte.";
#elif L==9
	"O banner <strong>%s</strong> &eacute; agora oculto.";
#endif

const char *Txt_The_banner_X_is_now_visible =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El banner <strong>%s</strong> ara est&agrave; visible.";
#elif L==2
	"Der Werbebanner <strong>%s</strong> ist jetzt sichtbar.";	// Need bersetzung
#elif L==3
	"The banner <strong>%s</strong> is now visible.";
#elif L==4
	"El banner <strong>%s</strong> ahora est&aacute; visible.";
#elif L==5
	"Le banner <strong>%s</strong> est maintenant visible.";
#elif L==6
	"El banner <strong>%s</strong> ahora est&aacute; visible.";	// Okoteve traduccin
#elif L==7
	"Il banner <strong>%s</strong> &egrave; ora visible.";
#elif L==8
	"The banner <strong>%s</strong> jest teraz widoczny.";
#elif L==9
	"O banner <strong>%s</strong> &eacute; agora visible.";
#endif

const char *Txt_The_centre_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El centro <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==2
	"Es wurde bereits ein Lehrinstitut mit dem Namen <strong>%s</strong> eingegeben.";
#elif L==3
	"The centre <strong>%s</strong> already exists.";
#elif L==4
	"El centro <strong>%s</strong> ya existe.";
#elif L==5
	"The centre <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"El centro <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7
	"Il centro <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The centre <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The centre <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_centre_X_has_been_moved_to_the_institution_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El centro <strong>%s</strong> se ha movido"
	" a la instituci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The centre <strong>%s</strong> has been moved"
	" to the institution <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"The centre <strong>%s</strong> has been moved"
	" to the institution <strong>%s</strong>.";
#elif L==4
	"El centro <strong>%s</strong> se ha movido"
	" a la instituci&oacute;n <strong>%s</strong>.";
#elif L==5
	"The centre <strong>%s</strong> has been moved"
	" to the institution <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"El centro <strong>%s</strong> se ha movido"
	" a la instituci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il centre <strong>%s</strong> &egrave; stato spostato"
	" alla istituzione <strong>%s</strong>.";
#elif L==8
	"The centre <strong>%s</strong> has been moved"
	" to the institution <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The centre <strong>%s</strong> has been moved"
	" to the institution <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_centre_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El centro <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El centro <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"El centro <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il centro <strong>%s</strong> &egrave; stato rinominato <strong>%s</strong>.";
#elif L==8
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The centre <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_comment_no_longer_exists =
#if   L==1
	"El comentari ja no existeix.";
#elif L==2
	"Der Kommentar nicht mehr existiert.";
#elif L==3
	"The comment no longer exists.";
#elif L==4
	"El comentario ya no existe.";
#elif L==5
	"Le commentaire a disparu.";
#elif L==6
	"El comentario ya no existe.";	// Okoteve traduccin
#elif L==7
	"Il commento non esiste pi&ugrave;.";
#elif L==8
	"Komentarz ju&zdot; nie istnieje.";
#elif L==9
	"O coment&aacute;rio n&atilde;o existe mais.";
#endif

const char *Txt_The_contents_of_the_folder_are_too_big =
#if   L==1
	"El contingut de la carpeta &eacute;s massa gran.";
#elif L==2
	"Der Inhalt der Ordner sind zu gro&szlig;.";
#elif L==3
	"The contents of the folder are too big.";
#elif L==4
	"El contenido de la carpeta es demasiado grande.";
#elif L==5
	"Le contenu du dossier est trop gros.";
#elif L==6
	"El contenido de la carpeta es demasiado grande.";	// Okoteve traduccin
#elif L==7
	"Il contenuto della cartella &egrave; troppo grande.";
#elif L==8
	"Zawarto&sacute;&cacute; folderu s&aogon; zbyt du&zdot;e.";
#elif L==9
	"O conte&uacute;do do diret&oacute;rio &eacute; muito grande.";
#endif

const char *Txt_The_copy_has_been_successful =
#if   L==1
	"La copia se ha realizado con &eacute;xito.";	// Necessita traduccio
#elif L==2
	"Die Kopie wurde erfolgreich.";
#elif L==3
	"The copy has been successful.";
#elif L==4
	"La copia se ha realizado con &eacute;xito.";
#elif L==5
	"La copie a &eacute;t&eacute; un succ&egrave;s.";
#elif L==6
	"La copia se ha realizado con &eacute;xito.";	// Okoteve traduccin
#elif L==7
	"La copia &egrave; avvenuta con successo.";
#elif L==8
	"The copy has been successful.";		// Potrzebujesz tlumaczenie
#elif L==9
	"A c&oacute;pia foi feita com sucesso.";
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Necessita traduccio
#elif L==2
	"Der Kopiervorgang wurde aus Platzgr&uuml;nden abgebrochen, als versucht wurde,"
	" die Datei <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Besoin de traduction
#elif L==6
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; esso superava la quota di gestione del disco.";
#elif L==8
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La copia se ha detenido al intentar pegar la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Necessita traduccio
#elif L==2
	"Der Kopiervorgang wurde aus Platzgr&uuml;nden abgebrochen, als versucht wurde,"
	" das Verzeichnis <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3
	"The copy has stopped when trying to paste the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4
	"La copia se ha detenido al intentar pegar la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5
	"The copy has stopped when trying to paste the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Besoin de traduction
#elif L==6
	"La copia se ha detenido al intentar pegar la carpeta <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7
	"La copia &egrave; terminata quando si &egrave; provato a incollare la cartella<strong>%s</strong>"
	" perch&egrave; essa superava la quota di gestione del disco.";
#elif L==8
	"The copy has stopped when trying to paste the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The copy has stopped when trying to paste the folder <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_link_X_because_it_would_exceed_the_disk_quota =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Necessita traduccio
#elif L==2
	"Der Kopiervorgang wurde aus Platzgr&uuml;nden abgebrochen, als versucht wurde,"
	" das Link <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the disk quota.";
#elif L==4
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";
#elif L==5
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Besoin de traduction
#elif L==6
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a la cuota de disco.";	// Okoteve traduccin
#elif L==7
	"La copia &egrave; terminata quando si &egrave; provato a incollare il link<strong>%s</strong>"
	" perch&egrave; essa superava la quota di gestione del disco.";
#elif L==8
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the disk quota.";		// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_it_would_exceed_the_maximum_allowed_number_of_levels =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Necessita traduccio
#elif L==2
	"Zu viele Ebenen. Der Kopiervorgang wurde abgebrochen, als versucht wurde,"
	" die Datei <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";
#elif L==4
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";
#elif L==5
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Besoin de traduction
#elif L==6
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Okoteve traduccin
#elif L==7
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; esso superava il massimo numero di livelli permesso.";
#elif L==8
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_folder_X_because_it_would_exceed_the_maximum_allowed_number_of_levels =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Necessita traduccio
#elif L==2
	"Zu viele Ebenen. Der Kopiervorgang wurde abgebrochen, als versucht wurde,"
	" die Datei <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";
#elif L==4
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";
#elif L==5
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Besoin de traduction
#elif L==6
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Okoteve traduccin
#elif L==7
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; superava il massimo numero di livelli permessi.";
#elif L==8
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_link_X_because_it_would_exceed_the_maximum_allowed_number_of_levels =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Necessita traduccio
#elif L==2
	"Zu viele Ebenen. Der Kopiervorgang wurde abgebrochen, als versucht wurde,"
	" die Link <strong>%s</strong> einzuf&uuml;gen.";
#elif L==3
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";
#elif L==4
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";
#elif L==5
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Besoin de traduction
#elif L==6
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque se exceder&iacute;a el n&uacute;mero m&aacute;ximo de niveles permitido.";	// Okoteve traduccin
#elif L==7
	"La copia &egrave; terminata quando si &egrave; provato a incollare il link <strong>%s</strong>"
	" perch&egrave; superava il massimo numero di livelli permessi.";
#elif L==8
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because it would exceed the maximum allowed number of levels.";			// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_there_is_already_an_object_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";	// Necessita traduccio
#elif L==2
	"Der Kopiervorgang wurde abgebrochen, als versucht wurde, die Datei <strong>%s</strong>"
	" einzuf&uuml;gen. Der Objektname wird bereits verwendet.";
#elif L==3
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because there is already an object with that name.";
#elif L==4
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";
#elif L==5
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because there is already an object with that name.";	// Besoin de traduction
#elif L==6
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque ya existe otro objecto con ese nombre.";	// Okoteve traduccin
#elif L==7
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; c'era gi&agrave; un oggetto con quel nome.";
#elif L==8
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because there is already an object with that name.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because there is already an object with that name.";	// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_link_X_because_there_is_already_an_object_with_that_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";	// Necessita traduccio
#elif L==2
	"Der Kopiervorgang wurde abgebrochen, als versucht wurde, die Link <strong>%s</strong>"
	" einzuf&uuml;gen. Der Objektname wird bereits verwendet.";
#elif L==3
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because there is already an object with that name.";
#elif L==4
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque ya existe otro objeto con ese nombre.";
#elif L==5
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because there is already an object with that name.";	// Besoin de traduction
#elif L==6
	"La copia se ha detenido al intentar pegar el enlace <strong>%s</strong>"
	" porque ya existe otro objecto con ese nombre.";	// Okoteve traduccin
#elif L==7
	"La copia &egrave; terminata quando si &egrave; provato a incollare il link <strong>%s</strong>"
	" perch&egrave; c'era gi&agrave; un oggetto con quel nome.";
#elif L==8
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because there is already an object with that name.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The copy has stopped when trying to paste the link <strong>%s</strong>"
	" because there is already an object with that name.";	// Necessita de traduo
#endif

const char *Txt_The_copy_has_stopped_when_trying_to_paste_the_file_X_because_you_can_not_paste_a_file_here_of_a_type_other_than_HTML =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";	// Necessita traduccio
#elif L==2
	"Der Kopiervorgang wurde abgebrochen, als versucht wurde, die Datei <strong>%s</strong>"
	" einzuf&uuml;gen. Es d&uuml;rfen nur HTML-Daten eingef&uumlgt werden.";
#elif L==3
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";
#elif L==4
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";
#elif L==5
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";		// Besoin de traduction
#elif L==6
	"La copia se ha detenido al intentar pegar el archivo <strong>%s</strong>"
	" porque no se puede pegar aqu&iacute; un archivo de un tipo distinto a HTML.";	// Okoteve traduccin
#elif L==7
	"La copia &egrave; terminata quando si &egrave; provato a incollare il file <strong>%s</strong>"
	" perch&egrave; non puoi incollare qui un file di un tipo diverso da HTML.";
#elif L==8
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The copy has stopped when trying to paste the file <strong>%s</strong>"
	" because you can not paste a file here of a type other than HTML.";		// Necessita de traduo
#endif

const char *Txt_The_country_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El pa&iacute;s <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2
	"Die Land <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3
	"The country <strong>%s</strong> already exists.";
#elif L==4
	"El pa&iacute;s <strong>%s</strong> ya existe.";
#elif L==5
	"The country <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"El pa&iacute;s <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7
	"Il paese <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The country <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The country <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_country_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El pa&iacute;s <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El pa&iacute;s <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"El pa&iacute;s <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il paese <strong>%s</strong> &egrave; stato rimosso come <strong>%s</strong>.";
#elif L==8
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The country <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_country_of_the_institution_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==3
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==4
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";
#elif L==5
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==6
	"El pa&iacute;s de la instituci&oacute;n <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il paese dell'istituzione <strong>%s</strong>"
	" &egrave; cambiato in <strong>%s</strong>.";
#elif L==8
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The country of the institution <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#endif

const char *Txt_The_course_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La asignatura <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==2
	"Die Kurs <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3
	"The course <strong>%s</strong> already exists.";
#elif L==4
	"La asignatura <strong>%s</strong> ya existe.";
#elif L==5
	"The course <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"La asignatura <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7
	"Il corso <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The course <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The course <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_course_X_already_exists_in_year_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"La asignatura <strong>%s</strong> ya existe en el a&ntilde;o/semestre %s.";	// Necessita traduccio
#elif L==2
	"Die Kurs <strong>%s</strong> wurde bereits in Studienjahr/Semester %s eingegeben.";
#elif L==3
	"The course <strong>%s</strong> already exists in year/semester %s.";
#elif L==4
	"La asignatura <strong>%s</strong> ya existe en el a&ntilde;o/semestre %s.";
#elif L==5
	"The course <strong>%s</strong> already exists in year/semester %s.";	// Besoin de traduction
#elif L==6
	"La asignatura <strong>%s</strong> ya existe en el a&ntilde;o/semestre %s.";	// Okoteve traduccin
#elif L==7
	"Il corso <strong>%s</strong> esiste gi&agrave; nell'anno/semester %s.";
#elif L==8
	"The course <strong>%s</strong> already exists in year/semester %s.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The course <strong>%s</strong> already exists in year/semester %s.";	// Necessita de traduo
#endif

const char *Txt_The_course_X_has_been_moved_to_the_degree_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"La asignatura <strong>%s</strong> se ha movido"
	" a la titulaci&oacute;n <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";
#elif L==4
	"La asignatura <strong>%s</strong> se ha movido"
	" a la titulaci&oacute;n <strong>%s</strong>.";
#elif L==5
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"La asignatura <strong>%s</strong> se ha movido"
	" a la titulaci&oacute;n <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il corso <strong>%s</strong> &egrave; stato spostato"
	" dalla laurea <strong>%s</strong>.";
#elif L==8
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The course <strong>%s</strong> has been moved"
	" to the degree <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_date_of_the_holiday_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"La fecha de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4
	"La fecha de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"La fecha de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"La data delle festivit&agrave; <strong>%s</strong> &egrave; cambiata a <strong>%s</strong>.";
#elif L==8
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The date of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_date_range_must_be_less_than_or_equal_to_X_days =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"El rango de fechas debe ser menor o igual a %u d&iacute;as.";	// Necessita traduccio
#elif L==2
	"The date range must be less than or equal to %u days.";	// Need bersetzung
#elif L==3
	"The date range must be less than or equal to %u days.";
#elif L==4
	"El rango de fechas debe ser menor o igual a %u d&iacute;as.";
#elif L==5
	"The date range must be less than or equal to %u days.";	// Besoin de traduction
#elif L==6
	"El rango de fechas debe ser menor o igual a %u d&iacute;as.";	// Okoteve traduccin
#elif L==7
	"Il range della data deve essere minore o uguale a %u giorni.";
#elif L==8
	"The date range must be less than or equal to %u days.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The date range must be less than or equal to %u days.";	// Necessita de traduo
#endif

const char *Txt_The_date_time_of_opening_of_groups_has_changed =
#if   L==1
	"La data / hora d'obertura de grups s'ha modificat.";
#elif L==2
	"The date / time of opening of groups has changed.";			// Need bersetzung
#elif L==3
	"The date / time of opening of groups has changed.";
#elif L==4
	"La fecha / hora de apertura ha cambiado.";
#elif L==5
	"The date / time of opening of groups has changed.";			// Besoin de traduction
#elif L==6
	"La fecha / hora de apertura ha cambiado.";	// Okoteve traduccin
#elif L==7
	"La data / ora di apertura dei gruppi &egrave; cambiata.";
#elif L==8
	"The date / time of opening of groups has changed.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The date / time of opening of groups has changed.";			// Necessita de traduo
#endif

const char *Txt_The_degree_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La titulaci&oacute;n <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2
	"Der Studiengang <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3
	"The degree <strong>%s</strong> already exists.";
#elif L==4
	"La titulaci&oacute;n <strong>%s</strong> ya existe.";
#elif L==5
	"The degree <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"La titulaci&oacute;n <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7
	"La laurea <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The degree <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The degree <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_degree_X_has_been_moved_to_the_centre_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"La titulaci&oacute;n <strong>%s</strong> se ha movido"
	" al centro <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The degree <strong>%s</strong> has been moved"
	" to the centre <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"The degree <strong>%s</strong> has been moved"
	" to the centre <strong>%s</strong>.";
#elif L==4
	"La titulaci&oacute;n <strong>%s</strong> se ha movido"
	" al centro <strong>%s</strong>.";
#elif L==5
	"The degree <strong>%s</strong> has been moved"
	" to the centre <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"La titulaci&oacute;n <strong>%s</strong> se ha movido"
	" al centro <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7
	"La laurea <strong>%s</strong> &egrave; stata spostata"
	" al centro <strong>%s</strong>.";
#elif L==8
	"The degree <strong>%s</strong> has been moved"
	" to the centre <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The degree <strong>%s</strong> has been moved"
	" to the centre <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_department_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El departamento <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2
	"Die Abteilung <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3
	"The department <strong>%s</strong> already exists.";
#elif L==4
	"El departamento <strong>%s</strong> ya existe.";
#elif L==5
	"The department <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"El departamento <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7
	"IL dipartimento <strong>%s</strong> esiste gi&agrave;.";
#elif L==8
	"The department <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The department <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_department_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El departamento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El departamento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"El departamento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il dipartimento <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The department <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_email_X_has_been_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El correo <strong>%s</strong> ha sido confirmado.";	// Necessita traduccio
#elif L==2
	"The email <strong>%s</strong> has been confirmed.";	// Need bersetzung
#elif L==3
	"The email <strong>%s</strong> has been confirmed.";
#elif L==4
	"El correo <strong>%s</strong> ha sido confirmado.";
#elif L==5
	"The email <strong>%s</strong> has been confirmed.";	// Besoin de traduction
#elif L==6
	"El correo <strong>%s</strong> ha sido confirmado.";	// Okoteve traduccin
#elif L==7
	"The email <strong>%s</strong> has been confirmed.";	// Bisogno di traduzione
#elif L==8
	"The email <strong>%s</strong> has been confirmed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The email <strong>%s</strong> has been confirmed.";	// Necessita de traduo
#endif

const char *Txt_The_email_address_entered_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La direcci&oacute;n de correo introducida"
	" <strong>%s</strong> no es v&aacute;lida.";	// Necessita traduccio
#elif L==2
	"The email address entered"
	" <strong>%s</strong> is not valid.";		// Need bersetzung
#elif L==3
	"The email address entered"
	" <strong>%s</strong> is not valid.";
#elif L==4
	"La direcci&oacute;n de correo introducida"
	" <strong>%s</strong> no es v&aacute;lida.";
#elif L==5
	"The email address entered"
	" <strong>%s</strong> is not valid.";		// Besoin de traduction
#elif L==6
	"La direcci&oacute;n de correo introducida"
	" <strong>%s</strong> no es v&aacute;lida.";	// Okoteve traduccin
#elif L==7
	"Il email immesso"
	" <strong>%s</strong> non &egrave; valido.";
#elif L==8
	"The email address entered"
	" <strong>%s</strong> is not valid.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The email address entered"
	" <strong>%s</strong> is not valid.";		// Necessita de traduo
#endif

const char *Txt_The_email_address_X_had_been_registered_by_another_user =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" ya ha sido registrada por otro usuario.";	// Necessita traduccio
#elif L==2
	"The email address <strong>%s</strong>"
	" had been registered by another user.";	// Need bersetzung
#elif L==3
	"The email address <strong>%s</strong>"
	" had been registered by another user.";
#elif L==4
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" ya ha sido registrada por otro usuario.";
#elif L==5
	"The email address <strong>%s</strong>"
	" had been enrolled by another user.";		// Besoin de traduction
#elif L==6
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" ya ha sido registrada por otro usuario.";	// Okoteve traduccin
#elif L==7
	"Il email <strong>%s</strong>"
	" &egrave; stato registrato da un altro utente.";
#elif L==8
	"The email address <strong>%s</strong>"
	" had been registered by another user.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The email address <strong>%s</strong>"
	" had been registered by another user.";	// Necessita de traduo
#endif

const char *Txt_The_email_address_X_has_been_registered_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El correo <strong>%s</strong>"
	" se ha registrado correctamente.";		// Necessita traduccio
#elif L==2
	"The email address <strong>%s</strong>"
	" has been registered successfully.";		// Need bersetzung
#elif L==3
	"The email address <strong>%s</strong>"
	" has been registered successfully.";
#elif L==4
	"El correo <strong>%s</strong>"
	" se ha registrado correctamente.";
#elif L==5
	"The email address <strong>%s</strong>"
	" has been registered successfully.";		// Besoin de traduction
#elif L==6
	"El correo <strong>%s</strong>"
	" se ha registrado correctamente.";		// Okoteve traduccin
#elif L==7
	"Il email <strong>%s</strong>"
	" &egrave; stato registrato con successo.";
#elif L==8
	"The email address <strong>%s</strong>"
	" has been registered successfully.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The email address <strong>%s</strong>"
	" has been registered successfully.";		// Necessita de traduo
#endif

const char *Txt_The_email_address_has_not_been_confirmed =
#if   L==1
	"La direcci&oacute;n de correo"
	" no ha podido confirmarse.";			// Necessita traduccio
#elif L==2
	"The email address has not been confirmed.";	// Need bersetzung
#elif L==3
	"The email address has not been confirmed.";
#elif L==4
	"La direcci&oacute;n de correo"
	" no ha podido confirmarse.";
#elif L==5
	"The email address has not been confirmed.";	// Besoin de traduction
#elif L==6
	"La direcci&oacute;n de correo"
	" no ha podido confirmarse.";			// Okoteve traduccin
#elif L==7
	"The email address has not been confirmed.";	// Bisogno di traduzione
#elif L==8
	"The email address has not been confirmed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The email address has not been confirmed.";	// Necessita de traduo
#endif

const char *Txt_The_email_address_X_matches_one_previously_registered =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" coincide con una registrada previamente.";	// Necessita traduccio
#elif L==2
	"The email address <strong>%s</strong>"
	" matches one previously registered.";		// Need bersetzung
#elif L==3
	"The email address <strong>%s</strong>"
	" matches one previously registered.";
#elif L==4
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" coincide con una registrada previamente.";
#elif L==5
	"The email address <strong>%s</strong>"
	" matches one previously registered.";		// Besoin de traduction
#elif L==6
	"La direcci&oacute;n de correo <strong>%s</strong>"
	" coincide con una registrada previamente.";	// Okoteve traduccin
#elif L==7
	"Il email <strong>%s</strong>"
	" coincide con uno precedentemente registrato.";
#elif L==8
	"The email address <strong>%s</strong>"
	" matches one previously registered.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The email address <strong>%s</strong>"
	" matches one previously registered.";		// Necessita de traduo
#endif

const char *Txt_The_email_domain_X_already_exists =
#if   L==1
	"El dominio de correo ya existe.";		// Necessita traduccio
#elif L==2
	"Die E-Mail-Dom&auml;ne wurde bereits eingegeben.";
#elif L==3
	"The email domain already exists.";
#elif L==4
	"El dominio de correo ya existe.";
#elif L==5
	"The email domain already exists.";		// Besoin de traduction
#elif L==6
	"El dominio de correo ya existe.";		// Okoteve traduccin
#elif L==7
	"Il campo mail gi&agrave; esiste.";
#elif L==8
	"The email domain already exists.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The email domain already exists.";		// Necessita de traduo
#endif

const char *Txt_The_email_domain_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El dominio de correo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The email domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Need bersetzung
#elif L==3
	"The email domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El dominio de correo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The email domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Besoin de traduction
#elif L==6
	"El dominio de correo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il campo mail <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8
	"The email domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The email domain <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_email_domain_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El dominio de correo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The email domain <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3
	"The email domain <strong>%s</strong> has not changed.";
#elif L==4
	"El dominio de correo <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The email domain <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6
	"El dominio de correo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il campo mail <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The email domain <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The email domain <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_enrollment_of_students_into_groups_of_type_X_is_now_mandatory =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es obligatoria.";	// Necessita traduccio
#elif L==2
	"The enrollment of students into groups of type <strong>%s</strong> is now mandatory.";			// Need bersetzung
#elif L==3
	"The enrollment of students into groups of type <strong>%s</strong> is now mandatory.";
#elif L==4
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es obligatoria.";
#elif L==5
	"The enrollment of students into groups of type <strong>%s</strong> is now mandatory.";			// Besoin de traduction
#elif L==6
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es obligatoria.";	// Okoteve traduccin
#elif L==7
	"La registrazione di studenti nei gruppi di tipo <strong>%s</strong> &egrave; ora obbligatoria.";
#elif L==8
	"The enrollment of students into groups of type <strong>%s</strong> is now mandatory.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The enrollment of students into groups of type <strong>%s</strong> is now mandatory.";			// Necessita de traduo
#endif

const char *Txt_The_enrollment_of_students_into_groups_of_type_X_is_now_voluntary =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es voluntaria.";	// Necessita traduccio
#elif L==2
	"The enrollment of students into groups of type <strong>%s</strong> is now voluntary.";			// Need bersetzung
#elif L==3
	"The enrollment of students into groups of type <strong>%s</strong> is now voluntary.";
#elif L==4
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es voluntaria.";
#elif L==5
	"The enrollment of students into groups of type <strong>%s</strong> is now voluntary.";			// Besoin de traduction
#elif L==6
	"La adscripci&oacute;n de estudiantes a grupos del tipo <strong>%s</strong> ahora es voluntaria.";	// Okoteve traduccin
#elif L==7
	"La registrazione di studenti nei gruppi di tipo <strong>%s</strong> &egrave; ora volontaria.";
#elif L==8
	"The enrollment of students into groups of type <strong>%s</strong> is now voluntary.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The enrollment of students into groups of type <strong>%s</strong> is now voluntary.";			// Necessita de traduo
#endif

const char *Txt_The_event_has_been_modified =
#if   L==1
	"El evento ha sido modificado.";	// Necessita traduccio
#elif L==2
	"Der Ereignis wurde ge&auml;ndert.";
#elif L==3
	"The event has been modified.";
#elif L==4
	"El evento ha sido modificado.";
#elif L==5
	"L'&eacute;v&eacute;nement a &eacute;t&eacute; modifi&eacute;.";
#elif L==6
	"El evento ha sido modificado.";	// Okoteve traduccin
#elif L==7
	"L'evento &egrave; stata modificato.";
#elif L==8
	"The event has been modified.";		// Potrzebujesz tlumaczenie
#elif L==9
	"O evento foi modificado.";
#endif

const char *Txt_The_file_X_has_been_placed_inside_the_folder_Y =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Necessita traduccio
#elif L==2
	"Die Datei <strong>%s</strong> wurde"
	" in das Verzeichnis <strong>%s</strong> gesteckt.";
#elif L==3
	"The file <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";
#elif L==4
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";
#elif L==5
	"Le fichier <strong>%s</strong> a &eacute;t&eacute; plac&eacute;"
	" dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6
	"El archivo <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7
	"Il file <strong>%s</strong> &egrave; stato messo"
	" all'interno della cartella <strong>%s</strong>.";
#elif L==8
	"The file <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"O arquivo <strong>%s</strong> foi colocado"
	" dentro do diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_The_file_is_not_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El archivo no es <em>%s</em>.";	// Necessita traduccio
#elif L==2
	"The file is not <em>%s</em>.";		// Need bersetzung
#elif L==3
	"The file is not <em>%s</em>.";
#elif L==4
	"El archivo no es <em>%s</em>.";
#elif L==5
	"The file is not <em>%s</em>.";		// Besoin de traduction
#elif L==6
	"El archivo no es <em>%s</em>.";	// Okoteve traduccin
#elif L==7
	"IL file non &egrave; <em>%s</em>.";
#elif L==8
	"The file is not <em>%s</em>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The file is not <em>%s</em>.";		// Necessita de traduo
#endif

const char *Txt_The_file_of_folder_no_longer_exists_or_is_now_hidden =
#if   L==1
	"L'arxiu o carpeta ja no existeix o est&agrave; ara ocult.";
#elif L==2
	"Die Datei oder Ordner existiert nicht mehr oder ist jetzt versteckt.";
#elif L==3
	"The file or folder no longer exists or is now hidden.";
#elif L==4
	"El archivo o carpeta ya no existe o est&aacute; ahora oculto.";
#elif L==5
	"Le fichier ou r&eacute;pertoire n'existe plus ou est maintenant cach&eacute;.";
#elif L==6
	"El archivo o carpeta ya no existe o est&aacute; ahora oculto.";	// Okoteve traduccin
#elif L==7
	"Il file o cartella non esiste pi&ugrave; o &egrave; nascosta.";
#elif L==8
	"Plik lub folder ju&zdot; nie istnieje lub jest ukryte.";
#elif L==9
	"O arquivo ou diret&oacute;rio n&atilde;o existe mais ou est&aacute; escondido.";
#endif

const char *Txt_The_file_type_is_X_and_should_be_HTML_or_ZIP =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El tipo de archivo es %s y debe ser <em>HTML</em> o <em>ZIP</em>.";	// Necessita traduccio
#elif L==2
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// Need bersetzung
#elif L==3
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";
#elif L==4
	"El tipo de archivo es %s y debe ser <em>HTML</em> o <em>ZIP</em>.";
#elif L==5
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// Besoin de traduction
#elif L==6
	"El tipo de archivo es %s y debe ser <em>HTML</em> o <em>ZIP</em>.";	// Okoteve traduccin
#elif L==7
	"Il tipo di file &egrave; %s e dovrebbe essere <em>HTML</em> o <em>ZIP</em>.";
#elif L==8
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The file type is %s and should be <em>HTML</em> or <em>ZIP</em>.";	// Necessita de traduo
#endif

const char *Txt_The_file_type_should_be_HTML_or_ZIP =
#if   L==1
	"El tipo de archivo debe ser <em>HTML</em> o <em>ZIP</em>.";	// Necessita traduccio
#elif L==2
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// Need bersetzung
#elif L==3
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";
#elif L==4
	"El tipo de archivo debe ser <em>HTML</em> o <em>ZIP</em>.";
#elif L==5
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// Besoin de traduction
#elif L==6
	"El tipo de archivo debe ser <em>HTML</em> o <em>ZIP</em>.";	// Okoteve traduccin
#elif L==7
	"Il tipo di file dovrebbe essere <em>HTML</em> o <em>ZIP</em>.";
#elif L==8
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The file type should be <em>HTML</em> or <em>ZIP</em>.";	// Necessita de traduo
#endif

const char *Txt_The_folder_X_has_been_created_inside_the_folder_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Das Verzeichnis <strong>%s</strong> wurde im Verzeichnis <strong>%s</strong> erstellt.";
#elif L==3
	"The folder <strong>%s</strong> has been created inside the folder <strong>%s</strong>.";
#elif L==4
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";
#elif L==5
	"Le r&eacute;pertoire <strong>%s</strong> a &eacute;t&eacute; cr&eacute;&eacute; dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6
	"Se ha creado la carpeta <strong>%s</strong> dentro de la carpeta <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"La cartella <strong>%s</strong> &egrave; stata creata all'interno della cartella <strong>%s</strong>.";
#elif L==8
	"The folder <strong>%s</strong> has been created inside the folder <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"O diret&oacute;rio <strong>%s</strong> foi criado dentro do diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_The_folder_is_empty =
#if   L==1
	"La carpeta est&agrave; buida.";
#elif L==2
	"Der Verzeichnis is leer.";
#elif L==3
	"The folder is empty.";
#elif L==4
	"La carpeta est&aacute; vac&iacute;a.";
#elif L==5
	"Le r&eacute;pertoire est vide.";
#elif L==6
	"La carpeta est&aacute; vac&iacute;a.";	// Okoteve traduccin
#elif L==7
	"La cartella &egrave; vuota.";
#elif L==8
	"Folder jest pusty.";
#elif L==9
	"O diret&oacute;rio est&aacute; vazio.";
#endif

const char *Txt_The_folder_name_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"Se ha cambiado el nombre de la carpeta <strong>%s</strong> por <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Das Verzeichnis <strong>%s</strong> wurde umbenannt in <strong>%s</strong>.";
#elif L==3
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4
	"Se ha cambiado el nombre de la carpeta <strong>%s</strong> por <strong>%s</strong>.";
#elif L==5
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"Se ha cambiado el nombre de la carpeta <strong>%s</strong> por <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il nome della cartella <strong>%s</strong> &egrave; cambiato in <strong>%s</strong>.";
#elif L==8
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The folder name <strong>%s</strong> has changed to <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_folder_name_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"Der Name des Verzeichnisses <strong>%s</strong> wurde nicht ge&auml;ndert.";
#elif L==3
	"The folder name <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The folder name <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome della cartella <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The folder name <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The folder name <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_folder_name_X_has_not_changed_because_there_is_already_a_folder_or_a_file_with_the_name_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Der Name des Verzeichnisses <strong>%s</strong> wurde nicht ge&auml;ndert,"
	" weil bereits ein Verzeichnis oder eine Datei mit Namen <strong>%s</strong> besteht.";
#elif L==3
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";
#elif L==4
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";
#elif L==5
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"El nombre de la carpeta <strong>%s</strong> no ha cambiado"
	" porque ya existe una carpeta o un archivo con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il nome della cartella <strong>%s</strong> non &egrave; cambiato"
	" perch&egrave; c'&egrave; gi&agrave; una cartella o un file con il nome <strong>%s</strong>.";
#elif L==8
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The folder name <strong>%s</strong> has not changed"
	" because there is already a folder or a file with the name <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_group_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El grupo <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==2
	"Die Gruppe <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3
	"The group <strong>%s</strong> already exists.";
#elif L==4
	"El grupo <strong>%s</strong> ya existe.";
#elif L==5
	"The group <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"El grupo <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7
	"Il gruppo <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The group <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The group <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_group_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"El grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il gruppo <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_group_X_is_now_closed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El grupo <strong>%s</strong> est&aacute; ahora cerrado.";	// Necessita traduccio
#elif L==2
	"The group <strong>%s</strong> is now closed.";			// Need bersetzung
#elif L==3
	"The group <strong>%s</strong> is now closed.";
#elif L==4
	"El grupo <strong>%s</strong> est&aacute; ahora cerrado.";
#elif L==5
	"The group <strong>%s</strong> is now closed.";			// Besoin de traduction
#elif L==6
	"El grupo <strong>%s</strong> est&aacute; ahora cerrado.";	// Okoteve traduccin
#elif L==7
	"Il gruppo <strong>%s</strong> &egrave; ora chiuso.";
#elif L==8
	"The group <strong>%s</strong> is now closed.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The group <strong>%s</strong> is now closed.";			// Necessita de traduo
#endif

const char *Txt_The_group_X_is_now_open =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El grupo <strong>%s</strong> est&aacute; ahora abierto.";	// Necessita traduccio
#elif L==2
	"The group <strong>%s</strong> is now open.";			// Need bersetzung
#elif L==3
	"The group <strong>%s</strong> is now open.";
#elif L==4
	"El grupo <strong>%s</strong> est&aacute; ahora abierto.";
#elif L==5
	"The group <strong>%s</strong> is now open.";			// Besoin de traduction
#elif L==6
	"El grupo <strong>%s</strong> est&aacute; ahora abierto.";	// Okoteve traduccin
#elif L==7
	"Il gruppo <strong>%s</strong> &egrave; ora aperto.";
#elif L==8
	"The group <strong>%s</strong> is now open.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The group <strong>%s</strong> is now open.";			// Necessita de traduo
#endif

const char *Txt_The_group_X_now_has_no_limit_of_students =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El grupo <strong>%s</strong> no tiene ahora l&iacute;mite de estudiantes.";	// Necessita traduccio
#elif L==2
	"The group <strong>%s</strong> now has no limit of students.";
#elif L==3
	"The group <strong>%s</strong> now has no limit of students.";
#elif L==4
	"El grupo <strong>%s</strong> no tiene ahora l&iacute;mite de estudiantes.";
#elif L==5
	"The group <strong>%s</strong> now has no limit of students.";
#elif L==6
	"El grupo <strong>%s</strong> no tiene ahora l&iacute;mite de estudiantes.";	// Okoteve traduccin
#elif L==7
	"Il gruppo <strong>%s</strong> ora non ha limite di studenti.";
#elif L==8
	"The group <strong>%s</strong> now has no limit of students.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The group <strong>%s</strong> now has no limit of students.";
#endif

const char *Txt_The_groups_will_automatically_open =
#if   L==1
	"Els grups s'obriran autom&agrave;ticament.";
#elif L==2
	"The groups will automatically open.";			// Need bersetzung
#elif L==3
	"The groups will automatically open.";
#elif L==4
	"Los grupos se abrir&aacute;n autom&aacute;ticamente.";
#elif L==5
	"The groups will automatically open.";			// Besoin de traduction
#elif L==6
	"Los grupos se abrir&aacute;n autom&aacute;ticamente.";	// Okoteve traduccin
#elif L==7
	"I gruppi si aprir&agrave; automaticamente.";
#elif L==8
	"The groups will automatically open.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The groups will automatically open.";			// Necessita de traduo
#endif

const char *Txt_The_groups_will_not_automatically_open =
#if   L==1
	"Els grups no s'obriran autom&agrave;ticament.";
#elif L==2
	"The groups will not automatically open.";			// Need bersetzung
#elif L==3
	"The groups will not automatically open.";
#elif L==4
	"Los grupos no se abrir&aacute;n autom&aacute;ticamente.";
#elif L==5
	"The groups will not automatically open.";			// Besoin de traduction
#elif L==6
	"Los grupos no se abrir&aacute;n autom&aacute;ticamente.";	// Okoteve traduccin
#elif L==7
	"I gruppi non si aprir&agrave; automaticamente.";
#elif L==8
	"The groups will not automatically open.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The groups will not automatically open.";			// Necessita de traduo
#endif

const char *Txt_The_HTML_file_has_been_received_successfully =
#if   L==1
	"El archivo <em>HTML</em> se ha recibido correctamente.";	// Necessita traduccio
#elif L==2
	"The <em>HTML</em> file has been received successfully.";	// Need bersetzung
#elif L==3
	"The <em>HTML</em> file has been received successfully.";
#elif L==4
	"El archivo <em>HTML</em> se ha recibido correctamente.";
#elif L==5
	"The <em>HTML</em> file has been received successfully.";	// Besoin de traduction
#elif L==6
	"El archivo <em>HTML</em> se ha recibido correctamente.";	// Okoteve traduccin
#elif L==7
	"Il <em>HTML</em> file &egrave; stato ricevuto con successo.";
#elif L==8
	"The <em>HTML</em> file has been received successfully.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The <em>HTML</em> file has been received successfully.";	// Necessita de traduo
#endif

const char *Txt_The_ID_X_has_been_confirmed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" ha sido confirmado.";		// Necessita traduccio
#elif L==2
	"The ID <strong>%s</strong>"
	" has been confirmed.";		// Need bersetzung
#elif L==3
	"The ID <strong>%s</strong>"
	" has been confirmed.";
#elif L==4
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" ha sido confirmado.";
#elif L==5
	"The ID <strong>%s</strong>"
	" has been confirmed.";		// Besoin de traduction
#elif L==6
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" ha sido confirmado.";		// Okoteve traduccin
#elif L==7
	"Il ID <strong>%s</strong>"
	" &egrave; stato confermato.";
#elif L==8
	"The ID <strong>%s</strong>"
	" has been confirmed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The ID <strong>%s</strong>"
	" has been confirmed.";		// Necessita de traduo
#endif

const char *Txt_The_ID_X_has_been_registered_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" se ha registrado correctamente.";		// Necessita traduccio
#elif L==2
	"The ID <strong>%s</strong>"
	" has been registered successfully.";		// Need bersetzung
#elif L==3
	"The ID <strong>%s</strong>"
	" has been registered successfully.";
#elif L==4
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" se ha registrado correctamente.";
#elif L==5
	"The ID <strong>%s</strong>"
	" has been registered successfully.";		// Besoin de traduction
#elif L==6
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" se ha registrado correctamente.";		// Okoteve traduccin
#elif L==7
	"Il ID <strong>%s</strong>"
	" &egrave; stato registrato con successo.";
#elif L==8
	"The ID <strong>%s</strong>"
	" has been registered successfully.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The ID <strong>%s</strong>"
	" has been registered successfully.";		// Necessita de traduo
#endif

const char *Txt_The_ID_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido."
	" Solo puede contener n&uacute;meros (al menos uno) y letras.";		// Necessita traduccio
#elif L==2
	"The ID <strong>%s</strong> is not valid."
	" It can only contain numbers (at least one) and letters.";		// Need bersetzung
#elif L==3
	"The ID <strong>%s</strong> is not valid."
	" It can only contain numbers (at least one) and letters.";
#elif L==4
	"El ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido."
	" Solo puede contener n&uacute;meros (al menos uno) y letras.";
#elif L==5
	"The ID <strong>%s</strong> is not valid."
	" It can only contain numbers (at least one) and letters.";		// Besoin de traduction
#elif L==6
	"El ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido."
	" Solo puede contener n&uacute;meros (al menos uno) y letras.";		// Okoteve traduccin
#elif L==7
	"L'ID <strong>%s</strong> non &egrave; valido."
	" Pu&ograve; contenere solo numeri (almeno uno) e lettere.";
#elif L==8
	"The ID <strong>%s</strong> is not valid."
	" It can only contain numbers (at least one) and letters.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The ID <strong>%s</strong> is not valid."
	" It can only contain numbers (at least one) and letters.";		// Necessita de traduo
#endif

const char *Txt_The_ID_is_used_in_order_to_facilitate_ =
#if   L==1
	"L'ID s'utilitza per tal de facilitar"
	" la inscripci&oacute; en les assignatures"
	" i la consulta de qualificacions.";
#elif L==2
	"Die ID wird zu erleichtern,"
	" um die Einschreibung in Kurse"
	" und Beratungskompetenz eingesetzt.";
#elif L==3
	"The ID is used in order to facilitate"
	" enrollment in courses"
	" and consulting marks.";
#elif L==4
	"El ID se utiliza con el fin de facilitar"
	" la inscripci&oacute;n en las asignaturas"
	" y la consulta de calificaciones.";
#elif L==5
	"L'ID est utilis&eacute; afin de faciliter"
	" l'inscription &agrave; des mati&egrave;res"
	" et la consulte des notes.";
#elif L==6
	"El ID se utiliza con el fin de facilitar"
	" la inscripci&oacute;n en las asignaturas"
	" y la consulta de calificaciones.";	// Okoteve traduccin
#elif L==7
	"L'ID &egrave; utilizzato per agevolare"
	" l'iscrizione ai corsi"
	" e consultare dei risultati.";
#elif L==8
	"Identyfikator jest u&zdot;ywany w celu u&lstrok;atwienia"
	" rejestracji w kursach"
	" i skonsultuje znak&oacute;w.";
#elif L==9
	"O n&ordm; de identifica&ccedil;&atilde;o &eacute; usado para facilitar"
	" a inscri&ccedil;&atilde;o em disciplinas"
	" e consultas de notas.";
#endif

const char *Txt_The_ID_nickname_or_email_X_is_not_valid =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El @apodo, correo o ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";	// Necessita traduccio
#elif L==2
	"The @nick, email or ID <strong>%s</strong> is not valid.";				// Need bersetzung
#elif L==3
	"The @nick, email or ID <strong>%s</strong> is not valid.";
#elif L==4
	"El @apodo, correo o ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";
#elif L==5
	"The @nick, email or ID <strong>%s</strong> is not valid.";				// Besoin de traduction
#elif L==6
	"El @apodo, correo o ID (DNI/c&eacute;dula) <strong>%s</strong> no es v&aacute;lido.";	// Okoteve traduccin
#elif L==7
	"Il @nome-utente, email o ID <strong>%s</strong> non &egrave; valido.";
#elif L==8
	"The @nick, email or ID <strong>%s</strong> is not valid.";				// Potrzebujesz tlumaczenie
#elif L==9
	"The @nick, email or ID <strong>%s</strong> is not valid.";				// Necessita de traduo
#endif

const char *Txt_The_ID_X_matches_one_of_the_existing =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" coincide con uno de los existentes.";		// Necessita traduccio
#elif L==2
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Need bersetzung
#elif L==3
	"The ID <strong>%s</strong>"
	" matches one of the existing.";
#elif L==4
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" coincide con uno de los existentes.";
#elif L==5
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Besoin de traduction
#elif L==6
	"El ID (DNI/c&eacute;dula) <strong>%s</strong>"
	" coincide con uno de los existentes.";		// Okoteve traduccin
#elif L==7
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Bisogno di traduzione
#elif L==8
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The ID <strong>%s</strong>"
	" matches one of the existing.";		// Necessita de traduo
#endif

const char *Txt_The_institution_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La instituci&oacute;n <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2
	"Die Hochschule <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3
	"The institution <strong>%s</strong> already exists.";
#elif L==4
	"La instituci&oacute;n <strong>%s</strong> ya existe.";
#elif L==5
	"The institution <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"La instituci&oacute;n <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7
	"L'istituzione <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The institution <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The institution <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_institution_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"La instituci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The institution <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Need bersetzung
#elif L==3
	"The institution <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"La instituci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The institution <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Besoin de traduction
#elif L==6
	"La instituci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"L'istituzione <strong>%s</strong> &egrave; stata rinominata <strong>%s</strong>.";
#elif L==8
	"The institution <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The institution <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_institution_of_the_centre_has_changed =
#if   L==1
	"La instituci&oacute;n del centro ha cambiado.";	// Necessita traduccio
#elif L==2
	"The institution of the centre has changed.";		// Need bersetzung
#elif L==3
	"The institution of the centre has changed.";
#elif L==4
	"La instituci&oacute;n del centro ha cambiado.";
#elif L==5
	"The institution of the centre .";			// Besoin de traduction
#elif L==6
	"La instituci&oacute;n del centro ha cambiado.";	// Okoteve traduccin
#elif L==7
	"L'istituzione del centro &egrave; cambiata.";
#elif L==8
	"The institution of the centre has changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The institution of the centre has changed.";		// Necessita de traduo
#endif

const char *Txt_The_institution_of_the_department_has_changed =
#if   L==1
	"La instituci&oacute;n del departamento ha cambiado.";	// Necessita traduccio
#elif L==2
	"The institution of the department has changed.";	// Need bersetzung
#elif L==3
	"The institution of the department has changed.";
#elif L==4
	"La instituci&oacute;n del departamento ha cambiado.";
#elif L==5
	"The institution of the department has changed.";	// Besoin de traduction
#elif L==6
	"La instituci&oacute;n del departamento ha cambiado.";	// Okoteve traduccin
#elif L==7
	"L'istituzione del dipartimento &egrave; cambiata.";
#elif L==8
	"The institution of the department has changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The institution of the department has changed.";	// Necessita de traduo
#endif

const char *Txt_The_institutional_code_of_the_course_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";			// Necessita traduccio
#elif L==2
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong> .";		// Need bersetzung
#elif L==3
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";
#elif L==4
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";
#elif L==5
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";			// Besoin de traduction
#elif L==6
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong>"
	" ha cambiado a <strong>%s</strong>.";			// Okoteve traduccin
#elif L==7
	"Il codice istituzionale del corso <strong>%s</strong>"
	" &egrave; cambiato a <strong>%s</strong>.";
#elif L==8
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The institutional code of the course <strong>%s</strong>"
	" has changed to <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_institutional_code_of_the_course_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The institutional code of the course <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3
	"The institutional code of the course <strong>%s</strong> has not changed.";
#elif L==4
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The institutional code of the course <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6
	"El c&oacute;digo institucional de la asignatura <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il codice istituzionale del corso <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The institutional code of the course <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The institutional code of the course <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_integrated_editor_is_not_yet_available =
#if   L==1
	"El editor integrado a&uacute;n no est&aacute; disponible.";	// Necessita traduccio
#elif L==2
	"Der integrierte Editor ist noch nicht verf&uuml;gbar.";
#elif L==3
	"The integrated editor is not yet available.";
#elif L==4
	"El editor integrado a&uacute;n no est&aacute; disponible.";
#elif L==5
	"The integrated editor is not yet available.";			// Besoin de traduction
#elif L==6
	"El editor integrado a&uacute;n no est&aacute; disponible.";	// Okoteve traduccin
#elif L==7
	"L'editor integrato non &egrave; ancora disponibile.";
#elif L==8
	"The integrated editor is not yet available.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The integrated editor is not yet available.";			// Necessita de traduo
#endif

const char *Txt_The_link_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El enlace <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2
	"Der Link <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3
	"The link <strong>%s</strong> already exists.";
#elif L==4
	"El enlace <strong>%s</strong> ya existe.";
#elif L==5
	"The link <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"El enlace <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7
	"Il link <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The link <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The link <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_link_X_has_been_placed_inside_the_folder_Y =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Necessita traduccio
#elif L==2
	"Die Link <strong>%s</strong> wurde"
	" in das Verzeichnis <strong>%s</strong> gesteckt.";
#elif L==3
	"The link <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";
#elif L==4
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";
#elif L==5
	"Le lien <strong>%s</strong> a &eacute;t&eacute; plac&eacute;"
	" dans le r&eacute;pertoire <strong>%s</strong>.";
#elif L==6
	"El enlace <strong>%s</strong> se ha almacenado"
	" dentro de la carpeta <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7
	"Il link <strong>%s</strong> &egrave; stato messo"
	" all'interno della cartella <strong>%s</strong>.";
#elif L==8
	"The link <strong>%s</strong> has been placed"
	" inside the folder <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"A liga&ccedil;&atilde;o <strong>%s</strong> foi colocado"
	" dentro do diret&oacute;rio <strong>%s</strong>.";
#endif

const char *Txt_The_link_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El enlace <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El enlace <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"El enlace <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il link <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The link <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_list_of_X_courses_is_too_large_to_be_displayed =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"La lista de %u asignaturas es demasiado grande para mostrarla.";	// Necessita traduccio
#elif L==2
	"The list of %u courses is too large to be displayed.";			// Need bersetzung
#elif L==3
	"The list of %u courses is too large to be displayed.";
#elif L==4
	"La lista de %u asignaturas es demasiado grande para mostrarla.";
#elif L==5
	"The list of %u courses is too large to be displayed.";			// Besoin de traduction
#elif L==6
	"La lista de %u asignaturas es demasiado grande para mostrarla.";	// Okoteve traduccin
#elif L==7
	"La lista di %u corsi &egrave; troppo grande per essere mostrata.";
#elif L==8
	"The list of %u courses is too large to be displayed.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The list of %u courses is too large to be displayed.";			// Necessita de traduo
#endif

const char *Txt_The_list_of_X_users_is_too_large_to_be_displayed =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"La lista de %u usuarios es demasiado grande para mostrarla.";	// Necessita traduccio
#elif L==2
	"The list of %u users is too large to be displayed.";		// Need bersetzung
#elif L==3
	"The list of %u users is too large to be displayed.";
#elif L==4
	"La lista de %u usuarios es demasiado grande para mostrarla.";
#elif L==5
	"The list of %u users is too large to be displayed.";		// Besoin de traduction
#elif L==6
	"La lista de %u usuarios es demasiado grande para mostrarla.";	// Okoteve traduccin
#elif L==7
	"La lista di %u utenti &egrave; troppo grande per essere mostrata.";
#elif L==8
	"The list of %u users is too large to be displayed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The list of %u users is too large to be displayed.";		// Necessita de traduo
#endif

const char *Txt_The_lower_limit_of_correct_answers_must_be_less_than_or_equal_to_the_upper_limit =
#if   L==1
	"El l&iacute;mite inferior del intervalo de respuestas correctas"
	" debe ser menor o igual que el l&iacute;mite superior.";	// Necessita traduccio
#elif L==2
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// Need bersetzung
#elif L==3
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";
#elif L==4
	"El l&iacute;mite inferior del intervalo de respuestas correctas"
	" debe ser menor o igual que el l&iacute;mite superior.";
#elif L==5
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// Besoin de traduction
#elif L==6
	"El l&iacute;mite inferior del intervalo de respuestas correctas"
	" debe ser menor o igual que el l&iacute;mite superior.";	// Okoteve traduccin
#elif L==7
	"Il limite inferiore di risposte corrette"
	" deve essere minore o uguale al limite superiore.";
#elif L==8
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The lower limit of correct answers"
	" must be less than or equal to the upper limit.";		// Necessita de traduo
#endif

const char *Txt_the_marks_of_a_student_chosen_at_random_ =
#if   L==1
	"se mostrar&aacute;n las calificaciones de un estudiante de la asignatura escogido al azar;<br />"
	" si ve m&aacute;s de un estudiante, significa que los n&uacute;meros de filas de cabecera o de pie no son correctos";	// Necessita traduccio
#elif L==2
	"es wird die Bewertung eines nach dem Zufallsprinzip ausgew&auml;ten Studenten angezeigt;<br />"
	" wenn Sie mehr als einen Student bedeutet dies, dass die Anzahl der Kopf oder Fu von Zeilen ist nicht korrekt";
#elif L==3
	"the marks of a student chosen at random from the course will be shown;<br />"
	" if you see more than one student, it means that the number of header or footer rows is not correct";
#elif L==4
	"se mostrar&aacute;n las calificaciones de un estudiante de la asignatura escogido al azar;<br />"
	" si ve m&aacute;s de un estudiante, significa que los n&uacute;meros de filas de cabecera o de pie no son correctos";
#elif L==5
	"les notes d'un &eacute;tudiant choisi au hasard dans la mati&egrave;re seront pr&eacute;sent&eacute;s;<br />"
	" si vous voyez plus d'un &eacute;tudiant, cela signifie que le nombre de ligned de la t&ecirc;te ou du pied n'est pas correcte";
#elif L==6
	"se mostrar&aacute;n las calificaciones de un estudiante de la asignatura escogido al azar;<br />"
	" si ve m&aacute;s de un estudiante, significa que los n&uacute;meros de filas de cabecera o de pie no son correctos";	// Okoteve traduccin
#elif L==7
	"i risultati di uno studente scelto a caso dal corso verranno mostrati;<br />"
	" se si vede pi&uacute; di uno studente, significa che il numero di righe di intestazione o a pi&egrave; di pagina non &egrave; corretto";
#elif L==8
	"the marks of a student chosen at random from the course will be shown;<br />"
	" if you see more than one student, it means that the number of header or footer rows is not correct";	// Potrzebujesz tlumaczenie
#elif L==9
	"vai mostrar as notas de um estudante da disciplina escolhido ao acaso;<br />"
	" se voc&ecirc; vir mais de um estudante, isso significa que o n&uacute;mero de cabe&ccedil;a ou p&eacute; linhas n&atilde;o est&aacute; correto";
#endif

const char *Txt_The_maximum_number_of_students_in_the_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The maximum number of students"
	" in the group <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3
	"The maximum number of students"
	" in the group <strong>%s</strong> has not changed.";
#elif L==4
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The maximum number of students"
	" in the group <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il numero massimo di studenti"
	" nel gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The maximum number of students"
	" in the group <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The maximum number of students"
	" in the group <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_maximum_number_of_students_in_the_group_X_is_now_Y =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> ahora es <strong>%u</strong>.";	// Necessita traduccio
#elif L==2
	"The maximum number of students"
	" in the group <strong>%s</strong> is now <strong>%u</strong>.";	// Need bersetzung
#elif L==3
	"The maximum number of students"
	" in the group <strong>%s</strong> is now <strong>%u</strong>.";
#elif L==4
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> ahora es <strong>%u</strong>.";
#elif L==5
	"The maximum number of students"
	" in the group <strong>%s</strong> is now <strong>%u</strong>.";	// Besoin de traduction
#elif L==6
	"El n&ordm; m&aacute;ximo de estudiantes"
	" del grupo <strong>%s</strong> ahora es <strong>%u</strong>.";		// Okoteve traduccin
#elif L==7
	"Il numero massimo di studenti"
	" nel gruppo <strong>%s</strong> &egrave; ora <strong>%u</strong>.";
#elif L==8
	"The maximum number of students"
	" in the group <strong>%s</strong> is now <strong>%u</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The maximum number of students"
	" in the group <strong>%s</strong> is now <strong>%u</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_message_has_not_been_sent_to_any_recipient =
#if   L==1
	"El missatge no ha estat enviat a cap destinatari.";
#elif L==2
	"The message has not been sent to any recipient.";		// Need bersetzung
#elif L==3
	"The message has not been sent to any recipient.";
#elif L==4
	"El mensaje no se ha enviado a ning&uacute;n destinatario.";
#elif L==5
	"The message has not been sent to any recipient.";		// Besoin de traduction
#elif L==6
	"El mensaje no se ha enviado a ning&uacute;n destinatario.";	// Okoteve traduccin
#elif L==7
	"Il messaggio non &egrave; stato inviato a nessun destinatario.";
#elif L==8
	"The message has not been sent to any recipient.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The message has not been sent to any recipient.";		// Necessita de traduo
#endif

const char *Txt_The_message_has_been_sent_to_1_recipient =
#if   L==1
	"El mensaje se ha enviado a 1 destinatario.";	// Necessita traduccio
#elif L==2
	"The message has been sent to 1 recipient.";	// Need bersetzung
#elif L==3
	"The message has been sent to 1 recipient.";
#elif L==4
	"El mensaje se ha enviado a 1 destinatario.";
#elif L==5
	"The message has been sent to 1 recipient.";	// Besoin de traduction
#elif L==6
	"El mensaje se ha enviado a 1 destinatario.";	// Okoteve traduccin
#elif L==7
	"Il messaggio &egrave; stato inviato a 1 destinatario.";
#elif L==8
	"The message has been sent to 1 recipient.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The message has been sent to 1 recipient.";	// Necessita de traduo
#endif

const char *Txt_The_message_has_been_sent_to_X_recipients =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"El mensaje se ha enviado a %u destinatarios.";	// Necessita traduccio
#elif L==2
	"The message has been sent to %u recipients.";	// Need bersetzung
#elif L==3
	"The message has been sent to %u recipients.";
#elif L==4
	"El mensaje se ha enviado a %u destinatarios.";
#elif L==5
	"The message has been sent to %u recipients.";	// Besoin de traduction
#elif L==6
	"El mensaje se ha enviado a %u destinatarios.";	// Okoteve traduccin
#elif L==7
	"Il messaggio &egrave; stato inviato a %u destinatari.";
#elif L==8
	"The message has been sent to %u recipients.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The message has been sent to %u recipients.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_banner_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre del banner <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the banner <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3
	"The name of the banner <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre del banner <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the banner <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6
	"El nombre del banner <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del banner <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the banner <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the banner <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_centre_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre del centro <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the centre <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3
	"The name of the centre <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre del centro <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the centre <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6
	"El nombre del centro <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del centro <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the centre <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the centre <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_country_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre del pa&iacute;s <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the country <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3
	"The name of the country <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre del pa&iacute;s <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the country <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6
	"El nombre del pa&iacute;s <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del paese <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the country <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the country <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_course_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El nombre de la asignatura <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong> .";	// Need bersetzung
#elif L==3
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4
	"El nombre de la asignatura <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"El nombre de la asignatura <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il nome del corso <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==8
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the course <strong>%s</strong> has changed to <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_course_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre de la asignatura <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the course <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3
	"The name of the course <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre de la asignatura <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the course <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6
	"El nombre de la asignatura <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del corso <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the course <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the course <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_degree_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El nombre de la titulaci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4
	"El nombre de la titulaci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"El nombre de la titulaci&oacute;n <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il nome della laurea <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==8
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the degree <strong>%s</strong> has changed to <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_degree_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre de la titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the degree <strong>%s</strong> has not changed.";			// Need bersetzung
#elif L==3
	"The name of the degree <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre de la titulaci&oacute;n <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the degree <strong>%s</strong> has not changed.";			// Besoin de traduction
#elif L==6
	"El nombre de la titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome della laurea <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the degree <strong>%s</strong> has not changed.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the degree <strong>%s</strong> has not changed.";			// Necessita de traduo
#endif

const char *Txt_The_name_of_the_department_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre del departamento <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the department <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3
	"The name of the department <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre del departamento <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the department <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6
	"El nombre del departamento <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del dipartimento <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the department <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the department <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_field_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre del campo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the field <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3
	"The name of the field <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre del campo <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the field <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6
	"El nombre del campo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del campo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the field <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the field <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre del grupo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the group <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3
	"The name of the group <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre del grupo <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the group <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6
	"El nombre del grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the group <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the group <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_holiday_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El nombre de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4
	"El nombre de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"El nombre de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il nome della festivit&agrave; <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==8
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_holiday_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre de la festividad <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the holiday <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3
	"The name of the holiday <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre de la festividad <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the holiday <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6
	"El nombre de la festividad <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome della festivit&agrave; <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the holiday <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the holiday <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_institution_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre de la instituci&oacute;n <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the institution <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3
	"The name of the institution <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre de la instituci&oacute;n <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the institution <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6
	"El nombre de la instituci&oacute;n <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome dell'istituzione <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the institution <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the institution <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_link_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre del enlace <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the link <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3
	"The name of the link <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre del enlace <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the link <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6
	"El nombre del enlace <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del link <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the link <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the link <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_place_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre del lugar <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the place <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3
	"The name of the place <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre del lugar <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the place <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6
	"El nombre del lugar <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del luogo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the place <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the place <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_name_of_the_plugin_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre del complemento <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the plugin <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3
	"The name of the plugin <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre del complemento <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the plugin <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6
	"El nombre del complemento <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del plugin <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the plugin <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the plugin <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_type_of_degree_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre del tipo de titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the type of degree <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3
	"The name of the type of degree <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre del tipo de titulaci&oacute;n <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the type of degree <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6
	"El nombre del tipo de titulaci&oacute;n <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del tipo di laurea <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the type of degree <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the type of degree <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_name_of_the_type_of_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nombre del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The name of the type of group <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3
	"The name of the type of group <strong>%s</strong> has not changed.";
#elif L==4
	"El nombre del tipo de grupo <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The name of the type of group <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6
	"El nombre del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il nome del tipo di gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The name of the type of group <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The name of the type of group <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_new_IP_address_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La nueva direcci&oacute;n IP es <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The new IP address is <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"The new IP address is <strong>%s</strong>.";
#elif L==4
	"La nueva direcci&oacute;n IP es <strong>%s</strong>.";
#elif L==5
	"The new IP address is <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"La nueva direcci&oacute;n IP es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il nuovo indirizzo IP &egrave; <strong>%s</strong>.";
#elif L==8
	"The new IP address is <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The new IP address is <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_new_logo_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nuevo logo es <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The new logo is <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"The new logo is <strong>%s</strong>.";
#elif L==4
	"El nuevo logo es <strong>%s</strong>.";
#elif L==5
	"The new logo is <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"El nuevo logo es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il nuovo logo &egrave; <strong>%s</strong>.";
#elif L==8
	"The new logo is <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The new logo is <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_nickname_entered_X_is_not_valid_ =	// Warning: it is very important to include %s and two %u in the following sentences
#if   L==1
	"El apodo <strong>%s</strong> no es v&aacute;lido.<br />"
	"El apodo debe tener una longitud entre %u y %u caracteres (sin contar la @ inicial).<br />"
	"Cada car&aacute;cter posterior a la @ inicial puede ser una letra (a-z, A-Z),"
	" un d&iacute;gito (0-9) o &quot;_&quot;.";	// Necessita traduccio
#elif L==2
	"The nickname <strong>%s</strong> is not valid.<br />"
	"Your nickname must have a length between %u and %u characters (not including the initial @).<br />"
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Need bersetzung
#elif L==3
	"The nickname <strong>%s</strong> is not valid.<br />"
	"Your nickname must have a length between %u and %u characters (not including the initial @).<br />"
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";
#elif L==4
	"El apodo <strong>%s</strong> no es v&aacute;lido.<br />"
	"El apodo debe tener una longitud entre %u y %u caracteres (sin contar la @ inicial).<br />"
	"Cada car&aacute;cter posterior a la @ inicial puede ser una letra (a-z, A-Z),"
	" un d&iacute;gito (0-9) o &quot;_&quot;.";
#elif L==5
	"The nickname <strong>%s</strong> is not valid.<br />"
	"Your nickname must have a length between %u and %u characters (not including initial the @).<br />"
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Besoin de traduction
#elif L==6
	"El apodo <strong>%s</strong> no es v&aacute;lido.<br />"
	"El apodo debe tener una longitud entre %u y %u caracteres (sin contar la @ inicial).<br />"
	"Cada car&aacute;cter posterior a la @ inicial puede ser una letra (a-z, A-Z),"
	" un d&iacute;gito (0-9) o &quot;_&quot;.";	// Okoteve traduccin
#elif L==7
	"Il nome utente <strong>%s</strong> non &egrave; valido.<br />"
	"Your nickname must have a length between %u and %u characters (not including the initial @).<br /> "
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Bisogno di traduzione
#elif L==8
	"The nickname <strong>%s</strong> is not valid.<br />"
	"Your nickname must have a length between %u and %u characters (not including the initial @).<br />"
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The nickname <strong>%s</strong> is not valid.<br />"
	"Your nickname must have a length between %u and %u characters (not including the initial @).<br />"
	"Each character after initial @ can be a letter (a-z, A-Z),"
	" a digit (0-9) or &quot;_&quot;.";		// Necessita de traduo
#endif

const char *Txt_The_notice_will_appear_as_a_yellow_note_ =
#if   L==1
	"L'av&iacute;s apareixer&agrave; en forma de nota groga"
	" a l'esquerra de la p&agrave;gina"
	" i ser&agrave; <strong>visible per tot el m&oacute;n</strong>"
	" (no nom&eacute;s pels usuaris de l'assignatura).<br />"
	"<strong>El missatge ha de ser breu.</strong>"
	" Per crear un av&iacute;s extens"
	" o d'acc&eacute;s exclusiu per a estudiants,"
	" pot utilitzar el f&ograve;rum de l'assignatura,"
	" enviar un missatge a tots,"
	" o pujar un arxiu amb el text a la zona de documents.";
#elif L==2
	"Die Ank&uuml;ndigung wird als ein gelber Hinweis"
	" auf der linken Seite der Seite erscheinen"
	" und wird <strong>weltweit</strong>"
	" (nicht nur von den Benutzern des Kurses) sichtbar sein.<br />"
	"<strong>Die Nachricht sollte kurz sein</strong>."
	" Wenn Sie eine lange Ank&uuml;ndigung"
	" oder einen Hinweis mit exklusivem Zugang f&uuml;r Studenten erstellen m&ouml;chten,"
	" k&ouml;nnen Sie das Forum des Kurses nutzen,"
	" eine Nachricht an alle senden"
	" oder eine Datei mit dem Text in den Dokumentenbereich hochladen.";
#elif L==3
	"The notice will appear as a yellow note"
	" in the left of the page"
	" and will be <strong>visible worldwide</strong>"
	" (not just by users of the course).<br />"
	"<strong>The message should be short</strong>."
	" If you want to create a long notice"
	" or a notice with exclusive access for students,"
	" you can use the forum of the course,"
	" send a message to all,"
	" or upload a file with the text to the documents area.";
#elif L==4
	"El aviso aparecer&aacute; en forma de nota amarilla"
	" a la izquierda de la p&aacute;gina"
	" y ser&aacute; <strong>visible por todo el mundo</strong>"
	" (no solo por los usuarios de la asignatura).<br />"
	"<strong>El mensaje debe ser breve</strong>."
	" Si desea crear un aviso extenso"
	" o de acceso exclusivo para estudiantes,"
	" puede usar el foro de la asignatura,"
	" enviar un mensaje a todos,"
	" o subir un archivo con el texto a la zona de documentos.";
#elif L==5
	"L'avis appara&icirc;tra comme une note jaune"
	" &agrave; gauche de la page"
	" et sera <strong>visible dans le monde entier</strong>"
	" (pas seulement par les utilisateurs du cours).<br />"
	"<strong>Le message doit &ecirc;tre court</strong>."
	" Si vous souhaitez cr&eacute;er un avis long"
	" ou un avis avec un acc&egrave;s exclusif pour les &eacute;tudiants,"
	" vous pouvez utiliser le forum du cours,"
	" envoyer un message &agrave; tous"
	" ou t&eacute;l&eacute;charger un fichier avec le texte dans la zone des documents.";
#elif L==6
	"El aviso aparecer&aacute; en forma de nota amarilla"
	" a la izquierda de la p&aacute;gina"
	" y ser&aacute; <strong>visible por todo el mundo</strong>"
	" (no solo por los usuarios de la asignatura).<br />"
	"<strong>El mensaje debe ser breve</strong>."
	" Si desea crear un aviso extenso"
	" o de acceso exclusivo para estudiantes,"
	" puede usar el foro de la asignatura,"
	" enviar un mensaje a todos,"
	" o subir un archivo con el texto a la zona de documentos.";	// Okoteve traduccin
#elif L==7
	"L'avviso verr&agrave; visualizzato come una nota di colore giallo"
	" nella parte sinistra della pagina"
	" e sar&agrave; <strong>visibile in tutto il mondo</strong>"
	" (non solo dagli utenti del corso).<br />"
	"<strong>Il messaggio deve essere breve</strong>."
	" Se si desidera creare un lungo avviso"
	" o una comunicazione con accesso esclusivo per gli studenti,"
	" &egrave; possibile utilizzare il forum del corso,"
	" inviare un messaggio a tutti,"
	" o caricare un file con il testo all'area documenti.";
#elif L==8
	"Og&lstrok;oszenie pojawi si&eogon; jako &zdot;&oacute;&lstrok;ty notatki"
	" w lewej strony"
	" i b&eogon;dzie <strong>widoczny na ca&lstrok;ym &sacute;wiecie</strong>"
	" (nie tylko przez u&zdot;ytkownik&oacute;w oczywi&sacute;cie).<br />"
	"<strong>Wiadomo&sacute;&cacute; powinna by&cacute; kr&oacute;tka</strong>."
	" Je&sacute;li chcesz stworzy&cacute; d&lstrok;ug&aogon; zawiadomienie"
	" lub zawiadomienie z wy&lstrok;&aogon;cznego dost&eogon;pu dla student&oacute;w,"
	" mo&zdot;na korzysta&cacute; z forum kursu,"
	" wys&lstrok;a&cacute; wiadomo&sacute;&cacute; do wszystkich,"
	" lub przes&lstrok;a&cacute; plik z tekstem do obszaru dokument&oacute;w.";
#elif L==9
	"O aviso aparecer&aacute; como uma nota amarela"
	" &agrave; esquerda da p&aacute;gina"
	" e ser&aacute; <strong>vis&iacute;vel em todo o mundo</strong>"
	" (n&atilde;o apenas pelos usu&aacute;rios do curso).<br />"
	"<strong>A mensagem deve ser curta</strong>."
	" Se voc&ecirc; quiser criar um aviso longo"
	" ou um aviso com acesso exclusivo para os alunos,"
	" voc&ecirc; pode usar o f&oacute;rum do curso,"
	" enviar uma mensagem para todos"
	" ou enviar um arquivo com o texto para a &aacute;rea de documentos.";
#endif

const char *Txt_The_new_web_address_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La nueva direcci&oacute;n web es <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The new web address is <strong>%s</strong>.";			// Need bersetzung
#elif L==3
	"The new web address is <strong>%s</strong>.";
#elif L==4
	"La nueva direcci&oacute;n web es <strong>%s</strong>.";
#elif L==5
	"The new web address is <strong>%s</strong>.";			// Besoin de traduction
#elif L==6
	"La nueva direcci&oacute;n web es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il nuovo indirizzo web &egrave; <strong>%s</strong>.";
#elif L==8
	"The new web address is <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The new web address is <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_new_description_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La nueva descripci&oacute;n web es <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The new description is <strong>%s</strong>.";			// Need bersetzung
#elif L==3
	"The new description is <strong>%s</strong>.";
#elif L==4
	"La nueva descripci&oacute;n web es <strong>%s</strong>.";
#elif L==5
	"The new description is <strong>%s</strong>.";			// Besoin de traduction
#elif L==6
	"La nueva descripci&oacute;n web es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"La nuova descrizione &egrave; <strong>%s</strong>.";
#elif L==8
	"The new description is <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The new description is <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_new_image_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La nueva imagen es <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The new image is <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"The new image is <strong>%s</strong>.";
#elif L==4
	"La nueva imagen es <strong>%s</strong>.";
#elif L==5
	"The new image is <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"La nueva imagen es <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"La nuova image &egrave; <strong>%s</strong>.";
#elif L==8
	"The new image is <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The new image is <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_new_URL_is_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El nuevo URL es <strong>%s</strong>.";		// Necessita traduccio
#elif L==2
	"The new URL is <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"The new URL is <strong>%s</strong>.";
#elif L==4
	"El nuevo URL es <strong>%s</strong>.";
#elif L==5
	"The new URL is <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"El nuevo URL es <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7
	"La nuova URL &egrave; <strong>%s</strong>.";
#elif L==8
	"The new URL is <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The new URL is <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_nickname_X_had_been_registered_by_another_user =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El apodo <strong>@%s</strong>"
	" ya hab&iacute;a sido registrado por otro usuario.";	// Necessita traduccio
#elif L==2
	"The nickname <strong>@%s</strong>"
	" had been registered by another user.";		// Need bersetzung
#elif L==3
	"The nickname <strong>@%s</strong>"
	" had been registered by another user.";
#elif L==4
	"El apodo <strong>@%s</strong>"
	" ya hab&iacute;a sido registrado por otro usuario.";
#elif L==5
	"The nickname <strong>@%s</strong>"
	" had been enrolled by another user.";			// Besoin de traduction
#elif L==6
	"El apodo <strong>@%s</strong>"
	" ya hab&iacute;a sido registrado por otro usuario.";	// Okoteve traduccin
#elif L==7
	"Il nome utente <strong>@%s</strong>"
	" &egrave; stato registrato da un altro utente.";
#elif L==8
	"The nickname <strong>@%s</strong>"
	" had been registered by another user.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The nickname <strong>@%s</strong>"
	" had been registered by another user.";		// Necessita de traduo
#endif

const char *Txt_The_nickname_X_matches_the_one_you_had_previously_registered =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El apodo <strong>@%s</strong>"
	" coincide con el que ya ten&iacute;a registrado previamente.";	// Necessita traduccio
#elif L==2
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";		// Need bersetzung
#elif L==3
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";
#elif L==4
	"El apodo <strong>@%s</strong>"
	" coincide con el que ya ten&iacute;a registrado previamente.";
#elif L==5
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";		// Besoin de traduction
#elif L==6
	"El apodo <strong>@%s</strong>"
	" coincide con el que ya ten&iacute;a registrado previamente.";	// Okoteve traduccin
#elif L==7
	"Il nome utente <strong>@%s</strong>"
	" coincide con quello che hai precedentemente registrato.";
#elif L==8
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The nickname <strong>@%s</strong>"
	" matches the one you had previously registered.";		// Necessita de traduo
#endif

const char *Txt_The_number_of_editing_lines_in_the_record_field_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The number of editing lines in the record field <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3
	"The number of editing lines in the record field <strong>%s</strong> has not changed.";
#elif L==4
	"El n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The number of editing lines in the record field <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6
	"El n&ordm; de l&iacute;neas de edici&oacute;n del campo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il numero di linee editate nel campo scheda <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The number of editing lines in the record field <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The number of editing lines in the record field <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_number_of_questions_must_be_in_the_interval_X =	// Warning: it is very important to include two %u in the following sentences
#if   L==1
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";	// Necessita traduccio
#elif L==2
	"Die Anzahl der Fragen muss sich im Intervall [%u,%u] befinden.";
#elif L==3
	"The number of questions must be in the interval [%u,%u].";
#elif L==4
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";
#elif L==5
	"The number of questions must be in the interval [%u,%u].";		// Besoin de traduction
#elif L==6
	"El n&uacute;mero de preguntas debe estar en el intervalo [%u,%u].";	// Okoteve traduccin
#elif L==7
	"Il numero di domande dovrebbe stare nell'intervallo [%u,%u].";
#elif L==8
	"The number of questions must be in the interval [%u,%u].";		// Potrzebujesz tlumaczenie
#elif L==9
	"The number of questions must be in the interval [%u,%u].";		// Necessita de traduo
#endif

const char *Txt_The_number_of_rows_is_now_X =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"El n&uacute;mero de filas ahora es <strong>%u</strong>.";	// Necessita traduccio
#elif L==2
	"Die Anzahl der Zeilen ist jetzt <strong>%u</strong>.";
#elif L==3
	"The number of rows is now <strong>%u</strong>.";
#elif L==4
	"El n&uacute;mero de filas ahora es <strong>%u</strong>.";
#elif L==5
	"The number of rows is now <strong>%u</strong>.";		// Besoin de traduction
#elif L==6
	"El n&uacute;mero de filas ahora es <strong>%u</strong>.";	// Okoteve traduccin
#elif L==7
	"Il numero di righe &egrave; ora <strong>%u</strong>.";
#elif L==8
	"The number of rows is now <strong>%u</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The number of rows is now <strong>%u</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_numerical_code_X_already_exists =	// Warning: it is very important to include %03lu in the following sentences
#if   L==1
	"El c&oacute;digo num&eacute;rico ISO 3166-1 %03ld ya existe.";	// Necessita traduccio
#elif L==2
	"The ISO 3166-1 numerical code %03ld already exists.";		// Need bersetzung
#elif L==3
	"The ISO 3166-1 numerical code %03ld already exists.";
#elif L==4
	"El c&oacute;digo num&eacute;rico ISO 3166-1 %03ld ya existe.";
#elif L==5
	"The ISO 3166-1 numerical code %03ld already exists.";		// Besoin de traduction
#elif L==6
	"El c&oacute;digo num&eacute;rico ISO 3166-1 %03ld ya existe.";	// Okoteve traduccin
#elif L==7
	"Il codice numerico ISO 3166-1 %03ld gi&agrave; esiste.";
#elif L==8
	"The ISO 3166-1 numerical code %03ld already exists.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The ISO 3166-1 numerical code %03ld already exists.";		// Necessita de traduo
#endif

const char *Txt_The_photo_no_longer_exists =
#if   L==1
	"La foto ja no existeix.";
#elif L==2
	"Das Foto nicht mehr existiert.";
#elif L==3
	"The photo no longer exists.";
#elif L==4
	"La foto ya no existe.";
#elif L==5
	"La photo a disparu.";
#elif L==6
	"La foto ya no existe.";	// Okoteve traduccin
#elif L==7
	"La foto non esiste pi&ugrave;.";
#elif L==8
	"Zdj&eogon;cie ju&zdot; nie istnieje.";
#elif L==9
	"La foto n&atilde;o existe mais.";
#endif

const char *Txt_The_original_post_no_longer_exists =
#if   L==1
	"La publicaci&oacute; original ja no existeix.";
#elif L==2
	"Der urspr&uuml;ngliche Post nicht mehr existiert.";
#elif L==3
	"The original post no longer exists.";
#elif L==4
	"La publicaci&oacute;n original ya no existe.";
#elif L==5
	"La publication originale a disparu.";
#elif L==6
	"La publicaci&oacute;n original ya no existe.";	// Okoteve traduccin
#elif L==7
	"La pubblicazione originale non esiste pi&ugrave;.";
#elif L==8
	"Oryginalna publikacja ju&zdot; nie istnieje.";
#elif L==9
	"A publica&ccedil;&atilde;o original n&atilde;o existe mais.";
#endif

const char *Txt_The_password_can_not_consist_only_of_digits =
#if   L==1
	"La contrase&ntilde;a no puede estar formada"
	" solo por d&iacute;gitos.";		// Necessita traduccio
#elif L==2
	"The password can not consist"
	" only of digits.";			// Need bersetzung
#elif L==3
	"The password can not consist"
	" only of digits.";
#elif L==4
	"La contrase&ntilde;a no puede estar formada"
	" solo por d&iacute;gitos.";
#elif L==5
	"The password can not consist"
	" only of digits.";			// Besoin de traduction
#elif L==6
	"La contrase&ntilde;a no puede estar formada"
	" solo por d&iacute;gitos.";		// Okoteve traduccin
#elif L==7
	"La password non pu&ograve; consistere"
	" solo di numeri.";
#elif L==8
	"The password can not consist"
	" only of digits.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The password can not consist"
	" only of digits.";			// Necessita de traduo
#endif

const char *Txt_The_password_can_not_contain_spaces =
#if   L==1
	"La contrase&ntilde;a no puede tener espacios.";	// Necessita traduccio
#elif L==2
	"The password can not contain spaces.";			// Need bersetzung
#elif L==3
	"The password can not contain spaces.";
#elif L==4
	"La contrase&ntilde;a no puede tener espacios.";
#elif L==5
	"The password can not contain spaces.";			// Besoin de traduction
#elif L==6
	"La contrase&ntilde;a no puede tener espacios.";	// Okoteve traduccin
#elif L==7
	"La password non pu&ograve; contenere spazi.";
#elif L==8
	"The password can not contain spaces.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The password can not contain spaces.";			// Necessita de traduo
#endif

const char *Txt_The_password_is_too_trivial_ =
#if   L==1
	"La contrase&ntilde;a es demasiado f&aacute;cil."
	" Use otra m&aacute;s dif&iacute;cil de averiguar.";	// Necessita traduccio
#elif L==2
	"The password is too trivial."
	" Use other harder to guess.";				// Need bersetzung
#elif L==3
	"The password is too trivial."
	" Use other harder to guess.";
#elif L==4
	"La contrase&ntilde;a es demasiado f&aacute;cil."
	" Use otra m&aacute;s dif&iacute;cil de averiguar.";
#elif L==5
	"The password is too trivial."
	" Use other harder to guess.";				// Besoin de traduction
#elif L==6
	"La contrase&ntilde;a es demasiado f&aacute;cil."
	" Use otra m&aacute;s dif&iacute;cil de averiguar.";	// Okoteve traduccin
#elif L==7
	"La password &egrave; troppo semplice."
	" Utilizza un'altra pi&ugrave; difficile da individuare.";
#elif L==8
	"The password is too trivial."
	" Use other harder to guess.";				// Potrzebujesz tlumaczenie
#elif L==9
	"The password is too trivial."
	" Use other harder to guess.";				// Necessita de traduo
#endif

const char *Txt_The_password_must_be_at_least_X_characters =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"La contrase&ntilde;a ha de tener al menos %u caracteres.";	// Necessita traduccio
#elif L==2
	"The password must be at least %u characters.";			// Need bersetzung
#elif L==3
	"The password must be at least %u characters.";
#elif L==4
	"La contrase&ntilde;a ha de tener al menos %u caracteres.";
#elif L==5
	"The password must be at least %u characters.";			// Besoin de traduction
#elif L==6
	"La contrase&ntilde;a ha de tener al menos %u caracteres.";	// Okoteve traduccin
#elif L==7
	"La password dovrebbe essere di almeno %u caratteri.";
#elif L==8
	"The password must be at least %u characters.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The password must be at least %u characters.";			// Necessita de traduo
#endif

const char *Txt_The_X_password_has_been_changed_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La contrase&ntilde;a de %s se ha modificado con &eacute;xito.";	// Necessita traduccio
#elif L==2
	"The %s password has been changed successfully.";			// Need bersetzung
#elif L==3
	"The %s password has been changed successfully.";
#elif L==4
	"La contrase&ntilde;a de %s se ha modificado con &eacute;xito.";
#elif L==5
	"The %s password has been changed successfully.";			// Besoin de traduction
#elif L==6
	"La contrase&ntilde;a de %s se ha modificado con &eacute;xito.";	// Okoteve traduccin
#elif L==7
	"La %s password &egrave; stata cambiata con successo.";
#elif L==8
	"The %s password has been changed successfully.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The %s password has been changed successfully.";			// Necessita de traduo
#endif

const char *Txt_The_place_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El lugar <strong>%s</strong> ya existe.";		// Necessita traduccio
#elif L==2
	"Der Standort <strong>%s</strong> wurde bereits eingegeben.";
#elif L==3
	"The place <strong>%s</strong> already exists.";
#elif L==4
	"El lugar <strong>%s</strong> ya existe.";
#elif L==5
	"The place <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"El lugar <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7
	"Il luogo <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The place <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The place <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_place_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El lugar <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El lugar <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"El lugar <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il luogo <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The place <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_place_of_the_centre_has_changed =
#if   L==1
	"El lugar del centro ha cambiado.";		// Necessita traduccio
#elif L==2
	"The place of the centre has changed.";		// Need bersetzung
#elif L==3
	"The place of the centre has changed.";
#elif L==4
	"El lugar del centro ha cambiado.";
#elif L==5
	"The place of the centre has changed.";		// Besoin de traduction
#elif L==6
	"El lugar del centro ha cambiado.";		// Okoteve traduccin
#elif L==7
	"Il luogo del centro &egrave; cambiato.";
#elif L==8
	"The place of the centre has changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The place of the centre has changed.";		// Necessita de traduo
#endif

const char *Txt_The_place_of_the_holiday_X_has_changed_to_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El lugar de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";
#elif L==4
	"El lugar de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";
#elif L==5
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"El lugar de la festividad <strong>%s</strong> ha cambiado a <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il luogo della festivit&agrave; <strong>%s</strong> &egrave; cambiato a <strong>%s</strong>.";
#elif L==8
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The place of the holiday <strong>%s</strong> has changed to <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_plugin_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El complemento <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2
	"Der Plugin <strong>%s</strong> wurde bereits installiert.";
#elif L==3
	"The plugin <strong>%s</strong> already exists.";
#elif L==4
	"El complemento <strong>%s</strong> ya existe.";
#elif L==5
	"The plugin <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"El complemento <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7
	"Il plugin <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The plugin <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The plugin <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_plugin_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El complemento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El complemento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"El complemento <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il plugin <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The plugin <strong>%s</strong> has been renamed as <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_properties_of_file_X_have_been_saved =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Las propiedades del archivo %s se han guardado.";	// Necessita traduccio
#elif L==2
	"The properties of file %s have been saved.";		// Need bersetzung
#elif L==3
	"The properties of file %s have been saved.";
#elif L==4
	"Las propiedades del archivo %s se han guardado.";
#elif L==5
	"The properties of file %s have been saved.";		// Besoin de traduction
#elif L==6
	"Las propiedades del archivo %s se han guardado.";	// Okoteve traduccin
#elif L==7
	"The properties of file %s have been saved.";		// Bisogno di traduzione
#elif L==8
	"The properties of file %s have been saved.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The properties of file %s have been saved.";		// Necessita de traduo
#endif

const char *Txt_The_record_field_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El campo de ficha <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2
	"Das Datenfeld <strong>%s</strong> wurde bereits erstellt.";
#elif L==3
	"The record field <strong>%s</strong> already exists.";
#elif L==4
	"El campo de ficha <strong>%s</strong> ya existe.";
#elif L==5
	"The record field <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"El campo de ficha <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7
	"Il campo di scheda <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The record field <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The record field <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_record_field_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El campo de ficha <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El campo de ficha <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"El campo de ficha <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il campo di scheda <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The record field <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_requested_group_changes_were_successful =
#if   L==1
	"Els canvis de grup sol&middot;licitats s'han realitzat correctament.";
#elif L==2
	"The requested group changes were successful.";				// Need bersetzung
#elif L==3
	"The requested group changes were successful.";
#elif L==4
	"Los cambios de grupo solicitados se han realizado correctamente.";
#elif L==5
	"Les changements de groupe demand&eacute;es ont r&eacute;ussi.";
#elif L==6
	"Los cambios de grupo solicitados se han realizado correctamente.";	// Okoteve traduccin
#elif L==7
	"I cambiamenti di gruppi richieste hanno avuto successo.";
#elif L==8
	"The requested group changes were successful.";				// Potrzebujesz tlumaczenie
#elif L==9
	"The requested group changes were successful.";				// Necessita de traduo
#endif

const char *Txt_The_role_of_THE_USER_X_in_the_course_Y_has_changed_from_A_to_B =	// Warning: it is very important to include four %s in the following sentences
#if   L==1
	"El rol de <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha cambiado de %s a %s.";		// Necessita traduccio
#elif L==2
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// Need bersetzung
#elif L==3
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";
#elif L==4
	"El rol de <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha cambiado de %s a %s.";
#elif L==5
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// Besoin de traduction
#elif L==6
	"El rol de <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>"
	" ha cambiado de %s a %s.";		// Okoteve traduccin
#elif L==7
	"Il ruolo de <strong>%s</strong>"
	" nel corso <strong>%s</strong>"
	" &egrave; cambiato da %s a %s.";
#elif L==8
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The role of <strong>%s</strong>"
	" in the course <strong>%s</strong>"
	" has changed from %s to %s.";		// Necessita de traduo
#endif

const char *Txt_The_X_students_who_belonged_to_the_course_Y_have_been_removed_from_it =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==1
	"Se han eliminado los %u estudiantes que hab&iacute;a en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The %u students who belonged to the course <strong>%s</strong> have been removed from it.";	// Need bersetzung
#elif L==3
	"The %u students who belonged to the course <strong>%s</strong> have been removed from it.";
#elif L==4
	"Se han eliminado los %u estudiantes que hab&iacute;a en la asignatura <strong>%s</strong>.";
#elif L==5
	"The %u students who belonged to the course <strong>%s</strong> have been removed from it.";	// Besoin de traduction
#elif L==6
	"Se han eliminado los %u estudiantes que hab&iacute;a en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Gli %u studenti che appartengono al corso <strong>%s</strong> sono stati rimossi da esso.";
#elif L==8
	"The %u students who belonged to the course <strong>%s</strong> have been removed from it.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The %u students who belonged to the course <strong>%s</strong> have been removed from it.";	// Necessita de traduo
#endif

const char *Txt_The_search_text_must_be_longer =
#if   L==1
	"El text a cercar ha de ser m&eacute;s llarg.";
#elif L==2
	"Der Suchtext muss l&auml;nger sein.";
#elif L==3
	"The search text must be longer.";
#elif L==4
	"El texto a buscar debe ser m&aacute;s largo.";
#elif L==5
	"Le texte de recherche doit &ecirc;tre plus longue.";
#elif L==6
	"El texto a buscar debe ser m&aacute;s largo.";	// Okoteve traduccin
#elif L==7
	"Il testo di ricerca deve essere pi&ugrave; lungo.";
#elif L==8
	"Tekst wyszukiwania powinien by&cacute; d&lstrok;u&zdot;szy.";
#elif L==9
	"O texto de pesquisa deve ser mais longo.";
#endif

const char *Txt_The_session_has_been_closed =	// Closed by user
#if   L==1
	"La sessi&oacute; ha finalitzat.";
#elif L==2
	"Die Sitzung wurde beendet.";
#elif L==3
	"The session has been closed.";
#elif L==4
	"La sesi&oacute;n ha finalizado.";
#elif L==5
	"La session a &eacute;t&eacute; ferm&eacute;e.";
#elif L==6
	"La sesi&oacute;n ha finalizado.";	// Okoteve traduccin
#elif L==7
	"La sessione &egrave; stata chiusa.";
#elif L==8
	"Sesja wygas&lstrok;a.";
#elif L==9
	"A sess&atilde;o foi encerrada.";
#endif

const char *Txt_The_session_has_expired_due_to_inactivity =	// Expired due to timeout
#if   L==1
	"La sessi&oacute; ha expirat per inactivitat.";
#elif L==2
	"Die Sitzung wurde wegen Inaktivit&uml;t abgelaufen.";
#elif L==3
	"The session has expired due to inactivity.";
#elif L==4
	"La sesi&oacute;n ha expirado por inactividad.";
#elif L==5
	"La session a expir&eacute; pour cause d'inactivit&eacute;.";
#elif L==6
	"La sesi&oacute;n ha expirado por inactividad.";	// Okoteve traduccin
#elif L==7
	"La sessione &egrave; scaduta per inattivit&agrave;.";
#elif L==8
	"Sesja wygas&lstrok;a z powodu bezczynno&sacute;ci.";
#elif L==9
	"A sess&atilde;o expirou devido &agrave; inatividade.";
#endif

const char *Txt_The_status_of_the_centre_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El estado del centro <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2
	"The status of the centre <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==3
	"The status of the centre <strong>%s</strong> has changed.";
#elif L==4
	"El estado del centro <strong>%s</strong> ha cambiado.";
#elif L==5
	"The status of the centre <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==6
	"El estado del centro <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il stato del centro <strong>%s</strong> &egrave; cambiato.";
#elif L==8
	"The status of the centre <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The status of the centre <strong>%s</strong> has changed.";	// Necessita de traduo
#endif

const char *Txt_The_status_of_the_course_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El estado de la asignatura <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2
	"The status of the course <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==3
	"The status of the course <strong>%s</strong> has changed.";
#elif L==4
	"El estado de la asignatura <strong>%s</strong> ha cambiado.";
#elif L==5
	"The status of the course <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==6
	"El estado de la asignatura <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il stato del corso <strong>%s</strong> &egrave; cambiato.";
#elif L==8
	"The status of the course <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The status of the course <strong>%s</strong> has changed.";	// Necessita de traduo
#endif

const char *Txt_The_status_of_the_degree_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El estado de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2
	"The status of the degree <strong>%s</strong> has changed.";		// Need bersetzung
#elif L==3
	"The status of the degree <strong>%s</strong> has changed.";
#elif L==4
	"El estado de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==5
	"The status of the degree <strong>%s</strong> has changed.";		// Besoin de traduction
#elif L==6
	"El estado de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il stato della laurea <strong>%s</strong> &egrave; cambiato.";
#elif L==8
	"The status of the degree <strong>%s</strong> has changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The status of the degree <strong>%s</strong> has changed.";		// Necessita de traduo
#endif

const char *Txt_The_status_of_the_institution_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El estado de la instituci&oacute;n <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2
	"The status of the institution <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==3
	"The status of the institution <strong>%s</strong> has changed.";
#elif L==4
	"El estado de la instituci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==5
	"The status of the institution <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==6
	"El estado de la instituci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il stato della istituzione <strong>%s</strong> &egrave; cambiato.";
#elif L==8
	"The status of the institution <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The status of the institution <strong>%s</strong> has changed.";	// Necessita de traduo
#endif

const char *Txt_The_survey_has_been_modified =
#if   L==1
	"La encuesta ha sido modificada.";	// Necessita traduccio
#elif L==2
	"Die Umfrage wurde ge&auml;ndert.";
#elif L==3
	"The survey has been modified.";
#elif L==4
	"La encuesta ha sido modificada.";
#elif L==5
	"Le sondage a &eacute;t&eacute; modifi&eacute;.";
#elif L==6
	"La encuesta ha sido modificada.";	// Okoteve traduccin
#elif L==7
	"Il sondaggio &egrave; stato modificato.";
#elif L==8
	"The survey has been modified.";	// Potrzebujesz tlumaczenie
#elif L==9
	"O inqu&eacute;rito foi modificado.";
#endif

const char *Txt_The_tag_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El descriptor <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Need bersetzung
#elif L==3
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El descriptor <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Besoin de traduction
#elif L==6
	"El descriptor <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"L'etichetta <strong>%s</strong> &egrave; stata rinominata come <strong>%s</strong>.";
#elif L==8
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The tag <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_tag_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El descriptor <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The tag <strong>%s</strong> has not changed.";		// Need bersetzung
#elif L==3
	"The tag <strong>%s</strong> has not changed.";
#elif L==4
	"El descriptor <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The tag <strong>%s</strong> has not changed.";		// Besoin de traduction
#elif L==6
	"El descriptor <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"L'etichetta <strong>%s</strong> non &egrave; cambiata.";
#elif L==8
	"The tag <strong>%s</strong> has not changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The tag <strong>%s</strong> has not changed.";		// Necessita de traduo
#endif

const char *Txt_The_test_X_has_already_been_assessed_previously =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"El test %u ya ha sido evaluado anteriormente.";	// Necessita traduccio
#elif L==2
	"The test %u has already been assessed previously.";	// Need bersetzung
#elif L==3
	"The test %u has already been assessed previously.";
#elif L==4
	"El test %u ya ha sido evaluado anteriormente.";
#elif L==5
	"The test %u has already been assessed previously.";	// Besoin de traduction
#elif L==6
	"El test %u ya ha sido evaluado anteriormente.";	// Okoteve traduccin
#elif L==7
	"Il test %u &egrave; stato valutato precedentemente.";
#elif L==8
	"The test %u has already been assessed previously.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The test %u has already been assessed previously.";	// Necessita de traduo
#endif

const char *Txt_The_test_configuration_has_been_updated =
#if   L==1
	"La configuraci&oacute;n de tests se ha actualizado.";	// Necessita traduccio
#elif L==2
	"The test configuration has been updated.";		// Need bersetzung
#elif L==3
	"The test configuration has been updated.";
#elif L==4
	"La configuraci&oacute;n de tests se ha actualizado.";
#elif L==5
	"The test configuration has been updated.";		// Besoin de traduction
#elif L==6
	"La configuraci&oacute;n de tests se ha actualizado.";	// Okoteve traduccin
#elif L==7
	"La configurazione di test &egrave; stata aggiornata.";
#elif L==8
	"The test configuration has been updated.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The test configuration has been updated.";		// Necessita de traduo
#endif

const char *Txt_The_thread_is_already_in_this_forum =
#if   L==1
	"La discusi&oacute;n ya est&aacute; en este foro.";	// Necessita traduccio
#elif L==2
	"The thread is already in this forum.";			// Need bersetzung
#elif L==3
	"The thread is already in this forum.";
#elif L==4
	"La discusi&oacute;n ya est&aacute; en este foro.";
#elif L==5
	"The thread is already in this forum.";			// Besoin de traduction
#elif L==6
	"La discusi&oacute;n ya est&aacute; en este foro.";	// Okoteve traduccin
#elif L==7
	"La discussione &egrave; gi&agrave; in questo forum.";
#elif L==8
	"The thread is already in this forum.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The thread is already in this forum.";			// Necessita de traduo
#endif

const char *Txt_The_thread_X_is_already_in_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La discusi&oacute;n <strong>%s</strong> ya est&aacute; en este foro.";	// Necessita traduccio
#elif L==2
	"The thread <strong>%s</strong> is already in this forum.";		// Need bersetzung
#elif L==3
	"The thread <strong>%s</strong> is already in this forum.";
#elif L==4
	"La discusi&oacute;n <strong>%s</strong> ya est&aacute; en este foro.";
#elif L==5
	"The thread <strong>%s</strong> is already in this forum.";		// Besoin de traduction
#elif L==6
	"La discusi&oacute;n <strong>%s</strong> ya est&aacute; en este foro.";	// Okoteve traduccin
#elif L==7
	"La discussione <strong>%s</strong> &egrave; gi&agrave; in questo forum.";
#elif L==8
	"The thread <strong>%s</strong> is already in this forum.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The thread <strong>%s</strong> is already in this forum.";		// Necessita de traduo
#endif

const char *Txt_The_type_of_degree_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";	// Necessita traduccio
#elif L==2
	"Die Abschlussart <strong>%s</strong> wurde bereits definiert.";
#elif L==3
	"The type of degree <strong>%s</strong> already exists.";
#elif L==4
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";
#elif L==5
	"The type of degree <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"El tipo de titulaci&oacute;n <strong>%s</strong> ya existe.";	// Okoteve traduccin
#elif L==7
	"Il tipo di laurea <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The type of degree <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The type of degree <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_type_of_degree_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El tipo de titulaci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Need bersetzung
#elif L==3
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El tipo de titulaci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Besoin de traduction
#elif L==6
	"El tipo de titulaci&oacute;n <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il tipo di laurea <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The type of degree <strong>%s</strong> has been renamed as <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_The_type_of_degree_of_the_degree_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El tipo de titulaci&oacute;n de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2
	"The type of degree of the degree <strong>%s</strong> has changed.";				// Need bersetzung
#elif L==3
	"The type of degree of the degree <strong>%s</strong> has changed.";
#elif L==4
	"El tipo de titulaci&oacute;n de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";
#elif L==5
	"The type of degree of the degree <strong>%s</strong> has changed.";				// Besoin de traduction
#elif L==6
	"El tipo de titulaci&oacute;n de la titulaci&oacute;n <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il tipo di laurea <strong>%s</strong> &egrave; cambiato.";
#elif L==8
	"The type of degree of the degree <strong>%s</strong> has changed.";				// Potrzebujesz tlumaczenie
#elif L==9
	"The type of degree of the degree <strong>%s</strong> has changed.";				// Necessita de traduo
#endif

const char *Txt_The_type_of_group_X_already_exists =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El tipus de grup <strong>%s</strong> ja existeix.";
#elif L==2
	"Der Gruppen-Typ <strong>%s</strong> wurde bereits definiert.";
#elif L==3
	"The type of group <strong>%s</strong> already exists.";
#elif L==4
	"El tipo de grupo <strong>%s</strong> ya existe.";
#elif L==5
	"The type of group <strong>%s</strong> already exists.";	// Besoin de traduction
#elif L==6
	"El tipo de grupo <strong>%s</strong> ya existe.";		// Okoteve traduccin
#elif L==7
	"Il tipo di gruppo <strong>%s</strong> gi&agrave; esiste.";
#elif L==8
	"The type of group <strong>%s</strong> already exists.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The type of group <strong>%s</strong> already exists.";	// Necessita de traduo
#endif

const char *Txt_The_type_of_group_X_has_been_renamed_as_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"El tipo de grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";
#elif L==4
	"El tipo de grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";
#elif L==5
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"El tipo de grupo <strong>%s</strong> ha pasado a denominarse <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Il tipo di gruppo <strong>%s</strong> &egrave; stato rinominato come <strong>%s</strong>.";
#elif L==8
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The type of group <strong>%s</strong> has been renamed as <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_The_type_of_group_of_the_group_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El tipo de grupo del grupo <strong>%s</strong> ha cambiado.";		// Necessita traduccio
#elif L==2
	"The type of group of the group <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==3
	"The type of group of the group <strong>%s</strong> has changed.";
#elif L==4
	"El tipo de grupo del grupo <strong>%s</strong> ha cambiado.";
#elif L==5
	"The type of group of the group <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==6
	"El tipo de grupo del grupo <strong>%s</strong> ha cambiado.";		// Okoteve traduccin
#elif L==7
	"Il tipo di gruppo del gruppo <strong>%s</strong> &egrave; cambiato.";
#elif L==8
	"The type of group of the group <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The type of group of the group <strong>%s</strong> has changed.";	// Necessita de traduo
#endif

const char *Txt_The_type_of_enrollment_of_the_type_of_group_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Necessita traduccio
#elif L==2
	"The type of enrollment of the type of group <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3
	"The type of enrollment of the type of group <strong>%s</strong> has not changed.";
#elif L==4
	"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The type of enrollment of the type of group <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6
	"El tipo de adscripci&oacute;n del tipo de grupo <strong>%s</strong> no ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il tipo di registrazione del tipo di gruppo <strong>%s</strong> non &egrave; cambiato.";
#elif L==8
	"The type of enrollment of the type of group <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The type of enrollment of the type of group <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_type_of_the_holiday_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El tipo de la festividad <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2
	"The type of the holiday <strong>%s</strong> has changed.";	// Need bersetzung
#elif L==3
	"The type of the holiday <strong>%s</strong> has changed.";
#elif L==4
	"El tipo de la festividad <strong>%s</strong> ha cambiado.";
#elif L==5
	"The type of the holiday <strong>%s</strong> has changed.";	// Besoin de traduction
#elif L==6
	"El tipo de la festividad <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7
	"Il tipo di festivit&agrave; <strong>%s</strong> &egrave; cambiato.";
#elif L==8
	"The type of the holiday <strong>%s</strong> has changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The type of the holiday <strong>%s</strong> has changed.";	// Necessita de traduo
#endif

const char *Txt_The_URL_X_has_been_updated =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El URL <strong>%s</strong> se ha actualizado.";	// Necessita traduccio
#elif L==2
	"The URL <strong>%s</strong> has been updated.";	// Need bersetzung
#elif L==3
	"The URL <strong>%s</strong> has been updated.";
#elif L==4
	"El URL <strong>%s</strong> se ha actualizado.";
#elif L==5
	"The URL <strong>%s</strong> has been updated.";	// Besoin de traduction
#elif L==6
	"El URL <strong>%s</strong> se ha actualizado.";	// Okoteve traduccin
#elif L==7
	"L'URL <strong>%s</strong> &egrave; stata archiviata.";
#elif L==8
	"The URL <strong>%s</strong> has been updated.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The URL <strong>%s</strong> has been updated.";	// Necessita de traduo
#endif

const char *Txt_The_URL_web_address_that_gives_direct_access_to_the_course_X_is =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El URL (direcci&oacute;n web) que permite acceder directamente"
	" a la asignatura <strong>%s</strong> es:";		// Necessita traduccio
#elif L==2
	"The URL (web address) that gives direct access"
	" to the course <strong>%s</strong> is:";		// Need bersetzung
#elif L==3
	"The URL (web address) that gives direct access"
	" to the course <strong>%s</strong> is:";
#elif L==4
	"El URL (direcci&oacute;n web) que permite acceder directamente"
	" a la asignatura <strong>%s</strong> es:";
#elif L==5
	"The URL (web address) that gives direct access"
	" to the course <strong>%s</strong> is:";		// Besoin de traduction
#elif L==6
	"El URL (direcci&oacute;n web) que permite acceder directamente"
	" a la asignatura <strong>%s</strong> es:";		// Okoteve traduccin
#elif L==7
	"L'URL (indirizzo web) che permette l'accesso diretto"
	" al corso <strong>%s</strong> &egrave;:";
#elif L==8
	"The URL (web address) that gives direct access"
	" to the course <strong>%s</strong> is:";		// Potrzebujesz tlumaczenie
#elif L==9
	"The URL (web address) that gives direct access"
	" to the course <strong>%s</strong> is:";		// Necessita de traduo
#endif

const char *Txt_The_user_does_not_exist_or_password_is_incorrect =
#if   L==1
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";	// Necessita traduccio
#elif L==2
	"Die Benutzer ist nicht vorhanden"
	" oder Passwort falsch ist.";
#elif L==3
	"The user does not exist"
	" or password is incorrect.";
#elif L==4
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";
#elif L==5
	"L'utilisateur n'existe pas"
	" ou le mot de passe est incorrect.";
#elif L==6
	"El usuario/a no existe"
	" o bien la contrase&ntilde;a es incorrecta.";	// Okoteve traduccin
#elif L==7
	"Il utente non esiste"
	" o la password non &egrave; corretta.";
#elif L==8
	"The user does not exist"
	" or password is incorrect.";			// Potrzebujesz tlumaczenie
#elif L==9
	"O utilizador n&atilde;o existe"
	" ou a senha est&aacute; incorreta.";
#endif

const char *Txt_THE_USER_X_already_exists_in_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"<strong>%s</strong> ya existe en %s.";		// Necessita traduccio
#elif L==2
	"<strong>%s</strong> already exists in %s.";	// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> already exists in %s.";
#elif L==4
	"<strong>%s</strong> ya existe en %s.";
#elif L==5
	"<strong>%s</strong> already exists in %s.";	// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ya existe en %s.";		// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> esiste gi&agrave; in %s.";
#elif L==8
	"<strong>%s</strong> already exists in %s.";	// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> already exists in %s.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_already_exists_in_Y_but_is_not_yet_enrolled_in_the_course_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1
	"<strong>%s</strong> ya existe en %s,"
	" pero a&uacute;n no est&aacute; inscrito"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> already exists in %s,"
	" but is not yet enrolled"
	" in the course <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"<strong>%s</strong> already exists in %s,"
	" but is not yet enrolled"
	" in the course <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> ya existe en %s,"
	" pero a&uacute;n no est&aacute; inscrito"
	" en la asignatura <strong>%s</strong>.";
#elif L==5
	"<strong>%s</strong> already exists in %s,"
	" but is not yet enrolled"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ya existe en %s,"
	" pero a&uacute;n no est&aacute; inscrito"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> esiste gi&agrave; in %s,"
	" ma non &egrave; ancora registrato"
	" al corso <strong>%s</strong>.";
#elif L==8
	"<strong>%s</strong> already exists in %s,"
	" but is not yet enrolled"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> already exists in %s,"
	" but is not yet enrolled"
	" in the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_enrolled_as_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" como administrador/a de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has been enrolled"
	" as administrator of <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has been enrolled"
	" as administrator of <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" como administrador/a de <strong>%s</strong>.";
#elif L==5
	"<strong>%s</strong> has been enrolled"
	" as administrator of <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" como administrador/a de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato registrato"
	" come amministratore de <strong>%s</strong>.";
#elif L==8
	"<strong>%s</strong> has been enrolled"
	" as administrator of <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has been enrolled"
	" as administrator of <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_enrolled_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"<strong>%s</strong> ha estat afegit/da"
	" a l'assignatura <strong>%s</strong>.";
#elif L==2
	"<strong>%s</strong> hat sich im"
	" im Kurs <strong>%s</strong> eingeschrieben.";
#elif L==3
	"<strong>%s</strong> has been enrolled"
	" in the course <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" a la asignatura <strong>%s</strong>.";
#elif L==5
	"<strong>%s</strong> a &eacute;t&eacute; inscrit"
	" dans la mati&egrave;re <strong>%s</strong>.";
#elif L==6
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" a la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato registrato/a"
	" nel corso <strong>%s</strong>.";
#elif L==8
	"<strong>%s</strong> zosta&lstrok; zapisany"
	"  do kursu <strong>%s</strong>.";
#elif L==9
	"<strong>%s</strong> foi matriculado/a"
	" na disciplina <strong>%s</strong>.";
#endif

const char *Txt_THE_USER_X_has_been_enrolled_in_the_group_of_type_Y_Z =	// Warning: it is very important to include three %s in the following sentences
#if   L==1
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";		// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has been enrolled"
	" in the group of type %s <strong>%s</strong>.";	// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has been enrolled"
	" in the group of type %s <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";
#elif L==5
	"<strong>%s</strong> has been enrolled"
	" in the group of type %s <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ha sido a&ntilde;adido/a"
	" al grupo de tipo %s <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato registrato"
	" nel gruppo di tipo %s <strong>%s</strong>.";
#elif L==8
	"<strong>%s</strong> has been enrolled"
	" in the group of type %s <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has been enrolled"
	" in the group of type %s <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_as_administrator =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"<strong>%s</strong> ha sido eliminado/a como administrador/a.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has been removed as administrator.";	// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has been removed as administrator.";
#elif L==4
	"<strong>%s</strong> ha sido eliminado/a como administrador/a.";
#elif L==5
	"<strong>%s</strong> has been removed as administrator.";	// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ha sido eliminado/a como administrador/a.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato rimosso come amministratore.";
#elif L==8
	"<strong>%s</strong> has been removed as administrator.";	// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has been removed as administrator.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_all_groups_in_all_courses =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de todas sus asignaturas.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has been removed"
	" from all groups in all courses.";			// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has been removed"
	" from all groups in all courses.";
#elif L==4
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de todas sus asignaturas.";
#elif L==5
	"<strong>%s</strong> has been removed"
	" from all groups in all courses.";			// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de todas sus asignaturas.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato rimosso"
	" da tutti i gruppi di tutti i corsi.";
#elif L==8
	"<strong>%s</strong> has been removed"
	" from all groups in all courses.";			// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has been removed"
	" from all groups in all courses.";			// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_all_groups_of_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has been removed"
	" from all groups of the course <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has been removed"
	" from all groups of the course <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de la asignatura <strong>%s</strong>.";
#elif L==5
	"<strong>%s</strong> has been removed"
	" from all groups of the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ha sido eliminado/a"
	" de todos los grupos de la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato rimosso"
	" da tutti i gruppi del corso <strong>%s</strong>.";
#elif L==8
	"<strong>%s</strong> has been removed"
	" from all groups of the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has been removed"
	" from all groups of the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_all_his_her_courses =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has been removed from all her/his courses.";	// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has been removed from all her/his courses.";
#elif L==4
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";
#elif L==5
	"<strong>%s</strong> has been removed from all her/his courses.";	// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ha sido eliminado/a de todas sus asignaturas.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato rimosso da tutti i suoi corsi.";
#elif L==8
	"<strong>%s</strong> has been removed from all her/his courses.";	// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has been removed from all her/his courses.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_Y_groups =	// Warning: it is very important to include %s and %u in the following sentences
#if   L==1
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has been removed from %u groups.";		// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has been removed from %u groups.";
#elif L==4
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";
#elif L==5
	"<strong>%s</strong> has been removed from %u groups.";		// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ha sido eliminado/a de %u grupos.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato rimosso da %u gruppi.";
#elif L==8
	"<strong>%s</strong> has been removed from %u groups.";		// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has been removed from %u groups.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_one_group =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has been removed from one group.";	// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has been removed from one group.";
#elif L==4
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";
#elif L==5
	"<strong>%s</strong> has been removed from one group.";	// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ha sido eliminado/a de un grupo.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato rimosso da un gruppo.";
#elif L==8
	"<strong>%s</strong> has been removed from one group.";	// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has been removed from one group.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"<strong>%s</strong> ha sido eliminado/a"
	" de la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";	// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> ha sido eliminado/a"
	" de la asignatura <strong>%s</strong>.";
#elif L==5
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ha sido eliminado/a"
	" de la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato rimosso"
	" dal corso <strong>%s</strong>.";
#elif L==8
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has been removed"
	" from the course <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_as_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"<strong>%s</strong> ha sido eliminado/a como administrador"
	" de <strong>%s</strong>.";		// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> ha sido eliminado/a como administrador"
	" de <strong>%s</strong>.";
#elif L==5
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ha sido eliminado/a como administrador"
	" de <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato rimosso come amministratore"
	" de <strong>%s</strong>.";
#elif L==8
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has been removed as administrator"
	" of <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_been_removed_from_the_group_of_type_Y_to_which_it_belonged =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";
#elif L==4
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";
#elif L==5
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ha sido eliminado/a"
	" del grupo de tipo %s al que pertenec&iacute;a.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; stato rimosso"
	" dal gruppo di tipo %s al quale apparteneva.";
#elif L==8
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has been removed"
	" from the group of type %s to which it belonged.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_has_not_been_removed_from_any_group =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> has not been removed"
	" from any group.";		// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> has not been removed"
	" from any group.";
#elif L==4
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";
#elif L==5
	"<strong>%s</strong> has not been removed"
	" from any group.";		// Besoin de traduction
#elif L==6
	"<strong>%s</strong> no ha sido eliminado/a"
	" de ning&uacute;n grupo.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> non &egrave; stato rimosso"
	" da alcun gruppo.";
#elif L==8
	"<strong>%s</strong> has not been removed"
	" from any group.";		// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> has not been removed"
	" from any group.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_is_already_an_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"<strong>%s</strong> ya es administrador/a de <strong>%s</strong>.";		// Necessita traduccio
#elif L==2
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";	// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> ya es administrador/a de <strong>%s</strong>.";
#elif L==5
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ya es administrador/a de <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; gi&agrave; un amministratore de <strong>%s</strong>.";
#elif L==8
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> is already an administrator of <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_THE_USER_X_is_already_in_the_course_Y_but_has_not_yet_accepted_the_enrollment =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"<strong>%s</strong> ya est&aacute;"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrollment.";			// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrollment.";
#elif L==4
	"<strong>%s</strong> ya est&aacute;"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";
#elif L==5
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrollment.";			// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ya est&aacute;"
	" en la asignatura <strong>%s</strong>,"
	" pero a&uacute;n no ha aceptado la inscripci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; gi&agrave;"
	" nel corso <strong>%s</strong>,"
	" ma non ha ancora accettato la registrazione.";
#elif L==8
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrollment.";			// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> is already"
	" in the course <strong>%s</strong>,"
	" but has not yet accepted the enrollment.";			// Necessita de traduo
#endif

const char *Txt_THE_USER_X_is_already_enrolled_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"<strong>%s</strong> ya est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> is already enrolled"
	" in the course <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> is already enrolled"
	" in the course <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> ya est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";
#elif L==5
	"<strong>%s</strong> is already enrolled"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"<strong>%s</strong> ya est&aacute; inscrito/a"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> &egrave; gi&agrave; registrato"
	" nel corso <strong>%s</strong>.";
#elif L==8
	"<strong>%s</strong> is already enrolled"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> is already enrolled"
	" in the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_The_user_is_new_not_yet_in_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"L'usuari &eacute;s nou, encara no existeix en %s.";
#elif L==2
	"Der Benutzer ist neu, noch nicht in %s.";
#elif L==3
	"The user is new, not yet in %s.";
#elif L==4
	"El usuario/a es nuevo/a, a&uacute;n no existe en %s.";
#elif L==5
	"L'utilisateur est nouveau, pas encore dans %s.";
#elif L==6
	"El usuario/a es nuevo/a, a&uacute;n no existe en %s.";	// Okoteve traduccin
#elif L==7
	"L'utente &egrave; nuovo, non esiste ancora in %s.";
#elif L==8
	"U&zdot;ytkownik jest nowa, jeszcze nie w %s.";
#elif L==9
	"O usu&aacute;rio &eacute; novo, ainda n&atilde;o existe o %s.";
#endif

const char *Txt_THE_USER_X_is_not_an_administrator_of_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"<strong>%s</strong> no es administrador/a de <strong>%s</strong>.";		// Necessita traduccio
#elif L==2
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";
#elif L==4
	"<strong>%s</strong> no es administrador/a de <strong>%s</strong>.";
#elif L==5
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"<strong>%s</strong> no es administrador/a de <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> non &egrave; un amministratore de <strong>%s</strong>.";
#elif L==8
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> is not an administrator of <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_THE_USER_X_is_not_found_in_the_file_of_marks =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No se ha encontrado a <strong>%s</strong> en el archivo de calificaciones.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong> is not found in the file of marks.";			// Need bersetzung
#elif L==3
	"<strong>%s</strong> is not found in the file of marks.";
#elif L==4
	"No se ha encontrado a <strong>%s</strong> en el archivo de calificaciones.";
#elif L==5
	"<strong>%s</strong> is not found in the file of marks.";			// Besoin de traduction
#elif L==6
	"No se ha encontrado a <strong>%s</strong> en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong> non &egrave; stato trovato nel file dei risultati.";
#elif L==8
	"<strong>%s</strong> is not found in the file of marks.";			// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong> is not found in the file of marks.";			// Necessita de traduo
#endif

const char *Txt_The_visibility_of_the_record_field_X_has_not_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La visibilidad del campo <strong>%s</strong> no ha cambiado.";			// Necessita traduccio
#elif L==2
	"The visibility of the record field <strong>%s</strong> has not changed.";	// Need bersetzung
#elif L==3
	"The visibility of the record field <strong>%s</strong> has not changed.";
#elif L==4
	"La visibilidad del campo <strong>%s</strong> no ha cambiado.";
#elif L==5
	"The visibility of the record field <strong>%s</strong> has not changed.";	// Besoin de traduction
#elif L==6
	"La visibilidad del campo <strong>%s</strong> no ha cambiado.";			// Okoteve traduccin
#elif L==7
	"La visibilit&agrave; del campo di scheda <strong>%s</strong> non &egrave; cambiata.";
#elif L==8
	"The visibility of the record field <strong>%s</strong> has not changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The visibility of the record field <strong>%s</strong> has not changed.";	// Necessita de traduo
#endif

const char *Txt_The_visibility_of_your_photo_has_changed =
#if   L==1
	"La visibilidad de su foto ha cambiado.";	// Necessita traduccio
#elif L==2
	"The visibility of your photo has changed.";	// Need bersetzung
#elif L==3
	"The visibility of your photo has changed.";
#elif L==4
	"La visibilidad de su foto ha cambiado.";
#elif L==5
	"The visibility of your photo has changed.";	// Besoin de traduction
#elif L==6
	"La visibilidad de su foto ha cambiado.";	// Okoteve traduccin
#elif L==7
	"The visibility of your photo has changed.";	// Bisogno di traduzione
#elif L==8
	"The visibility of your photo has changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The visibility of your photo has changed.";	// Necessita de traduo
#endif

const char *Txt_The_visibility_of_your_public_profile_has_changed =
#if   L==1
	"La visibilidad de su perfil p&uacute;blico ha cambiado.";	// Necessita traduccio
#elif L==2
	"The visibility of your public profile has changed.";	// Need bersetzung
#elif L==3
	"The visibility of your public profile has changed.";
#elif L==4
	"La visibilidad de su perfil p&uacute;blico ha cambiado.";
#elif L==5
	"The visibility of your public profile has changed.";	// Besoin de traduction
#elif L==6
	"La visibilidad de su perfil p&uacute;blico ha cambiado.";	// Okoteve traduccin
#elif L==7
	"The visibility of your public profile has changed.";	// Bisogno di traduzione
#elif L==8
	"The visibility of your public profile has changed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The visibility of your public profile has changed.";	// Necessita de traduo
#endif

const char *Txt_The_whole_course =
#if   L==1
	"Toda la asignatura";		// Necessita traduccio
#elif L==2
	"Die gesamte Kurs";
#elif L==3
	"The whole course";
#elif L==4
	"Toda la asignatura";
#elif L==5
	"L'ensemble de la mati&egrave;re";
#elif L==6
	"Toda la asignatura";		// Okoteve traduccin
#elif L==7
	"L'intero corso";
#elif L==8
	"The whole course";		// Potrzebujesz tlumaczenie
#elif L==9
	"Toda a disciplina";
#endif

const char *Txt_The_works_of_X_have_been_removed_in_a_total_of_Y_of_his_her_Z_courses =	// Warning: it is very important to include all %s and %u, in the correct order, in the following sentences
#if   L==1
	"Se han eliminado todos los trabajos"
	" de <strong>%s</strong>"
	" en un total de %u de sus %u asignaturas.";			// Necessita traduccio
#elif L==2
	"The works of <strong>%s</strong>"
	" have been removed in a total of %u of her/his %u courses.";	// Need bersetzung
#elif L==3
	"The works of <strong>%s</strong>"
	" have been removed in a total of %u of her/his %u courses.";
#elif L==4
	"Se han eliminado todos los trabajos"
	" de <strong>%s</strong>"
	" en un total de %u de sus %u asignaturas.";
#elif L==5
	"The works of <strong>%s</strong>"
	" have been removed in a total of %u of her/his %u courses.";	// Besoin de traduction
#elif L==6
	"Se han eliminado todos los trabajos"
	" de <strong>%s</strong>"
	" en un total de %u de sus %u asignaturas.";			// Okoteve traduccin
#elif L==7
	"I lavori de <strong>%s</strong>"
	" sono stati rimossi in un totale di %u dei suoi corsi.";
#elif L==8
	"The works of <strong>%s</strong>"
	" have been removed in a total of %u of her/his %u courses.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The works of <strong>%s</strong>"
	" have been removed in a total of %u of her/his %u courses.";	// Necessita de traduo
#endif

const char *Txt_The_year_X_is_not_allowed =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"El a&ntilde;o/semestre <strong>%u</strong> no est&aacute; permitido.";		// Necessita traduccio
#elif L==2
	"The year/semester <strong>%u</strong> is not enabled.";			// Need bersetzung
#elif L==3
	"The year/semester <strong>%u</strong> is not enabled.";
#elif L==4
	"El a&ntilde;o/semestre <strong>%u</strong> no est&aacute; permitido.";
#elif L==5
	"The year/semester <strong>%u</strong> is not enabled.";			// Besoin de traduction
#elif L==6
	"El a&ntilde;o/semestre <strong>%u</strong> no est&aacute; permitido.";		// Okoteve traduccin
#elif L==7
	"L'anno/semester <strong>%u</strong> non &egrave; consentito.";
#elif L==8
	"The year/semester <strong>%u</strong> is not allowed.";			// Potrzebujesz tlumaczenie
#elif L==9
	"The year/semester <strong>%u</strong> is not allowed.";			// Necessita de traduo
#endif

const char *Txt_The_year_of_the_course_X_has_changed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"El a&ntilde;o/semestre de la asignatura <strong>%s</strong> ha cambiado.";	// Necessita traduccio
#elif L==2
	"The year/semester of the couse <strong>%s</strong> has changed.";		// Need bersetzung
#elif L==3
	"The year/semester of the couse <strong>%s</strong> has changed.";
#elif L==4
	"El a&ntilde;o/semestre de la asignatura <strong>%s</strong> ha cambiado.";
#elif L==5
	"The year/semester of the couse <strong>%s</strong> has changed.";		// Besoin de traduction
#elif L==6
	"El a&ntilde;o/semestre de la asignatura <strong>%s</strong> ha cambiado.";	// Okoteve traduccin
#elif L==7
	"L'anno/semestre del corso <strong>%s</strong> &egrave; cambiato.";
#elif L==8
	"The year/semestre of the couse <strong>%s</strong> has changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"The year/semestre of the couse <strong>%s</strong> has changed.";		// Necessita de traduo
#endif

const char *Txt_The_ZIP_file_has_been_received_successfully =
#if   L==1
	"El archivo <em>ZIP</em> se ha recibido correctamente.";	// Necessita traduccio
#elif L==2
	"The <em>ZIP</em> file has been received successfully.";	// Need bersetzung
#elif L==3
	"The <em>ZIP</em> file has been received successfully.";
#elif L==4
	"El archivo <em>ZIP</em> se ha recibido correctamente.";
#elif L==5
	"The <em>ZIP</em> file has been received successfully.";	// Besoin de traduction
#elif L==6
	"El archivo <em>ZIP</em> se ha recibido correctamente.";	// Okoteve traduccin
#elif L==7
	"Il <em>ZIP</em> file &egrave; stato ricevuto con successo.";
#elif L==8
	"The <em>ZIP</em> file has been received successfully.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The <em>ZIP</em> file has been received successfully.";	// Necessita de traduo
#endif

const char *Txt_The_ZIP_file_has_been_unzipped_successfully =
#if   L==1
	"El archivo <em>ZIP</em> se ha descomprimido correctamente.";	// Necessita traduccio
#elif L==2
	"The <em>ZIP</em> file has been unzipped successfully.";	// Need bersetzung
#elif L==3
	"The <em>ZIP</em> file has been unzipped successfully.";
#elif L==4
	"El archivo <em>ZIP</em> se ha descomprimido correctamente.";
#elif L==5
	"The <em>ZIP</em> file has been unzipped successfully.";	// Besoin de traduction
#elif L==6
	"El archivo <em>ZIP</em> se ha descomprimido correctamente.";	// Okoteve traduccin
#elif L==7
	"Il <em>ZIP</em> file &egrave; stato decompresso con successo.";
#elif L==8
	"The <em>ZIP</em> file has been unzipped successfully.";	// Potrzebujesz tlumaczenie
#elif L==9
	"The <em>ZIP</em> file has been unzipped successfully.";	// Necessita de traduo
#endif

const char *Txt_Theme_SKIN =
#if   L==1
	"Tema";
#elif L==2
	"Skin";
#elif L==3
	"Theme";
#elif L==4
	"Tema";
#elif L==5
	"Th&egrave;me";
#elif L==6
	"Tema";	// Okoteve traduccin
#elif L==7
	"Tema";
#elif L==8
	"Temat";
#elif L==9
	"Skin";
#endif

const char *Txt_There_are_more_than_one_table_in_the_file_of_marks =
#if   L==1
	"Hay m&aacute;s de una tabla en el archivo de calificaciones.";	// Necessita traduccio
#elif L==2
	"There are more than one table in the file of marks.";		// Need bersetzung
#elif L==3
	"There are more than one table in the file of marks.";
#elif L==4
	"Hay m&aacute;s de una tabla en el archivo de calificaciones.";
#elif L==5
	"There are more than one table in the file of marks.";		// Besoin de traduction
#elif L==6
	"Hay m&aacute;s de una tabla en el archivo de calificaciones.";	// Okoteve traduccin
#elif L==7
	"C'&egrave; pi&ugrave; di una tabella nel file dei risultati.";
#elif L==8
	"There are more than one table in the file of marks.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are more than one table in the file of marks.";		// Necessita de traduo
#endif

const char *Txt_There_are_more_than_one_user_with_the_ID_X_Please_type_a_nick_or_email =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Hay m&aacute;s de un usuario con el ID <strong>%s</strong>."
	" Por favor, escriba un @apodo o correo.";	// Necessita traduccio
#elif L==2
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or email.";		// Need bersetzung
#elif L==3
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or email.";
#elif L==4
	"Hay m&aacute;s de un usuario con el ID <strong>%s</strong>."
	" Por favor, escriba un @apodo o correo.";
#elif L==5
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or email.";		// Besoin de traduction
#elif L==6
	"Hay m&aacute;s de un usuario con el ID <strong>%s</strong>."
	" Por favor, escriba un @apodo o correo.";	// Okoteve traduccin
#elif L==7
	"Ci sono pi&ugrave; di un utente con l'ID <strong>%s</strong>."
	" Si prega di digitare un @nome-utente o email.";
#elif L==8
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or email.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are more than one user with the ID <strong>%s</strong>."
	" Please type a @nick or email.";		// Necessita de traduo
#endif

const char *Txt_There_are_new_posts =
#if   L==1
	"Hi ha missatges nous";
#elif L==2
	"Es gibt neue Nachrichten";
#elif L==3
	"There are new posts";
#elif L==4
	"Hay mensajes nuevos";
#elif L==5
	"Il y a des nouveaux messages";
#elif L==6
	"Hay mensajes nuevos";	// Okoteve traduccin
#elif L==7
	"Ci sono nuovi post";
#elif L==8
	"Istnieje nowych post&oacute;w";
#elif L==9
	"H&aacute; novas posts";
#endif

const char *Txt_There_are_no_accesses_with_the_selected_search_criteria =
#if   L==1
	"No hay accesos con el criterio de b&uacute;squeda seleccionado.";	// Necessita traduccio
#elif L==2
	"There are no accesses with the selected search criteria.";		// Need bersetzung
#elif L==3
	"There are no accesses with the selected search criteria.";
#elif L==4
	"No hay accesos con el criterio de b&uacute;squeda seleccionado.";
#elif L==5
	"There are no accesses with the selected search criteria.";		// Besoin de traduction
#elif L==6
	"No hay accesos con el criterio de b&uacute;squeda seleccionado.";	// Okoteve traduccin
#elif L==7
	"Non ci sono accessi con il criterio di ricerca selezionato.";
#elif L==8
	"There are no accesses with the selected search criteria.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are no accesses with the selected search criteria.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_banners =
#if   L==1
	"No existen banners.";		// Necessita traduccio
#elif L==2
	"There are no banners.";	// Need bersetzung
#elif L==3
	"There are no banners.";
#elif L==4
	"No existen banners.";
#elif L==5
	"There are no banners.";	// Besoin de traduction
#elif L==6
	"No existen banners.";		// Okoteve traduccin
#elif L==7
	"Non ci sono banners.";
#elif L==8
	"There are no banners.";	// Potrzebujesz tlumaczenie
#elif L==9
	"There are no banners.";	// Necessita de traduo
#endif

const char *Txt_There_are_no_centres_with_requests_for_degrees_to_be_confirmed =
#if   L==1
	"No hi ha centres amb sol&middot;licituds de titulacions pendents de confirmar.";
#elif L==2
	"There are no centres with requests for degrees to be confirmed.";		// Need bersetzung
#elif L==3
	"There are no centres with requests for degrees to be confirmed.";
#elif L==4
	"No hay centros con solicitudes de titulaciones pendientes de confirmar.";
#elif L==5
	"Il n'y a pas de centres avec demandes de degr&eacute;s &agrave; confirmer.";
#elif L==6
	"No hay centros con solicitudes de titulaciones pendientes de confirmar.";	// Okoteve traduccin
#elif L==7
	"Non ci sono centri con richieste di gradi pendenti de confermare.";
#elif L==8
	"There are no centres with requests for degrees to be confirmed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are no centres with requests for degrees to be confirmed.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_countries_with_requests_for_institutions_to_be_confirmed =
#if   L==1
	"No hi ha pa&iuml;sos amb sol&middot;licituds de institucions pendents de confirmar.";
#elif L==2
	"There are no countries with requests for institutions to be confirmed.";		// Need bersetzung
#elif L==3
	"There are no countries with requests for institutions to be confirmed.";
#elif L==4
	"No hay pa&iacute;ses con solicitudes de instituciones pendientes de confirmar.";
#elif L==5
	"Il n'y a pas de pays avec demandes d'&eacute;tablissements &agrave; confirmer.";
#elif L==6
	"No hay pa&iacute;ses con solicitudes de instituciones pendientes de confirmar.";	// Okoteve traduccin
#elif L==7
	"Non ci sono paesi con richieste di istituzioni pendenti de confermare.";
#elif L==8
	"There are no countries with requests for institutions to be confirmed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are no countries with requests for institutions to be confirmed.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_degrees_with_requests_for_courses_to_be_confirmed =
#if   L==1
	"No hi ha titulacions amb sol&middot;licituds d'assignatures pendents de confirmar.";
#elif L==2
	"There are no degrees with requests for courses to be confirmed.";		// Need bersetzung
#elif L==3
	"There are no degrees with requests for courses to be confirmed.";
#elif L==4
	"No hay titulaciones con solicitudes de asignaturas pendientes de confirmar.";
#elif L==5
	"Il n'y a pas de degr&eacute;s avec demandes de cours &agrave; confirmer.";
#elif L==6
	"No hay titulaciones con solicitudes de asignaturas pendientes de confirmar.";	// Okoteve traduccin
#elif L==7
	"Non ci sono gradi con richieste di corsi pendenti de confermare.";
#elif L==8
	"There are no degrees with requests for courses to be confirmed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are no degrees with requests for courses to be confirmed.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_institutions_with_requests_for_centres_to_be_confirmed =
#if   L==1
	"No hi ha institucions amb sol&middot;licituds de centres pendents de confirmar.";
#elif L==2
	"There are no institutions with requests for centres to be confirmed.";		// Need bersetzung
#elif L==3
	"There are no institutions with requests for centres to be confirmed.";
#elif L==4
	"No hay instituciones con solicitudes de centros pendientes de confirmar.";
#elif L==5
	"Il n'y a pas d'&eacute;tablissements avec demandes de centres &agrave; confirmer.";
#elif L==6
	"No hay instituciones con solicitudes de centros pendientes de confirmar.";	// Okoteve traduccin
#elif L==7
	"Non ci sono istituzioni con richieste di centri pendenti de confermare.";
#elif L==8
	"There are no institutions with requests for centres to be confirmed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are no institutions with requests for centres to be confirmed.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_departments =
#if   L==1
	"No existen departamentos.";	// Necessita traduccio
#elif L==2
	"There are no departments.";	// Need bersetzung
#elif L==3
	"There are no departments.";
#elif L==4
	"No existen departamentos.";
#elif L==5
	"There are no departments.";	// Besoin de traduction
#elif L==6
	"No existen departamentos.";	// Okoteve traduccin
#elif L==7
	"Non ci sono dipartimenti.";
#elif L==8
	"There are no departments.";	// Potrzebujesz tlumaczenie
#elif L==9
	"There are no departments.";	// Necessita de traduo
#endif

const char *Txt_There_are_no_email_domains =
#if   L==1
	"No existen dominios de correo para notificaciones.";	// Necessita traduccio
#elif L==2
	"There are no email domains for notifications.";		// Need bersetzung
#elif L==3
	"There are no email domains for notifications.";
#elif L==4
	"No existen dominios de correo para notificaciones.";
#elif L==5
	"There are no email domains for notifications.";		// Besoin de traduction
#elif L==6
	"No existen dominios de correo para notificaciones.";	// Okoteve traduccin
#elif L==7
	"Non ci sono campi mail per le notifiche.";
#elif L==8
	"There are no email domains for notifications.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are no email domains for notifications.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_holidays =
#if   L==1
	"No existen d&iacute;as festivos.";	// Necessita traduccio
#elif L==2
	"There are no holidays.";		// Need bersetzung
#elif L==3
	"There are no holidays.";
#elif L==4
	"No existen d&iacute;as festivos.";
#elif L==5
	"There are no holidays.";		// Besoin de traduction
#elif L==6
	"No existen d&iacute;as festivos.";	// Okoteve traduccin
#elif L==7
	"Non ci sono festivit&agrave;.";
#elif L==8
	"There are no holidays.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are no holidays.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_links =
#if   L==1
	"No existen enlaces.";		// Necessita traduccio
#elif L==2
	"There are no links.";		// Need bersetzung
#elif L==3
	"There are no links.";
#elif L==4
	"No existen enlaces.";
#elif L==5
	"There are no links.";		// Besoin de traduction
#elif L==6
	"No existen enlaces.";		// Okoteve traduccin
#elif L==7
	"Non ci sono links.";
#elif L==8
	"There are no links.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are no links.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_places =
#if   L==1
	"No existen lugares.";		// Necessita traduccio
#elif L==2
	"There are no places.";		// Need bersetzung
#elif L==3
	"There are no places.";
#elif L==4
	"No existen lugares.";
#elif L==5
	"There are no places.";		// Besoin de traduction
#elif L==6
	"No existen lugares.";		// Okoteve traduccin
#elif L==7
	"Non ci sono luoghi.";
#elif L==8
	"There are no places.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are no places.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_plugins =
#if   L==1
	"No existen complementos.";	// Necessita traduccio
#elif L==2
	"There are no plugins.";	// Need bersetzung
#elif L==3
	"There are no plugins.";
#elif L==4
	"No existen complementos.";
#elif L==5
	"There are no plugins.";	// Besoin de traduction
#elif L==6
	"No existen complementos.";	// Okoteve traduccin
#elif L==7
	"Non ci sono plugin.";
#elif L==8
	"There are no plugins.";	// Potrzebujesz tlumaczenie
#elif L==9
	"There are no plugins.";	// Necessita de traduo
#endif

const char *Txt_There_are_no_record_fields_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No existen campos de fichas en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"There are no record fields in the course <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"There are no record fields in the course <strong>%s</strong>.";
#elif L==4
	"No existen campos de fichas en la asignatura <strong>%s</strong>.";
#elif L==5
	"There are no record fields in the course <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"No existen campos de fichas en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Non ci sono campi di schede nel corso <strong>%s</strong>.";
#elif L==8
	"There are no record fields in the course <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"There are no record fields in the course <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_There_are_no_types_of_degree =
#if   L==1
	"No existen tipos de titulaci&oacute;n.";	// Necessita traduccio
#elif L==2
	"There are no types of degree.";		// Need bersetzung
#elif L==3
	"There are no types of degree.";
#elif L==4
	"No existen tipos de titulaci&oacute;n.";
#elif L==5
	"There are no types of degree.";		// Besoin de traduction
#elif L==6
	"No existen tipos de titulaci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Non ci sono tipi di lauree.";
#elif L==8
	"There are no types of degree.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There are no types of degree.";		// Necessita de traduo
#endif

const char *Txt_There_are_no_types_of_group_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No existen tipos de grupo en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"There are no types of group in the course <strong>%s</strong>.";	// Need bersetzung
#elif L==3
	"There are no types of group in the course <strong>%s</strong>.";
#elif L==4
	"No existen tipos de grupo en la asignatura <strong>%s</strong>.";
#elif L==5
	"There are no types of group in the course <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"No existen tipos de grupo en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Non ci sono tipi di gruppi nel corso <strong>%s</strong>.";
#elif L==8
	"There are no types of group in the course <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"There are no types of group in the course <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_There_are_X_users_with_the_ID_Y =	// Warning: it is very important to include %u and %s in the following sentences
#if   L==1
	"Hi ha %u usuaris amb el ID (DNI/c&eacute;dula) <strong>%s</strong>.";
#elif L==2
	"There are %u users with the ID <strong>%s</strong>.";			// Need bersetzung
#elif L==3
	"There are %u users with the ID <strong>%s</strong>.";
#elif L==4
	"Hay %u usuarios con el ID (DNI/c&eacute;dula) <strong>%s</strong>.";
#elif L==5
	"Il y a %u utilisateurs avec le ID <strong>%s</strong>.";
#elif L==6
	"Hay %u usuarios con el ID (DNI/c&eacute;dula) <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Ci sono %u utenti con il ID <strong>%s</strong>.";
#elif L==8
	"There are %u users with the ID <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9
	"There are %u users with the ID <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_There_has_been_no_change_in_groups =
#if   L==1
	"No se ha realizado ning&uacute;n cambio de grupos.";	// Necessita traduccio
#elif L==2
	"There has been no change in groups.";			// bersetzung notwendig
#elif L==3
	"There has been no change in groups.";
#elif L==4
	"No se ha realizado ning&uacute;n cambio de grupos.";
#elif L==5
	"There has been no change in groups.";			// Besoin de traduction
#elif L==6
	"No se ha realizado ning&uacute;n cambio de grupos.";	// Okoteve traduccin
#elif L==7
	"Non ci sono stati cambiamenti in gruppi.";
#elif L==8
	"There has been no change in groups.";			// Potrzebujesz tlumaczenie
#elif L==9
	"There has been no change in groups.";			// Necessita de traduo
#endif

const char *Txt_There_have_been_X_errors_in_sending_the_message =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Ha habido %u errores enviando el mensaje.";		// Necessita traduccio
#elif L==2
	"There have been %u errors in sending the message.";	// bersetzung notwendig
#elif L==3
	"There have been %u errors in sending the message.";
#elif L==4
	"Ha habido %u errores enviando el mensaje.";
#elif L==5
	"There have been %u errors in sending the message.";	// Besoin de traduction
#elif L==6
	"Ha habido %u errores enviando el mensaje.";		// Okoteve traduccin
#elif L==7
	"Ci sono stati %u errori inviando il messaggio.";
#elif L==8
	"There have been %u errors in sending the message.";	// Potrzebujesz tlumaczenie
#elif L==9
	"There have been %u errors in sending the message.";	// Necessita de traduo
#endif

const char *Txt_There_is_already_a_file_named_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Ya existe un archivo con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"There is already a file named <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3
	"There is already a file named <strong>%s</strong>.";
#elif L==4
	"Ya existe un archivo con el nombre <strong>%s</strong>.";
#elif L==5
	"There is already a file named <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"Ya existe un archivo con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"C'&egrave; gi&agrave; un file chiamato <strong>%s</strong>.";
#elif L==8
	"There is already a file named <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There is already a file named <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_There_is_already_a_non_empty_folder_named_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Ya existe una carpeta no vac&iacute;a con el nombre <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"There is already a non-empty folder named <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3
	"There is already a non-empty folder named <strong>%s</strong>.";
#elif L==4
	"Ya existe una carpeta no vac&iacute;a con el nombre <strong>%s</strong>.";
#elif L==5
	"There is already a non-empty folder named <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"Ya existe una carpeta no vac&iacute;a con el nombre <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"C'&egrave; gi&agrave; una cartella non vuota chiamata <strong>%s</strong>.";
#elif L==8
	"There is already a non-empty folder named <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There is already a non-empty folder named <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_There_is_no_empty_account_associated_with_your_ID_X_ =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No hi ha cap compte buida (encara no usada)"
	" associada al seu ID <strong>%s</strong>."
	" Si creu que pot haver estat inscrit amb un altre ID,"
	" comprovi-ho, per favor.";
#elif L==2
	"Es gibt keine leeren Benutzerkonto (noch nicht verwendet)"
	" im Zusammenhang mit Ihrer ID <strong>%s</strong>."
	" Wenn Sie denken, dass Sie mit einer anderen ID registriert worden sein,"
	" &uuml;berpr&uuml;fen Sie es bitte.";
#elif L==3
	"There is no empty (not yet used) account"
	" associated with your ID <strong>%s</strong>."
	" If you think you may have been registered with another ID,"
	" check it, please.";
#elif L==4
	"No existe ninguna cuenta vac&iacute;a (a&uacute;n no usada)"
	" asociada a su ID <strong>%s</strong>."
	" Si cree que puede haber sido inscrito/a con otro ID"
	" (DNI con letra final o sin ella, NIE, c&eacute;dula, etc.), compru&eacute;belo, por favor.";
#elif L==5
	"Il n'y a pas de compte vide (pas encore utilis&eacute;)"
	" associ&eacute; &agrave; votre ID <strong>%s</strong>."
	" Si vous pensez que vous avez &eacute;t&eacute; enregistr&eacute; avec un autre ID,"
	" v&eacute;rifier, s'il vous pla&icirc;t.";
#elif L==6
	"No existe ninguna cuenta vac&iacute;a (a&uacute;n no usada)"
	" asociada a su ID <strong>%s</strong>."
	" Si cree que puede haber sido inscrito/a con otro ID,"
	" compru&eacute;belo, por favor.";	// Okoteve traduccin
#elif L==7
	"Non vi &egrave; alcun conto vuoto (non ancora utilizzato)"
	" associato con il tuo ID <strong>%s</strong>."
	" Se si pensa che potrebbe essere stato registrato/a con un altro ID,"
	" verificare, per favore.";
#elif L==8
	"Nie ma pusta konta (jeszcze nie u&zdot;ywany)"
	" powi&aogon;zany z identyfikatorem <strong>%s</strong>."
	" Je&sacute;li uwa&zdot;asz, &zdot;e mog&lstrok;a zosta&cacute; zarejestrowana w innym ID,"
	" sprawd&zacute; to, prosz&eogon;.";
#elif L==9
	"N&atilde;o existe nenhuma conta vazio (ainda n&atilde;o utilizado)"
	" associado ao seu ID <strong>%s</strong>."
	" Se voc&ecirc; acha que pode ter sido registado/a com outro ID,"
	" verifique-o, por favor.";
#endif

const char *Txt_There_is_no_knowing_how_many_users_not_logged_have_accessed =
#if   L==1
	"No se puede saber cu&aacute;ntos usuarios distintos han accedido sin identificarse.";	// Necessita traduccio
#elif L==2
	"There is no knowing how many users not logged have accessed.";				// bersetzung notwendig
#elif L==3
	"There is no knowing how many users not logged have accessed.";
#elif L==4
	"No se puede saber cu&aacute;ntos usuarios distintos han accedido sin identificarse.";
#elif L==5
	"There is no knowing how many users not logged have accessed.";				// Besoin de traduction
#elif L==6
	"No se puede saber cu&aacute;ntos usuarios distintos han accedido sin identificarse.";	// Okoteve traduccin
#elif L==7
	"Non si pu&ograve; sapere quanti utenti distinti hanno avuto accesso senza identificarsi.";
#elif L==8
	"There is no knowing how many users not logged have accessed.";				// Potrzebujesz tlumaczenie
#elif L==9
	"There is no knowing how many users not logged have accessed.";				// Necessita de traduo
#endif

const char *Txt_There_is_no_list_of_types_of_degree =
#if   L==1
	"No hay lista de tipos de titulaci&oacute;n.";	// Necessita traduccio
#elif L==2
	"There is no list of types of degree.";		// bersetzung notwendig
#elif L==3
	"There is no list of types of degree.";
#elif L==4
	"No hay lista de tipos de titulaci&oacute;n.";
#elif L==5
	"There is no list of types of degree.";		// Besoin de traduction
#elif L==6
	"No hay lista de tipos de titulaci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Non c'&egrave; la lista de tipi di laurea.";
#elif L==8
	"There is no list of types of degree.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There is no list of types of degree.";		// Necessita de traduo
#endif

const char *Txt_There_is_no_user_with_email_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"There is no user"
	" with email <strong>%s</strong>.";	// bersetzung notwendig
#elif L==3
	"There is no user"
	" with email <strong>%s</strong>.";
#elif L==4
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";
#elif L==5
	"There is no user"
	" with email <strong>%s</strong>.";	// Besoin de traduction
#elif L==6
	"No existe ning&uacute;n usuario"
	" con correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Non c'&egrave; alcun utente"
	" con email <strong>%s</strong>.";
#elif L==8
	"There is no user"
	" with email <strong>%s</strong>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"There is no user"
	" with email <strong>%s</strong>.";	// Necessita de traduo
#endif

const char *Txt_There_is_no_user_with_ID_nick_or_email_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";			// bersetzung notwendig
#elif L==3
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";
#elif L==4
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";
#elif L==5
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";			// Besoin de traduction
#elif L==6
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Non c'&egrave; alcun utente"
	" con ID, nome utente o email <strong>%s</strong>.";
#elif L==8
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";			// Potrzebujesz tlumaczenie
#elif L==9
	"There is no user"
	" with ID, nick or email <strong>%s</strong>.";			// Necessita de traduo
#endif

const char *Txt_There_is_no_user_with_ID_nick_or_e_mail_X_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"There is no user"
	" with ID, nick or email <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3
	"There is no user"
	" with ID, nick or email <strong>%s</strong>"
	" in the course <strong>%s</strong>.";
#elif L==4
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";
#elif L==5
	"There is no user"
	" with ID, nick or email <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"No existe ning&uacute;n usuario"
	" con ID (DNI/c&eacute;dula), apodo o correo <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Non c'&egrave; alcun utente"
	" con ID, nome utente o email <strong>%s</strong>"
	" nel corso <strong>%s</strong>.";
#elif L==8
	"There is no user"
	" with ID, nick or email <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There is no user"
	" with ID, nick or email <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_There_is_no_user_with_nickname_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";		// Necessita traduccio
#elif L==2
	"There is no user"
	" with nickname <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3
	"There is no user"
	" with nickname <strong>%s</strong>.";
#elif L==4
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";
#elif L==5
	"There is no user"
	" with nickname <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"No existe ning&uacute;n usuario"
	" con el apodo <strong>%s</strong>.";		// Okoteve traduccin
#elif L==7
	"Non c'&egrave; alcun utente"
	" con nome utente <strong>%s</strong>.";
#elif L==8
	"There is no user"
	" with nickname <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There is no user"
	" with nickname <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_There_was_a_problem_sending_an_email_automatically =
#if   L==1
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo.";	// Necessita traduccio
#elif L==2
	"There was a problem sending an email automatically.";			// bersetzung notwendig
#elif L==3
	"There was a problem sending an email automatically.";
#elif L==4
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo.";
#elif L==5
	"There was a problem sending an email automatically.";			// Besoin de traduction
#elif L==6
	"Ha ocurrido un problema enviando autom&aacute;ticamente un correo.";	// Okoteve traduccin
#elif L==7
	"C'&egrave; stato un problema inviando automaticamente un'email.";
#elif L==8
	"There was a problem sending an email automatically.";			// Potrzebujesz tlumaczenie
#elif L==9
	"There was a problem sending an email automatically.";			// Necessita de traduo
#endif

const char *Txt_There_was_an_error_in_assessing_the_test_X =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Se ha producido un error al evaluar el test %u.";	// Necessita traduccio
#elif L==2
	"There was an error in assessing the test %u.";		// Need bersetzung
#elif L==3
	"There was an error in assessing the test %u.";
#elif L==4
	"Se ha producido un error al evaluar el test %u.";
#elif L==5
	"There was an error in assessing the test %u.";		// Besoin de traduction
#elif L==6
	"Se ha producido un error al evaluar el test %u.";	// Okoteve traduccin
#elif L==7
	"C'&egrave; stato un errore nel valutare il test %u.";
#elif L==8
	"There was an error in assessing the test %u.";		// Potrzebujesz tlumaczenie
#elif L==9
	"There was an error in assessing the test %u.";		// Necessita de traduo
#endif

const char *Txt_this_field_is_filled_in_the_record_of_one_student =
#if   L==1
	" (dicho campo est&aacute; relleno en la ficha de un estudiante)";	// Necessita traduccio
#elif L==2
	" (this field is filled in the record of one student)";			// Need bersetzung
#elif L==3
	" (this field is filled in the record of one student)";
#elif L==4
	" (dicho campo est&aacute; relleno en la ficha de un estudiante)";
#elif L==5
	" (this field is filled in the record of one student)";			// Besoin de traduction
#elif L==6
	" (dicho campo est&aacute; relleno en la ficha de un estudiante)";	// Okoteve traduccin
#elif L==7
	" (questo campo &egrave; inserito nella scheda di uno studente)";
#elif L==8
	" (this field is filled in the record of one student)";			// Potrzebujesz tlumaczenie
#elif L==9
	" (this field is filled in the record of one student)";			// Necessita de traduo
#endif

const char *Txt_this_field_is_filled_in_the_records_of_X_students =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	" (dicho campo est&aacute; relleno en las fichas de %u estudiantes)";	// Necessita traduccio
#elif L==2
	" (this field is filled in the records of %u students)";		// Need bersetzung
#elif L==3
	" (this field is filled in the records of %u students)";
#elif L==4
	" (dicho campo est&aacute; relleno en las fichas de %u estudiantes)";
#elif L==5
	" (this field is filled in the records of %u students)";		// Besoin de traduction
#elif L==6
	" (dicho campo est&aacute; relleno en las fichas de %u estudiantes)";	// Okoteve traduccin
#elif L==7
	" (questo campo &egrave; inserito nella scheda di %u studenti)";
#elif L==8
	" (this field is filled in the records of %u students)";		// Potrzebujesz tlumaczenie
#elif L==9
	" (this field is filled in the records of %u students)";		// Necessita de traduo
#endif

const char *Txt_This_link_will_remain_active_as_long_as_your_user_s_account_exists =
#if   L==1
	"Aquest enlla&ccedil; romandr&agrave; actiu mentre existeixi el seu compte de l'usuari.";
#elif L==2
	"Diese Verbindung bleibt aktiv, solange das Konto Ihres Benutzers vorhanden.";
#elif L==3
	"This link will remain active as long as your user's account exists.";
#elif L==4
	"Este enlace permanecer&aacute; activo mientras exista su cuenta del usuario.";
#elif L==5
	"Ce lien restera actif aussi longtemps que votre compte de l'utilisateur existe.";
#elif L==6
	"Este enlace permanecer&aacute; activo mientras exista su cuenta del usuario.";	// Okoteve traduccin
#elif L==7
	"Questo link rimarr&agrave; attiva finch&eacute; esiste il tuo account utente.";
#elif L==8
	"Ten link pozostanie aktywna tak d&lstrok;ugo, jak istnieje konto u&zdot;ytkownika.";
#elif L==9
	"Este link permanecer&aacute; ativo enquanto existir a sua conta do usu&aacute;rio.";
#endif

const char *Txt_This_post_has_been_banned_probably_for_not_satisfy_the_rules_of_the_forums =
#if   L==1
	"Este mensaje ha sido censurado,"
	" probablemente por no cumplir las normas de los foros.";		// Necessita traduccio
#elif L==2
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// Need bersetzung
#elif L==3
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";
#elif L==4
	"Este mensaje ha sido censurado,"
	" probablemente por no cumplir las normas de los foros.";
#elif L==5
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// Besoin de traduction
#elif L==6
	"Este mensaje ha sido censurado,"
	" probablemente por no cumplir las normas de los foros.";		// Okoteve traduccin
#elif L==7
	"Questo post &egrave; stato censurato,"
	" probabilmente perch&egrave; non soddisfa le regole del forum.";
#elif L==8
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// Potrzebujesz tlumaczenie
#elif L==9
	"This post has been banned,"
	" probably for not satisfy the rules of the forums.";			// Necessita de traduo
#endif

const char *Txt_This_survey_has_no_questions =
#if   L==1
	"Aquesta enquesta no t&eacute; preguntes.";
#elif L==2
	"Diese Umfrage hat keine Fragen.";
#elif L==3
	"This survey has no questions.";
#elif L==4
	"Esta encuesta no tiene preguntas.";
#elif L==5
	"Ce sondage n'a pas de questions.";
#elif L==6
	"Esta encuesta no tiene preguntas.";	// Okoteve traduccin
#elif L==7
	"Questo sondaggio non ha domande.";
#elif L==8
	"Badanie to nie ma pytania.";
#elif L==9
	"Este inqu&eacute;rito n&atilde;o tem perguntas.";
#endif

const char *Txt_Thread =
#if   L==1
	"Discussi&oacute;";
#elif L==2
	"Thread";
#elif L==3
	"Thread";
#elif L==4
	"Discusi&oacute;n";
#elif L==5
	"Fil";
#elif L==6
	"Discusi&oacute;n";	// Okoteve traduccin
#elif L==7
	"Discussione";
#elif L==8
	"W&aogon;tek";
#elif L==9
	"Thread";
#endif

const char *Txt_thread =
#if   L==1
	"discussi&oacute;";
#elif L==2
	"Thread";
#elif L==3
	"thread";
#elif L==4
	"discusi&oacute;n";
#elif L==5
	"fil";
#elif L==6
	"discusi&oacute;n";	// Okoteve traduccin
#elif L==7
	"discussione";
#elif L==8
	"w&aogon;tek";
#elif L==9
	"thread";
#endif

const char *Txt_Thread_marked_to_be_moved =
#if   L==1
	"Discusi&oacute;n marcada para moverla.";	// Necessita traduccio
#elif L==2
	"Thread marked to be moved.";			// Need bersetzung
#elif L==3
	"Thread marked to be moved.";
#elif L==4
	"Discusi&oacute;n marcada para moverla.";
#elif L==5
	"Thread marked to be moved.";			// Besoin de traduction
#elif L==6
	"Discusi&oacute;n marcada para moverla.";	// Okoteve traduccin
#elif L==7
	"Discussione segnata per spostarla.";
#elif L==8
	"Thread marked to be moved.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Thread marked to be moved.";			// Necessita de traduo
#endif

const char *Txt_Thread_X_marked_to_be_moved =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Discusi&oacute;n <strong>%s</strong> marcada para moverla.";	// Necessita traduccio
#elif L==2
	"Thread <strong>%s</strong> marked to be moved.";		// Need bersetzung
#elif L==3
	"Thread <strong>%s</strong> marked to be moved.";
#elif L==4
	"Discusi&oacute;n <strong>%s</strong> marcada para moverla.";
#elif L==5
	"Thread <strong>%s</strong> marked to be moved.";		// Besoin de traduction
#elif L==6
	"Discusi&oacute;n <strong>%s</strong> marcada para moverla.";	// Okoteve traduccin
#elif L==7
	"Discussione <strong>%s</strong> segnata per spostarla.";
#elif L==8
	"Thread <strong>%s</strong> marked to be moved.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Thread <strong>%s</strong> marked to be moved.";		// Necessita de traduo
#endif

const char *Txt_Thread_moved_to_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Discusi&oacute;n movida a este foro.";	// Necessita traduccio
#elif L==2
	"Thread moved to this forum.";		// Need bersetzung
#elif L==3
	"Thread moved to this forum.";
#elif L==4
	"Discusi&oacute;n movida a este foro.";
#elif L==5
	"Thread moved to this forum.";		// Besoin de traduction
#elif L==6
	"Discusi&oacute;n movida a este foro.";	// Okoteve traduccin
#elif L==7
	"Discussione spostata in questo forum.";
#elif L==8
	"Thread moved to this forum.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Thread moved to this forum.";		// Necessita de traduo
#endif

const char *Txt_Thread_X_moved_to_this_forum =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Discusi&oacute;n <strong>%s</strong> movida a este foro.";	// Necessita traduccio
#elif L==2
	"Thread <strong>%s</strong> moved to this forum.";		// Need bersetzung
#elif L==3
	"Thread <strong>%s</strong> moved to this forum.";
#elif L==4
	"Discusi&oacute;n <strong>%s</strong> movida a este foro.";
#elif L==5
	"Thread <strong>%s</strong> moved to this forum.";		// Besoin de traduction
#elif L==6
	"Discusi&oacute;n <strong>%s</strong> movida a este foro.";	// Okoteve traduccin
#elif L==7
	"Discussione <strong>%s</strong> spostata in questo forum.";
#elif L==8
	"Thread <strong>%s</strong> moved to this forum.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Thread <strong>%s</strong> moved to this forum.";		// Necessita de traduo
#endif

const char *Txt_Thread_removed =
#if   L==1
	"Discusi&oacute;n eliminada.";	// Necessita traduccio
#elif L==2
	"Faden entfernt.";
#elif L==3
	"Thread removed.";
#elif L==4
	"Discusi&oacute;n eliminada.";
#elif L==5
	"Thread removed.";		// Besoin de traduction
#elif L==6
	"Discusi&oacute;n eliminada.";	// Okoteve traduccin
#elif L==7
	"Discussione rimossa.";
#elif L==8
	"Thread removed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Thread removed.";		// Necessita de traduo
#endif

const char *Txt_Thread_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Discusi&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2
	"Faden <strong>%s</strong> entfernt.";
#elif L==3
	"Thread <strong>%s</strong> removed.";
#elif L==4
	"Discusi&oacute;n <strong>%s</strong> eliminada.";
#elif L==5
	"Thread <strong>%s</strong> removed.";			// Besoin de traduction
#elif L==6
	"Discusi&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7
	"Discussione <strong>%s</strong> rimossa.";
#elif L==8
	"Thread <strong>%s</strong> removed.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Thread <strong>%s</strong> removed.";			// Necessita de traduo
#endif

const char *Txt_Thread_with_posts_from_you =
#if   L==1
	"Discusi&oacute;n con mensajes de usted.";	// Necessita traduccio
#elif L==2
	"Thread with posts from you.";			// Need bersetzung
#elif L==3
	"Thread with posts from you.";
#elif L==4
	"Discusi&oacute;n con mensajes de usted.";
#elif L==5
	"Thread with posts from you.";			// Besoin de traduction
#elif L==6
	"Discusi&oacute;n con mensajes de usted.";	// Okoteve traduccin
#elif L==7
	"Discussione con post per te.";
#elif L==8
	"Thread with posts from you.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Thread with posts from you.";			// Necessita de traduo
#endif

const char *Txt_Threads =
#if   L==1
	"Discussions";
#elif L==2
	"Threads";
#elif L==3
	"Threads";
#elif L==4
	"Discusiones";
#elif L==5
	"Fils";
#elif L==6
	"Discusiones";	// Okoteve traduccin
#elif L==7
	"Discussione";
#elif L==8
	"W&aogon;tki";
#elif L==9
	"Threads";
#endif

const char *Txt_threads =
#if   L==1
	"discussions";
#elif L==2
	"Threads";
#elif L==3
	"threads";
#elif L==4
	"discusiones";
#elif L==5
	"fils";
#elif L==6
	"discusiones";	// Okoteve traduccin
#elif L==7
	"discussione";
#elif L==8
	"w&aogon;tki";
#elif L==9
	"threads";
#endif

const char *Txt_TIME_Since =
#if   L==1
	"Des de";
#elif L==2
	"Seit dem";
#elif L==3
	"Since";
#elif L==4
	"Desde";
#elif L==5
	"Depuis";
#elif L==6
	"Desde";	// Okoteve traduccin
#elif L==7
	"Dal";
#elif L==8
	"Od";
#elif L==9
	"Desde";
#endif

const char *Txt_TIME_until =
#if   L==1
	"fins al";
#elif L==2
	"bis zum";
#elif L==3
	"until";
#elif L==4
	"hasta";
#elif L==5
	"jusqu'au";
#elif L==6
	"hasta";	// Okoteve traduccin
#elif L==7
	"al";
#elif L==8
	"do";
#elif L==9
	"at&eacute;";
#endif

const char *Txt_time =
#if   L==1
	"temps";
#elif L==2
	"Zeit";
#elif L==3
	"time";
#elif L==4
	"tiempo";
#elif L==5
	"temps";
#elif L==6
	"tiempo";	// Okoteve traduccin
#elif L==7
	"tempo";
#elif L==8
	"czas";
#elif L==9
	"tempo";
#endif

const char *Txt_Time_zone_used_in_the_calculation_of_these_statistics =
#if   L==1
	"Zona hor&agrave;ria usada en el c&agrave;lcul d'aquesta estad&iacute;stica";
#elif L==2
	"Zeitzone in die Berechnung dieser Statistik verwendet";
#elif L==3
	"Time zone used in the calculation of these statistics";
#elif L==4
	"Zona horaria usada en el c&aacute;lculo de esta estad&iacute;stica";
#elif L==5
	"Fuseau horaire utilis&eacute; dans le calcul de ces statistiques";
#elif L==6
	"Zona horaria usada en el c&aacute;lculo de esta estad&iacute;stica";	// Okoteve traduccin
#elif L==7
	"Fuso orario utilizzato per il calcolo di queste statistiche";
#elif L==8
	"Strefa czasowa u&zdot;ywana w obliczeniach tych statystyk";
#elif L==9
	"Fuso hor&aacute;rio utilizado no c&aacute;lculo dessas estat&iacute;sticas";
#endif

const char *Txt_TIMETABLE_TYPES[TT_NUM_TIMETABLE_TYPES] =
	{
	// TT_COURSE_TIMETABLE
#if   L==1
	"Horari de classes de l'assignatura"
#elif L==2
	"Stundenplan"
#elif L==3
	"Class timetable"
#elif L==4
	"Horario de clases de la asignatura"
#elif L==5
	"Horaire"
#elif L==6
	"Horario de clases de la asignatura"	// Okoteve traduccin
#elif L==7
	"Orario del corso"
#elif L==8
	"Harmonogram klasy"
#elif L==9
	"Hor&aacute;rio"
#endif
	 ,
	// TT_MY_TIMETABLE
#if   L==1
	"Horari de les meves assignatures"
#elif L==2
	"Stundenplan"
#elif L==3
	"My timetable"
#elif L==4
	"Horario de mis asignaturas"
#elif L==5
	"Horaire mes mati&egrave;res"
#elif L==6
	"Horario de mis asignaturas"	// Okoteve traduccin
#elif L==7
	"Orario dei miei corsi"
#elif L==8
	"M&oacute;j plan lekcji"
#elif L==9
	"Hor&aacute;rio minha disciplinas"
#endif
	 ,
	// TT_TUTOR_TIMETABLE
#if   L==1
	"Horari de tutories"
#elif L==2
	"Gesch&auml;ftszeiten"
#elif L==3
	"Office hours"
#elif L==4
	"Horario de tutor&iacute;as"
#elif L==5
	"Heures de tuteur"
#elif L==6
	"Horario de tutor&iacute;as"	// Okoteve traduccin
#elif L==7
	"Orario di tutoraggi"
#elif L==8
	"Godziny pracy"
#elif L==9
	"Hor&aacute;rio de tutor"
#endif
	};

const char *Txt_TIMETABLE_CLASS_TYPES[TT_NUM_CLASS_TYPES] =
	{
   // TT_NO_CLASS
#if   L==1
	 "Lliure"
#elif L==2
	 "Keine Klasse"
#elif L==3
	 "No class"
#elif L==4
	 "Libre"
#elif L==5
	 "Pas de classe"
#elif L==6
	 "Libre"			// Okoteve traduccin
#elif L==7
	 "Libero"
#elif L==8
	 "Bez klasy"
#elif L==9
	 "Livre"
#endif
	 ,
	 // TT_THEORY_CLASS
#if   L==1
	 "Teoria"
#elif L==2
	 "Vorlesung"
#elif L==3
	 "Lectures"
#elif L==4
	 "Teor&iacute;a"
#elif L==5
	 "Th&eacute;orie"
#elif L==6
	 "Teor&iacute;a"		// Okoteve traduccin
#elif L==7
	 "Teoria"
#elif L==8
	 "Wyklady"
#elif L==9
	 "Teoria"
#endif
	 ,
	 // TT_PRACT_CLASS
#if   L==1
	 "Pr&agrave;ct."
#elif L==2
	 "&Uuml;bung"
#elif L==3
	 "Practicals"
#elif L==4
	 "Pr&aacute;ct."
#elif L==5
	 "Pratique"
#elif L==6
	 "Pr&aacute;ct."		// Okoteve traduccin
#elif L==7
	 "Pratica"
#elif L==8
	 "praktyki"
#elif L==9
	 "Pr&aacute;ct."
#endif
	 ,
	 // TT_TUTOR_CLASS
#if   L==1
	 "Tutories"
#elif L==2
	 "Anleitungen"
#elif L==3
	 "Tutoring"
#elif L==4
	 "Tutor&iacute;as"
#elif L==5
	 "Tutorat"
#elif L==6
	 "Tutor&iacute;as"	// Okoteve traduccin
#elif L==7
	 "Tutoraggio"
#elif L==8
	 "Korepetycje"
#elif L==9
	 "Tutoria"
#endif
	};

const char *Txt_Title =
#if   L==1
	"T&iacute;tol";
#elif L==2
	"Titel";
#elif L==3
	"Title";
#elif L==4
	"T&iacute;tulo";
#elif L==5
	"Titre";
#elif L==6
	"Moak&atilde;ha";
#elif L==7
	"Titolo";
#elif L==8
	"Tytu&lstrok;";
#elif L==9
	"T&iacute;tulo";
#endif

const char *Txt_To_get_the_official_list_of_your_groups_and_students_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==1
	"Per obtenir la llista oficial dels seus grups i estudiants"
	" ha d'entrar a %s des <a href=\"%s\" target=\"_blank\">%s</a>.";
#elif L==2
	"To get the official list of your groups and students"
	" you must log in to %s from <a href=\"%s\" target=\"_blank\">%s</a>.";	// Need bersetzung
#elif L==3
	"To get the official list of your groups and students"
	" you must log in to %s from <a href=\"%s\" target=\"_blank\">%s</a>.";
#elif L==4
	"Para obtener la lista oficial de sus grupos y estudiantes"
	" debe entrar a %s desde <a href=\"%s\" target=\"_blank\">%s</a>.";
#elif L==5
	"To get the official list of your groups and students"
	" you must log in to %s from <a href=\"%s\" target=\"_blank\">%s</a>.";	// Besoin de traduction
#elif L==6
	"Para obtener la lista oficial de sus grupos y estudiantes"
	" debe entrar a %s desde <a href=\"%s\" target=\"_blank\">%s</a>.";	// Okoteve traduccin
#elif L==7
	"Per ottenere la lista ufficiale dei tuoi gruppi e studenti"
	" devi loggarti a %s da <a href=\"%s\" target=\"_blank\">%s</a>.";
#elif L==8
	"To get the official list of your groups and students"
	" you must log in to %s from <a href=\"%s\" target=\"_blank\">%s</a>.";	// Potrzebujesz tlumaczenie
#elif L==9
	"To get the official list of your groups and students"
	" you must log in to %s from <a href=\"%s\" target=\"_blank\">%s</a>.";	// Necessita de traduo
#endif

const char *Txt_To_remove_a_centre_you_must_first_remove_all_degrees_and_teachers_in_the_centre =
#if   L==1
	"Para eliminar un centro, primero debe eliminar"
	" todas las titulaciones y todos los profesores de ese centro.";	// Necessita traduccio
#elif L==2
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";				// Need bersetzung
#elif L==3
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";
#elif L==4
	"Para eliminar un centro, primero debe eliminar"
	" todas las titulaciones y todos los profesores de ese centro.";
#elif L==5
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";				// Besoin de traduction
#elif L==6
	"Para eliminar un centro, primero debe eliminar"
	" todas las titulaciones y todos los profesores de ese centro.";	// Okoteve traduccin
#elif L==7
	"Per rimuovere un centro, devi prima rimuovere"
	" tutte le lauree e professori di quel centro.";
#elif L==8
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";				// Potrzebujesz tlumaczenie
#elif L==9
	"To remove a centre, you must first remove"
	" all degrees and teachers in the centre.";				// Necessita de traduo
#endif

const char *Txt_To_remove_a_course_you_must_first_remove_all_users_in_the_course =
#if   L==1
	"Para eliminar una asignatura, primero debe eliminar"
	" todos los usuarios de esa asignatura.";		// Necessita traduccio
#elif L==2
	"To remove a course, you must first remove"
	" all users in the course.";				// Need bersetzung
#elif L==3
	"To remove a course, you must first remove"
	" all users in the course.";
#elif L==4
	"Para eliminar una asignatura, primero debe eliminar"
	" todos los usuarios de esa asignatura.";
#elif L==5
	"To remove a course, you must first remove"
	" all users in the course.";				// Besoin de traduction
#elif L==6
	"Para eliminar una asignatura, primero debe eliminar"
	" todos los usuarios de esa asignatura.";		// Okoteve traduccin
#elif L==7
	"Per rimuovere un corso, devi prima rimuovere"
	" tutti gli utenti di quel corso.";
#elif L==8
	"To remove a course, you must first remove"
	" all users in the course.";				// Potrzebujesz tlumaczenie
#elif L==9
	"To remove a course, you must first remove"
	" all users in the course.";				// Necessita de traduo
#endif

const char *Txt_To_remove_a_degree_you_must_first_remove_all_courses_in_the_degree =
#if   L==1
	"Para eliminar una titulaci&oacute;n, primero debe eliminar"
	" todas las asignaturas de esa titulaci&oacute;n.";	// Necessita traduccio
#elif L==2
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// Need bersetzung
#elif L==3
	"To remove a degree, you must first remove"
	" all courses in the degree.";
#elif L==4
	"Para eliminar una titulaci&oacute;n, primero debe eliminar"
	" todas las asignaturas de esa titulaci&oacute;n.";
#elif L==5
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// Besoin de traduction
#elif L==6
	"Para eliminar una titulaci&oacute;n, primero debe eliminar"
	" todas las asignaturas de esa titulaci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Per rimuovere una laurea, devi prima rimuovere"
	" tutti i corsi della laurea.";
#elif L==8
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// Potrzebujesz tlumaczenie
#elif L==9
	"To remove a degree, you must first remove"
	" all courses in the degree.";				// Necessita de traduo
#endif

const char *Txt_To_remove_a_department_you_must_first_remove_all_teachers_in_the_department =
#if   L==1
	"Para eliminar un departamento, primero debe eliminar"
	" todos los profesores de ese departamento.";	// Necessita traduccio
#elif L==2
	"To remove a department, you must first remove"
	" all teachers in the department.";		// Need bersetzung
#elif L==3
	"To remove a department, you must first remove"
	" all teachers in the department.";
#elif L==4
	"Para eliminar un departamento, primero debe eliminar"
	" todos los profesores de ese departamento.";
#elif L==5
	"To remove a department, you must first remove"
	" all teachers in the department.";		// Besoin de traduction
#elif L==6
	"Para eliminar un departamento, primero debe eliminar"
	" todos los profesores de ese departamento.";	// Okoteve traduccin
#elif L==7
	"Per rimuovere un dipartimento, devi prima rimuovere"
	" tutti i professori di quel dipartimento.";
#elif L==8
	"To remove a department, you must first remove"
	" all teachers in the department.";		// Potrzebujesz tlumaczenie
#elif L==9
	"To remove a department, you must first remove"
	" all teachers in the department.";		// Necessita de traduo
#endif

const char *Txt_To_remove_a_place_you_must_first_remove_all_centres_of_that_place =
#if   L==1
	"Para eliminar un lugar, primero debe eliminar"
	" todos los centros de ese lugar.";		// Necessita traduccio
#elif L==2
	"To remove a place, you must first remove"
	" all centres of that place.";			// Need bersetzung
#elif L==3
	"To remove a place, you must first remove"
	" all centres of that place.";
#elif L==4
	"Para eliminar un lugar, primero debe eliminar"
	" todos los centros de ese lugar.";
#elif L==5
	"To remove a place, you must first remove"
	" all centres of that place.";			// Besoin de traduction
#elif L==6
	"Para eliminar un lugar, primero debe eliminar"
	" todos los centros de ese lugar.";		// Okoteve traduccin
#elif L==7
	"Per rimuovere un luogo, devi prima rimuovere"
	" tutte le centri di quel luogo.";
#elif L==8
	"To remove a place, you must first remove"
	" all centres of that place.";			// Potrzebujesz tlumaczenie
#elif L==9
	"To remove a place, you must first remove"
	" all centres of that place.";			// Necessita de traduo
#endif

const char *Txt_To_remove_a_type_of_degree_you_must_first_remove_all_degrees_of_that_type =
#if   L==1
	"Para eliminar un tipo de titulaci&oacute;n, primero debe eliminar"
	" todas las titulaciones de ese tipo.";		// Necessita traduccio
#elif L==2
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";			// Need bersetzung
#elif L==3
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";
#elif L==4
	"Para eliminar un tipo de titulaci&oacute;n, primero debe eliminar"
	" todas las titulaciones de ese tipo.";
#elif L==5
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";			// Besoin de traduction
#elif L==6
	"Para eliminar un tipo de titulaci&oacute;n, primero debe eliminar"
	" todas las titulaciones de ese tipo.";		// Okoteve traduccin
#elif L==7
	"Per rimuovere un tipo di laurea, devi prima rimuovere"
	" tutte le lauree di quel tipo.";
#elif L==8
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";			// Potrzebujesz tlumaczenie
#elif L==9
	"To remove a type of degree, you must first remove"
	" all degrees of that type.";			// Necessita de traduo
#endif

const char *Txt_To_remove_an_institution_you_must_first_remove_all_centres_and_users_in_the_institution =
#if   L==1
	"Para eliminar una instituci&oacute;n, primero debe eliminar"
	" todos los centros y todos los usuarios de esa instituci&oacute;n.";	// Necessita traduccio
#elif L==2
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";				// Need bersetzung
#elif L==3
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";
#elif L==4
	"Para eliminar una instituci&oacute;n, primero debe eliminar"
	" todos los centros y todos los usuarios de esa instituci&oacute;n.";
#elif L==5
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";				// Besoin de traduction
#elif L==6
	"Para eliminar una instituci&oacute;n, primero debe eliminar"
	" todos centros y todos los usuarios de esa instituci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Per rimuovere un'istituzione, devi prima rimuovere"
	" tutti gli centri e utenti di quell'istituzione.";
#elif L==8
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";				// Potrzebujesz tlumaczenie
#elif L==9
	"To remove an institution, you must first remove"
	" all centres and users in the institution.";				// Necessita de traduo
#endif

const char *Txt_To_use_chat_you_must_have_installed_the_software_X_and_add_Y_ =	// Warning: it is very important to include three %s in the following sentences
#if   L==1
	"Per utilitzar el xat cal tenir instal lat el programari"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" i afegir <strong>https://%s</strong>"
	" a la llista de llocs permesos en el panell de control de Java.";
#elif L==2
	"So verwenden Sie chatten, m&uuml;ssen Sie die Software installiert haben"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" und f&uuml;gen Sie <strong>https://%s</strong>"
	" in die Liste der erlaubten Seiten in der Java-Bedienfeld.";
#elif L==3
	"To use chat you must have installed the software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" and add <strong>https://%s</strong>"
	" to the list of allowed sites in the Java control panel.";
#elif L==4
	"Para usar el chat es necesario tener instalado el software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" y a&ntilde;adir <strong>https://%s</strong>"
	" a la lista de sitios permitidos en el panel de control de Java.";
#elif L==5
	"Pour utiliser le chat, vous devez avoir install&eacute; le logiciel"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" et ajouter <strong>https://%s</strong>"
	" &agrave; la liste des sites autoris&eacute;s dans le panneau de contr&ocirc;le Java.";
#elif L==6
	"Para usar el chat es necesario tener instalado el software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" y a&ntilde;adir <strong>https://%s</strong>"
	" a la lista de sitios permitidos en el panel de control de Java.";	// Okoteve traduccin
#elif L==7
	"Per utilizzare la chat devi aver installato il software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" e aggiungere <strong>https://%s</strong>"
	" alla lista dei siti consentiti nel pannello di controllo Java.";
#elif L==8
	"To use chat you must have installed the software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" and add <strong>https://%s</strong>"
	" to the list of allowed sites in the Java control panel.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Para usar o chat &eacute; necess&aacute;rio ter instalado o software"
	" <a href=\"%s\" target=\"_blank\"><em>%s</em></a>"
	" e adicionar <strong>https://%s</strong>"
	" &agrave; lista de sites permitidos no painel de controle Java.";
#endif

const char *Txt_Today =
#if   L==1
	"Avui";
#elif L==2
	"Heute";
#elif L==3
	"Today";
#elif L==4
	"Hoy";
#elif L==5
	"Aujourd'hui";
#elif L==6
	"Ko &aacute;ra";
#elif L==7
	"Oggi";
#elif L==8
	"Dzisiaj";
#elif L==9
	"Hoje";
#endif

const char *Txt_Total =
#if   L==1
	"Total";
#elif L==2
	"Gesamtzahl";
#elif L==3
	"Total";
#elif L==4
	"Total";
#elif L==5
	"Total";
#elif L==6
	"Total";	// Okoteve traduccin
#elif L==7
	"Totale";
#elif L==8
	"Ca&lstrok;kowity";
#elif L==9
	"Total";
#endif

const char *Txt_Total_BR_score =
#if   L==1
	"Puntuaci&oacute;<br />total";
#elif L==2
	"Gesamt";
#elif L==3
	"Total<br />score";
#elif L==4
	"Puntuaci&oacute;n<br />total";
#elif L==5
	"Note<br />total";
#elif L==6
	"Puntuaci&oacute;n<br />total ";	// Okoteve traduccin
#elif L==7
	"Punteggio<br />totale";
#elif L==8
	"Ca&lstrok;kowity<br />wynik";
#elif L==9
	"Pontua&ccedil;&atilde;o<br />total";
#endif

const char *Txt_TST_Correct_ANSWER =
#if   L==1
	"Correcta";
#elif L==2
	"Richtige";
#elif L==3
	"Correct";
#elif L==4
	"Correcta";
#elif L==5
	"Bonne";
#elif L==6
	"Correcta";	// Okoteve traduccin
#elif L==7
	"Corretta";
#elif L==8
	"Poprawna";
#elif L==9
	"Correta";
#endif

const char *Txt_TST_PLUGGABLE[Tst_NUM_OPTIONS_PLUGGABLE] =
	{
	// Tst_PLUGGABLE_UNKNOWN
	"",
	// Tst_PLUGGABLE_NO
#if   L==1
	"Tests no visibles des de m&ograve;bils (SWADroid, TriSWADos)"
#elif L==2
	"Tests nicht zum Download aus Handys (SWADroid, TriSWADos)"
#elif L==3
	"Tests not downloadable from mobile phones (SWADroid, TriSWADos)"
#elif L==4
	"Tests no visibles desde m&oacute;viles (SWADroid, TriSWADos)"
#elif L==5
	"Tests non t&eacute;l&eacute;chargeables &agrave; partir de mobiles (SWADroid, TriSWADos)"
#elif L==6
	"Tests no visibles desde m&oacute;viles (SWADroid, TriSWADos)"	// Okoteve traduccin
#elif L==7
	"Test non scaricabili da cellulari (SWADroid, TriSWADos)"
#elif L==8
	"Nie Testy pobrania z kom&oacute;rkowych (SWADroid, TriSWADos)"
#elif L==9
	"Testes n&atilde;o para download de celulares (SWADroid, TriSWADos)"
#endif
	,
	// Tst_PLUGGABLE_YES
#if   L==1
	"Tests visibles des de m&ograve;bils (SWADroid, TriSWADos), opci&oacute; recomanada"
#elif L==2
	"Tests zum Download aus Handys (SWADroid, TriSWADos), empfohlene Option"
#elif L==3
	"Tests downloadable from mobile phones (SWADroid, TriSWADos), recommended option"
#elif L==4
	"Tests visibles desde m&oacute;viles (SWADroid, TriSWADos), opci&oacute;n recomendada"
#elif L==5
	"Tests t&eacute;l&eacute;chargeables &agrave; partir de mobiles (SWADroid, TriSWADos), option recommand&eacute;e"
#elif L==6
	"Tests visibles desde m&oacute;viles (SWADroid, TriSWADos), opci&oacute;n recomendada"// Okoteve traduccin
#elif L==7
	"Test scaricabili da cellulari (SWADroid, TriSWADos), opzione consigliata"
#elif L==8
	"Testy do pobrania z kom&oacute;rkowych (SWADroid, TriSWADos), zalecana opcja"
#elif L==9
	"Testes para download de celulares (SWADroid, TriSWADos), op&ccedil;&atilde;o recomendada"
#endif
	};

const char *Txt_TST_STR_ANSWER_TYPES[Tst_NUM_ANS_TYPES] =
   {
#if   L==1
	 "Nombre enter"
#elif L==2
	 "Ganze Zahl"
#elif L==3
	 "Integer number"
#elif L==4
	 "N&ordm; entero"
#elif L==5
	 "Nombre entier"
#elif L==6
	 "N&ordm; entero"	// Okoteve traduccin
#elif L==7
	 "Numero intero"
#elif L==8
	 "Integer number"	// Potrzebujesz tlumaczenie
#elif L==9
	 "N&ordm; inteiro"
#endif
	 ,
#if   L==1
	 "Nombre real"
#elif L==2
	 "Reelle Zahl"
#elif L==3
	 "Real number"
#elif L==4
	 "N&ordm; real"
#elif L==5
	 "Nombre r&eacute;el"
#elif L==6
	 "N&ordm; real"	// Okoteve traduccin
#elif L==7
	 "Numero reale"
#elif L==8
	 "Liczba rzeczywista"
#elif L==9
	 "N&ordm; real"
#endif
	 ,
#if   L==1
	 "V/F"
#elif L==2
	 "R/F"
#elif L==3
	 "T/F"
#elif L==4
	 "V/F"
#elif L==5
	 "V/F"
#elif L==6
	 "V/F"	// Okoteve traduccin
#elif L==7
	 "V/F"
#elif L==8
	 "P/F"
#elif L==9
	 "V/F"
#endif
	 ,
#if   L==1
	 "Elecci&oacute; &uacute;nica"
#elif L==2
	 "Unique-Choice"
#elif L==3
	 "Unique choice"
#elif L==4
	 "Elecci&oacute;n &uacute;nica"
#elif L==5
	 "Choix unique"
#elif L==6
	 "Elecci&oacute;n &uacute;nica"		// Okoteve traduccin
#elif L==7
	 "Scelta unica"
#elif L==8
	 "Unikalny wyb&oacute;r"
#elif L==9
	 "&Uacute;nica escolha"
#endif
	 ,
#if   L==1
	 "Elecci&oacute; m&uacute;ltiple"
#elif L==2
	 "Multiple-Choice"
#elif L==3
	 "Multiple choice"
#elif L==4
	 "Elecci&oacute;n m&uacute;ltiple"
#elif L==5
	 "Choix multiple"
#elif L==6
	 "Elecci&oacute;n m&uacute;ltiple"	// Okoteve traduccin
#elif L==7
	 "Scelta multipla"
#elif L==8
	 "Wielokrotnego wyboru"
#elif L==9
	 "M&uacute;ltipla escolha"
#endif
	 ,
#if   L==1
	 "Text"
#elif L==2
	 "Text"
#elif L==3
	 "Text"
#elif L==4
	 "Texto"
#elif L==5
	 "Texte"
#elif L==6
	 "Mba'ehaipyre"
#elif L==7
	 "Testo"
#elif L==8
	 "Tekst"
#elif L==9
	 "Texto"
#endif
	,
	};

const char *Txt_TST_STR_FEEDBACK[Tst_NUM_TYPES_FEEDBACK] =
	{
#if   L==1
	 "Ninguna realimentaci&oacute;n"	// Necessita traduccio
#elif L==2
	 "Keine Feedback"
#elif L==3
	 "No feedback"
#elif L==4
	 "Ninguna realimentaci&oacute;n"
#elif L==5
	 "Pas de feedback"
#elif L==6
	 "Ninguna realimentaci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Nessun feedback"
#elif L==8
	 "No feedback"				// Potrzebujesz tlumaczenie
#elif L==9
	 "Nenhum feedback"
#endif
	 ,
#if   L==1
	 "Poca: solo puntuaci&oacute;n total"	// Necessita traduccio
#elif L==2
	 "Kleine: nur die Gesamtnote"
#elif L==3
	 "Little: only the total score"
#elif L==4
	 "Poca: solo puntuaci&oacute;n total"
#elif L==5
	 "Petit: seul le note total"
#elif L==6
	 "Poca: solo puntuaci&oacute;n total"	// Okoteve traduccin
#elif L==7
	 "Poco: solo il punteggio totale"
#elif L==8
	 "Little: only the total score"			// Potrzebujesz tlumaczenie
#elif L==9
	 "Pouco: apenas a pontua&ccedil;&atilde;o total"
#endif
	 ,
#if   L==1
	 "Media: puntuaci&oacute;n de cada pregunta"	// Necessita traduccio
#elif L==2
	 "Medium: Ergebnisse der einzelnen Frage"
#elif L==3
	 "Medium: result of each question"
#elif L==4
	 "Media: puntuaci&oacute;n de cada pregunta"
#elif L==5
	 "Medium: le r&eacute;sultat de chaque question"
#elif L==6
	 "Media: puntuaci&oacute;n de cada pregunta"	// Okoteve traduccin
#elif L==7
	 "Medio: risultato di ogni domanda"
#elif L==8
	 "Medium: result of each question"		// Potrzebujesz tlumaczenie
#elif L==9
	 "M&eacute;dio: pontua&ccedil;&atilde;o de cada quest&atilde;o"
#endif
	 ,
#if   L==1
	 "Alta: respuesta correcta en cada pregunta"			// Necessita traduccio
#elif L==2
	 "Maximale Details: die richtige Antwort f&uuml;r jede Frage"
#elif L==3
	 "High: correct answer for each question"
#elif L==4
	 "Alta: respuesta correcta en cada pregunta"
#elif L==5
	 "Haute: bonne r&eacute;ponse pour chaque question"
#elif L==6
	 "Alta: respuesta correcta en cada pregunta"			// Okoteve traduccin
#elif L==7
	 "Alto: la corretta risposta per ogni domanda"
#elif L==8
	 "High: correct answer for each question"			// Potrzebujesz tlumaczenie
#elif L==9
	 "Alto: resposta correta para cada quest&atilde;o"
#endif
	 ,
#if   L==1
	 "M&aacute;xima: mostrar texto de realimentaci&oacute;n"	// Necessita traduccio
#elif L==2
	 "Maximale: zeigen Feedback text"
#elif L==3
	 "Maximum: show feedback text"
#elif L==4
	 "M&aacute;xima: mostrar texto de realimentaci&oacute;n"
#elif L==5
	 "Maximum: afficher le texte de feedback"
#elif L==6
	 "M&aacute;xima: mostrar texto de realimentaci&oacute;n"	// Okoteve traduccin
#elif L==7
	 "Massimo: visualizzare il testo de feedback"
#elif L==8
	 "Maximum: show feedback text"					// Potrzebujesz tlumaczenie
#elif L==9
	 "M&aacute;ximo: mostrar o texto de feedback"
#endif
	};

const char *Txt_TST_STR_ORDER_FULL[Tst_NUM_TYPES_ORDER_QST] =
	{
#if   L==1	// Tst_ORDER_STEM
	"Ordenar por enunciado de la pregunta"	// Necessita traduccio
#elif L==2
	"Sortieren nach Stammzellen in Frage"
#elif L==3
	"Sort by stem of the question"
#elif L==4
	"Ordenar por enunciado de la pregunta"
#elif L==5
	"Trier par question"
#elif L==6
	"Ordenar por enunciado de la pregunta"	// Okoteve traduccin
#elif L==7
	"Ordina per domanda"
#elif L==8
	"Sort by stem of the question"		// Potrzebujesz tlumaczenie
#elif L==9
	"Classificar por formula&ccedil;&atilde;o da quest&atilde;o"
#endif
	,
#if   L==1	// Tst_ORDER_NUM_HITS
	"Ordenar por n&ordm; de veces que se ha respondido la pregunta"	// Necessita traduccio
#elif L==2
	"Sortieren nach Anzahl der mal die Frage beantwortet wurde"
#elif L==3
	"Sort by number of times the question has been answered"
#elif L==4
	"Ordenar por n&ordm; de veces que se ha respondido la pregunta"
#elif L==5
	"Trier par nombre de fois qu'on a r&eacute;pondu &agrave; la question"
#elif L==6
	"Ordenar por n&ordm; de veces que se ha respondido la pregunta"	// Okoteve traduccin
#elif L==7
	"Ordina per numero di volte che hai risposto alla domanda"
#elif L==8
	"Sort by number of times the question has been answered"		// Potrzebujesz tlumaczenie
#elif L==9
	"Classificar por n&uacute;mero de vezes que a quest&atilde;o foi respondida"
#endif
	,
#if   L==1	// TST_ORDER_AVERAGE_SCORE
	"Ordenar por puntuaci&oacute;n media"	// Necessita traduccio
#elif L==2
	"Sortieren nach Durchschnittsnote"
#elif L==3
	"Sort by average score"
#elif L==4
	"Ordenar por puntuaci&oacute;n media"
#elif L==5
	"Trier par note moyenne"
#elif L==6
	"Ordenar por puntuaci&oacute;n media"	// Okoteve traduccin
#elif L==7
	"Ordina per punteggio medio"
#elif L==8
	"Sort by average score"			// Potrzebujesz tlumaczenie
#elif L==9
	"Classificar por pontua&ccedil;&atilde;o m&eacute;dia"
#endif
#if   L==1	// Tst_ORDER_NUM_HITS_NOT_BLANK
	"Ordenar por n&ordm; de veces que se ha respondido"
	" la pregunta (excluyendo respuestas en blanco)"	// Necessita traduccio
#elif L==2
	"Sortieren nach Anzahl der mal"
	" die Frage beantwortet wurde (nicht leer)"
#elif L==3
	"Sort by number of times the question"
	" has been answered (excluding blank answers)"
#elif L==4
	"Ordenar por n&ordm; de veces que se ha respondido"
	" la pregunta (excluyendo respuestas en blanco)"
#elif L==5
	"Trier par nombre de fois qu'on a r&eacute;pondu"
	" &agrave; la question (non vide)"
#elif L==6
	"Ordenar por n&ordm; de veces que se ha respondido"
	" la pregunta (excluyendo respuestas en blanco)"	// Okoteve traduccin
#elif L==7
	"Ordina per numero di volte che hai risposto"
	" alla domanda (non vuoto)"
#elif L==8
	"Sort by number of times the question"
	" has been answered (excluding blank answers)"	// Potrzebujesz tlumaczenie
#elif L==9
	"Classificar por n&uacute;mero de vezes"
	" que a quest&atilde;o foi respondida (n&atilde;o em branco)"
#endif
	,
#if   L==1	// TST_ORDER_AVERAGE_SCORE_NOT_BLANK
	"Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"	// Necessita traduccio
#elif L==2
	"Sortieren nach Durchschnittsnote (nicht leer)"
#elif L==3
	"Sort by average score (excluding blank answers)"
#elif L==4
	"Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"
#elif L==5
	"Trier par note moyenne (non vide)"
#elif L==6
	"Ordenar por puntuaci&oacute;n media (excluyendo respuestas en blanco)"	// Okoteve traduccin
#elif L==7
	"Ordina per punteggio medio (non vuoto)"
#elif L==8
	"Sort by average score (excluding blank answers)"				// Potrzebujesz tlumaczenie
#elif L==9
	"Classificar por pontua&ccedil;&atilde;o m&eacute;dia (n&atilde;o em branco)"
#endif
	};

const char *Txt_TST_STR_ORDER_SHORT[Tst_NUM_TYPES_ORDER_QST] =
	{
#if   L==1	// Tst_ORDER_STEM
	"Pregunta"
#elif L==2
	"Frage"
#elif L==3
	"Question"
#elif L==4
	"Pregunta"
#elif L==5
	"Question"
#elif L==6
	"Pregunta"	// Okoteve traduccin
#elif L==7
	"Domanda"
#elif L==8
	"Pytanie"
#elif L==9
	"Quest&atilde;o"
#endif
	,
#if   L==1	// Tst_ORDER_NUM_HITS
	"N&ordm;<br />resp."			// Necessita traduccio
#elif L==2
	"Anzahl<br />Antw."
#elif L==3
	"No.<br />ans."
#elif L==4
	"N&ordm;<br />resp."
#elif L==5
	"Nombre<br />r&eacute;p."
#elif L==6
	"N&ordm;<br />resp."			// Okoteve traduccin
#elif L==7
	"Num.<br />risp."
#elif L==8
	"No.<br />ans."			// Potrzebujesz tlumaczenie
#elif L==9
	"N&ordm;<br />resp."
#endif
	,
#if   L==1	// TST_ORDER_AVERAGE_SCORE
	"Puntuaci&oacute;n<br />media"	// Necessita traduccio
#elif L==2
	"Durchschnitt"
#elif L==3
	"Average<br />score"
#elif L==4
	"Puntuaci&oacute;n<br />media"
#elif L==5
	"Note<br />moyenne"
#elif L==6
	"Puntuaci&oacute;n<br />media"	// Okoteve traduccin
#elif L==7
	"Punteggio<br />medio"
#elif L==8
	"Average<br />score"			// Potrzebujesz tlumaczenie
#elif L==9
	"Pontua&ccedil;&atilde;o<br />m&eacute;dia"
#endif
	,
#if   L==1	// Tst_ORDER_NUM_HITS
	"N&ordm;<br />resp.<br />no en<br />blanco"		// Necessita traduccio
#elif L==2
	"Anzahl<br />Antw.<br />nicht<br />leer"
#elif L==3
	"No.<br />ans.<br />not<br />blank"
#elif L==4
	"N&ordm;<br />resp.<br />no en<br />blanco"
#elif L==5
	"Nombre<br />r&eacute;p.<br />non<br />vide"
#elif L==6
	"N&ordm;<br />resp.<br />no en<br />blanco"		// Okoteve traduccin
#elif L==7
	"Num.<br />risp.<br />non<br />vuoto"
#elif L==8
	"No.<br />ans.<br />not<br />blank"			// Potrzebujesz tlumaczenie
#elif L==9
	"N&ordm;<br />resp.<br />n&atilde;o em<br />branco"
#endif
	,
#if   L==1	// TST_ORDER_AVERAGE_SCORE
	"Puntuaci&oacute;n<br />media<br />no en<br />blanco"	// Necessita traduccio
#elif L==2
	"Durchschnitt<br />nicht<br />leer"
#elif L==3
	"Average<br />score<br />not<br />blank"
#elif L==4
	"Puntuaci&oacute;n<br />media<br />no en<br />blanco"
#elif L==5
	"Note<br />moyenne<br />non<br />vide"
#elif L==6
	"Puntuaci&oacute;n<br />media<br />no en<br />blanco"	// Okoteve traduccin
#elif L==7
	"Punteggio<br />medio<br />non<br />vuoto"
#elif L==8
	"Average<br />score<br />not<br />blank"			// Potrzebujesz tlumaczenie
#elif L==9
	"Pontua&ccedil;&atilde;o<br />m&eacute;dia<br />n&atilde;o em<br />branco"
#endif
	};

const char *Txt_TF_QST[2] =
   {
#if   L==1	// True
	 "V"
#elif L==2
	 "R"
#elif L==3
	 "T"
#elif L==4
	 "V"
#elif L==5
	 "V"
#elif L==6
	 "V"	// Okoteve traduccin
#elif L==7
	 "V"
#elif L==8
	 "P"
#elif L==9
	 "V"
#endif
	 ,
#if   L==1	// False
	 "F"
#elif L==2
	 "F"
#elif L==3
	 "F"
#elif L==4
	 "F"
#elif L==5
	 "F"
#elif L==6
	 "F"	// Okoteve traduccin
#elif L==7
	 "F"
#elif L==8
	 "F"
#elif L==9
	 "F"
#endif
	};

const char *Txt_Type =
#if   L==1
	"Tipus";
#elif L==2
	"Art";
#elif L==3
	"Type";
#elif L==4
	"Tipo";
#elif L==5
	"Type";
#elif L==6
	"Teko";
#elif L==7
	"Tipo";
#elif L==8
	"Typ";
#elif L==9
	"Tipo";
#endif

const char *Txt_Type_of_BR_answers =		// Answers of a test
#if   L==1
	"Tipus de<br />resposte";
#elif L==2
	"Antworttype";
#elif L==3
	"Type of<br />answers";
#elif L==4
	"Tipo de<br />respuestas";
#elif L==5
	"Type de<br />r&eacute;ponses";
#elif L==6
	"Tipo de<br />respuestas";	// Okoteve traduccin
#elif L==7
	"Tipo di<br />risposte";
#elif L==8
	"Typu<br />odpowiedzi";
#elif L==9
	"Tipo de<br />respostas";
#endif

const char *Txt_Type_of_degree =
#if   L==1
	"Tipo de titulaci&oacute;n";	// Necessita traduccio
#elif L==2
	"Abschlussart";
#elif L==3
	"Type of degree";
#elif L==4
	"Tipo de titulaci&oacute;n";
#elif L==5
	"Type d'&eacute;tude";
#elif L==6
	"Tipo de titulaci&oacute;n";	// Okoteve traduccin
#elif L==7
	"Tipo di laurea";
#elif L==8
	"Type of degree";		// Potrzebujesz tlumaczenie
#elif L==9
	"Tipo de titula&ccedil;&atilde;o";
#endif

const char *Txt_Type_of_degree_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada.";	// Necessita traduccio
#elif L==2
	"Type of degree <strong>%s</strong> removed.";			// bersetzung notwendig
#elif L==3
	"Type of degree <strong>%s</strong> removed.";
#elif L==4
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada.";
#elif L==5
	"Type of degree <strong>%s</strong> removed.";			// Besoin de traduction
#elif L==6
	"Tipo de titulaci&oacute;n <strong>%s</strong> eliminada.";	// Okoteve traduccin
#elif L==7
	"Tipo di laurea <strong>%s</strong> rimossa.";
#elif L==8
	"Type of degree <strong>%s</strong> removed.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Type of degree <strong>%s</strong> removed.";			// Necessita de traduo
#endif

const char *Txt_Type_of_group =
#if   L==1
	"Tipo de grupo";	// Necessita traduccio
#elif L==2
	"Art der Gruppe";
#elif L==3
	"Type of group";
#elif L==4
	"Tipo de grupo";
#elif L==5
	"Type de groupe";
#elif L==6
	"Tipo de grupo";	// Okoteve traduccin
#elif L==7
	"Tipo di gruppo";
#elif L==8
	"Type of group";	// Potrzebujesz tlumaczenie
#elif L==9
	"Tipo de grupo";
#endif

const char *Txt_Type_BR_of_group =
#if   L==1
	"Tipo<br />de grupo";	// Necessita traduccio
#elif L==2
	"Art<br />der Gruppe";
#elif L==3
	"Type<br />of group";
#elif L==4
	"Tipo<br />de grupo";
#elif L==5
	"Type<br />de groupe";
#elif L==6
	"Tipo<br />de grupo";	// Okoteve traduccin
#elif L==7
	"Tipo<br />di gruppo";
#elif L==8
	"Type<br />of group";	// Potrzebujesz tlumaczenie
#elif L==9
	"Tipo<br />de grupo";
#endif

const char *Txt_Type_of_group_X_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Tipo de grupo <strong>%s</strong> eliminado.";	// Necessita traduccio
#elif L==2
	"Gruppen-Typ <strong>%s</strong> wurde entfernt.";
#elif L==3
	"Type of group <strong>%s</strong> removed.";
#elif L==4
	"Tipo de grupo <strong>%s</strong> eliminado.";
#elif L==5
	"Type of group <strong>%s</strong> removed.";	// Besoin de traduction
#elif L==6
	"Tipo de grupo <strong>%s</strong> eliminado.";	// Okoteve traduccin
#elif L==7
	"Tipo di gruppo <strong>%s</strong> rimosso.";
#elif L==8
	"Type of group <strong>%s</strong> removed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"Type of group <strong>%s</strong> removed.";	// Necessita de traduo
#endif

const char *Txt_Type_or_paste_a_list_of_IDs_nicks_or_emails_ =
#if   L==1
	"Escriba o pegue una lista"
	" con @apodos, correos o ID (DNI/c&eacute;dulas)."
	" Los @apodos, correos o ID (DNI/c&eacute;dulas)"
	" pueden ir separados por cadenas de caracteres arbitrarias"
	" (por ej. nombres, apellidos, espacios, etc.)."
	" Solo se tendr&aacute;n en cuenta aquellas palabras"
	" que parezcan @apodos, correos o ID (DNI/c&eacute;dulas);"
	" el resto del texto se ignorar&aacute;.";		// Necessita traduccio
#elif L==2
	"Type or paste a list"
	" of @nicks, emails or IDs."
	" The @nicks, emails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, emails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// Need bersetzung
#elif L==3
	"Type or paste a list"
	" of @nicks, emails or IDs."
	" The @nicks, emails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, emails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";
#elif L==4
	"Escriba o pegue una lista"
	" con @apodos, correos o ID (DNI/c&eacute;dulas)."
	" Los @apodos, correos o ID (DNI/c&eacute;dulas)"
	" pueden ir separados por cadenas de caracteres arbitrarias"
	" (por ej. nombres, apellidos, espacios, etc.)."
	" Solo se tendr&aacute;n en cuenta aquellas palabras"
	" que parezcan @apodos, correos o ID (DNI/c&eacute;dulas);"
	" el resto del texto se ignorar&aacute;.";
#elif L==5
	"Type or paste a list"
	" of @nicks, emails or IDs."
	" The @nicks, emails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, emails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// Besoin de traduction
#elif L==6
	"Escriba o pegue una lista"
	" con @apodos, correos o ID (DNI/c&eacute;dulas)."
	" Los @apodos, correos o ID (DNI/c&eacute;dulas)"
	" pueden ir separados por cadenas de caracteres arbitrarias"
	" (por ej. nombres, apellidos, espacios, etc.)."
	" Solo se tendr&aacute;n en cuenta aquellas palabras"
	" que parezcan @apodos, correos o ID (DNI/c&eacute;dulas);"
	" el resto del texto se ignorar&aacute;.";		// Okoteve traduccin
#elif L==7
	"Scrivi o incolla la lista"
	" di @nome-utenti, emails o IDs."
	" I @nome-utenti, emails o IDs"
	" possono essere separati da caratteri arbitrari"
	" (eg. nomi, luoghi, etc.)."
	" Solo le parole che sembrano"
	" @nome-utenti, emails o IDs"
	" saranno presi in considerazione;"
	" il resto del testo sar&agrave; ignorato.";
#elif L==8
	"Type or paste a list"
	" of @nicks, emails or IDs."
	" The @nicks, emails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, emails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Type or paste a list"
	" of @nicks, emails or IDs."
	" The @nicks, emails or IDs"
	" can be separated by arbitrary strings"
	" (eg. names, places, etc.)."
	" Only the words that seems @nicks, emails or IDs"
	" will be taken into account;"
	" the rest of the text will be ignored.";		// Necessita de traduo
#endif

const char *Txt_Types_of_degree =
#if   L==1
	"Tipus de titulaci&oacute;";
#elif L==2
	"Abschlussart";
#elif L==3
	"Types of degree";
#elif L==4
	"Tipos de titulaci&oacute;n";
#elif L==5
	"Types d'&eacute;tude";
#elif L==6
	"Arandur&atilde; l&aacute;ja";
#elif L==7
	"Tipi di laurea";
#elif L==8
	"Types of degree";		// Potrzebujesz tlumaczenie
#elif L==9
	"Tipos de titula&ccedil;&atilde;o";
#endif

const char *Txt_TYPES_OF_DEGREE_With_degrees =
#if   L==1
	"Amb titulacions";
#elif L==2
	"Mit Studiengang";
#elif L==3
	"With degrees";
#elif L==4
	"Con titulaciones";
#elif L==5
	"Avec &eacute;tudes";
#elif L==6
	"Con titulaciones";	// Okoteve traduccin
#elif L==7
	"Con lauree";
#elif L==8
	"Z Stopnie";
#elif L==9
	"Com titula&ccedil;oes";
#endif

const char *Txt_TYPES_OF_DEGREE_Without_degrees =
#if   L==1
	"Sense titulacions";
#elif L==2
	"Ohne Studiengang";
#elif L==3
	"Without degrees";
#elif L==4
	"Sin titulaciones";
#elif L==5
	"Sans &eacute;tudes";
#elif L==6
	"Sin titulaciones";	// Okoteve traduccin
#elif L==7
	"Senza lauree";
#elif L==8
	"Bez Stopnie";
#elif L==9
	"Sem titula&ccedil;oes";
#endif

const char *Txt_Types_of_answers =		// Answers of a test
#if   L==1
	"Tipus de respostes";
#elif L==2
	"Antworttypen";
#elif L==3
	"Types of answers";
#elif L==4
	"Tipos de respuestas";
#elif L==5
	"Types de r&eacute;ponses";
#elif L==6
	"Tipos de respuestas";	// Okoteve traduccin
#elif L==7
	"Tipi di risposte";
#elif L==8
	"Typy odpowiedzi";
#elif L==9
	"Tipos de respostas";
#endif

const char *Txt_Types_of_group =
#if   L==1
	"Tipus de grup";
#elif L==2
	"Gruppen-Typen";
#elif L==3
	"Types of group";
#elif L==4
	"Tipos de grupo";
#elif L==5
	"Types de groupe";
#elif L==6
	"Tipos de grupo";	// Okoteve traduccin
#elif L==7
	"Tipi di gruppo";
#elif L==8
	"Types of group";	// Potrzebujesz tlumaczenie
#elif L==9
	"Tipos de grupo";
#endif

const char *Txt_Undo =
#if   L==1
	"Desfer";
#elif L==2
	"R&uuml;ckg&auml;ngig";
#elif L==3
	"Undo";
#elif L==4
	"Deshacer";
#elif L==5
	"Annuler";
#elif L==6
	"Deshacer";	// Okoteve traduccin
#elif L==7
	"Annulla";
#elif L==8
	"Anulowa&cacute;";
#elif L==9
	"Desfazer";
#endif

const char *Txt_Unfollow =
#if   L==1
	"Deixar de seguir";
#elif L==2
	"Entfolgen";
#elif L==3
	"Unfollow";
#elif L==4
	"Dejar de seguir";
#elif L==5
	"Se d&eacute;sabonner";
#elif L==6
	"Dejar de seguir";	// Okoteve traduccin
#elif L==7
	"Smetti";
#elif L==8
	"Nie obserwuj";
#elif L==9
	"Deixar de seguir";
#endif

const char *Txt_unknown_removed_course =
#if   L==1
	"assignatura desconeguda / eliminada";
#elif L==2
	"Kurs unbekannt / entfernt";
#elif L==3
	"unknown / removed course";
#elif L==4
	"asignatura desconocida / eliminada";
#elif L==5
	"mati&egrave;re inconnue / supprim&eacute;e";
#elif L==6
	"asignatura desconocida / eliminada";	// Okoteve traduccin
#elif L==7
	"corso sconosciuto / rimosso";
#elif L==8
	"kurs nieznany / usuni&eogon;te";
#elif L==9
	"disciplina desconhecida / removida";
#endif

const char *Txt_unknown_assignment =
#if   L==1
	"activitat desconeguda";
#elif L==2
	"Aufgabe unbekannt";
#elif L==3
	"unknown assignment";
#elif L==4
	"actividad desconocida";
#elif L==5
	"activit&eacute; inconnue";
#elif L==6
	"actividad desconocida";	// Okoteve traduccin
#elif L==7
	"attivit&agrave; sconosciuta";
#elif L==8
	"unknown assignment";		// Potrzebujesz tlumaczenie
#elif L==9
	"atividade desconhecida";
#endif

const char *Txt_unknown_recipient =
#if   L==1
	"destinatario desconocido";	// Necessita traduccio
#elif L==2
	"Empf&auml;nger unbekannt";
#elif L==3
	"unknown recipient";
#elif L==4
	"destinatario desconocido";
#elif L==5
	"destinataire inconnu";
#elif L==6
	"destinatario desconocido";	// Okoteve traduccin
#elif L==7
	"destinatario sconosciuto";
#elif L==8
	"unknown recipient";		// Potrzebujesz tlumaczenie
#elif L==9
	"destinat&aacute;rio desconhecido";
#endif

const char *Txt_unknown_recipients =
#if   L==1
	"destinatarios desconocidos";	// Necessita traduccio
#elif L==2
	"Empf&auml;nger unbekannt";
#elif L==3
	"unknown recipients";
#elif L==4
	"destinatarios desconocidos";
#elif L==5
	"destinataires inconnus";
#elif L==6
	"destinatarios desconocidos";	// Okoteve traduccin
#elif L==7
	"destinatari sconosciuti";
#elif L==8
	"unknown recipients";		// Potrzebujesz tlumaczenie
#elif L==9
	"destinat&aacute;rios desconhecidos";
#endif

const char *Txt_unknown_TIME =
#if   L==1
	"desconocido";	// Necessita traduccio
#elif L==2
	"unbekannt";
#elif L==3
	"unknown";
#elif L==4
	"desconocido";
#elif L==5
	"inconnu";
#elif L==6
	"desconocido";	// Okoteve traduccin
#elif L==7
	"sconosciuto";
#elif L==8
	"unknown";	// Potrzebujesz tlumaczenie
#elif L==9
	"desconhecido";
#endif

const char *Txt_unread_MESSAGE =
#if   L==1
	"no le&iacute;do";	// Necessita traduccio
#elif L==2
	"ungelesene";
#elif L==3
	"unread";
#elif L==4
	"no le&iacute;do";
#elif L==5
	"non lu";
#elif L==6
	"no le&iacute;do";	// Okoteve traduccin
#elif L==7
	"non letto";
#elif L==8
	"unread";		// Potrzebujesz tlumaczenie
#elif L==9
	"n&atilde;o lida";
#endif

const char *Txt_unread_MESSAGES =
#if   L==1
	"no le&iacute;dos";	// Necessita traduccio
#elif L==2
	"ungelesene";
#elif L==3
	"unread";
#elif L==4
	"no le&iacute;dos";
#elif L==5
	"non lus";
#elif L==6
	"no le&iacute;dos";	// Okoteve traduccin
#elif L==7
	"non letti";
#elif L==8
	"unread";		// Potrzebujesz tlumaczenie
#elif L==9
	"n&atilde;o lidas";
#endif

const char *Txt_Unread_BR_msgs =
#if   L==1
	"No<br />le&iacute;dos";	// Necessita traduccio
#elif L==2
	"Ungel.<br />Nachr.";
#elif L==3
	"Unread<br />msgs.";
#elif L==4
	"No<br />le&iacute;dos";
#elif L==5
	"Non<br />lus";
#elif L==6
	"No<br />le&iacute;dos";	// Okoteve traduccin
#elif L==7
	"Non<br />letti";
#elif L==8
	"Unread<br />msgs.";		// Potrzebujesz tlumaczenie
#elif L==9
	"N&atilde;o<br />lidas";
#endif

const char *Txt_Update =
#if   L==1
	"Actualitzar";
#elif L==2
	"Update";
#elif L==3
	"Update";
#elif L==4
	"Actualizar";
#elif L==5
	"Mise &agrave; jour";
#elif L==6
	"Actualizar";	// Okoteve traduccin
#elif L==7
	"Aggiornamento";
#elif L==8
	"Update";	// Potrzebujesz tlumaczenie
#elif L==9
	"Atualizar";
#endif

const char *Txt_Update_attendance =
#if   L==1
	"Actualitzar assist&egrave;ncia";
#elif L==2
	"Update Anwesenheit";
#elif L==3
	"Update attendance";
#elif L==4
	"Actualizar asistencia";
#elif L==5
	"Mise &agrave; jour pr&eacute;sence";
#elif L==6
	"Actualizar asistencia";	// Okoteve traduccin
#elif L==7
	"Aggiornamento presenza";
#elif L==8
	"Aktualizacja bytno&sacute;&cacute;";
#elif L==9
	"Atualizar presen&ccedil;a";
#endif

const char *Txt_Update_attendance_according_to_selected_events =
#if   L==1
	"Actualitzar assist&egrave;ncia segons els esdeveniments seleccionats";
#elif L==2
	"Update Anwesenheit nach ausgew&auml;hlten Ereignisse";
#elif L==3
	"Update attendance according to selected events";
#elif L==4
	"Actualizar asistencia seg&uacute;n los eventos seleccionados";
#elif L==5
	"Mise &agrave; jour pr&eacute;sence en fonction d'&eacute;v&eacute;nements s&eacute;lectionnes";
#elif L==6
	"Actualizar asistencia seg&uacute;n los eventos seleccionados";	// Okoteve traduccin
#elif L==7
	"Aggiornamento presenza in base a eventi selezionati";
#elif L==8
	"Update bytno&sacute;&cacute; according to selected events";	// Potrzebujesz tlumaczenie
#elif L==9
	"Atualizar presen&ccedil;a de acordo com eventos selecionados";
#endif

const char *Txt_Update_messages =
#if   L==1
	"Actualitzar missatges";
#elif L==2
	"Update Nachrichten";
#elif L==3
	"Update messages";
#elif L==4
	"Actualizar mensajes";
#elif L==5
	"Mise &agrave; jour des messages";
#elif L==6
	"Actualizar mensajes";		// Okoteve traduccin
#elif L==7
	"Aggiornamento messaggi";
#elif L==8
	"Wiadomo&sacute;ci aktualizacji";
#elif L==9
	"Atualizar mensagens";
#endif

const char *Txt_Update_students =
#if   L==1
	"Actualitzar estudiants";
#elif L==2
	"Update Studenten";
#elif L==3
	"Update students";
#elif L==4
	"Actualizar estudiantes";
#elif L==5
	"Mise &agrave; jour des &eacute;tudiants";
#elif L==6
	"Actualizar estudiantes";	// Okoteve traduccin
#elif L==7
	"Aggiornamento studenti";
#elif L==8
	"Aktualizacja student&oacute;w";
#elif L==9
	"Atualizar os estudantes";
#endif

const char *Txt_Update_students_according_to_selected_groups =
#if   L==1
	"Actualitzar estudiants segons els grups seleccionats";
#elif L==2
	"Update Studenten nach ausgew&auml;hlten Gruppen";
#elif L==3
	"Update students according to selected groups";
#elif L==4
	"Actualizar estudiantes seg&uacute;n los grupos seleccionados";
#elif L==5
	"Mise &agrave; jour des &eacute;tudiants en fonction de groupes s&eacute;lectionnes";
#elif L==6
	"Actualizar estudiantes seg&uacute;n los grupos seleccionados";	// Okoteve traduccin
#elif L==7
	"Aggiornamento studenti in base a gruppi selezionati";
#elif L==8
	"Update students according to selected groups";			// Potrzebujesz tlumaczenie
#elif L==9
	"Atualizar os estudantes de acordo com grupos selecionados";
#endif

const char *Txt_UPLOAD_FILE_Invalid_link =
#if   L==1
	"Enlace no v&aacute;lido.";	// Necessita traduccio
#elif L==2
	"Invalid link.";		// Need bersetzung
#elif L==3
	"Invalid link.";
#elif L==4
	"Enlace no v&aacute;lido.";
#elif L==5
	"Invalid link.";		// Besoin de traduction
#elif L==6
	"Enlace no v&aacute;lido.";	// Okoteve traduccin
#elif L==7
	"Link non valido.";
#elif L==8
	"Invalid link.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Invalid link.";		// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_Invalid_name =
#if   L==1
	"Nombre no v&aacute;lido.";	// Necessita traduccio
#elif L==2
	"Invalid name.";		// Need bersetzung
#elif L==3
	"Invalid name.";
#elif L==4
	"Nombre no v&aacute;lido.";
#elif L==5
	"Invalid name.";		// Besoin de traduction
#elif L==6
	"Nombre no v&aacute;lido.";	// Okoteve traduccin
#elif L==7
	"Nome non valido.";
#elif L==8
	"Invalid name.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Invalid name.";		// Necessita de traduo
#endif

const char *Txt_UPLOAD_FILE_X_invalid_name =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"<strong>%s</strong>: nombre no v&aacute;lido.";	// Necessita traduccio
#elif L==2
	"<strong>%s</strong>: invalid name.";			// Need bersetzung
#elif L==3
	"<strong>%s</strong>: invalid name.";
#elif L==4
	"<strong>%s</strong>: nombre no v&aacute;lido.";
#elif L==5
	"<strong>%s</strong>: invalid name.";			// Besoin de traduction
#elif L==6
	"<strong>%s</strong>: nombre no v&aacute;lido.";	// Okoteve traduccin
#elif L==7
	"<strong>%s</strong>: nome non valido.";
#elif L==8
	"<strong>%s</strong>: invalid name.";			// Potrzebujesz tlumaczenie
#elif L==9
	"<strong>%s</strong>: invalid name.";			// Necessita de traduo
#endif

const char *Txt_Upload_file =
#if   L==1
	"Enviar archivo";	// Necessita traduccio
#elif L==2
	"Upload Datei";
#elif L==3
	"Upload file";
#elif L==4
	"Enviar archivo";
#elif L==5
	"T&eacute;l&eacute;charger fichier";
#elif L==6
	"Enviar archivo";	// Okoteve traduccin
#elif L==7
	"Caricare file";
#elif L==8
	"Upload file";		// Potrzebujesz tlumaczenie
#elif L==9
	"Fazer upload de arquivo";
#endif

const char *Txt_Upload_file_or_create_folder_in_FOLDER =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"A&ntilde;adir archivos o crear carpeta en %s";	// Necessita traduccio
#elif L==2
	"Datei uploaden oder Verzeichnis in %s erstellen";
#elif L==3
	"Upload files or create folder in %s";
#elif L==4
	"A&ntilde;adir archivos o crear carpeta en %s";
#elif L==5
	"Charger fichiers ou cr&eacute;er r&eacute;pertoire dans %s";
#elif L==6
	"A&ntilde;adir archivos o crear carpeta en %s";	// Okoteve traduccin
#elif L==7
	"Caricare file o creare cartella in %s";
#elif L==8
	"Upload file or create folder in %s";		// Potrzebujesz tlumaczenie
#elif L==9
	"Fazer upload de arquivos ou criar diret&oacute;rio em %s";
#endif

const char *Txt_Upload_files =
#if   L==1
	"Enviar fitxers";
#elif L==2
	"Upload Dateien";
#elif L==3
	"Upload files";
#elif L==4
	"Subir archivos";
#elif L==5
	"T&eacute;l&eacute;charger fichiers";
#elif L==6
	"Subir archivos";	// Okoteve traduccin
#elif L==7
	"Caricare file";
#elif L==8
	"Upload files";		// Potrzebujesz tlumaczenie
#elif L==9
	"Fazer upload de arquivos";
#endif

const char *Txt_Upload_files_QUESTION =
#if   L==1
	"Enviar<br />fitxers?";
#elif L==2
	"Upload<br />Dateien?";
#elif L==3
	"Upload<br />files?";
#elif L==4
	"&iquest;Enviar<br />archivos?";
#elif L==5
	"T&eacute;l&eacute;charger<br />fichiers?";
#elif L==6
	"&iquest;Enviar<br />archivos?";	// Okoteve traduccin
#elif L==7
	"Caricare<br />file?";
#elif L==8
	"Upload<br />files?";			// Potrzebujesz tlumaczenie
#elif L==9
	"Fazer upload<br />de arquivos?";
#endif

const char *Txt_Upload_logo =
#if   L==1
	"Enviar logo";
#elif L==2
	"Upload Logo";
#elif L==3
	"Upload logo";
#elif L==4
	"Subir logo";
#elif L==5
	"T&eacute;l&eacute;charger logo";
#elif L==6
	"Subir logo";		// Okoteve traduccin
#elif L==7
	"Caricare logo";
#elif L==8
	"Upload logo";		// Potrzebujesz tlumaczenie
#elif L==9
	"Fazer upload de logotipo";
#endif

const char *Txt_Upload_my_picture =
#if   L==1
	"Pujar la meva fotografia";
#elif L==2
	"Hochladen Mein Foto";
#elif L==3
	"Upload my picture";
#elif L==4
	"Subir mi fotograf&iacute;a";
#elif L==5
	"Ajouter ma photo";
#elif L==6
	"Subir mi fotograf&iacute;a";		// Okoteve traduccin
#elif L==7
	"Caricare una foto";
#elif L==8
	"Prze&sacute;lij swoje zdj&eogon;cia";
#elif L==9
	"Carregar minha foto";
#endif

const char *Txt_Upload_photo =
#if   L==1
	"Enviar foto";
#elif L==2
	"Upload Foto";
#elif L==3
	"Upload photo";
#elif L==4
	"Subir foto";
#elif L==5
	"T&eacute;l&eacute;charger photo";
#elif L==6
	"Subir foto";		// Okoteve traduccin
#elif L==7
	"Caricare foto";
#elif L==8
	"Upload photo";		// Potrzebujesz tlumaczenie
#elif L==9
	"Upload foto";
#endif

const char *Txt_Uploaded_by =
#if   L==1
	"Penjat per";
#elif L==2
	"Hochgeladen von";
#elif L==3
	"Uploaded by";
#elif L==4
	"Subido por";
#elif L==5
	"Post&eacute; par";
#elif L==6
	"Subido por";	// Okoteve traduccin
#elif L==7
	"Caricato da";
#elif L==8
	"Wyslane przez";
#elif L==9
	"Enviado por";
#endif

const char *Txt_URL =		// Uniform Resource Locator
#if   L==1
	"URL";
#elif L==2
	"URL";
#elif L==3
	"URL";
#elif L==4
	"URL";
#elif L==5
	"URL";
#elif L==6
	"URL";	// Okoteve traduccin
#elif L==7
	"URL";
#elif L==8
	"URL";
#elif L==9
	"URL";
#endif

const char *Txt_Use_this_email =
#if   L==1
	"Utilitzar aquest correu";
#elif L==2
	"Verwenden Sie diese E-Mail";
#elif L==3
	"Use this email";
#elif L==4
	"Usar este correo";
#elif L==5
	"Utiliser ce courrier";
#elif L==6
	"Usar este correo";	// Okoteve traduccin
#elif L==7
	"Utilizzare questa email";
#elif L==8
	"U&zdot;yj tej email";
#elif L==9
	"Utilizar este email";
#endif

const char *Txt_Use_this_nickname =
#if   L==1
	"Utilitzar aquest sobrenom";
#elif L==2
	"Verwenden Sie diese Benutzername";
#elif L==3
	"Use this nickname";
#elif L==4
	"Usar este apodo";
#elif L==5
	"Utiliser ce surnom";
#elif L==6
	"Usar este apodo";	// Okoteve traduccin
#elif L==7
	"Utilizzare questo nome utente";
#elif L==8
	"U&zdot;yj tej nick";
#elif L==9
	"Utilizar esta alcunha";
#endif

const char *Txt_User[Usr_NUM_SEXS] =
	{	// Usr_SEX_UNKNOWN
#if   L==1
	"Usuari/a"
#elif L==2
	"Benutzer"
#elif L==3
	"User"
#elif L==4
	"Usuario/a"
#elif L==5
	"Utilisateur/se"
#elif L==6
	"Puruh&atilde;ra"
#elif L==7
	"Utente"
#elif L==8
	"U&zdot;ytkownika"
#elif L==9
	"Utilizador/a"
#endif
	,	// Usr_SEX_FEMALE
#if   L==1
	"Usuaria"
#elif L==2
	"Benutzer"
#elif L==3
	"User"
#elif L==4
	"Usuaria"
#elif L==5
	"Utilisateuse"
#elif L==6
	"Puruh&atilde;ra"
#elif L==7
	"Utente"
#elif L==8
	"U&zdot;ytkownika"
#elif L==9
	"Utilizadora"
#endif
	,	// Usr_SEX_MALE
#if   L==1
	"Usuari"
#elif L==2
	"Benutzer"
#elif L==3
	"User"
#elif L==4
	"Usuario"
#elif L==5
	"Utilisateur"
#elif L==6
	"Puruh&atilde;ra"
#elif L==7
	"Utente"
#elif L==8
	"U&zdot;ytkownika"
#elif L==9
	"Utilizador"
#endif
	,	// Usr_SEX_ALL
#if   L==1
	"Usuari/a"
#elif L==2
	"Benutzer"
#elif L==3
	"User"
#elif L==4
	"Usuario/a"
#elif L==5
	"Utilisateur/se"
#elif L==6
	"Puruh&atilde;ra"
#elif L==7
	"Utente"
#elif L==8
	"U&zdot;ytkownika"
#elif L==9
	"Utilizador/a"
#endif
	};

const char *Txt_user[Usr_NUM_SEXS] =
	{	// Usr_SEX_UNKNOWN
#if   L==1
	"usuari/a"
#elif L==2
	"Benutzer"
#elif L==3
	"user"
#elif L==4
	"usuario/a"
#elif L==5
	"utilisateur/se"
#elif L==6
	"puruh&atilde;ra"
#elif L==7
	"utente"
#elif L==8
	"u&zdot;ytkownika"
#elif L==9
	"utilizador/a"
#endif
	,	// Usr_SEX_FEMALE
#if   L==1
	"usuaria"
#elif L==2
	"Benutzer"
#elif L==3
	"user"
#elif L==4
	"usuaria"
#elif L==5
	"utilisateuse"
#elif L==6
	"usuaria"	// Okoteve traduccin
#elif L==7
	"utente"
#elif L==8
	"u&zdot;ytkownika"
#elif L==9
	"utilizadora"
#endif
	,	// Usr_SEX_MALE
#if   L==1
	"usuari"
#elif L==2
	"Benutzer"
#elif L==3
	"user"
#elif L==4
	"usuario"
#elif L==5
	"utilisateur"
#elif L==6
	"puruh&atilde;ra"
#elif L==7
	"utente"
#elif L==8
	"u&zdot;ytkownika"
#elif L==9
	"utilizador"
#endif
	,	// Usr_SEX_ALL
#if   L==1
	"usuari/a"
#elif L==2
	"Benutzer"
#elif L==3
	"user"
#elif L==4
	"usuario/a"
#elif L==5
	"utilisateur/se"
#elif L==6
	"puruh&atilde;ra"
#elif L==7
	"utente"
#elif L==8
	"u&zdot;ytkownika"
#elif L==9
	"utilizador/a"
#endif
	};

const char *Txt_User_account =
#if   L==1
	"Compte d'usuari";
#elif L==2
	"Benutzerkonto";
#elif L==3
	"User account";
#elif L==4
	"Cuenta de usuario/a";
#elif L==5
	"Compte d'utilisateur";
#elif L==6
	"Cuenta de usuario/a";	// Okoteve traduccin
#elif L==7
	"Account utente";
#elif L==8
	"Konto u&zdot;ytkownika";
#elif L==9
	"Conta de usu&aacute;rio";
#endif

const char *Txt_USER_comments =
#if   L==1
	"Observaciones";	// Necessita traduccio
#elif L==2
	"Kommentare";
#elif L==3
	"Comments";
#elif L==4
	"Observaciones";
#elif L==5
	"Commentaires";
#elif L==6
	"Observaciones";	// Okoteve traduccin
#elif L==7
	"Commenti";
#elif L==8
	"Comments";		// Potrzebujesz tlumaczenie
#elif L==9
	"Coment&aacute;rios";
#endif

const char *Txt_User_ID =
#if   L==1
	"ID (DNI/c&egrave;dula/passaport) de l'usuari";
#elif L==2
	"Ausweis-Nr.";
#elif L==3
	"User ID";
#elif L==4
	"ID (DNI/c&eacute;dula) del usuario";
#elif L==5
	"Num&eacute;ro d'identit&eacute; d'utilisateur";
#elif L==6
	"ID (DNI/c&eacute;dula) del usuario";		// Okoteve traduccin
#elif L==7
	"Numero carta d'identit&agrave; utente";
#elif L==8
	"User ID";					// Potrzebujesz tlumaczenie
#elif L==9
	"N&ordm; de identif. do utilizador";
#endif

const char *Txt_USER_in_COURSE =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"%s en";
#elif L==2
	"%s im";
#elif L==3
	"%s in";
#elif L==4
	"%s en";
#elif L==5
	"%s dans";
#elif L==6
	"%s en";	// Okoteve traduccin
#elif L==7
	"%s in";
#elif L==8
	"%s w";
#elif L==9
	"%s em";
#endif

const char *Txt_User_not_found_or_you_do_not_have_permission_ =
#if   L==1
	"Usuari no trobat"
	" o b&eacute; vost&egrave; no t&eacute; perm&iacute;s"
	" per realitzar aquesta acci&oacute;.";
#elif L==2
	"Benutzer nicht gefunden"
	" oder Sie haben keine Berechtigung,"
	" um diese Handlung durchzuf&uuml;hren.";
#elif L==3
	"User not found"
	" or you do not have permission"
	" to perform this action.";
#elif L==4
	"Usuario/a no encontrado/a"
	" o bien usted no tiene permiso"
	" para realizar esta acci&oacute;n.";
#elif L==5
	"Utilisateur introuvable"
	" ou vous n'avez pas l'autorisation"
	" d'effectuer cette action.";
#elif L==6
	"Usuario/a no encontrado/a"
	" o bien usted no tiene permiso"
	" para realizar esta acci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"IUtente non trovato"
	" o non si dispone dell'autorizzazione"
	" per eseguire questa azione.";
#elif L==8
	"User not found"
	" or you do not have permission"
	" to perform this action.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Utilizador n&atilde;o encontrado"
	" ou voc&ecirc; n&atilde;o tem permiss&atilde;o"
	" para realizar esta a&ccedil;&atilde;o.";
#endif

const char *Txt_User_record_card_in_the_course_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"La ficha del usuario en la asignatura <strong>%s</strong> ha sido eliminada.";	// Necessita traduccio
#elif L==2
	"User record card in the course <strong>%s</strong> has been removed.";		// Need bersetzung
#elif L==3
	"User record card in the course <strong>%s</strong> has been removed.";
#elif L==4
	"La ficha del usuario en la asignatura <strong>%s</strong> ha sido eliminada.";
#elif L==5
	"User record card in the course <strong>%s</strong> has been removed.";		// Besoin de traduction
#elif L==6
	"La ficha del usuario en la asignatura <strong>%s</strong> ha sido eliminada.";	// Okoteve traduccin
#elif L==7
	"La scheda dell'utente nel corso <strong>%s</strong> &egrave; stata rimossa.";
#elif L==8
	"User record card in the course <strong>%s</strong> has been removed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"User record card in the course <strong>%s</strong> has been removed.";		// Necessita de traduo
#endif

const char *Txt_User_record_cards_in_all_courses_have_been_removed =
#if   L==1
	"Se han eliminado las fichas del usuario en todas las asignaturas.";	// Necessita traduccio
#elif L==2
	"User record cards in all courses have been removed.";			// Need bersetzung
#elif L==3
	"User record cards in all courses have been removed.";
#elif L==4
	"Se han eliminado las fichas del usuario en todas las asignaturas.";
#elif L==5
	"User record cards in all courses have been removed.";			// Besoin de traduction
#elif L==6
	"Se han eliminado las fichas del usuario en todas las asignaturas.";	// Okoteve traduccin
#elif L==7
	"Le schede dell'utente in tutti i corsi sono state rimosse.";
#elif L==8
	"User record cards in all courses have been removed.";			// Potrzebujesz tlumaczenie
#elif L==9
	"User record cards in all courses have been removed.";			// Necessita de traduo
#endif

const char *Txt_Users =
#if   L==1
	"Usuaris";
#elif L==2
	"Benutzer";
#elif L==3
	"Users";
#elif L==4
	"Usuarios";
#elif L==5
	"Utilisateurs";
#elif L==6
	"Puruh&aacute;ra";
#elif L==7
	"Utenti";
#elif L==8
	"U&zdot;ytkownicy";
#elif L==9
	"Utilizadores";
#endif

const char *Txt_users[Usr_NUM_SEXS] =
	{	// Usr_SEX_UNKNOWN
#if   L==1
	"usuaris"
#elif L==2
	"Benutzer"
#elif L==3
	"users"
#elif L==4
	"usuarios"
#elif L==5
	"utilisateurs"
#elif L==6
	"puruh&aacute;ra"
#elif L==7
	"utenti"
#elif L==8
	"u&zdot;ytkownicy"
#elif L==9
	"utilizadores"
#endif
	,	// Usr_SEX_FEMALE
#if   L==1
	"usuarias"
#elif L==2
	"Benutzer"
#elif L==3
	"users"
#elif L==4
	"usuarias"
#elif L==5
	"utilisateuses"
#elif L==6
	"usuarias"	// Okoteve traduccin
#elif L==7
	"utenti"
#elif L==8
	"u&zdot;ytkownicy"
#elif L==9
	"utilizadoras"
#endif
	,	// Usr_SEX_MALE
#if   L==1
	"usuaris"
#elif L==2
	"Benutzer"
#elif L==3
	"users"
#elif L==4
	"usuarios"
#elif L==5
	"utilisateurs"
#elif L==6
	"puruh&aacute;ra"
#elif L==7
	"utenti"
#elif L==8
	"u&zdot;ytkownicy"
#elif L==9
	"utilizadores"
#endif
	,	// Usr_SEX_ALL
#if   L==1
	"usuaris"
#elif L==2
	"Benutzer"
#elif L==3
	"users"
#elif L==4
	"usuarios"
#elif L==5
	"utilisateurs"
#elif L==6
	"puruh&aacute;ra"
#elif L==7
	"utenti"
#elif L==8
	"u&zdot;ytkownicy"
#elif L==9
	"utilizadores"
#endif
	};

const char *Txt_X_users_have_been_eliminated =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Se ha(n) eliminado %u usuario(s).";		// Necessita traduccio
#elif L==2
	"%u user(s) have been eliminated.";		// Need bersetzung
#elif L==3
	"%u user(s) have been eliminated.";
#elif L==4
	"Se ha(n) eliminado %u usuario(s).";
#elif L==5
	"%u users have been eliminated.";		// Besoin de traduction
#elif L==6
	"Se ha(n) eliminado %u usuario(s).";		// Okoteve traduccin
#elif L==7
	"%u utenti sono stati rimossi.";
#elif L==8
	"%u user(s) have been eliminated.";		// Potrzebujesz tlumaczenie
#elif L==9
	"%u user(s) have been eliminated.";		// Necessita de traduo
#endif

const char *Txt_X_users_have_been_enrolled_including_possible_repetitions =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Se han inscrito %u usuarios (incluyendo posibles repeticiones).";	// Necessita traduccio
#elif L==2
	"%u users have been enrolled (including possible repetitions).";	// Need bersetzung
#elif L==3
	"%u users have been enrolled (including possible repetitions).";
#elif L==4
	"Se han inscrito %u usuarios (incluyendo posibles repeticiones).";
#elif L==5
	"%u users have been enrolled (including possible repetitions).";	// Besoin de traduction
#elif L==6
	"Se han inscrito %u usuarios (incluyendo posibles repeticiones).";	// Okoteve traduccin
#elif L==7
	"%u utenti sono stati registrati (comprese le eventuali ripetizioni).";
#elif L==8
	"%u users have been enrolled (including possible repetitions).";	// Potrzebujesz tlumaczenie
#elif L==9
	"%u users have been enrolled (including possible repetitions).";	// Necessita de traduo
#endif

const char *Txt_X_users_have_been_removed =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Se han eliminado %u usuarios.";	// Necessita traduccio
#elif L==2
	"%u users have been removed.";		// Need bersetzung
#elif L==3
	"%u users have been removed.";
#elif L==4
	"Se han eliminado %u usuarios.";
#elif L==5
	"%u users have been removed.";		// Besoin de traduction
#elif L==6
	"Se han eliminado %u usuarios.";	// Okoteve traduccin
#elif L==7
	"%u utenti sono stati rimossi.";
#elif L==8
	"%u users have been removed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"%u users have been removed.";		// Necessita de traduo
#endif

const char *Txt_Users_of_the_centre =
#if   L==1
	 "Usuaris del centre";
#elif L==2
	 "Benutzer der Lehrinstitut";
#elif L==3
	 "Users of the centre";
#elif L==4
	 "Usuarios del centro";
#elif L==5
	 "Utilisateurs du centre";
#elif L==6
	 "Usuarios del centro";	// Okoteve traduccin
#elif L==7
	 "Utenti del centro";
#elif L==8
	 "U&zdot;ytkownicy centrum";
#elif L==9
	 "Utilizadores do centro";
#endif

const char *Txt_Users_of_the_country =
#if   L==1
	 "Usuaris del pa&iacute;s";
#elif L==2
	 "Benutzer des Landes";
#elif L==3
	 "Users of the country";
#elif L==4
	 "Usuarios del pa&iacute;s";
#elif L==5
	 "Utilisateurs du pays";
#elif L==6
	 "Usuarios del pa&iacute;s";	// Okoteve traduccin
#elif L==7
	 "Utenti del paese";
#elif L==8
	 "U&zdot;ytkownicy kraju";
#elif L==9
	 "Utilizadores do pa&iacute;s";
#endif

const char *Txt_Users_of_the_institution =
#if   L==1
	 "Usuaris de la instituci&oacute;";
#elif L==2
	 "Benutzer der Hochschule";
#elif L==3
	 "Users of the institution";
#elif L==4
	 "Usuarios de la instituci&oacute;n";
#elif L==5
	 "Utilisateurs du &eacute;tablissement";
#elif L==6
	 "Usuarios de la instituci&oacute;n";	// Okoteve traduccin
#elif L==7
	 "Utenti della istituzione";
#elif L==8
	 "U&zdot;ytkownicy instytucji";
#elif L==9
	 "Utilizadores da institu&ccedil;&atilde;o";
#endif

const char *Txt_usr =		// Abbreviation of user (three characters + dot)
#if   L==1
	"usr.";	// Necessita traduccio
#elif L==2
	"Ben.";
#elif L==3
	"usr.";
#elif L==4
	"usr.";
#elif L==5
	"uti.";
#elif L==6
	"usr.";	// Okoteve traduccin
#elif L==7
	"ute.";
#elif L==8
	"usr.";	// Potrzebujesz tlumaczenie
#elif L==9
	"uti.";
#endif

const char *Txt_USR_LIST_TYPES[Usr_NUM_USR_LIST_TYPES] =
   {
	 ""	// Usr_LIST_UNKNOWN
	 ,	// Usr_LIST_AS_CLASS_PHOTO
#if   L==1
	 "Orla"
#elif L==2
	 "Klass Foto"
#elif L==3
	 "Class photo"
#elif L==4
	 "Orla"
#elif L==5
	 "Photo de classe"
#elif L==6
	 "Orla"		// Okoteve traduccin
#elif L==7
	 "Foto di classe"
#elif L==8
	 "Class photo"	// Potrzebujesz tlumaczenie
#elif L==9
	 "Foto de formatura"
#endif
	 ,	// Usr_LIST_AS_LISTING
#if   L==1
	 "Llista"
#elif L==2
	 "Liste"
#elif L==3
	 "List"
#elif L==4
	 "Lista"
#elif L==5
	 "Liste"
#elif L==6
	 "Lista"	// Okoteve traduccin
#elif L==7
	 "Lista"
#elif L==8
	 "List"		// Potrzebujesz tlumaczenie
#elif L==9
	 "Lista"
#endif
	};

const char *Txt_Vacants =
#if   L==1
	"Vacants";
#elif L==2
	"Frei";
#elif L==3
	"Vacants";
#elif L==4
	"Vacantes";
#elif L==5
	"Vacants";
#elif L==6
	"Vacantes";	// Okoteve traduccin
#elif L==7
	"Liberi";
#elif L==8
	"Vacants";	// Potrzebujesz tlumaczenie
#elif L==9
	"Desocupados";
#endif

const char *Txt_View =
#if   L==1
	"Veure";
#elif L==2
	"Anzeigen";
#elif L==3
	"View";
#elif L==4
	"Ver";
#elif L==5
	"Voir";
#elif L==6
	"Ver";	// Okoteve traduccin
#elif L==7
	"Vedi";
#elif L==8
	"Wy&sacute;wietl";
#elif L==9
	"Ver";
#endif

const char *Txt_View_all_recipients =
#if   L==1
	"Ver todos los destinatarios";	// Necessita traduccio
#elif L==2
	"Anzeigen alle Empf&auml;nger";
#elif L==3
	"View all recipients";
#elif L==4
	"Ver todos los destinatarios";
#elif L==5
	"Voir tous les destinataires";
#elif L==6
	"Ver todos los destinatarios";	// Okoteve traduccin
#elif L==7
	"Vedi tutti i destinatari";
#elif L==8
	"View all recipients";		// Potrzebujesz tlumaczenie
#elif L==9
	"Ver todos os estinat&aacute;rios";
#endif

const char *Txt_View_attendance =
#if   L==1
	"Veure assist&egrave;ncia";
#elif L==2
	"Siehe Anwesenheit";
#elif L==3
	"View attendance";
#elif L==4
	"Ver asistencia";
#elif L==5
	"Voir pr&eacute;sence";
#elif L==6
	"Ver asistencia";	// Okoteve traduccin
#elif L==7
	"Vedi presenza";
#elif L==8
	"Wiadomo&sacute;ci bytno&sacute;&cacute;";
#elif L==9
	"Veja presen&ccedil;a";
#endif

const char *Txt_View_data_of_FILE_OR_LINK_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Veure dades de %s";
#elif L==2
	"Siehe Daten von %s";
#elif L==3
	"View data of %s";
#elif L==4
	"Ver datos de %s";
#elif L==5
	"Voir les donn&eacute;es de %s";
#elif L==6
	"Ver datos de %s";	// Okoteve traduccin
#elif L==7
	"Visualizza i dati de %s";
#elif L==8
	"Zobacz dane z %s";
#elif L==9
	"Veja os dados de %s";
#endif

const char *Txt_View_event =
#if   L==1
	"Veure esdeveniment";
#elif L==2
	"Anzeigen Ereignis";
#elif L==3
	"View event";
#elif L==4
	"Ver evento";
#elif L==5
	"Voir &eacute;v&eacute;nement";
#elif L==6
	"Ver evento";			// Okoteve traduccin
#elif L==7
	"Vedi evento";
#elif L==8
	"View event";			// Potrzebujesz tlumaczenie
#elif L==9
	"Ver evento";
#endif

const char *Txt_View_in_a_new_window =
#if   L==1
	"Mostrar en una finestra nova";
#elif L==2
	"Ansicht in einem neuen Fenster";
#elif L==3
	"View in a new window";
#elif L==4
	"Mostrar en una ventana nueva";
#elif L==5
	"Voir dans une nouvelle fen&ecirc;tre";
#elif L==6
	"Mostrar en una ventana nueva";	// Okoteve traduccin
#elif L==7
	"Visualizza in una nuova finestra";
#elif L==8
	"View in a new window";		// Potrzebujesz tlumaczenie
#elif L==9
	"Ver em uma nova janela";
#endif

const char *Txt_View_record_and_office_hours =
#if   L==1
	"Veure fitxa i horari de tutories";
#elif L==2
	"Anzeigen profil und Gesch&auml;ftszeiten";
#elif L==3
	"View record and office hours";
#elif L==4
	"Ver ficha y horario de tutor&iacute;as";
#elif L==5
	"Voir carte et heures de tuteur";
#elif L==6
	"Ver ficha y horario de tutor&iacute;as";	// Okoteve traduccin
#elif L==7
	"Vedi scheda e orario di tutoraggi";
#elif L==8
	"Zobacz rekord i godziny pracy";
#elif L==9
	"Ver cart&atilde;o e hor&aacute;rio de tutor";
#endif

const char *Txt_View_record_for_this_course =
#if   L==1
	"Veure fitxa en aquesta assignatura";
#elif L==2
	"Anzeigen profil bei Kurs";
#elif L==3
	"View record for this course";
#elif L==4
	"Ver ficha en esta asignatura";
#elif L==5
	"Voir carte dans mati&egrave;re";
#elif L==6
	"Ver ficha en esta asignatura";	// Okoteve traduccin
#elif L==7
	"Vedi scheda per questo corso";
#elif L==8
	"Zobacz rekord na ten kurs";
#elif L==9
	"Ver cart&atilde;o em disciplina";
#endif

const char *Txt_View_survey =
#if   L==1
	"Veure enquesta";
#elif L==2
	"Anzeigen Umfrage";
#elif L==3
	"View survey";
#elif L==4
	"Ver encuesta";
#elif L==5
	"Voir sondage";
#elif L==6
	"Ver encuesta";			// Okoteve traduccin
#elif L==7
	"Vedi sondaggio";
#elif L==8
	"View survey";			// Potrzebujesz tlumaczenie
#elif L==9
	"Ver inqu&eacute;rito";
#endif

const char *Txt_View_survey_results =
#if   L==1
	"Ver resultados";	// Necessita traduccio
#elif L==2
	"Anzeigen Ergebnisse";
#elif L==3
	"View survey results";
#elif L==4
	"Ver resultados";
#elif L==5
	"Voir r&eacute;sultats";
#elif L==6
	"Ver resultados";	// Okoteve traduccin
#elif L==7
	"Vedi risultati";
#elif L==8
	"View survey results";	// Potrzebujesz tlumaczenie
#elif L==9
	"Ver resultados";
#endif

const char *Txt_View_test =
#if   L==1
	"Veure test";
#elif L==2
	"Siehe Test";
#elif L==3
	"View test";
#elif L==4
	"Ver test";
#elif L==5
	"Voir test";
#elif L==6
	"Ver test";	// Okoteve traduccin
#elif L==7
	"Vedi test";
#elif L==8
	"Wiadomo&sacute;ci test";
#elif L==9
	"Veja teste";
#endif

const char *Txt_View_test_results =
#if   L==1
	"Veure resultats de tests";
#elif L==2
	"Siehe Testergebnisse";
#elif L==3
	"View test results";
#elif L==4
	"Ver resultados de tests";
#elif L==5
	"Voir r&eacute;sultats de tests";
#elif L==6
	"Ver resultados de tests";	// Okoteve traduccin
#elif L==7
	"Vedi risultati dei test";
#elif L==8
	"Zobacz wyniki test&oacute;w";
#elif L==9
	"Ver resultados de testes";
#endif

const char *Txt_View_homework =
#if   L==1
	"Veure treballs";
#elif L==2
	"Anzeigen Hausarbeiten";
#elif L==3
	"View homework";
#elif L==4
	"Ver trabajos";
#elif L==5
	"Voir travaux";
#elif L==6
	"Ver trabajos";		// Okoteve traduccin
#elif L==7
	"Vedi lavori";
#elif L==8
	"View homework";	// Potrzebujesz tlumaczenie
#elif L==9
	"Ver trabalhos";
#endif

const char *Txt_Briefcase_of_THE_USER_X_has_been_removed =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Se ha eliminado el malet&iacute;n de <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"Briefcase of <strong>%s</strong> has been removed.";		// Need bersetzung
#elif L==3
	"Briefcase of <strong>%s</strong> has been removed.";
#elif L==4
	"Se ha eliminado el malet&iacute;n de <strong>%s</strong>.";
#elif L==5
	"Briefcase of <strong>%s</strong> has been removed.";		// Besoin de traduction
#elif L==6
	"Se ha eliminado el malet&iacute;n de <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"La penna USB de <strong>%s</strong> &egrave; stata rimossa.";
#elif L==8
	"Briefcase of <strong>%s</strong> has been removed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Briefcase of <strong>%s</strong> has been removed.";		// Necessita de traduo
#endif

const char *Txt_Visible_by_BR_the_student =
#if   L==1
	"&iquest;Visible por<br />el estudiante?";	// Necessita traduccio
#elif L==2
	"Sichtbare durch<br />die Student?";
#elif L==3
	"Visible by<br />the student?";
#elif L==4
	"&iquest;Visible por<br />el estudiante?";
#elif L==5
	"Visible par<br />l'&eacute;tudiant/e?";
#elif L==6
	"&iquest;Visible por<br />el estudiante?";	// Okoteve traduccin
#elif L==7
	"Visibili dallo<br />studente?";
#elif L==8
	"Visible by<br />the student?";			// Potrzebujesz tlumaczenie
#elif L==9
	"Vis&iacute;vel pelo<br />estudante?";
#endif

const char *Txt_Visible_MALE_PLURAL =
#if   L==1
	"Visibles";
#elif L==2
	"Sichtbar";
#elif L==3
	"Visible";
#elif L==4
	"Visibles";
#elif L==5
	"Visibles";
#elif L==6
	"Visibles";	// Okoteve traduccin
#elif L==7
	"Visibili";
#elif L==8
	"Widoczne";
#elif L==9
	"Vis&iacute;veis";
#endif

const char *Txt_Visible_tests =
#if   L==1
	"Tests visibles";
#elif L==2
	"Sichtbare Pr%uuml;fungen";
#elif L==3
	"Visible tests";
#elif L==4
	"Tests visibles";
#elif L==5
	"Tests visibles";
#elif L==6
	"Tests visibles";	// Okoteve traduccin
#elif L==7
	"Test visibili";
#elif L==8
	"Widoczne testy";
#elif L==9
	"Testes vis&iacute;veis";
#endif

const char *Txt_Visible_survey =
#if   L==1
	"Encuesta visible";	// Necessita traduccio
#elif L==2
	"Sichtbare Umfrage";
#elif L==3
	"Visible survey";
#elif L==4
	"Encuesta visible";
#elif L==5
	"Sondage visible";
#elif L==6
	"Encuesta visible";	// Okoteve traduccin
#elif L==7
	"Sondaggio visibile";
#elif L==8
	"Visible survey";	// Potrzebujesz tlumaczenie
#elif L==9
	"Inqu&eacute;rito vis&iacute;vel";
#endif

const char *Txt_Visits_to_course =
#if   L==1
	"Accessos assignatura";
#elif L==2
	"Anmeldungen bei Kurs";
#elif L==3
	"Visits to course";
#elif L==4
	"Accesos asignatura";
#elif L==5
	"Visites mati&egrave;re";
#elif L==6
	"Accesos asignatura";	// Okoteve traduccin
#elif L==7
	"Accessi al corso";
#elif L==8
	"Wizyty w trakcie";
#elif L==9
	"Visitas disciplina";
#endif

const char *Txt_Web =
#if   L==1
	"Web";
#elif L==2
	"Web";
#elif L==3
	"Web";
#elif L==4
	"Web";
#elif L==5
	"Web";
#elif L==6
	"Web";	// Okoteve traduccin
#elif L==7
	"Web";
#elif L==8
	"Web";
#elif L==9
	"Web";
#endif

const char *Txt_Web_page =
#if   L==1
	"P&agrave;gina web";
#elif L==2
	"Webseite";
#elif L==3
	"Web page";
#elif L==4
	"P&aacute;gina web";
#elif L==5
	"Page Web";
#elif L==6
	"P&aacute;gina web";	// Okoteve traduccin
#elif L==7
	"Pagina Web";
#elif L==8
	"Strona internetowa";
#elif L==9
	"P&aacute;gina da Web";
#endif

const char *Txt_Web_page_of_the_course =
#if   L==1
	"P&aacute;gina web de la asignatura";	// Necessita traduccio
#elif L==2
	"Webseite der Kurs";
#elif L==3
	"Web page of the course";
#elif L==4
	"P&aacute;gina web de la asignatura";
#elif L==5
	"Page Web de la mati&egrave;re";
#elif L==6
	"P&aacute;gina web de la asignatura";	// Okoteve traduccin
#elif L==7
	"Pagina Web del corso";
#elif L==8
	"Web page of the course";		// Potrzebujesz tlumaczenie
#elif L==9
	"P&aacute;gina da Web da disciplina";
#endif

const char *Txt_Web_social_network =
#if   L==1
	"Web / xarxa social";
#elif L==2
	"Web / Soziales Netzwerk";
#elif L==3
	"Web / social network";
#elif L==4
	"Web / red social";
#elif L==5
	"Web / r&eacute;seau social";
#elif L==6
	"Web / red social";	// Okoteve traduccin
#elif L==7
	"Web / social network";
#elif L==8
	"Web / sieci spo&lstrok;eczne";
#elif L==9
	"Web / rede social";
#endif

const char *Txt_Webs_social_networks =
#if   L==1
	"Webs / xarxes socials";
#elif L==2
	"Webs / Soziale Netzwerke";
#elif L==3
	"Webs / social networks";
#elif L==4
	"Webs / redes sociales";
#elif L==5
	"Webs / r&eacute;seaux sociaux";
#elif L==6
	"Webs / redes sociales";	// Okoteve traduccin
#elif L==7
	"Webs / reti sociali";
#elif L==8
	"Webs / sieci spo&lstrok;eczne";
#elif L==9
	"Webs / redes sociais";
#endif

const char *Txt_Week =
#if   L==1
	"Semana";	// Necessita traduccio
#elif L==2
	"Woche";
#elif L==3
	"Week";
#elif L==4
	"Semana";
#elif L==5
	"Semaine";
#elif L==6
	"Arapok&otilde;indy";
#elif L==7
	"Settimana";
#elif L==8
	"Week";		// Potrzebujesz tlumaczenie
#elif L==9
	"Semana";
#endif

const char *Txt_Welcome[Usr_NUM_SEXS] =
   {
#if   L==1	// Usr_SEX_UNKNOWN
	 "Benvingut/da."
#elif L==2
	 "Willkommen."
#elif L==3
	 "Welcome."
#elif L==4
	 "Bienvenido/a."
#elif L==5
	 "Bienvenu(e)."
#elif L==6
	 "Bienvenido/a."	// Okoteve traduccin
#elif L==7
	 "Benvenuto/a."
#elif L==8
	 "Witamy."
#elif L==9
	 "Bem-vindo/a."
#endif
	 ,
#if   L==1	// Usr_SEX_FEMALE
	 "Benvinguda."
#elif L==2
	 "Willkommen."
#elif L==3
	 "Welcome."
#elif L==4
	 "Bienvenida."
#elif L==5
	 "Bienvenue."
#elif L==6
	 "Bienvenida."	// Okoteve traduccin
#elif L==7
	 "Benvenuta."
#elif L==8
	 "Witamy."
#elif L==9
	 "Bem-vinda."
#endif
	 ,
#if   L==1	// Usr_SEX_MALE
	 "Benvingut."
#elif L==2
	 "Willkommen."
#elif L==3
	 "Welcome."
#elif L==4
	 "Bienvenido."
#elif L==5
	 "Bienvenu."
#elif L==6
	 "Bienvenido."	// Okoteve traduccin
#elif L==7
	 "Benvenuto."
#elif L==8
	 "Witamy."
#elif L==9
	 "Bem-vindo."
#endif
	 ,
#if   L==1	// Usr_SEX_ALL
	 "Benvingut/da."
#elif L==2
	 "Willkommen."
#elif L==3
	 "Welcome."
#elif L==4
	 "Bienvenido/a."
#elif L==5
	 "Bienvenu(e)."
#elif L==6
	 "Bienvenido/a."	// Okoteve traduccin
#elif L==7
	 "Benvenuto/a."
#elif L==8
	 "Witamy."
#elif L==9
	 "Bem-vindo/a."
#endif
	};

const char *Txt_Welcome_X[Usr_NUM_SEXS] =	// Warning: it is very important to include %s in the following sentences
	{
#if   L==1	// Usr_SEX_UNKNOWN
	"Benvingut/da, %s."
#elif L==2
	"Willkommen, %s."
#elif L==3
	"Welcome, %s."
#elif L==4
	"Bienvenido/a, %s."
#elif L==5
	"Bienvenu(e), %s."
#elif L==6
	"Bienvenido/a, %s."	// Okoteve traduccin
#elif L==7
	"Benvenuto/a, %s."
#elif L==8
	"Witamy, %s."
#elif L==9
	"Bem-vindo/a, %s."
#endif
	,
#if   L==1	// Usr_SEX_FEMALE
	"Benvinguda, %s."
#elif L==2
	"Willkommen, %s."
#elif L==3
	"Welcome, %s."
#elif L==4
	"Bienvenida, %s."
#elif L==5
	"Bienvenue, %s."
#elif L==6
	"Bienvenida, %s."	// Okoteve traduccin
#elif L==7
	"Benvenuta, %s."
#elif L==8
	"Witamy, %s."
#elif L==9
	"Bem-vinda, %s."
#endif
	,
#if   L==1	// Usr_SEX_MALE
	"Benvingut, %s."
#elif L==2
	"Willkommen, %s."
#elif L==3
	"Welcome, %s."
#elif L==4
	"Bienvenido, %s."
#elif L==5
	"Bienvenu, %s."
#elif L==6
	"Bienvenido, %s."	// Okoteve traduccin
#elif L==7
	"Benvenuto, %s."
#elif L==8
	"Witamy, %s."
#elif L==9
	"Bem-vindo, %s."
#endif
	,
#if   L==1	// Usr_SEX_ALL
	"Benvingut/da, %s."
#elif L==2
	"Willkommen, %s."
#elif L==3
	"Welcome, %s."
#elif L==4
	"Bienvenido/a, %s."
#elif L==5
	"Bienvenu(e), %s."
#elif L==6
	"Bienvenido/a, %s."	// Okoteve traduccin
#elif L==7
	"Benvenuto/a, %s."
#elif L==8
	"Witamy, %s."
#elif L==9
	"Bem-vindo/a, %s."
#endif
	};

const char *Txt_Welcome_X_and_happy_birthday[Usr_NUM_SEXS] =	// Warning: it is very important to include %s in the following sentences
	{
#if   L==1	// Usr_SEX_UNKNOWN
	"Benvingut/da, %s, i feli&ccedil; aniversari!"
#elif L==2
	"Willkommen, %s, und alles Gute zum Geburtstag!"
#elif L==3
	"Welcome, %s, and happy birthday!"
#elif L==4
	"Bienvenido/a, %s, &iexcl;y feliz cumplea&ntilde;os!"
#elif L==5
	"Bienvenu(e), %s, et joyeux anniversaire!"
#elif L==6
	"Bienvenido/a, %s, &iexcl;y feliz cumplea&ntilde;os!"// Okoteve traduccin
#elif L==7
	"Benvenuto/a, %s, e buon compleanno!"
#elif L==8
	"Witamy, %s, i wszystkiego najlepszego!"
#elif L==9
	"Bem-vindo/a, %s, e feliz anivers&aacute;rio!"
#endif
	,
#if   L==1	// Usr_SEX_FEMALE
	"Benvinguda, %s, i feli&ccedil; aniversari!"
#elif L==2
	"Willkommen, %s, und alles Gute zum Geburtstag!"
#elif L==3
	"Welcome, %s, and happy birthday!"
#elif L==4
	"Bienvenida, %s, &iexcl;y feliz cumplea&ntilde;os!"
#elif L==5
	"Bienvenue, %s, et joyeux anniversaire!"
#elif L==6
	"Bienvenida, %s, &iexcl;y feliz cumplea&ntilde;os!"// Okoteve traduccin
#elif L==7
	"Benvenuta, %s, e buon compleanno!"
#elif L==8
	"Witamy, %s, i wszystkiego najlepszego!"
#elif L==9
	"Bem-vinda, %s, e feliz anivers&aacute;rio!"
#endif
	,
#if   L==1	// Usr_SEX_MALE
	"Benvingut, %s, i feli&ccedil; aniversari!"
#elif L==2
	"Willkommen, %s, und alles Gute zum Geburtstag!"
#elif L==3
	"Welcome, %s, and happy birthday!"
#elif L==4
	"Bienvenido, %s, &iexcl;y feliz cumplea&ntilde;os!"
#elif L==5
	"Bienvenu, %s, et joyeux anniversaire!"
#elif L==6
	"Bienvenido, %s, &iexcl;y feliz cumplea&ntilde;os!"// Okoteve traduccin
#elif L==7
	"Benvenuto, %s, e buon compleanno!"
#elif L==8
	"Witamy, %s, i wszystkiego najlepszego!"
#elif L==9
	"Bem-vindo, %s, e feliz anivers&aacute;rio!"
#endif
	,
#if   L==1	// Usr_SEX_ALL
	"Benvingut/da, %s, i feli&ccedil; aniversari!"
#elif L==2
	"Willkommen, %s, und alles Gute zum Geburtstag!"
#elif L==3
	"Welcome, %s, and happy birthday!"
#elif L==4
	"Bienvenido/a, %s, &iexcl;y feliz cumplea&ntilde;os!"
#elif L==5
	"Bienvenu(e), %s, et joyeux anniversaire!"
#elif L==6
	"Bienvenido/a, %s, &iexcl;y feliz cumplea&ntilde;os!"// Okoteve traduccin
#elif L==7
	"Benvenuto/a, %s, e buon compleanno!"
#elif L==8
	"Witamy, %s, i wszystkiego najlepszego!"
#elif L==9
	"Bem-vindo/a, %s, e feliz anivers&aacute;rio!"
#endif
	};

const char *Txt_What_would_you_like_to_do =
#if   L==1
	"Qu&egrave; voleu fer?";
#elif L==2
	"Was w&uuml;rden Sie tun?";
#elif L==3
	"What would you like to do?";
#elif L==4
	"&iquest;Qu&eacute; desea hacer?";
#elif L==5
	"Que feriez-vous?";
#elif L==6
	"&iquest;Qu&eacute; desea hacer?";	// Okoteve traduccin
#elif L==7
	"Cosa si dovrebbe fare?";
#elif L==8
	"Co by&sacute; zrobi&lstrok;?";
#elif L==9
	"O que voc&ecirc; faria?";
#endif

const char *Txt_Where_s_USER =
#if   L==1
	"On &eacute;s %s?";
#elif L==2
	"Wo ist %s?";
#elif L==3
	"Where's %s";
#elif L==4
	"&iquest;D&oacute;nde est&aacute; %s?";
#elif L==5
	"O&ugrave; est %s?";
#elif L==6
	"&iquest;D&oacute;nde est&aacute; %s?";	// Okoteve traduccin
#elif L==7
	"Dove &egrave; %s?";
#elif L==8
	"Gdzie jest %s?";
#elif L==9
	"Onde est&aacute; %s?";
#endif

const char *Txt_Who_to_follow =
#if   L==1
	"A qui seguir";
#elif L==2
	"Wer zu folgen";
#elif L==3
	"Who to follow";
#elif L==4
	"A qui&eacute;n seguir";
#elif L==5
	"Qui &agrave; suivre";
#elif L==6
	"A qui&eacute;n seguir";	// Okoteve traduccin
#elif L==7
	"Chi da seguire";
#elif L==8
	"Kto do na&sacute;ladowania";
#elif L==9
	"Quem seguir";
#endif

const char *Txt_With_centres =
#if   L==1
	"Amb centres";
#elif L==2
	"Mit Lehrinstitute";
#elif L==3
	"With centres";
#elif L==4
	"Con centros";
#elif L==5
	"Avec centres";
#elif L==6
	"Con centros";	// Okoteve traduccin
#elif L==7
	"Con centri";
#elif L==8
	"Z centra";
#elif L==9
	"Com centros";
#endif

const char *Txt_With_courses =
#if   L==1
	"Amb assignatures";
#elif L==2
	"Mit Kursen";
#elif L==3
	"With courses";
#elif L==4
	"Con asignaturas";
#elif L==5
	"Avec mati&egrave;res";
#elif L==6
	"Con asignaturas";	// Okoteve traduccin
#elif L==7
	"Con corsi";
#elif L==8
	"Z kurs&oacute;w";
#elif L==9
	"Com disciplinas";
#endif

const char *Txt_With_degrees =
#if   L==1
	"Amb titulacions";
#elif L==2
	"Mit Studiengang";
#elif L==3
	"With degrees";
#elif L==4
	"Con titulaciones";
#elif L==5
	"Avec &eacute;tudes";
#elif L==6
	"Con titulaciones";	// Okoteve traduccin
#elif L==7
	"Con lauree";
#elif L==8
	"Z stopnie";
#elif L==9
	"Com titula&ccedil;oes";
#endif

const char *Txt_With_institutions =
#if   L==1
	"Amb institucions";
#elif L==2
	"Mit Hochschulen";
#elif L==3
	"With institutions";
#elif L==4
	"Con instituciones";
#elif L==5
	"Avec &eacute;tablissements";
#elif L==6
	"Con instituciones";	// Okoteve traduccin
#elif L==7
	"Con istituzioni";
#elif L==8
	"Z instytucje";
#elif L==9
	"Com institu&ccedil;&otilde;es";
#endif

const char *Txt_with_new_posts =
#if   L==1
	"con mensajes nuevos";	// Necessita traduccio
#elif L==2
	"mit neuen Beitr&auml;ge";
#elif L==3
	"with new posts";
#elif L==4
	"con mensajes nuevos";
#elif L==5
	"avec de nouveaux messages";
#elif L==6
	"con mensajes nuevos";	// Okoteve traduccin
#elif L==7
	"con nuovi post";
#elif L==8
	"with new posts";	// Potrzebujesz tlumaczenie
#elif L==9
	"com novas posts";
#endif

const char *Txt_With_students =
#if   L==1
	"Amb estudiants";
#elif L==2
	"Mit Teilnehmern";
#elif L==3
	"With students";
#elif L==4
	"Con estudiantes";
#elif L==5
	"Avec &eacute;tudiants";
#elif L==6
	"Con estudiantes";	// Okoteve traduccin
#elif L==7
	"Con studenti";
#elif L==8
	"Z student&oacute;w";
#elif L==9
	"Com estudantes";
#endif

const char *Txt_With_teachers =
#if   L==1
	"Amb professors";
#elif L==2
	"Mit Lehrkr&auml;ften";
#elif L==3
	"With teachers";
#elif L==4
	"Con profesores";
#elif L==5
	"Avec enseignants";
#elif L==6
	"Con profesores";	// Okoteve traduccin
#elif L==7
	"Con professori";
#elif L==8
	"Z nauczycielami";
#elif L==9
	"Com professores";
#endif

const char *Txt_works_ZIP_FILE_NAME =	// Name (without extension) of the ZIP file for works. Must be a valid filename in small letters!
#if   L==1
	"treballs";
#elif L==2
	"Hausarbeiten";
#elif L==3
	"homework";
#elif L==4
	"trabajos";
#elif L==5
	"travaux";
#elif L==6
	"tembiapo";
#elif L==7
	"lavori";
#elif L==8
	"homework";	// Potrzebujesz tlumaczenie
#elif L==9
	"trabalhos";
#endif

const char *Txt_Works_of_X_in_Y_removed =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"Trabajos de <strong>%s</strong>"
	" en <strong>%s</strong> eliminados.";		// Necessita traduccio
#elif L==2
	"Die Hausarbeiten von <strong>%s</strong>"
	" in <strong>%s</strong> wurden entfernt.";
#elif L==3
	"The homework of <strong>%s</strong>"
	" in <strong>%s</strong> has been removed.";
#elif L==4
	"Trabajos de <strong>%s</strong>"
	" en <strong>%s</strong> eliminados.";
#elif L==5
	"Homework of <strong>%s</strong>"
	" in <strong>%s</strong> removed.";		// Besoin de traduction
#elif L==6
	"Trabajos de <strong>%s</strong>"
	" en <strong>%s</strong> eliminados.";		// Okoteve traduccin
#elif L==7
	"Lavori de <strong>%s</strong>"
	" in <strong>%s</strong> rimossi.";
#elif L==8
	"The homework of <strong>%s</strong>"
	" in <strong>%s</strong> has been removed.";	// Potrzebujesz tlumaczenie
#elif L==9
	"Homework of <strong>%s</strong>"
	" in <strong>%s</strong> removed.";		// Necessita de traduo
#endif

const char *Txt_Works_area =
#if   L==1
	"Zona d'altres treballs";
#elif L==2
	"Hausarbeiten-Bereich";
#elif L==3
	"Homework area";
#elif L==4
	"Zona de otros trabajos";
#elif L==5
	"Zone de travaux";
#elif L==6
	"Zona de otros trabajos";	// Okoteve traduccin
#elif L==7
	"Area lavori";
#elif L==8
	"Homework area";		// Potrzebujesz tlumaczenie
#elif L==9
	"Zona de trabalhos";
#endif

const char *Txt_works_area =
#if   L==1
	"zona de trabajos";	// Necessita traduccio
#elif L==2
	"Hausarbeiten-Bereich";
#elif L==3
	"homework area";
#elif L==4
	"zona de trabajos";
#elif L==5
	"zone de travaux";
#elif L==6
	"zona de trabajos";	// Okoteve traduccin
#elif L==7
	"area lavori";
#elif L==8
	"homework area";	// Potrzebujesz tlumaczenie
#elif L==9
	"zona de trabalhos";
#endif

const char *Txt_WriBRters =
#if   L==1
	"Escri-<br />tores";	// Necessita traduccio
#elif L==2
	"Schrift-<br />steller";
#elif L==3
	"Wri-<br />ters";
#elif L==4
	"Escri-<br />tores";
#elif L==5
	"&Eacute;cri-<br />vains";
#elif L==6
	"Escri-<br />tores";	// Okoteve traduccin
#elif L==7
	"Scri-<br />ttori";
#elif L==8
	"Wri-<br />ters";	// Potrzebujesz tlumaczenie
#elif L==9
	"Escri-<br />tores";
#endif

const char *Txt_Write_a_message =
#if   L==1
	"Escriu un missatge";
#elif L==2
	"Schreiben eine Nachricht";
#elif L==3
	"Write a message";
#elif L==4
	"Escribir un mensaje";
#elif L==5
	"&Eacute;crire un message";
#elif L==6
	"Escribir un mensaje";		// Okoteve traduccin
#elif L==7
	"Scrivi un messaggio";
#elif L==8
	"Napisz wiadomo&sacute;&cacute;";
#elif L==9
	"Escrever uma mensagem";
#endif

const char *Txt_Wrong_file_type =
#if   L==1
	"Tipus d'arxiu incorrecte.";
#elif L==2
	"Falsche Dateityp.";
#elif L==3
	"Wrong file type.";
#elif L==4
	"Tipo de archivo incorrecto.";
#elif L==5
	"Mauvais type de fichier.";
#elif L==6
	"Tipo de archivo incorrecto.";		// Okoteve traduccin
#elif L==7
	"Tipo di file sbagliato.";
#elif L==8
	"Nieprawid&lstrok;owy typ pliku.";
#elif L==9
	"Tipo de arquivo errado.";
#endif

const char *Txt_WWW =		// World Wide Web
#if   L==1
	"WWW";
#elif L==2
	"WWW";
#elif L==3
	"WWW";
#elif L==4
	"WWW";
#elif L==5
	"WWW";
#elif L==6
	"WWW";
#elif L==7
	"WWW";
#elif L==8
	"WWW";
#elif L==9
	"WWW";
#endif

const char *Txt_XML_file =
#if   L==1
	"Fitxer XML";
#elif L==2
	"XML-Datei";
#elif L==3
	"XML file";
#elif L==4
	"Archivo XML";
#elif L==5
	"Fichier XML";
#elif L==6
	"Archivo XML";	// Okoteve traduccin
#elif L==7
	"File XML";
#elif L==8
	"Plik XML";
#elif L==9
	"Arquivo XML";
#endif

const char *Txt_XML_file_content =
#if   L==1
	"Contingut del fitxer XML";
#elif L==2
	"XML-Datei-Inhalt";
#elif L==3
	"XML file content";
#elif L==4
	"Contenido del archivo XML";
#elif L==5
	"Contenu du fichier XML";
#elif L==6
	"Contenido del archivo XML";	// Okoteve traduccin
#elif L==7
	"Contenuto di file XML";
#elif L==8
	"Zawarto&sacute;&cacute; pliku XML";
#elif L==9
	"Conte&uacute;do do arquivo XML";
#endif

const char *Txt_Year_OF_A_DEGREE =	// 1, 2, 3..., not 1984, 2011...
#if   L==1
	"Any o semestre";
#elif L==2
	"Jahr oder Semester";
#elif L==3
	"Year or semester";
#elif L==4
	"A&ntilde;o o semestre";
#elif L==5
	"Ann&eacute;e ou semestre";
#elif L==6
	"A&ntilde;o o semestre";	// Okoteve traduccin
#elif L==7
	"Anno o semestre";
#elif L==8
	"Rok lub semestr";
#elif L==9
	"Ano ou semestre";
#endif

const char *Txt_YEAR_OF_DEGREE[1 + Deg_MAX_YEARS_PER_DEGREE] =
   {
#if   L==1
	 "N.A."
#elif L==2
	 "N.A."
#elif L==3
	 "N.A."
#elif L==4
	 "N.A."
#elif L==5
	 "N.A."
#elif L==6
	 "N.A."		// Okoteve traduccin
#elif L==7
	 "N.A."
#elif L==8
	 "N.D."
#elif L==9
	 "N.A."
#endif
	 ,
#if   L==1
	 "1&ordm;"	// Necessita traduccio
#elif L==2
	 "1st"		// Need bersetzung
#elif L==3
	 "1st"
#elif L==4
	 "1&ordm;"
#elif L==5
	 "1st"		// Besoin de traduction
#elif L==6
	 "1&ordm;"	// Okoteve traduccin
#elif L==7
	 "Primo"
#elif L==8
	 "1st"		// Potrzebujesz tlumaczenie
#elif L==9
	 "1st"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "2&ordm;"	// Necessita traduccio
#elif L==2
	 "2nd"		// Need bersetzung
#elif L==3
	 "2nd"
#elif L==4
	 "2&ordm;"
#elif L==5
	 "2nd"		// Besoin de traduction
#elif L==6
	 "2&ordm;"	// Okoteve traduccin
#elif L==7
	 "Secondo"
#elif L==8
	 "2nd"		// Potrzebujesz tlumaczenie
#elif L==9
	 "2nd"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "3&ordm;"	// Necessita traduccio
#elif L==2
	 "3rd"
#elif L==3
	 "3rd"
#elif L==4
	 "3&ordm;"
#elif L==5
	 "3rd"		// Besoin de traduction
#elif L==6
	 "3&ordm;"	// Okoteve traduccin
#elif L==7
	 "Terzo"
#elif L==8
	 "3rd"		// Potrzebujesz tlumaczenie
#elif L==9
	 "3rd"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "4&ordm;"	// Necessita traduccio
#elif L==2
	 "4th"		// Need bersetzung
#elif L==3
	 "4th"
#elif L==4
	 "4&ordm;"
#elif L==5
	 "Quarto"
#elif L==6
	 "4&ordm;"	// Okoteve traduccin
#elif L==7
	 "4th"		// Bisogno di traduzione
#elif L==8
	 "4th"		// Potrzebujesz tlumaczenie
#elif L==9
	 "4th"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "5&ordm;"	// Necessita traduccio
#elif L==2
	 "5th"		// Need bersetzung
#elif L==3
	 "5th"
#elif L==4
	 "5&ordm;"
#elif L==5
	 "5th"		// Besoin de traduction
#elif L==6
	 "5&ordm;"	// Okoteve traduccin
#elif L==7
	 "Quinto"
#elif L==8
	 "5th"		// Potrzebujesz tlumaczenie
#elif L==9
	 "5th"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "6&ordm;"	// Necessita traduccio
#elif L==2
	 "6th"		// Need bersetzung
#elif L==3
	 "6th"
#elif L==4
	 "6&ordm;"
#elif L==5
	 "6th"		// Besoin de traduction
#elif L==6
	 "6&ordm;"	// Okoteve traduccin
#elif L==7
	 "Sesto"
#elif L==8
	 "6th"		// Potrzebujesz tlumaczenie
#elif L==9
	 "6th"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "7&ordm;"	// Necessita traduccio
#elif L==2
	 "7th"		// Need bersetzung
#elif L==3
	 "7th"
#elif L==4
	 "7&ordm;"
#elif L==5
	 "7th"		// Besoin de traduction
#elif L==6
	 "7&ordm;"	// Okoteve traduccin
#elif L==7
	 "Settimo"
#elif L==8
	 "7th"		// Potrzebujesz tlumaczenie
#elif L==9
	 "7th"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "8&ordm;"	// Necessita traduccio
#elif L==2
	 "8th"		// Need bersetzung
#elif L==3
	 "8th"
#elif L==4
	 "8&ordm;"
#elif L==5
	 "8th"		// Besoin de traduction
#elif L==6
	 "8&ordm;"	// Okoteve traduccin
#elif L==7
	 "Ottavo"
#elif L==8
	 "8th"		// Potrzebujesz tlumaczenie
#elif L==9
	 "8th"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "9&ordm;"	// Necessita traduccio
#elif L==2
	 "9th"		// Need bersetzung
#elif L==3
	 "9th"
#elif L==4
	 "9&ordm;"
#elif L==5
	 "9th"		// Besoin de traduction
#elif L==6
	 "9&ordm;"	// Okoteve traduccin
#elif L==7
	 "Nono"
#elif L==8
	 "9th"		// Potrzebujesz tlumaczenie
#elif L==9
	 "9th"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "10&ordm;"	// Necessita traduccio
#elif L==2
	 "10th"		// Need bersetzung
#elif L==3
	 "10th"
#elif L==4
	 "10&ordm;"
#elif L==5
	 "10th"		// Besoin de traduction
#elif L==6
	 "10&ordm;"	// Okoteve traduccin
#elif L==7
	 "Decimo"
#elif L==8
	 "10th"		// Potrzebujesz tlumaczenie
#elif L==9
	 "10th"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "11&ordm;"	// Necessita traduccio
#elif L==2
	 "11th"		// Need bersetzung
#elif L==3
	 "11th"
#elif L==4
	 "11&ordm;"
#elif L==5
	 "11th"		// Besoin de traduction
#elif L==6
	 "11&ordm;"	// Okoteve traduccin
#elif L==7
	 "Undicesimo"
#elif L==8
	 "11th"		// Potrzebujesz tlumaczenie
#elif L==9
	 "11th"		// Necessita de traduo
#endif
	 ,
#if   L==1
	 "12&ordm;"	// Necessita traduccio
#elif L==2
	 "12th"		// Need bersetzung
#elif L==3
	 "12th"
#elif L==4
	 "12&ordm;"
#elif L==5
	 "12th"		// Besoin de traduction
#elif L==6
	 "12&ordm;"	// Okoteve traduccin
#elif L==7
	 "Sesto"
#elif L==8
	 "12th"		// Potrzebujesz tlumaczenie
#elif L==9
	 "12th"		// Necessita de traduo
#endif
	};

const char *Txt_YES =
#if   L==1
	"S&Iacute;";
#elif L==2
	"JA";
#elif L==3
	"YES";
#elif L==4
	"S&Iacute;";
#elif L==5
	"OUI";
#elif L==6
	"HE&#7868;";
#elif L==7
	"SI";
#elif L==8
	"YES";		// Potrzebujesz tlumaczenie
#elif L==9
	"SIM";
#endif

const char *Txt_Yes =
#if   L==1
	"S&iacute;";
#elif L==2
	"Ja";
#elif L==3
	"Yes";
#elif L==4
	"S&iacute;";
#elif L==5
	"Oui";
#elif L==6
	"He&#7869;";
#elif L==7
	"S&igrave;";
#elif L==8
	"Yes";		// Potrzebujesz tlumaczenie
#elif L==9
	"Sim";
#endif

const char *Txt_Yesterday =
#if   L==1
	"Ahir";
#elif L==2
	"Gestern";
#elif L==3
	"Yesterday";
#elif L==4
	"Ayer";
#elif L==5
	"Hier";
#elif L==6
	"Kuehe";
#elif L==7
	"Ieri";
#elif L==8
	"Yesterday";	// Potrzebujesz tlumaczenie
#elif L==9
	"Ontem";
#endif

const char *Txt_yet_unnamed =
#if   L==1
	"encara sense nom";
#elif L==2
	"noch unbenannte";
#elif L==3
	"yet unnamed";
#elif L==4
	"a&uacute;n sin nombre";
#elif L==5
	"encore sans nom";
#elif L==6
	"a&uacute;n sin nombre";
#elif L==7
	"ancora senza nome";
#elif L==8
	"jeszcze nienazwany";
#elif L==9
	"ainda sem nome";
#endif

const char *Txt_You_already_answered_this_survey_before =
#if   L==1
	"Usted ya respondi&oacute; esta encuesta anteriormente.";	// Necessita traduccio
#elif L==2
	"Sie haben bereits beantwortet dieser Umfrage vor.";
#elif L==3
	"You already answered this survey before.";
#elif L==4
	"Usted ya respondi&oacute; esta encuesta anteriormente.";
#elif L==5
	"Vous avez d&eacute;j&agrave; r&eacute;pondu &agrave; ce sondage avant.";
#elif L==6
	"Usted ya respondi&oacute; esta encuesta anteriormente.";	// Okoteve traduccin
#elif L==7
	"Hai gi&agrave; risposto a questo sondaggio prima.";
#elif L==8
	"You already answered this survey before.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Voc&ecirc; j&aacute; respondeu a esso inqu&eacute;rito antes.";
#endif

const char *Txt_You_are_now_LOGGED_IN_as_X =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"Vost&egrave; est&agrave; ara %s com a <strong>%s</strong>.";
#elif L==2
	"Sie sind nun %s als <strong>%s</strong>.";
#elif L==3
	"You are now %s as <strong>%s</strong>.";
#elif L==4
	"Usted est&aacute; ahora %s como <strong>%s</strong>.";
#elif L==5
	"Vous &ecirc;tes maintenant %s en tant que <strong>%s</strong>.";
#elif L==6
	"Usted est&aacute; ahora %s como <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Sei ora %s come <strong>%s</strong>.";
#elif L==8
	"Jeste&sacute; teraz %s jako <strong>%s</strong>.";
#elif L==9
	"Agora voc&ecirc; est&aacute; %s como <strong>%s</strong>.";
#endif

const char *Txt_You_are_not_enrolled_in_any_course[Usr_NUM_SEXS] =
	{
#if   L==1	// Usr_SEX_UNKNOWN
	"Usted no est&aacute; inscrito/a en ninguna asignatura."	// Necessita traduccio
#elif L==2
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3
	"You are not enrolled in any course."
#elif L==4
	"Usted no est&aacute; inscrito/a en ninguna asignatura."
#elif L==5
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6
	"Usted no est&aacute; inscrito/a en ninguna asignatura."	// Okoteve traduccin
#elif L==7
	"Non ti sei iscritto/a ad alcun corso."
#elif L==8
	"You are not enrolled in any course."			// Potrzebujesz tlumaczenie
#elif L==9
	"Voc&ecirc; n&atilde;o est&aacute; matriculado/a em nenhuma disciplina."
#endif
	,
#if   L==1	// Usr_SEX_FEMALE
	"Usted no est&aacute; inscrita en ninguna asignatura."	// Necessita traduccio
#elif L==2
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3
	"You are not enrolled in any course."
#elif L==4
	"Usted no est&aacute; inscrita en ninguna asignatura."
#elif L==5
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6
	"Usted no est&aacute; inscrita en ninguna asignatura."	// Okoteve traduccin
#elif L==7
	"Non ti sei iscritta ad alcun corso."
#elif L==8
	"You are not enrolled in any course."			// Potrzebujesz tlumaczenie
#elif L==9
	"Voc&ecirc; n&atilde;o est&aacute; matriculada em nenhuma disciplina."
#endif
	,
#if   L==1	// Usr_SEX_MALE
	"Usted no est&aacute; inscrito en ninguna asignatura."	// Necessita traduccio
#elif L==2
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3
	"You are not enrolled in any course."
#elif L==4
	"Usted no est&aacute; inscrito en ninguna asignatura."
#elif L==5
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6
	"Usted no est&aacute; inscrito en ninguna asignatura."	// Okoteve traduccin
#elif L==7
	"Non ti sei iscritto ad alcun corso."
#elif L==8
	"You are not enrolled in any course."			// Potrzebujesz tlumaczenie
#elif L==9
	"Voc&ecirc; n&atilde;o est&aacute; matriculado em nenhuma disciplina."
#endif
	,
#if   L==1	// Usr_SEX_ALL
	"Usted no est&aacute; inscrito/a en ninguna asignatura."	// Necessita traduccio
#elif L==2
	"Sie sind zu keiner Kurs eingeschrieben."
#elif L==3
	"You are not enrolled in any course."
#elif L==4
	"Usted no est&aacute; inscrito/a en ninguna asignatura."
#elif L==5
	"Vous n'&ecirc;tes pas inscrit &agrave; une mati&egrave;re."
#elif L==6
	"Usted no est&aacute; inscrito/a en ninguna asignatura."	// Okoteve traduccin
#elif L==7
	"Non ti sei iscritto/a ad alcun corso."
#elif L==8
	"You are not enrolled in any course."			// Potrzebujesz tlumaczenie
#elif L==9
	"Voc&ecirc; n&atilde;o est&aacute; matriculado/a em nenhuma disciplina."
#endif
	};

const char *Txt_You_can_create_a_new_folder_inside_the_folder_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Podeu crear una nova carpeta"
	" dins de la carpeta <strong>%s</strong>:";
#elif L==2
	"Sie k&ouml;nnen ein neues Verzeichnis"
	" im Verzeichnis <strong>%s</strong> anlegen:";
#elif L==3
	"You can create a new folder"
	" inside the folder <strong>%s</strong>:";
#elif L==4
	"Puede crear una nueva carpeta"
	" dentro de la carpeta <strong>%s</strong>:";
#elif L==5
	"Vous pouvez cr&eacute;er un nouveau r&eacute;pertoire"
	" dans le r&eacute;pertoire <strong>%s</strong>:";
#elif L==6
	"Puede crear una nueva carpeta"
	" dentro de la carpeta <strong>%s</strong>:";	// Okoteve traduccin
#elif L==7
	"&Egrave; possibile creare una nuova cartella"
	" all'interno della cartella <strong>%s</strong>:";
#elif L==8
	"Mo&zdot;esz utworzy&cacute; nowy folder"
	" wewn&aogon;trz folderu <strong>%s</strong>:";
#elif L==9
	"Voc&ecirc; pode criar um novo diret&oacute;rio"
	" dentro do diret&oacute;rio <strong>%s</strong>:";
#endif

const char *Txt_You_can_not_create_folders_files_or_links_here =
#if   L==1
	"No puede crear carpetas, archivos o enlaces aqu&iacute;.";	// Necessita traduccio
#elif L==2
	"You can not create folders, files or links here.";		// Need bersetzung
#elif L==3
	"You can not create folders, files or links here.";
#elif L==4
	"No puede crear carpetas, archivos o enlaces aqu&iacute;.";
#elif L==5
	"You can not create folders, files or links here.";		// Besoin de traduction
#elif L==6
	"No puede crear carpetas, archivos o enlaces aqu&iacute;.";	// Okoteve traduccin
#elif L==7
	"Non puoi creare cartelle, file o linki qui.";
#elif L==8
	"You can not create folders, files or links here.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not create folders, files or links here.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_create_folders_here =
#if   L==1
	"No puede crear carpetas aqu&iacute;.";	// Necessita traduccio
#elif L==2
	"You can not create folders here.";	// Need bersetzung
#elif L==3
	"You can not create folders here.";
#elif L==4
	"No puede crear carpetas aqu&iacute;.";
#elif L==5
	"You can not create folders here.";	// Besoin de traduction
#elif L==6
	"No puede crear carpetas aqu&iacute;.";	// Okoteve traduccin
#elif L==7
	"Non puoi creare cartelle qui.";
#elif L==8
	"You can not create folders here.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You can not create folders here.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_create_links_here =
#if   L==1
	"No puede crear enlaces aqu&iacute;.";	// Necessita traduccio
#elif L==2
	"You can not create links here.";	// Need bersetzung
#elif L==3
	"You can not create links here.";
#elif L==4
	"No puede crear enlaces aqu&iacute;.";
#elif L==5
	"You can not create links here.";	// Besoin de traduction
#elif L==6
	"No puede crear enlaces aqu&iacute;.";	// Okoteve traduccin
#elif L==7
	"Non puoi creare linki qui.";
#elif L==8
	"You can not create links here.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You can not create links here.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_delete_your_current_nickname =
#if   L==1
	"No puede eliminar su apodo actual.";		// Necessita traduccio
#elif L==2
	"You can not delete your current nickname.";	// Need bersetzung
#elif L==3
	"You can not delete your current nickname.";
#elif L==4
	"No puede eliminar su apodo actual.";
#elif L==5
	"You can not delete your current nickname.";	// Besoin de traduction
#elif L==6
	"No puede eliminar su apodo actual.";		// Okoteve traduccin
#elif L==7
	"Non puoi cancellare il tuo nome utente attuale.";
#elif L==8
	"You can not delete your current nickname.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You can not delete your current nickname.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_delete_this_ID =
#if   L==1
	"No puede eliminar este ID (DNI/c&eacute;dula).";	// Necessita traduccio
#elif L==2
	"You can not delete this ID.";				// Need bersetzung
#elif L==3
	"You can not delete this ID.";
#elif L==4
	"No puede eliminar este ID (DNI/c&eacute;dula).";
#elif L==5
	"You can not delete this ID.";				// Besoin de traduction
#elif L==6
	"No puede eliminar este ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==7
	"Non puoi cancellare questo ID.";
#elif L==8
	"You can not delete this ID.";				// Potrzebujesz tlumaczenie
#elif L==9
	"You can not delete this ID.";				// Necessita de traduo
#endif

const char *Txt_You_can_not_disable_file_uploading_once_folders_have_been_created =
#if   L==1
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";	// Necessita traduccio
#elif L==2
	"Sie k&ouml;nnen den Upload nicht deaktivieren,"
	" wenn bereits Verzeichnisse angelegt worden sind.";
#elif L==3
	"You can not disable file uploading"
	" once folders have been created.";
#elif L==4
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";
#elif L==5
	"Vous ne pouvez pas d&eacute;sactiver le t&eacute;l&eacute;chargement de fichiers,"
	" une fois des r&eacute;pertoires ont &eacute;t&eacute; cr&eacute;&eacute;s.";
#elif L==6
	"No puede desactivar la subida de archivos"
	" una vez que se han creado carpetas.";	// Okoteve traduccin
#elif L==7
	"Non &egrave; possibile disabilitare il caricamento dei file"
	" una volta che le cartelle sono state create.";
#elif L==8
	"You can not disable file uploading"
	" once folders have been created.";	// Potrzebujesz tlumaczenie
#elif L==9
	"Voc&ecirc; n&atilde;o pode desativar o upload de arquivos,"
	" uma vez diret&oacute;rios foram criados.";
#endif

const char *Txt_You_can_not_leave_empty_intermediate_answers =
#if   L==1
	"No puede dejar respuestas intermedias sin rellenar.";	// Necessita traduccio
#elif L==2
	"You can not leave empty intermediate answers.";	// Need bersetzung
#elif L==3
	"You can not leave empty intermediate answers.";
#elif L==4
	"No puede dejar respuestas intermedias sin rellenar.";
#elif L==5
	"You can not leave empty intermediate answers.";	// Besoin de traduction
#elif L==6
	"No puede dejar respuestas intermedias sin rellenar.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare vuote domande intermedie.";
#elif L==8
	"You can not leave empty intermediate answers.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave empty intermediate answers.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_description_empty =
#if   L==1
	"No puede dejar la descripci&oacute;n vac&iacute;a.";	// Necessita traduccio
#elif L==2
	"You can not leave the description empty.";		// Need bersetzung
#elif L==3
	"You can not leave the description empty.";
#elif L==4
	"No puede dejar la descripci&oacute;n vac&iacute;a.";
#elif L==5
	"You can not leave the description empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar la descripci&oacute;n vac&iacute;a.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare vuota la descrizione.";
#elif L==8
	"You can not leave the description empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the description empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_image_empty =
#if   L==1
	"No puede dejar la imagen vac&iacute;a.";	// Necessita traduccio
#elif L==2
	"You can not leave the image empty.";		// Need bersetzung
#elif L==3
	"You can not leave the image empty.";
#elif L==4
	"No puede dejar la imagen vac&iacute;a.";
#elif L==5
	"You can not leave the image empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar la imagen vac&iacute;a.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare l'image vuota.";
#elif L==8
	"You can not leave the image empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the image empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_IP_address_empty =
#if   L==1
	"No puede dejar la direcci&oacute;n IP vac&iacute;a.";	// Necessita traduccio
#elif L==2
	"You can not leave the IP address empty.";		// Need bersetzung
#elif L==3
	"You can not leave the IP address empty.";
#elif L==4
	"No puede dejar la direcci&oacute;n IP vac&iacute;a.";
#elif L==5
	"You can not leave the IP address empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar la direcci&oacute;n IP vac&iacute;a.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare vuoto l'indirizzo IP.";
#elif L==8
	"You can not leave the IP address empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the IP address empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_logo_empty =
#if   L==1
	"No puede dejar el logo vac&iacute;o.";		// Necessita traduccio
#elif L==2
	"You can not leave the logo empty.";		// Need bersetzung
#elif L==3
	"You can not leave the logo empty.";
#elif L==4
	"No puede dejar el logo vac&iacute;o.";
#elif L==5
	"You can not leave the logo empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar el logo vac&iacute;o.";		// Okoteve traduccin
#elif L==7
	"Non puoi lasciare vuoto il logo.";
#elif L==8
	"You can not leave the logo empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the logo empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_banner_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del banner <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del banner <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del banner <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del banner <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the banner <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_centre_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del centro <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del centro <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del centro <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del centro <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the centre <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_course_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" de la asignatura <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" de la asignatura <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" de la asignatura <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del corso <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the course <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_country_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del pa%iacute;s <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del pa%iacute;s <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del pa%iacute;s <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del paese <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the country <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_degree_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" de la titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";			// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" de la titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";			// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" de la titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" della laurea <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the degree <strong>%s</strong> empty.";			// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_department_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del departamento <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";	// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del departamento <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del departamento <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del dipartimento <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the department <strong>%s</strong> empty.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_email_domain_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre "
	"del dominio de correo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the email domain <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the email domain <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del dominio de correo <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the email domain <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del dominio de correo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del campo mail <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the email domain <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the email domain <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_field_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del campo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";	// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del campo <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del campo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del campo <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the field <strong>%s</strong> empty.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_group_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del grupo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";	// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del grupo <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del grupo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del gruppo <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the group <strong>%s</strong> empty.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_holiday_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" de la festividad <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" de la festividad <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" de la festividad <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" della festivit&agrave; <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the holiday <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_institution_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" de la instituci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" de la instituci&oacute;n <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" de la instituci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" dell'istituzione <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the institution <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_link_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del enlace <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del enlace <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del enlace <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del link <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the link <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_place_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del lugar <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the place <strong>%s</strong> empty.";	// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the place <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del lugar <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the place <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del lugar <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del luogo <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the place <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the place <strong>%s</strong> empty.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_plugin_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del complemento <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";		// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del complemento <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del complemento <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del plugin <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the plugin <strong>%s</strong> empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_tag_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del descriptor <strong>%s</strong> vac&iacute;o.";		// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";			// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del descriptor <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";			// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del descriptor <strong>%s</strong> vac&iacute;o.";		// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" dell'etichetta <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the tag <strong>%s</strong> empty.";			// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_type_of_degree_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del tipo de titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";			// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del tipo de titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";			// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del tipo de titulaci&oacute;n <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del tipo di laurea <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the type of degree <strong>%s</strong> empty.";			// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_name_of_the_type_of_group_X_empty =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"No puede dejar el nombre"
	" del tipo de grupo <strong>%s</strong> vac&iacute;o.";	// Necessita traduccio
#elif L==2
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";	// Need bersetzung
#elif L==3
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";
#elif L==4
	"No puede dejar el nombre"
	" del tipo de grupo <strong>%s</strong> vac&iacute;o.";
#elif L==5
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";	// Besoin de traduction
#elif L==6
	"No puede dejar el nombre"
	" del tipo de grupo <strong>%s</strong> vac&iacute;o.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare il nome"
	" del tipo di gruppo <strong>%s</strong> vuoto.";
#elif L==8
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the name"
	" of the type of group <strong>%s</strong> empty.";	// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_URL_empty =
#if   L==1
	"No puede dejar el URL vac&iacute;o.";		// Necessita traduccio
#elif L==2
	"You can not leave the URL empty.";		// Need bersetzung
#elif L==3
	"You can not leave the URL empty.";
#elif L==4
	"No puede dejar el URL vac&iacute;o.";
#elif L==5
	"You can not leave the URL empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar el URL vac&iacute;o.";		// Okoteve traduccin
#elif L==7
	"Non puoi lasciare l'URL vuoto.";
#elif L==8
	"You can not leave the URL empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the URL empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_leave_the_web_address_empty =
#if   L==1
	"No puede dejar la direcci&oacute;n web vac&iacute;a.";	// Necessita traduccio
#elif L==2
	"You can not leave the web address empty.";		// Need bersetzung
#elif L==3
	"You can not leave the web address empty.";
#elif L==4
	"No puede dejar la direcci&oacute;n web vac&iacute;a.";
#elif L==5
	"You can not leave the web address empty.";		// Besoin de traduction
#elif L==6
	"No puede dejar la direcci&oacute;n web vac&iacute;a.";	// Okoteve traduccin
#elif L==7
	"Non puoi lasciare l'indirizzo web vuoto.";
#elif L==8
	"You can not leave the web address empty.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not leave the web address empty.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_paste_file_or_folder_here =
#if   L==1
	"No puede pegar un archivo o carpeta aqu&iacute;.";		// Necessita traduccio
#elif L==2
	"You can not paste file or folder here.";			// Need bersetzung
#elif L==3
	"You can not paste file or folder here.";
#elif L==4
	"No puede pegar un archivo o carpeta aqu&iacute;.";
#elif L==5
	"You can not paste file or folder here.";			// Besoin de traduction
#elif L==6
	"No puede pegar un archivo o carpeta aqu&iacute;.";		// Okoteve traduccin
#elif L==7
	"Non puoi copiare file o cartella qui.";
#elif L==8
	"You can not paste file or folder here.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You can not paste file or folder here.";			// Necessita de traduo
#endif

const char *Txt_You_can_not_remove_this_file_or_link =
#if   L==1
	"No puede eliminar este archivo o enlace.";		// Necessita traduccio
#elif L==2
	"You can not remove this file or link.";		// Need bersetzung
#elif L==3
	"You can not remove this file or link.";
#elif L==4
	"No puede eliminar este archivo o enlace.";
#elif L==5
	"You can not remove this file or link.";		// Besoin de traduction
#elif L==6
	"No puede eliminar este archivo o enlace.";		// Okoteve traduccin
#elif L==7
	"Non puoi rimuovere questo file o link.";
#elif L==8
	"You can not remove this file or link.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not remove this file or link.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_remove_this_folder =
#if   L==1
	"No puede eliminar esta carpeta.";		// Necessita traduccio
#elif L==2
	"You can not remove this folder.";		// Need bersetzung
#elif L==3
	"You can not remove this folder.";
#elif L==4
	"No puede eliminar esta carpeta.";
#elif L==5
	"You can not remove this folder.";		// Besoin de traduction
#elif L==6
	"No puede eliminar esta carpeta.";		// Okoteve traduccin
#elif L==7
	"Non puoi rimuovere questa cartella.";
#elif L==8
	"You can not remove this folder.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not remove this folder.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_remove_a_country_with_institutions_or_users =
#if   L==1
	"No puede eliminar un pa&iacute;s con instituciones o usuarios.";	// Necessita traduccio
#elif L==2
	"You can not remove a country with institutions or users.";		// Need bersetzung
#elif L==3
	"You can not remove a country with institutions or users.";
#elif L==4
	"No puede eliminar un pa&iacute;s con instituciones o usuarios.";
#elif L==5
	"You can not remove a country with instituciones or users.";		// Besoin de traduction
#elif L==6
	"No puede eliminar un pa&iacute;s con instituciones o usuarios.";	// Okoteve traduccin
#elif L==7
	"Non puoi rimuovere un paese con istituzioni o utenti.";
#elif L==8
	"You can not remove a country with instituciones or users.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not remove a country with instituciones or users.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_rename_this_folder =
#if   L==1
	"No puede cambiar el nombre de esta carpeta.";	// Necessita traduccio
#elif L==2
	"You can not rename this folder.";		// Need bersetzung
#elif L==3
	"You can not rename this folder.";
#elif L==4
	"No puede cambiar el nombre de esta carpeta.";
#elif L==5
	"You can not rename this folder.";		// Besoin de traduction
#elif L==6
	"No puede cambiar el nombre de esta carpeta.";	// Okoteve traduccin
#elif L==7
	"Non puoi rinominare questa cartella.";
#elif L==8
	"You can not rename this folder.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not rename this folder.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_send_a_message_to_so_many_recipients_ =
#if   L==1
	"No puede enviar un mensaje a tantsimos destinatarios."
	" Es mejor usar los foros.";		// Necessita traduccio
#elif L==2
	"You can not send a message to so many recipients."
	" Better use the forums.";		// Need bersetzung
#elif L==3
	"You can not send a message to so many recipients."
	" Better use the forums.";
#elif L==4
	"No puede enviar un mensaje a tantsimos destinatarios."
	" Es mejor usar los foros.";
#elif L==5
	"You can not send a message to so many recipients."
	" Better use the forums.";		// Besoin de traduction
#elif L==6
	"No puede enviar un mensaje a tantsimos destinatarios."
	" Es mejor usar los foros.";		// Okoteve traduccin
#elif L==7
	"Non puoi inviare un messaggio a cos tanti destinatari."
	" Meglio usare il forum.";
#elif L==8
	"You can not send a message to so many recipients."
	" Better use the forums.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can not send a message to so many recipients."
	" Better use the forums.";		// Necessita de traduo
#endif

const char *Txt_You_can_not_take_a_new_test_until =
#if   L==1
	"No podr&agrave; fer un test fins";
#elif L==2
	"Sie k&ouml;nnen nicht einen neuen Test bis";
#elif L==3
	"You can not take a new test until";
#elif L==4
	"No podr&aacute; hacer un nuevo test hasta";
#elif L==5
	"Vous ne pouvez pas passer un nouveau test avant";
#elif L==6
	"No podr&aacute; hacer un nuevo test hasta";	// Okoteve traduccin
#elif L==7
	"Non si pu&ograve; fare un nuovo test fino";
#elif L==8
	"Nie mo&zdot;na wykona&cacute; nowy test do";
#elif L==9
	"Voc&ecirc; n&atilde;o pode fazer um novo teste at&eacute;";
#endif

const char *Txt_You_can_only_perform_X_further_actions_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Solo podr&aacute; realizar %u acciones m&aacute;s"
	" si no env&iacute;a su fotograf&iacute;a.";		// Necessita traduccio
#elif L==2
	"You can only perform %u further actions"
	" if you don't send your photo.";			// Need bersetzung
#elif L==3
	"You can only perform %u further actions"
	" if you don't send your photo.";
#elif L==4
	"Solo podr&aacute; realizar %u acciones m&aacute;s"
	" si no env&iacute;a su fotograf&iacute;a.";
#elif L==5
	"You can only perform %u further actions"
	" if you don't send your photo.";			// Besoin de traduction
#elif L==6
	"Solo podr&aacute; realizar %u acciones m&aacute;s"
	" si no env&iacute;a su fotograf&iacute;a.";		// Okoteve traduccin
#elif L==7
	"Puoi solo realizzare %u massime azioni"
	" se non hai inviato la tua foto.";
#elif L==8
	"You can only perform %u further actions"
	" if you don't send your photo.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You can only perform %u further actions"
	" if you don't send your photo.";			// Necessita de traduo
#endif

const char *Txt_You_can_only_receive_email_notifications_if_ =	// Warning: it is very important to include four %s in the following sentences
#if   L==1
	"Nom&eacute;s podr&agrave; rebre notificacions per correu"
	" si la seva adre&ccedil;a de correu est&agrave; confirmada (%s &gt; %s)"
	" i acaba en un dels dominis permesos (%s &gt; %s).";
#elif L==2
	"Sie k&ouml;nnen nur E-Mail-Anmeldungen zu erhalten,"
	" wenn Sie Ihre E-Mail-Adresse best&uuml;tigt wird (%s &gt; %s),"
	" und endet in einer der erlaubt Domains (%s &gt; %s).";
#elif L==3
	"You can only receive email notifications"
	" if your email address is confirmed (%s &gt; %s)"
	" and ends in one of the allowed domains (%s &gt; %s).";
#elif L==4
	"Solo podr&aacute; recibir notificaciones por correo"
	" si su direcci&oacute;n de correo est&aacute; confirmada (%s &gt; %s)"
	" y termina en uno de los dominios permitidos (%s &gt; %s).";
#elif L==5
	"Vous ne pouvez recevoir des notifications par email"
	" si votre adresse email est confirm&eacute; (%s &gt; %s)"
	" et se termine dans l'un des domaines autoris&eacute;s (%s &gt; %s).";
#elif L==6
	"Solo podr&aacute; recibir notificaciones por correo"
	" si su direcci&oacute;n de correo est&aacute; confirmada (%s &gt; %s)"
	" y termina en uno de los dominios permitidos (%s &gt; %s).";				// Okoteve traduccin
#elif L==7
	"Puoi solo ricevere email di notifica"
	" se la tua email &egrave; confermata (%s &gt; %s)"
	" e si termina in uno dei domini consentiti (%s &gt; %s).";
#elif L==8
	"Mo&zdot;na otrzyma&cacute; tylko informacj&eogon;,"
	" czy adres email jest potwierdzone (%s &gt; %s),"
	" a ko&nacute;czy si&eogon; w jednej z dozwolone domeny (%s &gt; %s).";
#elif L==9
	"Voc&ecirc; s&oacute; pode receber notifica&ccedil;&otilde;es de email"
	" se o seu endere&ccedil;o de email est&aacute; confirmado (%s &gt; %s)"
	" e termina em um dos dom&iacute;nios permitidos (%s &gt; %s).";
#endif

const char *Txt_You_can_register_voluntarily_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Puede apuntarse voluntariamente a un grupo de tipo %s.";	// Necessita traduccio
#elif L==2
	"You can register voluntarily in one group of type %s.";	// Need bersetzung
#elif L==3
	"You can register voluntarily in one group of type %s.";
#elif L==4
	"Puede apuntarse voluntariamente a un grupo de tipo %s.";
#elif L==5
	"You can register voluntarily in one group of type %s.";	// Besoin de traduction
#elif L==6
	"Puede apuntarse voluntariamente a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==7
	"Ti puoi registrare volontariamente in uno dei gruppi di tipo %s.";
#elif L==8
	"You can register voluntarily in one group of type %s.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You can register voluntarily in one group of type %s.";	// Necessita de traduo
#endif

const char *Txt_You_can_register_voluntarily_in_one_or_more_groups_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Puede apuntarse voluntariamente"
	" a uno o a varios grupos de tipo %s.";		// Necessita traduccio
#elif L==2
	"You can register voluntarily"
	" in one or more groups of type %s.";		// Need bersetzung
#elif L==3
	"You can register voluntarily"
	" in one or more groups of type %s.";
#elif L==4
	"Puede apuntarse voluntariamente"
	" a uno o a varios grupos de tipo %s.";
#elif L==5
	"You can register voluntarily"
	" in one or more groups of type %s.";		// Besoin de traduction
#elif L==6
	"Puede apuntarse voluntariamente"
	" a uno o a varios grupos de tipo %s.";		// Okoteve traduccin
#elif L==7
	"Ti puoi registrare volontariamente"
	" in uno o pi&ugrave; gruppi di tipo %s.";
#elif L==8
	"You can register voluntarily"
	" in one or more groups of type %s.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You can register voluntarily"
	" in one or more groups of type %s.";		// Necessita de traduo
#endif

const char *Txt_You_can_search_for_courses_select_them_and_request_your_enrollment_in_them =
#if   L==1
	"Podeu cercar assignatures, seleccionar-les"
	" i sol&middot;licitar la seva inscripci&oacute; en elles.";
#elif L==2
	"Sie k&ouml;nnen f&uuml;r Kurse zu suchen, w&auml;hlen Sie sie aus"
	" und fordern Sie Ihre Anmeldung in ihnen.";
#elif L==3
	"You can search for courses, select them"
	" and request your enrollment in them.";
#elif L==4
	"Puede buscar asignaturas, seleccionarlas"
	" y solicitar su inscripci&oacute;n en ellas.";
#elif L==5
	"Vous pouvez rechercher des cours, s&eacute;lectionnez-les"
	" et demander votre inscription en eux.";
#elif L==6
	"Puede buscar asignaturas, seleccionarlas"
	" y solicitar su inscripci&oacute;n en ellas.";		// Okoteve traduccin"
#elif L==7
	"&Egrave; possibile cercare corsi, selezionarli"
	" e richiedere l'iscrizione in loro.";
#elif L==8
	"Mo&zdot;esz szuka&cacute; kurs&oacute;w, zaznacz je"
	" i poprosi&cacute; o rejestracj&eogon; w nich.";
#elif L==9
	"Voc&ecirc; pode procurar por cursos, selecion&aacute;-los"
	" e solicitar a sua inscri&ccedil;&atilde;o nos mesmos.";
#endif

const char *Txt_You_can_send_a_file_with_an_image_in_PNG_format_transparent_background_and_size_X_Y =	// Warning: it is very important to include two %u in the following sentences
#if   L==1
	"Puede enviar un archivo con una imagen en formato <em>PNG</em>,"
	" fondo transparente y tama&ntilde;o %u&times;%u p&iacute;xeles.";	// Necessita traduccio
#elif L==2
	"You can send a file with an image in <em>PNG</em> format,"
	" transparent background and size %u&times;%u pixels.";			// Need bersetzung
#elif L==3
	"You can send a file with an image in <em>PNG</em> format,"
	" transparent background and size %u&times;%u pixels.";
#elif L==4
	"Puede enviar un archivo con una imagen en formato <em>PNG</em>,"
	" fondo transparente y tama&ntilde;o %u&times;%u p&iacute;xeles.";
#elif L==5
	"You can send a file with an image in <em>PNG</em> format,"
	" transparent background and size %u&times;%u pixels.";			// Besoin de traduction
#elif L==6
	"Puede enviar un archivo con una imagen en formato <em>PNG</em>,"
	" fondo transparente y tama&ntilde;o %u&times;%u p&iacute;xeles.";	// Okoteve traduccin
#elif L==7
	"Puoi inviare un file con un'immagine in formato <em>PNG</em>,"
	" sfondo trasparente e di dimensione %u&times;%u pixel.";
#elif L==8
	"You can send a file with an image in <em>PNG</em> format,"
	" transparent background and size %u&times;%u pixels.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You can send a file with an image in <em>PNG</em> format,"
	" transparent background and size %u&times;%u pixels.";			// Necessita de traduo
#endif

const char *Txt_You_can_send_a_file_with_an_image_in_JPEG_format_ =
#if   L==1
	"Podeu enviar un arxiu amb una imatge en format <em>JPEG</em>.<br />"
	"La imatge ha de contenir almenys un rostre en posici&oacute; vertical i frontal,"
	" i amb el fons blanc o molt clar.";
#elif L==2
	"Sie k&ouml;nnen eine Datei mit einem Bild im <em>JPEG</em>-Format senden.<br />"
	"Das Bild muss mindestens eine Fl&auml;che in vertikaler und frontaler Position enthalten,"
	" und der Hintergrund muss wei&szlig; oder sehr hell sein.";
#elif L==3
	"You can send a file with an image in <em>JPEG</em> format.<br />"
	"The image must contain at least one face in vertical and frontal position,"
	" and the background must be white or very light.";
#elif L==4
	"Puede enviar un archivo con una imagen en formato <em>JPEG</em>.<br />"
	"La imagen debe contener al menos un rostro en posici&oacute;n vertical y frontal,"
	" y con el fondo blanco o muy claro.";
#elif L==5
	"Vous pouvez envoyer un fichier avec une image au format JPEG.<br />"
	"L'image doit contenir au moins un visage en position verticale et frontale,"
	" et le fond doit &ecirc;tre blanc ou tr&egrave;s clair.";
#elif L==6
	"Puede enviar un archivo con una imagen en formato <em>JPEG</em>.<br />"
	"La imagen debe contener al menos un rostro en posici&oacute;n vertical y frontal,"
	" y con el fondo blanco o muy claro.";				// Okoteve traduccin
#elif L==7
	"&Egrave; possibile inviare un file in un'immagine con formato <em>JPEG</em>.<br />"
	"L'immagine deve contenere almeno una faccia in posizione verticale e frontale,"
	" e lo sfondo deve essere bianco o molto chiaro.";
#elif L==8
	"Mo&zdot;esz wys&lstrok;a&cacute; plik obrazu w formacie <em>JPEG</em>.<br />"
	"Obraz musi zawiera&cacute; co najmniej jedn&aogon; twarz w pozycji pionowej jak i czo&lstrok;owego,"
	" a t&lstrok;o musi by&cacute; bia&lstrok;y lub bardzo jasnym.";
#elif L==9
	"Voc&ecirc; pode enviar um arquivo em uma imagem com formato <em>JPEG</em>.<br />"
	"A imagem deve conter pelo menos um rosto na posi&ccedil;&atilde;o vertical e frontal,"
	" eo fundo deve ser branco ou muito claro.";
#endif

const char *Txt_You_dont_follow_any_user =
#if   L==1
	"Vost&egrave; no segueix a cap usuari.";
#elif L==2
	"Sie haben noch keine Benutzer folgen.";
#elif L==3
	"You don't follow any user.";
#elif L==4
	"Usted no sigue a ning&uacute;n usuario.";
#elif L==5
	"Vous ne suivez pas aucun utilisateur.";
#elif L==6
	"Usted no sigue a ning&uacute;n usuario.";	// Okoteve traduccin
#elif L==7
	"Non si seguono gli utenti.";
#elif L==8
	"Nie po &zdot;adnej u&zdot;ytkownika.";
#elif L==9
	"Voc&ecirc; n&atilde;o segue nenhum usu&aacute;rio.";
#endif

const char *Txt_You_dont_have_permission_to_access_to_this_forum =
#if   L==1
	"No t&eacute; perm&iacute;s per accedir a aquest f&ograve;rum.";
#elif L==2
	"Sie haben keine Berechtigung, dieses Forum benutzen.";
#elif L==3
	"You don't have permission to access to this forum.";
#elif L==4
	"Usted no tiene permiso para acceder a este foro.";
#elif L==5
	"Vous n'avez pas la permission d'acc&eacute;der &agrave; ce forum.";
#elif L==6
	"Usted no tiene permiso para acceder a este foro.";	// Okoteve traduccin
#elif L==7
	"Non si dispone dell'autorizzazione per accedere a questo forum.";
#elif L==8
	"Nie masz dost&eogon;pu do tej forum.";
#elif L==9
	"Voc&ecirc; n&atilde;o tem permiss&atilde;o para acessar este f&oacute;rum.";
#endif

const char *Txt_You_dont_have_permission_to_change_the_properties_of_file_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Usted no tiene permiso para cambiar las propiedades del archivo %s.";		// Necessita traduccio
#elif L==2
	"You don't have permission to change the properties of file %s.";		// Need bersetzung
#elif L==3
	"You don't have permission to change the properties of file %s.";
#elif L==4
	"Usted no tiene permiso para cambiar las propiedades del archivo %s.";
#elif L==5
	"You don't have permission to change the properties of file %s.";		// Besoin de traduction
#elif L==6
	"Usted no tiene permiso para cambiar las propiedades del archivo %s.";		// Okoteve traduccin
#elif L==7
	"You don't have permission to change the properties of file %s.";		// Bisogno di traduzione
#elif L==8
	"You don't have permission to change the properties of file %s.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You don't have permission to change the properties of file %s.";		// Necessita de traduo
#endif

const char *Txt_You_dont_have_permission_to_edit_this_course =
#if   L==1
	"Usted no tiene permiso para editar esta asignatura.";		// Necessita traduccio
#elif L==2
	"You don't have permission to edit this course.";		// Need bersetzung
#elif L==3
	"You don't have permission to edit this course.";
#elif L==4
	"Usted no tiene permiso para editar esta asignatura.";
#elif L==5
	"You don't have permission to edit this course.";		// Besoin de traduction
#elif L==6
	"Usted no tiene permiso para editar esta asignatura.";		// Okoteve traduccin
#elif L==7
	"Non hai l'autorizzazione di editare questo corso.";
#elif L==8
	"You don't have permission to edit this course.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You don't have permission to edit this course.";		// Necessita de traduo
#endif

const char *Txt_You_dont_have_permission_to_perform_this_action =
#if   L==1
	"Vost&egrave; no t&eacute; perm&iacute;s"
	" per realitzar aquesta acci&oacute;.";
#elif L==2
	"Sie haben keine Berechtigung,"
	" um diese Handlung durchzuf&uuml;hren.";
#elif L==3
	"You don't have permission"
	" to perform this action.";
#elif L==4
	"Usted no tiene permiso"
	" para realizar esta acci&oacute;n.";
#elif L==5
	"Vous n'avez pas la permission"
	" d'effectuer cette action.";
#elif L==6
	"Usted no tiene permiso"
	" para realizar esta acci&oacute;n.";		// Okoteve traduccin
#elif L==7
	"Non si dispone dell'autorizzazione"
	" per eseguire questa azione.";
#elif L==8
	"You don't have permission"
	" to perform this action.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Voc&ecirc; n&atilde;o tem permiss&atilde;o"
	" para realizar esta a&ccedil;&atilde;o.";
#endif

const char *Txt_You_have_confirmed_that_you_have_read_this_information =
#if   L==1
	"Usted ha confirmado que ha leido esta informaci&oacute;n.";	// Necessita traduccio
#elif L==2
	"You have confirmed that you have read this information.";	// Need bersetzung
#elif L==3
	"You have confirmed that you have read this information.";
#elif L==4
	"Usted ha confirmado que ha leido esta informaci&oacute;n.";
#elif L==5
	"You have confirmed that you have read this information.";	// Besoin de traduction
#elif L==6
	"Usted ha confirmado que ha leido esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Hai confermato che hai letto questa informazione.";
#elif L==8
	"You have confirmed that you have read this information.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You have confirmed that you have read this information.";	// Necessita de traduo
#endif

const char *Txt_You_have_confirmed_your_enrollment_in_the_course_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Usted ha confirmado su inscripci&oacute;n"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"You have confirmed your enrollment"
	" in the course <strong>%s</strong>.";		// Need bersetzung
#elif L==3
	"You have confirmed your enrollment"
	" in the course <strong>%s</strong>.";
#elif L==4
	"Usted ha confirmado su inscripci&oacute;n"
	" en la asignatura <strong>%s</strong>.";
#elif L==5
	"You have confirmed your enrollment"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"Usted ha confirmado su inscripci&oacute;n"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"Hai confermato la tua registrazione"
	" in questo corso <strong>%s</strong>.";
#elif L==8
	"You have confirmed your enrollment"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You have confirmed your enrollment"
	" in the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_You_have_eliminated_the_confirmation_that_you_have_read_this_information =
#if   L==1
	"Usted ha eliminado la confirmaci&oacute;n"
	" de que ha leido esta informaci&oacute;n.";	// Necessita traduccio
#elif L==2
	"You have eliminated the confirmation"
	" that you have read this information.";	// Need bersetzung
#elif L==3
	"You have eliminated the confirmation"
	" that you have read this information.";
#elif L==4
	"Usted ha eliminado la confirmaci&oacute;n"
	" de que ha leido esta informaci&oacute;n.";
#elif L==5
	"You have eliminated the confirmation"
	" that you have read this information.";	// Besoin de traduction
#elif L==6
	"Usted ha eliminado la confirmaci&oacute;n"
	" de que ha leido esta informaci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Hai rimosso la conferma"
	" che hai letto questa informazione.";
#elif L==8
	"You have eliminated the confirmation"
	" that you have read this information.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You have eliminated the confirmation"
	" that you have read this information.";	// Necessita de traduo
#endif

const char *Txt_You_have_no_notifications =
#if   L==1
	"No tens notificacions.";
#elif L==2
	"Sie haben keine Benachrichtigungen.";
#elif L==3
	"You have no notifications.";
#elif L==4
	"No tiene notificaciones.";
#elif L==5
	"Vous n'avez aucune notification.";
#elif L==6
	"No tiene notificaciones.";	// Okoteve traduccin
#elif L==7
	"Non ci sono notifiche.";
#elif L==8
	"Nie masz &zdot;adnych powiadomie&nacute;.";
#elif L==9
	"Voc&ecirc; n&atilde;o tem notifica&ccedil;&otilde;es.";
#endif

const char *Txt_You_have_no_unread_notifications =
#if   L==1
	"No tens notificacions sense llegir.";
#elif L==2
	"Sie haben keine ungelesenen Benachrichtigungen.";
#elif L==3
	"You have no unread notifications.";
#elif L==4
	"No tiene notificaciones sin leer.";
#elif L==5
	"Vous n'avez aucune notification non lue.";
#elif L==6
	"No tiene notificaciones sin leer.";	// Okoteve traduccin
#elif L==7
	"Non ci sono notifiche non lette.";
#elif L==8
	"Nie masz nieprzeczytanych powiadomie&nacute;.";
#elif L==9
	"Voc&ecirc; n&atilde;o tem notifica&ccedil;&otilde;es n&atilde;o lidas.";
#endif

const char *Txt_You_have_not_banned_any_sender =
#if   L==1
	"Usted no ha bloqueado ning&uacute;n remitente.";	// Necessita traduccio
#elif L==2
	"You have not banned any sender.";			// Need bersetzung
#elif L==3
	"You have not banned any sender.";
#elif L==4
	"Usted no ha bloqueado ning&uacute;n remitente.";
#elif L==5
	"You have not banned any sender.";			// Besoin de traduction
#elif L==6
	"Usted no ha bloqueado ning&uacute;n remitente.";	// Okoteve traduccin
#elif L==7
	"You have not banned any sender.";			// Bisogno di traduzione
#elif L==8
	"You have not banned any sender.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You have not banned any sender.";			// Necessita de traduo
#endif

const char *Txt_You_have_not_confirmed_the_action =
#if   L==1
	"No ha confirmat l'acci&oacute;.";
#elif L==2
	"Sie haben nicht die Aktion best&auml;tigt.";
#elif L==3
	"You have not confirmed the action.";
#elif L==4
	"No ha confirmado la acci&oacute;n.";
#elif L==5
	"Vous n'avez pas confirm&eacute; l'action.";
#elif L==6
	"No ha confirmado la acci&oacute;n.";		// Okoteve traduccin
#elif L==7
	"Non hai inserito correttamente la tua password.";
#elif L==8
	"Nie potwierdzi&lstrok;y dzia&lstrok;anie.";
#elif L==9
	"Voc&ecirc; ainda n&atilde;o confirmou a a&ccedil;&atilde;o.";
#endif

const char *Txt_You_have_not_entered_your_password_correctly =
#if   L==1
	"No ha introdu&iuml;t correctament la seva contrasenya.";
#elif L==2
	"Sie haben Ihr Passwort nicht korrekt eingegeben wurde.";
#elif L==3
	"You have not entered your password correctly.";
#elif L==4
	"No ha introducido correctamente su contrase&ntilde;a.";
#elif L==5
	"Vous n'avez pas entr&eacute; votre mot de passe correctement.";
#elif L==6
	"No ha introducido correctamente su contrase&ntilde;a.";	// Okoteve traduccin
#elif L==7
	"Non hai inserito correttamente la tua password.";
#elif L==8
	"Nie wpisa&lstrok;e&sacute; poprawnie has&lstrok;o.";
#elif L==9
	"Voc&ecirc; n&atilde;o digitou sua senha corretamente.";
#endif

const char *Txt_You_have_not_written_twice_the_same_new_password =
#if   L==1
	"Usted no ha escrito dos veces la misma contrase&ntilde;a nueva.";	// Necessita traduccio
#elif L==2
	"You have not written twice the same new password.";			// Need bersetzung
#elif L==3
	"You have not written twice the same new password.";
#elif L==4
	"Usted no ha escrito dos veces la misma contrase&ntilde;a nueva.";
#elif L==5
	"You have not written twice the same new password.";			// Besoin de traduction
#elif L==6
	"Usted no ha escrito dos veces la misma contrase&ntilde;a nueva.";	// Okoteve traduccin
#elif L==7
	"Non hai scritto due volte la stessa password nuova.";
#elif L==8
	"You have not written twice the same new password.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You have not written twice the same new password.";			// Necessita de traduo
#endif

const char *Txt_You_have_to_register_compulsorily_at_least_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Tiene que apuntarse obligatoriamente al menos a un grupo de tipo %s.";	// Necessita traduccio
#elif L==2
	"You have to register compulsorily at least in one group of type %s.";	// Need bersetzung
#elif L==3
	"You have to register compulsorily at least in one group of type %s.";
#elif L==4
	"Tiene que apuntarse obligatoriamente al menos a un grupo de tipo %s.";
#elif L==5
	"You have to register compulsorily at least in one group of type %s.";	// Besoin de traduction
#elif L==6
	"Tiene que apuntarse obligatoriamente al menos a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==7
	"Devi registrarti obbligatoriamente almeno in uno dei gruppi del tipo %s.";
#elif L==8
	"You have to register compulsorily at least in one group of type %s.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You have to register compulsorily at least in one group of type %s.";	// Necessita de traduo
#endif

const char *Txt_You_have_to_register_compulsorily_in_one_group_of_type_X =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Tiene que apuntarse obligatoriamente a un grupo de tipo %s.";	// Necessita traduccio
#elif L==2
	"You have to register compulsorily in one group of type %s.";	// Need bersetzung
#elif L==3
	"You have to register compulsorily in one group of type %s.";
#elif L==4
	"Tiene que apuntarse obligatoriamente a un grupo de tipo %s.";
#elif L==5
	"You have to register compulsorily in one group of type %s.";	// Besoin de traduction
#elif L==6
	"Tiene que apuntarse obligatoriamente a un grupo de tipo %s.";	// Okoteve traduccin
#elif L==7
	"Devi registrarti obbligatoriamente in uno dei gruppi del tipo %s.";
#elif L==8
	"You have to register compulsorily in one group of type %s.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You have to register compulsorily in one group of type %s.";	// Necessita de traduo
#endif

const char *Txt_You_have_written_1_post_in_this_thread =
#if   L==1
	"Usted ha escrito 1 mensaje en esta discusi&oacute;n";		// Necessita traduccio
#elif L==2
	"Sie haben 1 Nachricht in diesem Thread";
#elif L==3
	"You have written 1 post in this thread";
#elif L==4
	"Usted ha escrito 1 mensaje en esta discusi&oacute;n";
#elif L==5
	"Vous avez &eacute;crit 1 message dans ce fil";
#elif L==6
	"Usted ha escrito 1 mensaje en esta discusi&oacute;n";		// Okoteve traduccin
#elif L==7
	"Hai scritto 1 post in questa discussione";
#elif L==8
	"You have written 1 post in this thread";			// Potrzebujesz tlumaczenie
#elif L==9
	"Voc&ecirc; escreveu 1 post neste thread";
#endif

const char *Txt_You_have_written_X_posts_in_this_thread =		// Warning: it is very important to include %u in the following sentences
#if   L==1
	"Usted ha escrito %u mensajes en esta discusi&oacute;n";	// Necessita traduccio
#elif L==2
	"Sie haben %u Nachrichten in diesem Thread";
#elif L==3
	"You have written %u posts in this thread";
#elif L==4
	"Usted ha escrito %u mensajes en esta discusi&oacute;n";
#elif L==5
	"Vous avez &eacute;crit %u messages dans ce fil";
#elif L==6
	"Usted ha escrito %u mensajes en esta discusi&oacute;n";	// Okoteve traduccin
#elif L==7
	"Hai scritto %u post in questa discussione";
#elif L==8
	"You have written %u posts in this thread";			// Potrzebujesz tlumaczenie
#elif L==9
	"Voc&ecirc; escreveu %u posts neste thread";
#endif

const char *Txt_You_must_create_at_least_one_type_of_degree_before_creating_degrees =
#if   L==1
	"Debe crear al menos un tipo de titulaci&oacute;n"
	" antes de crear titulaciones.";		// Necessita traduccio
#elif L==2
	"You must create at least one type of degree"
	" before creating degrees.";			// bersetzung notwendig
#elif L==3
	"You must create at least one type of degree"
	" before creating degrees.";
#elif L==4
	"Debe crear al menos un tipo de titulaci&oacute;n"
	" antes de crear titulaciones.";
#elif L==5
	"You must create at least one type of degree"
	" before creating degrees.";			// Besoin de traduction
#elif L==6
	"Debe crear al menos un tipo de titulaci&oacute;n"
	" antes de crear titulaciones.";		// Okoteve traduccin
#elif L==7
	"Devi creare almeno un tipo di laurea"
	" prima di creare lauree.";
#elif L==8
	"You must create at least one type of degree"
	" before creating degrees.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You must create at least one type of degree"
	" before creating degrees.";			// Necessita de traduo
#endif

const char *Txt_You_must_enter_an_integer_value_as_the_correct_answer =
#if   L==1
	"Debe escribir un valor entero como respuesta correcta.";	// Necessita traduccio
#elif L==2
	"You must enter an integer value as the correct answer.";	// bersetzung notwendig
#elif L==3
	"You must enter an integer value as the correct answer.";
#elif L==4
	"Debe escribir un valor entero como respuesta correcta.";
#elif L==5
	"You must enter an integer value as the correct answer.";	// Besoin de traduction
#elif L==6
	"Debe escribir un valor entero como respuesta correcta.";	// Okoteve traduccin
#elif L==7
	"Devi inserire un valore intero come risposta corretta.";
#elif L==8
	"You must enter an integer value as the correct answer.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You must enter an integer value as the correct answer.";	// Necessita de traduo
#endif

const char *Txt_You_must_enter_the_range_of_floating_point_values_allowed_as_answer =
#if   L==1
	"Debe escribir el intervalo de valores reales permitidos como respuesta.";	// Necessita traduccio
#elif L==2
	"You must enter the range of floating point values allowed as answer.";		// bersetzung notwendig
#elif L==3
	"You must enter the range of floating point values allowed as answer.";
#elif L==4
	"Debe escribir el intervalo de valores reales permitidos como respuesta.";
#elif L==5
	"You must enter the range of floating point values allowed as answer.";		// Besoin de traduction
#elif L==6
	"Debe escribir el intervalo de valores reales permitidos como respuesta.";	// Okoteve traduccin
#elif L==7
	"Devi inserire il range di valori reali permessi come risposta.";
#elif L==8
	"You must enter the range of floating point values allowed as answer.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must enter the range of floating point values allowed as answer.";		// Necessita de traduo
#endif

const char *Txt_You_must_enter_your_nick_email_or_ID =
#if   L==1
	"Ha d'escriure el seu @sobrenom, correu o ID (DNI/c&egrave;dula).";
#elif L==2
	"Bitte geben Sie Ihre @Benutzername, Email oder Ausweis-Nr.";
#elif L==3
	"You must enter your @nick, email or ID.";
#elif L==4
	"Debe escribir su @apodo, correo o ID (DNI/c&eacute;dula).";
#elif L==5
	"Vous devez entrer votre @surnom, courrier ou num&eacute;ro d'identit&eacute;.";
#elif L==6
	"Debe escribir su @apodo, correo o ID (DNI/c&eacute;dula).";	// Okoteve traduccin
#elif L==7
	"Devi inserire il tuo @nome-utente, email o numero carta d'identit&agrave;.";
#elif L==8
	"Musisz napisa&cacute; @pseudonim, email lub ID.";
#elif L==9
	"Voc&ecirc; deve digitar seu @alcunha, email ou n&ordm; identif.";
#endif

const char *Txt_You_must_mark_an_answer_as_correct =
#if   L==1
	"Debe marcar alguna respuesta como correcta.";	// Necessita traduccio
#elif L==2
	"You must mark an answer as correct.";		// Need bersetzung
#elif L==3
	"You must mark an answer as correct.";
#elif L==4
	"Debe marcar alguna respuesta como correcta.";
#elif L==5
	"You must mark an answer as correct.";		// Besoin de traduction
#elif L==6
	"Debe marcar alguna respuesta como correcta.";	// Okoteve traduccin
#elif L==7
	"Devi segnare una risposta come corretta.";
#elif L==8
	"You must mark an answer as correct.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must mark an answer as correct.";		// Necessita de traduo
#endif

const char *Txt_You_must_select_a_T_F_answer =
#if   L==1
	"Debe seleccionar una respuesta V o F.";	// Necessita traduccio
#elif L==2
	"You must select a T/F answer.";		// Need bersetzung
#elif L==3
	"You must select a T/F answer.";
#elif L==4
	"Debe seleccionar una respuesta V o F.";
#elif L==5
	"You must select a T/F answer.";		// Besoin de traduction
#elif L==6
	"Debe seleccionar una respuesta V o F.";	// Okoteve traduccin
#elif L==7
	"Devi selezionare una risposta V/F.";
#elif L==8
	"You must select a T/F answer.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must select a T/F answer.";		// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_recipients =
#if   L==1
	"Debe seleccionar uno o m&aacute;s destinatarios.";	// Necessita traduccio
#elif L==2
	"Es muss mindestens ein Empf&auml;nger angegeben werden.";
#elif L==3
	"You must select one or more recipients.";
#elif L==4
	"Debe seleccionar uno o m&aacute;s destinatarios.";
#elif L==5
	"You must select one or more recipients.";		// Besoin de traduction
#elif L==6
	"Debe seleccionar uno o m&aacute;s destinatarios.";	// Okoteve traduccin
#elif L==7
	"Devi selezionare uno o pi&ugrave; destinatari.";
#elif L==8
	"You must select one or more recipients.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must select one or more recipients.";		// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_students =
#if   L==1
	"Debe seleccionar uno o m&aacute;s estudiantes.";	// Necessita traduccio
#elif L==2
	"Es muss mindestens ein Student ausgew&auml;hlt werden.";
#elif L==3
	"You must select one or more students.";
#elif L==4
	"Debe seleccionar uno o m&aacute;s estudiantes.";
#elif L==5
	"You must select one or more students.";		// Besoin de traduction
#elif L==6
	"Debe seleccionar uno o m&aacute;s estudiantes.";	// Okoteve traduccin
#elif L==7
	"Devi selezionare uno o pi&ugrave; studenti.";
#elif L==8
	"You must select one or more students.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must select one or more students.";		// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_tags =
#if   L==1
	"Debe seleccionar uno o m&aacute;s descriptores.";	// Necessita traduccio
#elif L==2
	"Es muss mindestens ein Tag ausgew&auml;hlt werden.";
#elif L==3
	"You must select one or more tags.";
#elif L==4
	"Debe seleccionar uno o m&aacute;s descriptores.";
#elif L==5
	"You must select one or more tags.";			// Besoin de traduction
#elif L==6
	"Debe seleccionar uno o m&aacute;s descriptores.";	// Okoteve traduccin
#elif L==7
	"Devi selezionare una o pi&ugrave; etichette.";
#elif L==8
	"You must select one or more tags.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You must select one or more tags.";			// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_teachers =
#if   L==1
	"Debe seleccionar uno o m&aacute;s profesores.";		// Necessita traduccio
#elif L==2
	"Es muss mindestens ein Lehrer ausgew&auml;hlt werden.";
#elif L==3
	"You must select one or more teachers.";
#elif L==4
	"Debe seleccionar uno o m&aacute;s profesores.";
#elif L==5
	"You must select one or more teachers.";			// Besoin de traduction
#elif L==6
	"Debe seleccionar uno o m&aacute;s profesores.";		// Okoteve traduccin
#elif L==7
	"Devi selezionare uno o pi&ugrave; professori.";
#elif L==8
	"You must select one or more teachers.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You must select one or more teachers.";			// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_types_of_answer =
#if   L==1
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.";	// Necessita traduccio
#elif L==2
	"Es muss mindestens ein Antworttyp ausgew&auml;hlt werden.";
#elif L==3
	"You must select one or more types of answer.";
#elif L==4
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.";
#elif L==5
	"You must select one or more types of answer.";			// Besoin de traduction
#elif L==6
	"Debe seleccionar uno o m&aacute;s tipos de respuesta.";	// Okoteve traduccin
#elif L==7
	"Devi selezionare uno o pi&ugrave; tipi di risposta.";
#elif L==8
	"You must select one or more types of answer.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You must select one or more types of answer.";			// Necessita de traduo
#endif

const char *Txt_You_must_select_one_ore_more_users =
#if   L==1
	"Debe seleccionar uno o m&aacute;s usuarios.";	// Necessita traduccio
#elif L==2
	"Es muss mindestens ein Benutzer ausgew&auml;hlt werden.";
#elif L==3
	"You must select one or more users.";
#elif L==4
	"Debe seleccionar uno o m&aacute;s usuarios.";
#elif L==5
	"You must select one or more users.";		// Besoin de traduction
#elif L==6
	"Debe seleccionar uno o m&aacute;s usuarios.";	// Okoteve traduccin
#elif L==7
	"Devi selezionare uno o pi&ugrave; utenti.";
#elif L==8
	"You must select one or more users.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must select one or more users.";		// Necessita de traduo
#endif

const char *Txt_You_must_send_your_photo_because_ =
#if   L==1
	"Tiene que enviar su fotograf&iacute;a,"
	" ya que ha agotado el n&uacute;mero de accesos"
	" permitido sin fotograf&iacute;a.";	// Necessita traduccio
#elif L==2
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// Need bersetzung
#elif L==3
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";
#elif L==4
	"Tiene que enviar su fotograf&iacute;a,"
	" ya que ha agotado el n&uacute;mero de accesos"
	" permitido sin fotograf&iacute;a.";
#elif L==5
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// Besoin de traduction
#elif L==6
	"Tiene que enviar su fotograf&iacute;a,"
	" ya que ha agotado el n&uacute;mero de accesos"
	" permitido sin fotograf&iacute;a.";	// Okoteve traduccin
#elif L==7
	"Devi inviare la tua foto,"
	" perch&egrave; hai esaurito il numero di accessi"
	" permessi senza foto.";
#elif L==8
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must send your photo,"
	" because you have exhausted the number of clicks"
	" allowed without photo.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_image_of_the_new_banner =
#if   L==1
	"Debe especificar la imagen"
	" del nuevo banner.";		// Necessita traduccio
#elif L==2
	"You must specify the image"
	" of the new banner.";		// Need bersetzung
#elif L==3
	"You must specify the image"
	" of the new banner.";
#elif L==4
	"Debe especificar la imagen"
	" del nuevo banner.";
#elif L==5
	"You must specify the image"
	" of the new banner.";		// Besoin de traduction
#elif L==6
	"Debe especificar la imagen"
	" del nuevo banner.";		// Okoteve traduccin
#elif L==7
	"Devi specificare l'image"
	" del nuovo banner.";
#elif L==8
	"You must specify the image"
	" of the new banner.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the image"
	" of the new banner.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_logo_the_application_key_the_URL_and_the_IP_address_of_the_new_plugin =
#if   L==1
	"Debe especificar el logo, la clave de aplicaci&oacute;n,"
	" el URL y la IP del nuevo complemento.";		// Necessita traduccio
#elif L==2
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// Need bersetzung
#elif L==3
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";
#elif L==4
	"Debe especificar el logo, la clave de aplicaci&oacute;n,"
	" el URL y la IP del nuevo complemento.";
#elif L==5
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// Besoin de traduction
#elif L==6
	"Debe especificar el logo, la clave de aplicaci&oacute;n,"
	" el URL y la IP del nuevo complemento.";		// Okoteve traduccin
#elif L==7
	"Devi specificare il logo, il codice di applicazione,"
	" l'URL e l'indirizzo IP del nuovo plugin.";
#elif L==8
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the logo, the application key,"
	" the URL and the IP address of the new plugin.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_numerical_code_of_the_new_country =
#if   L==1
	"Debe especificar el c&oacute;digo num&eacute;rico ISO 3166-1 del nuevo pa&iacute;s.";	// Necessita traduccio
#elif L==2
	"You must specify the ISO 3166-1 numerical code of the new country.";			// Need bersetzung
#elif L==3
	"You must specify the ISO 3166-1 numerical code of the new country.";
#elif L==4
	"Debe especificar el c&oacute;digo num&eacute;rico ISO 3166-1 del nuevo pa&iacute;s.";
#elif L==5
	"You must specify the ISO 3166-1 numerical code of the new country.";			// Besoin de traduction
#elif L==6
	"Debe especificar el c&oacute;digo num&eacute;rico ISO 3166-1 del nuevo pa&iacute;s.";	// Okoteve traduccin
#elif L==7
	"Devi specificare il codice numerico ISO 3166-1 del nuovo paese.";
#elif L==8
	"You must specify the ISO 3166-1 numerical code of the new country.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the ISO 3166-1 numerical code of the new country.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_and_the_description_of_the_new_plugin =
#if   L==1
	"Debe especificar el nombre y la descripci&oacute;n del nuevo complemento.";	// Necessita traduccio
#elif L==2
	"You must specify the name and the description of the new plugin.";		// Need bersetzung
#elif L==3
	"You must specify the name and the description of the new plugin.";
#elif L==4
	"Debe especificar el nombre y la descripci&oacute;n del nuevo complemento.";
#elif L==5
	"You must specify the name and the description of the new plugin.";		// Besoin de traduction
#elif L==6
	"Debe especificar el nombre y la descripci&oacute;n del nuevo complemento.";	// Okoteve traduccin
#elif L==7
	"Devi specificare il nome e la descrizione del nuovo plugin.";
#elif L==8
	"You must specify the name and the description of the new plugin.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the name and the description of the new plugin.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_country_in_all_languages =
#if   L==1
	"Debe especificar el nombre del nuevo pa&iacute;s en todos los idiomas.";	// Necessita traduccio
#elif L==2
	"You must specify the name of the new country in all languages.";		// Need bersetzung
#elif L==3
	"You must specify the name of the new country in all languages.";
#elif L==4
	"Debe especificar el nombre del nuevo pa&iacute;s en todos los idiomas.";
#elif L==5
	"You must specify the name of the new country in all languages.";		// Besoin de traduction
#elif L==6
	"Debe especificar el nombre del nuevo pa&iacute;s en todos los idiomas.";	// Okoteve traduccin
#elif L==7
	"Devi specificare il nome del nuovo paese in tutte le lingue.";
#elif L==8
	"You must specify the name of the new country in all languages.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the name of the new country in all languages.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_group =
#if   L==1
	"Debe especificar el nombre del nuevo grupo.";	// Necessita traduccio
#elif L==2
	"You must specify the name of the new group.";	// Need bersetzung
#elif L==3
	"You must specify the name of the new group.";
#elif L==4
	"Debe especificar el nombre del nuevo grupo.";
#elif L==5
	"You must specify the name of the new group.";	// Besoin de traduction
#elif L==6
	"Debe especificar el nombre del nuevo grupo.";	// Okoteve traduccin
#elif L==7
	"Devi specificare il nome del nuovo gruppo.";
#elif L==8
	"You must specify the name of the new group.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the name of the new group.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_holiday =
#if   L==1
	"Debe especificar el nombre de la nueva festividad.";	// Necessita traduccio
#elif L==2
	"You must specify the name of the new holiday.";	// Need bersetzung
#elif L==3
	"You must specify the name of the new holiday.";
#elif L==4
	"Debe especificar el nombre de la nueva festividad.";
#elif L==5
	"You must specify the name of the new holiday.";	// Besoin de traduction
#elif L==6
	"Debe especificar el nombre de la nueva festividad.";	// Okoteve traduccin
#elif L==7
	"Devi specificare il nome della nuova festivit&agrave;.";
#elif L==8
	"You must specify the name of the new holiday.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the name of the new holiday.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_record_field =
#if   L==1
	"Debe especificar el nombre del nuevo campo de ficha.";	// Necessita traduccio
#elif L==2
	"You must specify the name of the new record field.";	// Need bersetzung
#elif L==3
	"You must specify the name of the new record field.";
#elif L==4
	"Debe especificar el nombre del nuevo campo de ficha.";
#elif L==5
	"You must specify the name of the new record field.";	// Besoin de traduction
#elif L==6
	"Debe especificar el nombre del nuevo campo de ficha.";	// Okoteve traduccin
#elif L==7
	"Devi specificare il nome del nuovo campo di scheda.";
#elif L==8
	"You must specify the name of the new record field.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the name of the new record field.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_type_of_degree =
#if   L==1
	"Debe especificar el nombre del nuevo tipo de titulaci&oacute;n.";	// Necessita traduccio
#elif L==2
	"You must specify the name of the new type of degree.";			// Need bersetzung
#elif L==3
	"You must specify the name of the new type of degree.";
#elif L==4
	"Debe especificar el nombre del nuevo tipo de titulaci&oacute;n.";
#elif L==5
	"You must specify the name of the new type of degree.";			// Besoin de traduction
#elif L==6
	"Debe especificar el nombre del nuevo tipo de titulaci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Devi specificare il nome del nuovo tipo di laurea.";
#elif L==8
	"You must specify the name of the new type of degree.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the name of the new type of degree.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_name_of_the_new_type_of_group =
#if   L==1
	"Debe especificar el nombre del nuevo tipo de grupo.";	// Necessita traduccio
#elif L==2
	"You must specify the name of the new type of group.";	// Need bersetzung
#elif L==3
	"You must specify the name of the new type of group.";
#elif L==4
	"Debe especificar el nombre del nuevo tipo de grupo.";
#elif L==5
	"You must specify the name of the new type of group.";	// Besoin de traduction
#elif L==6
	"Debe especificar el nombre del nuevo tipo de grupo.";	// Okoteve traduccin
#elif L==7
	"Devi specificare il nome del nuovo tipo di gruppo.";
#elif L==8
	"You must specify the name of the new type of group.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the name of the new type of group.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_banner =
#if   L==1
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo banner.";		// Necessita traduccio
#elif L==2
	"You must specify the short name and the full name"
	" of the new banner.";		// Need bersetzung
#elif L==3
	"You must specify the short name and the full name"
	" of the new banner.";
#elif L==4
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo banner.";
#elif L==5
	"You must specify the short name and the full name"
	" of the new banner.";		// Besoin de traduction
#elif L==6
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo banner.";		// Okoteve traduccin
#elif L==7
	"Devi specificare il nome breve e completo"
	" del nuovo banner.";
#elif L==8
	"You must specify the short name and the full name"
	" of the new banner.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the short name and the full name"
	" of the new banner.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_centre =
#if   L==1
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo centro.";		// Necessita traduccio
#elif L==2
	"You must specify the short name and the full name"
	" of the new centre.";		// Need bersetzung
#elif L==3
	"You must specify the short name and the full name"
	" of the new centre.";
#elif L==4
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo centro.";
#elif L==5
	"You must specify the short name and the full name"
	" of the new centre.";		// Besoin de traduction
#elif L==6
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo centro.";		// Okoteve traduccin
#elif L==7
	"Devi specificare il nome breve e completo"
	" del nuovo centro.";
#elif L==8
	"You must specify the short name and the full name"
	" of the new centre.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the short name and the full name"
	" of the new centre.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_course =
#if   L==1
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva asignatura.";		// Necessita traduccio
#elif L==2
	"You must specify the short name and the full name"
	" of the new course.";			// Need bersetzung
#elif L==3
	"You must specify the short name and the full name"
	" of the new course.";
#elif L==4
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva asignatura.";
#elif L==5
	"You must specify the short name and the full name"
	" of the new course.";			// Besoin de traduction
#elif L==6
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva asignatura.";		// Okoteve traduccin
#elif L==7
	"Devi specificare il nome breve e completo"
	" del nuovo corso.";
#elif L==8
	"You must specify the short name and the full name"
	" of the new course.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the short name and the full name"
	" of the new course.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_degree =
#if   L==1
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva titulaci&oacute;n.";	// Necessita traduccio
#elif L==2
	"You must specify the short name and the full name"
	" of the new degree.";			// Need bersetzung
#elif L==3
	"You must specify the short name and the full name"
	" of the new degree.";
#elif L==4
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva titulaci&oacute;n.";
#elif L==5
	"You must specify the short name and the full name"
	" of the new degree.";			// Besoin de traduction
#elif L==6
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva titulaci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"Devi specificare il nome breve e completo"
	" della nuova laurea.";
#elif L==8
	"You must specify the short name and the full name"
	" of the new degree.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the short name and the full name"
	" of the new degree.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_department =
#if   L==1
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo departamento.";		// Necessita traduccio
#elif L==2
	"You must specify the short name and the full name"
	" of the new department.";		// Need bersetzung
#elif L==3
	"You must specify the short name and the full name"
	" of the new department.";
#elif L==4
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo departamento.";
#elif L==5
	"You must specify the short name and the full name"
	" of the new department.";		// Besoin de traduction
#elif L==6
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo departamento.";		// Okoteve traduccin
#elif L==7
	"Devi specificare il nome breve e completo"
	" del nuovo dipartimento.";
#elif L==8
	"You must specify the short name and the full name"
	" of the new department.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the short name and the full name"
	" of the new department.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_institution =
#if   L==1
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva instituci&oacute;n.";		// Necessita traduccio
#elif L==2
	"You must specify the short name and the full name"
	" of the new institution.";			// Need bersetzung
#elif L==3
	"You must specify the short name and the full name"
	" of the new institution.";
#elif L==4
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva instituci&oacute;n.";
#elif L==5
	"You must specify the short name and the full name"
	" of the new institution.";			// Besoin de traduction
#elif L==6
	"Debe especificar el nombre breve y el nombre completo"
	" de la nueva instituci&oacute;n.";		// Okoteve traduccin
#elif L==7
	"Devi specificare il nome breve e completo"
	" della nuova instituzione.";
#elif L==8
	"You must specify the short name and the full name"
	" of the new institution.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the short name and the full name"
	" of the new institution.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_link =
#if   L==1
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo enlace.";		// Necessita traduccio
#elif L==2
	"You must specify the short name and the full name"
	" of the new link.";		// Need bersetzung
#elif L==3
	"You must specify the short name and the full name"
	" of the new link.";
#elif L==4
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo enlace.";
#elif L==5
	"You must specify the short name and the full name"
	" of the new link.";		// Besoin de traduction
#elif L==6
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo enlace.";		// Okoteve traduccin
#elif L==7
	"Devi specificare il nome breve e completo"
	" del nuovo link.";
#elif L==8
	"You must specify the short name and the full name"
	" of the new link.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the short name and the full name"
	" of the new link.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_email_domain =
#if   L==1
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo dominio de correo.";	// Necessita traduccio
#elif L==2
	"You must specify the short name and the full name"
	" of the new email domain.";		// Need bersetzung
#elif L==3
	"You must specify the short name and the full name"
	" of the new email domain.";
#elif L==4
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo dominio de correo.";
#elif L==5
	"You must specify the short name and the full name"
	" of the new email domain.";		// Besoin de traduction
#elif L==6
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo dominio de correo.";	// Okoteve traduccin
#elif L==7
	"Devi specificare il nome breve e completo"
	" del nuovo campo mail.";
#elif L==8
	"You must specify the short name and the full name"
	" of the new email domain.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the short name and the full name"
	" of the new email domain.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_short_name_and_the_full_name_of_the_new_place =
#if   L==1
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo lugar.";		// Necessita traduccio
#elif L==2
	"You must specify the short name and the full name"
	" of the new place.";		// Need bersetzung
#elif L==3
	"You must specify the short name and the full name"
	" of the new place.";
#elif L==4
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo lugar.";
#elif L==5
	"You must specify the short name and the full name"
	" of the new place.";		// Besoin de traduction
#elif L==6
	"Debe especificar el nombre breve y el nombre completo"
	" del nuevo lugar.";		// Okoteve traduccin
#elif L==7
	"Devi specificare il nome breve e completo"
	" del nuovo luogo.";
#elif L==8
	"You must specify the short name and the full name"
	" of the new place.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the short name and the full name"
	" of the new place.";		// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_title_of_the_assignment =
#if   L==1
	"Cal especificar el t&iacute;tol de l'activitat.";
#elif L==2
	"Die Aufgabe muss angegeben werden.";
#elif L==3
	"You must specify the title of the assignment.";
#elif L==4
	"Debe especificar el t&iacute;tulo de la actividad.";
#elif L==5
	"Vous devez sp&eacute;cifier le titre de l'activit&eacute;.";
#elif L==6
	"Debe especificar el t&iacute;tulo de la actividad.";	// Okoteve traduccin
#elif L==7
	"&Egrave; necessario specificare il titolo dell'attivit&agrave;.";
#elif L==8
	"Musisz poda&cacute; tytu&lstrok; zadania.";
#elif L==9
	"Voc&ecirc; deve especificar o t&iacute;tulo da atividade.";
#endif

const char *Txt_You_must_specify_the_title_of_the_event =
#if   L==1
	"Debe especificar el t&iacute;tulo del evento.";	// Necessita traduccio
#elif L==2
	"Der Ereignis muss angegeben werden.";
#elif L==3
	"You must specify the title of the event.";
#elif L==4
	"Debe especificar el t&iacute;tulo del evento.";
#elif L==5
	"Vous devez sp&eacute;cifier le titre du &eacute;v&eacute;nement.";
#elif L==6
	"Debe especificar el t&iacute;tulo del evento.";	// Okoteve traduccin
#elif L==7
	"&Egrave; necessario specificare il titolo del evento.";
#elif L==8
	"You must specify the title of the event.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Voc&ecirc; deve especificar o t&iacute;tulo do evento.";
#endif

const char *Txt_You_must_specify_the_title_of_the_survey =
#if   L==1
	"Debe especificar el t&iacute;tulo de la encuesta.";	// Necessita traduccio
#elif L==2
	"Die Umfrage muss angegeben werden.";
#elif L==3
	"You must specify the title of the survey.";
#elif L==4
	"Debe especificar el t&iacute;tulo de la encuesta.";
#elif L==5
	"Vous devez sp&eacute;cifier le titre du sondage.";
#elif L==6
	"Debe especificar el t&iacute;tulo de la encuesta.";	// Okoteve traduccin
#elif L==7
	"&Egrave; necessario specificare il titolo del sondaggio.";
#elif L==8
	"You must specify the title of the survey.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Voc&ecirc; deve especificar o t&iacute;tulo do inqu&eacute;rito.";
#endif

const char *Txt_You_must_specify_the_URL_of_the_new_banner =
#if   L==1
	"Debe especificar el URL del nuevo banner.";	// Necessita traduccio
#elif L==2
	"You must specify the URL of the new banner.";	// Need bersetzung
#elif L==3
	"You must specify the URL of the new banner.";
#elif L==4
	"Debe especificar el URL del nuevo banner.";
#elif L==5
	"You must specify the URL of the new banner.";	// Besoin de traduction
#elif L==6
	"Debe especificar el URL del nuevo banner.";	// Okoteve traduccin
#elif L==7
	"Devi specificare l'URL del nuovo banner.";
#elif L==8
	"You must specify the URL of the new banner.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the URL of the new banner.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_URL_of_the_new_link =
#if   L==1
	"Debe especificar el URL del nuevo enlace.";	// Necessita traduccio
#elif L==2
	"You must specify the URL of the new link.";	// Need bersetzung
#elif L==3
	"You must specify the URL of the new link.";
#elif L==4
	"Debe especificar el URL del nuevo enlace.";
#elif L==5
	"You must specify the URL of the new link.";	// Besoin de traduction
#elif L==6
	"Debe especificar el URL del nuevo enlace.";	// Okoteve traduccin
#elif L==7
	"Devi specificare l'URL del nuovo link.";
#elif L==8
	"You must specify the URL of the new link.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the URL of the new link.";	// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_web_address_of_the_new_centre =
#if   L==1
	"Debe especificar la direcci&oacute;n web"
	" del nuevo centro.";				// Necessita traduccio
#elif L==2
	"You must specify the web address"
	" of the new centre.";				// Need bersetzung
#elif L==3
	"You must specify the web address"
	" of the new centre.";
#elif L==4
	"Debe especificar la direcci&oacute;n web"
	" del nuevo centro.";
#elif L==5
	"You must specify the web address"
	" of the new centre.";				// Besoin de traduction
#elif L==6
	"Debe especificar la direcci&oacute;n web"
	" del nuevo centro.";				// Okoteve traduccin
#elif L==7
	"Devi specificare l'indirizzo web"
	" del nuovo centro.";
#elif L==8
	"You must specify the web address"
	" of the new centre.";				// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the web address"
	" of the new centre.";				// Necessita de traduo
#endif


const char *Txt_You_must_specify_the_web_address_of_the_new_degree =
#if   L==1
	"Debe especificar la direcci&oacute;n web"
	" de la nueva titulaci&oacute;n.";		// Necessita traduccio
#elif L==2
	"You must specify the web address"
	" of the new degree.";				// Need bersetzung
#elif L==3
	"You must specify the web address"
	" of the new degree.";
#elif L==4
	"Debe especificar la direcci&oacute;n web"
	" de la nueva titulaci&oacute;n.";
#elif L==5
	"You must specify the web address"
	" of the new degree.";				// Besoin de traduction
#elif L==6
	"Debe especificar la direcci&oacute;n web"
	" de la nueva titulaci&oacute;n.";		// Okoteve traduccin
#elif L==7
	"Devi specificare l'indirizzo web"
	" della nuova laurea.";
#elif L==8
	"You must specify the web address"
	" of the new degree.";				// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the web address"
	" of the new degree.";				// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_web_address_of_the_new_department =
#if   L==1
	"Debe especificar la direcci&oacute;n web"
	" del nuevo departamento.";			// Necessita traduccio
#elif L==2
	"You must specify the web address"
	" of the new department.";			// Need bersetzung
#elif L==3
	"You must specify the web address"
	" of the new department.";
#elif L==4
	"Debe especificar la direcci&oacute;n web"
	" del nuevo departamento.";
#elif L==5
	"You must specify the web address"
	" of the new department.";			// Besoin de traduction
#elif L==6
	"Debe especificar la direcci&oacute;n web"
	" del nuevo departamento.";			// Okoteve traduccin
#elif L==7
	"Devi specificare l'indirizzo web"
	" del nuovo dipartimento.";
#elif L==8
	"You must specify the web address"
	" of the new department.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the web address"
	" of the new department.";			// Necessita de traduo
#endif

const char *Txt_You_must_specify_the_web_address_of_the_new_institution =
#if   L==1
	"Debe especificar la direcci&oacute;n web"
	" de la nueva instituci&oacute;n.";		// Necessita traduccio
#elif L==2
	"You must specify the web address"
	" of the new institution.";			// Need bersetzung
#elif L==3
	"You must specify the web address"
	" of the new institution.";
#elif L==4
	"Debe especificar la direcci&oacute;n web"
	" de la nueva instituci&oacute;n.";
#elif L==5
	"You must specify the web address"
	" of the new institution.";			// Besoin de traduction
#elif L==6
	"Debe especificar la direcci&oacute;n web"
	" de la nueva instituci&oacute;n.";		// Okoteve traduccin
#elif L==7
	"Devi specificare l'indirizzo web"
	" della nuova istituzione.";
#elif L==8
	"You must specify the web address"
	" of the new institution.";			// Potrzebujesz tlumaczenie
#elif L==9
	"You must specify the web address"
	" of the new institution.";			// Necessita de traduo
#endif

const char *Txt_You_must_type_at_least_one_tag_for_the_question =
#if   L==1
	"Debe escribir al menos un descriptor para la pregunta.";	// Necessita traduccio
#elif L==2
	"You must type at least one tag for the question.";		// Need bersetzung
#elif L==3
	"You must type at least one tag for the question.";
#elif L==4
	"Debe escribir al menos un descriptor para la pregunta.";
#elif L==5
	"You must type at least one tag for the question.";		// Besoin de traduction
#elif L==6
	"Debe escribir al menos un descriptor para la pregunta.";	// Okoteve traduccin
#elif L==7
	"Devi scrivere almeno un'etichetta per la domanda.";
#elif L==8
	"You must type at least one tag for the question.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You must type at least one tag for the question.";		// Necessita de traduo
#endif

const char *Txt_You_must_type_at_least_the_first_answer =
#if   L==1
	"Debe escribir al menos la primera respuesta.";	// Necessita traduccio
#elif L==2
	"You must type at least the first answer.";	// Need bersetzung
#elif L==3
	"You must type at least the first answer.";
#elif L==4
	"Debe escribir al menos la primera respuesta.";
#elif L==5
	"You must type at least the first answer.";	// Besoin de traduction
#elif L==6
	"Debe escribir al menos la primera respuesta.";	// Okoteve traduccin
#elif L==7
	"Devi scrivere almeno la prima risposta.";
#elif L==8
	"You must type at least the first answer.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You must type at least the first answer.";	// Necessita de traduo
#endif

const char *Txt_You_must_type_at_least_the_first_two_answers =
#if   L==1
	"Debe escribir al menos las dos primeras respuestas.";	// Necessita traduccio
#elif L==2
	"You must type at least the first two answers.";	// Need bersetzung
#elif L==3
	"You must type at least the first two answers.";
#elif L==4
	"Debe escribir al menos las dos primeras respuestas.";
#elif L==5
	"You must type at least the first two answers.";	// Besoin de traduction
#elif L==6
	"Debe escribir al menos las dos primeras respuestas.";	// Okoteve traduccin
#elif L==7
	"Devi scrivere almeno le prime due risposte.";
#elif L==8
	"You must type at least the first two answers.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You must type at least the first two answers.";	// Necessita de traduo
#endif

const char *Txt_You_must_type_the_stem_of_the_question =
#if   L==1
	"Debe escribir el enunciado de la pregunta.";	// Necessita traduccio
#elif L==2
	"You must type the stem of the question.";	// Need bersetzung
#elif L==3
	"You must type the stem of the question.";
#elif L==4
	"Debe escribir el enunciado de la pregunta.";
#elif L==5
	"You must type the stem of the question.";	// Besoin de traduction
#elif L==6
	"Debe escribir el enunciado de la pregunta.";	// Okoteve traduccin
#elif L==7
	"Devi scrivere l'enunciato della domanda.";
#elif L==8
	"You must type the stem of the question.";	// Potrzebujesz tlumaczenie
#elif L==9
	"You must type the stem of the question.";	// Necessita de traduo
#endif

const char *Txt_You_need_an_XML_file_containing_a_list_of_questions =
#if   L==1
	"Es necesario un archivo XML"
	" que contenga una lista de preguntas.";	// Necessita traduccio
#elif L==2
	"You need an XML file"
	" containing a list of questions.";		// Need bersetzung
#elif L==3
	"You need an XML file"
	" containing a list of questions.";
#elif L==4
	"Es necesario un archivo XML"
	" que contenga una lista de preguntas.";
#elif L==5
	"You need an XML file"
	" containing a list of questions.";		// Besoin de traduction
#elif L==6
	"Es necesario un archivo XML"
	" que contenga una lista de preguntas.";	// Okoteve traduccin
#elif L==7
	"You need an XML file"
	" containing a list of questions.";		// Bisogno di traduzione
#elif L==8
	"You need an XML file"
	" containing a list of questions.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You need an XML file"
	" containing a list of questions.";		// Necessita de traduo
#endif

const char *Txt_You_should_read_the_following_information =
#if   L==1
	"Vost&egrave; ha de llegir la seg%uuml;ent informaci&oacute;:";
#elif L==2
	"Sie sollten die folgenden Informationen lesen:";
#elif L==3
	"You should read the following information:";
#elif L==4
	"Debe leer la siguiente informaci&oacute;n:";
#elif L==5
	"Vous devez lire les informations suivantes:";
#elif L==6
	"Debe leer la siguiente informaci&oacute;n:";	// Okoteve traduccin
#elif L==7
	"Dovresti leggere le seguenti informazioni:";
#elif L==8
	"Nale&zdot;y przeczyta&cacute; nast&eogon;puj&aogon;ce informacje:";
#elif L==9
	"Voc&ecirc; deve ler as seguintes informa&ccedil;&otilde;es:";
#endif

const char *Txt_You_were_already_enrolled_as_X_in_the_course_Y =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"Usted ya estaba inscrito/a como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";	// Necessita traduccio
#elif L==2
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// bersetzung notwendig
#elif L==3
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";
#elif L==4
	"Usted ya estaba inscrito/a como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";
#elif L==5
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Besoin de traduction
#elif L==6
	"Usted ya estaba inscrito/a como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong>.";	// Okoteve traduccin
#elif L==7
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Bisogno di traduzione
#elif L==8
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Potrzebujesz tlumaczenie
#elif L==9
	"You were already enrolled as <strong>%s</strong>"
	" in the course <strong>%s</strong>.";		// Necessita de traduo
#endif

const char *Txt_Your_comment_has_been_updated =
#if   L==1
	"El seu comentari ha estat actualitzat.";
#elif L==2
	"Ihr Kommentar wurde aktualisiert.";
#elif L==3
	"Your comment has been updated.";
#elif L==4
	"Su comentario ha sido actualizado.";
#elif L==5
	"Votre commentaire a &eacute;t&eacute; mis &agrave; jour.";
#elif L==6
	"Su comentario ha sido actualizado.";		// Okoteve traduccin
#elif L==7
	"Il tuo commento &egrave; stato aggiornato.";
#elif L==8
	"Komentarz zosta&lstrok; zaktualizowany.";
#elif L==9
	"O seu coment&aacute;rio foi atualizado.";
#endif

const char *Txt_Your_nickname_X_has_been_registered_successfully =	// Warning: it is very important to include %s in the following sentences
#if   L==1
	"Su apodo <strong>@%s</strong>"
	" se ha registrado correctamente.";		// Necessita traduccio
#elif L==2
	"Your nickname <strong>@%s</strong>"
	" has been registered successfully.";		// Need bersetzung
#elif L==3
	"Your nickname <strong>@%s</strong>"
	" has been registered successfully.";
#elif L==4
	"Su apodo <strong>@%s</strong>"
	" se ha registrado correctamente.";
#elif L==5
	"Your nickname <strong>@%s</strong>"
	" has been registered successfully.";		// Besoin de traduction
#elif L==6
	"Su apodo <strong>@%s</strong>"
	" se ha registrado correctamente.";		// Okoteve traduccin
#elif L==7
	"Il tuo nome utente <strong>@%s</strong>"
	" &egrave; stato registrato con successo.";
#elif L==8
	"Your nickname <strong>@%s</strong>"
	" has been registered successfully.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Your nickname <strong>@%s</strong>"
	" has been registered successfully.";		// Necessita de traduo
#endif

const char *Txt_Your_password_has_been_changed_successfully =
#if   L==1
	"Su contrase&ntilde;a se ha modificado con &eacute;xito.";	// Necessita traduccio
#elif L==2
	"Your password has been changed successfully.";			// Need bersetzung
#elif L==3
	"Your password has been changed successfully.";
#elif L==4
	"Su contrase&ntilde;a se ha modificado con &eacute;xito.";
#elif L==5
	"Your password has been changed successfully.";			// Besoin de traduction
#elif L==6
	"Su contrase&ntilde;a se ha modificado con &eacute;xito.";	// Okoteve traduccin
#elif L==7
	"La tua password &egrave; stata cambiata con successo.";
#elif L==8
	"Your password has been changed successfully.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Your password has been changed successfully.";			// Necessita de traduo
#endif

const char *Txt_Your_password_is_not_secure_enough =
#if   L==1
	"Su contrase&ntilde;a no es suficientemente segura.";	// Necessita traduccio
#elif L==2
	"Your password is not secure enough.";			// Need bersetzung
#elif L==3
	"Your password is not secure enough.";
#elif L==4
	"Su contrase&ntilde;a no es suficientemente segura.";
#elif L==5
	"Your password is not secure enough.";			// Besoin de traduction
#elif L==6
	"Su contrase&ntilde;a no es suficientemente segura.";	// Okoteve traduccin
#elif L==7
	"La tua password non &egrave; sicura abbastanza.";
#elif L==8
	"Your password is not secure enough.";			// Potrzebujesz tlumaczenie
#elif L==9
	"Your password is not secure enough.";			// Necessita de traduo
#endif

const char *Txt_Your_password_must_be_at_least_X_characters_and_can_not_contain_spaces_ =	// Warning: it is very important to include %u in the following sentences
#if   L==1
	"La contrase&ntilde;a debe tener al menos %u caracteres"
	" y no puede contener espacios."
	" Es muy importante que no use solo"
	" d&iacute;gitos num&eacute;ricos, ni un nombre o apellido, ni una palabra conocida.";	// Necessita traduccio
#elif L==2
	"Your password must be at least %u characters"
	" and can not contain spaces."
	" It's very important not to use only"
	" numeric digits, or a name or surname, or a known word.";				// Need bersetzung
#elif L==3
	"Your password must be at least %u characters"
	" and can not contain spaces."
	" It's very important not to use only"
	" numeric digits, or a name or surname, or a known word.";
#elif L==4
	"La contrase&ntilde;a debe tener al menos %u caracteres"
	" y no puede contener espacios."
	" Es muy importante que no use solo"
	" d&iacute;gitos num&eacute;ricos, ni un nombre o apellido, ni una palabra conocida.";
#elif L==5
	"Your password must be at least %u characters"
	" and can not contain spaces."
	" It's very important not to use only"
	" numeric digits, or a name or surname, or a known word.";				// Besoin de traduction
#elif L==6
	"La contrase&ntilde;a debe tener al menos %u caracteres"
	" y no puede contener espacios."
	" Es muy importante que no use solo"
	" d&iacute;gitos num&eacute;ricos, ni un nombre o apellido, ni una palabra conocida.";	// Okoteve traduccin
#elif L==7
	"La tua password deve essere almeno di %u caratteri"
	" e non pu&ograve; contenere spazi."
	" E' molto importante non usare solo"
	" tasti numerici, o un nome o cognome, o una parola conosciuta.";
#elif L==8
	"Your password must be at least %u characters"
	" and can not contain spaces."
	"It's very important not to use only"
	" numeric digits, or a name or surname, or a known word.";				// Potrzebujesz tlumaczenie
#elif L==9
	"Your password must be at least %u characters"
	" and can not contain spaces."
	" It's very important not to use only"
	" numeric digits, or a name or surname, or a known word.";				// Necessita de traduo
#endif

const char *Txt_Your_personal_data_have_been_updated =
#if   L==1
	"Sus datos personales se han actualizado.";	// Necessita traduccio
#elif L==2
	"Your personal data have been updated.";	// Need bersetzung
#elif L==3
	"Your personal data have been updated.";
#elif L==4
	"Sus datos personales se han actualizado.";
#elif L==5
	"Your personal data have been updated.";	// Besoin de traduction
#elif L==6
	"Sus datos personales se han actualizado.";	// Okoteve traduccin
#elif L==7
	"I tuoi dati personali sono stati aggiornati.";
#elif L==8
	"Your personal data have been updated.";	// Potrzebujesz tlumaczenie
#elif L==9
	"Your personal data have been updated.";	// Necessita de traduo
#endif

const char *Txt_Your_preferences_about_notifications_have_changed =
#if   L==1
	"Les seves prefer&egrave;ncies sobre les notificacions han canviat.";
#elif L==2
	"Your preferences about notifications have changed.";		// Need bersetzung
#elif L==3
	"Your preferences about notifications have changed.";
#elif L==4
	"Sus preferencias sobre las notificaciones han cambiado.";
#elif L==5
	"Your preferences about notifications have changed.";		// Besoin de traduction
#elif L==6
	"Sus preferencias sobre las notificaciones han cambiado.";	// Okoteve traduccin
#elif L==7
	"Le tue preferenze sulle notifiche sono cambiate.";
#elif L==8
	"Your preferences about notifications have changed.";		// Potrzebujesz tlumaczenie
#elif L==9
	"Your preferences about notifications have changed.";		// Necessita de traduo
#endif

const char *Txt_Your_record_card_in_this_course_has_been_updated =
#if   L==1
	"Su ficha en la asignatura se ha actualizado:";		// Necessita traduccio
#elif L==2
	"Your record card in this course has been updated:";	// Need bersetzung
#elif L==3
	"Your record card in this course has been updated:";
#elif L==4
	"Su ficha en la asignatura se ha actualizado:";
#elif L==5
	"Your record card in this course has been updated:";	// Besoin de traduction
#elif L==6
	"Su ficha en la asignatura se ha actualizado:";		// Okoteve traduccin
#elif L==7
	"La tua scheda in questo corso &egrave; stata aggiornata:";
#elif L==8
	"Your record card in this course has been updated:";	// Potrzebujesz tlumaczenie
#elif L==9
	"Your record card in this course has been updated:";	// Necessita de traduo
#endif

const char *Txt_Your_request_for_enrollment_as_X_in_the_course_Y_has_been_accepted_for_processing =	// Warning: it is very important to include two %s in the following sentences
#if   L==1
	"La seva petici&oacute; d'inscripci&oacute; com <strong>%s</strong>"
	" en l'assignatura <strong>%s</strong> ha estat realitzada."
	" La petici&oacute; ser&agrave; revisada per un professor o un administrador."
	" Vost&egrave; rebr&agrave; una notificaci&oacute; quan s'accepti la seva inscripci&oacute;.";
#elif L==2
	"Sein Antrag auf Registrierung als <strong>%s</strong>"
	" in Kurs <strong>%s</strong> gemacht wurde."
	" Ihre Anfrage wird durch einen Lehrer oder Administrator &uuml;berpr&uuml;ft werden."
	" Sie werden benachrichtigt, sobald die Anmeldung akzeptiert wird.";
#elif L==3
	"Your request for enrollment as <strong>%s</strong>"
	" in the course <strong>%s</strong> has been accepted for processing."
	" Your request will be reviewed by a teacher or administrator."
	" You will be notified when registration is accepted.";
#elif L==4
	"Su petici&oacute;n de inscripci&oacute;n como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong> ha sido realizada."
	" La petici&oacute;n ser&aacute; revisada por un/a profesor/a o un/a administrador/a."
	" Usted recibir&aacute; una notificaci&oacute;n cuando se acepte su inscripci&oacute;n.";
#elif L==5
	"Votre demande d'enregistrement en tant que <strong>%s</strong>"
	" dans la mati&egrave;re <strong>%s</strong> a &eacute;t&eacute; fait."
	" Votre demande sera examin&eacute;e par un enseignant ou un administrateur."
	" Vous serez avis&eacute; lorsque l'enregistrement est accept&eacute;e.";
#elif L==6
	"Su petici&oacute;n de inscripci&oacute;n como <strong>%s</strong>"
	" en la asignatura <strong>%s</strong> ha sido realizada."
	" La petici&oacute;n ser&aacute; revisada por un/a profesor/a o un/a administrador/a."
	" Usted recibir&aacute; una notificaci&oacute;n cuando se acepte su inscripci&oacute;n.";	// Okoteve traduccin
#elif L==7
	"&Egrave; stato fatto la sua domanda di registrazione come <strong>%s</strong>"
	" nel corso <strong>%s</strong>."
	" La richiesta verr&agrave; esaminata da un insegnante o un amministratore."
	" Riceverai una notifica quando la registrazione &egrave; stata accettata.";
#elif L==8
	"Jego wniosek o rejestracj&eogon; jako <strong>%s</strong>"
	" w <strong>%s</strong> przedmiotu zosta&lstrok;a dokonana."
	" Twoja pro&sacute;ba zostanie poddane przegl&aogon;dowi przez nauczyciela lub administratora."
	" Zostaniesz powiadomiony, gdy rejestracja zostanie zaakceptowana.";
#elif L==9
	"Seu pedido de registo como <strong>%s</strong>"
	" na disciplina <strong>%s</strong> tem sido feito."
	" O pedido ser&aacute; analisado por um professor ou administrador."
	" Voc&ecirc; ser&aacute; notificado quando o registro &eacute; aceito.";
#endif

const char *Txt_ZIP_file =
#if   L==1
	"Fitxer ZIP";
#elif L==2
	"ZIP-Datei";
#elif L==3
	"ZIP file";
#elif L==4
	"Archivo ZIP";
#elif L==5
	"Fichier ZIP";
#elif L==6
	"Archivo ZIP";	// Okoteve traduccin
#elif L==7
	"File ZIP";
#elif L==8
	"Plik ZIP";
#elif L==9
	"Arquivo ZIP";
#endif
